USE [CL_MDB]
GO
CREATE PROCEDURE GET_CL
	@CLCL int,
	@COD	int	
AS
BEGIN
	SELECT * FROM ^cl c,
		^clrelclcl r where c.fwid = r.fwid_cl and r.fcl = @CLCL
		and r.fcod = @COD
END
GO

USE [CL_MDB]
GO
CREATE PROCEDURE GET_DOC_HISTORY
	@IDDOC	INT
AS
BEGIN
	SELECT * FROM ^archive WHERE fid_doc = @IDDOC ORDER BY FMDATE DESC
END
GO

USE [ALL_MDB]
GO
CREATE PROCEDURE GET_PROV_BY_DOC_ID
	@IDDOC int
AS
BEGIN
	SELECT FWID FROM ^schet where fdoc<> '_R' and fid_doc = @IDDOC
END
GO

USE [ALL_MDB]
GO
CREATE PROCEDURE GET_MDOCREG_BY_HID_DOC
	@IDDOC int
AS
BEGIN
	SELECT HNKNOM, HNKDAT, HISNOV1  FROM ^MDOCREG WHERE hid_doc = @IDDOC and HNKNOM is not null
END
GO

USE [ALL_MDB]
GO
CREATE PROCEDURE GET_PDOCREG_BY_HID_DOC
	@IDDOC int
AS
BEGIN
	SELECT HNKNOM, HNKDAT, HISNOV1 FROM ^PDOCREG WHERE hid_doc = @IDDOC and HNKNOM is not null
END
GO

USE [ALL_MDB]
GO
CREATE PROCEDURE GET_MDOCREG_BY_HID_DOC_HREG
	@IDDOC int,
	@REG varchar(5)
AS
BEGIN
	SELECT HNKNOM, HNKDAT, HISNOV1  FROM ^MDOCREG WHERE hid_doc = @IDDOC and HREG = @REG
END
GO

USE [ALL_MDB]
GO
CREATE PROCEDURE GET_PDOCREG_BY_HID_DOC_HREG
	@IDDOC int,
	@REG varchar(5)
AS
BEGIN
	SELECT HNKNOM, HNKDAT, HISNOV1  FROM ^PDOCREG WHERE hid_doc = @IDDOC and HREG = @REG
END
GO

USE [CL_MDB]
GO
CREATE PROCEDURE GET_CL_NMK_BY_FWID
@ID int
AS
BEGIN
	SELECT * from ^CL_NMK where fwid = @ID
END	
GO	




USE [ALL_MDB]
GO
CREATE PROCEDURE [dbo].[dp_buildtreekeys]
	@tblName	nvarchar(max),		-- имя таблицы, где есть @idField, @parField, FGRPDEEP, FLEFTKEY, FRIGHTKEY
	@idField	nvarchar(max),		-- поле ID (напр. FWID)
	@parField	nvarchar(max),		-- поле для связи по схеме «родитель-предок» (напр. FWID_PAR)
	@parKey		int,				-- значение поля @parField, с которого начинается построение структуры
	@order		nvarchar(max)		-- поле, по которому упорядочиваются строки на одном уровне (напр. FORDER)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SQL nvarchar(MAX)
	-- CTE with two numbers: 1 and 2
	
	SET @SQL = '
	WITH TwoNums AS
	(
		SELECT 1 AS n
		UNION ALL
		SELECT 2 AS n
	),
	' +
	-- CTE with two binary sort paths for each node:
	-- One smaller than descendants sort paths
	-- One greater than descendants sort paths
	'
	SortPath AS
	(
		SELECT '+@idField+', 0 AS FGRPDEEP, n, CAST(n AS VARBINARY(MAX)) AS sort_path
		FROM '+@tblName+'
			CROSS JOIN TwoNums
		WHERE '+@parField+' = '+CAST(@parKey AS NVARCHAR(MAX))+'
		
		UNION ALL
		
		SELECT C.'+@idField+', P.FGRPDEEP + 1, TN.n, P.sort_path + 
			CAST((-1+ROW_NUMBER() OVER(PARTITION BY C.'+@parField+' ORDER BY C.'+@order+')) / 2 * 2+TN.n AS VARBINARY(MAX))
		FROM SortPath AS P
			INNER JOIN '+@tblName+' AS C ON P.n = 1 AND C.'+@parField+' = P.'+@idField+'
			CROSS JOIN TwoNums AS TN
	),
	' +

	-- CTE with Row Numbers Representing sort_path Order
	'
	Sort AS
	(
		SELECT '+@idField+', FGRPDEEP,
		ROW_NUMBER() OVER(ORDER BY sort_path) AS sortval
		FROM SortPath
	),
	' +

	-- CTE with Left and Right Values Representing
	-- Nested Sets Relationships
	'
	NestedSets AS
	(
		SELECT '+@idField+', FGRPDEEP, MIN(sortval) AS FLEFTKEY, MAX(sortval) AS FRIGHTKEY
		FROM Sort
		GROUP BY '+@idField+', FGRPDEEP
	)

	UPDATE T
	SET T.FLEFTKEY = NS.FLEFTKEY,
		T.FRIGHTKEY = NS.FRIGHTKEY,
		T.FGRPDEEP = NS.FGRPDEEP
	FROM '+@tblName+' T
		INNER JOIN NestedSets NS ON T.'+@idField+' = NS.'+@idField
		
	EXEC(@SQL)
	
	
	SET @SQL = '
	SELECT MAX(FRIGHTKEY)+1
	FROM '+@tblName+'
	'
	EXEC(@SQL)
END
GO