/*
	Описание объекта "Автотранспорт": DpAvto
	Конструктор DpAvto.
	Пример использования:
		// рождаем объект
		var obj = new DpAvto();
		// устанавливаем дату расчета остатков, МОЛа, подразделение, номенклатуру
		obj.setVar("FDAT", new Date(2009,11,1));
		obj.setVar("FMOL", mFmol);
		obj.setVar("FPODR", mFpodr);
		obj.setVar("FNMKL", mFwidNmkl);
		// вызываем метод расчета остатков по учету ТМЦ
		ost = obj.getFuelTMC();
		// ИЛИ метод расчета остатков по путевым листам
		ost = obj.getFuelWB();
*/

include("sys\\udfs.js")
//include("plugPth.js");
include("mtr_ost.js");
include("avto:AVTO_const.js");

// Конструктор объекта DpAvto
function DpAvto(mode)
{
	RO("DpAvto", this);

	this.mode			= mode;				// режим - пока не используется
	this.setVar			= DpAvto_setVar;	// установить значение переменной
	this.getVar			= DpAvto_getVar;	// получить значение переменной

	this.getFuelTMC	= DpAvto_getFuelTMC;	// получить остаток по учету ТМЦ
	this.getFuelWB	= DpAvto_getFuelWB;		// получить остаток по путевым листам
	this.getFuelWBTMC	= DpAvto_getFuelWBTMC;// получить остаток по путевым листам
	this.getLoadFuelTMC	= DpAvto_getLoadFuelTMC;	// получить заправку по учету ТМЦ
	this.getRascTkm	= DpAvto_getRascTkm;	// получить расценку для т/км
	this.getRascT		= DpAvto_getRascT;			// получить расценку для т
	this.getRascDopr	= DpAvto_getRascDopr;		// получить расценку для дополнительных работ
	this.getParamWorkType	= DpAvto_getParamWorkType;	//получить параметры справочника виды работ
	this.initPar			= DpAvto_initPar;		//рождает параметры по трансп. средству

	this.crRegVar		= DpAvto_crRegVar;

	this.aVal			= new Array();		//массив переменных и значений
	
	this.sprWorktype = getPar(const_GETPAR_WORKTYPE); 	//справочник видов работ
	this.sprAvto = getPar(const_AVTOTRANSPORT); 	//справочник трансп. средств
	return this;
}

/*
	Установить значение переменной
*/

/*optional*/
function DpAvto_setVar(fld, cValue, varDescr) 
{
	fld = fld.toUpperCase();

	// Указанная переменная становиться членом this
	if(varDescr != false)
	{
		this.crRegVar(fld, typeof(cValue), varDescr);
	}

	this.aVal[fld] = cValue;
	
	if(fld == "FAVTO")
	{
		//если изменилися код авто то нужно перечитать остальные его параметры
		this.initPar();
	}
	return true;
}

/*
	
*/
function DpAvto_crRegVar(name, type, descr)
{
	if(type=="object" || type=="unknown"  || type=="undefined")
	{
		return;
	}
	var v	= null;
	//ищем, может такая переменная уже есть
	for(var i=0, nLen=this.aVal.length; i < nLen; i++)
	{
		if(this.aVal[i] != null && this.aVal[i].name.toUpperCase()==name.toUpperCase())
		{
			v	= this.aVal[i];
		}
	}

	if(v == null)
	{
		v	= new Object();
		this.aVal[i]	= v;
	}
	//анализируем тип
	v.name	= name;
	v.descr	= descr;
	v.type	= 0;	//тип никакой
	switch(type)
	{
		case dbMemo:
		case dbText:
		case "C":
		case "string":
			v.type	= crString;
			break;
		case dbByte:
		case dbInteger:
		case dbLong:
		case dbCurrency:
		case dbSingle:
		case dbDouble:
		case dbFloat:
		case "N":
		case "number":
			v.type	= crNumber;//double
			break;
		case dbDate:
		case "D":
		case "date":
			v.type	= crDate;
			break;
		case dbBoolean:
		case "L":
		case "boolean":
			v.type	= crBool;
			break;
		default:
			break;
	}
}

/*
	получить значение переменной
*/

function DpAvto_getVar(fld)
{
	fld = fld.toUpperCase();
	var ret = null;

	if (typeof(fld) != "undefined")
	{
		ret = this.aVal[fld];
	}

	return ret;
}

/*
	получить остаток топлива (или масла) по учету ТМЦ
	mode - пока не используется
*/
function DpAvto_getFuelTMC(mode)
{
	var retVal = 0;
	
	var ost = new DpMtrOb();
	ost.tblName = getTmpTableName();
	ost.fdate = this.aVal["FDAT"];

	// если передавали номенклатуру
	if (this.aVal["FNMKL"] != null && this.aVal["FNMKL"] != "undefined")
	{
		ost.fnmkl = this.aVal["FNMKL"];
	}

	// если передавали подразделение
	if (this.aVal["FPODR"] != null && this.aVal["FPODR"] != "undefined")
	{
		ost.fpodr = this.aVal["FPODR"];
	}
	
	// если передавали МОЛа
	if (this.aVal["FMOL"] != null && this.aVal["FMOL"] != "undefined")
	{
		ost.fmol  = this.aVal["FMOL"];
	}
	
	ost.doCalc();	// считаем остатки
	
	var tbl = ost.getTbl();
	// если нам вернули непустую таблицу, то получим остаток номенклатуры
	if (!tbl.IsEOF())
	{
		retVal = new Object();
		retVal.fost = toScrObj(tbl.getValue("FOST"));
		retVal.fcena = toScrObj(tbl.getValue("FCENA"));
	}
	tbl = null;
	
	DropTable(ost.tblName);
	ost = null;
	
	return retVal;
}

/*
	получить остаток топлива (или масла) по путевым листам
	mode - пока не используется
	Остаток топлива будем брать из последнего путевого листа по этому автомобилю
*/
function DpAvto_getFuelWB(mode)
{
	var retVal = 0;
	var hTbl = this.aVal["FDOC"] == "avto:com.debet.avto" ? "^WAYBILL" : "^WORKORDER";
	strSQL = "SELECT RF.FEFUEL"
			+" FROM "
				+" "+hTbl+" WB"
				+", ^RFUEL RF"
			+" WHERE 1=1"
				+" AND WB.FAVTO="+sqlTo(this.aVal["FAVTO"])
				+" AND WB.FDOC="+sqlTo(this.aVal["FDOC"])
				+" AND WB.FWID=RF.FID_DOC"
				+" AND WB.FDOC_DAT=("
							+"SELECT MAX(FDOC_DAT)"
							+" FROM "+hTbl
							+" WHERE 1=1"
								+" AND FAVTO="+sqlTo(this.aVal["FAVTO"])
								+" AND FDOC="+sqlTo(this.aVal["FDOC"])
								+" AND FFUEL="+sqlTo(this.aVal["FNMKL"])
								+" AND FDOC_DAT<"+sqlTo(this.aVal["FDAT"])
							+")"
	var sn = getSnap(strSQL);
	if (sn != null && sn[0] != null)
	{
		retVal = sn[0];
	}
	sn = null;

	return retVal;
}

/*
	получить остаток топлива (или масла) по путевым листам 
	и движениям из ТМЦ до нашей даты от последнего листа
	mode - пока не используется
	Остаток топлива будем брать из последнего путевого листа по этому автомобилю
*/
function DpAvto_getFuelWBTMC(mode)
{
	var retVal = 0;
	var hTbl = this.aVal["FDOC"] == "avto:com.debet.avto" ? "^WAYBILL" : "^WORKORDER";
	//Найдем предыдущий документ
	strSQL = "SELECT RF.FEFUEL"
				+", WB.FDOC_DAT"
				+", WB.FWID"
				+", WB.FSMENA"
			+" FROM "
				+" "+hTbl+" WB"
				+", ^RFUEL RF"
			+" WHERE 1=1"
				+" AND WB.FAVTO="+sqlTo(this.aVal["FAVTO"])
				+" AND WB.FDOC="+sqlTo(this.aVal["FDOC"])
				+" AND WB.FWID=RF.FID_DOC"
				+" AND WB.FDOC_DAT=("
							+"SELECT MAX(FDOC_DAT)"
							+" FROM "+hTbl
							+" WHERE 1=1"
								+" AND FAVTO="+sqlTo(this.aVal["FAVTO"])
								+" AND FDOC="+sqlTo(this.aVal["FDOC"])
								+" AND FFUEL="+sqlTo(this.aVal["FNMKL"])
								+" AND (FDOC_DAT<"+sqlTo(this.aVal["FDAT"])
								+" OR (FDOC_DAT = "+sqlTo(this.aVal["FDAT"]) +" AND FSMENA < "+sqlTo(this.aVal["FSMENA"]) +")) "
								+" AND FWID <> "+sqlTo(this.aVal["FID_DOC"])
							+")"
				+" AND WB.FSMENA = ("
							+"SELECT MAX(FSMENA)"
							+" FROM "+hTbl
							+" WHERE 1=1"
								+" AND FAVTO="+sqlTo(this.aVal["FAVTO"])
								+" AND FDOC="+sqlTo(this.aVal["FDOC"])
								+" AND FFUEL="+sqlTo(this.aVal["FNMKL"])
								+" AND (FDOC_DAT<"+sqlTo(this.aVal["FDAT"])
								+" OR (FDOC_DAT = "+sqlTo(this.aVal["FDAT"]) +" AND FSMENA < "+sqlTo(this.aVal["FSMENA"]) +")) "
								+" AND FDOC_DAT = WB.FDOC_DAT "
								+" AND FWID <> "+sqlTo(this.aVal["FID_DOC"])
							+")"
	var sn = getSnap(strSQL);
	var prevDocDat = d00min;
	var prevEkol = 0;
	var prevSmena = 0;
	var prevWid = null;
	if (sn != null && sn[0] != null)
	{
		prevEkol = sn[0];
		prevDocDat = sn[1];
		prevWid = sn[2];
		prevSmena = sn[3];
	}
	sn = null;
	
	var mFloadPer = 0;	//Заправка за период с последнего наряда до дня документа

	//if (prevDocDat >= mFdat && prevDocDat <= mFdat && prevSmena == mFsmena)
	if (differ(new Date(prevDocDat), new Date(this.aVal["FDAT"])) == 0 && prevSmena < mFsmena)
	{
		mFbkol = prevEkol;
		//На дату нашего документа есть еще один, поэтому считаем, что следует движение топлива
		//заполнить вручную
	}
	else if(differ(new Date(prevDocDat), new Date(this.aVal["FDAT"]))<0)
	{
		mFload = 0;

		//Ищем заправку за период
		var mFnmkl = this.aVal["FNMKL"];
		var strSQL = "SELECT SUM(a.FKOL*"+sRecSign+") AS FKOL"
		+" FROM ^ANLTK a"
		+" WHERE "
			+" a.FNMKL = "+sqlTo(mFnmkl)
			+" AND a.FPODR = "+sqlTo(this.aVal["FPODR"])
			+" AND a.FMOL = "+sqlTo(this.aVal["FMOL"])
			+" AND a.FDATE > "+sqlTo(prevDocDat)
			+" AND a.FDATE < "+sqlTo(this.aVal["FDAT"])
			//Не надо брать списания, потому что после списания потом нельзя уже редактировать 
			//документ так ка меняет заправку
			+" AND "
				+"( "
					+"a.Frtype =" + sqlTo(RTYPE_PMP)
					+"OR a.Frtype =" + sqlTo(RTYPE_PMR)
					+"OR a.Frtype =" + sqlTo(RTYPE_PR)
			+")"

		var sn = getSnap(strSQL);
		if (sn && sn[0]!=null)
		{
			mFloadPer = sn[0];
		}
		sn = null;
		
		//Остаток на начало - это остаток на конец из предыдущего наряда 
		//плюс движение ГСМ за период (Дата пред.наряда;Дата документа) (не включая даты начала и конца периода)
		mFbkol = prevEkol+mFloadPer;
	}

	retVal = mFbkol;
	
	return retVal;	
}

/*
	получить заправку топлива (или масла) по учету ТМЦ
	mode - пока не используется
*/
function DpAvto_getLoadFuelTMC(mode)
{
	var retVal = 0;
	var hTbl = this.aVal["FDOC"] == "avto:com.debet.avto" ? "^WAYBILL" : "^WORKORDER";
	strSQL = " SELECT WB.FDOC_DAT"
				+", WB.FWID"
				+", WB.FSMENA"
			+" FROM "
				+" "+hTbl+" WB"
			+" WHERE 1=1"
				+" AND WB.FAVTO="+sqlTo(this.aVal["FAVTO"])
				+" AND WB.FDOC="+sqlTo(this.aVal["FDOC"])
				+" AND WB.FDOC_DAT = "+sqlTo(this.aVal["FDAT"])
				+" AND FSMENA < "+sqlTo(this.aVal["FSMENA"])
	var sn = getSnap(strSQL);
	//Ищем заправку за наш день
	//заправка только у первого документа
	if (isEmpty(sn))
	{
		var strSQL = "SELECT SUM(a.FKOL*"+sRecSign+") AS FKOL"
		+" FROM ^ANLTK a"
		+" WHERE "
			+" a.FNMKL = "+sqlTo(this.aVal["FNMKL"])
		+" AND a.FPODR = "+sqlTo(this.aVal["FPODR"])
		+" AND a.FMOL = "+sqlTo(this.aVal["FMOL"])
			+" AND a.FDATE = "+sqlTo(this.aVal["FDAT"])
			//Не надо брать списания, потому что после списания потом нельзя уже редактировать 
			//документ так ка меняет заправку
			+" AND "
				+"( "
					+"a.Frtype =" + sqlTo(RTYPE_PMP)
					+"OR a.Frtype =" + sqlTo(RTYPE_PMR)
					+"OR a.Frtype =" + sqlTo(RTYPE_PR)
			+")"			
		var sn1 = getSnap(strSQL);
		if (sn1 != null && sn1[0] != null)
		{
			retVal = sn1[0];
		}
		sn1 = null;
	}
	sn = null;
	
	return retVal;
}

/*
*/
function DpAvto_getRascTkm()
{
	var retVal = 0;
	
	strSQL = "SELECT RR.FRASC"
			+" FROM "
				+" ^CL_RASC10TKM R LEFT JOIN ^CL_RASC10TKM_ROWS RR ON R.FWID_CL = RR.FWID_CL"
			+" WHERE 1=1"
				+" AND R.FCLASS=" + sqlTo(this.aVal["FCLASS"])
				+" AND R.FAVTOGR=" + sqlTo(this.aVal["FGROUPTS"])
				+ sqlw0And(" FBDAT<=", this.aVal["FDAT"])
				+ sqlw0And(" FEDAT>=", this.aVal["FDAT"])
				+ sqlw0And(" R.FVANT<=", this.aVal["FVANT"])
				+ sqlw0And(" R.FROADGR=", this.aVal["FGRDOR"])
				+" ORDER BY FVANT DESC"
	var sn = getSnap(strSQL);
	if (sn != null && sn[0] != null)
	{
		retVal = sn[0];
	}
	sn = null;
	return retVal;
}

/*
*/
function DpAvto_getRascT()
{
	var retVal = 0;
	strSQL = "SELECT RR.FRASC"
			+" FROM (^CLRELCLCL REL LEFT JOIN "
				+" ^CL_RASC10T R ON (REL.FWID_CL = R.FWID_CL AND REL.FCL = "+sqlTo(getPar(const_GETPAR_METHODLOADING))+")) "
				+" LEFT JOIN ^CL_RASC10T_ROWS RR ON R.FWID_CL = RR.FWID_CL"
			+" WHERE 1=1"
				+" AND REL.FCOD=" + sqlTo(this.aVal["FSPPOGR"])
				+" AND R.FCLASS=" + sqlTo(this.aVal["FCLASS"])
				+" AND R.FAVTOGR=" + sqlTo(this.aVal["FGROUPTS"])
				+ sqlw0And(" R.FVANT<=", this.aVal["FVANT"])
				+ sqlw0And(" R.FROADGR=", this.aVal["FGRDOR"])
				+ sqlw0And(" FBDAT<=", this.aVal["FDAT"])
				+ sqlw0And(" FEDAT>=", this.aVal["FDAT"])
				+" ORDER BY FVANT DESC"
	var sn = getSnap(strSQL);
	if (sn != null && sn[0] != null)
	{
		retVal = sn[0];
	}
	sn = null;
	return retVal;
}

function DpAvto_getRascDopr()
{
	var retVal = 0;
	strSQL = "SELECT RR.FRASC"
			+" FROM (^CLRELCLCL REL LEFT JOIN "
				+" ^CL_RASCDOPR R ON (REL.FWID_CL = R.FWID_CL AND REL.FCL = "+sqlTo(getPar(const_GETPAR_DOPRAB))+")) "
				+" LEFT JOIN ^CL_RASCDOPR_ROWS RR ON R.FWID_CL = RR.FWID_CL"
			+" WHERE 1=1"
				+" AND REL.FCOD=" + sqlTo(this.aVal["FDOPRAB"])
				+" AND R.FAVTOGR=" + sqlTo(this.aVal["FGROUPTS"])
				+ sqlw0And(" R.FVANT<=", this.aVal["FVANT"])
				+ sqlw0And(" R.FROADGR=", this.aVal["FGRDOR"])
				+ sqlw0And(" FBDAT<=", this.aVal["FDAT"])
				+ sqlw0And(" FEDAT>=", this.aVal["FDAT"])
				+" ORDER BY FVANT DESC"
	var sn = getSnap(strSQL);
	if (sn != null && sn[0] != null)
	{
		retVal = sn[0];
	}
	sn = null;
	return retVal;
}

//для работы функции нужно установить поле FCODWORKTYPE
//пример 
//oAvto.setVar("FCODWORKTYPE", cod)
//oAvto.getParamWorkType("NORMVYR")
//
//1. mode = "NORMVYR" получить норму выработки 
//2. mode = "NORMFUEL" получить норму топлива 
function DpAvto_getParamWorkType(mode)
{
	var ret = 0;
	var codWT = this.getVar("FCODWORKTYPE");
		
	if(!isEmpty(codWT))
	{
		//вытянем из справочника работ нормы выработки и норму топлива
		var strSQL = "SELECT WT.FNORM_VYROB, WT.FNORM_FUEL, WT.FQUOT" 
					+" FROM ^CLRELCLCL REL LEFT JOIN ^CL_WORKTYPE WT ON REL.FWID_CL = WT.FWID_CL"
					+" WHERE 1=1"
						+" AND REL.FCL = "+sqlTo(this.sprWorktype)
						+" AND REL.FCOD = "+sqlTo(codWT)
		var sn = getSnap(strSQL);
	}
	else
	{
		return ret;
	}
	if(sn != null)
	{
		switch (mode)
		{
			case "NORMVYR":
				ret = (isEmpty(sn[0]) ? 0 : sn[0])
				break;
			case "NORMFUEL":
				ret = (isEmpty(sn[1]) ? 0 : sn[1])
				break;
			case "RASC":
				ret = (isEmpty(sn[2]) ? 0 : sn[2])
				break;
			default:
				break;
		}
	}
	sn = null;
	return ret;
}

//функция вытягивает переменные из справочника транспортные средства 
//и устанавливает их как переменные объекта 
//какие поля нужно получить пишем в переменной fldList
//для работы функции должна быть установлена переменная FAVTO (код из справочника ТС)
function DpAvto_initPar()
{
	if(isEmpty(this.getVar("FAVTO")))
	{
		return;
	}
		
	//Список полей из табл. CL_AVTO которые нужно вытянуть и установить как переменную
	var fldList = "FET_VIR,FGRUZ,FDISTANCE,FNUM,FGARAGE_NUM,FLICENSE";
	
	fldList4SQL = fldList.replace(",", ",A.");
	var strSQL = "SELECT A."+fldList 
				+" FROM ^CLRELCLCL REL LEFT JOIN ^CL_AVTO A ON REL.FWID_CL = A.FWID_CL"
					+" WHERE 1=1"
						+" AND REL.FCL = "+sqlTo(this.sprAvto)
						+" AND REL.FCOD = "+sqlTo(this.getVar("FAVTO"))
	var sn  = getSnap(strSQL);
	
	var n = 0;
	var aFld = fldList.split(",");
	
	//устанавливаем переменные
	for(var i in aFld)
	{
		this.setVar(aFld[i], sn[i])
	}
	
	sn = null;
	return true;
}