/*
	Объект для импорта разной фигни.
    
  	Пример использования:
  	 
	include("Objects/DbImporter.js");
	var oImp = new DbImporter("C:\\nmk.xls", "TMP_NMK");
	//oImp.source = "C:\\nmk.xls";			//источник  
	//oImp.recipient = "TMP_NMK";			//получатель - таблица TMP_NMK
	//oImp.setRange(1, 5);					//установка диапазона импортируемых строк
	oImp.preview = true;					//перед загрузкой в получатель откроется browse с данными
	oImp.addVar("FCOD", 1, "int");			//первый столбик пойдет в поле FCOD
	oImp.addVar("FTXTS", 2, "text(200)");	//второй столбик - в FTXTS
	oImp.run();								//пуск
*/

include("Objects/DpXMLDomDocument.js");
include("Objects/DpExtentionBuilder.js");
include("Objects/DpClEx.js");
include("Objects/DpEval.js");
include("servis/date.js");
include("servis/format.js");
include("doc/DpDoc.js");

function DbImporter(sSource, sRecipient, sRecType, sFileParam, sEncoding, sDelimiter)
{
	this.init = DbImporter_init;	
	this.loadVars = DbImporter_loadVars;	//загрузка переменных из XML
	this.fillVars = DbImporter_fillVars;
	this.saveParam = DbImporter_saveParam;	//сохраняет название переменных и таблиц
	this.runXML = DbImporter_runXML;		//обработка XML
	this.runDBF = DbImporter_runDBF;		//обработка DBF
	this.runCSV = DbImporter_runCSV;		//обработка CSV (*.txt)
	this.runXLS = DbImporter_runXLS;		//обработка Excel (*.xls)
	this.runMDB = DbImporter_runMDB;		//обработка MDB
	this.runJDBC = DbImporter_runJDBC;		//обработка JDBC
	this.run = DbImporter_run;
	this.getTblMap = DbImporter_getTblMap;
	this.getListTbl = DbImporter_getListTbl;
	this.getListMdbTbl = DbImporter_getListMdbTbl;
	this.getListJDBCTbl = DbImporter_getListJDBCTbl;
	this.fillBaseCombo = DbImporter_fillBaseCombo;	
	this.importTbl = DbImporter_importTbl;
	this.importCl = DbImporter_importCl;	//импорт справочников аналитики
	this.importDoc = DbImporter_importDoc;	//импорт документов
	this.importRows = DbImporter_importRows;//создание строк
	this.importDocRows = DbImporter_importDocRows;//импорт строк документов
	this.importPrk = DbImporter_importPrk;//импорт приказов
	this.createSourceTbl = DbImporter_createSourceTbl;
	this.createVarsTbl = DbImporter_createVarsTbl;
	this.createTbl = DbImporter_createTbl;
	this.getSourceName = DbImporter_getSourceName;
	this.getVarsTblName = DbImporter_getVarsTblName;
	this.getVarType = DbImporter_getVarType;
	this.getCorValue = DbImporter_getCorValue;
	this.getNullValue = DbImporter_getNullValue;
	this.getValue = DbImporter_getValue;
	this.getJSFile = DbImporter_getJSFile;
	this.getParFile = DbImporter_getParFile;
	this.setParFile = DbImporter_setParFile;
	this.getPath = DbImporter_getPath;
	this.getObjTableName = DbImporter_getObjTableName;
	this.updateTbl = DbImporter_updateTbl;
	this.updateCl = DbImporter_updateCl;
	this.updateDoc = DbImporter_updateDoc;
	this.getLinks = DbImporter_getLinks;
	this.getMode = DbImporter_getMode;	
	this.isReplace = DbImporter_isReplace;
	this.importObjects = DbImporter_importObjects;
	this.createObject = DbImporter_createObject;
	this.createNmk = DbImporter_createNmk;
	this.createCl = DbImporter_createCl;
	this.createEdi = DbImporter_createEdi;
	this.createCntrct = DbImporter_createCntrct;
	this.getCntrctByOrg = DbImporter_getCntrctByOrg;
	this.findCl = DbImporter_findCl;
	this.saveExt = DbImporter_saveExt;
	this.getDpType = DbImporter_getDpType;
	this.getFieldName = DbImporter_getFieldName;
	this.getColNum = DbImporter_getColNum;
	this.isExt = DbImporter_isExt;
	this.isExpr = DbImporter_isExpr;
	this.getExpr = DbImporter_getExpr;
	this.isObject = DbImporter_isObject;
	this.getObjCod = DbImporter_getObjCod;
	this.getSourceType = DbImporter_getSourceType;
	this.getRowCount = DbImporter_getRowCount;
	this.getXmlTbl = DbImporter_getXmlTbl;
	this.deleteParam = DbImporter_deleteParam;
	this.mergeSettings = DbImporter_mergeSettings;
	this.changeSource = DbImporter_changeSource;
	this.getVarLength = DbImporter_getVarLength;
	this.isFile = DbImporter_isFile;
	this.fillStruTables = DbImporter_fillStruTables;
	this.isStruTable = DbImporter_isStruTable;
	this.addVar = DbImporter_addVar;
	this.setRange = DbImporter_setRange;
	this.browse = DbImporter_browse;
	this.checkCod = DbImporter_checkCod;
	this.checkCondition = DbImporter_checkCondition;
	this.createExtentions = DbImporter_createExtentions;
	this.saveErrors = DbImporter_saveErrors;
		
	this.xmlImporter = lpProxy.getUtils().getDBImporter();
	this.mdbImporter = lpProxy.getUtils().getMDBImporter();
	this.jdbcImporter = lpProxy.getUtils().getJDBCImporter();
	this.DpAPI = lpProxy.getDpAPI();
	this.oEval = new DpEval();
	this.varsTblName = getTmpTableName();	
	this.sourceTblName = getTmpTableName();
	this.recTblName = "TMP_IMPORT";
	
	this.fldMap = new Object();		//массив полей
	this.varMap = new Object();		//массив переменных
	this.arrVars = new Object();		
	this.rowMap = new Object();		//массив строк документов
	this.arrObj = new Object();
	this.arrType = new Object();
	this.arrTbl = new Array();		//массив импортируемых таблиц
	this.arrErr = new Array(); 		//ошибки
	this.arrStruTables = null;
	this.path = getOrgPath() +"import/";
	this.sFileParam = ""; 			//файл настройки
	this.source = ""; 				//источник
	this.sourceNum = 0;
	this.encoding = "cp1251";		//кодировка
	this.type = "";					//тип источника
	this.base = "";
	this.mode = "";
	this.sJsFile = "";				//выполняемый скрипт
	this.delimiter = "";			//разделитель для CSV
	this.replace = false;
	this.recipient = this.recTblName;
	this.objType = "";
	this.objCod = 0;
	this.objID = 0;
	this.linkName = "";
	this.linkSource = "";
	this.groupRow = false;
	this.groupCond = "";
	this.groupField = "";
	this.groupKey = "";	
	this.condition = "";
	this.conditionMode = "";
	this.conditionCol = "";
	this.showMsg = true;
	this.preview = false;
	this.nStart = 1;
	this.nEnd = 0;
	this.maxCod = 0;
	this.rowsGroupCount = SUBD==ORACLE?1:10;
	this.addNotExisting = false;
	this.passExisting = false;
	
	this.init(sSource, sFileParam, sEncoding, sDelimiter, sRecipient, sRecType);
}

function DbImporter_init(sSource, sFileParam, sEncoding, sDelimiter, sRecipient, sRecType)
{	
	this.source = sSource;
	if(sFileParam)
		this.sFileParam = sFileParam;		
	if(sEncoding)
		this.encoding = sEncoding;		
	if(sDelimiter)
		this.delimiter = sDelimiter;
	if(sRecipient)
		this.recipient = sRecipient;
	if(sRecType)
		this.objType = sRecType;	
	this.type = this.getSourceType(sSource);
}

function DbImporter_getSourceType(sFile)
{
	var sType = "XML";
	if(!sFile)
		sFile = this.source;
	if(!sFile || sFile.indexOf(".")==-1)
		return sType;
	
	sFile = sFile.toLowerCase();
	if(sFile.endsWith(".xml"))
		sType = "XML";	
	else if(sFile.endsWith(".dbf"))
		sType = "DBF";
	else if(sFile.endsWith(".xls"))
		sType = "XLS";
	else if(sFile.endsWith(".mdb"))
		sType = "MDB";
	else //if(sFile.endsWith(".txt"))
		sType = "CSV";
		
	return sType;
}

function DbImporter_getRowCount(sFileSource, encoding, delimiter)
{	
	if(!sFileSource)
		sFileSource = this.source;
	if(!encoding)
		encoding = this.encoding;
	if(!delimiter)
		delimiter = this.delimiter;
	var sType = this.getSourceType(sFileSource);
	var nCount = 0;
	if(sType=="XML")
		return 0;
	if(sType=="DBF")
	{
		var tbl = OpenTable("f", sFileSource, encoding);
		nCount = tbl.recordCount;
	}
	else if(sType=="XLS")
	{
		var exldoc = lpProxy.openExcelWorkbook(sFileSource);
		if(exldoc)
			nCount = Number(exldoc.getSheet(exldoc.getSheetName(0)).getLastRowNum())+1;	
	}
	else if(sType=="CSV")
	{
		var txt = lpProxy.readFile(sFileSource, encoding);
		var arrCSV = lpProxy.csvParse(txt, String(delimiter));
		if(arrCSV)
			nCount = Number(arrCSV.length);
	}
	return nCount;
}

function DbImporter_loadVars(sXmlFile, sSourceName, nSourceNum, sTblVars)
{		
	this.fldMap = new Object();
	this.varMap = new Object();
	this.arrVars = new Object();		
	this.rowMap = new Object();	
	this.arrType = new Object();
	
	if(!sXmlFile)
		sXmlFile = this.sFileParam;
	if(!sSourceName)
		sSourceName = this.getSourceName();
	if(!sTblVars)
		sTblVars = this.getVarsTblName();	
		
	this.createVarsTbl(sTblVars);
	
	var sPath = this.getPath();
	if(!isDir(sPath))
		createDirectory(sPath);

	if(isFile(sXmlFile))
	{		
		var xmlDoc = new DpXMLDOMDocument();
		xmlDoc.load(sXmlFile);
		var xmlDocElement = xmlDoc.getDocumentElement();
		var eTbl = this.getXmlTbl(xmlDocElement, sSourceName, nSourceNum);
		if(eTbl)
		{		
			if(!this.source)
				this.source = eTbl.getAttribute("source");
			this.sourceNum = eTbl.getAttribute("num");
			this.recipient = eTbl.getAttribute("name");
			this.encoding = eTbl.getAttribute("encoding");
			this.delimiter = eTbl.getAttribute("delimiter");
			this.objType = eTbl.getAttribute("type");
			this.objCod = eTbl.getAttribute("objCod");
			this.objID = eTbl.getAttribute("objID");
			this.mode = eTbl.getAttribute("mode");
			this.linkName = eTbl.getAttribute("linkName");			
			this.linkSource = eTbl.getAttribute("linkSource");
			this.groupRow = String(eTbl.getAttribute("groupRow"))=="true" || String(eTbl.getAttribute("groupRow"))=="1";
			this.groupCond = eTbl.getAttribute("groupCond");
			this.groupField = eTbl.getAttribute("groupField");
			this.groupKey = eTbl.getAttribute("groupKey");				
			this.condition = eTbl.getAttribute("condition");
			this.conditionMode = eTbl.getAttribute("conditionMode");
			this.conditionCol = eTbl.getAttribute("conditionCol");
			this.replace = String(eTbl.getAttribute("replace"))=="true" || String(eTbl.getAttribute("replace"))=="1";
			this.addNotExisting = String(eTbl.getAttribute("addNotExisting"))=="true" || String(eTbl.getAttribute("addNotExisting"))=="1";
			var sPassExisting = String(eTbl.getAttribute("passExisting"));
			this.passExisting = (sPassExisting=="true" || sPassExisting=="1");
			this.preview = String(eTbl.getAttribute("preview"))=="true" || String(eTbl.getAttribute("preview"))=="1";
							
			var eFields = eTbl.getElementsByTagName("fields")[0];		
			var varList = eFields.getChildNodes();			
			for (var i=0; i<varList.length; i++)
			{
				var eVar = varList[i];	
				var sSource = eVar.getAttribute("source");				
				var sFldTo = eVar.getAttribute("name");
				var sType = eVar.getAttribute("type");
				var newSource = eVar.getAttribute("newsource");				
				var sFldFrom = sSource;
								
				if(this.getFieldName(sFldFrom) && this.type!="DBF")
					sFldFrom = this.getFieldName(sFldFrom);
				if(sFldFrom)			
					this.fldMap[sFldFrom] = sFldTo;
				if(sFldTo)
				{
					this.varMap[sFldTo] = sFldFrom;
					this.arrType[sFldTo] = sType;
					var oVar = new DbImporter_var(sFldTo, sFldFrom, sType);
					oVar.length = Number(eVar.getAttribute("length"));
					this.arrVars[sFldTo] = oVar;
				}					
				
				var m = new Object();
				m.FSOURCE = sSource;
				if(this.type=="XML" && !sSource)
					m.FSOURCE = sFldTo;
				m.FREC = sFldTo;
				m.FTYPE = sType;
				m.FPARAM = eVar.getAttribute("param");
				m.FNEWSOURCE = (newSource=="1" || newSource=="true");
				m.FOBJSOURCE = eVar.getAttribute("objsource");
				m.FLINK = eVar.getAttribute("link");
				m.FOBJCOD = eVar.getAttribute("objcod");
				m.FTXT = eVar.getText();
				m.FLENGTH = Number(eVar.getAttribute("length"));
	
				if(sTblVars)
					UpdateTable(sTblVars, m, true);			
			}
			
			if(this.objType=="cl" || this.objType=="doc"  || this.objType=="docrow" || this.objType=="prk")
			{
				var eRows = eTbl.getElementsByTagName("rows")[0];
				if(!eRows)
					return this.varMap;
				var rowList = eRows.getChildNodes();
				for (var i=0; i<rowList.length; i++)
				{
					var eRow = rowList[i];
					var sRowClass = eRow.getAttribute("class");					
					var varList = eRow.getChildNodes();					
					var oVars = new Object();
					for (var j=0; j<varList.length; j++)
					{
						var eVar = varList[j];	
						var sSource = eVar.getAttribute("source");
						var sParam = eVar.getAttribute("param");
						var sFldTo = eVar.getAttribute("name");
						var sType = eVar.getAttribute("type");
						var newSource = eVar.getAttribute("newsource");
						var sObjSource = eVar.getAttribute("objsource");
						var sTxt = eVar.getText();			
						var sFldFrom = sSource;
												
						if(this.getFieldName(sFldFrom))
							sFldFrom = this.getFieldName(sFldFrom);
						if(sFldTo)
						{
							oVars[sFldTo] = sFldFrom;
							this.arrType[sFldTo] = sType;
						}
						if(sFldFrom)
							this.fldMap[sFldFrom] = sFldTo;
											
						var m = new Object();
						m.FSOURCE = sSource;
						m.FPARAM = sParam;
						m.FREC = sFldTo;
						m.FTYPE = sType;
						m.FROW = true;
						m.FROWCLASS = sRowClass;
						m.FNEWSOURCE = (newSource=="1" || newSource=="true");
						m.FOBJSOURCE = sObjSource;
						m.FLINK = eVar.getAttribute("link");
						m.FOBJCOD = eVar.getAttribute("objcod");
						m.FLENGTH = Number(eVar.getAttribute("length"));
						m.FTXT = sTxt;
						
						if(sTblVars)
							UpdateTable(sTblVars, m, true);
					}					
					this.rowMap[sRowClass] = oVars;
				}
			}
			var arrLinks = this.getLinks();
			if(arrLinks)
			{
				for(var fld in arrLinks)
				{
					var sFld = this.getFieldName(fld);
					if(!this.fldMap[sFld] && !this.fldMap[fld])
						this.fldMap[sFld] = arrLinks[fld];
				}
			}
			
			return this.varMap;
		}
	}	
		
	this.fillVars(this.varMap, sTblVars);
	this.fldMap = this.varMap;
	
	return this.varMap;	
}

function DbImporter_fillVars(arrVars, sTblVars)
{
	if(!arrVars)
		arrVars = new Object();
	if(!isFile(this.source))
		return arrVars;
	if(sTblVars && !IsTableExist(sTblVars))
		this.createVarsTbl(sTblVars);
	this.type = this.getSourceType(this.source);
	if(this.type=="DBF")
	{
		var tbl = OpenTable("f", this.source, this.encoding);		
		var varList = tbl.getFields("").split(",");
		for (var i=0; i<varList.length; i++)
		{
			var sFld = varList[i];
			arrVars[sFld] = sFld;			
						
			var m = new Object();
			m.FNUM = Number(i+1);
			m.FCOD = String(i+1);
			m.FSOURCE = sFld;
			m.FREC = sFld;
			m.FTYPE = "text(255)";
			m.FTXT = "";
			if(tbl.getValue(sFld))
				m.FTXT = String(tbl.getValue(sFld));
			if(sTblVars)
				UpdateTable(sTblVars, m, true);
		}
	}
	
	if(this.type=="CSV")
	{
		var txt = lpProxy.readFile(this.source, this.encoding);
		var arrCSV = lpProxy.csvParse(txt, String(this.delimiter));
		if(!arrCSV)
			return arrVars;
		var str = arrCSV[0];
		var i = 0;
		while(!str || str.length<1)
		{
			str = arrCSV[i++];
		}
		if(!str)
			return arrVars;

		for(var j=0; j<str.length; j++)
		{
			var nFld = Number(j+1);
			var sFld = "F"+ nFld;
			var m = new Object();
			
			m.FNUM = nFld;
			m.FCOD = String(nFld);
			m.FSOURCE = String(nFld);
			m.FREC = sFld;
			m.FTYPE = "text(255)";
			m.FTXT = "";
			if(str[j])
				m.FTXT = String(str[j]);
			if(sTblVars)
				UpdateTable(sTblVars, m, true);
			arrVars[nFld] = sFld;			
		}
	}
	
	if(this.type=="XLS")
	{
		var exldoc = lpProxy.openExcelWorkbook(this.source);
		if(!exldoc)
			return arrVars;
		var lastRow	= exldoc.getSheet(exldoc.getSheetName(0)).getLastRowNum();
		var oRow = exldoc.getSheet(exldoc.getSheetName(0)).getRow(0);
		if(!oRow || oRow.getPhysicalNumberOfCells()<1)
			oRow = exldoc.getSheet(exldoc.getSheetName(0)).getRow(1);
		if(!oRow)
			return arrVars;
		for(var i=0; i<oRow.getPhysicalNumberOfCells(); i++)
		{
			var nFld = Number(i+1);
			var sFld = "F"+ nFld;
			var m = new Object();
			
			m.FNUM = nFld;
			m.FCOD = String(nFld);
			m.FSOURCE = String(nFld);
			m.FREC = sFld;
			m.FTYPE = "text(255)";
			m.FTXT = "";
			var oCell = oRow.getCell(i);
			if(oCell)
			{				
				if(oCell.getCellType()==0)
					m.FTXT = String(oCell.getNumericCellValue());
				else if(oCell.getCellType()==1)
					m.FTXT = String(oCell.getStringCellValue());
			}
			if(sTblVars)
				UpdateTable(sTblVars, m, true);
			arrVars[nFld] = sFld;			
		}
	}
	return arrVars;
}

function DbImporter_addVar(sVar, sSourceVar, sType)
{
	this.varMap[sVar] = sSourceVar;
	var sFld = this.getFieldName(sSourceVar);
	if(sFld)
		this.fldMap[sFld] = sVar;
	this.arrType[sVar] = sType;
}

function DbImporter_getListTbl(sXmlFile)
{
	if(!isFile(sXmlFile))
		return null;
	if(String(sXmlFile).endsWith(".mdb"))
	{
		return this.getListMdbTbl(sXmlFile);
	}	
		
	var xmlDoc = new DpXMLDOMDocument();
	xmlDoc.load(sXmlFile);
	var xmlDocElement = xmlDoc.getDocumentElement();
	var eTables = xmlDocElement.getElementsByTagName("tables")[0];
	if(!eTables)
		return null;
	var tblList = eTables.getChildNodes();
	return tblList;
}

function DbImporter_getListMdbTbl(sMdbFile)
{
	if(!isFile(sMdbFile))
		return null;
	var db = null;
	var e;
	try
	{	
		var db = lpProxy.getUtils().openMDB(sMdbFile);		
	}
	catch(e)
	{
		
	}
	if (db == null)
		return; 
	
	var tables = db.getTableNames().toArray();
	var tblList = new Array();	
	for (var i = 0; i < tables.length; i++)
	{
		var m = new Object()
		m.tableName = tables[i];
		m.getAttribute = mdbTable_getAttribute;
		tblList[i] = m;
		
	}
	db.close();
	return tblList;
}

function DbImporter_getListJDBCTbl(strConn, user, password, catalog,schema)
{
	var conn = null;
	
	try
	{	
		conn = getDBConnectionEx(strConn, user, password);		
	}
	catch(e)
	{
		alert(e)
	}
	
	if(conn == null)
		return;
	
	var arrTypes = new Array();
	arrTypes[0] = "TABLE";
	var dbmd = conn.getMetaData();
	var isMSAccess = String(dbmd.getDatabaseProductName()) == "ACCESS";
	if (isMSAccess)
	{
		catalog = null;
		schema = null;
	}
	var rs = dbmd.getTables(catalog?catalog:null, schema?schema:null, null, arrTypes);	
	
	var tblList = new Array();	
	while (rs.next())
	{
		
		var m = new Object();
		m.tableName = String(rs.getString("TABLE_NAME"));
		m.tableSchem = isMSAccess?null:(rs.getString("TABLE_SCHEM"));
		m.tableCat = isMSAccess?null:(rs.getString("TABLE_CAT"));
		trace(m.tableName + " - " + m.tableSchem + " - " + m.tableCat);
		if(m.tableCat && m.tableSchem)
			m.tableName = m.tableCat + "." + m.tableSchem + "." + m.tableName;
		if(m.tableCat && m.tableSchem == null)
			m.tableName = m.tableCat + "." + m.tableName;
		if(m.tableCat == null && m.tableSchem)
			m.tableName = m.tableSchem + "." + m.tableName;		
		trace(m.tableName);
		m.getAttribute = jdbcTable_getAttribute;
		tblList[tblList.length] = m;
		
	}
	rs.close();
	conn.close();
	return tblList;
}

function mdbTable_getAttribute(attrName)
{
	if (attrName == "name" || attrName == "source")
		return this.tableName;
	else
		return "";
}

function jdbcTable_getAttribute(attrName)
{
	if (attrName == "source")
	{
		return this.tableName;
	}
	else if (attrName == "name")
	{
		return this.tableName.substr(this.tableName.lastIndexOf(".") + 1);;
	}
	else
		return "";
}





function DbImporter_getXmlTbl(xmlDocElement, sSourceName, nSourceNum)
{		
	var eTables = xmlDocElement.getElementsByTagName("tables")[0];
	var tblList = eTables.getChildNodes();
	for (var i=0; i<tblList.length; i++)
	{
		var elem = tblList[i];		
		var num = Number(elem.getAttribute("num"));
		var sName = elem.getAttribute("source");
		if(!sName)
			sName = elem.getAttribute("name");
		if(nSourceNum)
		{
			if(num==Number(nSourceNum))
				return elem;
		}
		else if(sSourceName && sName == sSourceName)
		{
			return elem;
		}
	}
	if(tblList.length==1)
		return tblList[0];
	return null;
}

function DbImporter_getTblMap()
{	
	return this.arrTbl;
}

function DbImporter_fillBaseCombo(objCtrl)
{
	var arrBase = lpProxy.getDBManager().getDatabases();
	for (var nB = 0; nB < arrBase.length; nB++)
	{
		objCtrl.addItem(arrBase[nB],arrBase[nB]);
	}
}

function DbImporter_mergeSettings(sNewXml)
{
	var sFileParam = this.sFileParam;
	var xmlFile = new DpFile(sFileParam);
	if(!xmlFile.exists())
		return false;
	
	var xmlSource = new DpFile(sNewXml);
	if(!xmlSource.exists())
		return false;
	
	var xmlDoc = new DpXMLDOMDocument();
	xmlDoc.load(sFileParam);
	var xmlImp = xmlDoc.getDocumentElement();
	var eTables = xmlImp.getElementsByTagName("tables")[0];
	if(!eTables)
		return;
		
	var xmlDocSource = new DpXMLDOMDocument();
	xmlDocSource.load(sNewXml);
	var eSource = xmlDocSource.getDocumentElement();
	var eSourceTables = eSource.getElementsByTagName("tables")[0];
	if(!eSourceTables)
		return;
	
	var curNum = eTables.length+1;
	var tblList = eSourceTables.getChildNodes();
	for (var i=0; i<tblList.length; i++)
	{
		var eTbl = tblList[i];
		var nTbl = Number(eTbl.getAttribute("num"));
		var sTbl = eTbl.getAttribute("source");		
		this.sourceNum = curNum++;
		/*
		var newTbl =  xmlDoc.createElement("table");
		newTbl.setAttribute("num", this.sourceNum);
		newTbl.setAttribute("source", sTbl);
		eTables.appendChild(newTbl);
		xmlDoc.save(sFileParam);
		*/
		this.loadVars(sNewXml, sTbl, nTbl);	
		this.saveParam(sFileParam, curNum, "", "ADD");
	}	
}

function DbImporter_deleteParam(nSourceNum, sSource, sXmlFile)
{
	if(!sXmlFile)
		sXmlFile = this.sFileParam;
	var xmlFile = new DpFile(sXmlFile);
	var xmlDoc = new DpXMLDOMDocument();
	if(xmlFile.exists())
	{
		xmlDoc.load(sXmlFile);
		var xmlImp = xmlDoc.getDocumentElement();
		var eTables = xmlImp.getElementsByTagName("tables")[0];
		var eTbl = this.getXmlTbl(xmlImp, sSource, nSourceNum);
		if(eTbl && eTables)
		{			
			eTables.removeChild(eTbl);			
			return xmlDoc.save(xmlFile);	
		}
	}
}

function DbImporter_changeSource(nSourceNum, sNewSource, sOldSource)
{
	var sXmlFile = this.sFileParam;
	var xmlFile = new DpFile(sXmlFile);
	var xmlDoc = new DpXMLDOMDocument();
	if(xmlFile.exists())
	{
		xmlDoc.load(sXmlFile);
		var xmlImp = xmlDoc.getDocumentElement();		
		var eTbl = this.getXmlTbl(xmlImp, sOldSource, nSourceNum);
		if(eTbl)
		{			
			eTbl.setAttribute("source", sNewSource);
			return xmlDoc.save(xmlFile);	
		}
	}
}

function DbImporter_saveParam(sXmlFile, nSourceNum, sSource, sMode)
{				
	var sTblVars = this.getVarsTblName();
	var tVars = OpenTable("select * from "+ sTblVars 
						+" where FROW<>"+ sqlTrue +" or FROW is NULL");
	var tRowVars = OpenTable("select FROWCLASS from "+ sTblVars 
						+" where FROW="+ sqlTrue 
						+" group by FROWCLASS");
						
	if(!sXmlFile)
		sXmlFile = this.sFileParam;
	var xmlFile = new DpFile(sXmlFile);
	var sPath = xmlFile.getParent();	
	var path = new DpFile(sPath)
	if(!path.isDirectory())
		createDirectory(sPath);
	if(!nSourceNum)
		nSourceNum = this.sourceNum;
	if(!sSource)
		sSource = this.source;
	var xmlImp;
	var eFields;
	var eLinks;
	var eRows;
	var xmlDoc = new DpXMLDOMDocument();
	if(isFile(sXmlFile))
	{		
		xmlDoc.load(sXmlFile);
		xmlImp = xmlDoc.getDocumentElement();
		var eTables = xmlImp.getElementsByTagName("tables")[0];
		if(sMode!="ADD" && sMode!="COPY")
		{
			var eTbl = this.getXmlTbl(xmlImp, null, nSourceNum);
			if(!eTbl)
				eTbl = this.getXmlTbl(xmlImp, sSource, nSourceNum);
			if(eTbl)
				eTables.removeChild(eTbl);
		}
		var eTbl = xmlDoc.createElement("table");
		eTables.appendChild(eTbl);
						
		eLinks = eTbl.getElementsByTagName("links")[0];
		if(eLinks!=null)	
			eTbl.removeChild(eLinks);
		eLinks = xmlDoc.createElement("links");
		eTbl.appendChild(eLinks);			
		
		eFields = eTbl.getElementsByTagName("fields")[0];		
		if(eFields!=null)
			eTbl.removeChild(eFields);
		eFields = xmlDoc.createElement("fields");
		eTbl.appendChild(eFields);
		
		eRows = eTbl.getElementsByTagName("rows")[0];
		if(this.type!="XML")
		{		
			if(eRows!=null)	
				eTbl.removeChild(eRows);
			eRows = xmlDoc.createElement("rows");			
		}
		if(eRows)
			eTbl.appendChild(eRows);
	}	
	else
	{
		xmlImp = xmlDoc.createElement("import");		
		xmlDoc.appendChild(xmlImp);	
		var eTables = xmlDoc.createElement("tables");		
		xmlImp.appendChild(eTables);
		var eTbl = xmlDoc.createElement("table");		
		eTables.appendChild(eTbl);
		eLinks = xmlDoc.createElement("links");
		eTbl.appendChild(eLinks);	
		eFields = xmlDoc.createElement("fields");
		eTbl.appendChild(eFields);
		eRows = xmlDoc.createElement("rows");
		eTbl.appendChild(eRows);
	}
	
	if(eLinks && this.linkName && this.linkName!="" && this.linkSource && this.linkSource!="")
	{
		var link = xmlDoc.createElement("link");
		link.setAttribute("name", String(this.linkName));
		link.setAttribute("source", String(this.linkSource));		
		eLinks.appendChild(link);	
	}
	
	for(tVars.MoveFirst(); !tVars.isEOF(); tVars.MoveNext())
	{
		var fld = xmlDoc.createElement("field");
		var sTxt = tVars.getValue("FTXT");		
		var sFldTo = tVars.getValue("FREC");
		var sType = tVars.getValue("FTYPE");
		var sFldFrom = tVars.getValue("FSOURCE");
		if(sFldFrom===null)
			sFldFrom = "";
		
		fld.setText(sTxt);
		setAttribute(fld, "source", sFldFrom);	
		setAttribute(fld, "name", tVars.getValue("FREC"));
		setAttribute(fld, "type", tVars.getValue("FTYPE"));
		setAttribute(fld, "length", Number(tVars.getValue("FLENGTH")));
		setAttribute(fld, "param", tVars.getValue("FPARAM"));
		setAttribute(fld, "newsource", tVars.getValue("FNEWSOURCE"));
		setAttribute(fld, "objsource", tVars.getValue("FOBJSOURCE"));
		setAttribute(fld, "link", tVars.getValue("FLINK"));
		setAttribute(fld, "objcod", tVars.getValue("FOBJCOD"));		
		
		eFields.appendChild(fld);		
	}
	tVars.close();
	
	for(tRowVars.MoveFirst(); !tRowVars.isEOF(); tRowVars.MoveNext())
	{
		var sRowClass = String(tRowVars.getValue("FROWCLASS"));
		tVars = OpenTable("select * from "+ sTblVars 
						+" where FROWCLASS="+ sqlTo(sRowClass));
		var eRow = xmlDoc.createElement("row");
		eRow.setAttribute("class", sRowClass);
		for(tVars.MoveFirst(); !tVars.isEOF(); tVars.MoveNext())
		{
			var fld = xmlDoc.createElement("field");
			var sFldFrom = tVars.getValue("FSOURCE");
			if(sFldFrom===null)
				sFldFrom = "";		
			fld.setText(tVars.getValue("FTXT"));
			setAttribute(fld, "source", sFldFrom);
			setAttribute(fld, "param", tVars.getValue("FPARAM"));	
			setAttribute(fld, "name", tVars.getValue("FREC"));	
			setAttribute(fld, "type", tVars.getValue("FTYPE"));
			setAttribute(fld, "length", Number(tVars.getValue("FLENGTH")));
			setAttribute(fld, "newsource", tVars.getValue("FNEWSOURCE"));
			setAttribute(fld, "objsource", tVars.getValue("FOBJSOURCE"));				
			setAttribute(fld, "link", tVars.getValue("FLINK"));
			setAttribute(fld, "objcod", tVars.getValue("FOBJCOD"));
			eRow.appendChild(fld);
		}
		tVars.close();
		if(eRows)
			eRows.appendChild(eRow);
	}
	tRowVars.close();
	
	setAttribute(eTbl, "num", this.sourceNum);
	setAttribute(eTbl, "source", this.source);		
	setAttribute(eTbl, "name", this.recipient);
	setAttribute(eTbl, "encoding", this.encoding);
	setAttribute(eTbl, "delimiter", this.delimiter);
	setAttribute(eTbl, "type", this.objType);
	setAttribute(eTbl, "objCod", this.objCod);
	setAttribute(eTbl, "objID", this.objID);
	setAttribute(eTbl, "mode", this.mode);
	setAttribute(eTbl, "linkName", this.linkName);			
	setAttribute(eTbl, "linkSource", this.linkSource);
	setAttribute(eTbl, "groupRow", this.groupRow);
	setAttribute(eTbl, "groupCond", this.groupCond);
	setAttribute(eTbl, "groupField", this.groupField);
	setAttribute(eTbl, "groupKey", this.groupKey);				
	setAttribute(eTbl, "replace", this.replace);
	setAttribute(eTbl, "addNotExisting", this.addNotExisting);
	setAttribute(eTbl, "passExisting", this.passExisting);
	setAttribute(eTbl, "preview", this.preview);
	setAttribute(eTbl, "condition", this.condition);
	setAttribute(eTbl, "conditionMode", this.conditionMode);
	setAttribute(eTbl, "conditionCol", this.conditionCol);
	
	xmlDoc.save(xmlFile);	
}

function setAttribute(elem, sAttr, oVal)
{
	if(elem && oVal!=null && oVal!="null" && String(oVal)!="")
		elem.setAttribute(sAttr, String(oVal));
}

function DbImporter_run(bLoadSettingsFromXML)
{
	if(bLoadSettingsFromXML)
	{
		if(!this.isFile(this.sFileParam))
			return false;
		
		this.loadVars(this.sFileParam, this.source, this.sourceNum);
	}
	
	cleanParam();
	
	if (this.sFileParam && this.sFileParam.endsWith(".mdb"))	
		this.type = "MDB";
	else if (this.jdbcConnectString)	
		this.type = "JDBC";
	else	
		this.type = this.getSourceType(this.source);
	
	this.getJSFile();
	
	if(typeof startImport=="function")
	{
		this.startImport = startImport;
		this.startImport();
	}
	
	var createArchive = bCreateArchive;
	bCreateArchive = false;
	
	var bRet = true;
	switch(this.type)
	{
		case "XML":
			bRet = this.runXML();
			break;
			
		case "MDB":		
			bRet = this.runMDB();
			break;
		
		case "DBF":
			bRet = this.runDBF();
			break;
		
		case "CSV":
			bRet = this.runCSV();
			break;
			
		case "XLS":
			bRet = this.runXLS();
			break;
			
		case "JDBC":
			bRet = this.runJDBC();
			break;
		
		default:
			alert(ru("Неизвестный формат: '"+ this.type +"'"));
			bRet = false;
			break;
	}
	
	if(typeof endImport=="function")
	{
		this.endImport = endImport;
		this.endImport();
	}
	bCreateArchive = createArchive;
	
	this.saveErrors();
	CLEARWAIT();
	return bRet;
}

function DbImporter_saveErrors(arrErr, sPath)
{
	if(!arrErr)
		arrErr = this.arrErr;
	if(arrErr.length==0)
		return;
	if(!sPath)
		sPath = DpFile(this.sFileParam).getParent();
	var sParam = String(DpFile(this.sFileParam).getName());
	sParam = sParam.substring(0, sParam.length-4);
	var sFile = sPath +"/"+ sParam +"_"+ this.sourceNum +"_ERRORS.txt";
	trace("\nERRORS: "+ arrErr.length);
	var sError = format("dd.mm.yyyy  HH:MM", new Date()) +" - "+ this.source +"\n";
	for(var s=0; s<arrErr.length; s++)		
	{
		trace(arrErr[s]);
		sError += "\n"+ arrErr[s];
	}
	lpProxy.writeToFile(sError, sFile, "utf-8");
}

function DbImporter_isFile(sFile)
{
	if(!sFile)
		sFile = this.sFileParam;
	if(!isFile(sFile))
	{
		alert(UR("Не знайдено файл: |Не найден файл: ") + sFile);
		return false;
	}
	return true;
}

function DbImporter_runXML(oPar)
{
	if(!this.isFile(this.sFileParam))
		return false;
	
	if(!oPar)
	{
		oPar = new Object();
		oPar.importer = this.xmlImporter;	
		oPar.sFile = this.sFileParam;
		oPar.sBase = this.base;
		oPar.tables = this.arrTbl;
		oPar.actionType = "IMPORT";
	}
	if(!oPar.tables || oPar.tables.length<=0)
		return false;
	
	showWindow("export_progress.xml", SW_MODAL, oPar);
	return true;
}

function DbImporter_runMDB(oPar)
{
	if(!this.isFile(this.sFileParam))
		return false;
	
	if(!oPar)
	{
		oPar = new Object();
		oPar.importer = this.mdbImporter;	
		oPar.sFile = this.sFileParam;
		oPar.sBase = this.base;
		oPar.tables = this.arrTbl;
		oPar.actionType = "IMPORT";
	}
	if(!oPar.tables || oPar.tables.length<=0)
		return false;
	
	showWindow("export_progress.xml", SW_MODAL, oPar);
	return true;
}

function DbImporter_runJDBC(oPar)
{
	if(!oPar)
	{
		oPar = new Object();
		oPar.importer = this.jdbcImporter;	
		oPar.jdbcConnectString = this.jdbcConnectString;
		oPar.jdbcUser = this.jdbcUser;
		oPar.jdbcPassword = this.jdbcPassword;
		oPar.sBase = this.base;
		oPar.tables = this.arrTbl;
		oPar.actionType = "IMPORT";
	}
	if(!oPar.tables || oPar.tables.length<=0)
		return false;
	
	showWindow("export_progress.xml", SW_MODAL, oPar);
	return true;
}

function DbImporter_runDBF(sTblTo)
{
	if(!this.isFile(this.source))
		return false;
	
	var tbl = OpenTable("f", this.source, this.encoding);
		
	if(!sTblTo)
		sTblTo = this.recipient;
	var sType = this.objType;
	var objCod = this.objCod;
	var objID = this.objID;
	var bGroupRow = this.groupRow;
	var sGroupCond = this.groupCond;
	var sGroupField = this.groupField;
	var sGroupKey = this.groupKey;
	
	if(this.preview)
	{
		var bRet = this.browse(tbl);
		if(!bRet)
		{
			tbl.close();
			return true;
		}
	}
		
	var bRet = this.importObjects();
	if(!bRet)
		return false;
		
	this.oEval.setTable(tbl);
	if(sType == "cl")
		bRet = this.importCl(sTblTo, objCod, tbl);
	else if(sType == "doc")
		bRet = this.importDoc(sTblTo, objCod, tbl, bGroupRow, sGroupCond, sGroupField, sGroupKey);
	else if(sType == "docrow")
		bRet = this.importDocRows(tbl, sTblTo, objCod, objID);
	else if(sType == "prk")
		bRet = this.importPrk(sTblTo, tbl);
	else
		bRet = this.importTbl(sTblTo, tbl, !this.replace, this.base, objCod);	
	tbl.close();
	return bRet;
}

function DbImporter_runCSV(sTblTo, bPreview)
{
	if(!this.isFile(this.source))
		return false;
	
	//this.loadVars(this.sFileParam, this.source, this.sourceNum);
	
	if(!sTblTo)
		sTblTo = this.recipient;
	var sType = this.objType;
	var objCod = this.objCod;
	var objID = this.objID;
	var bGroupRow = this.groupRow;
	var sGroupCond = this.groupCond;
	var sGroupField = this.groupField;
	var sGroupKey = this.groupKey;
	
	var sTmpTbl = this.getSourceName();
	if(!this.createSourceTbl(sTmpTbl))
		return false;
	
	var bRet = this.importObjects();
	if(!bRet)
		return false;
	
	var txt = lpProxy.readFile(this.source, this.encoding);
	if(txt.indexOf("\"")!=-1 && confirm(UR("Вирізати лапки'?|Вырезать кавычки?")))
		txt = txt.replaceAll("\"", "");
	var arrCSV = lpProxy.csvParse(txt, String(this.delimiter));
	if(!arrCSV)
		return false;
	
	var nrow = 0;
	var lastRow = arrCSV.length;
	if(this.nStart>1)
		nrow = this.nStart-1;
	if(this.nEnd>0 && this.nEnd<lastRow)
		lastRow = this.nEnd;
	
	for(; nrow<lastRow; nrow++)
	{
		var str = arrCSV[nrow];
		if(!str)
			continue;
		
		if(Number(this.conditionCol)>0)
		{
			var oVal = str[Number(this.conditionCol)-1];
			if(!this.checkCondition(oVal))
				continue;
		}
			
		var isEmpty = true;	
		var m = new Object();
		for(fld in this.fldMap)
		{			
			var sFld = this.getFieldName(fld);
			if(!sFld)
				continue;
			var n = Number(sFld.substring(1));
			var sVal = String(str[n-1]);
			if(typeOf(sVal)=="string" || typeOf(sVal)=="String")
			{
				sVal = sVal.trim();
				sVal = sVal.substring(0, 250);
			}
			//sVal = this.getCorValue(sVal, this.getVarType(fld));
			//trace(">>> DbImporter_runCSV: "+ sFld +" - "+ sVal)
			m[sFld] = sVal;
			isEmpty = false;
		}
		var bRet = true;
		if(!isEmpty)
			bRet = UpdateTable(sTmpTbl, m, true);
		if(!bRet && stopMassage(this.source))
			return false;
	}	
	
	if(bPreview)
		return sTmpTbl;
	
	sGroupField = this.getFieldName(sGroupField);
	var sSql = "select * from "+ sTmpTbl;
	if(bGroupRow && sGroupField && sGroupCond=="GROUP_FIELD")
		sSql += " order by "+ sGroupField;
	var tbl = OpenTable(sSql);
	this.oEval.setTable(tbl);
	
	if(this.preview)
	{
		var bRet = this.browse(tbl);
		if(!bRet)
		{
			tbl.close();
			return true;
		}
	}

	if(sType == "cl")
		bRet = this.importCl(sTblTo, objCod, tbl);
	else if(sType == "doc")
		bRet = this.importDoc(sTblTo, objCod, tbl, bGroupRow, sGroupCond, sGroupField, sGroupKey);
	else if(sType == "docrow")
		bRet = this.importDocRows(tbl, sTblTo, objCod, objID);
	else if(sType == "prk")
		bRet = this.importPrk(sTblTo, tbl);
	else
		bRet = this.importTbl(sTblTo, tbl, !this.replace, this.base, objCod);	
	tbl.close();
	return bRet;
}

function DbImporter_runXLS(sTblTo, bPreview)
{
	if(!this.isFile(this.source))
		return false;
	
	//this.loadVars(this.sFileParam, this.source, this.sourceNum);
	
	if(!sTblTo)
		sTblTo = this.recipient;
	var sType = this.objType;
	var objCod = this.objCod;
	var objID = this.objID;
	var bGroupRow = this.groupRow;
	var sGroupCond = this.groupCond;
	var sGroupField = this.groupField;
	var sGroupKey = this.groupKey;
	
	var sTmpTbl = this.getSourceName();
	if(!this.createSourceTbl(sTmpTbl))
		return false;
	
	var bRet = this.importObjects();
	if(!bRet)
		return false;

	//trace("\n>>> DbImporter_runXLS: "+ sType +" > "+ sTblTo +" > "+ objCod +"\n")		
	var exldoc = lpProxy.openExcelWorkbook(this.source);
	if(!exldoc)
		return false;
	var lastRow	= exldoc.getSheet(exldoc.getSheetName(0)).getLastRowNum();
	var nrow = 0;
	if(this.nStart>1)
		nrow = this.nStart-1;
	if(this.nEnd>0 && this.nEnd<lastRow)
		lastRow = this.nEnd-1;

	while(nrow<=lastRow)
	{
		trace("\n>>> "+ nrow +"/"+ lastRow);
		var oRow = exldoc.getSheet(exldoc.getSheetName(0)).getRow(nrow++);		
		if(!oRow)
			continue;
		
		if(Number(this.conditionCol)>0)
		{
			var oCell = oRow.getCell(Number(this.conditionCol)-1);
			if(oCell)
			{
				var oVal = null;
				if(oCell.getCellType()==0)
					oVal = oCell.getNumericCellValue();
				else if(oCell.getCellType()==1)
					oVal = oCell.getStringCellValue();
				if(!this.checkCondition(oVal))
					continue;
			}
		}
		
		var isEmpty = true;
		var m = new Object();
		for(fld in this.fldMap)
		{			
			var sFld = this.getFieldName(fld);
			if(!sFld || !Number(sFld.substring(1)))
				continue;			
			var n = Number(sFld.substring(1));			
			var oCell = oRow.getCell(n-1);
			if(!oCell)
				continue;

			var sVal = "";
			if(oCell.getCellType()==0 && oCell.getCellStyle().dataFormat==14)
				sVal = format("dd.mm.yyyy", oCell.getDateCellValue());
			else if(oCell.getCellType()==0)// || oCell.getCellType()==2
				sVal = String(oCell.getNumericCellValue());
			else
				sVal = String(oCell.getStringCellValue());
			//trace(">>> "+ fld +" - "+ this.fldMap[fld] +": "+ sVal +" > "+ oCell.getCellType() +" - "+ oCell.getCellStyle().dataFormat);
			if(typeOf(sVal)=="string" || typeOf(sVal)=="String")
			{
				sVal = sVal.trim();
				sVal = sVal.substring(0, 250);
			}			
			//trace(">>> "+ nrow +" DbImporter_runXLS "+ sType +": "+ sFld +" - "+ sVal)
			m[sFld] = sVal;
			isEmpty = false;
		}
		//trace("\n");
		var bRet = true;
		if(!isEmpty)
			bRet = UpdateTable(sTmpTbl, m, true);
		if(!bRet && stopMassage(this.source))
			return false;
	}	
	
	if(bPreview)
		return sTmpTbl;
	
	sGroupField = this.getFieldName(sGroupField);
	var sSql = "select * from "+ sTmpTbl;
	if(bGroupRow && sGroupField && sGroupCond=="GROUP_FIELD")
		sSql += " order by "+ sGroupField;
	var tbl = OpenTable(sSql);
	this.oEval.setTable(tbl);
	
	if(this.preview)
	{
		var bRet = this.browse(tbl);
		if(!bRet)
		{
			tbl.close();
			return true;
		}
	}
		
	if(sType == "cl")
		bRet = this.importCl(sTblTo, objCod, tbl);
	else if(sType == "doc")
		bRet = this.importDoc(sTblTo, objCod, tbl, bGroupRow, sGroupCond, sGroupField, sGroupKey);
	else if(sType == "docrow")
		bRet = this.importDocRows(tbl, sTblTo, objCod, objID);
	else if(sType == "prk")
		bRet = this.importPrk(sTblTo, tbl);
	else
		bRet = this.importTbl(sTblTo, tbl, !this.replace, this.base, objCod);	
	tbl.close();
	return bRet;
}

function DbImporter_getColNum(str)
{
	var nCol = str;
	str = java.lang.String(str).trim().toUpperCase();
	if(str.startsWith("F"))
		nCol = str.substring(1);
	return Number(nCol);
}

function DbImporter_getFieldName(str)
{
	if(!str)
		return null;
	str = String(str).trim().toUpperCase();
	if(!str || this.isExpr(str))
		return null;
	if(Number(str))
	{
		if(this.type=="DBF")
			return Number(str);
		return "F"+ str;
	}
	else if(Number(str.substring(1)))
	{
		if(this.type=="DBF")
			return Number(str.substring(1));
	}
	return str;
}

function DbImporter_isExpr(str)
{
	str = String(str).trim();
	return str.indexOf("=")!=-1 || str.indexOf("\"")!=-1 || str.indexOf("'")!=-1;
}

function DbImporter_getExpr(str)
{
	str = java.lang.String(str).trim();
	if(str.startsWith("="))
		return str.substring(1);
	return str;
}

function DbImporter_isExt(sVar)
{
	sVar = java.lang.String(sVar).trim();
	return sVar.startsWith("X_");
}

function DbImporter_importObjects()
{
	var sVarsTbl = this.getVarsTblName();
	if(!IsTableExist(sVarsTbl))
		return false;
	var bRet = true;
	var tblVars = OpenTable("select * from "+ sVarsTbl);
	for(tblVars.MoveFirst(); !tblVars.IsEOF(); tblVars.MoveNext())
	{			
		var sObjSource = tblVars.getValue("FOBJSOURCE");
		var sObjParam = tblVars.getValue("FPARAM");
		var sRec = tblVars.getValue("FREC");
		var bNewSource = tblVars.getValue("FNEWSOURCE")==true || tblVars.getValue("FNEWSOURCE")==1;				
		if(!bNewSource)
			continue;
		if(!isFile(sObjSource))
		{
			var path = DpFile(this.sFileParam).getParent();
			sObjSource = path +"/"+ DpFile(sObjSource).getName();			
		}
		if(!isFile(sObjParam))
		{
			var path = DpFile(this.sFileParam).getParent();
			sObjParam = path +"/"+ DpFile(sObjParam).getName();			
		}
		var oImporter = new DbImporter();		
		oImporter.init(sObjSource, sObjParam);		
		/*				
		oImporter.recipient = String(tblVars.getValue("FREC"));
		oImporter.encoding = String(tblVars.getValue("FENCODING"));
		oImporter.delimiter = String(tblVars.getValue("FDELIMITER"));
		oImporter.mode = String(tblVars.getValue("FMODE"));
		oImporter.replace = tblVars.getValue("FREPLACE")==true;
		oImporter.linkSource = String(tblVars.getValue("FLINKSOURCE"));
		oImporter.linkName = String(tblVars.getValue("FLINKNAME"));
		oImporter.nStart = String(tblVars.getValue("FSTART"));
		oImporter.nEnd = String(tblVars.getValue("FEND"));
		oImporter.objType = String(tblVars.getValue("FTYPE"));
		oImporter.objCod = String(tblVars.getValue("FOBJCOD"));
		oImporter.objID = String(tblVars.getValue("FOBJID"));
		oImporter.groupRow = tblVars.getValue("FGROUPROW")==true;
		oImporter.groupCond = String(tblVars.getValue("FGROUPCOND"));
		oImporter.groupField = String(tblVars.getValue("FGROUPFIELD"));
		oImporter.groupKey = String(tblVars.getValue("FGROUPKEY"));
		*/
		oImporter.base = this.base;		
		oImporter.showMsg = false;		
		
		bRet = oImporter.run(true);
		if(!bRet && stopMassage(sObjSource))
			return false;
		
		this.arrObj[sRec] = oImporter;
	}
	tblVars.close();
	return true;
}

function DbImporter_importTbl(sTblName, tblSource, bInsert, sBase, objCod)
{
	var sFullTblName = sTblName;
	if(sBase && sTblName.indexOf(".")==-1)
		sFullTblName = sBase +"."+ sTblName;
	
	if(sTblName=="MTR_CL_NMK")
		this.createExtentions("CL_NMK");
	
	if(typeof importTbl=="function")
	{
		this.importTbl = importTbl;
		return this.importTbl(sFullTblName, tblSource, this);
	}
	
	if(this.getMode()=="edit")
	{
		return this.updateTbl(sFullTblName, this.getSourceName(), tblSource);
	}
	if(!bInsert)
		ExecuteSQL("drop table "+ sFullTblName, false);	
	if(bInsert && !IsTableExist(sTblName) && !IsTableEx(sBase, sTblName))
		bInsert = false;
	if(!bInsert && !this.createTbl(sFullTblName) && stopMassage(sFullTblName))
		return false;
	
	//BeginTransaction(TRANSACTION_SERIALIZABLE);
	
	var nPr = 1;
	var maxPr = tblSource.recordCount;
	var sPr = "Импорт "+ sFullTblName +": ";
	//var progressM = new DpProgressMonitor(sPr, maxPr);
	//progressM.start();
	var nTime = new Date().getTime();
	
	var nRow = 0;
	var sqlVals = "";
	var arrIDs = new Array();
	tblSource.MoveFirst();
	while(!tblSource.isEOF())
	{		
		if(nPr%10==0)
		{
			WAIT(sPr + nPr +"/"+ maxPr);
			//progressM.setPos(nPr, nPr +"/"+ maxPr);
			trace(">>> TIME: "+ (new Date().getTime()-nTime));
			nTime = new Date().getTime();
			trace("\n>>> "+ sPr + (nPr) +"/"+ maxPr);
		}
		nPr++;
		
		var passRow = false;				
		if(this.passExisting)
		{						
			var sWhere = "";
			var arrLinks = this.getLinks();
			if(arrLinks)
			{				
				for(fld in arrLinks)
				{
					var sVar = arrLinks[fld];
					var fld = this.getFieldName(fld);
					var sType = this.getVarType(sVar);
					var oVal = this.getValue(fld, tblSource, sType, sVar);
					if(oVal==null)
						continue;
					if(this.isExt(sVar))
					{
						sVar = sVar.substring(2);			
						if(sTblName.toUpperCase().indexOf("MTR_CL_NMK")!=-1)
						{
							var sExtTblName = this.base +".CL_NMK";
							if(SUBD==MSSQL)
								sExtTblName = this.base +"..CL_NMK";
							sWhere += " and FWID=(select max(ext.FMAINWID) "
												+" from "+ sExtTblName +" as ext " 
												+" where ext."+ sVar +"="+ sqlTo(oVal)
												+")";											
						}
					}
					else
					{
						sWhere += " and "+ sVar +"="+ sqlTo(oVal);
					}				
				}
			}
			else
			{
				for(sFldTo in this.varMap)
				{			
					var fld = this.varMap[sFldTo];
					var sType = this.getVarType(sFldTo);
					var oVal = this.getValue(fld, tblSource, sType, sFldTo);					
					if(oVal==null || this.isObject(sType))
						continue;
					if(this.isExt(sFldTo))
					{
						sFldTo = sFldTo.substring(2);			
						if(sTblName.toUpperCase().indexOf("MTR_CL_NMK")!=-1)
						{
							var sExtTblName = this.base +".CL_NMK";
							if(SUBD==MSSQL)
								sExtTblName = this.base +"..CL_NMK";
							sWhere += " and FWID=(select max(ext.FMAINWID) "
											+" from "+ sExtTblName +" as ext " 
											+" where ext."+ sFldTo +"="+ sqlTo(oVal)
											+")";
						}
					}
					else
					{
						sWhere += " and "+ sFldTo +"="+ sqlTo(oVal);
					}
				}				
			}
			if(sWhere=="")
				sWhere = " 1<>1 ";
			else
				sWhere = sWhere.substring(5);
				
			var oSnap = getSnap("select * from "+ sFullTblName +" where "+ sWhere);			
			if(oSnap)
			{
				passRow = true;
			}
			else
			{
				passRow = false;
			}
		}
		if(passRow)
		{
			tblSource.MoveNext();
			continue;
		}
				
		var sFld = "";
		var sVal = "";
		var isEmpty = true;
		var fwid = GetUID();
		var fcod = 0;
		var arrExt = new Object();		
		for(sFldTo in this.varMap)
		{			
			var fld = this.varMap[sFldTo];
			var type = this.getVarType(sFldTo);
			var oVal = this.getValue(fld, tblSource, type, sFldTo);
			if(oVal==null)
				oVal = this.getNullValue(type);			
			if(this.isExt(sFldTo))
			{
				arrExt[sFldTo] = oVal;
				continue;
			}
			var sType = this.getVarType(sFldTo);
			if(this.isObject(sType))
				oVal = this.createObject(sType, oVal, sFldTo);				

			if(sFldTo=="FWID")
			{
				if(!oVal)
					continue;
				fwid = oVal;
			}
			
			if(sFldTo=="FCOD")
				fcod = oVal;
			
			sVal += ","+ sqlTo(oVal);
			sFld += ","+ sFldTo;
			
			if(oVal)
				isEmpty = false;			
		}
		
		if(isEmpty || sVal=="")
		{
			tblSource.MoveNext();
			continue;
		}
			
		sVal = sVal.substring(1);
		sFld = sFld.substring(1);		
		if(sFld.toUpperCase().indexOf("FWID")==-1)
		{
			sFld += ",FWID";
			sVal += ","+ fwid;
		}
		
		if(sTblName=="MTR_CL_NMK")
		{
			if(sFld.toUpperCase().indexOf("FCOD")==-1)
			{
				sFld += ",FCOD";
				sVal += ","+ sqlTo(nextNmkCod(objCod));
			}
			if(sFld.toUpperCase().indexOf("FCEDIB")==-1)
			{
				sFld += ",FCEDIB";
				sVal += ","+ sqlTo(1);
			}
			if(sFld.toUpperCase().indexOf("FEDI_SPR")==-1)
			{
				sFld += ",FEDI_SPR";
				sVal += ","+ sqlTo(1);
			}			
			if(sFld.toUpperCase().indexOf("FEDI1")==-1)
			{
				sFld += ",FEDI1";
				sVal += ","+ sqlTo(1);
			}
			if(sFld.toUpperCase().indexOf("FFAS1")==-1)
			{
				sFld += ",FFAS1";
				sVal += ","+ sqlTo(0);
				sFld += ",FFAS2";
				sVal += ","+ sqlTo(0);
				sFld += ",FFAS3";
				sVal += ","+ sqlTo(0);
				sFld += ",FFAS4";
				sVal += ","+ sqlTo(0);
				sFld += ",FFAS5";
				sVal += ","+ sqlTo(0);
			}
			if(sFld.toUpperCase().indexOf("FGRPID")==-1)
			{
				sFld += ",FGRPID";
				sVal += ","+ createNmkGroup(objCod);			
				sFld += ",FISGRP";
				sVal += ","+ sqlTo(false);
			}
			
			if(!this.checkCod(fcod, 0, "nmk"))
			{
				tblSource.MoveNext();
				continue;
			}			
		}
		
		arrIDs[arrIDs.length] = fwid;
		if(sqlVals!="")
			sqlVals += ",";
		sqlVals += "("+ sVal +")";
		
		var count = this.rowsGroupCount;
		if(maxPr<count || nRow%count==0 || (maxPr-nPr)<=count)
		{
			var sSQL = "insert into "+ sFullTblName +" ("+ sFld +") "
						+" values "+ sqlVals;
			var bRet = ExecuteSQL(sSQL);
			if(!bRet && stopMassage(this.source, sSQL))
				return false;
			//trace(">>> "+ sqlVals +"\n");
			sqlVals = "";		
		}
		
		this.saveExt(sTblName, arrExt, fwid);
		tblSource.MoveNext();
		nRow++;			
	}
	//CommitTrans();
	
	//progressM.stop();
	if(typeof afterImportTbl=="function")
	{
		this.afterImportTbl = afterImportTbl;
		return this.afterImportTbl(sFullTblName, tblSource, this);
	}
			
	if(sTblName=="MTR_CL_NMK")
	{		
		buildNmkGroups(arrIDs, objCod, true);		
	}
	
	if(this.showMsg)
	{
		var nImp = this.sourceNum? this.sourceNum+" - " : "";
		sMsg = UR("Імпорт "+ nImp + this.source +" завершено.\nЗавантажено рядків: "+ nRow 
				+"|Импорт "+ nImp + this.source +" завершен.\nЗагружено строк: "+ nRow);
		if(this.arrErr.length>0)
			sMsg += "\n"+ UR("Помилок: |Ошибок: ") + this.arrErr.length;
		alert(sMsg);
	}
	return true;
}

function createNmkGroup(sGroupCod)
{
	if(!sGroupCod)
		return 0;
	var oSnap = getSnap("select FWID from ^CL_NMK where FCOD="+ sqlTo(sGroupCod), false);
	if(oSnap)
		return oSnap[0];
	var m = new Object();
	m.FWID = GetUID();
	m.FCOD = sGroupCod;
	m.FFTXTS = "";
	m.FTXTS = "";
	m.FTXT0 = "";
	m.FGRPID = 0;
	m.FISGRP = true;
	UpdateTable("^CL_NMK", m, true);
	return m.FWID;
}

//создание групп для новой номенклатуры
function buildNmkGroups(arrIDs, groupCod, bConfirm)
{
	include("sys/servis.js");
	include("nom_trans.js");
	
	//если номеклатура импортировалась в одну группу - нужно просто обновить ключи для дерева
	if(groupCod && arrIDs && arrIDs.length>0)
	{
		for(var i=0; i<arrIDs.length; i++)
		{
			if(arrIDs[i])
				updateTreeKeys("^CL_NMK", arrIDs[i], new Array("fcod", "ftxts"));
		}
		return true;
	}
	
	if(bConfirm)
	{
		var sCapt = UR("Імпорт номенклатури|Импорт номенклатуры");
		var sMsg = UR("Створити групи для імпортованої номенклатури?"
						+"\nНеіснуючі групи будуть додані з пустою назвою."
						+"\nНазви можна буде проставити відкривши довідник номенклатури."
					+"|Создать группы для импортированной номенклатуры?"
						+"\nНесуществующие группы будут созданы с пустым названием."
						+"\nНазвания можно будет проставить открыв справочник номенклатуры.");
		if(!confirm(sMsg, sCapt))
		{
			//buildNestedKeys("^cl_nmk", -1, 0, "fcod, fisgrp, ftxts");
			if(arrIDs && arrIDs.length>0)
			{
				for(var i=0; i<arrIDs.length; i++)
				{
					if(arrIDs[i])
						updateTreeKeys("^CL_NMK", arrIDs[i], new Array("fcod", "ftxts"));
				}
			}
			else
			{
				sMsg = UR("Щоб імпортовані рядки з'явилися у дереві"
							+" потрібно запустити меню 'Відновити дерево' у списку номенклатури."
						+"|Чтобы импортированные строки появились в дереве"
							+" нужно запустить меню 'Восстановить дерево' в списке номенклатуры."
						);
				alert(sMsg);
			}
			return true;
		}
	}
	
	ExecuteSQL("update ^CL_NMK set FGRPID=0 where FGRPID IS NULL");
	ExecuteSQL("drop table TMP_CL_NMK", false);
	ExecuteSQL("select * into TMP_CL_NMK from ^CL_NMK", false);
	
	var anySymb = "_";
	//if (SUBD==ORACLE || SUBD==MSSQL || SUBD==DERBY || SUBD==MYSQL)
	//	anySymb = "_";
	var sqlNmkCod = sqlStr4Like + "."+ anySymb + sqlStr4Like;
	var tbl = OpenTable("select * from ^CL_NMK "
						+" where FGRPID=0 "
							+" and FISGRP<>"+ sqlTrue 
							+" and FCOD like "+ sqlTo(sqlNmkCod)
						);
	for(; !tbl.isEOF(); tbl.MoveNext())
	{
		var fwid = Number(tbl.getValue("FWID"));
		var sCod = String(tbl.getValue("FCOD"));
		var grpCod = sCod.substring(0, sCod.indexOf(".")+1);
		var groupID = createNmkGroup(grpCod);
		if(groupID)
			ExecuteSQL("update ^CL_NMK set FGRPID="+ groupID +" where FWID="+ fwid);
		updateTreeKeys("^CL_NMK", fwid, new Array("fcod", "ftxts"));
	}
	tbl.close();
	
	//makeTrans_new();
	//buildNestedKeys("^CL_NMK", -1, 0, "fcod, fisgrp, ftxts");
}

function DbImporter_checkCod(cod, cl, type, vid, parentCod)
{
	if(!cod)
		return true;
	if(!type)
		type = this.objType;
	var sType = String(type).toLowerCase();
	switch(sType)
	{
		case "nmk":
			cod = String(cod);
			var oSnap = getSnap("select FWID, FTXTS from ^CL_NMK "
								+" where FCOD="+ sqlTo(cod));
			if(!oSnap)
				return true;
			//if(oSnap && oSnap[0] && this.passExisting)
			//	return false;
			var sMsg = UR("Номенклатура з кодом "+ cod +" вже існує:"
							+"\n\""+ cod +" - "+ oSnap[1] +"\""
							+"\nДодати ще одну?"
						+"|Номенклатура с кодом "+ cod +" уже существует:"
							+"\n\""+ cod +" - "+ oSnap[1] +"\""
							+"\nДобавить еще одну?"
						);
			var sCapt = UR("Імпорт номенклатури|Импорт номенклатуры");
			return confirm(sMsg, sCapt, "IMPORT_EXISTING_NMK");
			break;
			
		case "cls.cl":			
			cl = Number(cl);
			var isOrg = false;
			if(vid.toLowerCase()=="cl2")
				isOrg = true;
			var sSQL = "select FWID, FTXT from ^CL cl "
						+" inner join ^CLRELCLCL rel "
							+" on rel.FWID_CL=cl.FWID "
						+" where FCOD="+ sqlTo(cod)
							+" and (rel.FCL="+ sqlTo(cl)
							+ (isOrg? " or rel.FCL=0" : "") +")"							
			var oSnap = getSnap(sSQL);
			if(!oSnap)
				return true;
			var sTxt = UR("Аналітика|Аналитика");
			if(isOrg)
				sTxt = UR("Контрагент|Контрагент");
			var sMsg = sTxt + UR(" з кодом "+ cod +" вже існує:"
							+"\n\""+ cod +" - "+ oSnap[1] +"\""
							+"\nДодати ще "+ (isOrg? "одного" : "одну") +"?"
						+"| с кодом "+ cod +" уже существует:"
							+"\n\""+ cod +" - "+ oSnap[1] +"\""
							+"\nДобавить еще "+ (isOrg? "одного" : "одну") +"?"
						);
			var sCapt = UR("Імпорт налалітики|Импорт аналитики");
			return confirm(sMsg, sCapt, "IMPORT_EXISTING_CL");
			break;
			
		case "cls.cntrct":
			cod = String(cod);
			var oSnap = getSnap("select FWID, FTXT, FDOC, FNOP, FDOC_DAT "
								+" from ^CONTRACT "
								+" where FDOG_NUM="+ sqlTo(cod));
			if(!oSnap)
				return true;
			var sMsg = UR("Договір з номером "+ cod +" вже існує:"
							+"\nПапка "+ oSnap[2] +": "+ oSnap[3] 
								+"\n№ "+ cod 
								+"\nДата "+ StrDat(oSnap[4], "DMY")
							+"\nДодати ще одного?"
						+"|Договор с номером "+ cod +" уже существует:"
							+"\nПапка "+ oSnap[2] +": "+ oSnap[3] 
								+"\n№ "+ cod 
								+"\nДата "+ StrDat(oSnap[4], "DMY")
							+"\nДобавить еще одного?"
						);
			var sCapt = UR("Імпорт документів|Импорт документов");
			return confirm(sMsg, sCapt, "IMPORT_EXISTING_DOC");
			break;
	}	
	return true;
}

function DbImporter_isObject(sType, sVar)
{
	if(!sType && sVar)
		sType = this.getVarType(sVar);
	if(!sType)
		return false;
	sType = String(sType.toLowerCase());
	if(sType.indexOf("cls.")!=-1)
		return true;
	if(sType=="nmk")
		return true;
	return false;
}

function DbImporter_createObject(sType, sTxt, sVar)
{
	if(!sType)
		sType = this.getVarType(sVar);
	var nObjCod = this.getObjCod(sVar);
	trace(">>> createObject: "+ sType +" > "+ sTxt +" > "+ sVar +" > "+ nObjCod)
	if(!sType || !sTxt || String(sTxt)=="0" || String(sTxt).trim()=="")
		return 0;
	
	sType = String(sType).toLowerCase();
	if(sType.indexOf("cls.")!=-1)
		sType = sType.substring(4);
	switch(sType)
	{
		case "nmk":
			return this.createNmk(nObjCod, sTxt);
		
		case "cl":			
			return this.createCl(nObjCod, sTxt, sVar);
		
		case "cntrct":
			return this.createCntrct(nObjCod, sTxt);
		
		case "edi":
			return this.createEdi(sTxt);
	}
	return 0;
}

function DbImporter_createCntrct(sDoc, sID)
{
	if(!sID)
		return 0;
	var oSnap = getSnap("select FWID from ^CONTRACT "
					+" where (UPPER(FDOG_NUM)="+ sqlTo(sID.toUpperCase())
						//+" or UPPER(FDOC_NUM)="+ sqlTo(sID.toUpperCase())
						+" or UPPER(FOUTID)="+ sqlTo(sID.toUpperCase()) +")"
						+(sDoc? " and UPPER(FDOC)="+ sqlTo(sDoc.toUpperCase()) : "")
						+(this.orgCod? " and FORG="+ this.orgCod : "")
				, false);
	//trace("\n>>> DbImporter_createCntrct: "+ sID +" > "+ (oSnap? oSnap[0] : "NULL"));
	if(oSnap && oSnap[0])
		return oSnap[0];
	
	include("doc/DpAutoNo.js");
	var oDoc = new DpDoc(sDoc, 1);
	oDoc.autoNo = new dpAutoNo(oDoc);
	var num = oDoc.autoNo.get();
	oDoc.setVar("HNOM", num);
	oDoc.setVar("HDOG_NUM", sID);
	oDoc.setVar("HTXT", sID);
	oDoc.setVar("HOUTID", sID);
	if(this.orgCod)
		oDoc.setVar("HORG", this.orgCod);
	else
		oDoc.setVar("HORG", -999);	//'Все'
	oDoc.save(true, "ADD");
	return oDoc.getID();
}

function DbImporter_getCntrctByOrg(nOrg)
{
	if(!nOrg)
		return 0;
	var oSnap = getSnap("select count(*) as cnt from ^CONTRACT "
						+" where FORG="+ sqlTo(nOrg)						
				, false);
	if(oSnap && Number(oSnap[0])==1)
	{
		oSnap = getSnap("select FWID from ^CONTRACT "
						+" where FORG="+ sqlTo(nOrg)						
				, false);
		if(oSnap)
			return oSnap[0];
	}
	return 0;
}

function DbImporter_createEdi(sID)
{
	if(!sID)
		return 0;
	var sTxts = String(sID).substring(0, 7);
	var sTxtf = String(sID).substring(0, 25);
	var oSnap = getSnap("select FCOD from ^CL_EDI "
					+" where UPPER(FTXTS)="+ sqlTo(sTxts.toUpperCase())
						+" or UPPER(FTXTF)="+ sqlTo(sTxtf.toUpperCase())
						+" or UPPER(FOUTID)="+ sqlTo(sID.toUpperCase())
				, false);
	if(oSnap && oSnap[0])
		return oSnap[0];
	
	var cod = 1;
	var oSnap = getSnap("select max(FCOD) from ^CL_EDI ", false);
	if(oSnap && Number(oSnap[0]))
		cod = Number(oSnap[0])+1;
	
	var m = new Object();
	m.FCOD = cod;	
	m.FTXTF = sTxtf;
	m.FTXTS = sTxts;
	m.FWID = GetUID();
	UpdateTable("^CL_EDI", m, true);
	
	return m.FCOD;
}

function DbImporter_findCl(nCl, sTxt, sID, isFWID)
{
	if(!sTxt && !sID)
		return 0;
	if(!nCl)
		nCl = 0;
	nCl = Number(nCl);	
	if(sTxt)
		sTxt = String(sTxt).trim().toUpperCase();
	if(sID && Number(sID))
		sID = Number(sID);
	if(sID)
		sID = String(sID).trim().toUpperCase();
	var sWhere = "UPPER(cl.FTXT)="+ sqlTo(sTxt)
				+" or UPPER(cl.FTXTS)="+ sqlTo(sTxt) 
				+(sID? " or UPPER(cl.FOUTID)="+ sqlTo(sID) : "")
	if(!sTxt)
		sWhere = "UPPER(cl.FOUTID)="+ sqlTo(sID);
	var oSnap = getSnap("select rel.FCOD, rel.FWID_CL from ^CLRELCLCL rel "
					+" inner join ^CL cl on cl.FWID=rel.FWID_CL "
					+" where rel.FCL="+ sqlTo(nCl) +" and ("+ sWhere +")"
				, false);
	if(oSnap && Number(oSnap[0]))
	{
		if(isFWID && Number(oSnap[1]))
			return Number(oSnap[1]);
		else
			return Number(oSnap[0]);
	}		
	return 0;
}

function DbImporter_createCl(nCl, sTxt, sVar)
{
	var codCL = 0;
	var sID = sTxt;
	nCl = Number(nCl);
	var isFWID = false;
	if(this.recipient=="LICSC" && sVar!="HORG")
		isFWID = true;	
	if(this.recipient=="clhouse" || this.recipient=="clapartment")
	{
		if(sVar=="HSTREET" || sVar=="HHOUSE")
			codCL = this.findCl(nCl, "", sID, true);
		else if(sVar=="HIDPAR")
			codCL = this.findCl(nCl, "", sID, false);
	}
	else
	{
		codCL = this.findCl(nCl, sTxt, sID, isFWID);
	}
	trace(">>> createCl: "+ sVar +" > "+ sID +" > "+ codCL)
	if(codCL && codCL!=0)
		return codCL;
	
	var oSnap = getSnap("select FVID from ^CLCL "
					+" where FCOD="+ sqlTo(nCl)					
				, false);
	if(!oSnap)
		return 0;
	
	var sVid = oSnap[0];
	if(Number(sVid))
		sVid = "CL"+ sVid;

	var oCL = new DpClEx(sVid, nCl);
	var codCL = oCL.nextcod(nCl);
	oCL.setVar("HCOD", codCL);
	oCL.setVar("HTXT", sTxt);
	var noXML = oCL.getVar("ATTR.NOXML");
	oCL.setAttr("NOXML", true);
	if(isFWID)
		codCL = oCL.getID();
	var bRet = oCL.save("ADD", true);
	oCL.setAttr("NOXML", noXML);
	return codCL;
}

function DbImporter_createNmk(sGroupCod, sID)
{	
	var oSnap = getSnap("select FWID from ^CL_NMK "
						+" where UPPER(FTXTS)="+ sqlTo(sID.toUpperCase())
							+" or UPPER(FCOD)="+ sqlTo(sID.toUpperCase())
							+" or UPPER(FOUTID)="+ sqlTo(sID.toUpperCase())
					, false);
	if(oSnap)
		return oSnap[0];
			
	var m = new Object();
	m.FCOD = nextNmkCod(sGroupCod);
	m.FGRPID = createNmkGroup(sGroupCod);	
	m.FWID = GetUID();
	m.FTXTS = sID;	
	UpdateTable("^CL_NMK", m, true);
	return m.FWID;
}

function nextNmkCod(sGroupCod)
{
	var cod = "0001";
	var maxCode = String(this.maxCod);
	if(!this.maxCod)
	{
		maxCode = cod;
		var sSQL = "SELECT FCOD"
				   	+" FROM ^cl_nmk"
				   	+" WHERE (FISGRP<>"+ sqlTrue
				   		+" OR FISGRP IS NULL)"
				   	+" order by FWID desc";
		var oSnap = getSnap(sSQL);
		if(oSnap && oSnap[0])
			maxCode = String(oSnap[0]);			
	}
	if(maxCode)
	{
		var nInd = maxCode.lastIndexOf(".");
		if (nInd!=-1)
			maxCode = maxCode.substr(nInd + 1);
		if (Number(maxCode))
		{
			maxCode = String(Number(maxCode)+1);
			while (maxCode.length < 4)
				maxCode = "0" + maxCode;
			cod = maxCode;
		}
		this.maxCod = cod;
	}
	if(sGroupCod)
		cod = sGroupCod + cod;
	return cod;
}

function DbImporter_updateTbl(sTblName, sTblSource, tbl)
{	
	if(!sTblSource)
		sTblSource = this.getSourceName();
		
	if(typeof importTbl=="function")
	{
		this.importTbl = importTbl;
		return this.importTbl(sTblName, sTblSource, this);
	}
	
	var arrLinks = this.getLinks();
	if(!arrLinks)
	{
		alert(UR("Не налаштована зв'язка для оновлення "+ sTblName +"!"
				+"|Не настроена связка для обновления "+ sTblName +"!")
				);
		return false;
	}
	
	var nPr = 1;
	var maxPr = tbl.recordCount;
	var sPr = UR("Оновлення: |Обновление: ");
	var progressM = new DpProgressMonitor(sPr, maxPr);
	progressM.start();
	
	var sourceRowCount = maxPr;
	var updateRowCount = 0;
	
	tbl.MoveFirst();
	while(!tbl.isEOF())
	{
		if(nPr%10==0)
		{
			WAIT(sPr + (nPr) +"/"+ maxPr);
			progressM.setPos(nPr, (nPr) +"/"+ maxPr);
		}
		nPr++;
		
		var sWhere = "";
		for(fld in arrLinks)
		{
			var sVar = arrLinks[fld];
			fld = this.getFieldName(fld);
			var sType = this.getVarType(sVar);
			var oVal = this.getValue(fld, tbl, sType, sVar);
			if(oVal==null)
				continue;
			if(this.isExt(sVar))
			{
				//обновление номенклатуры по расширению
				sVar = sVar.substring(2);			
				if(sTblName.toUpperCase().indexOf("MTR_CL_NMK")!=-1)
				{
					var sExtTblName = this.base +".CL_NMK";
					if(SUBD==MSSQL)
						sExtTblName = this.base +"..CL_NMK";
					sWhere += " and FWID=(select max(ext.FMAINWID) "
								+" from "+ sExtTblName +" as ext " 
								+" where ext."+ sVar +"="+ sqlTo(oVal)
								+")";
				}
			}
			else
			{
				sWhere += " and "+ sVar +"="+ sqlTo(oVal)
			}
		}
		
		var oSnap = getSnap("select FWID from "+ sTblName +" where 1=1 "+ sWhere, false);
		if(!sWhere || !oSnap || !oSnap[0])
		{
			tbl.MoveNext();
			continue;
		}
		
		var m = new Object();
		m.FWID = Number(oSnap[0]);
		for(sFldTo in this.varMap)
		{
			var sSource = this.varMap[sFldTo];
			var fld = this.getFieldName(sSource);
			var sType = this.getVarType(sFldTo);
			var oVal = this.getValue(fld, tbl, sType, sFldTo);
			if(this.isObject(sType))
				oVal = this.createObject(sType, oVal, sFldTo);
			m[sFldTo] = oVal;
		}
		var bRet = UpdateTable(sTblName, m, false);
		if(bRet)
			updateRowCount++;
		tbl.MoveNext();
	}
	
	/*
	var bRet = false;
	var fld = "";
	var sFldTo = "";
	var sFields = "";
	for(sFldTo in this.varMap)
	{
		var sSource = this.varMap[sFldTo];
		fld = this.getFieldName(sSource);
		if(fld && !arrLinks[sSource])
		{
			WAIT(UR("Оновлення поля |Обновление поля ") + sFldTo);
			bRet = false;
			var sType = this.getVarType(sFldTo);
			if(this.isObject(sType))
			{
				var sObjFld = "FOBJFLD999";
				tbl.close();				
				ExecuteSQL("alter table "+ sTblSource +" add COLUMN "+ sObjFld +" "+ sqlType("LONG"), false);
				tbl = OpenTable(sTblSource);
				for(tbl.MoveFirst(); !tbl.isEOF(); tbl.MoveNext())
				{
					var oVal = this.getValue(fld, tbl, sType, sFldTo);
					if(!oVal)
						continue;
					oVal = this.createObject(sType, oVal, sFldTo);
					ExecuteSQL("update "+ sTblSource +" set "+ sObjFld +"="+ sqlTo(oVal) +" where FWID="+ tbl.getValue("FWID"), false);
				}
				fld = sObjFld;
			}
			if(this.isExt(sFldTo))
			{
				sFldTo = sFldTo.substring(2);
				if(sTblName.toUpperCase().indexOf("MTR_CL_NMK")!=-1)
				{
					var sExtTblName = this.base +".CL_NMK";
					if(SUBD==MSSQL)
						sExtTblName = this.base +"..CL_NMK";
					var tmpTbl = getTmpTableName();
					var sSQL = "select s."+ fld +", t.FWID"
								+" into "+ tmpTbl
								+" from "+ sTblSource +" as s, "+ sTblName +" as t"
								+" where "+ sWhere;
					bRet = ExecuteSQL(sSQL);
					if(bRet)
					{
						sSQL = "update "+ sExtTblName +" as t "
								+" set t."+ sFldTo 
									+"=(select max(s."+ fld +") "
									+" from "+ tmpTbl +" as s " 
									+" where s.FWID=t.FMAINWID)"
								+" where exists (select s."+ fld
									+" from "+ tmpTbl +" as s " 
									+" where s.FWID=t.FMAINWID)";
						bRet = ExecuteSQL(sSQL);
					}
					ExecuteSQL("drop table "+ tmpTbl, false);
				}
			}
			else
			{
				var sSQL = "update "+ sTblName +" as t "
							+" set t."+ sFldTo 
								+"=(select max(s."+ fld +") "
								+" from "+ sTblSource +" as s " 
								+" where "+ sWhere +")"
							+" where exists (select "+ fld
								+" from "+ sTblSource +" as s " 
								+" where "+ sWhere +")";
				bRet = ExecuteSQL(sSQL);
			}
			if(bRet)
				sFields += sFldTo +"; ";
		}
	}
	*/
	var sFields = "";
	for(sFldTo in this.varMap)
		sFields += sFldTo +"; ";
	var sMsg = UR("Оновлення таблиці "+ sTblName +" завершено."
					+"\n  Оновлено рядків: "+ updateRowCount +"."
					+"\n  Оновлені поля: "+ sFields					
				+"|Обновление таблицы "+ sTblName +" завершено."					
					+"\n  Обновлено строк: "+ updateRowCount +"."
					+"\n  Обновлены поля: "+ sFields
				);
	alert(sMsg);
	/*
	if(sourceRowCount>updateRowCount && this.addNotExisting)
	{
		sMsg += UR("\n\nЗалишилися рядки, які не знайшлися у приймачі! "	//+ (sourceRowCount-updateRowCount) 
					+"\nІмпортувати нові рядки?"
				+"|\n\nОстались строки, которые не нашлись в приемнике! "	//+ (sourceRowCount-updateRowCount)
					+"\nИмпортировать новые строки?"
				);
		if(confirm(sMsg))
		{
			var newSource = getTmpTableName();
			var sSQL = "select * into "+ newSource
						+" from "+ sTblSource +" as s " 
						+" where not exists (select "+ sFldTo
							+" from "+ sTblName +" as t " 
							+" where 1=1 "+ sWhere 
							+")";
			var bRet = ExecuteSQL(sSQL);
			if(!bRet)
				return true;
			
			this.mode = "add";
			this.sourceTblName = newSource;
			var tbl = OpenTable("select * from "+ newSource);
			this.oEval.setTable(tbl);
			if(sTblName.indexOf(".")!=-1)
				sTblName = sTblName.substring(sTblName.lastIndexOf(".")+1);
			this.importTbl(sTblName, tbl, true, this.base, this.objCod);
			tbl.close();
		}
	}
	else
	{
		alert(sMsg);
	}
	*/
	return true;
}

function DbImporter_importCl(sVid, nCl, tbl)
{	
	if(typeof importCl=="function")
	{
		this.importCl = importCl;
		return this.importCl(sVid, nCl, tbl, this);
	}
	
	if(this.getMode()=="edit")
	{
		return this.updateCl(sVid, nCl, tbl);
	}
		
	var maxPr = tbl.recordCount;
	var sPr = "Импорт аналитики: ";
	var progressM = new DpProgressMonitor(sPr, maxPr);
	progressM.start();
	var nPr = 1;
	var nTime = new Date().getTime();
	
	var codCl = Number(nCl);
	if(codCl==12)
		codCl = 0;
	/*
	var cod = 1;
	var oSnap = getSnap("select max(FCOD) from ^CLRELCLCL "
						+" where FCL="+ codCl);
	if(oSnap && Number(oSnap[0]))
		cod = Number(oSnap[0])+1;
	*/	
	if(sVid.toUpperCase().indexOf("CL")==-1 && Number(sVid))
	{
		sVid = "CL"+ sVid;
	}	
	
	var oCL = new DpClEx(sVid, Number(nCl));
	var nCod = oCL.nextcod(codCl);
	
	var sOutID = allt(this.varMap["HOUTID"]);	
	var arrLinks = this.getLinks();
	var bPassExisting = this.passExisting;
	if(sOutID && !arrLinks)
		bPassExisting = false;
	var nRow = 0;
	tbl.MoveFirst();
	while(!tbl.isEOF())
	{	
		if(nPr%10==0)
		{
			WAIT(sPr + (nPr) +"/"+ maxPr);
			progressM.setPos(nPr, (nPr) +"/"+ maxPr);
		}
		trace(">>> TIME: "+ (new Date().getTime()-nTime));
		nTime = new Date().getTime();
		trace("\n>>> "+ sPr + (nPr) +"/"+ maxPr);
		
		if(this.type=="DBF")
		{
			if(this.nStart>1 && nPr<this.nStart)
			{
				tbl.MoveNext();
				continue;
			}
			if(this.nEnd>0 && nPr>this.nEnd)
				break;
		}
		nPr++;
				
		if(sOutID && this.passExisting)
		{
			var sID = tbl.getValue(this.varMap["HOUTID"]);
			var codCL = this.findCl(nCl, "", sID);			
			if(codCL && codCL!=0)
			{
				trace(">>> FOUTID: "+ sID +" > cod: "+ codCL)
				tbl.MoveNext();
				continue;
			}
		}
		
		var passRow = false;
		if(bPassExisting)
		{						
			var sWhere = "";
			if(arrLinks)
			{				
				for(fld in arrLinks)
				{
					var sVar = arrLinks[fld];
					var fld = this.getFieldName(fld);
					var sType = this.getVarType(sVar);
					var oVal = this.getValue(fld, tbl, sType, sVar);
					if(oVal==null)
						continue;
					if(this.isExt(sVar))
					{
						sVar = sVar.substring(2);			
						var sExtTblName = this.base +".CL_"+ nCl;
						if(SUBD==MSSQL)
							sExtTblName = this.base +"..CL_"+ nCl;
						sWhere += " and FWID=(select max(ext.FMAINWID) "
											+" from "+ sExtTblName +" as ext " 
											+" where ext."+ sVar +"="+ sqlTo(oVal)
											+")";						
					}
					else
					{
						sFldTo = oCL.getFieldName(sVar);
						sWhere += " and "+ sFldTo +"="+ sqlTo(oVal);
					}				
				}
			}
			else
			{
				for(sVar in this.varMap)
				{			
					var fld = this.varMap[sVar];
					var sType = this.getVarType(sVar);
					var oVal = this.getValue(fld, tbl, sType, sVar);					
					if(oVal==null || this.isObject(sType))
						continue;
					if(this.isExt(sVar))
					{
						sVar = sVar.substring(2);			
						var sExtTblName = this.base +".CL_"+ nCl;
						if(SUBD==MSSQL)
							sExtTblName = this.base +"..CL_"+ nCl;
						sWhere += " and FWID=(select max(ext.FMAINWID) "
										+" from "+ sExtTblName +" as ext " 
										+" where ext."+ sVar +"="+ sqlTo(oVal)
										+")";						
					}
					else
					{
						sFldTo = oCL.getFieldName(sVar);
						sWhere += " and "+ sFldTo +"="+ sqlTo(oVal);
					}
				}
			}
			
			var sSQL = "select * from ^CL cl inner join ^CLRELCLCL rel on rel.FWID_CL=cl.FWID";
			var arrTbls = oCL.getTables();
			if(arrTbls && arrTbls.length>0)
			{
				for(var i=0; i<arrTbls.length; i++)
				{
					if(!arrTbls[i])
						continue;
					var sTable = String(arrTbls[i]).toUpperCase().trim();
					if(sTable!="CL" && sTable!="CLRELCLCL")
						sSQL += " inner join ^"+ sTable +" as t"+i+" on t"+i+".FWID_CL=cl.FWID"
				}
			}
			sSQL += " where rel.FCL="+ nCl + sWhere;
			var oSnap = getSnap(sSQL);
			if(oSnap)
			{
				passRow = true;
			}
			else
			{
				passRow = false;
			}
		}
		if(passRow)
		{
			tbl.MoveNext();
			continue;
		}
		var parentCod = 0;				
		var isEmpty = true;
		oCL = new DpClEx(sVid, Number(nCl));
		for(sVar in this.varMap)
		{
			var fld = this.varMap[sVar];
			if(allt(fld)=="")
				continue;
			var sType = this.getVarType(sVar);									
			var oVal = this.getValue(fld, tbl, sType, sVar);
			if(oVal!=null && oVal!="null")
			{
				if(this.isObject(sType))
					oVal = this.createObject(sType, oVal, sVar);
				if(this.isExt(sVar))
				{
					var sExt = sVar.substring(2);					
					oCL.setExt(sExt, oVal);
				}
				else
				{
					oCL.setVar(sVar, oVal);
				}
				if(sVar=="HIDPAR")
					parentCod = oVal;
				isEmpty = false;
			}
		}
		if(isEmpty)
		{
			tbl.MoveNext();
			continue;
		}
		
		if(oCL.getVar("HCOD_NDS")!=0 && oCL.getVar("HCOD_NDS")!=null && oCL.getVar("HCOD_NDS")!="")
			oCL.setVar("HPLATNDS", true);
		
		var cod = oCL.getVar("HCOD");
		if(cod==0 || cod==null || cod=="")
		{
			oCL.setVar("HCOD", nCod++);
		}
		else
		{
			cod = Number(cod);
			if(cod>nCod)
				nCod = cod+1;
			if(!importExistingCL() && !this.passExisting)
			{
				if(!this.checkCod(cod, Number(nCl), "cls.cl", sVid, parentCod))
				{
					tbl.MoveNext();
					continue;
				}
			}
		}
		var noXML = oCL.getVar("ATTR.NOXML");
		oCL.setAttr("NOXML", true);
		
		this.importRows(oCL, tbl);
		
		var bRet = oCL.save("ADD", true);
		oCL.setAttr("NOXML", noXML);
		if(!bRet && stopMassage(sVid +" - "+ oCL.getVar("HCOD") +": "+ oCL.getVar("HTXT")))
			return false;
		if(oCL.hasExt())
			oCL.saveExtToDB();
			
		tbl.MoveNext();
		nRow++;
	}
	progressM.stop();
	
	if(this.showMsg)
	{
		var nImp = this.sourceNum? this.sourceNum+" - " : "";
		alert(UR("Імпорт "+ nImp + this.source +" завершено.\nЗавантажено рядків: "+ nRow 
				+"|Импорт "+ nImp + this.source +" завершен.\nЗагружено строк: "+ nRow));
	}
	return true;
}

function DbImporter_updateCl(sVid, nCl, tbl)
{				
	var arrLinks = this.getLinks();
	if(!arrLinks)
		return false;
	
	var maxPr = tbl.recordCount;
	var sPr = "Импорт аналитики: ";
	var progressM = new DpProgressMonitor(sPr, maxPr);
	progressM.start();
	var nPr = 1;
	
	if(sVid.toUpperCase().indexOf("CL")==-1 && Number(sVid))
		sVid = "CL"+ sVid;
	var oCL = new DpClEx(sVid, Number(nCl));
		
	tbl.MoveFirst();
	while(!tbl.isEOF())
	{		
		if(nPr%10==0)
		{
			WAIT(sPr + (nPr) +"/"+ maxPr);
			progressM.setPos(nPr, (nPr) +"/"+ maxPr);
		}
		nPr++;
		var sSQL = "select max(cl.FWID) as FWID "
					+" from ^CL cl "
					+" inner join ^CLRELCLCL rel on rel.FWID_CL=cl.FWID"
					+" where rel.FCL="+ nCl;
		var sLinkWhere = "";
		for(fld in arrLinks)
		{
			var sVar = arrLinks[fld];
			var sFld = oCL.getFieldName(sVar);
			var oVal = this.getValue(fld, tbl, null, sVar);
			if(sVar=="HCOD")
				oVal = Number(oVal);
			if(sFld)
				sLinkWhere += " and "+ sFld +"="+ sqlTo(oVal);
		}		
		var oSnap = getSnap(sSQL + sLinkWhere, false);
		if(!sLinkWhere || !oSnap || !oSnap[0])
		{
			tbl.MoveNext();
			continue;
		}

		var nID = Number(oSnap[0]);
		oCL.clear();		
		if(!oCL.loadByID(nID))
		{
			tbl.MoveNext();
			continue;
		}
		
		for(sVar in this.varMap)
		{
			var fld = this.varMap[sVar];
			var sType = this.getVarType(sVar);					
			var oVal = this.getValue(fld, tbl, this.getVarType(sVar), sVar);
			if(this.isObject(sType))
				oVal = this.createObject(sType, oVal, sVar);
			if(this.isExt(sVar))
			{
				var sExt = sVar.substring(2);					
				oCL.setExt(sExt, oVal);
			}
			else
			{
				oCL.setVar(sVar, oVal);
			}
		}
		
		if(!oCL.getVar("HCOD") || oCL.getVar("HTXT")=="" || oCL.getVar("HTXT")==null  || oCL.getVar("HTXT")=="null")
		{
			//tbl.MoveNext();
			//continue;
		}
		this.importRows(oCL, tbl);
				
		var bRet = oCL.save("EDIT", true);
		if(!bRet && stopMassage(sVid +" - "+ oCL.getVar("HTXT")))
			return false;
		tbl.MoveNext();
	}
	progressM.stop();
	return true;
}

function DbImporter_importDoc(sDoc, sNop, tbl, bGroupRow, sGroupCond, sGroupField, sGroupKey)
{		
	this.sGroupKey = sGroupKey = String(sGroupKey);
	
	if(typeof importDoc=="function")
	{
		this.importDoc = importDoc;
		return this.importDoc(sDoc, sNop, tbl, this);
	}
	
	if(this.getMode()=="edit")
	{
		return this.updateDoc(sDoc, sNop, tbl);
	}
	
	var maxPr = tbl.recordCount;
	var sPr = UR("Імпорт документів: |Импорт документов: ");
	var progressM = new DpProgressMonitor(sPr, maxPr);
	progressM.start();
	var nPr = 1;
	var nTime = new Date().getTime();
	
	var nRow = 0;	
	tbl.MoveFirst();
	
	var oDoc = null;
	var groupVal = "";	
	var bNewDoc = true;
	var bSaveDoc = true;
	var rowNum = 1;
	
	while(!tbl.isEOF())
	{
		if(nPr%10==0)
		{
			WAIT(sPr + (nPr) +"/"+ maxPr);		
			progressM.setPos(nPr, (nPr) +"/"+ maxPr);
		}
		//if(nPr>100)
		//	break;
		trace(">>> TIME: "+ (new Date().getTime()-nTime));
		nTime = new Date().getTime();
		trace("\n>>> "+ sPr + (nPr) +"/"+ maxPr);
		nPr++;
		
		var bRet = true;		
		if(bGroupRow)
		{
			bNewDoc = false;
			var sFld = this.getFieldName(sGroupField);
			var oVal = String(tbl.getValue(sFld));
			//trace("\n\nsGroupCond: "+ sGroupCond +"\nsGroupField: "+ sGroupField +"\nsFld: "+ sFld +"\noVal: "+ oVal +"\ngroupVal: "+ groupVal +"\nsGroupKey: "+ sGroupKey)
			if(sGroupCond=="GROUP_KEY")
			{
				if(oVal==sGroupKey)
					bNewDoc = true;
			}			
			else if(groupVal!=oVal)
			{
				groupVal = oVal;
				bNewDoc = true;
			}
			//trace("bNewDoc: "+ (bNewDoc) +" > oDoc : "+ oDoc +"\n\n")
			if(bNewDoc && oDoc)
				bRet = oDoc.save(true, "ADD");
		}		
		if(bNewDoc)
		{
			rowNum = 1;
			oDoc = new DpDoc(sDoc, Number(sNop));
		}
		if(!oDoc)
		{
			tbl.MoveNext();
			continue;
		}	
		//var s = "\n";
		var arrExpr = new Object();
		var docNum = nRow;
		var isEmpty = true;
		
		this.orgCod = 0;
		var orgVar = oDoc.getVar("attr.FLDORG");
		if(!orgVar)
			orgVar = "HORG";
		if(this.varMap[orgVar])
		{
			var fld = this.varMap[orgVar];
			var sType = this.getVarType(orgVar);
			var oVal = this.getValue(fld, tbl, sType, orgVar);
			this.orgCod = this.createObject(sType, oVal, orgVar);
		}
		
		for(sVar in this.varMap)
		{
			var fld = this.varMap[sVar];
			if(!fld || !sVar)
				continue;			
			//s += "\n"+ sVar +" - "+ fld +" > "+ (this.isExpr(fld))
			if(this.isExpr(fld))
			{
				arrExpr[sVar] = this.getExpr(fld);
				//continue;
			}
			var sType = this.getVarType(sVar);
			var oVal = this.getValue(fld, tbl, sType, sVar);
			if(oVal==null || oVal=="undefined")
				continue;
			if(this.isObject(sType))
			{
				oVal = this.createObject(sType, oVal, sVar);				
				if(sType.indexOf(".cl")!=-1)
				{
					var objCod = this.getObjCod(sVar);
					if(Number(objCod)==Number(getPar("CODPLAT")))
						this.orgCod = oVal;
				}				
			}
			if(this.isExt(sVar))
			{
				var sExt = sVar.substring(2);					
				oDoc.setExt(sExt, oVal);
			}
			else
			{
				oDoc.setVar(sVar, oVal);
			}
			if(oVal)
				isEmpty = false;
		}
		if(oDoc.getVar("HNOM"))
		{
			//var nom = Number(oDoc.getVar("HNOM"));
			//if(nom)
			//	oDoc.jDoc.setVar("HNOM", java.lang.Integer(nom));
		}
		else
		{
			include("doc/DpAutoNo.js");
			oDoc.autoNo = new dpAutoNo(oDoc);
			oDoc.jDoc.setVar("HNOM", oDoc.autoNo.get());
		}
		if(this.orgCod && !oDoc.getVar("HID_DOG"))
		{
			var dogID = this.getCntrctByOrg(this.orgCod);
			if(dogID)
				oDoc.setVar("HID_DOG", dogID);
		}
		
		if(oDoc.instanceOf("com.debet.dogovor"))
		{
			if(!this.checkCod(oDoc.getVar("HDOG_NUM"), 0, "cls.cntrct"))
			{
				tbl.MoveNext();
				continue;
			}
		}
				
		this.importRows(oDoc, tbl, rowNum++);
		//trace(s +"\n\n")
		for(sVar in arrExpr)
		{
			var sExp = arrExpr[sVar];
			var oVal = oDoc.eval(sExp);			
			if(sVar && oVal)
				oDoc.setVar(sVar, oVal);
		}
		
		tbl.MoveNext();
		
		if(!bGroupRow || tbl.isEOF())
			bRet = oDoc.save(true, "ADD");
		if(!bRet && stopMassage(sDoc +" - "+ sNop))
			return false;
		
		if(oDoc.hasExt())
			oDoc.saveExtToDB();
			
		nRow++;		
	}
	progressM.stop();
	
	if(this.showMsg)
	{
		var nImp = this.sourceNum? this.sourceNum+" - " : "";
		alert(UR("Імпорт "+ nImp + this.source +" завершено.\nЗавантажено рядків: "+ nRow 
				+"|Импорт "+ this.source +" завершен.\nЗагружено строк: "+ nRow));
	}
	return true;
}

function DbImporter_updateDoc(sDoc, sNop, tbl)
{				
	var arrLinks = this.getLinks();
	if(!arrLinks)
		return false;
	
	var maxPr = tbl.recordCount;
	var sPr = UR("Імпорт документів: |Импорт документов: ");
	var progressM = new DpProgressMonitor(sPr, maxPr);
	progressM.start();
	var nPr = 1;
	var nTime = new Date().getTime();
	
	var oDoc = new DpDoc(sDoc, Number(sNop));
	var sTblDoc = oDoc.getTableName();
		
	tbl.MoveFirst();
	while(!tbl.isEOF())
	{	
		if(nPr%10==0)
		{
			WAIT(sPr + (nPr) +"/"+ maxPr);
			progressM.setPos(nPr, (nPr) +"/"+ maxPr);
		}
		trace(">>> TIME: "+ (new Date().getTime()-nTime));
		nTime = new Date().getTime();
		trace("\n>>> "+ sPr + (nPr) +"/"+ maxPr);
		nPr++;
		
		var sSQL = "select max(FWID) "
					+" from ^"+ sTblDoc
					+" where 1=1";
		for(fld in arrLinks)
		{
			var sVar = arrLinks[fld];
			var sFld = oDoc.getFieldName(sVar);
			sSQL += " and "+ sFld +"="+ sqlTo(this.getValue(fld, tbl, null, sVar));
		}
		var oSnap = getSnap(sSQL, false);
		if(!oSnap || !oSnap[0])
		{
			tbl.MoveNext();
			continue;
		}

		var nID = Number(oSnap[0]);
		oDoc.clear();		
		if(!oDoc.loadByID(nID))
		{
			tbl.MoveNext();
			continue;
		}
		
		for(sVar in this.varMap)
		{
			var fld = this.varMap[sVar];						
			var sType = this.getVarType(sVar);
			var oVal = this.getValue(fld, tbl, sType, sVar);
			if(this.isObject(sType))
				oVal = this.createObject(sType, oVal, sVar);
			if(oVal!=null && oVal!="undefined")
				oDoc.setVar(sVar, oVal);
		}
		
		this.importRows(oDoc, tbl);
		
		var bRet = oDoc.save(true, "EDIT");
		if(!bRet && stopMassage(sDoc +" - "+ sNop))
			return false;
		tbl.MoveNext();		
	}
	progressM.stop();
	return true;
}

function DbImporter_importDocRows(tbl, sDoc, sNop, docID)
{	
	var oDoc = new DpDoc(sDoc, Number(sNop));
	var sMode = "ADD";
	if(docID && oDoc.loadByID(docID))
		sMode = "EDIT";
	
	var maxPr = tbl.recordCount;
	var sPr = UR("Импорт рядків: |Импорт строк: ");
	var progressM = new DpProgressMonitor(sPr, maxPr);
	progressM.start();
	var nPr = 1;
	
	var nRow = 0;	
	tbl.MoveFirst();
	while(!tbl.isEOF())
	{
		if(nPr%10==0)
		{
			WAIT(sPr + (nPr) +"/"+ maxPr);		
			progressM.setPos(nPr, (nPr) +"/"+ maxPr);
		}
		nPr++;
						
		this.importRows(oDoc, tbl, nRow+1);		
				
		tbl.MoveNext();
		nRow++;		
	}
	progressM.stop();
	
	var bRet = oDoc.save(true, sMode);
	
	if(bRet && this.showMsg)
	{
		var nImp = this.sourceNum? this.sourceNum+" - " : "";
		alert(UR("Імпорт "+ nImp + this.source +" завершено.\nЗавантажено рядків: "+ nRow 
				+"|Импорт "+ nImp + this.source +" завершен.\nЗагружено строк: "+ nRow));
	}
	return bRet;
}

function DbImporter_importRows(obj, tbl, nRow)
{
	if(!this.rowMap || this.rowMap.length<1)
		return;	
	for(var sRowClass in this.rowMap)
	{		
		var varMap = this.rowMap[sRowClass];
		if(!varMap || !sRowClass || sRowClass=="CLNUM")
			continue;
		var oRow = obj.createRow(sRowClass);
		//trace("\n>>> "+ sRowClass +" > "+ oRow)
		if(!oRow)
			continue;		
		var jObj = oRow.getJObject();
		var sRowTbl = this.getObjTableName(jObj);
		obj.appendRow(sRowClass, oRow);		
		var rowNum = nRow;
		var isEmpty = true;
		for(var sVar in varMap)
		{
			var fld = varMap[sVar];			
			if(allt(fld)=="")
				continue;
			var oVal = this.getValue(fld, tbl, this.getVarType(sVar), sVar);			
			if(oVal==null)
				continue;			
			var sType = this.getVarType(sVar);			
			if(this.isObject(sType))
				oVal = this.createObject(sType, oVal, sVar);				

			if(sVar=="HID")
			{
				if(!oVal)
					continue;				
			}
			if(sVar=="RNOM")
				rowNum = oVal;			
			
			if(oVal!=null && oVal!="null" && oVal!="undefined")
				oRow.setVar(sVar, oVal);
			
			if(oVal)
				isEmpty = false;
		}		
			
		if(rowNum)
			oRow.setVar("RNOM", rowNum);
			
		obj2DB(oRow.getID(), jObj, null, sRowTbl);
	}
}

function DbImporter_importPrk(nVid, tbl)
{
	var nVid = Number(nVid);
	var sPrkLbl = "";
	var snap = getSnap("select Fedit from ^cl_prik where Fvid="+nVid);
	if(snap && allt(snap[0])!="")
		sPrkLbl = allt(snap[0]);
	else
	{
		alert(ru("Не удалось определить метку приказа с кодом ","Не вдалося визначити мітку наказу з кодом ")+nVid,2);
		return false;
	}
	if(typeof importPrk=="function")
	{
		this.importPrk = importPrk;
		return this.importPrk(sPrkLbl, nVid, tbl, this);
	}
	
	if(this.getMode()=="edit")
	{
		//TO DO: updatePrk()
		//return this.updatePrk(sPrkLbl, nVid, tbl);
		alert("Режим обновления приказов пока не поддерживается.",1);
		return false;
	}
		
	var maxPr = tbl.recordCount;
	var sPr = "Импорт приказов: ";
	var progressM = new DpProgressMonitor(sPr, maxPr);
	progressM.start();
	var nPr = 1;
	var nTime = new Date().getTime();
	
	var oPrk = new DpPrik(sPrkLbl, nVid);
	
	var sOutID = allt(this.varMap["HOUTID"]);	
	var arrLinks = this.getLinks();
	var bPassExisting = this.passExisting;
	if(sOutID && !arrLinks)
		bPassExisting = false;
	var nRow = 0;
	tbl.MoveFirst();
	while(!tbl.isEOF())
	{	
		if(nPr%10==0)
		{
			WAIT(sPr + (nPr) +"/"+ maxPr);
			progressM.setPos(nPr, (nPr) +"/"+ maxPr);
		}
		trace(">>> TIME: "+ (new Date().getTime()-nTime));
		nTime = new Date().getTime();
		trace("\n>>> "+ sPr + (nPr) +"/"+ maxPr);
		
		if(this.type=="DBF")
		{
			if(this.nStart>1 && nPr<this.nStart)
			{
				tbl.MoveNext();
				continue;
			}
			if(this.nEnd>0 && nPr>this.nEnd)
				break;
		}
		nPr++;
				
//		if(sOutID && this.passExisting)
//		{
//			var sID = tbl.getValue(this.varMap["HOUTID"]);
//			var codCL = this.findCl(nVid, "", sID);			
//			if(codCL && codCL!=0)
//			{
//				trace(">>> FOUTID: "+ sID +" > cod: "+ codCL)
//				tbl.MoveNext();
//				continue;
//			}
//		}
		
		var passRow = false;
		if(bPassExisting)
		{						
			var sWhere = "";
			if(arrLinks)
			{				
				for(fld in arrLinks)
				{
					var sVar = arrLinks[fld];
					var fld = this.getFieldName(fld);
					var sType = this.getVarType(sVar);
					var oVal = this.getValue(fld, tbl, sType, sVar);
					if(oVal==null)
						continue;
					if(this.isExt(sVar))
					{
						sVar = sVar.substring(2);			
						var sExtTblName = this.base +".PRK_"+ nVid;
						if(SUBD==MSSQL)
							sExtTblName = this.base +"..PRK_"+ nVid;
						sWhere += " and FWID=(select max(ext.FMAINWID) "
											+" from "+ sExtTblName +" as ext " 
											+" where ext."+ sVar +"="+ sqlTo(oVal)
											+")";						
					}
					else
					{
						sFldTo = oPrk.getFieldName(sVar);
						sWhere += " and "+ sFldTo +"="+ sqlTo(oVal);
					}				
				}
			}
			else
			{
				for(sVar in this.varMap)
				{			
					var fld = this.varMap[sVar];
					var sType = this.getVarType(sVar);
					var oVal = this.getValue(fld, tbl, sType, sVar);					
					if(oVal==null || this.isObject(sType))
						continue;
					if(this.isExt(sVar))
					{
						sVar = sVar.substring(2);			
						var sExtTblName = this.base +".PRK_"+ nVid;
						if(SUBD==MSSQL)
							sExtTblName = this.base +"..PRK_"+ nVid;
						sWhere += " and FWID=(select max(ext.FMAINWID) "
										+" from "+ sExtTblName +" as ext " 
										+" where ext."+ sVar +"="+ sqlTo(oVal)
										+")";						
					}
					else
					{
						sFldTo = oPrk.getFieldName(sVar);
						sWhere += " and "+ sFldTo +"="+ sqlTo(oVal);
					}
				}
			}
			
			var sSQL = "select p.* from ^prk_all as p ";
			var arrTbls = oPrk.getTables();
			if(arrTbls && arrTbls.length>0)
			{
				for(var i=0; i<arrTbls.length; i++)
				{
					if(!arrTbls[i])
						continue;
					var sTable = String(arrTbls[i]).toUpperCase().trim();
					if(sTable!="PRK_ALL")
						sSQL += " inner join ^"+ sTable +" as t"+i+" on t"+i+".FWID=p.FWID"
				}
			}
			sSQL += " where p.Fvid_prk="+ nVid + sWhere;
			var oSnap = getSnap(sSQL);
			if(oSnap)
			{
				passRow = true;
			}
			else
			{
				passRow = false;
			}
		}
		if(passRow)
		{
			tbl.MoveNext();
			continue;
		}
		var parentCod = 0;				
		var isEmpty = true;
		oPrk = new DpPrik(sPrkLbl, nVid);
		var hasExts = false;
		for(sVar in this.varMap)
		{
			var fld = this.varMap[sVar];
			if(allt(fld)=="")
				continue;
			var sType = this.getVarType(sVar);									
			var oVal = this.getValue(fld, tbl, sType, sVar);
			if(oVal!=null && oVal!="null")
			{
				if(this.isObject(sType))
					oVal = this.createObject(sType, oVal, sVar);
				if(this.isExt(sVar))
				{
					var sExt = sVar.substring(2);					
					oPrk.setExt(sExt, oVal);
					hasExts = true;
				}
				else
				{
					oPrk.setVar(sVar, oVal);
				}
				if(sVar=="HIDPAR")
					parentCod = oVal;
				isEmpty = false;
			}
		}
		if(isEmpty)
		{
			tbl.MoveNext();
			continue;
		}
		if(Number(oPrk.getVar("FTABN_ID"))==0)
		{
			var msg = ru("Отсутствует идентификатор сотрудника для записи ","Немає ідентифікатора співробітника для запису ")+tbl.bookmark;
			trace(msg);
			WAIT(msg);
			tbl.MoveNext();
			continue;
		}
		
		var noXML = oPrk.getVar("ATTR.NOXML");
		oPrk.setAttr("NOXML", true);
		
		this.importRows(oPrk, tbl);
		
		var bRet = oPrk.save("ADD", true);
		oPrk.setAttr("NOXML", noXML);
		if(!bRet && stopMassage(sPrkLbl +" - "+ nVid +": "+ getCodFromID(oPrk.getVar("FTABN_ID"))))
			return false;
		if(hasExts)
			oPrk.saveExtToDB();
			
		tbl.MoveNext();
		nRow++;
	}
	progressM.stop();
	
	if(this.showMsg)
	{
		var nImp = this.sourceNum? this.sourceNum+" - " : "";
		alert(UR("Імпорт "+ nImp + this.source +" завершено.\nЗавантажено наказів: "+ nRow 
				+"|Импорт "+ nImp + this.source +" завершен.\nЗагружено приказов: "+ nRow));
	}
	return true;
}

function DbImporter_getObjTableName(jObj)
{
	var mapper = jObj.getDefaultMapper();
	if (!mapper)
		return null;
	
	var arrTables = mapper.getTables();
	if (arrTables.size() == 0)
		return null;
		
	tableName = String(arrTables.get(0).name); 
	return tableName;
}

function stopMassage(msg, sql)
{
	var sCapt = ru("Импорт","Імпорт");
	var sMsg = ru("Ошибка при импорте: ", "Помилка при імпорті: ") + msg;
	var sError = lpProxy.getDBManager().getLastErrorMessage();
	if(sError)
	{
		sMsg += "\n"+ sError;
		this.arrErr[this.arrErr.length] = msg +" - "+ sError +(sql? " : "+ sql : "");
	}
	sMsg += "\n"+ ru("Продолжить импорт?", "Продовжити імпорт?");
	var ret = confirm(sMsg, sCapt, "IMPORT");
	return ret==false;
}

function DbImporter_createTbl(sTblName, arrFld)
{
	if(!arrFld)
		arrFld = this.varMap;
	//ExecuteSQL("DROP TABLE "+ sTblName, false);
	var sSql = "create table "+ sTblName +"( ";
	for(fld in arrFld)
	{
		var sType = this.getVarType(fld);
		if(sType && sType!="" && fld.toUpperCase()!="FWID")
			sSql += fld +" "+ getStruType(sType) +",";
	}
	//sSql = sSql.substring(0, sSql.length-1);
	sSql += " FWID "+ SqlType("LONG");	
	sSql += " )";
	return ExecuteSQL(sSql);
}

function DbImporter_createSourceTbl(sTblName, fldMap)
{
	if(!fldMap)
		fldMap = this.fldMap;
	ExecuteSQL("DROP TABLE "+ sTblName, false);
	var sSql = "create table "+ sTblName +"( ";
	for(fld in fldMap)
	{		
		var sFld = this.getFieldName(fld);
		if(sFld && sFld!="" && sFld.toLowerCase()!="null")
			sSql += sFld +" "+ SqlType("TEXT") +" default "+ sqlEmpStr +",";
	}
	//sSql = sSql.substring(0, sSql.length-1) + " )";
	sSql += " FWID "+ SqlType("COUNTER");	
	sSql += " )";	
	return ExecuteSQL(sSql);
}

function DbImporter_createVarsTbl(sTblName)
{
	if(!sTblName)
		sTblName = this.getVarsTblName();
	ExecuteSQL("DROP TABLE "+ sTblName, false);
	var sSQL = "CREATE TABLE " + sTblName
		 	+"( FWID "+ SqlType("COUNTER")
			+",FSOURCE "+ SqlType("TEXT") +" default "+ sqlEmpStr
			+",FREC "+ SqlType("TEXT", 254) +" default "+ sqlEmpStr
			+",FTYPE " + SqlType("TEXT", 30) +" default "+ sqlEmpStr
			+",FLENGTH "+ SqlType("INT") +" default 0 "
			+",FTXT " + SqlType("TEXT") +" default "+ sqlEmpStr
			+",FROWCLASS " + SqlType("TEXT", 30) +" default "+ sqlEmpStr
			+",FROW " + SqlType("BIT") +" default "+ sqlFalse
			+",FPARAM "+ SqlType("LONGTEXT")
			+",FNUM "+ SqlType("INT") +" default 0 "
			+",FCOD "+ SqlType("TEXT") +" default "+ sqlEmpStr
			+",FCAPTION "+ SqlType("LONGTEXT") 
			+",FNEWSOURCE "+ SqlType("BIT") +" default "+ sqlFalse
			+",FOBJSOURCE "+ SqlType("LONGTEXT") 
			+",FLINK "+ SqlType("LONGTEXT") 	
			+",FOBJCOD "+ SqlType("TEXT", 254) +" default "+ sqlEmpStr
		+")";
	if(!ExecuteSQL(sSQL, false))
		ExecuteSQL("delete from "+ sTblName, false)
	return true;
}

function DbImporter_getVarsTblName()
{
	return this.varsTblName;
}

function DbImporter_getVarType(sFld, sVarsTbl)
{
	if(this.arrType[sFld])
		return this.arrType[sFld];
	var sType = null;
	if(!sVarsTbl)
		sVarsTbl = this.getVarsTblName();
	var oSnap = getSnap("select FTYPE from "+ sVarsTbl 
					+" where UPPER(FREC)="+ sqlTo(sFld.toUpperCase()));
	if(oSnap)
		sType = oSnap[0];
	if(!this.arrType[sFld])
		this.arrType[sFld] = sType;
	return sType;
}

function DbImporter_getVarLength(sVar, sType)
{
	var nLength = 0;
	var oVar = this.arrVars[sVar];
	if(sVar && oVar)
	{		
		if(oVar.length)
			return oVar.length;
	}
	if(!sType)
	{
		var oSnap = getSnap("select FLENGTH from "+ this.varsTblName 
					+" where UPPER(FREC)="+ sqlTo(sVar.toUpperCase()));
		if(oSnap && Number(oSnap[0]))
			nLength = Number(oSnap[0]);
		return nLength;
	}
	
	sType = String(sType).trim().toLowerCase();
	if(sType.indexOf("(")!=-1)
	{
		var str = sType.substring(sType.indexOf("(")+1, sType.indexOf(")")).trim();
		if(str && Number(str))
			nLength = Number(str);
	}
	if(sType.indexOf("text")!=-1 && (nLength>255 || nLength==0))
		sType = "longtext";
	if(!oVar)
	{
		oVar = new DbImporter_var(sVar);
		oVar.length = nLength;
		this.arrVars[sVar] = oVar;
	}
	if(oVar && !oVar.length)
	{
		oVar.length = nLength;
		this.arrVars[sVar] = oVar;
	}
	return nLength;
}

function DbImporter_getObjCod(sVar)
{
	var cod = null;
	var sVarsTbl = this.getVarsTblName()
	var oSnap = getSnap("select FOBJCOD from "+ sVarsTbl 
					+" where UPPER(FREC)="+ sqlTo(sVar.toUpperCase()));
	if(oSnap)
		cod = oSnap[0];
	return cod;
}

function DbImporter_getValue(sFld, tblSource, sType, sVar)
{
	var oVal = null;
	if(this.isExpr(sFld))
	{
		oVal = this.oEval.eval(this.getExpr(sFld));		
	}
	else
	{
		sFld = this.getFieldName(sFld);
		oVal = tblSource.getValue(sFld);
	}
	if(oVal==null)
		return null;	
	if(!sType)
		sType = this.getVarType(sVar)
	oVal = this.getCorValue(oVal, sType, sVar);
	return oVal;
}

function DbImporter_getCorValue(tVal, sType, sVar)
{
	if(tVal==null)
		return tVal;
	var oVal = tVal;
	if(typeOf(oVal)=="string" || typeOf(oVal)=="String")
		oVal = oVal.trim().replace("\n", "");		
	var nType = 0;	
	sType = String(sType).trim().toLowerCase();	
	var nLength = this.getVarLength(sVar, sType);
	if(sType.indexOf("(")!=-1)
		sType = sType.substring(0, sType.indexOf("("));
	
	switch(sType)
	{
		case "text":
			oVal = String(oVal);
			if(!nLength)
				nLength = this.getVarLength(sVar);
			if(nLength>0 && nLength<oVal.length)				
				oVal = oVal.substring(0, nLength);
			break;
			
		case "int":
		case "short":
		case "long":
		case "currency":
		case "double":		
			oVal = String(oVal).replace(",", ".").trim();
			oVal = java.lang.String(oVal).replaceAll("[^0-9.]", "")
			oVal = Number(oVal);
			if(isNaN(oVal))
				oVal = null;
			break;
		
		case "bit":
			if(String(oVal)=="true" || String(oVal)=="1")
				oVal = true;
			if(String(oVal)=="false" || String(oVal).trim()=="")
				oVal = false;
			oVal = Boolean(oVal);
			if(isNaN(oVal))
				oVal = null;
			break;
		
		case "date":
			nType = 3;			
			if(!oVal || oVal=="" || oVal=="null" || oVal=="undefined")
				return d00;
			if(isDateType(oVal))
				return new Date(oVal).getVarDate();
			var dpVal = this.DpAPI.str2Obj(oVal, nType);
			if(dpVal)
				return dpVal;
			if(typeOf(oVal)=="string" || typeOf(oVal)=="String")
				oVal = strToDate(String(oVal));
			else
			{
				oVal = new Date(oVal);
				if(oVal!=null && !isNaN(oVal))
					oVal = oVal.getVarDate();
				else
					oVal = null;
			}
			break;
			
		default:
			oVal = String(oVal);
			break;		
	}
	if(oVal=="undefined" || oVal=="nan")
		oVal = null;
	if(typeof transImportValue=="function")
		oVal = transImportValue(tVal, sType, sVar, oVal);
	return oVal;
}

function DbImporter_getNullValue(sType)
{	
	var oVal = "";
	sType = String(sType).trim().toLowerCase();	
	switch(sType)
	{
		case "int":
		case "short":
		case "long":
		case "currency":
		case "double":		
			oVal = 0;
			break;
		
		case "bit":
			oVal = false;			
			break;
		
		case "date":
			oVal = d00;			
			break;
			
		default:
			oVal = "";
			break;		
	}
	return oVal;
}

function DbImporter_getJSFile(sParFile)
{
	this.sJsFile = "";
	if(!sParFile)
		sParFile = this.sFileParam;
	if(!isFile(sParFile))
		return false;
	sParFile = String(DpFile(sParFile).getPath());
	if(sParFile!="" && sParFile.length>4)
		this.sJsFile = sParFile.substring(0, sParFile.length-4) +".js";	
	this.sJsFile = DpFile(this.sJsFile).getPath();
	this.sJsFile = "/"+ java.lang.String(this.sJsFile).replace("\\", "/");
	if(this.sJsFile && isFile(this.sJsFile))
		include(this.sJsFile);
	//trace("\n>>> "+ sParFile +"\n>>> "+ this.sJsFile +" > "+ (isFile(this.sJsFile)) +" > "+ (include(this.sJsFile)) +" > "+ (typeof transImportValue) +"\n")
	return this.sJsFile;
}

function DbImporter_setParFile(sFileName)
{
	this.sFileParam = sFileName;
}

function DbImporter_getParFile(sSource)
{
	if(!sSource)
		return this.sFileParam;
	var sFileName = sSource;
	if(this.type=="DBF" || this.type=="CSV" || this.type=="XLS")
	{
		sFileName = new DpFile(sFileName).getName();
		sFileName = sFileName.substring(0, sFileName.length()-4);
		sFileName = this.getPath() + sFileName +".xml";
	}
	this.sFileParam = sFileName;
	return sFileName;
}

function DbImporter_getPath()
{	
	return this.path;
}

function DbImporter_getSourceName()
{
	return this.sourceTblName; 
}

function DbImporter_getLinks(sSource, nSource)
{
	if(!nSource)
		nSource = this.sourceNum;
	if(!sSource)
		sSource = this.source;
	var links = new Object();
	var sParFile = this.getParFile();
	if(!isFile(sParFile))
		return null;
	var xmlDoc = new DpXMLDOMDocument();
	xmlDoc.load(sParFile);
	var xmlDocElement = xmlDoc.getDocumentElement();
	var eTbl = this.getXmlTbl(xmlDocElement, sSource, nSource);
	if(!eTbl)
		return null;
	var eLinks = eTbl.getElementsByTagName("links")[0];
	if(!eLinks)
		return null;
	var listLinks = eLinks.getChildNodes();
	if(listLinks.length==0)
		return null;
	for (var i=0; i<listLinks.length; i++)
	{
		var eLink = listLinks[i];		
		var sSource = eLink.getAttribute("source");
		var sLink = eLink.getAttribute("name");
		links[sSource] = sLink;
	}
	return links;
}

function DbImporter_isReplace(sTblName)
{
	return this.replace;
}

function DbImporter_getMode(sTblName)
{
	return this.mode;
}

function DbImporter_saveExt(obj, arrExt, rowID)
{
	if(!obj || !arrExt || !rowID)
		return false;
		
	if(obj=="MTR_CL_NMK")
	{
		var sFullTblName = this.base +".CL_NMK";
		if(SUBD==MSSQL)
			sFullTblName = this.base +"..CL_NMK";
		var oSnap = getSnap("select FWID from "+ sFullTblName 
							+" where FMAINWID="+ rowID);
		var m = new Object();		
		for(sVar in arrExt)
		{
			if(arrExt[sVar]==null)
				continue;
			m[sVar.substring(2)] = arrExt[sVar];
		}
		m.FMAINWID = rowID;
		if(oSnap && oSnap[0])
			m.FWID = oSnap[0];
		else
			m.FWID = GetUID();
		UpdateTable(sFullTblName, m, oSnap==null);
		return true;
	}
	
	if(obj.hasExt && obj.hasExt())
	{
		for(sVar in arrExt)
		{
			if(arrExt[sVar]==null)
				continue;
			var sExt = sVar.substring(2);
			obj.setExt(sExt, arrExt[sVar]);
		}
		return true;
	}
	
	return false;
}

function DbImporter_getDpType(sType)
{
	sType = String(sType).toLowerCase();
	if(sType.indexOf("text")!=-1)
		return "text";
	switch(sType)
	{
		case "bool":
		case "5":
			return "bit";		
		case "datetime":		
		case "3":
			return "date";
		case "float":
		case "4":
			return "double";
		case "int":
		case "2":
			return "long";		
		case "nmk":
			return "cid";
		case "longtext":
		case "1":
			return "text";		
		default:
			return sType;
		return sType;
	}
}

function DbImporter_fillStruTables()
{
	if(this.arrStruTables)
		return this.arrStruTables;
	
	this.arrStruTables = new Object();
	var xmlDom = new DpXMLDOMDocument();
	xmlDom.load(rootDirectoryEx() + "dbstru.xml");
	var listInfo = xmlDom.getDocumentElement();
	var listTbl = listInfo.getElementsByTagName("table");
	for(var nTbl=0; nTbl<listTbl.length; nTbl++)
	{
		var eTbl = listTbl[nTbl];
		if(!eTbl)
			continue;
		var oTable = new DpTable();
		oTable.name = String(eTbl.getAttribute("table")).toUpperCase();
		oTable.pattern = String(eTbl.getAttribute("pattern")).toUpperCase();
		oTable.sys = String(eTbl.getAttribute("sys")).toUpperCase();
		oTable.attr = eTbl.getAttribute("attr");
		oTable.rem = UR(eTbl.getText());
		//trace(oTable.name +" > "+ oTable.sys +" > "+ oTable.rem)
		this.arrStruTables[oTable.name] = oTable;
		if(oTable.sys)
			this.arrStruTables[oTable.sys +"_"+ oTable.name] = oTable;
	}
}

function DpTable()
{
	this.name = "";
	this.pattern = "";
	this.sys = "";
	this.attr = "";
	this.rem = "";	
}

function DbImporter_isStruTable(sName)
{
	if(!this.arrStruTables)
		this.fillStruTables();
	if(this.arrStruTables)
		return this.arrStruTables[sName]!=null;
	return true;
}

function DbImporter_setRange(nFirstRow, nLastRow)
{
	this.nStart = Number(nFirstRow);
	this.nEnd = Number(nLastRow);
}

function DbImporter_browse(tbl)
{
	var par = new Object();
	par.caption = UR("Для продовження імпорту закрийте це вікно.|Для продолжения импорта закройте это окно.");
	
	if(this.type=="DBF")
	{
		browse(tbl, par, SW_MODAL);	
		return continueMessage();
	}
	
	par.cells = new Object();
	var nFld = 1;
	for(var fld in this.fldMap)
	{
		var sField = this.getFieldName(fld);
		if(Number(sField))
			continue;
		par.cells[nFld] = new Object();
		par.cells[nFld].name = sField;
		par.cells[nFld].width = 15;
		par.cells[nFld].text = this.fldMap[fld];
		nFld++;
	}	
	
	if(!tbl)
		tbl = OpenTable("select * from "+ this.getSourceName());
	if(typeof tbl=="String" || typeof tbl=="string")
		tbl = OpenTable("select * from "+ tbl);
		
	browse(tbl, par, SW_MODAL);
	
	return continueMessage();
}

function continueMessage()
{
	var sMsg = UR("Продовжити імпорт?|Продолжить импорт?");
	return confirm(sMsg);
}

function DbImporter_checkCondition(oVal)
{
	if(!this.conditionCol)
		return true;
	var sCond = String(this.condition).toLowerCase();	
	switch(this.conditionMode)
	{
		case "==":
			if(String(oVal).toLowerCase()==sCond)
				return true;
			break;
			
		case "=":
			if(String(oVal).toLowerCase().indexOf(sCond)!=-1)
				return true;
			break;
		
		case "!":
			if(String(oVal).toLowerCase()!=sCond)
				return true;
			break;
		
		case "!=":
			if(String(oVal).toLowerCase().indexOf(sCond)==-1)
				return true;
			break;
		
		case ">":
			if(Number(oVal)>Number(sCond))
				return true;
			break;
		
		case "<":
			if(Number(oVal)<Number(sCond))
				return true;
			break;
		
		case ">=":
			if(Number(oVal)>=Number(sCond))
				return true;
			break;
		
		case "<=":
			if(Number(oVal)<=Number(sCond))
				return true;
			break;
		
		default:
			return true;
	}
	return false;
}

//создаются новые расширения
function DbImporter_createExtentions(sClass)
{
	for(sVar in this.varMap)
	{
		if(!sVar)
			continue;
		sVar = String(sVar).trim();
		if(this.isExt(sVar))
		{
			var sType = this.getVarType(sVar);
	        var sTxt = "";
	        var oSnap = getSnap("select FTXT from "+ this.varsTblName +" where UPPER(FREC)="+ sqlTo(sVar.toUpperCase()));
	        if(oSnap)
	        	sTxt = oSnap[0]; 
			var sLabel = sVar.substring(3);
			if(sClass=="CL_NMK")
				addNmkExt(sLabel, sType, sTxt);
		}
	}
}

//функция для добавления расширений номенклатуры
function addNmkExt(sLabel, sType, sTxt)
{
	sLabel = sLabel.toUpperCase().trim();
	var oSnap = getSnap("select FWID, FTXT from ^CL_EXT where UPPER(FCLASS)='CL_NMK' and UPPER(FLABEL)="+ sqlTo(sLabel));
	if(oSnap && oSnap[0])
		return;
	
	var num = 1;
	oSnap = getSnap("select max(FPOS) from ^CL_EXT where UPPER(FCLASS)='CL_NMK'");
	if(oSnap && oSnap[0])
		num = Number(oSnap[0])+1;
	
	var extBuilder = new DpExtentionBuilder("CL_NMK");
	extBuilder.load();
		
	var mCl = new Object();	
	mCl.FWID = GetUID();
	mCl.Flabel = sLabel;
	if(sTxt)
		mCl.Ftxt = sTxt;
	else
		mCl.Ftxt = sLabel;
	mCl.Fcomment = sTxt;
	mCl.Fwidth = 10;
	mCl.Fpos = num;
	mCl.Fused = true;
	mCl.FISCHECK = false;
	mCl.Ftype = extType(sType);
	mCl.Fformat = "";
	mCl.Fisculc = false;	
	mCl.FCLASS = extBuilder.FCLASS;
	if (extBuilder.FSUBCLASS)
		mCl.FSUBCLASS = extBuilder.FSUBCLASS;
	UpdateTable(extBuilder.getInfoTableName(), mCl, true);
	
	extBuilder.commitStructure();
	mCl.Fdefault = "";		
	extBuilder.save(mCl);
}

function extType(sType)
{
	sType = String(sType).toLowerCase();
	if(sType.indexOf("text")!=-1)
		return "TEXT";
	switch(sType)
	{
		case "bit":
		case "5":
			return "BOOL";	
		case "double":
		case "4":
			return "FLOAT";
		case "long":
		case "2":
			return "INT";			
		case "1":
			return "TEXT";	
	}
	return sType.toUpperCase();
}

function cleanParam()
{
	WriteParameter(getSysName() +"_IMPORT_EXISTING_NMK", "");
	WriteParameter(getSysName() +"_IMPORT_EXISTING_CL", "");
	WriteParameter(getSysName() +"_IMPORT_EXISTING_DOC", "");
}

function importExistingCL()
{
	var sPar = ReadParameter(getSysName() +"_IMPORT_EXISTING_CL");
	return String(sPar)=="true" || String(sPar)=="1";
}

function IsTableExist(sTable)
{
	var oSnap = getSnap("select * from "+ sTable, false);
	return oSnap!=null;
}

function DbImporter_var(sName, sField, sType)
{
	this.name = sName;
	this.field = sField;
	this.type = sType;
	this.length = 0;
}
