// JScript source code
DpBProcess.prototype.STATE_STARTED = 1;
DpBProcess.prototype.STATE_TERMINATED = 2;
DpBProcess.prototype.STATE_WAITING = 3;

function DpBProcess(classID, owner)
{
	this.classID = classID;
	this.ID = -1;
	
	this.init = DpBProcess_init;
	this.load = DpBProcess_load;
	this.edit = DpBProcess_edit;
	this.save = DpBProcess_save;
	this.objToXml = DpBProcess_objToXml;
	this.xmlToObj = DpBProcess_xmlToObj;
	this.getModelPools = DpBProcess_getModelPools;
	this.getPools = DpBProcess_getPools;
	this.getBPMPath =  DpBProcess_getBPMPath;
	this.getPoolByID = DpBProcess_getPoolByID;
	this.getPoolByChild = DpBProcess_getPoolByChild;
	this.getID = DpBProcess_getID;
	this.createNew = DpBProcess_createNew;
	this.execute = DpBProcess_execute;
	this.activateNextStages = DpBProcess_activateNextStages;
	this.addConnectionTrace = DpBProcess_addConnectionTrace;
	this.checkCondition = DpBProcess_checkCondition;
	this.showState = DpBProcess_showState;
	this.executeStage = DpBProcess_executeStage;
	this.executeTask = DpBProcess_executeTask;
	this.arrPools = new Array();
	this.owner = owner ? owner : getUserName();
	this.getActiveStagesIDs = DpBProcess_getActiveStagesIDs;
	this.init();
	this.rem = "";
	
	this.createDate = today.getVarDate();
	this.startTime = d00;
	this.state = 0;
	this.isStepping = true;
	
	this.start = DpBProcess_start;
}

function DpBProcess_getBPMPath()
{
	return rootDirectory() + "/BPM/" + this.classID;
}


function DpBProcess_init()
{
	var fileName = rootDirectory() + "/BPM/" + this.classID + "/" + this.classID + ".dpbpmn";
	
	if (!isFile(fileName))
		return;
	
	this.bpmnModel = lpProxy.createBPMNDiagram();
	
	var doc = lpProxy.openXmlFile(fileName);   
	var root = doc.getDocumentElement();
    if (root == null)
    	return;
    
    this.bpmnModel.fromXML(root);
    
    var pools = this.bpmnModel.getPools();
    
    for (var i = 0; i < pools.length; i++)
    {
    	var ID = pools[i].getID();
    	var oPool = new DpBProcessPool(this, pools[i]);
    	oPool.setExecutor(this.owner);
    	this.arrPools[ID] = oPool;
    }
    
    
}



function DpBProcess_createNew()
{
	this.rem = "";
	this.owner = getUserName();
	this.createDate = today.getVarDate();
	this.startTime = d00;
	this.state = 0;
	this.isStepping = true;
	this.skippedTasks = new Array();
}

function DpBProcess_getID()
{
	if (this.ID == -1)
		this.ID = GetUID();
	return this.ID;
}

function DpBProcess_load(id)
{
	id = Number(id);
	/*var tbl = OpenTable("select * from ^BPM_EKZ where fwid = " + sqlTo(id))
	if (tbl.isEmpty())
	{
		tbl.close();
		return false;
	}*/
	var oSnap = getSnap("select FREM,FOWNER,FCREATE_DATE,FSTART_TIME,FSTATE,FISSTEPPING"
						+" from ^BPM_EKZ where FWID="+ sqlTo(id)
						, false
						);
	if(!oSnap)
		return false;
	this.skippedTasks = new Array();
	this.ID = id;
	this.rem = String(oSnap[0]);
	this.owner = String(oSnap[1]);
	this.createDate = oSnap[2];
	this.startTime = oSnap[3];
	this.state = Number(oSnap[4]);
	this.isStepping = toScrObj(oSnap[5]);
	
	var tblEx = OpenTable("select * from ^BPM_EXECUTORS where fid_bpm = " + sqlTo(id) + " order by fid_pool")
	if (tblEx.isEmpty())
	{
		tblEx.close();
		return false;
	}
	
	while (!tblEx.isEOF())
	{
		var idPool = tblEx.getValue("FID_POOL"); 
		var executor = String(tblEx.getValue("FEXECUTOR"));
		var oPool = this.getPoolByID(idPool);
		if (oPool)
		{
			oPool.setExecutor(executor);
		}
		tblEx.moveNext();
	}
	
	tblEx.close();
	
	
	return true;
}

function DpBProcess_edit()
{
	var par = new Object();
	par.bProcess = this;	
	showWindow("e_bpmekz.xml","XMLDLG",par);
	return !par.escape;
}

function DpBProcess_save()
{
	ExecuteSQL("delete from ^BPM_EKZ where fwid = " + sqlTo(this.getID()));
	var m = new Object();
	m.FWID = this.getID();
	m.FCLASS = this.classID;
	m.FREM = this.rem;
	m.FOWNER = this.owner;
	m.FCREATE_DATE  = this.createDate;
	m.FSTART_TIME = this.startTime;
	m.FSTATE =  this.state;
	m.FISSTEPPING = this.isStepping;
	UpdateTable("^BPM_EKZ", m, true);
	
	ExecuteSQL("delete from ^BPM_EXECUTORS where fid_bpm = " + sqlTo(this.getID()));
	
	for (var idPool in this.arrPools)
	{
		var m = new Object()
		m.FID_BPM = this.getID();
		m.FID_POOL = Number(idPool);
		m.FEXECUTOR = this.arrPools[idPool].getExecutor();
		
		UpdateTable("^BPM_EXECUTORS", m, true);
	}
}

function DpBProcess_getModelPools()
{
	if (this.bpmnModel)
		return this.bpmnModel.getPools();
	
	return null;
}

function DpBProcess_getPools()
{
	return this.arrPools;
}

function DpBProcess_getPoolByID(ID)
{
	return this.arrPools[ID];
}

function DpBProcess_getPoolByChild(oChild)
{
	for (var id in this.arrPools)
	{
		var pool = this.arrPools[id]
		var obj = pool.model.findByID(oChild.getID());
		return pool;
	}
}

function DpBProcess_start()
{
	this.state = this.STATE_STARTED;
	var startObj = this.bpmnModel.getStartObject();
	if (!startObj)
		return;
	this.skippedTasks = new Array();
	ExecuteSQL("delete from ^BPM_STAGES where fid_bpm = " + sqlTo(this.getID()));
	ExecuteSQL("delete from ^BPM_CONNECTIONS where fid_bpm = " + sqlTo(this.getID()));
	var pool = this.getPoolByChild(startObj);
	var executor = pool.getExecutor();	
	var stage = new DpBProcessStage(this, startObj);
	stage.setExecutor(executor);
	stage.isActive = true;
	stage.save();
	this.save();
	lpProxy.getBProcessManager().skipProcess(this.getID(), false);
}

function DpBProcess_getActiveStagesIDs()
{
	var ret = new Array();
	var tbl = OpenTable ("select FWID from ^BPM_STAGES where fid_bpm = " + this.getID() + " and fis_active = " + sqlTrue);
	while (!tbl.isEOF())
	{
		ret[ret.length] = Number(tbl.getValue("FID_STAGE"));
	}
	tbl.close();
	
	return ret;
}

function DpBProcess_execute()
{
	lpProxy.getBProcessManager().skipProcess(this.getID());
	var isExec = false; 
	var tbl = OpenTable ("select FID_STAGE, FEXECUTOR, FSTARTPARAM from ^BPM_STAGES where fid_bpm = " + this.getID() + " and fis_active = " + sqlTrue);
	while (!tbl.isEOF())
	{
		var idStage =  Number(tbl.getValue("FID_STAGE"));
		if (!this.skippedTasks[idStage])
		{		
			var stageModel = this.bpmnModel.findByID(idStage);
			if (stageModel)
			{
				var stage = new DpBProcessStage(this, stageModel);
				stage.setExecutor(String(tbl.getValue("FEXECUTOR")));
				var strParam = String(tbl.getValue("FSTARTPARAM"));
				if (strParam)
				{
					stage.setStartPar(strParam);
				}
				
				var ret = this.executeStage(stage);
				if (ret)
					isExec = true;
			}
		}
		tbl.moveNext();			
	}
	
	tbl.close();
	if (isExec && this.state != this.STATE_TERMINATED)
	{
		this.execute();
	}
	else
	{
	}
}

function DpBProcess_showState()
{
	var tbl = OpenTable ("select FID_STAGE, FEXECUTOR, FSTARTPARAM from ^BPM_STAGES where fid_bpm = " + this.getID() + " and fis_active = " + sqlTrue);
	this.bpmnModel.setText(this.rem);
	while (!tbl.isEOF())
	{
		var idStage =  Number(tbl.getValue("FID_STAGE"));
		var stageModel = this.bpmnModel.findByID(idStage);
		if (stageModel)
		{
			stageModel.setActive(true);
		}
		tbl.moveNext();
			
	}
	tbl.close();
	
	var arrCon = new Array();
	var tbl = OpenTable ("select distinct FID_CONNECTION from ^BPM_CONNECTIONS where fid_bpm = " + this.getID());
	while (!tbl.isEOF())
	{
		var idConn =  Number(tbl.getValue("FID_CONNECTION"));
		arrCon[idConn] = true;
		tbl.moveNext();
	}
	
	var connections = this.bpmnModel.getAllSourceConnection();
	for (var i = 0; i < Number(connections.size()); i++ )
	{
		var idConn =  Number(connections.get(i).getID());
		if (arrCon[idConn])
		{
			connections.get(i).setPassed(true);
		}
	}
	
	lpProxy.showBProcessState(this.bpmnModel);
}

function DpBProcess_executeStage(oStage)
{
	var typeName = typeOf(oStage.model);
	var isExec = false;
	switch(String(typeName))
	{
	case "DpStartEventObject":
	case "DpGatewayDBExclusiveObject":
		oStage.param = oStage.getStartPar();
		isExec = true;
		break;
	case "DpTaskObject":
		isExec = this.executeTask(oStage);
		break;
	case "DpEndEventObject":
		isExec = true;
		this.state = this.STATE_TERMINATED;
		this.save();
		break;	
	default:
		alert(String(typeName))
		isExec = false;	
	
	}
	if (isExec)
	{
		this.activateNextStages(oStage);
		oStage.isActive = false;
		oStage.save();
		
	}
	return isExec;
}

function DpBProcess_executeTask(oStage)
{
	var startPar = oStage.getStartPar();
	document.BPROCESS = startPar ? startPar : new Object();
	document.BPROCESS.abort = false;
	var fileName = String(Number(oStage.model.getID())) + ".js";
	
	var fullPath = this.getBPMPath() + "/" + fileName;
	if (isFile(fullPath))
	{
		var dpTask = lpProxy.createDpTask();
		dpTask.setPar(document.BPROCESS);
		dpTask.runTask("r", fullPath);
		dpTask.dispose();
	}
	oStage.param = document.BPROCESS;
	
	if (BPROCESS.abort)
	{
		this.skippedTasks[Number(oStage.model.getID())] = true;
	}
	
	return !document.BPROCESS.abort;
}

function DpBProcess_activateNextStages(oStage)
{
	var listCon = oStage.model.getSourceConnections();
	for (var i = 0; i < listCon.size(); i++)
	{
		if (typeOf(oStage.model) == "DpGatewayDBExclusiveObject")
		{
			if (!this.checkCondition(oStage.getStartPar(), String(listCon.get(i).getCondition())))
				continue;
		}
		this.addConnectionTrace(listCon.get(i).getID());
		var stageModel = listCon.get(i).getTarget();
		var pool = this.getPoolByChild(stageModel);
		var executor = pool.getExecutor();	
		var stage = new DpBProcessStage(this, stageModel);
		stage.setStartPar(oStage.param);		
		stage.setExecutor(executor);
		stage.isActive = true;
		stage.save();
		
		if (typeOf(oStage.model) == "DpGatewayDBExclusiveObject")
		{
			break;
		}
	}
}

function DpBProcess_addConnectionTrace(idConn)
{
	var sql = "insert into ^bpm_connections (FID_BPM, FID_CONNECTION, FTIME) values ("
		+ sqlTo(this.getID()) + "," + sqlTo(idConn) + ", " + sqlTo(new Date().getVarDate() ,false, true) + ")";
	ExecuteSQL(sql);
}

function DpBProcess_checkCondition(oPar, cond)
{
	if (!cond)
		return true;
	include ("Objects/DpEval.js")
	var oEval = new DpEval()
	for (var i in oPar)
	{
		oEval.addVar(i, oPar[i]);
	}
	
	var ret = oEval.eval(cond);
	if (ret)
		 return true
	return false;	 
}

function DpBProcessPool(process, modelPool)
{
	this.process = process;
	this.model = modelPool;
	this.executor = null;
	this.setExecutor = DpBProcessPool_setExecutor;
	this.getExecutor = DpBProcessPool_getExecutor;
	this.getText = DpBProcessPool_getText;
}

function DpBProcessPool_setExecutor(executor)
{
	this.executor = executor;
	this.model.setText(this.executor);
}

function DpBProcessPool_getExecutor()
{
	return this.executor;
}

function DpBProcessPool_getText()
{
	return this.model.getText();
}


function DpBProcessStage(process, modelTask)
{
	this.process = process;
	this.model = modelTask;
	this.getText = DpBProcessPool_getText;
	this.executor = "";
	this.isActive = false;
	this.startParam = null;
	this.setExecutor = DpBProcessPool_setExecutor;
	this.getExecutor = DpBProcessPool_getExecutor;
	this.getStartPar = DpBProcessStage_getStartPar;
	this.setStartPar = DpBProcessStage_setStartPar;
	this.save = DpBProcessStage_save;
}

function DpBProcessStage_getStartPar()
{
	return this.startParam;
}

function DpBProcessStage_setStartPar(oPar)
{
	if (typeof(oPar) == "string")
		oPar = this.process.xmlToObj(oPar);
	this.startParam = oPar;
}

function DpBProcessStage_save()
{
	ExecuteSQL("delete from ^BPM_STAGES where fid_bpm = " + sqlTo(this.process.getID())
			+ " and fid_stage = " + sqlTo(this.model.getID()));
	var m = new Object();
	
	m.FID_BPM = this.process.getID();
	m.FID_STAGE = this.model.getID();
	m.FEXECUTOR = this.executor;
	m.FIS_ACTIVE = this.isActive;
	if (this.startParam)
		m.FSTARTPARAM = this.process.objToXml(this.startParam);
	UpdateTable("^BPM_STAGES", m, true);
}

function createBProcess(classID, owner, par)
{
	if (!classID)
		return null;
	
	if (!owner)
		owner = getUserName();
	var oProcess = new DpBProcess(classID, owner);
	
	oProcess.save();	
	return oProcess;
}

function DpBProcess_objToXml(obj, tagName)
{
	if (!tagName)
		tagName = "object";
	include ("Objects/DpXMLDomDocument.js");
	
	var oDom = new DpXMLDOMDocument();
	var root = oDom.createElement(tagName);
	for (var prop in obj)
	{
		value = toScrObj(obj[prop]);
		var type = "text";
		if (typeof(value) == "number")
			type = "number";
		else if (typeof(value) == "boolean")
			type = "boolean";
		else if (typeof(value) == "Date")
		{
			type = "date";
			value = value.getVarDate();
		}
		else if (isDateType(value))
			type = "date";
		
		var strValue = lpProxy.getDpAPI().obj2XmlStr(value);
		var xmlChild = root.createChild(prop, strValue);
		xmlChild.setAttribute("type", type);
	}
	
	return root.getXML();
	
	
}

function DpBProcess_xmlToObj(xmlStr)
{
	if (!xmlStr)
		return new Object();
	include ("Objects/DpXMLDomDocument.js");	
	var ret = new Object();
	var oDom = new DpXMLDOMDocument();
	oDom.loadXML(xmlStr);
	
	var oDoc = oDom.getDocumentElement();
	if (!oDoc)
		return ret;
	
	var childs = oDoc.getChildNodes();
	for (var i = 0; i < childs.length; i++)
	{
		var xmlProp = childs[i];
		var name = xmlProp.getNodeName();
		var sValue = xmlProp.getText();
		var type = xmlProp.getAttribute("type");
		var value = String(sValue);
		
		if (type == "boolean")
		{
			value = toScrObj(lpProxy.getDpAPI().str2Obj(sValue, "boolean"));
		}
		else if (type == "number")
		{
			value = Number(sValue);
		}
		else if (type == "date")
		{
			value = toScrObj(lpProxy.getDpAPI().str2Obj(sValue, "date"));
		}
		ret[String(name)] = value;
	}
	
	return ret;
	
}


