/*
   Описание объекта "элемент справочника банков": DpBank

   Конструктор DpBank, если параметр конструктора true,
   то будет использоваться "жесткая" валидность МФО и названия банка,
   в противном случае валидность проверяться не будет.
    
   Внешние методы:
   .setMFO(MFO,bUpdate)	- установить МФО, если bUpdate==false, то не устанавливаем
   .getMFO()			- получить МФО
   .getNazv()			- получить название банка
   .getAdress()			- получить адресс банка
   .getComments()		- получить примечания по банку
   .valid()				- проверка валидности
   .linkCtrl()			- связать с полем ввода
   .validCtrl()			- реакция на выход из связанного поля ввода
   .zaval()				- осуществляется завал в справочник банков
   .updateCtrl()		- обновляет занчения котролов

   Внутренние свойства:
   .mfo			- МФО банка
   .nazv		- название банка
   .adress		- адресс банка
   .comments	- примечания по банку
   .checkValid	- отвечает за проверку валидности
*/

function DpBank(checkValid)
{
	RO("DpBank", this);
	this.getFWID = DpBank_getFWID;
	this.setFWID = DpBank_setFWID;
	this.setMFO = DpBank_setMFO;
	this.getMFO = DpBank_getMFO;
	this.getCorSc = DpBank_getCorSc;
	this.getNazv = DpBank_getNazv;
	this.getAdress = DpBank_getAdress;
	this.getComments = DpBank_getComments;
	this.valid = DpBank_valid;
	this.linkCtrl = DpBank_linkCtrl;
	this.validCtrl = DpBank_validCtrl;
	this.zaval = DpBank_zaval;
	this.updateCtrl = DpBank_updateCtrl;
	this.mfo = 0;
	this.nazv = "???";
	this.adress = "???";
	this.comments = "";
	this.cor_sc = "";
	this.fwid = 0;
	if(typeOf(checkValid)=="boolean")
		this.checkValid = checkValid;
	else
		this.checkValid = false;
}

function DpBank_setFWID(fwid)
{
	var snap = getSnap("SELECT Fnazv, Faddr, Frem, Fmfo, Fcor_sc "
					+ " FROM ^cl_banks  "
					+ " WHERE Fwid="+fwid);
	if(snap!=null)
	{
		this.mfo = snap[3];
		this.nazv = snap[0];
		this.adress = snap[1];
		this.comments = snap[2];
		this.cor_sc = snap[4];
		this.fwid = fwid;
	}
	else
	{
		this.mfo = 0;
		this.nazv = "???";
		this.adress = "???";
		this.comments = "";
		this.cor_sc = "";
		this.fwid = 0;
	}
	this.updateCtrl();
}

function DpBank_getFWID()
{
	return this.fwid;
}

function DpBank_setMFO(MFO, bUpdate)
{
	if(MFO==0 && !bUpdate)
	{
		this.mfo = 0;
		this.nazv = "???";
		this.adress = "???";
		this.comments = "";
		this.cor_sc = "";
		this.fwid = 0;
		this.updateCtrl();
		return true;
	}
	if(String(bUpdate)!="undefined" && !bUpdate)
		return true;
	if(MFO==null)
		MFO = 0;
	var snap = getSnap("SELECT Fnazv, Faddr, Frem, Fwid, Fcor_sc "
					+ " FROM ^cl_banks  "
					+ " WHERE Fmfo="+sqlTo(String(MFO)));
	if(snap!=null)
	{
		this.mfo = MFO;
		this.nazv = snap[0];
		this.adress = snap[1];
		this.comments = snap[2];
		this.fwid = snap[3];
		this.cor_sc = snap[4];
	}
	else
	{
		this.mfo = 0;
		this.nazv = "???";
		this.adress = "???";
		this.comments = "";
		this.cor_sc = "";
		this.fwid = 0;
	}
	this.updateCtrl();
}

function DpBank_getMFO()
{
	return this.mfo;
}

function DpBank_getNazv()
{
	return this.nazv;
}

function DpBank_getAdress()
{
	return this.adress;
}

function DpBank_getComments()
{
	return this.comments;
}

function DpBank_getCorSc()
{
	return this.cor_sc;
}

function DpBank_linkCtrl(mfo_ctrl, nazv_ctrl, cor_sc_ctrl)
{
	this.mfo_ctrl = mfo_ctrl;
	if (typeof(mfo_ctrl) != "object")
		return;
	this.mfo_ctrl.setMask(MASK_MFO);
	this.mfo_ctrl.isButton = true;
	setValid(this.mfo_ctrl, this, "validCtrl");	
	setButton(this.mfo_ctrl, this, "zaval");
	
	if (typeof(nazv_ctrl) == "object")
	{
		this.nazv_ctrl = nazv_ctrl;
		this.nazv_ctrl.setMask("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&"); // 60
		if(this.checkValid)
			Disable(this.nazv_ctrl);
	}
	if (typeof(cor_sc_ctrl) == "object")
	{
		this.cor_sc_ctrl = cor_sc_ctrl;
		this.cor_sc_ctrl.setMask(MASK_COR_SC);
		if(this.checkValid)
			Disable(this.cor_sc_ctrl);
	}
	this.updateCtrl();
}

function DpBank_updateCtrl()
{
	if(this.mfo_ctrl)
		this.mfo_ctrl.value = this.getMFO();
	if(this.nazv_ctrl)
		this.nazv_ctrl.value = this.getNazv();
	if(this.cor_sc_ctrl)
		this.cor_sc_ctrl.value = this.getCorSc();
}

function DpBank_validCtrl()
{
	this.setMFO(this.mfo_ctrl.value, this.checkValid);	
	if (!this.valid())
		return this.zaval();
	return true;
}

function DpBank_valid()
{
	if(this.checkValid)
		return (this.getFWID() > 0);
	else
		return true;
}

function DpBank_zaval()
{
	var par = new Object();
	par.zaval = true;
	//par.fnazv = this.getNazv();
	par.ret = new Object();
	//par.ret.fnazv = this.nazv;
	//par.ret.fmfo = this.mfo;
	par.ret.fwid = this.fwid;
	//par.ret.faddr = this.adress;
	//par.ret.frem = this.comments;
	showWindow("l_cl_banks.js",SW_MODAL,par);

	if( !par.escape )
	{
		//this.mfo = Number(par.ret.fmfo);
		this.setFWID(par.ret.fwid);
		//this.nazv = toScrObj(par.ret.fnazv);
		//this.adress = toScrObj(par.ret.faddr);
		//this.comments = toScrObj(par.ret.frem);
		//this.updateCtrl();
	}
	return this.valid();
}