DpCalculations.prototype.getDoc = DpCalculations_getDoc;
DpCalculations.prototype.getPath = DpCalculations_getPath;
DpCalculations.prototype.createStub = DpCalculations_createStub;
DpCalculations.prototype.parse = DpCalculations_parse;
DpCalculations.prototype.listGroups = DpCalculations_listGroups;
DpCalculations.prototype.commitChanges = DpCalculations_commitChanges;
DpCalculations.prototype.parseGroup = DpCalculations_parseGroup;
DpCalculations.prototype.addGroup = DpCalculations_addGroup;
DpCalculations.prototype.deleteGroup = DpCalculations_deleteGroup;
DpCalculations.prototype.getCalculationGroup = DpCalculations_getCalculationGroup;

function DpCalculations(sDocLbl)
{
	this.sClassID = String(sDocLbl).toUpperCase();
	this.xmlDoc = null;
	this.calcGroups = null;
}

function DpCalculations_getCalculationGroup(nNop)
{
	var oSnap = getSnap(" SELECT FCALC FROM ^cl_lnk"
					  + " WHERE FDOC=" + sqlTo(this.sClassID)
					  + " AND FNOP=" + sqlTo(nNop), false);
	if (oSnap != null)
		return oSnap[0];
	return 0;
}

function DpCalculations_deleteGroup(nDelGrp)
{
	this.listGroups();
	var newList = new Array();
	for (var nGrp in this.calcGroups)
		if (nGrp != nDelGrp)
			newList[nGrp] = this.calcGroups[nGrp];
			
	this.calcGroups = newList;
}

function DpCalculations_addGroup(nNum,sComment,sCond)
{
	var group = new DpCalculationGroup();
	group.nNum = nNum;
	group.sComment = sComment;
	group.sCondition = sCond;
	
	this.listGroups(false);
	this.calcGroups[nNum] = group;
}

function DpCalculations_getDoc(bRefresh)
{
	if (this.xmlDoc == null || bRefresh)
	{
		var sFilePath = this.getPath() + "entry.xml";
		if (!isFile(sFilePath))
			this.createStub();
		this.xmlDoc = openXmlFile(sFilePath);
	}
	return this.xmlDoc;
}

function DpCalculations_getPath()
{
	var retPath = "settings/calculations/" + String(this.sClassID).toUpperCase() + "/";
	retPath = retPath.replace(":", "_");
	retPath = getOrgPath()  + retPath; 
	createDirectory(retPath);	
	return retPath;
}

function DpCalculations_createStub()
{
	var sFilePath = this.getPath() + "entry.xml";
	var oDoc = createXmlDoc();
	var root = oDoc.createElement("DpCalculationsFolder");
	oDoc.appendChild(root);
	saveXml(sFilePath,oDoc);
}

function DpCalculations_parse()
{
	var doc = this.getDoc();
	var elCalcGrps = doc.getElementsByTagName("DpCalculationGroup");
	this.calcGroups = new Array();
	
	for (var nGrp = 0; nGrp < elCalcGrps.getLength(); nGrp++)
	{
		var clcGrp = this.parseGroup(elCalcGrps.item(nGrp));
		this.calcGroups[clcGrp.getNum()] = clcGrp;
	}
}

function DpCalculations_listGroups(bRefresh)
{
	if (this.calcGroups == null || bRefresh)
		this.parse();
	return this.calcGroups;
}

function DpCalculations_commitChanges()
{
	var oDoc = createXmlDoc();
	var root = oDoc.createElement("DpCalculationsFolder");
	oDoc.appendChild(root);
	var grpList = this.listGroups(false);
	for (var nGrp in grpList)
	{
		grpList[nGrp].toXml(root);
	}
	var sFilePath = this.getPath() + "entry.xml";
	saveXml(sFilePath,oDoc);
}

function DpCalculations_parseGroup(grpEl)
{
	var group = new DpCalculationGroup();
	group.calculations = new Array();
	
	if (!isNaN(parseInt(grpEl.getAttribute("num"))))
		group.nNum = parseInt(grpEl.getAttribute("num"));

	group.sComment = grpEl.getAttribute("txt");
	if (grpEl.getElementsByTagName("condition").getLength() > 0)
		group.sCondition = grpEl.getElementsByTagName("condition")
											.item(0).getTextContent();
	var clcEls = grpEl.getElementsByTagName("DpCalculation");
	for (var nClc = 0; nClc < clcEls.getLength(); nClc++)
	{
		var clcEl = clcEls.item(nClc);
		var calc = new DpCalculation();
		
		if (!isNaN(parseInt(clcEl.getAttribute("num"))))
			calc.nNum = parseInt(clcEl.getAttribute("num"));
		calc.sComment = clcEl.getAttribute("txt");
		
		if (clcEl.getElementsByTagName("condition").getLength() > 0)
			calc.sCondition = clcEl.getElementsByTagName("condition").item(0).getTextContent();
			
		if (clcEl.getElementsByTagName("variable").getLength() > 0)
			calc.sVAR = clcEl.getElementsByTagName("variable").item(0).getTextContent();
		
		if (clcEl.getElementsByTagName("value").getLength() > 0)
			calc.sVALUE = clcEl.getElementsByTagName("value").item(0).getTextContent();
		
		if (clcEl.getElementsByTagName("proportion").getLength() > 0)
			calc.sPROPORTION = clcEl.getElementsByTagName("proportion").item(0).getTextContent();
		
		if (clcEl.getElementsByTagName("filter").getLength() > 0)
			calc.sFILTER = clcEl.getElementsByTagName("filter").item(0).getTextContent();
			
		if (clcEl.getElementsByTagName("action").getLength() > 0)
			calc.sACT = clcEl.getElementsByTagName("action").item(0).getTextContent();
		
		if (clcEl.getElementsByTagName("sign").getLength() > 0)
			calc.nSIGN = parseInt(clcEl.getElementsByTagName("sign").item(0).getTextContent());
			
		group.calculations[calc.nNum] = calc;
	}
	
	return group;
}
//----------------------------- DpCalculationGroup -------------------------------
DpCalculationGroup.prototype.listCalculations = DpCalculationGroup_listCalculations;
DpCalculationGroup.prototype.getNum = DpCalculationGroup_getNum;
DpCalculationGroup.prototype.toXml = DpCalculationGroup_toXml;
DpCalculationGroup.prototype.deleteCalculation = DpCalculationGroup_deleteCalculation;
DpCalculationGroup.prototype.getCalculationsCount = DpCalculationGroup_getCalculationsCount;
DpCalculationGroup.prototype.listCalculationsOrdered = DpCalculationGroup_listCalculationsOrdered;

function DpCalculationGroup()
{
	this.nNum = 0;
	this.sComment = "";
	this.sCondition = "";
	
	this.calculations = null;
}

function DpCalculationGroup_listCalculationsOrdered()
{
	var orderedList = new Array();
	if (this.calculations != null)
		for (var idCalc in this.calculations)
			orderedList[orderedList.length] = this.calculations[idCalc];
	orderedList.sort
	(
		function (arg1,arg2)
		{
			return arg1.nNum - arg2.nNum;
		}
	);
	return orderedList;
}

function DpCalculationGroup_getCalculationsCount()
{
	var nCount = 0;
	if (this.calculations != null)
		for (var idCalc in this.calculations)
			nCount++;
	return nCount;
}

function DpCalculationGroup_deleteCalculation(nDelCalc)
{
	var newList = new Array();
	for (var nCalc in this.calculations)
		if (nCalc != nDelCalc)
			newList[nCalc] = this.calculations[nCalc];
	this.calculations = newList;
}

function DpCalculationGroup_listCalculations()
{
	return this.calculations;
}

function DpCalculationGroup_getNum()
{
	return this.nNum;
}

function DpCalculationGroup_toXml(elRoot)
{
	var doc = elRoot.getOwnerDocument();
	var grpEl = doc.createElement("DpCalculationGroup");
	
	grpEl.setAttribute("num",String(parseInt(this.nNum)));
	grpEl.setAttribute("txt",String(this.sComment));
	var condEl = doc.createElement("condition");
	condEl.appendChild(doc.createCDATASection(String(this.sCondition)));
	grpEl.appendChild(condEl);
	
	for (var nCalc in this.calculations)
	{
		var oCalc = this.calculations[nCalc];
		var calcEl = doc.createElement("DpCalculation");
		calcEl.setAttribute("num",String(parseInt(oCalc.nNum)));
		calcEl.setAttribute("txt",String(oCalc.sComment));
		
		var cndEl = doc.createElement("condition");
		cndEl.appendChild(doc.createCDATASection(String(oCalc.sCondition)));
		calcEl.appendChild(cndEl);
		
		var varEl = doc.createElement("variable");
		varEl.setTextContent(oCalc.sVAR);
		calcEl.appendChild(varEl);
		
		var actEl = doc.createElement("action");
		actEl.setTextContent(oCalc.sACT);
		calcEl.appendChild(actEl);
		
		var valEl = doc.createElement("value");
		valEl.appendChild(doc.createCDATASection(oCalc.sVALUE));
		calcEl.appendChild(valEl);
		
		var propEl = doc.createElement("proportion");
		propEl.appendChild(doc.createCDATASection(String(oCalc.sPROPORTION)));
		calcEl.appendChild(propEl);
		
		var filterEl = doc.createElement("filter");
		filterEl.appendChild(doc.createCDATASection(String(oCalc.sFILTER)));
		calcEl.appendChild(filterEl);
		
		var signEl = doc.createElement("sign");
		signEl.setTextContent(String(parseInt(oCalc.nSIGN)));
		calcEl.appendChild(signEl);
		
		grpEl.appendChild(calcEl);
	}
	
	elRoot.appendChild(grpEl);
}

//---------------------------- DpCalculation -------------------------------------

function DpCalculation()
{
	this.nNum = 0;
	this.sComment = "";
	this.sCondition = "";
	
	this.sVAR = "";
	this.sVALUE = "";
	this.sPROPORTION = "";
	this.sFILTER = "";
	this.nSIGN = 2;
	this.sACT = "=";
}