include ("Objects/DpXMLDomDocument.js");
var defDesktopIcon = rootDirectoryEx() + "icons/Application.gif";
function DpDesktop()
{
	this.arrItem = new Array(); 
	this.linkCtrl = DpDesktop_linkCtrl; 
	this.fillDesktop = DpDesktop_fillDesktop; 
	this.loadDesktopItems = DpDesktop_loadDesktopItems;
	this.loadDesktopSettings = DpDesktop_loadDesktopSettings;
	this.fillDesktopItems = DpDesktop_fillDesktopItems;
	this.saveDesktopItems = DpDesktop_saveDesktopItems;
	this.createDesktopMenu = DpDesktop_createDesktopMenu;
	this.createItemMenu = DpDesktop_createItemMenu;
	this.execute = DpDesktop_execute;
	this.onItemMove = DpDesktop_onItemMove;
	this.menuHandler = DpDesktop_menuHandler;
	this.refresh = DpDesktop_refresh;
	this.addDesktopItem = DpDesktop_addDesktopItem;
	this.loadDesktopItems();
}

function DpDesktop_linkCtrl(desktopCtrl)
{
	this.desktopCtrl = desktopCtrl;
	this.desktopCtrl.setSelectionHandler(this, "execute");
	this.desktopCtrl.setItemMoveHandler(this, "onItemMove");
	this.fillDesktop();
}

function DpDesktop_fillDesktop()
{
	
	this.loadDesktopSettings();
	this.fillDesktopItems(); 
	this.createDesktopMenu();
	this.createItemMenu();
}

function DpDesktop_refresh()
{
	this.loadDesktopItems();
	this.fillDesktopItems(); 
}

function DpDesktop_loadDesktopItems()
{
	this.arrItem = new Array();
	var desktopDir = userDirectory() + "desktop";
	createDirectory(desktopDir);
	
	var itemsFile = desktopDir + "/desktop.xml";
	if (!isFile(itemsFile))
	{
		itemsFile = rootDirectoryEx() + "Source/desktop_etalon.xml";		
	}
	
	if (!isFile(itemsFile))
		return;
	
	
	var xmlDom = new DpXMLDOMDocument();
	if (!xmlDom.load(itemsFile, false))
		return;
	var doc = xmlDom.getDocumentElement();
	if (doc == null)
		return;
	var items = doc.getElementsByTagName("item");
	
	for (var i = 0; i < items.length; i++)
	{
		var xmlItem = items[i];
		var dItem = new DpDesktopItem();
		dItem.loadFromXML(xmlItem);
		this.arrItem[this.arrItem.length] = dItem;
	}
	
}

function DpDesktop_saveDesktopItems()
{
	var xmlDom = new DpXMLDOMDocument();
	var root = xmlDom.createChild("desktop");
	for (var i = 0; i < this.arrItem.length; i++)
	{
		var dItem = this.arrItem[i];
		
		var attr = new Object();
		attr.type = dItem.type;
		if (dItem.type == "doc")
		{
			attr.doc = dItem.doc;
			attr.nop = dItem.nop;
		}
		else if (dItem.type == "cl")
		{
			attr.cl = dItem.cl;
		}
		else
		{
			attr.command = dItem.command;
			attr.param = dItem.param;
		}
		if (dItem.iconFile)
			attr.icon = dItem.iconFile;
		attr.text = dItem.text;
		attr.rem = dItem.rem;
		if (dItem.xPos && Number(dItem.xPos) > 0)
			attr.xPos = dItem.xPos;
		if (dItem.yPos && Number(dItem.yPos) > 0)
			attr.yPos = dItem.yPos;
		root.createChild("item", null, attr);		
	}
	var desktopDir = userDirectory() + "desktop";
	createDirectory(desktopDir);
	
	var itemsFile = new java.io.File(desktopDir + "/desktop.xml");
	xmlDom.save(itemsFile);
	
}

function DpDesktop_loadDesktopSettings()
{
	desktop.setActive(getPar("USEACTIVDESK"));
	desktop.setWallPaper(lpProxy.getImage(getPar("WALLPAPER")));
	desktop.setAutoArrange(getPar("AUTOARRANGE_DESKTOP"));
}

function DpDesktop_fillDesktopItems()
{
	this.desktopCtrl.removeAll();
	for (var i = 0; i < this.arrItem.length; i++)
	{
		var dItem = this.arrItem[i];
		var item = this.desktopCtrl.addItem(dItem.icon, UR(dItem.text));
		var tooltip = dItem.rem || dItem.text;
		item.setTooltipText(UR(tooltip));
		item.setData(dItem);
		if (dItem.xPos && dItem.xPos > 0)
			item.setXPos(dItem.xPos);
		if (dItem.yPos && dItem.yPos > 0)
			item.setYPos(dItem.yPos);
		
	}

	desktop.refresh();
}

function DpDesktop_execute(item)
{
	if (item == null)
		return;
	dItem = item.getData();
	if (dItem == null)
		return;
	
	dItem.execute();
}

function DpDesktop_onItemMove(item)
{
	if (item == null)
		return;
	dItem = item.getData();
	if (dItem == null)
		return;
	
	dItem.xPos = Number(item.getXPos());
	dItem.yPos = Number(item.getYPos());
	this.saveDesktopItems();
}

function DpDesktop_createDesktopMenu()
{
	var menu  = DpAPI.createMenu("main");
	var menuItem = DpAPI.createMenuItem("CREATEICON");
	menuItem.setText("Створити ярлик|Создать ярлык");
	menu.add(menuItem);
	var menuItem = DpAPI.createMenuItem("PARAM");
	menuItem.setText("Параметри|Параметры");
	menu.add(menuItem);
	this.desktopCtrl.setDesktopMenu(menu, this, "menuHandler");
}

function DpDesktop_createItemMenu()
{
	var menu  = DpAPI.createMenu("main");
	var menuItem = DpAPI.createMenuItem("OPEN");
	menuItem.setText("Відкрити|Открыть");
	menu.add(menuItem);
	var menuItem = DpAPI.createMenuItem("DEL");
	menuItem.setText("Видалити|Удалить");
	menu.add(menuItem);
	var menuItem = DpAPI.createMenuItem("PROPERTY");
	menuItem.setText("Властивості|Свойства");
	menu.add(menuItem);
	this.desktopCtrl.setItemMenu(menu, this, "menuHandler");
}

function DpDesktopItem()
{
	this.type = "";
	this.doc = null;
	this.nop = 0;
	this.cl = 0;	
	this.iconFile = "Application.gif";
	this.icon = lpProxy.getImage(defDesktopIcon);
	this.text = "";
	this.rem = "";
	this.command = "";
	this.param = "";
	this.xPos = -1;
	this.yPos = -1;
	
	this.loadFromXML = DpDesktopItem_loadFromXML;
	this.execute = DpDesktopItem_execute;
	this.edit = DpDesktopItem_edit;
}

function DpDesktopItem_loadFromXML(xmlItem)
{
	this.type = xmlItem.getAttribute("type");
	this.doc = xmlItem.getAttribute("doc");
	this.nop = Number(xmlItem.getAttribute("nop"));
	if (isNaN(this.nop))
		this.nop = 0;
	
	this.cl = Number(xmlItem.getAttribute("cl"));
	if (isNaN(this.cl))
		this.cl = 0;
	
	this.text = xmlItem.getAttribute("text");
	this.rem = xmlItem.getAttribute("rem");
	this.iconFile = xmlItem.getAttribute("icon");
	
	var iconFullPath = "";
	if (!this.iconFile || this.iconFile == "")
	{
		this.iconFile = "Application.gif";
		iconFullPath = defDesktopIcon;
	}
	this.icon = lpProxy.getImage(iconFullPath);
	
	if (this.icon == null)
	{
		iconFullPath = lpProxy.getAppPlugLoc() + "/icons/" + this.iconFile;
		this.icon = lpProxy.getImage(iconFullPath);
	}
	
	if (this.icon == null)
	{
		iconFullPath = rootDirectoryEx() + "/icons/" + this.iconFile;
		this.icon = lpProxy.getImage(iconFullPath);
	}
	
	if (this.icon == null)
	{
		iconFullPath = defDesktopIcon;
		this.icon = lpProxy.getImage(iconFullPath);
	}
	
	this.command = xmlItem.getAttribute("command");
	this.param	= xmlItem.getAttribute("param");
	
	var xPos = Number(xmlItem.getAttribute("xPos"));
	if (!isNaN(xPos))
		this.xPos = xPos;
	
	var yPos = Number(xmlItem.getAttribute("yPos"));
	if (!isNaN(yPos))
		this.yPos = yPos;
	
}

function DpDesktopItem_execute()
{
	switch(String(this.type).toLowerCase())
	{
		case "cl":
			var par = new Object();
			par.Fcl = this.cl;
			//par.Fvid = info.vid;
			showWindow("l_cl.xml",SW_NOMODAL, par,"");
			break;
		case "doc":
			var par = new Object();
			par.doc = this.doc;
			par.nop = this.nop;
			showWindow("l_docekz.xml", SW_NOMODAL, par, "");
			break;
		default:
			lpProxy.getMenuManager().execute(this.type, this.command, this.param);
			break;	
							
	}
}

function DpDesktop_menuHandler(ID, dItem)
{
	switch(String(ID))
	{
	case "OPEN":
		var item = dItem.getData();	
		if (item != null)
			item.execute();
		break;
	case "DEL":
		var item = dItem.getData();
		if (item == null)
			return;
		var str = UR("Ви дійсно бажаєте видалити цей ярлик?|Вы действительно хотите удалить этот ярлык?");
		var ret = confirm(str);
		if (!ret)
			return;
		var arr = new Array();
		for (var i = 0; i < this.arrItem.length; i++)
		{
			var dItem = this.arrItem[i];
			if (item != dItem)
				arr[arr.length] = dItem;
		}
		this.arrItem = arr;
		this.saveDesktopItems();
		this.refresh();
		break;
	case "PROPERTY":
		var item = dItem.getData();	
		if (item == null)
			return;
		var obj = new Object();
		obj.dItem = item;
		showWindow("e_desktopitem.xml", SW_MODAL, obj);
		if (!obj.escape)
		{
			this.saveDesktopItems();
			this.refresh();
		}
		break;
	case "CREATEICON":
		var item = new DpDesktopItem();;	
		var obj = new Object();
		obj.dItem = item;
		showWindow("e_desktopitem.xml", SW_MODAL, obj);
		if (!obj.escape)
		{
			this.arrItem[this.arrItem.length] = item;
			this.saveDesktopItems();
			this.refresh();
		}
		break;
	case "PARAM":
		var par = {sel:"DESKTOP"};
		showWindow("e_param.xml", SW_MODAL, par);
		if (!par.escape)
		{
			this.loadDesktopSettings();
		}
		break;
		
	}
}

function DpDesktopItem_edit()
{
}

function DpDesktop_addDesktopItem(type, command, param, text, image, tooltipText)
{
	var dItem = new DpDesktopItem();
	dItem.type = type;
	if (image)
		dItem.iconFile = image;	
	dItem.text = text;
	dItem.rem = tooltipText || text;
	dItem.command = command;
	dItem.param = param;
	
	this.arrItem[this.arrItem.length] = dItem;
	this.saveDesktopItems();
	lpProxy.sendMessageToEditors("desktop.xml", "REFRESH", null);
}

function addDocToDesktop(docLbl, nop)
{
	if (!nop)
		nop = 0;
	include ("Objects/DpDesktop.js");
	var oDesk = new DpDesktop();
	var menuMode = docLbl;
	if (nop)
		menuMode += ("/" +Number(nop));
	
	var text = allt(tflnk("",docLbl, nop));
	var tooltipText = tflnk("F",docLbl, nop);
	if (!text || text == "")
		text = tooltipText; 
	oDesk.addDesktopItem("l", "l_docekz.xml", menuMode, text, "open.gif", tooltipText);	
}

function addCLToDesktop(nomCL)
{
	var oDesk = new DpDesktop();
	var menuMode = "CL" + nomCL;
	var text = "";
	if (nomCL && nomCL != 0)
		text = tfclcl("", Number(nomCL));
	else
		text = "Повний довідник контрагентів|Полный справочник контрагентов"
	var tooltipText = text;
	oDesk.addDesktopItem("l", "l_cl.xml", menuMode, text, "t44.gif", tooltipText);	
}



