include("doc/DpDoc.js");

DpDocImportExport.prototype.restoreDoc = DpDocImportExport_restoreDoc;
DpDocImportExport.prototype.RunExport = DpDocImportExport_export;
DpDocImportExport.prototype.RunImport = DpDocImportExport_import;
DpDocImportExport.prototype.getSyncInfo = DpDocImportExport_getSyncInfo;
DpDocImportExport.prototype.exportEx = DpDocImportExport_exportEx;
DpDocImportExport.prototype.importEx = DpDocImportExport_importEx;
DpDocImportExport.prototype.getEqualityMap = DpDocImportExport_getEqualityMap;

function DpDocImportExport(bProgress)
{
	if (bProgress == true)
		this.Progress = getProgressMonitor();
}

function DpDocImportExport_getSyncInfo(sDocLbl,nID)
{
	var ret = new SyncInfo();
	ret.sClass = sDocLbl;
	ret.nID = nID;
	var oSnap = getSnap("SELECT FMUSER,FMHOST,FMDATE FROM ^archive "
					  + " WHERE FWID=(SELECT MAX(FWID) FROM ^archive "
					  + " WHERE FDOC=" + sqlTo(sDocLbl)+ " AND FID_DOC=" + sqlTo(nID) + ")");
	if (oSnap != null)
	{
		ret.sUser = oSnap[0];
		ret.sHost = oSnap[1];
		ret.dSync = javaDate2Script(oSnap[2],true);
	}
	return ret;
}

function DpDocImportExport_getEqualityMap(sFromFolder)
{
	var fileName = sFromFolder + "/export.xml";
	var oDoc = openXmlFile(fileName);
	if (oDoc == null)
		return null;
	var retMap = new Array();
	var entries = oDoc.getDocumentElement().getElementsByTagName("entry");
	var dpApi = lpProxy.getDpAPI();

	for (var nEl = 0; nEl < entries.getLength(); nEl++)
	{
		var entry = entries.item(nEl);
		var nID = dpApi.str2Obj(entry.getAttribute("id"),dpApi.TYPE_NUMBER);
		var dRight = javaDate2Script(dpApi.str2Obj(entry.getAttribute("date"),dpApi.TYPE_DATETIME),true);
		var sClass = entry.getAttribute("class");
		var oLeft = this.getSyncInfo(sClass,nID);
		retMap[sClass + "@" + parseInt(nID)] = new CmpInfo(oLeft.dSync,dRight);
	}
	return retMap;
}

function DpDocImportExport_importEx(sFromFolder)
{
	var dir = new DpFile(sFromFolder);
	if (!dir.exists() || !dir.isDirectory())
		return false;
	var eqMap = this.getEqualityMap(sFromFolder);
	var fileList = dir.listFiles();
	var noloadDocs = new Array();
	if (this.Progress)
		this.Progress.beginTask(UR("Імпорт...|Импорт..."),fileList.length);
	for (var nFile = 0; nFile < fileList.length; nFile++)
	{
		if (this.Progress)
			this.Progress.worked(1);
		var curFile = fileList[nFile];
		var fName = String(curFile.getName()).toLowerCase();
		if (fName.substr(fName.length - 4) != ".xml")
			continue;
		if (isNaN(parseInt(fName.substring(0,fName.length - 4))))
			continue;
		var xmlDoc = openXmlFile(curFile.getPath());
		var root = xmlDoc.getDocumentElement();
		var type = root.getAttribute("type");
		var oDoc = new DpDoc(type,0);
		if (!oDoc.jDoc.loadFromXML(curFile.getPath()))
		{
			noloadDocs[noloadDocs.length] = curFile.getPath();
			continue;
		}
		var mapID = oDoc.getVar("HDOC") + "@" + parseInt(oDoc.getID());
		if (eqMap == null
			|| !eqMap[mapID]
			|| eqMap[mapID].areEqual())
		{
			oDoc.jDoc.setClassID(oDoc.getVar("HDOC"));
			oDoc.serverSave();
		}
		else
		{
			var obj1 = oDoc;
			var obj2 = new DpDoc(oDoc.getVar("HDOC"),oDoc.getVar("HOP"));
			if (!obj2.loadByID(oDoc.getID(),true,false))
			{
				oDoc.jDoc.setClassID(oDoc.getVar("HDOC"));
				oDoc.serverSave();
			}
			else
			{
				var oPar = new Object();
				oPar.obj1 = obj2;
				oPar.obj2 = obj1;
				oPar.canWriteLeft = true;
				oPar.canWriteRight = false;
				showWindow("e_object_compare.xml",SW_NOMODAL,oPar);
			}
		}
	}
	if (this.Progress)
		this.Progress.done();
	
	return true;
}



function DpDocImportExport_exportEx(sToFolder,sDocLbl,nIDarr)
{
//-	создаём папочку для экспортa
	var toFldr = new DpFile(sToFolder);
	createDirectory(sToFolder);
	var noloadDocs = new Array();
	var arrSync = new Array();
	
	if (this.Progress)
		this.Progress.beginTask(UR("Експорт...|Экспорт..."),nIDarr.length);
	
	var oDoc = new DpDoc(sDocLbl,0);
	for (var nnId = 0; nnId < nIDarr.length; nnId++)
	{
		if (this.Progress)
			this.Progress.worked(1);
		oDoc.clear();
		if (!oDoc.loadByID(nIDarr[nnId],true,true))
		{
			noloadDocs[noloadDocs.length] = nIDarr[nnId];
			continue;
		}
		arrSync[arrSync.length] = this.getSyncInfo(sDocLbl,nIDarr[nnId]);
		var fileName = String(parseInt(nIDarr[nnId])) + ".xml";
		oDoc.jDoc.storeToXML(toFldr.getAbsolutePath() + "/" + fileName);
	}
	
	var dpApi = lpProxy.getDpAPI();
	var xmlDoc = createXmlDoc();
	var root = xmlDoc.createElement("ExportInfo");
	xmlDoc.appendChild(root);
	for (var nInfo = 0; nInfo < arrSync.length; nInfo++)
	{
		var syncEl = xmlDoc.createElement("entry");
		var oInfo = arrSync[nInfo];
		syncEl.setAttribute("class",oInfo.sClass);
		syncEl.setAttribute("id",dpApi.obj2XmlStr(oInfo.nID,dpApi.TYPE_NUMBER));
		syncEl.setAttribute("user",oInfo.sUser);
		syncEl.setAttribute("host",oInfo.sHost);
		syncEl.setAttribute("date",dpApi.obj2XmlStr(oInfo.dSync.getVarDate(),dpApi.TYPE_DATETIME));
		root.appendChild(syncEl);
	}
	var infoFile = toFldr.getAbsolutePath() + "/export.xml"
	saveXml(infoFile,xmlDoc);
	if (noloadDocs.length > 0)
		alert(UR("Не було експортовано |Не было экспортировано ") + noloadDocs.length + UR("об'єктів|объектов"));
	if (this.Progress)
		this.Progress.done();
}

/*
	тупо копируем файлы xml в указаную папочку
*/
function DpDocImportExport_export
		(
			sToFolder,
			sDocLbl,
			nIDarr
		)
{
	var oDoc = new DpDoc(sDocLbl,0);
	var toFldr = new DpFile(sToFolder);
	createDirectory(sToFolder);
	var sPath = getOrgPath() + "data/" + oDoc.jDoc.getDataDir();
	var dir = new DpFile(sPath);
	var fileList = dir.listFiles();
	if (this.Progress)
		this.Progress.beginTask(UR("Експорт...|Экспорт..."),fileList.length);
	var nidMap = null;
	if (nIDarr)
	{
		nidMap = new Array();
		for (var n = 0; n < nIDarr.length; n++)
		{
			nidMap[nIDarr[n]] = true;
		}
	}
	var arrSync = new Array();
	for (var nFile = 0; nFile < fileList.length; nFile++)
	{
		if (this.Progress)
			this.Progress.worked(1);
		var curFile = fileList[nFile];
		var fName = String(curFile.getName()).toLowerCase();
		if (fName.indexOf("docpar") >= 0 || fName.substr(fName.length - 4) != ".xml")
			continue;
		if (nidMap != null)
		{
			if (fName.lastIndexOf(".") < 0)
				continue;
			var curFnid = parseInt(fName.substring(0,fName.lastIndexOf(".")));
			if (isNaN(curFnid))
				continue;
			if (!nidMap[curFnid])
				continue;
			arrSync[arrSync.length] = this.getSyncInfo(sDocLbl,curFnid);
		}
		copyFile(sPath + fName,sToFolder + "/" + fName);
	}
	var dpApi = lpProxy.getDpAPI();
	var xmlDoc = createXmlDoc();
	var root = xmlDoc.createElement("ExportInfo");
	xmlDoc.appendChild(root);
	for (var nInfo = 0; nInfo < arrSync.length; nInfo++)
	{
		var syncEl = xmlDoc.createElement("entry");
		var oInfo = arrSync[nInfo];
		syncEl.setAttribute("class",oInfo.sClass);
		syncEl.setAttribute("id",dpApi.obj2XmlStr(oInfo.nID,dpApi.TYPE_NUMBER));
		syncEl.setAttribute("user",oInfo.sUser);
		syncEl.setAttribute("host",oInfo.sHost);
		syncEl.setAttribute("date",dpApi.obj2XmlStr(oInfo.dSync.getVarDate(),dpApi.TYPE_DATETIME));
		root.appendChild(syncEl);
	}
	var infoFile = toFldr.getAbsolutePath() + "/export.xml"
	saveXml(infoFile,xmlDoc);

	if (this.Progress)
		this.Progress.done();
}

function DpDocImportExport_import(sFromFolder)
{
	var dir = new DpFile(sFromFolder);
	if (!dir.exists() || !dir.isDirectory())
		return false;
	var fileList = dir.listFiles();
	if (this.Progress)
		this.Progress.beginTask(UR("Імпорт...|Импорт..."),fileList.length);
	for (var nFile = 0; nFile < fileList.length; nFile++)
	{
		if (this.Progress)
			this.Progress.worked(1);
		var curFile = fileList[nFile];
		var fName = String(curFile.getName()).toLowerCase();
		if (fName.substr(fName.length - 4) != ".xml")
			continue;
		if (isNaN(parseInt(fName.substring(0,fName.length - 4))))
			continue;
		var oDoc = openXmlFile(curFile.getPath());
		var root = oDoc.getDocumentElement();
		var type = root.getAttribute("type");
		var oDoc = new DpDoc(type,0);
		if (!oDoc.jDoc.loadFromXML(curFile.getPath()))
		{
			continue;
		}
		oDoc.jDoc.setClassID(oDoc.getVar("HDOC"));
		oDoc.serverSave();
	}
	if (this.Progress)
		this.Progress.done();
}

function DpDocImportExport_restoreDoc(sDocLbl,nID)
{
	return true;
	include("objects/DpDocumentDecoder.js");
	var oDec = new DpDocumentDecoder();
	var oPar = new Object();
	oPar.doc = sDocLbl;
	oPar.fiddoc = Number(nID);
	oPar.notRegister = true;
	oDec.run(sDocLbl,oPar);
}

function CmpInfo(dLeft,dRight)
{
	this.dateLeft = dLeft;
	this.dateRight = dRight;
	
	this.areEqual = function()
					{
						if (this.dateLeft == null
							|| this.dateLeft == null)
							return true;
							
						return differ(this.dateLeft,this.dateRight) == 0;
					}
}

function SyncInfo()
{
	this.sClass = "";
	this.nID = -1;
	this.dSync = new Date();
	this.sUser = getUserName();
	this.sHost = getHostName();
}