/*
	Объект-wrapper над классом sun.net.ftp.FtpClient,
	реализующий методы для работы с файлами по FTP протоколу

	© 2010 Черкасское представительство ДЕБЕТ Плюс

	Пример использования:
		var ftp = new DpFTP();
		
		ftp.setConnectTimeout(500);
		ftp.setReadTimeout(5000);
		ftp.connect("192.168.0.51", 21, "anonymous", "1");
		//ftp.login("anonymous", "1");

		ftp.changeDirUp();
		ftp.changeDir("SQL");

		ftp.setModeBinary();

		ftp.downloadFile("A", "test.tst", "C:/test.tst");
		ftp.uploadFile("A", "test.tst", "C:/DebetV12/DebetPlusV12_OBL/test.tst");

		showObject(ftp.getDirList());

		ftp.release();
		ftp = null;

	Команды FTP и коды ответов сервера находятся в конце файла.


	Предупреждения:
		1) UNIX-овые FTP-сервера чувствительны к регистру в имени файла!
		2) есть подозрение, что fileExists для каталогов не работает (на UNIX).
*/

function DpFTP()
{
	RO("DpFTP", this);

	// Свойства
	this.ftp = getJavaClass("sun.net.ftp.FtpClient").newInstance();
	this.FTP_DEFAULT_PORT = 21;
	this.FTP_DEFAULT_BUF_SIZE = 4096;

	// Методы
	this.release			= DpFTP_release;
	this.getFTPobj			= DpFTP_getFTPobj;

	this.connect			= DpFTP_connect;
	this.disconnect			= DpFTP_disconnect;

	this.login				= DpFTP_login;
	
	this.errorMsg			= "";
	this.getLastErrorMsg 	= DpFTP_getLastErrorMsg;
	
	this.makeDir			= DpFTP_makeDir;
	this.removeDir			= DpFTP_removeDir;
	this.renameDir			= DpFTP_renameDir;
	this.changeDir			= DpFTP_changeDir;
	this.changeDirUp		= DpFTP_changeDirUp;
	this.changeDirRoot		= DpFTP_changeDirRoot;
	this.getWorkingDir		= DpFTP_getWorkingDir;
	this.getDirList			= DpFTP_getDirList;
	this.fileExists			= DpFTP_fileExists;
	this.getFileSize		= DpFTP_getFileSize;
	this.renameFile			= DpFTP_renameFile;
	this.deleteFile			= DpFTP_deleteFile;

	this.downloadFile		= DpFTP_downloadFile;
	this.uploadFile			= DpFTP_uploadFile;

	this.noOperation		= DpFTP_noOperation;
	this.sendCommand		= DpFTP_sendCommand;
	this.getResponseString	= DpFTP_getResponseString;
	this.readServerResponse	= DpFTP_readServerResponse;

	this.setModeBinary		= DpFTP_setModeBinary;
	this.setModeASCII		= DpFTP_setModeASCII;

	this.getConnectTimeout	= DpFTP_getConnectTimeout;
	this.setConnectTimeout	= DpFTP_setConnectTimeout;

	this.getReadTimeout		= DpFTP_getReadTimeout;
	this.setReadTimeout		= DpFTP_setReadTimeout;
}

// закрытие подключений и освобождение памяти
function DpFTP_release()
{
	this.disconnect();
	this.ftp = null;
}

// возвращает JAVA-объект ftp
function DpFTP_getFTPobj()
{
	return this.ftp;
}

// создает подключение к серверу на указанный порт (по умолчанию — 21)
// если указаны учетные данные, также проводит аутентификацию
// @mFserver: адрес сервера
// @mFpodr: порт
// @mFlogin: логин
// @mFpass: пароль
function DpFTP_connect(mFserver, mFport, mFlogin, mFpass)
{
	if (typeof(mFport) == "undefined")
	{
		mFport = FTP_DEFAULT_PORT;
	}
	this.ftp.openServer(mFserver, mFport);

	if (typeof(mFlogin) != "undefined" &&
		typeof(mFpass) != "undefined")
	{
		this.login(mFlogin, mFpass);
	}
}

// разрывает активное подключение
function DpFTP_disconnect()
{
	this.ftp.closeServer();
}

// проводит аутентификацию на сервере с указанными учетными данными
// @mFlogin: логин
// @mFpass: пароль
function DpFTP_login(mFlogin, mFpass)
{
	this.ftp.login(mFlogin, mFpass);
	this.setModeBinary();		// по умолчанию будет двоичный режим
}

// отправляет серверу "холостую" команду "NOOP" — нет действия
// используется для поддержания подключения
function DpFTP_noOperation()
{
	this.ftp.noop();
}

// отправляет указанную команду серверу в таком виде, в котором она передана
// @mFcmd: строка, например:
//		"MKD dir"		— создать в текущей директории каталог dir
//		"DELE test.txt"	— удалить из текущей директории файл test.txt
// см. внизу перечень команд
function DpFTP_sendCommand(mFcmd)
{
	this.ftp.sendServer(mFcmd + "\r\n");
}

// возвращает текущий статус подключения в текстовом формате, например
// "220 Golden FTP Server ready v4.50"
// "230 User logged in, proceed."
// !!! этот статус устанавливается предыдущей командой или действием !!!
function DpFTP_getResponseString()
{
	return this.ftp.getResponseString();
}

// возвращает числовой код статуса подключения
// см. внизу перечень кодов статусов
function DpFTP_readServerResponse()
{
	return this.ftp.readServerResponse();
}

// устанавливает режим передачи данных в двоичный
function DpFTP_setModeBinary()
{
	this.ftp.binary();
}

// устанавливает режим передачи данных в текстовый
function DpFTP_setModeASCII()
{
	this.ftp.ascii();
}

// создает каталог в текущей директории
// @returns: код статуса в текстовом формате (см. внизу)
function DpFTP_makeDir(mFdir)
{
	this.ftp.sendServer("MKD " + mFdir + "\r\n");
	this.ftp.readServerResponse();
	return this.ftp.getResponseString();
}

// удаляет каталог в текущей директории
// @returns: код статуса в текстовом формате (см. внизу)
function DpFTP_removeDir(mFdir)
{
	this.ftp.sendServer("RMD " + mFdir + "\r\n");
	this.ftp.readServerResponse();
	return this.ftp.getResponseString();
}

// переименовывает каталог в текущей директории (использует процедуру переименования файла, т.к. для FTP файл и каталог это одно и то же)
// @mFold: имя каталога, который надо переименовать
// @mFnew: новое имя каталога
// @returns: код статуса в текстовом виде (см. внизу)
function DpFTP_renameDir(mFold, mFnew)
{
	this.renameFile(mFold, mFnew);
}

// переходит в указанный каталог. допускается изменение каталога через несколько уровней,
// например, changeDir("pub/cat1/data")
// @mFdir: имя каталога
function DpFTP_changeDir(mFdir)
{
	this.ftp.cd(mFdir);
}

// переходит в родительский каталог
function DpFTP_changeDirUp()
{
	this.ftp.cdUp();
}

// переходит в корневой каталог сервера
function DpFTP_changeDirRoot()
{
	this.changeDir("/");
}

// возвращает текущую рабочую директорию
function DpFTP_getWorkingDir()
{
	this.ftp.pwd();
}

// возвращает перечень каталогов и файлов в рабочей папке
// @returns массив строк
function DpFTP_getDirList()
{
	var inStream = new java.io.BufferedReader(new java.io.InputStreamReader(this.ftp.nameList("")));
	var aValues = new Array();
	var str = null;

	while ((str = inStream.readLine()) != null)
	{
		aValues[aValues.length] = String(str);
	}
	inStream.close();

	return aValues;
}

// возвращает размер файла в текущей директории
// @mFfileName: имя файла
// @returns: размер в байтах
function DpFTP_getFileSize(mFfileName)
{
	var OKcode = "213";
	this.ftp.sendServer("SIZE " + mFfileName + "\r\n");
	this.ftp.readServerResponse();
	var response = String(this.ftp.getResponseString());
	if (response.startsWith(OKcode))
	{
		return Number(response.substr(OKcode.length).trim());
	}
	else
	{
		return -1;
	}
}

// возвращает true/false, существует ли указанный файл/каталог
// в текущей директории
// @mFfileName: имя файла
function DpFTP_fileExists(mFfileName)
{
	return this.getFileSize(mFfileName) != -1;
}

// переименовывает файл в текущей директории
// @mFold: имя файла, который надо переименовать
// @mFnew: новое имя файла
// @returns: код статуса в текстовом виде (см. внизу)
function DpFTP_renameFile(mFold, mFnew)
{
	this.sendCommand("RNFR " + mFold);
	this.sendCommand("RNTO " + mFnew);
	this.ftp.readServerResponse();
	return this.ftp.getResponseString();
}

// удаляет файл из текущей директории
// @mFfilename: имя файла
// @returns: код статуса в текстовом виде (см. внизу)
function DpFTP_deleteFile(mFfilename)
{
	this.ftp.sendServer("DELE " + mFfilename + "\r\n");
	this.ftp.readServerResponse();
	return this.ftp.getResponseString();
}

// загружает файл с сервера в указанную папку
// @mode: режим "R" — перезаписать файл, "A" — дозаписать файл
// @mFname: имя файла на сервере в рабочей папке
// @mFlocalPath: локальный путь к файлу (включая имя файла, причем имя файла может не совпадать с тем, что на сервере)
//		путь должен существовать, файл — необязательно. он будет перезаписан
function DpFTP_downloadFile(mode, mFname, mFlocalPath)
{
	if (typeof(mode) == "undefined")
	{
		mode = "R";
	}
	if (!inList(mode.toUpperCase(), "R", "A"))
	{
		return false;
	}

	var buf = new java.lang.reflect.Array.newInstance(java.lang.Byte.TYPE, this.FTP_DEFAULT_BUF_SIZE);
	var outStream = new java.io.FileOutputStream(mFlocalPath, (mode == "A"));
	var inStream = this.ftp.get(mFname);
	var counter = 0;
	while (true)
	{
		var bytes = inStream.read(buf);
		if (bytes < 0)
		{
			break;
		}

		outStream.write(buf, 0, bytes);
		counter += bytes;
	}
	outStream.close();
	inStream.close();

	inStream = null;
	outStream = null;
}

// загружает указанный файл на сервер в рабочую папку
// @mode: режим "R" – перезаписать файл, "A" — дозаписать файл
// @mFname: имя файла на сервере
// @mFlocalPath: локальный путь к файлу (включая имя файла)
function DpFTP_uploadFile(mode, mFname, mFlocalPath, bNoMsg)
{
	try
	{
		if (typeof(mode) == "undefined")
		{
			mode = "R";
		}
		if (!inList(mode.toUpperCase(), "R", "A"))
		{
			return false;
		}
	
		var buf = new java.lang.reflect.Array.newInstance(java.lang.Byte.TYPE, this.FTP_DEFAULT_BUF_SIZE);
		var inStream = new java.io.FileInputStream(mFlocalPath);
		var outStream = (mode == "R")
			? this.ftp.put(mFname)
			: this.ftp.append(mFname);
		var counter = 0;
		while (true)
		{
			var bytes = inStream.read(buf);
			if (bytes < 0)
			{
				break;
			}
	
			outStream.write(buf, 0, bytes);
			counter += bytes;
		}
		outStream.close();
		inStream.close();
	
		inStream = null;
		outStream = null;
		return true;
	}
	catch(ex)
	{
		if(!bNoMsg)
			alert(ex);
		this.errorMsg = ex;
		return false;
	}
}

// возвращает тайм-аут по подключению к серверу
// @returns: время в мс
function DpFTP_getConnectTimeout()
{
	return this.ftp.getConnectTimeout();
}

// устанавливает тайм-аут по подключению к серверу
// @mFtimeout: время в мс
function DpFTP_setConnectTimeout(mFtimeout)
{
	this.ftp.setConnectTimeout(mFtimeout);
}

// возвращает тайм-аут по чтению данных
// @returns: время в мс
function DpFTP_getReadTimeout()
{
	return this.ftp.getReadTimeout();
}

// устанавливает тайм-аут по чтению данных
// @mFtimeout: время в мс
function DpFTP_setReadTimeout(mFtimeout)
{
	this.ftp.setReadTimeout(mFtimeout);
}

function DpFTP_getLastErrorMsg()
{
	return this.errorMsg;
}


/*
Команды FTP

(Warning: this is a technical document, not necessary for most FTP use.)
Note that commands marked with a * are not implemented in a number of FTP servers.

Common commands
	ABOR	— abort a file transfer
	CWD		— change working directory
	DELE	— delete a remote file
	LIST	— list remote files
	MDTM	— return the modification time of a file
	MKD		— make a remote directory
	NLST	— name list of remote directory
	PASS	— send password
	PASV	— enter passive mode
	PORT	— open a data port
	PWD		— print working directory
	QUIT	— terminate the connection
	RETR	— retrieve a remote file
	RMD		— remove a remote directory
	RNFR	— rename from
	RNTO	— rename to
	SITE	— site-specific commands
	SIZE	— return the size of a file
	STOR	— store a file on the remote host
	TYPE	— set transfer type
	USER	— send username

Less common commands
	ACCT*	— send account information
	APPE	— append to a remote file
	CDUP	— CWD to the parent of the current directory
	HELP	— return help on using the server
	MODE	— set transfer mode
	NOOP	— do nothing
	REIN*	— reinitialize the connection
	STAT	— return server status
	STOU	— store a file uniquely
	STRU	— set file transfer structure
	SYST	— return system type
*/

/*
Коды статусов

Code		Description
100 Codes	The requested action is being taken. Expect a reply before proceeding with a new command.
110			Restart marker reply.
120			Service ready in (n) minutes.
125			Data connection already open, transfer starting.
150			File status okay, about to open data connection.
200 Codes	The requested action has been successfully completed.
200			Command okay.
202			Command not implemented
211			System status, or system help reply.
212			Directory status.
213			File status.
214			Help message.
215			NAME system type. (NAME is an official system name from the list in the Assigned Numbers document.)
220			Service ready for new user.
221			Service closing control connection. (Logged out if appropriate.)
225			Data connection open, no transfer in progress.
226			Closing data connection. Requested file action successful (file transfer, abort, etc.).
227			Entering Passive Mode
230			User logged in, proceed.
250			Requested file action okay, completed.
257			"PATHNAME" created.
300 Codes	The command has been accepted, but the requested action is being held pending receipt of further information.
331			User name okay, need password.
332			Need account for login.
350			Requested file action pending further information.
400 Codes	The command was not accepted and the requested action did not take place. The error condition is temporary, however, and the action may be requested again.
421			Service not available, closing control connection. (May be a reply to any command if the service knows it must shut down.)`
425			Can't open data connection.
426			Connection closed, transfer aborted.
450			Requested file action not taken. File unavailable (e.g., file busy).
451			Requested action aborted, local error in processing.
452			Requested action not taken. Insufficient storage space in system.
500	Codes	The command was not accepted and the requested action did not take place.
500			Syntax error, command unrecognized. This may include errors such as command line too long.
501			Syntax error in parameters or arguments.
502			Command not implemented.
503			Bad sequence of commands.
504			Command not implemented for that parameter.
530			User not logged in.
532			Need account for storing files.
550			Requested action not taken. File unavailable (e.g., file not found, no access).
552			Requested file action aborted, storage allocation exceeded
553			Requested action not taken. Illegal file name.
*/