/*
   Описание объекта "элемент справочника итогов": DpItogs
   (C) ДЕБЕТ Плюс, 1998-1999

   Конструктор DpItogs.
      Возвращает this

  Mетоды:
   .setTask(lst)      - Установить задачу из которой вызывается расчёт итога
   .setCod(new_cod)   - Установить номер итога
   .zSetCod(new_cod)  - Установить номер итога используя завал
   .getCod()          - Получить номер итога
   .getTxt()          - Получить наименование итога
   .valid()           - Проверка, существует ли запись по заданному номеру итога
   .linkCtrl()        - Связать с полем ввода
   .validCtrl()       - Реакция на выход из связанного поля ввода
   .zaval()           - Осуществляется завал при нажатии кнопки.
   .canNoItog()       - Устанавливает возможность "Без итогов"
   .noItog()          - Проверка, находимся ли мы в режиме "Без итогов"
   .updateCtrl()      - Обновление поля ввода.
   .empty()           - Обнуление номера и наименования итога
   .fillUpFromTbl()   - Заполнение значениями из таблицы номера и наименования итога 
   
   
  Cвойства:
   .cod            - номер итога
   .txt            - наименование итога
   .lst            - задача вызова
   .table          - таблица для внутреннего использования в объекте.
   .ctrl           - связь с полем ввода. 
   .isButton       - кнопка для завала.
   .no_itog        - возможно ли "Без итогов"   
*/

// Конструктор объекта DpItogs
function DpItogs(lst)
{
	RO("DpItogs", this);
   this.setTask = DpItogs_setTask;
   this.canNoItog = DpItogs_canNoItog;
   this.noItog = DpItogs_noItog;
   this.setCod = DpItogs_setCod;
   this.zSetCod = DpItogs_zSetCod;
   this.getCod = DpItogs_getCod;
   this.getTxt = DpItogs_getTxt;
   this.valid = DpItogs_valid;
   this.linkCtrl = DpItogs_linkCtrl;
   this.validCtrl = DpItogs_validCtrl;
   this.zaval = DpItogs_zaval;
   this.updateCtrl = DpItogs_updateCtrl;
   this.empty = DpItogs_empty;
   this.fillUpFromTbl = DpItogs_FillUpFromTbl;
   
   this.no_itog=false;
   if (lst)
      this.lst = lst.toUpperCase();
}

//Установить задачу(lst-задача, для которой созданы итоги)
function DpItogs_setTask(lst)
{ 
   if(!this.lst) 
   {
      this.lst=lst.toUpperCase();
      return true;
   }  
   else
      return false; 
}

//Установть возможность "Без итогов"
function DpItogs_canNoItog()
{ 
   this.no_itog=true; 
}

//Находимся ли в режиме "Без итогов"
function DpItogs_noItog()
{   
   return (this.no_itog && Number(this.cod)==0); 
}

//Установить номер итога (new_cod-порядковый номер итога, lst-задача, для которой созданы итоги, например vdost)
function DpItogs_setCod(new_cod)
{ 
   this.empty();
   this.cod = new_cod;
   this.updateCtrl();
}

//Установить номер итога используя завал (new_cod-порядковый номер итога, lst-задача, для которой созданы итоги, например vdost)
function DpItogs_zSetCod(new_cod)
{ 
   var retValue = true;
   this.empty();
   this.table = OpenTable("s","SELECT * FROM ^cl_tt "
   			+" WHERE UPPER(flist) = "+ sqlTo(String(this.lst).toUpperCase()) 
   				+ " AND fno = " + new_cod);   
   
   if (this.no_itog && !new_cod)
      this.cod=new_cod;
   else if (!this.table.isEmpty() || this.no_itog)
      this.fillUpFromTbl();
   else
   {
      var par = new Object();
      par.fcod = new_cod;
      par.callerName = this.lst;
      par.zaval = true;
      par.escape = true;
      par.ret = new Object();
      par.ret.fno = 0;
      showWindow("l_itogs.js",SW_MODAL,par);
      if (!par.escape)
         this.cod = par.ret.fno;
      else
         retValue=false;  
   }       
   this.updateCtrl(); 
   return retValue;
}

// Получить номер итога
function DpItogs_getCod()
{  
	if (!this.cod)
	{
		if (this.ctrl)
		{
			this.setCod(this.ctrl.value);
		}
	}
   return this.cod;
}

// Получить наименование итога
function DpItogs_getTxt()
{
   if (this.valid())
      this.fillUpFromTbl();
   else if (this.noItog())
      this.txt="Без итогов";
   else
      this.txt="???";    
   return this.txt;
}

// Проверка, существует ли запись по заданному номеру итога
function DpItogs_valid()
{
   this.table = OpenTable("s","SELECT * FROM ^cl_tt "
   			+" WHERE UPPER(flist) = "+ sqlTo(String(this.lst).toUpperCase()) 
   				+ " AND fno = " + this.getCod());  
   return !this.table.isEmpty();
}

//Обнуление
function DpItogs_empty()
{
   this.cod = null;
   this.txt = null;
}

//Заполнение значениями из таблицы
function DpItogs_FillUpFromTbl()
{
   this.cod = Number(this.table.getValue("Fno"));
   this.txt = String(this.table.getValue("Ftxt"));
}

// Связать с полем ввода
function DpItogs_linkCtrl(ctrl,ctrl_text)
{
   if (ctrl!=null)
   {
      if(typeof(ctrl)!="object")
      {
         alert("Передайте в DpItogs для связки control");
         return;
      }
      this.ctrl = ctrl; 
      this.ctrl.mask = "##"; 
      setValid(this.ctrl, this, "validCtrl");
      if(JAVA || this.ctrl.DpType=="DpGet")
      {
         ctrl.isButton = true;
         ctrl.buttonHandler(this, "zaval");
      }
   }
       
   if(ctrl_text!=null)
   {
      this.ctrl_text=ctrl_text;
      this.ctrl_text.mask = "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&"; 
      Disable(ctrl_text);
   }
	
   this.updateCtrl();

}

function DpItogs_validCtrl()
{
   if(typeof(this.ctrl)=="object")
   {
      this.zSetCod(this.ctrl.value);
   }
   return (this.valid() || this.noItog());
}

function DpItogs_zaval()
{
   var par = new Object();
   par.fcod = this.ctrl.value;
   par.callerName = this.lst;
   par.zaval = true;
   par.escape = true;
   par.ret = new Object();
   par.ret.fno = "?";
   showWindow("l_itogs.js",SW_MODAL,par);
   if(SQL)
      this.table.Requery();
   if (!par.escape)
      this.cod = par.ret.fno;
   this.updateCtrl(); 
   return (!par.escape);
}

function DpItogs_updateCtrl()
{ 
   if(this.ctrl!= null)
   {
      this.ctrl.value = this.getCod();
   }
   if(this.ctrl_text!= null)
   {
      this.ctrl_text.value = this.getTxt();
   }   
}