include("objects/DpJournal.js");
include("objects/DpJRegistrator.js");

DpJDecoder.prototype.makeJournals = DpJDecoder_makeJournals;
DpJDecoder.prototype.registerDocs = DpJDecoder_registerDocs;
DpJDecoder.prototype.fillRegVars = DpJDecoder_fillRegVars;
DpJDecoder.prototype.run = DpJDecoder_run;
DpJDecoder.prototype.getQry = DpJDecoder_getQry;
DpJDecoder.prototype.divideDocreg = DpJDecoder_divideDocreg;

function DpJDecoder()
{
	
}

function DpJDecoder_run()
{
	this.makeJournals();
	this.registerDocs();
	this.divideDocreg();
}

function DpJDecoder_makeJournals()
{
	var sSQL = "SELECT * FROM ^cl_reg WHERE " + sqlStrIsNull("FDOC") + " OR FDOC IS NULL OR FDOC=''";
	var tbl = OpenTable("s",sSQL);
	setProgressRangeEx(1,tbl.getRecordCount() + 1);
	setProgressPosEx(1,"Создание журналов...");
	for (tbl.MoveFirst(); !tbl.isEOF(); tbl.MoveNext())
	{
		setProgressPosEx(tbl.getAbsolute(),"Создание журналов...");
		var jType = "S";
		if (tbl.getValue("FVID") == "+")
			jType += "P";
		else if (tbl.getValue("FVID") == "-")
			jType += "M";
		else
			continue;
			
		var jID = "S" + tbl.getValue("FREG");
		var jRem = tbl.getValue("FTXT");
		
		var oJrnl = new DpJournal(jType,jID);
		oJrnl.setName(jRem);
		oJrnl.setRem(jRem);
		oJrnl.commitStructure(true);
	}
	setProgressPosEx(0);
	tbl.Close();
}

function DpJDecoder_registerDocs()
{
	var sSQL = "SELECT * FROM ^cl_reg WHERE FNOM=0 AND " + sqlStrIsNotEmp("FDOC");
	var tbl = OpenTable("s",sSQL);
	setProgressRangeEx(1,tbl.getRecordCount() + 1);
	setProgressPosEx(1,"Регистрация документов...");
	for (tbl.MoveFirst(); !tbl.isEOF(); tbl.MoveNext())
	{
		setProgressPosEx(tbl.getAbsolute(),"Регистрация документов...");
		var doc = tbl.getValue("FDOC");
		var nop = tbl.getValue("FNOP");
		var jID = "S" + tbl.getValue("FREG");
		
		var oReg = new DpJRegistrator(doc,nop,"S");
		oReg.regTo(jID,"");
		
		this.fillRegVars(oReg,tbl.getValue("FREG"));
		
		oReg.saveReg();
	}
	setProgressPosEx(0);
	tbl.Close();
}

function DpJDecoder_fillRegVars(oReg,freg)
{
	oReg.listRegJournals(true);
	oReg.listJournals(true);
	
	var sSQL = "SELECT * FROM ^cl_reg"
			 + " WHERE FDOC=" + sqlTo(oReg.doc)
			 + " AND FNOP=" + sqlTo(oReg.nop)
			 + " AND FNOM <> 0"
			 + " AND FREG=" + sqlTo(freg);
	var tbl = OpenTable("s",sSQL);
	
	for (tbl.MoveFirst(); !tbl.isEOF(); tbl.MoveNext())
	{
		var varID = tbl.getValue("FVAR");
		var sVal = tbl.getValue("FVALUE");
		var datStart = tbl.getValue("FBDAT");
		var datEnd = tbl.getValue("FEDAT");
		var jID = "S" + tbl.getValue("FREG");
		
		var isOk = oReg.setRegVar(jID,varID,sVal,datStart,datEnd);
		if (!isOk)
			trace(jID + "-" + varID+"-" +sVal+"-" +datStart+"-" +datEnd + " = " + isOk);
	}
	tbl.Close();
}

function DpJDecoder_getQry(sType,Freg,jID)
{
	var drFlds =  new Array("FWID","FDOC_LEV","FDT_OTSE","FIDCONTR","FIDDEAL","FISNOV1","FNROWS",sqlTo(jID),"FSORT");
	var regFlds = new Array("FWID","HDOC_LEV","HDT_OTSE","HIDCONTR","HIDDEAL","HISNOV1","HNROWS","HREG","HSORT");
	
	var objMinus = {FDOC:'HDOC', FNOP:'HNOP', FDOC_DAT:'HDAT', FDOC_NUM:'HNOM', FID_DOC:'HID_DOC'
		, FORG:'HORG', FVAL:'HV', FNK_NUM:'HNKNOM', FNK_DAT:'HNKDAT', FPR_COND:'HPRCOND', FOP_FORM:'HOPFORM'
		,FOP_NDAT:'HOPNDAT', FDT_OTS:'HDTOTS', FGROUP:'HGROUP', FMNG:'HMENEGER', FREM:'HREM'
		, FSC:'HSC', FSU:'HSU', FST:'HST', FSTPO_20:'HTPO20', FSDPO_20:'HDPO20', FSTVO_20:'HTVO20'
		, FSDVO_20:'HDVO20', FSTPO_NDS:'HTPONDS', FSDPO_NDS:'HDPONDS', FSTVO_NDS:'HTVONDS', FSDVO_NDS:'HDVONDS'
		, FSTPO_00:'HTPO00', FSDPO_00:'HDPO00', FSTVO_00:'HTVO00', FSDVO_00:'HDVO00', FSTPO_A0:'HTPOA0', FSDPO_A0:'HDPOA0'
		, FSTVO_A0:'HTVOA0', FSDVO_A0:'HDVOA0', FSTPO_B0:'HTPOB0', FSDPO_B0:'HDPOB0', FSTVO_B0:'HTVOB0', FSDVO_B0:'HDVOB0'
		, FSTPO_E0:'HTPOE0', FSDPO_E0:'HDPOE0', FSTVO_E0:'HTVOE0', FSDVO_E0:'HDVOE0', FSTPO_XX:'HTPOXX', FSDPO_XX:'HDPOXX'
		, FSTVO_XX:'HTVOXX', FSDVO_XX:'HDVOXX', FSTSO_20:'HTSO20', FSTSO_NDS:'HTSONDS', FSTSO_00:'HTSO00', FSTSO_A0:'HTSOA0'
		, FSTSO_B0:'HTSOB0', FSTSO_E0:'HTSOE0', FSTSO_XX:'HTSOXX', FSTPU:'HSTPU', FSDPU:'HSDPU', FSTVU:'HSTVU', FSDVU:'HSDVU'};
	
	var objPlus = {FDOC:'HDOC', FNOP:'HNOP', FDOC_DAT:'HDAT', FDOC_NUM:'HNOM', FID_DOC:'HID_DOC'
		, FORG:'HORG', FVAL:'HV', FNK_NUM:'HNKNOM', FNK_DAT:'HNKDAT', FPR_COND:'HPRCOND', FOP_FORM:'HOPFORM'
		, FOP_NDAT:'HOPNDAT', FDT_OTS:'HDTOTS', FGROUP:'HGROUP', FMNG:'HMENEGER', FREM:'HREM'
		, FSC:'HSC', FSU:'HSU', FST:'HST', FSTPO_20:'HTP20', FSDPO_20:'HDP20', FSTVO_20:'HTV20'
		, FSDVO_20:'HDV20', FSTPO_NDS:'HTPNDS', FSDPO_NDS:'HDPNDS', FSTVO_NDS:'HTVNDS', FSDVO_NDS:'HDVNDS'
		, FSTPO_PV:'HTPPV', FSDPO_PV:'HDPPV', FSTVO_PV:'HTVPV', FSDVO_PV:'HDVPV', FSTPO_IVS:'HTPIV', FSDPO_IVS:'HDPIV'
		, FSTVO_IVS:'HTVIV', FSDVO_IVS:'HDVIV', FSTPO_IVN:'HTPIVNDS', FSDPO_IVN:'HDPIVNDS', FSTVO_IVN:'HTVIVNDS'
		, FSDVO_IVN:'HDVIVNDS', FSTPO_NR:'HTPIMNR', FSDPO_NR:'HDPIMNR', FSTVO_NR:'HTVIMNR', FSDVO_NR:'HDVIMNR'
		, FSTPO_NNR:'HTPNDSNR', FSDPO_NNR:'HDPNDSNR', FSTVO_NNR:'HTVNDSNR', FSDVO_NNR:'HDVNDSNR', FSTPO_00:'HTPIMT'
		, FSDPO_00:'HDPIMT', FSTVO_00:'HTVIMT', FSDVO_00:'HDVIMT', FSTPO_A0:'HTPIMV', FSDPO_A0:'HDPIMV', FSTVO_A0:'HTVIMV'
		, FSDVO_A0:'HDVIMV', FSTPO_B0:'HTPN00', FSDPO_B0:'HDPN00', FSTVO_B0:'HTVN00', FSDVO_B0:'HDVN00', FSTPO_NO:'HTPNDSOS'
		, FSDPO_NO:'HDPNDSOS', FSTVO_NO:'HTVNDSOS', FSDVO_NO:'HDVNDSOS', FSTPO_XX:'HTP00N', FSDPO_XX:'HDP00N'
		, FSTVO_XX:'HTV00N', FSDVO_XX:'HDV00N', FSTPO_E0:'HTP000', FSDPO_E0:'HDP000', FSTVO_E0:'HTV000', FSDVO_E0:'HDV000'
		, FSTPO_IT:'HTPIMTN', FSDPO_IT:'HDPIMTN', FSTVO_IT:'HTVIMTN', FSDVO_IT:'HDVIMTN', FSTPO_IV:'HTPIMVN'
		, FSDPO_IV:'HDPIMVN', FSTVO_IV:'HTVIMVN', FSDVO_IV:'HDVIMVN', FSTPO_IND:'HTPNDSN', FSDPO_IND:'HDPNDSN'
		, FSTVO_IND:'HTVNDSN', FSDVO_IND:'HDVNDSN', FSTPO_ITN:'HTPIMBN', FSDPO_ITN:'HDPIMBN', FSTVO_ITN:'HTVIMBN'
		, FSDVO_ITN:'HDVIMBN', FSTPO_NA:'HTPIMNV', FSDPO_NA:'HDPIMNV', FSTVO_NA:'HTVIMNV', FSDVO_NA:'HDVIMNV'
		, FSTPO_NAN:'HTPNDSNV', FSDPO_NAN:'HDPNDSNV', FSTVO_NAN:'HTVNDSNV', FSDVO_NAN:'HDVNDSNV', FSTPO_INA:'HTPNV'
		, FSDPO_INA:'HDPNV', FSTVO_INA:'HTVNV', FSDVO_INA:'HDVNV'}
	
	var objVars = (strIn("M",sType)?objMinus:objPlus);
	for (var fld in objVars)
	{
		drFlds[drFlds.length] = fld;
		regFlds[regFlds.length] = objVars[fld];
	}
	
	/*
	var gV = "-";
	if (strIn("M",sType))
		gV = "+";
	
	var sSQL = "SELECT * FROM ^varreg WHERE NOT FATTR LIKE "
			 + sqlTo(SqlStr4Like + gV + SqlStr4Like);
	var tbl = OpenTable("s",sSQL);
	for (tbl.MoveFirst(); !tbl.isEOF(); tbl.MoveNext())
	{
		if (String(tbl.getValue("FVAR")).substring(0,1) == "R")
			continue;
		drFlds[drFlds.length] = String(tbl.getValue("FFIELD"));
		regFlds[regFlds.length] = String(tbl.getValue("FVAR"));
	}
	*/
	
	var qry = "INSERT INTO " + getPath("ALL_MDB")
			+ "." + (SUBD==MSSQL?".":"") + (strIn("P",sType)?"PDOCREG":"MDOCREG") + "(";
	for (var nRFld = 0; nRFld < regFlds.length; nRFld++)
	{
		if (nRFld != 0)
			qry += ",";
		qry += regFlds[nRFld];
	}
	qry += ") SELECT "
	for (var nDFld = 0; nDFld < drFlds.length; nDFld++)
	{
		if (nDFld != 0)
			qry += ",";
		qry += drFlds[nDFld];
	}
	
	qry += " FROM ^docreg";
	qry += " WHERE FREG=" + Freg;
	return qry;
}

function DpJDecoder_divideDocreg()
{
	var oReg = new DpJRegistrator(null,null,"S");
	var arrJ = oReg.listJournals(true);
	var nJ = 0;
	for (var jID in arrJ)
		nJ++;
	setProgressRangeEx(1,nJ + 1);
	setProgressPosEx(1,"Перенос регистрационных записей...");
	nJ = 1;
	for (var jID in arrJ)
	{
		setProgressPosEx(nJ++,"Перенос регистрационных записей...");
		var Freg = jID.substr(1);
		var jType = arrJ[jID].getType();
		trace(">>" + jType);
		var qry = this.getQry(jType,Freg,jID);
//-		trace(qry);
		if (!ExecuteSQL(qry,false))
			trace("Перенос рег.записей - Ошибка:" + lpProxy.getLastDBErrorMessage());
	}
	setProgressPosEx(0);
}