include("Objects/DpObjectViewer.js");

//- объект для сравнения 2х ДП-объектов
//- не рекомендуется сравнивать объекты разных классов

DpObjectComparer.prototype.getCMPVarList = DpObjectComparer_getCMPVarList;
DpObjectComparer.prototype.getCMPExtList = DpObjectComparer_getCMPExtList;
DpObjectComparer.prototype.getCMPList = DpObjectComparer_getCMPList;
DpObjectComparer.prototype.getChildClasses = DpObjectComparer_getChildClasses;
DpObjectComparer.prototype.getCompareStatus = DpObjectComparer_getCompareStatus;
DpObjectComparer.prototype.getCMPChildsList = DpObjectComparer_getCMPChildsList;
DpObjectComparer.prototype.getLeftObject = DpObjectComparer_getLeftObject;
DpObjectComparer.prototype.getRightObject = DpObjectComparer_getRightObject;
DpObjectComparer.prototype.getCMPVar = DpObjectComparer_getCMPVar;
DpObjectComparer.prototype.getLeftViewer = DpObjectComparer_getLeftViewer;
DpObjectComparer.prototype.getRightViewer = DpObjectComparer_getRightViewer;
DpObjectComparer.prototype.getCMPChild = DpObjectComparer_getCMPChild;

function DpObjectComparer(dpObj1,dpObj2)
{
	this.CMP_EQUAL = 0;
	this.CMP_NOTEXISTS_LEFT = 2;
	this.CMP_NOTEXISTS_RIGHT = 4;
	this.CMP_NOTEQUAL = 8;

	this.dpObj1 = dpObj1;
	this.dpObj2 = dpObj2;
	
	if (this.dpObj1 != null)
		this.viewer1 = new DpObjectViewer(this.dpObj1);
	else
		this.viewer1 = null;
	
	if (this.dpObj2 != null)
		this.viewer2 = new DpObjectViewer(this.dpObj2);
	else
		this.viewer2 = null;
}

function DpObjectComparer_getCMPVarList()
{
	return this.getCMPList("VAR");
}

function DpObjectComparer_getCMPExtList()
{
	return this.getCMPList("EXT");
}

function DpObjectComparer_getCMPVar(sGroup,nId,sSub,sVar)
{
	var cmpRet = new CMPVarInfo();
	cmpRet.varName = sVar;
	
	var bLeftExists = false;
	var bRightExists = false;
	
	if (this.dpObj1 != null)
	{
		bLeftExists = this.viewer1.getObject(sGroup,nId) != null;
		cmpRet.value1 = this.viewer1.getValue(sGroup,nId,sSub,sVar)
	}
	if (this.dpObj2 != null)
	{
		bRightExists = this.viewer2.getObject(sGroup,nId) != null;
		cmpRet.value2 = this.viewer2.getValue(sGroup,nId,sSub,sVar);
	}
	
	if (!bLeftExists && !bRightExists)
		cmpRet.CMPStatus = cmpRet.CMP_EQUAL;
	else if (!bLeftExists)
		cmpRet.CMPStatus = cmpRet.CMP_NOTEXISTS_LEFT;
	else if (!bRightExists)
		cmpRet.CMPStatus = cmpRet.CMP_NOTEXISTS_RIGHT;
	else
	{
		if (differ(cmpRet.value1,cmpRet.value2) == 0)
			cmpRet.CMPStatus = cmpRet.CMP_EQUAL;
		else
			cmpRet.CMPStatus = cmpRet.CMP_NOTEQUAL;
	}
	return cmpRet;
}

//private:
function DpObjectComparer_getCMPList(sSubGroup)
{
	var vList1; 
	var vList2; 
	if (sSubGroup == "VAR")
	{
		if (this.dpObj1 != null)
			vList1 = this.viewer1.getVarList();
		else
			vList1 = new Array();
			
		if (this.dpObj2 != null)
			vList2 = this.viewer2.getVarList();
		else
			vList2 = new Array();
	}
	else if (sSubGroup == "EXT")
	{
		if (this.dpObj1 != null)
			vList1 = this.viewer1.getExtList();
		else
			vList1 = new Array();

		if (this.dpObj2 != null)
			vList2 = this.viewer2.getExtList();
		else
			vList2 = new Array();
	}
	else
		return null;
	
	var retCMPArr = new Array();
	
	for (var nVar = 0; nVar < vList1.length; nVar++)
	{
		var cmpRes = new CMPVarInfo();
		cmpRes.varName = vList1[nVar];
		cmpRes.value1 = this.viewer1.getValue("HEAD",null,sSubGroup,cmpRes.varName);
		cmpRes.varText = this.viewer1.getDescription("HEAD",null,sSubGroup,cmpRes.varName);

		var bRightExists = false;
		for (var nVar2 = 0; nVar2 < vList2.length; nVar2++)
			if (cmpRes.varName == vList2[nVar2])
			{
				bRightExists = true;
				break;
			}
		if (bRightExists)
		{
			cmpRes.value2 = this.viewer2.getValue("HEAD",null,sSubGroup,cmpRes.varName);
			if (differ(cmpRes.value1,cmpRes.value2) == 0)
				cmpRes.CMPStatus = cmpRes.CMP_EQUAL;
			else
				cmpRes.CMPStatus = cmpRes.CMP_NOTEQUAL;
		}
		else
			cmpRes.CMPStatus = cmpRes.CMP_NOTEXISTS_RIGHT;
		retCMPArr[cmpRes.varName] = cmpRes;
	}
	
	for (var nVar = 0; nVar < vList2.length; nVar++)
	{
		if (retCMPArr[vList2[nVar]])
			continue;
		var cmpRes = new CMPVarInfo();
		cmpRes.varName = vList2[nVar];
		cmpRes.CMPStatus = cmpRes.CMP_NOTEXISTS_LEFT;
		cmpRes.value2 = this.viewer2.getValue("HEAD",null,sSubGroup,cmpRes.varName);
		cmpRes.varText= this.viewer2.getDescription("HEAD",null,sSubGroup,cmpRes.varName);
		retCMPArr[cmpRes.varName] = cmpRes;
	}
	return retCMPArr;
}

function DpObjectComparer_getChildClasses()
{
	if (this.dpObj1 != null)
		return this.viewer1.getChildClasses();
	else if (this.dpObj2 != null)
		return this.viewer2.getChildClasses();
	return new Array();
}

function DpObjectComparer_getCMPChild(sClass,nID)
{
	var retCmp = new CMPRowInfo();
	
	var viewer1 = this.getLeftViewer();
	var viewer2 = this.getRightViewer();
	
	var obj1 = null;
	var obj2 = null;
	
	if (viewer1)
		obj1 = viewer1.getValue(sClass,nID);
	if (viewer2)
		obj2 = viewer2.getValue(sClass,nID);
	retCmp.objComparer = new DpObjectComparer(obj1,obj2);
	retCmp.CMPStatus = retCmp.objComparer.getCompareStatus();
	
	return retCmp;
}

function DpObjectComparer_getCMPChildsList(sClass)
{
	var chIDs1 = (this.dpObj1 != null? this.viewer1.getChildIDs(sClass):new Array());
	var chIDs2 = (this.dpObj2 != null? this.viewer2.getChildIDs(sClass):new Array());
	
	var retArr = new Array();
	
	for (var nChID = 0; nChID < chIDs1.length; nChID++)
	{
		var row1 = this.viewer1.getObject(sClass,chIDs1[nChID]);
		var row2 = (this.dpObj2 != null? this.viewer2.getObject(sClass,chIDs1[nChID]):null);
		var sRowIDX = sClass + "@" + chIDs1[nChID];
		var cmpRes = new CMPRowInfo();
		cmpRes.objComparer = new DpObjectComparer(row1,row2);
		cmpRes.CMPStatus = cmpRes.objComparer.getCompareStatus();
		retArr[sRowIDX] = cmpRes;
	}
	
	for (var nChID = 0; nChID < chIDs2.length; nChID++)
	{
		var sRowIDX = sClass + "@" + chIDs2[nChID];
		if (retArr[sRowIDX])
			continue;
		var row1 = (this.dpObj1 != null? this.viewer1.getObject(sClass,chIDs2[nChID]):null);
		var row2 = this.viewer2.getObject(sClass,chIDs2[nChID]);
		var cmpRes = new CMPRowInfo();
		cmpRes.objComparer = new DpObjectComparer(row1,row2);
		cmpRes.CMPStatus = cmpRes.objComparer.getCompareStatus();
		retArr[sRowIDX] = cmpRes;
	}
	return retArr;
}

function DpObjectComparer_getCompareStatus()
{
	if (this.dpObj1 == null && this.dpObj2 != null)
		return this.CMP_NOTEXISTS_LEFT;
	if (this.dpObj2 == null && this.dpObj1 != null)
		return this.CMP_NOTEXISTS_RIGHT;
	if (this.dpObj1 == null && this.dpObj2 == null)
		return this.CMP_EQUAL;
	
	var bEqual = true;
	
	var cmpVarList = this.getCMPVarList();
	for (var sVar in cmpVarList)
	{
		bEqual = bEqual && (cmpVarList[sVar].CMPStatus == this.CMP_EQUAL);
		if (!bEqual)
			return this.CMP_NOTEQUAL;
	}
	
	var cmpExtList = this.getCMPExtList();
	for (var sExt in cmpExtList)
	{
		bEqual = bEqual && (cmpExtList[sExt].CMPStatus == this.CMP_EQUAL);
		if (!bEqual)
			return this.CMP_NOTEQUAL;
	}
	
	var arrClasses = this.getChildClasses();
	for (var nClass = 0; nClass < arrClasses.length; nClass++)
	{
		var cmpClassList = this.getCMPChildsList(arrClasses[nClass]);
		for (var sRowIDX in cmpClassList)
		{
			bEqual = bEqual && (cmpClassList[sRowIDX].CMPStatus == this.CMP_EQUAL);
			if (!bEqual)
				return this.CMP_NOTEQUAL;
		}
	}
	return this.CMP_EQUAL;
}

function DpObjectComparer_getLeftObject()
{
	return this.dpObj1;
}

function DpObjectComparer_getRightObject()
{
	return this.dpObj2;
}

function DpObjectComparer_getLeftViewer()
{
	return this.viewer1;
}

function DpObjectComparer_getRightViewer()
{
	return this.viewer2;
}

function CMPRowInfo()
{
	this.CMP_EQUAL = 0;
	this.CMP_NOTEXISTS_LEFT = 2;
	this.CMP_NOTEXISTS_RIGHT = 4;
	this.CMP_NOTEQUAL = 8;
	
	this.objComparer = null;
	this.CMPStatus = 0;
}

function CMPVarInfo()
{
	this.CMP_EQUAL = 0;
	this.CMP_NOTEXISTS_LEFT = 2;
	this.CMP_NOTEXISTS_RIGHT = 4;
	this.CMP_NOTEQUAL = 8;

	this.varName = "";
	this.varText = "";
	this.value1 = null;
	this.value2 = null;
	this.CMPStatus = 0;
}