DpObjectSyncronizer.prototype.putObjectForSynchronize = DpObjectSyncronizer_putObjectForSynchronize;
DpObjectSyncronizer.prototype.delObjectFromSynchro = DpObjectSyncronizer_delObjectFromSynchro;
DpObjectSyncronizer.prototype.getTblName = DpObjectSyncronizer_getTblName;

var DpObjectSyncronizer_DELETED = 100;
var DpObjectSyncronizer_UPDATED = 200;
var DpObjectSyncronizer_NO_SYNC = 300;

function DpObjectSyncronizer(obj)
{
	this.sycronize = DpObjectSyncronizer_sycronize;
	this.obj = obj;
	this.syncRegistry = lpProxy.getSynchronizeRegistry();
	this.replaceLocal = DpObjectSyncronizer_replaceLocal;
	this.removeLocal = DpObjectSyncronizer_removeLocal;	
	this.isDoc = true;
	this.lastSyncroVer = 0;
	this.isLocalObject = true;
	this.isUpdated = true; 
	
	
	if (obj == null) return;
	
	if (objTypeOf(obj) == "DpDoc")
	{
		this.fullClassID = obj.sDocLbl;
		this.subClass = obj.nNop;
		this.isDoc = true;
	}
	else
	{
		this.fullClassID = "CLS." + obj.sClassId;
		this.subClass = obj.nCl;
		this.isDoc = false;
	}
	
	if (getPar("ISCLI") 
		&& this.syncRegistry.isSynchronized(this.isDoc?("DOC."+this.fullClassID):this.fullClassID, this.subClass))
	{
		this.isLocalObject = false;
	}
		
}

function DpObjectSyncronizer_sycronize(ID)
{
	if (this.isLocalObject)
	{
		return DpObjectSyncronizer_UPDATED;	
	}
		
	
	var jObjectSelector = this.obj.getJObject().getSelector();
	var localInfo = jObjectSelector.getObjectArchiveInfo(ID);  
	if (localInfo == null)
	{
		this.isUpdated = false;
		this.lastSyncroVer = 0;
		return DpObjectSyncronizer_NO_SYNC;
	}
	this.isSync = !toScrObj(localInfo.bInSynchroList);	
	var remoteInfo = jObjectSelector.getRemoteObjectArchiveInfo(ID);

	if (remoteInfo == null)
	{
		this.isUpdated = false;
		if (this.isSync)
			this.lastSyncroVer = Math.max(localInfo.version, 0);
		else
			this.lastSyncroVer = Math.max(localInfo.sync_version, 0);
		return DpObjectSyncronizer_NO_SYNC;
	}

	
	var localVer = Number(localInfo.version);
	var remoteVer = Number(remoteInfo.version);
	
	var isRemoteDel = String(remoteInfo.sArchiveAction) == "D";
	
	if (isRemoteDel)
	{
		if (confirm("Объект на сервере удален.\nУдалить локально?"))
		{
			this.removeLocal(ID, remoteInfo);
			return DpObjectSyncronizer_DELETED;
		}
		else
		{
			this.lastSyncroVer = Math.min(remoteVer, localVer); 
			return DpObjectSyncronizer_NO_SYNC;
		} 
	}
	
	if (this.isSync)
	{
		if (localVer < remoteVer)
		{
			if (!this.obj.getJObject().loadRemote(ID))
			{				
				this.isUpdated = false;
				return DpObjectSyncronizer_NO_SYNC;
			}
			this.replaceLocal(remoteInfo);
			this.lastSyncroVer = remoteVer; 
			return DpObjectSyncronizer_UPDATED;
		}
		else
		{
			this.lastSyncroVer = remoteVer; 
			return DpObjectSyncronizer_UPDATED
		}
	}
	else
	{
		this.lastSyncroVer = localInfo.sync_version
		if (this.lastSyncroVer >= remoteVer)
		{
			this.delObjectFromSynchro(this.fullClassID, ID);
			return DpObjectSyncronizer_UPDATED;
		}
		else
		{
			this.isUpdated = false;
			return DpObjectSyncronizer_NO_SYNC;
		}
	}
}

function DpObjectSyncronizer_replaceLocal(remoteInfo)
{
	this.obj.fillSynchroVars();
	this.obj.setVar("SYNC_DATE",javaDate2Script(remoteInfo.dArchDate,true).getVarDate());
    this.obj.setVar("SYNC_USER",remoteInfo.sUser);
	this.obj.toArchive("EDIT");
	this.obj.saveExtToDB();
	obj2DB(this.obj.getJObject().getObjectID(),this.obj.getJObject());
	this.obj.saveRows();
}

function DpObjectSyncronizer_removeLocal(ID,remoteInfo)
{
	this.obj.fillSynchroVars();
	this.obj.setVar("SYNC_DATE",javaDate2Script(remoteInfo.dArchDate,true).getVarDate());
    this.obj.setVar("SYNC_USER",remoteInfo.sUser);
    this.obj.getJObject().loadFromDB(ID);
	this.obj.toArchive("DEL");
	delObjFromDB(ID,this.obj.getJObject());
	
	this.obj.delExts(ID);	
	if (this.isDoc)
		this.obj.delRegistration();
	
}

function DpObjectSyncronizer_getTblName()
{
	return "^OBJSYNC";
}

function DpObjectSyncronizer_putObjectForSynchronize(sClassID,nObjID)
{
	this.delObjectFromSynchro(sClassID,nObjID);
	var m = new Object();
	m.FCLASS = String(sClassID);
	m.FOBJID = Number(nObjID);
	m.FDATE = new Date();
	m.FUSER = getUserName();
	m.FHOST = getHostName();
	m.FVER = this.lastSyncroVer;
	
	UpdateTable(this.getTblName(),m,true);
}

function DpObjectSyncronizer_delObjectFromSynchro(sClassID,nObjID)
{
	return ExecuteSQL("DELETE FROM " + this.getTblName()
			 + " WHERE FCLASS=" + sqlTo(String(sClassID))
			 + " AND FOBJID=" + sqlTo(Number(nObjID)));
}