
DpOperation.prototype.setNum = DpOperation_setNum;
DpOperation.prototype.getXMLPath = DpOperation_getXMLPath;
DpOperation.prototype.getXMLFile = DpOperation_getXMLFile;
DpOperation.prototype.getFullXMLPath = DpOperation_getFullXMLPath;
DpOperation.prototype.getXMLStub = DpOperation_getXMLStub;
DpOperation.prototype.getXMLDoc = DpOperation_getXMLDoc;
DpOperation.prototype.save = DpOperation_save;
DpOperation.prototype.parseXML = DpOperation_parseXML;
DpOperation.prototype.getSection = DpOperation_getSection;
DpOperation.prototype.addSection = DpOperation_addSection;
DpOperation.prototype.removeSection = DpOperation_removeSection;
DpOperation.prototype.setVar = DpOperation_setVar;
DpOperation.prototype.getVar = DpOperation_getVar;


function DpOperation(doc,nop)
{
	this.doc = doc;
	this.nop = nop;
	
	this.xmlDoc = null;
	this.sectionList = new Array();
	
	this.path = getOrgPath() + "settings/operations/" 
		+ String(this.doc).toUpperCase().replace(":", "_")+ "/";	
	createDirectory(this.path);
	
	
	this.parseXML();
}

function DpOperation_setNum(nop)
{
	this.nop = nop;
}

function DpOperation_getXMLPath()
{
	return this.path;
}

function DpOperation_getXMLFile()
{
	return this.nop + ".xml";
}

function DpOperation_getFullXMLPath()
{
	return this.getXMLPath() + this.getXMLFile();
}

function DpOperation_getXMLStub()
{
	var oDoc = createXmlDoc();
	var root = oDoc.createElement("operation");
	root.setAttribute("doc",this.doc);
	oDoc.appendChild(root);
	
	var nullSection = oDoc.createElement("section");
	nullSection.setAttribute("num","0");
	root.appendChild(nullSection);
	return oDoc;
}

function DpOperation_getXMLDoc()
{
	if (this.xmlDoc != null)
		return this.xmlDoc;
	if (isFile(this.getFullXMLPath()))
		this.xmlDoc = openXmlFile(this.getFullXMLPath());
	else
		this.xmlDoc = this.getXMLStub();
	return this.xmlDoc;
}

function DpOperation_parseXML()
{
	var oDoc = this.getXMLDoc();
	var root = oDoc.getDocumentElement();
	
	this.sectionTbl = OpenTable("xml","NOFILE","section");
	this.sectionTbl.requery(oDoc);
	this.sectionTbl.setDoSave(false);
	
	var sectLst = root.getElementsByTagName("section");
	for (var nSect = 0; nSect < sectLst.getLength(); nSect ++)
	{
		var curEl = sectLst.item(nSect);
		var sectNum = parseInt(curEl.getAttribute("num"));
		this.sectionList[sectNum] = new DpSection(this);
		this.sectionList[sectNum].parseXML(curEl);
	}
}

function DpOperation_getSection(sectNum)
{
	if (this.sectionList[sectNum])
		return this.sectionList[sectNum];
	return null;
}

function DpOperation_addSection(sectNum)
{
	sectNum = parseInt(sectNum);
	if (this.getSection(sectNum) == null)
	{
		this.sectionTbl.AddNew();
		this.sectionTbl.setValue("num",String(sectNum));
		this.sectionTbl.Update();
		this.sectionList[sectNum] = new DpSection(this);
		
		var sList = this.getXMLDoc().getElementsByTagName("section");
		for (var nSect = 0; nSect < sList.getLength(); nSect++)
		{
			if (sList.item(nSect).getAttribute("num") == sectNum)
			{
				this.sectionList[sectNum].parseXML(sList.item(nSect));
				break;
			}
		}
	}
	else
		return false;
	return true;
}

function DpOperation_removeSection(sectNum)
{
	sectNum = parseInt(sectNum);
	if (this.sectionList[sectNum])
	{
		this.sectionList[sectNum] = false;
		var tbl = this.sectionTbl;
		for (tbl.MoveFirst();!tbl.isEOF();tbl.MoveNext())
		{
			if (parseInt(tbl.getValue("num")) == sectNum)
			{
				tbl.Delete();
				break;
			}
		}
	}
	else
		return false;
	return true;
}

function DpOperation_save()
{
	saveXml(this.getFullXMLPath(),this.getXMLDoc());
}

function DpOperation_setVar(varName,Value)
{
	if (Value == null || Value == "null")
		return;
	var oDoc = this.getXMLDoc();
	var root = oDoc.getDocumentElement();
	root.setAttribute(varName,Value);
}

function DpOperation_getVar(varName)
{
	var oDoc = this.getXMLDoc();
	var root = oDoc.getDocumentElement();
	return root.getAttribute(varName);
}
//----------------------------- DpSection ------------------------------------------

DpSection.prototype.parseXML = DpSection_parseXML;
DpSection.prototype.gotoProv = DpSection_gotoProv;
DpSection.prototype.addProv = DpSection_addProv;
DpSection.prototype.removeProv = DpSection_removeProv;
DpSection.prototype.setVar = DpSection_setVar;
DpSection.prototype.getVar = DpSection_getVar;
DpSection.prototype.refresh = DpSection_refresh;
DpSection.prototype.getProvTbl = DpSection_getProvTbl;

function DpSection(oOper)
{
	this.oOper = oOper;
	this.num = 0;
}

function DpSection_refresh()
{
	this.parseXML(this.xmlEl);
}

function DpSection_getProvTbl()
{
	if (this.provTbl == null)
		return null;
	return this.provTbl.clone();
}

function DpSection_parseXML(oEl)
{
	this.xmlEl = oEl;
	this.num = parseInt(oEl.getAttribute("num"));
	this.txt = oEl.getAttribute("txt");
	this.cond = oEl.getAttribute("cond");
	this.filter = oEl.getAttribute("filter");
	this.group = oEl.getAttribute("group");
	this.description = oEl.getAttribute("description");
	
	this.provTbl = OpenTable("xml","NOFILE","section[@num='" + this.num + "']/prov");
	this.provTbl.requery(oEl.getOwnerDocument());
	this.provTbl.setDoSave(false);
}

function DpSection_gotoProv(provNum)
{
	for (this.provTbl.MoveFirst();!this.provTbl.isEOF();this.provTbl.MoveNext())
	{
		if (parseInt(provNum) == parseInt(this.provTbl.getValue("num")))
			return true;
	}
	return false;
}

function DpSection_addProv(provNum)
{
	if (this.gotoProv(provNum))
		return false;
	this.provTbl.AddNew();
	this.provTbl.setValue("num",String(provNum));
	this.provTbl.Update();
	return true;
}

function DpSection_removeProv(provNum)
{
	if (!this.gotoProv(provNum))
		return false;
	this.provTbl.Delete();
	this.provTbl.requery();
	return true;
}

function DpSection_setVar(varName,value)
{
	if (value == null || value == "null")
		return;
	this.xmlEl.setAttribute(varName,value);
}

function DpSection_getVar(varName)
{
	return this.xmlEl.getAttribute(varName);
}

//---------------------------- DpOperaionList --------------------------------------

DpOperationList.prototype.getPath = DpOperationList_getPath;
DpOperationList.prototype.listOperations = DpOperationList_listOperations;
DpOperationList.prototype.removeOperation = DpOperationList_removeOperation;
DpOperationList.prototype.addOperation = DpOperationList_addOperation;
DpOperationList.prototype.save = DpOperationList_save;
DpOperationList.prototype.sort = DpOperationList_sort;

function DpOperationList(doc)
{
	this.doc = doc;
	
	this.operList = null;
	this.path = new DpOperation(doc).getXMLPath();	
}

function DpOperationList_getPath()
{
	return this.path;
}

function DpOperationList_listOperations(bRefresh)
{
	if (this.operList != null && !bRefresh)
		return this.operList;
	this.operList = new Array();
	var file = new DpFile(this.getPath());
	var files = file.listFiles();
	if (files == null)
		return this.operList;
	for (var nF = 0; nF < files.length; nF++)
	{
		var currF = files[nF];
		if (currF.isDirectory())
			continue;
		var name = String(currF.getName());
		var nInd = name.lastIndexOf(".");
		if (nInd == -1 || name.substr(nInd) != ".xml")
			continue;
		var operNum = name.substring(0,nInd);
		if (!isNaN(operNum))
			operNum = parseInt(operNum);
			
		this.operList[operNum] = new DpOperation(this.doc,operNum);
	}
	return this.operList;
}

function DpOperationList_removeOperation(nOp)
{
	if(!isNaN(nOp))
		nOp = parseInt(nOp);
	if (this.listOperations(true)[nOp])
	{
		deleteFile(this.listOperations(false)[nOp].getFullXMLPath());
		this.listOperations(true);
	}
}

function DpOperationList_addOperation(nOp)
{
	if(!isNaN(nOp))
		nOp = parseInt(nOp);
	if (this.listOperations(true)[nOp])
		return false;
	this.operList[nOp] = new DpOperation(this.doc,nOp);
	this.operList[nOp].save();
}

function DpOperationList_save()
{
	var arrOps = this.listOperations(false);
	for (var nOp in arrOps)
	{
		arrOps[nOp].save();
	}
}

function DpOperationList_sort(oLst)
{
	var ret = new Array();
	for (var nOp in oLst)
	{
		ret[nOp] = oLst[nOp];
	}
//-	this.operList.sort(DpOperation_sortFnk);
	ret.sort(DpOperation_sortFnk);
	var newList = new Array();
	for(var nOp in ret)
	{
		if (!ret[nOp])
			continue;
		newList[nOp] = ret[nOp];
	}
	return newList;
}

function DpOperation_sortFnk(arg0,arg1)
{
	if (arg0.nop > arg1.nop)
		return 1;
	return -1;
}