/*
   Описание объекта "Диапазон" для подсистемы "Недвижимость": DpRange
*/

// Конструктор объекта DpRange
//Объект предназначен:
//1)для организации контрола для ввода диапазона значений числового вида
//  при этом значения пишутся в 3 поля: 
//  оператор : "=", "<>", ">=", "<=", ">", "<", "-"
//	начальное значение
//	конечное значение
//2)для построения запросов к таблице, содержащей либо конкретные значения поля, 
//  либо описания диапазонов(метод getWhere()).
//  Поведение объекта для случая, когда в таблице в поле, для которого строится запрос, пусто,
//  регулируется свойством mode4EmptyValue, принимающим значения:
//		DpRange.MODE_4_EMPTY_VALUE_TRUE - (по умолчанию) возвращать true
//		DpRange.MODE_4_EMPTY_VALUE_FALSE -  возвращать false
// valuesField - префикс имени поля в таблице значений, из которого будут сделаны все 3 имени полей
// valuesField - префикс имени поля в таблице условий, из которого будут сделаны все 3 имени полей
function DpRange(valuesField, condField)
{
	RO("DpRange", this);

	//public:
	this.setFieldName = DpRange_setFieldName;	//установить префикс имен полей 
	this.load = DpRange_load;	//загрузить значения

	this.get = DpRange_get;	//Получить любой из атрибутов

	this.valid   = DpRange_valid;

	this.linkCtrl = DpRange_linkCtrl;

	this.validCtrl = DpRange_validCtrl;

	this.updateCtrl = DpRange_updateCtrl;

	this.getWhere = DpRange_getWhere;	//вернет кусок условия для использования в запросе

	//Константы
	//  Поведение объекта для случая, когда в таблице в поле, для которого строится запрос, пусто,
	//  регулируется свойством mode4EmptyValue, принимающим значения:
	this.MODE_4_EMPTY_VALUE_TRUE = 1;	// - (по умолчанию) возвращать true
	this.MODE_4_EMPTY_VALUE_FALSE = 2;	// -  возвращать false
	
	this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE;
	
	//private:
	this.ctrl = null;

	this.setFieldName(valuesField, condField);

	//Суффиксы названий полей
	this.suffOp = "_OP";	//оператор
	this.suffStart = "_B";	//начальное значение
	this.suffEnd = "_E";		//конечное значение

	//значения
	this.op = null;	//оператор
	this.startVal = null;	//начальное значение
	this.endVal = null;		//конечное значение

	return this;
}

//установить имена полей 
function DpRange_setFieldName(valuesField, condField)
{
	this.condField = condField;	//префикс имени поля, из которого будут сделаны все 3 имени полей
	this.valuesField = valuesField;	//имя поля в таблице значений, для которого будут строиться условия
}

// Установить значения
function DpRange_load(op, startVal, endVal)
{ 
	this.op = op;	//оператор
	this.startVal = startVal;	//начальное значение
	this.endVal = endVal;		//конечное значение

	this.updateCtrl();
}

//Получить любой из атрибутов
// mode = {"FIELD", "VAR", "VALUE"} - получить имя поля, имя переменной для документа, значение
// attrName = {"OP", "START", "END"}
function DpRange_get(mode, attrName)
{
	var retVal = null;
	switch(mode)
	{
		case "FIELD":
			switch(attrName)
			{
				case "OP":
					retVal = this.condField + this.suffOp;
					break;
				case "START":
					retVal = this.condField + this.suffStart;
					break;
				case "END":
					retVal = this.condField + this.suffEnd;
					break;
			}
			break;
		case "VAR":
			var sVarPrefix = "H"+this.condField.substr(1);
			switch(attrName)
			{
				case "OP":
					retVal = sVarPrefix + this.suffOp;
					break;
				case "START":
					retVal = sVarPrefix + this.suffStart;
					break;
				case "END":
					retVal = sVarPrefix + this.suffEnd;
					break;
			}
			break;
		case "VALUE":
			switch(attrName)
			{
				case "OP":
					retVal = this.op;
					break;
				case "START":
					retVal = this.startVal;
					break;
				case "END":
					retVal = this.endVal;
					break;
			}
			break;
	}	
	return retVal;
}

// валидация значения в контроле.
// попутно заменим в числах запятые на точки
function DpRange_valid()
{  
	var retVal = false;

	var sValue = String(this.ctrl.value);
	//Для начала заменим запятые на точки
	sValue = sValue.replace(/\,/gi,".");
	//теперь уберем пробелы
	sValue = sValue.replace(/\s/gi,"");

	//проверим случай "<число>-<число>"
	if(sValue == "")
	{
		//значения
		this.op = null;	//оператор
		this.startVal = null;	//начальное значение
		this.endVal = null;		//конечное значение
		retVal = true;
	}
	else if(sValue.match(/(^[0-9]+(\.[0-9]+){0,1})(\-)([0-9]+(\.[0-9]+){0,1}$)/gi))
	{
		this.op = "-";
		var startVal = parseFloat(sValue.replace(/(^[0-9]+(\.[0-9]+){0,1})(\-)([0-9]+(\.[0-9]+){0,1}$)/gi, "$1"));
		var endVal = parseFloat(sValue.replace(/(^[0-9]+(\.[0-9]+){0,1})(\-)([0-9]+(\.[0-9]+){0,1}$)/gi, "$4"));
		//на всякий случай проконтролируем, чтобы выполнялось условие this.startVal<this.endVal
		if(startVal<endVal)
		{
			this.startVal = startVal;
			this.endVal = endVal;
			retVal = true;
		}
		else
		{
			retVal = softInvalid;
			WAIT(UR("Початкове значення діапазону не має перевищувати кінцеве значення"
			+"|Начальное значение диапазона не должно превышать конечное значение"));
		}
	}
	//теперь проверим случай, когда "<число>", т.е. равенство
	else if(sValue.match(/^([0-9]+(\.[0-9]+){0,1}$)/gi))
	{
		this.op = "=";
		this.startVal = parseFloat(sValue);
		this.endVal = null;
		retVal = true;
	}
	//теперь проверим случай, когда "<оператор><число>"
	else if(sValue.match(/^((<)|(<=)|(>)|(>=)|(=))([0-9]+(\.[0-9]+){0,1}$)/gi))
	{
		this.op = sValue.replace(/^((<)|(<=)|(>)|(>=)|(=))([0-9]+(\.[0-9]+){0,1}$)/gi, "$1");
		this.startVal = parseFloat(sValue.replace(/^((<)|(<=)|(>)|(>=)|(=))([0-9]+(\.[0-9]+){0,1}$)/gi, "$7"));
		this.endVal = null;
		retVal = true;
	
	}
	else if(sValue!="")
	{
		retVal = softInvalid;
		WAIT(UR("Неприпустиме значення. Приклади припустимих значень: \"5\" або \"1-6\" або \">=3\""
		+"|Недопустимое значение. Примеры допустимых значений: \"5\" или \"1-6\" или \">=3\""));
	}
	return retVal;
}

// Связать с полем ввода
function DpRange_linkCtrl(ctrl)
{
	if (typeof ctrl != "undefined")
	{
		this.ctrl = ctrl;
///		this.ctrl.setMask("&&&&&&&&&&&&&&&&&&&&");
		this.ctrl.mask = "&&&&&&&&&&&&&&&&&&&&";

		setValid(this.ctrl, this, "validCtrl");
	}

	this.updateCtrl();
}

function DpRange_validCtrl()
{
	if (!this.valid()) 
	{
		return false;
	}
//	this.setCod(this.ctrl.value);
	this.updateCtrl();
	return true;
}

function DpRange_updateCtrl()
{   
	var sValue = "";
	if (this.ctrl)
	{
		switch(this.op)
		{
			case "=":
				if(this.startVal!=null)
				{
					sValue = String(this.startVal);
				}
				break;
			case "<>":
			case ">=":
			case "<=":
			case ">":
			case "<":
				if(this.startVal!=null)
				{
					sValue = this.op + String(this.startVal);
				}
				break;
			case "-":
				if(this.startVal!=null && this.endVal!=null)
				{
					sValue = String(this.startVal)+this.op+String(this.endVal);
				}
				break;
		}
//		alert("this.op="+this.op+"\nthis.ctrl.value = "+sValue)
		this.ctrl.value = sValue;
	}
}

//вернет кусок условия для использования в запросе
// outerMode = {"VALUES", "RANGES"} - режим для таблицы, к которой строим запрос
//		"VALUES" - таблица содержит конкретные значения в this.valuesField
// 		"RANGES" - таблица содержит диапазоны значений(условия) в полях с именами
// 			this.valuesField+this.suffOp;	//оператор
// 			this.valuesField+this.suffStart;	//начальное значение
// 			this.valuesField+this.suffEnd;		//конечное значение
// 		outerMode может также содержать символ "&" - тогда к результату спереди будет 
// 		добавлен "AND"
// innerMode = {"VALUE", "TABLE"} - режим для нашего объекта
//		"VALUE" - строим запрос исходя из конкретных значений в объекте
//		"TABLE" - строим запрос для случая, когда условия - в таблице с алиасом innerAlias
// outerAlias - псевдоним таблицы, к которой строим запрос
// innerAlias - псевдоним таблицы с условиями в случае, когда innerMode == "TABLE"

//Примеры:
// 1. Строим условие для поля FKOL т-цы с алиасом "SALE", используя конкретные значения
// в объекте
//		oRange = new DpRange("FKOL");
//		oRange.load("-", 100, 300)
//		sWhere = oRange.getWhere("&VALUES", "VALUE", "SALE")
//		Получим: AND (SALE.FKOL>=100 AND SALE.FKOL<=300)
// 2. Строим условие для поля FKOL т-цы с алиасом "SALE", используя
// условия из т-цы с алиасом "BUY"
//		oRange = new DpRange("FKOL");
//		sWhere = oRange.getWhere("VALUES", "TABLE", "SALE", "BUY")
//		Получим: IIF(BUY.FKOL_OP="=", SALE.FKOL=BUY.FKOL_B, 
//				IIF(BUY.FKOL_OP=">=", SALE.FKOL>=BUY.FKOL_B,
//				...
//					) 
//					)
function DpRange_getWhere(outerMode, innerMode, outerAlias, innerAlias)
{
	//alert(outerMode);
	var sValue = "";
	var svOuterMode = outerMode;
//	var toAddAnd = (/\&/gi).test(outerMode);	//почему-то при последовательных вызовах срабатывало через раз

	//Уберем амперсанд из режима
	outerMode = outerMode.replace(/\&/gi, "");

	var toAddAnd = outerMode != svOuterMode;

	//alert(outerMode);
	//alert(toAddAnd);

	var iop = " "+innerAlias+"."+this.condField+this.suffOp;
	var ib = " "+innerAlias+"."+this.condField+this.suffStart;
	var ie = " "+innerAlias+"."+this.condField+this.suffEnd;
	var ival = ib;

	var oop = " "+outerAlias+"."+this.valuesField+this.suffOp;
	var ob = " "+outerAlias+"."+this.valuesField+this.suffStart;
	var oe = " "+outerAlias+"."+this.valuesField+this.suffEnd;
	var oval = ob;

	switch(innerMode)
	{
		case "TABLE":	// строим запрос для случая, когда условия - в таблице с алиасом innerAlias
			switch(outerMode)
			{
				case "VALUES":	//таблица значений содержит конкретные значения в this.valuesField
					var oval = " "+outerAlias+"."+this.valuesField;
					sValue = IIF(
							iop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
												//в зависимости от режима
							, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
						,IIF(
							iop+"="+sqlTo("=")
							,oval+"="+ival
						,IIF(
							iop+"="+sqlTo("<>")
							,oval+"<>"+ival
						,IIF(
							iop+"="+sqlTo(">")
							,oval+">"+ival
						,IIF(
							iop+"="+sqlTo(">=")
							,oval+">="+ival
						,IIF(
							iop+"="+sqlTo("<")
							,oval+"<"+ival
						,IIF(
							iop+"="+sqlTo("<=")
							,oval+"<="+ival, 
						IIF(iop+"="+sqlTo("-"), 
							oval+">="+ib
							+" AND "+oval+"<="+ie
					)
					)
					)
					)
					)
					)
					)
					);
					break;
				case "RANGES":	// - таблица содержит диапазоны значений(условия)
					sValue = IIF(
						iop+"="+sqlTo("")+" OR "+oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
																	//в зависимости от режима
						, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
						,IIF(
						iop+"="+sqlTo("=")+" AND "+oop+"="+sqlTo("=")				//1 =,=
						, IIF(ival+"="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("=")+" AND "+oop+"="+sqlTo("<>")	//2 =,<>
						, IIF(ival+"<>"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("=")+" AND "+oop+"="+sqlTo(">")		//3 =,>
						, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("=")+" AND "+oop+"="+sqlTo(">=")	//4 =,>=
						, IIF(ival+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("=")+" AND "+oop+"="+sqlTo("<")		//5 =,<
						, IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("=")+" AND "+oop+"="+sqlTo("<=")	//6 =,<=
						, IIF(ival+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo(">")+" AND "+oop+"="+sqlTo("<")		//17 >,<
						, IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo(">=")+" AND "+oop+"="+sqlTo("<")	//23 >=,<
						, IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo(">=")+" AND "+oop+"="+sqlTo("<=")	//24 >=,<=
						, IIF(ival+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("<")+" AND "+oop+"="+sqlTo(">")		//27 <,>
						, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("<")+" AND "+oop+"="+sqlTo(">=")	//28 <,>=
						, IIF(ival+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("<=")+" AND "+oop+"="+sqlTo(">")	//33 <=,>
						, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						,IIF(
						iop+"="+sqlTo("<=")+" AND "+oop+"="+sqlTo(">=")	//34 <=,>=
						, IIF(ival+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
						//, ival+oop+oval	// (1)
						,IIF(
						iop+"="+sqlTo("<>")+" AND "+oop+"="+sqlTo("=")				//7 <>,=
						+" OR " + iop+"="+sqlTo("<>")+" AND "+oop+"="+sqlTo(">=")	//10 <>,>=
						+" OR " + iop+"="+sqlTo("<>")+" AND "+oop+"="+sqlTo("<=")	//12 <>,<=
						, IIF(ival+"<>"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (2)
						,IIF(
						iop+"="+sqlTo("<>")+" AND "+oop+"="+sqlTo("<>")				//8 <>,<>
						+" OR " + iop+"="+sqlTo("<>")+" AND "+oop+"="+sqlTo(">")	//9 <>,>
						+" OR " + iop+"="+sqlTo("<>")+" AND "+oop+"="+sqlTo("<")	//11 <>,<
						+" OR " + iop+"="+sqlTo(">")+" AND "+oop+"="+sqlTo("<>")	//14 >,<>
						+" OR " + iop+"="+sqlTo(">")+" AND "+oop+"="+sqlTo(">")		//15 >,>
						+" OR " + iop+"="+sqlTo(">")+" AND "+oop+"="+sqlTo(">=")	//16 >,>=
						+" OR " + iop+"="+sqlTo(">=")+" AND "+oop+"="+sqlTo("<>")	//20 >=,<>
						+" OR " + iop+"="+sqlTo(">=")+" AND "+oop+"="+sqlTo(">")	//21 >=,>
						+" OR " + iop+"="+sqlTo(">=")+" AND "+oop+"="+sqlTo(">=")	//22 >=,>=
						+" OR " + iop+"="+sqlTo("<")+" AND "+oop+"="+sqlTo("<>")	//26 <,<>
						+" OR " + iop+"="+sqlTo("<")+" AND "+oop+"="+sqlTo("<")		//29 <,<
						+" OR " + iop+"="+sqlTo("<")+" AND "+oop+"="+sqlTo("<=")	//30 <,<=
						+" OR " + iop+"="+sqlTo("<=")+" AND "+oop+"="+sqlTo("<>")	//32 <=,<>
						+" OR " + iop+"="+sqlTo("<=")+" AND "+oop+"="+sqlTo("<")	//35 <=,<
						+" OR " + iop+"="+sqlTo("<=")+" AND "+oop+"="+sqlTo("<=")	//36 <=,<=
						+" OR " + iop+"="+sqlTo("<>")+" AND "+oop+"="+sqlTo("-")	//38 <>,-
						+" OR " + iop+"="+sqlTo("-")+" AND "+oop+"="+sqlTo("<>")	//44 -,<>
						,"("+sqlCmpTrue+")"		// (3)
						,IIF(
						iop+"="+sqlTo(">")+" AND "+oop+"="+sqlTo("=")				//13 >,=
						+" OR " + iop+"="+sqlTo(">")+" AND "+oop+"="+sqlTo("<=")	//18 >,<=
						, IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo(">=")+" AND "+oop+"="+sqlTo("=")				//19 >=,=
						, IIF(ival+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("<")+" AND "+oop+"="+sqlTo("=")				//25 <,=
						, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("<=")+" AND "+oop+"="+sqlTo("=")				//31 <=,=
						, IIF(ival+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("=")+" AND "+oop+"="+sqlTo("-")				//37 =,-
						, IIF(ival+">="+ob+" AND "+ival+"<="+oe, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo(">")+" AND "+oop+"="+sqlTo("-")				//39 >,-
						, IIF(ival+"<"+oe, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo(">=")+" AND "+oop+"="+sqlTo("-")				//40 >=,-
						, IIF(ival+"<="+oe, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("<")+" AND "+oop+"="+sqlTo("-")				//41 <,-
						, IIF(ival+">"+ob, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("<=")+" AND "+oop+"="+sqlTo("-")				//42 <=,-
						, IIF(ival+">="+ob, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("-")+" AND "+oop+"="+sqlTo("=")				//43 -,=
						, IIF(ib+"<="+oval+" AND "+ie+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("-")+" AND "+oop+"="+sqlTo(">")				//45 -,>
						, IIF(ie+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("-")+" AND "+oop+"="+sqlTo(">=")				//46 -,>=
						, IIF(ie+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("-")+" AND "+oop+"="+sqlTo("<")				//47 -,<
						, IIF(ib+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("-")+" AND "+oop+"="+sqlTo("<=")				//48 -,<=
						, IIF(ib+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,IIF(
						iop+"="+sqlTo("-")+" AND "+oop+"="+sqlTo("-")				//49 -,-
						, IIF(ib+"<="+ob+" AND "+ie+">="+ob+" OR "+ob+"<="+ib+" AND "+oe+">="+ib, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
						,"("+sqlCmpFalse+")"
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					)
					);
					break;
			}
			break;
		case "VALUE":	// строим запрос исходя из конкретных значений в объекте
			switch(outerMode)
			{
				case "VALUES":	//таблица значений содержит конкретные значения в this.valuesField
					var oval = " "+outerAlias+"."+this.valuesField;
					switch(this.op)
					{
						case "=":
						case "<>":
						case ">=":
						case "<=":
						case ">":
						case "<":
							if(this.startVal!=null)
							{
								sValue = oval + this.op + sqlTo(this.startVal);
							}
							break;
						case "-":
							if(this.startVal!=null && this.endVal!=null)
							{
								sValue = oval + ">="+sqlTo(this.startVal)
								+" AND "+ oval + "<="+sqlTo(this.endVal);
							}
							break;
						default:
							
					}
					break;
				case "RANGES":	// - таблица содержит диапазоны значений(условия)
					var iop = sqlTo(this.get("VALUE", "OP"));
					var ib = sqlTo(this.get("VALUE", "START"));
					var ie = sqlTo(this.get("VALUE", "END"));
					var ival = ib;

					switch(this.op)
					{
						case "=":
							sValue = IIF(
								oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
													//в зависимости от режима
								, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
								,IIF(
								oop+"="+sqlTo("=")				//1 =,=
								,IIF(ival+"="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("<>")				//2 =,<>
								,IIF(ival+"<>"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo(">")				//3 =,>
								,IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo(">=")				//4 =,>=
								,IIF(ival+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("<")				//5 =,<
								,IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("<=")				//6 =,<=
								,IIF(ival+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("-")				//37 =,-
								, IIF(ival+">="+ob+" AND "+ival+"<="+oe, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,"("+sqlCmpFalse+")"
							)
							)
							)
							)
							)
							)
							)
							)
							
							break;
						case "<>":
							sValue = IIF(
								oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
													//в зависимости от режима
								, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
								,IIF(
								oop+"="+sqlTo("=")				//7 <>,=
								+" OR "+oop+"="+sqlTo(">=")	//10 <>,>=
								+" OR "+oop+"="+sqlTo("<=")	//12 <>,<=
								, IIF(ival+"<>"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (2)
								,IIF(
								oop+"="+sqlTo("<>")				//8 <>,<>
								+" OR "+oop+"="+sqlTo(">")	//9 <>,>
								+" OR "+oop+"="+sqlTo("<")	//11 <>,<
								+" OR "+oop+"="+sqlTo("-")	//38 <>,-
								,"("+sqlCmpTrue+")"		// (3)
								,"("+sqlCmpFalse+")"
							)
							)
							);
							break;
						case ">":
							sValue = IIF(
								oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
													//в зависимости от режима
								, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
								,IIF(
								oop+"="+sqlTo("<")				//17 >,<
								, IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
								,IIF(
								oop+"="+sqlTo("<>")				//14 >,<>
								+" OR "+oop+"="+sqlTo(">")		//15 >,>
								+" OR "+oop+"="+sqlTo(">=")	//16 >,>=
								,"("+sqlCmpTrue+")"		// (3)
								,IIF(
								oop+"="+sqlTo("=")				//13 >,=
								+" OR "+oop+"="+sqlTo("<=")	//18 >,<=
								, IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("-")				//39 >,-
								, IIF(ival+"<"+oe, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,"("+sqlCmpFalse+")"
							)
							)
							)
							)
							);
							break;
						case ">=":
							sValue = IIF(
								oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
													//в зависимости от режима
								, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
								,IIF(
								oop+"="+sqlTo("<")				//23 >=,<
								, IIF(ival+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
								,IIF(
								oop+"="+sqlTo("<=")	//24 >=,<=
								, IIF(ival+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
								,IIF(
								oop+"="+sqlTo("<>")				//20 >=,<>
								+" OR "+oop+"="+sqlTo(">")	//21 >=,>
								+" OR "+oop+"="+sqlTo(">=")	//22 >=,>=
								,"("+sqlCmpTrue+")"		// (3)
								,IIF(
								oop+"="+sqlTo("=")				//19 >=,=
								, IIF(ival+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("-")				//40 >=,-
								, IIF(ival+"<="+oe, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,"("+sqlCmpFalse+")"
							)
							)
							)
							)
							)
							);
							break;
						case "<":
							sValue = IIF(
								oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
													//в зависимости от режима
								, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
								,IIF(
								oop+"="+sqlTo(">")				//27 <,>
								, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
								,IIF(
								oop+"="+sqlTo(">=")	//28 <,>=
								, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
								,IIF(
								oop+"="+sqlTo("<>")				//26 <,<>
								+" OR "+oop+"="+sqlTo("<")		//29 <,<
								+" OR "+oop+"="+sqlTo("<=")	//30 <,<=
								,"("+sqlCmpTrue+")"		// (3)
								,IIF(
								oop+"="+sqlTo("=")				//25 <,=
								, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("-")				//41 <,-
								, IIF(ival+">"+ob, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,"("+sqlCmpFalse+")"
							)
							)
							)
							)
							)
							);
							break;
						case "<=":
							sValue = IIF(
								oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
													//в зависимости от режима
								, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
								,IIF(
								oop+"="+sqlTo(">")				//33 <=,>
								, IIF(ival+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
								,IIF(
								oop+"="+sqlTo(">=")	//34 <=,>=
								, IIF(ival+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")	// (1)
								,IIF(
								oop+"="+sqlTo("<>")	//32 <=,<>
								+" OR "+oop+"="+sqlTo("<")	//35 <=,<
								+" OR "+oop+"="+sqlTo("<=")	//36 <=,<=
								,"("+sqlCmpTrue+")"		// (3)
								,IIF(
								oop+"="+sqlTo("=")				//31 <=,=
								, IIF(ival+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("-")				//42 <=,-
								, IIF(ival+">="+ob, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,"("+sqlCmpFalse+")"
							)
							)
							)
							)
							)
							);
							break;
						case "-":
							sValue = IIF(
								oop+"="+sqlTo("")	//если в поле условия ничего нет, то просто вернем true либо false
													//в зависимости от режима
								, (this.mode4EmptyValue = this.MODE_4_EMPTY_VALUE_TRUE ? ("("+sqlCmpTrue+")") : ("("+sqlCmpFalse+")"))
								,IIF(
								oop+"="+sqlTo("=")				//43 -,=
								, IIF(ib+"<="+oval+" AND "+ie+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("<>")	//44 -,<>
								,"("+sqlCmpTrue+")"		// (3)
								,IIF(
								oop+"="+sqlTo(">")				//45 -,>
								, IIF(ie+">"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo(">=")				//46 -,>=
								, IIF(ie+">="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("<")				//47 -,<
								, IIF(ib+"<"+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("<=")				//48 -,<=
								, IIF(ib+"<="+oval, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,IIF(
								oop+"="+sqlTo("-")				//49 -,-
								, IIF(ib+"<="+ob+" AND "+ie+">="+ob+" OR "+ob+"<="+ib+" AND "+oe+">="+ib, "("+sqlCmpTrue+")", "("+sqlCmpFalse+")")
								,"("+sqlCmpFalse+")"
							)
							)
							)
							)
							)
							)
							)
							);
							break;
					}
					break;
			}
			break;
	}
	//Теперь обрамим в скобки
	if(sValue!="")
	{
		sValue = "("+sValue+")";
		if(toAddAnd)
		{
			sValue = " AND "+sValue;
		}
	}
	return sValue;
}