/**
	Описание объекта DpShtat (выбор сотрудника из штатного расписания)
	© Дебет Плюс
	Автор: Ганзенко Леонид
	Редактор: Степанчук Д.В. (draco)
	
	DpShtat(mode, bdat, edat)
		mode - режим
			"NOTSHTID" - режим при котором в проверке валидности не учитывается код ШР
		bdat - дата начала периода
		edat - дата конца периода

	Методы для работы с объектом
		linkCtrl(ind, oCtrl)	- связать контрола с объектом
			ind параметр может принимать такие значения
			"fcod" - табельный номер
			"o.f..." - поле из таблицы подразделений
			"d.f..." - поле из таблицы должностей
			"dcl.f..." - поле из справочника аналитики (должности)
			"w.f..." - поле из таблицы штатного расписания
			"wcl.f..." - поле из справочника аналитики (персонал)
		updateCtrls 			- обновить связаные контролы
		zaval					- выбор сотрудника из списка
		setPeriod(bdat, edat)	- установка периода
		setDate("инд", dat)		- установка дат
		getDate("инд")			- получить дату
		setMode(mode)			- установка режима
		getMode					- получить режим
		validCod				- валидность таб.номера
		
	Функции для работы со штатным расписанием 
		setFwid - установить индетификатор из таблицы штатного расписания (загрузка сотрудника)
		getFwid - получить индентификатор из таблицы штатного расписания

	Функции для работы с текущим сотрудником
		getEmployeCod		- получить табельный номер
		getEmployeSubDiv    - получить код подразделения
		getEmployeFwid		- получить индентификатор
		getEmployeShtatID	- получить код штатного расписания
		getEmployeSurname	- получить фамилию, имя, отчество
		
		
		// лучше не использовать от греха подальше )
		setEmployeCod		- установить табельный номер сотрудника
		setEmployeFwid		- установить индетификатор сотрудника
*/

function DpShtat(mode, bdat, edat)
{
	this.valid = false;
	this.arrCtrls = new Array();
	this.mode = mode;
	this.linkCtrl = DpShtat_linkCtrl;
	this.updateCtrls = DpShtat_updateCtrls;
	this.validCtrl = DpShtat_validCtrl;
	this.validCod = DpShtat_validCod;
	this.zaval = DpShtat_zaval;
	this.bdat = bdat;
	this.edat = edat;
	this.setFwid = DpShtat_setFwid;
	this.getFwid = DpShtat_getFwid;
	this.setFshpz = DpShtat_setFshpz;	
	this.getFshpz = DpShtat_getFshpz;
	this.setCalend = DpShtat_setCalend;
	this.getCalend = DpShtat_getCalend;	
	this.fwid = 0;
	this.cod = 0;
	this.tabnorg = 0;
	this.ftabn_id = 0;
	this.subDiv = 0;
	this.calend = 0;
	this.zavalInAllShtat = false; //если true то завал всегда будет отображеть все записи штатного
	this.setPeriod = DpShtat_setPeriod;
	this.setDate = DpShtat_setDate;
	this.getDate = DpShtat_getDate;
	this.setEmployeCod = DpShtat_setEmployeCod;
	this.setCod = DpShtat_setEmployeCod;
	this.getEmployeCod = DpShtat_getEmployeCod;
	this.getCod = DpShtat_getEmployeCod;
	this.getEmployeFwid = DpShtat_getEmployeFwid;
	this.getEmployeShtatID = DpShtat_getEmployeShtatID;
	this.getEmployeSurname = DpShtat_getEmployeSurname;
	this.getEmployeSubDiv = DpShtat_getEmployeSubDiv;
	this.resetPar = DpShtat_resetPar;
	
	return this;
}

function DpShtat_linkCtrl(ind, oCtrl)
{	
	ind = ind.toLowerCase();
	if (this.arrCtrls[ind] != null)
		return;
	this.arrCtrls[ind] = oCtrl;
	if ("fcod" == ind)
	{
		if (typeOf(this.arrCtrls[ind]) == "DpTextEx")
		{
			this.arrCtrls[ind].mask = "#######";
			this.arrCtrls[ind].isButton = true;
			setButton(this.arrCtrls[ind], this, "zaval");
			setValid(this.arrCtrls[ind], this, "validCtrl");
		}
	}
	else
	{
		this.arrCtrls[ind].setEnabled(false);
	}
}

function DpShtat_updateCtrls()
{
	var strFields = "";
	for (var ind in this.arrCtrls)
	{
		if ("fcod" == ind)
		{
			//если табельные номера в ШР ведутся в разрезе подразделений, тогда выводим табельный номер по подразделению
			if(getPar("TABNORG"))
				ind = "w.ftabnorg";
			else
				ind = "wr.fcod";
		}
			
		if (strFields == "")
			strFields += ind;
		else
			strFields += (", " + ind);
	}
	
	var codDol = getPar("CODDOL");
	if(strFields != "")
	{
		var strSQL = "SELECT " + strFields 
				+ " FROM (((((^work_pers AS w INNER JOIN ^cl AS wcl ON w.fwid_cl=wcl.fwid)"
				+ " INNER JOIN ^clrelclcl AS wr ON w.fwid_cl=wr.fwid_cl)"
				+ " INNER JOIN ^stru_dol AS d ON w.fwid_dol=d.fwid)"
				+ " INNER JOIN ^clrelclcl AS dr ON d.fcod = dr.fcod)"
				+ " INNER JOIN ^cl AS dcl ON dr.fwid_cl=dcl.fwid)"
				+ " INNER JOIN ^stru_org AS o ON d.fwid_org=o.fwid"
				+ " WHERE dr.fcl=" + sqlTo(codDol) + " AND w.fwid=" + sqlTo(this.fwid);
				
		var snap = getSnap(strSQL);
		if (snap != null)
		{
			var i = 0;
			for(var ind in this.arrCtrls)
				this.arrCtrls[ind].value = snap[i++];
		}
		else
		{
			for(var ind in this.arrCtrls)
				this.arrCtrls[ind].value = "";
		}
	}
}

function DpShtat_validCtrl()
{
	var result = false;
	switch (this.mode)
	{
		case "TN":
			var cod = this.cod = this.getEmployeCod();
		break;	
		default:
			var cod = this.arrCtrls["fcod"].value;
		break;
	}
	
	if(cod==0 && this.mode && this.mode.indexOf("0")!=-1)
	{
		this.setFwid(0);
		return true;
	}
	
	this.subDiv = 0;
	
	var codPers = getPar("CODPERS");
	if(!codPers)
		alert(ru("В настройках параметров не указан код справочника персонала!","В налагодженні параметрів відсутній код довідника персоналу!"));
		
	var tmp = " FROM ^work_pers AS w INNER JOIN ^clrelclcl AS r ON w.fwid_cl = r.fwid_cl INNER JOIN ^stru_dol d on w.fwid_dol=d.fwid"
			+ " WHERE "
			+ (getPar("TABNORG")?"w.Ftabnorg="+cod:" r.fcod="+cod)
			+ " AND r.fcl=" + sqlTo(codPers)
			+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);	
			
	var strSQL = "SELECT w.fwid AS fwid, w.fwid_cl AS ftabn_id, d.fshpz, w.Fcalend " + tmp;
	
	var table = OpenTable("m", strSQL);
	var Ok = !table.IsEmpty();	
	if (Ok)
	{
		for(; !table.IsEOF(); table.MoveNext())
		{
			if (this.getFwid() == Number(table.getValue("fwid")))
			{				
				this.setFwid(this.getFwid());
				this.ftabn_id = table.getValue("ftabn_id");
		        this.setFshpz(table.getValue("fshpz"));
		        this.setCalend(table.getValue("Fcalend"));
				return true;
			}
		}
		table.MoveFirst();
		if (!result && table.getRecordCount() == 1)
		{
			this.setFwid(Number(table.getValue("fwid")));
			this.ftabn_id = table.getValue("ftabn_id");
		    this.setFshpz(table.getValue("fshpz"));
		    this.setCalend(table.getValue("Fcalend"));			
			return true;
		}
		else
		{
			//если стоит режим не учитывать код ШР, тогда не заваливаемся в ШР
			if(this.mode!=null && this.mode.toUpperCase().indexOf("NOTSHTID")!=-1)
			{
				//берём первую попавшуюся запись
				this.setFwid(Number(table.getValue("fwid")));
				this.ftabn_id = table.getValue("ftabn_id");
		    	this.setFshpz(table.getValue("fshpz"));
		    	this.setCalend(table.getValue("Fcalend"));
				return true;
			}
			else
			{
		    	this.ftabn_id = table.getValue("ftabn_id");
				this.setFshpz(table.getValue("fshpz"));
		    	this.setCalend(table.getValue("Fcalend"));
				return this.zaval(table.getValue("ftabn_id"));
			}
		}
	}
	table.close();
	table = null;
	/*
	var strSQL = "SELECT COUNT(*) " + tmp;
	var snap = getSnap(strSQL);
		
	if (snap != null && snap[0] == 1)
	{
		strSQL = "SELECT w.fwid " + tmp;
		snap = getSnap(strSQL);
		this.setFwid(snap[0]);
		result = true;
	}
	*/		
	if (this.mode == "TN")
	{
		return Ok;
	}
	else
	{
		return this.zaval();
	}
}

function DpShtat_zaval(wid_cl)
{
	par = new Object();
	if (wid_cl != null && !this.zavalInAllShtat)
	{
		if(typeOf(wid_cl)=="Text")
		{
			if(isNaN(wid_cl.text))
				par.widcl = 0;
			else
				par.widcl = Number(wid_cl.text);
		}
		else
			par.wid_cl = Number(wid_cl);
	}
	else
	{
		//если выбраный таб.номер есть в ШР, то при завале показываем только эти таб.номера
		if(this.validCod() && !this.zavalInAllShtat)
			par.wid_cl = Number(this.ftabn_id);
	}
	par.userRet = true;
	par.escape  = true;
	par.dat1 = this.bdat;
	par.dat2 = this.edat;	
	par.ret = new Object();
	par.ret.fwid = 0;
	//showWindow("l_shtat.xml", SW_MODAL, par);
	showWindow("l_shtatRas.xml", SW_MODAL, par);
	if (par.escape == false)
	{
		this.setFwid(Number(par.ret.fwid));
		this.valid = true;
	}	
	this.updateCtrls();
	return !par.escape;
}

function DpShtat_validCod()
{
	var cod = this.arrCtrls["fcod"].value;
	var codPers = getPar("CODPERS");
	if(!codPers)
		alert(ru("В настройках параметров не указан код справочника персонала!","В налагодженні параметрів відсутній код довідника персоналу!"));
		
	var tmp = " FROM ^work_pers AS w INNER JOIN ^clrelclcl AS r ON w.fwid_cl = r.fwid_cl"
			+ " WHERE "
			+ (getPar("TABNORG")?"w.Ftabnorg="+cod:" r.fcod="+cod)
			+ " AND r.fcl=" + sqlTo(codPers)			
			+ " AND w.fbdat <= " + sqlTo(this.edat) 
			+ " AND w.fedat >= " + sqlTo(this.bdat);	
			
	var strSQL = "SELECT DISTINCT w.fwid_cl " + tmp;
	var snap = getSnap(strSQL);
	if(snap!=null)
	{
		this.ftabn_id = snap[0];
		return true;
	}
	else
		return false;
}

function DpShtat_setFwid(fwid)
{
	if (fwid == null || fwid == this.fwid)
		return;
	
	var strSQL = "SELECT fwid, fwid_cl FROM ^work_pers"
				+ " WHERE fwid=" + sqlTo(fwid) 
					+ " AND fbdat<=" + sqlTo(this.edat) 
					+ " AND fedat>=" + sqlTo(this.bdat);				

	var snap = getSnap(strSQL);

	if (snap != null)
	{
		this.valid = true;
		this.fwid = snap[0];				
		this.ftabn_id = snap[1];			
		this.subDiv = this.getEmployeSubDiv();
		this.cod = this.getEmployeCod();
	}
	else
	{
		this.valid = true;
		this.resetPar();
	}
	this.updateCtrls();
}

function DpShtat_getFwid()
{
	return this.fwid;
}

function DpShtat_setPeriod(bdat, edat)
{
	this.bdat = bdat;
	this.edat = edat;
}

function DpShtat_setDate(ind, dat)
{
	eval("this." + ind + "=dat");
}

function DpShtat_getDate(ind)
{
	return eval("this." + ind);
}

function DpShtat_getEmployeSubDiv()
{
	var result = 0;
	var codDol = getPar("CODDOL");
	var strSQL = "SELECT d.fwid_org"
			+ " FROM (^work_pers AS w INNER JOIN ^stru_dol AS d ON w.fwid_dol=d.fwid)"
			+ " INNER JOIN ^clrelclcl AS dr ON d.fcod = dr.fcod"
			+ " WHERE dr.fcl=" + sqlTo(codDol) + " AND w.fwid=" + sqlTo(this.fwid);
	var snap = getSnap(strSQL);
	if (snap != null)
			result = snap[0];
	return result;
}

// Установить шифр произв.затрат
function DpShtat_setFshpz(cod)
{
   this.shpz = cod;
}

// Получить шифр произв.затрат
function DpShtat_getFshpz()
{
    return this.shpz;
}

//Установить номер календаря
function DpShtat_setCalend(calNum)
{
	this.calend = calNum;
}

//Получить номер календаря
function DpShtat_getCalend()
{
	return this.calend;
}


function DpShtat_setEmployeCod(cod)
{	
	var strSQL = "SELECT COUNT(*)"
			+ " FROM ^work_pers AS w INNER JOIN ^clrelclcl AS r ON w.fwid_cl=r.fwid_cl"
			+ " WHERE r.fcod=" + sqlTo(cod)
			+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);
	var snap = getSnap(strSQL);
	
	//при исп. функций count, sum и т.д. snap никогда не будет равен null
	if(snap[0] > 0)
	{		
		if(1==snap[0])
		{
			strSQL = "SELECT w.fwid"
				+ " FROM ^work_pers AS w INNER JOIN ^clrelclcl AS r ON w.fwid_cl=r.fwid_cl"
				+ " WHERE r.fcod=" + sqlTo(cod)
				+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);
			snap = getSnap(strSQL);
			this.fwid = snap[0];
			this.valid = true;
		}
		else if(1<snap[0])
		{
			if(this.mode.toUpperCase().indexOf("NOTSHTID")==-1)
			{
				strSQL = "SELECT DISTINCT w.fwid_cl"
					+ " FROM ^work_pers AS w INNER JOIN ^clrelclcl AS r ON w.fwid_cl=r.fwid_cl"
					+ " WHERE r.fcod=" + sqlTo(cod)
					+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);
				snap = getSnap(strSQL);
				this.zaval(snap[0]);
			}
			else
			{
				strSQL = "SELECT MAX(w.fwid)"
					+ " FROM ^work_pers AS w INNER JOIN ^clrelclcl AS r ON w.fwid_cl=r.fwid_cl"
					+ " WHERE r.fcod=" + sqlTo(cod)
					+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);
				snap = getSnap(strSQL);
				this.fwid = snap[0];
				this.valid = true;
			}
		}
	}
	else
	{
		if(this.mode == "TN")
		{
			this.resetPar();
			return;
		}
	}
	
	this.updateCtrls();
}

function DpShtat_getEmployeCod()
{
	var result = 0;	
	if (this.valid)
	{
		var strSQL = "SELECT r.fcod"
			+ " FROM ^work_pers AS w INNER JOIN ^clrelclcl AS r ON w.fwid_cl = r.fwid_cl"
			+ " WHERE w.fwid=" + sqlTo(this.fwid);
		var snap = getSnap(strSQL);
		if (snap != null)
			result = snap[0];
	}
	return result;
}

function DpShtat_setEmployeFwid(wid_cl)
{
	var strSQL = "SELECT COUNT(*)"
			+ " FROM ^work_pers AS w"
			+ " WHERE w.fwid_cl=" + sqlTo(wid_cl)
			+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);
	var snap = getSnap(strSQL);
	if(null!=snap)
	{
		if(1==snap[0])
		{
			strSQL = "SELECT w.fwid"
				+ " FROM ^work_pers AS w"
				+ " WHERE w.fwid_cl=" + sqlTo(wid_cl)
				+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);
			snap = getSnap(strSQL);
			this.fwid = snap[0];
			this.valid = true;
		}
		else if(1<snap[0])
		{
			if(this.mode.toUpperCase().indexOf("NOTSHTID")==-1)
				this.zaval(wid_cl);
			else
			{
				strSQL = "SELECT MAX(w.fwid) "
					+ " FROM ^work_pers AS w"
					+ " WHERE w.fwid_cl=" + sqlTo(wid_cl)
					+ " AND w.fbdat<=" + sqlTo(this.edat) + " AND w.fedat>=" + sqlTo(this.bdat);
				snap = getSnap(strSQL);
				this.fwid = snap[0];
				this.valid = true;
			}
		}
	}

	this.updateCtrls();
}

function DpShtat_getEmployeFwid()
{
	var result = 0;
	if(this.valid)
	{
		var strSQL = "SELECT w.fwid_cl"
			+ " FROM ^work_pers AS w"
			+ " WHERE w.fwid=" + sqlTo(this.fwid);
		var snap = getSnap(strSQL);
		if(snap!=null)
			result = snap[0];
	}	
	return result;
}

function DpShtat_getEmployeShtatID()
{
	var result = 0;
	if(this.valid)
	{
		var strSQL = "SELECT w.fid"
			+ " FROM ^work_pers AS w"
			+ " WHERE w.fwid=" + sqlTo(this.fwid);
		var snap = getSnap(strSQL);
		if(snap)
			result = snap[0];
	}	
	return result;
}

function DpShtat_getEmployeSurname()
{
	var result = "";
	if(this.valid)
	{
		var strSQL = "SELECT c.ftxt"
			+ " FROM ^work_pers AS w INNER JOIN ^cl AS c ON w.fwid_cl=c.fwid"
			+ " WHERE w.fwid=" + sqlTo(this.fwid);
		var snap = getSnap(strSQL);
		result = snap[0];
	}
	
	return result;
}

function DpShtat_resetPar()
{	
	this.fwid = 0;
	this.cod = 0;
	this.tabnorg = 0;
	this.ftabn_id = 0;
	this.subDiv = 0;
	this.calend = 0;	
	
	this.valid = false;
}