/*										
   Описание объекта "формирования итогов": DpTotals
   
   (C) ДЕБЕТ Плюс, 2000

   Конструктор DpTotals - возвращает this

	Внешние методы:
a)	AddItog(cField, cFunction) - добавление итога
	Параметры:
		cField      - имя поля
		cFunction   - что делать (MIN, MAX, SUM)

b) AddGroup(cField, cItogType, сEmptyVal, cItogID, cCond) - добавление поля группировки
		(итоговые записи создаются по указанным полям)
	Параметры:
		cField    - имя поля (пустое - общий итог)
		cItogType - тип итога 
		(	"-"(умолчание) - нет итога,
			"+" - итог сверху,
			"_" - итог снизу 
		)
		сEmptyVal - пустое значения для поля в кавычках, например "0"	
		cItogID   - значения поля-признака итоговой записи
		cCond     - условие создания итога

с) MkTotals(cMode, cInTable, cNDoc, nItogs, cFCrit, cOutTable, KeyPref ) - расчет итогов
	 Параметры:
		cMode     - режим формирования итогов
		          'S' - не удалять итоги (по умолчанию - удалять);
		          'O' - создавать вспомогательные поля для упорядочения (по умолчанию - не создавать)
		cInTable  - входная таблица (по которой, собственно, формировать итоги)
		cNDoc     - идентификатор итога (пока не используется, но, наверное,
		            будет нужен для запоминания параметров)
		nItogs    - не используется. Количество полей, по которым можно
		            делать итоги
		cFCrit    - имя поля-признака итоговой записи (строковое, у не
		            итоговых пустое). По-умолчанию = "FTT"
		cOutTable - выходная таблица. По-умолчанию берется входная таблица
		KeyPref   - не используется.
		
d)	setOrderBy(fieldCrit) - установка списка полей упорядочения
		fieldCrit - имя поля-признака итоговой записи
		retUsedField - масив полей учавстующих в упорядочивании
	getOrderBy()		 - получение списка полей упорядочения
	
e) ResetTotals - сбрасывает вычисленные ранее итоги
		
	Внутренние свойства:
   .aAgrs			  - массив объектов-итогов
   .aGroups			  - массив объектов-группировки   
	.nAgrsDim		  - к-во элементов в aAgrs
	.nGroupsDim		  - к-во элементов в aGroups
	.OrderBy			  - список полей упорядочения
	.createHelpField - использовать вспомогательные поля упорядочения

*/

include("objects\\DpExtentionBuilder.js");

// Конструктор объекта DpTotals
function DpTotals()
{
	RO("DpTotals", this);
   this.AddItog     = DpTotals_AddItog;
   this.AddGroup    = DpTotals_AddGroup;
   this.IsGroupExists = DpTotals_IsGroupExists;   
   this.MkTotals    = DpTotals_MkTotals;   
   this.setOrderBy  = DpTotals_setOrderBy;
   this.getOrderBy  = DpTotals_getOrderBy;   
   this.ResetTotals = DpTotals_ResetTotals;
   this.getExtArray = DpTotals_getExtArray;
   this.Add2ItogExtMap	= DpTotals_Add2ItogExtMap;
   this.GetFromItogExtMap	= DpTotals_GetFromItogExtMap;
 
	this.aAgrs		= new Array();
	this.aGroups	= new Array();
	this.extAr		= null; // расширения по которым ведется группировка
	this.nAgrsDim	= 0;//длина массива - кол-во объектов-итогов
	this.nGroupsDim = 0;//длина массива - кол-во объектов-группировки

	this.createHelpField = false;
	this.setOrderBy(null);

	/* 
	массив в котором храняться значение переменных при динамическом формировании
	итогов, например: ?xxcl(fst,ext_fst) XCL 0  Расш.спр. аналитики (статья)
	ext_fst - это переменная, и ее значение формируется в "листе", 
	и добавляется в нашу карту m_oItogExtMap["ext_fst"]
	*/
	this.m_oItogExtMap = new Array();
	this.bAddFwid = false;

	return this;
}

function DpTotals_Add2ItogExtMap(label, value)
{
	this.m_oItogExtMap[label] = value;
}

function DpTotals_GetFromItogExtMap(label, value)
{
	return this.m_oItogExtMap[label];
}

function DpTotals_getExtArray()
{
	return this.extAr;
}

//Формирование свойства - упорядочения таблицы
function DpTotals_setOrderBy(fieldCrit, retUsedField, cMode)
{
	if (cMode)
		this.createFttLevel	= (cMode.indexOf("L") != -1);
	if (!retUsedField)
		retUsedField = new Object();
	var cFieldExpr = "";
	var cFieldExprOrd = "";		
	var cItogExprOrd = 0;
	var cOrderBy = "";	
	var num = 0;

   if (typeof(fieldCrit) != "string")
      fieldCrit = "ftt";

	for (var i=0;i<this.nGroupsDim;i++)
    {
		cFieldExpr = this.aGroups[i].ttl_gr_Field;
		if (typeof this.aGroups[i].ttl_gr_cOrderBy != "undefined" && this.aGroups[i].ttl_gr_cOrderBy != "")
			cFieldExpr = this.aGroups[i].ttl_gr_cOrderBy
			
		retUsedField[cFieldExpr] = true;
		cFieldExprOrd = this.aGroups[i].ttl_gr_FieldOrd;
		num = (this.aGroups[i].ttl_gr_ItogType=="+" ? -1 : (this.aGroups[i].ttl_gr_ItogType=="_" ? 1 : 0));
		if (SQL) {
			if(!this.createFttLevel)
				cItogExprOrd = IIF(fieldCrit+"='"+this.aGroups[i].ttl_gr_ItogID+"'" ,sqlTo(num) , "0");
			else
				cItogExprOrd = IIF("FttLevel ="+(i + 1),sqlTo(num),0);
		}
		else {
			if(!this.createFttLevel)
				cItogExprOrd = "IIF("+fieldCrit+"='"+this.aGroups[i].ttl_gr_ItogID+"',"+sqlTo(num)+",0)";
			else
				cItogExprOrd = "IIF(FttLevel ="+(i + 1)+","+sqlTo(num)+",0)";
		}
		if (cFieldExpr!="")
 			cOrderBy = cOrderBy+","+cFieldExpr;
		if (this.createHelpField)	
			cOrderBy = cOrderBy+","+cFieldExprOrd
		else
			cOrderBy = cOrderBy+","+cItogExprOrd;
	}
	cOrderBy = cOrderBy.substr(1);

	this.OrderBy = cOrderBy;
}

function DpTotals_getOrderBy()
{
	return this.OrderBy;
}

function DpTotals_AddItog(cField, cFunction)
{
   for (var nAgr=0;nAgr<this.nAgrsDim;nAgr++)
    {
		if (this.aAgrs[nAgr].ttl_ag_Field.toUpperCase() == cField.toUpperCase())
		{
			return;			
		}
    }

	var n = this.nAgrsDim;
    this.nAgrsDim = n+1;
   	var objAgrs = new Object();
   objAgrs.ttl_ag_Field	= cField;
   objAgrs.ttl_ag_Func  = cFunction;
	this.aAgrs[n] = objAgrs;
}

function DpTotals_IsGroupExists(cField)
{
	var n = this.nGroupsDim;	
	for (j = 0; j < n; j++)
	{
		if (this.aGroups[j].ttl_gr_Field.toUpperCase() == cField.toUpperCase() 
			|| (this.aGroups[j].ttl_gr_Field == "" && cField=="!"))
		{
			return true;
		}
	}
	
	return false;
}

function DpTotals_AddGroup(cField, cItogType, cEmptyVal, cItogID, cCond, cTxt, cOptions, cOrderBy)
{
	var extBuilder;
	var clExtTbl = null;
	var n = this.nGroupsDim;	
	
	//VT Бывает такой хитрый случай, когда после RIGHT JOIN-а в dplist.js(1765)
	// получаем несколько одинаковых полей.Сие неправильно есть.
	//Написал сначала там DISTINCT. Oracle съел очень хорошо, а любимый Access начал крутить
	// носом: то не могу DISTINCT с ORDER BY, то ещё ерунда какая-то. Проще так:
	for (j = 0; j < n; j++)
		if (this.aGroups[j].ttl_gr_Field.toUpperCase() == cField.toUpperCase() || (this.aGroups[j].ttl_gr_Field == "" && cField=="!"))
			return;

	this.nGroupsDim = n+1;			

	if (typeof(cEmptyVal) == "undefined")
		cEmptyVal = "NULL";
	
	//Общий итог должен стоять на первом месте	
	if (cField=="!")
	{
		this.aGroups[n] = new Object();
		var i;
		for (i=n;i>0;i--)
		{
			this.aGroups[i].ttl_gr_Field	  = this.aGroups[i-1].ttl_gr_Field;
			this.aGroups[i].ttl_gr_FieldOrd = this.aGroups[i-1].ttl_gr_FieldOrd;			
			this.aGroups[i].ttl_gr_EmptyVal = this.aGroups[i-1].ttl_gr_EmptyVal;
			this.aGroups[i].ttl_gr_ItogID   = this.aGroups[i-1].ttl_gr_ItogID;
			this.aGroups[i].ttl_gr_Cond	  = this.aGroups[i-1].ttl_gr_Cond;
			this.aGroups[i].ttl_gr_Txt	  = this.aGroups[i-1].ttl_gr_Txt;
			this.aGroups[i].ttl_gr_ItogType = this.aGroups[i-1].ttl_gr_ItogType;		
		}
		n = 0;
		cField="";
	}

	var objGroups = new Object();
	objGroups.ttl_gr_Field	  = cField;
	objGroups.ttl_gr_FieldOrd = cField+"f_Ord";	
	objGroups.ttl_gr_EmptyVal = cEmptyVal;
	objGroups.ttl_gr_ItogID	  = cItogID;
	objGroups.ttl_gr_Cond	  = (cCond!=null?cCond:"");
//-	alert("ag-ttl_gr_Cond=" + objGroups.ttl_gr_Cond==null);
	objGroups.ttl_gr_Txt	  = cTxt;
	objGroups.ttl_gr_cOptions = cOptions;
	objGroups.ttl_gr_cOrderBy = cOrderBy;

	if(objGroups.ttl_gr_ItogID == 'XCL')
	{	
		strCL = getFCL_value(objGroups.ttl_gr_Field);		
		var sLabel = getLabel(objGroups.ttl_gr_Field);
		extBuilder = new DpExtentionBuilder("CL", strCL);
		objGroups.ttl_gr_Txt += ": "+ extBuilder.getAttr(sLabel, "TXT");		
	}

	if(objGroups.ttl_gr_ItogID == 'XN')
	{			
		var sLabel = objGroups.ttl_gr_Field.substring(5);
		extBuilder = new DpExtentionBuilder("CL_NMK");
		var txt = extBuilder.getAttr(sLabel, "TXT");		
		if(txt)
			objGroups.ttl_gr_Txt += ": "+ txt;
		else
			objGroups.ttl_gr_Txt += ": "+ objGroups.ttl_gr_Field.toUpperCase();		
	}	

	if(objGroups.ttl_gr_ItogID == 'XP')
	{		
		var sLabel = objGroups.ttl_gr_Field.substring(5);
		extBuilder = new DpExtentionBuilder("H", "PN");
		var txt = extBuilder.getAttr(sLabel, "TXT");
		if(!txt)
			txt = extBuilder.getAttr(sLabel, "TXT", "H.SA");	
		if(txt)
			objGroups.ttl_gr_Txt += ": "+ txt;
		else
			objGroups.ttl_gr_Txt += ": "+ objGroups.ttl_gr_Field.toUpperCase();
	}	
	
	if (typeof(cItogType) == "string")
	{
		switch (cItogType)
		{
			case "+":
			case "_":
			case "-":
				objGroups.ttl_gr_ItogType = cItogType;
				break;
			default:
				objGroups.ttl_gr_ItogType = "-";
				break;
		}		
	}		
	this.aGroups[n] = objGroups;
}

function DpTotals_MkTotals(cMode, cInTable, cNDoc, nItogs, cFCrit, cOutTable, KeyPref)
{
   if (typeof(cMode) != "string")
		cMode = ""
   else
      cMode = cMode.toUpperCase();
	this.createHelpField	= (cMode.indexOf("O") != -1);
	this.createFttLevel		= (cMode.indexOf("L") != -1);
   if (typeof(cFCrit) != "string")
      cFCrit = "ftt";
	var cTmpTable = getTmpTableName(null, true);
  var equInOut;// = cOutTable == cInTable;
   //Если не задана выходная таблица - итоги создаються во входной таблице
 
  	// удаляем алиас (если есть)  	
  	//if (cInTable.indexOf(" ") != -1)
  	//	cInTable = cInTable.left(cInTable.indexOf(" "));
  	
   if (typeof(cOutTable) != "string")
   {
      cOutTable = cInTable;
      equInOut = true;
   }
   else
   {
		if (String(cInTable).substr(0, 6).toUpperCase() == "SELECT")
		{
			var sFld = "*";
			//var bFtt = lpProxy.getDBManager().testSQL("SELECT FTTLEVEL FROM "+ cInTable +" WHERE 1<>1")
			//if(!bFtt && this.createFttLevel)
			//	sFld += ", 0 as FttLevel";
			//ExecuteSQL("SELECT "+ sFld +" INTO "+ cTmpTable 
			//			+" FROM "+ cInTable);

			strSQL = cInTable;
			var strCheck = strSQL;
			var nIndex = strCheck.toUpperCase().lastIndexOf(" ORDER ");
			if (nIndex != -1)
				strSQL = strSQL.substr(0, nIndex);
			
			// create view
			var viewName = getTmpTableName(null, true);
			ExecuteSQL("CREATE VIEW " + viewName + " AS " + strSQL);							
			
			// fill view
			ExecuteSQL("SELECT * INTO " + cTmpTable + " FROM " + viewName);
			
			// drop view
			ExecuteSQL("DROP VIEW " + viewName);							
		}
		else
		{
			var sFld = cInTable +".*";
			var bFtt = lpProxy.getDBManager().testSQL("SELECT FTTLEVEL FROM "+ cInTable +" WHERE 1<>1")
			if(!bFtt && this.createFttLevel)
				sFld += ", 0 as FttLevel";
			ExecuteSQL("SELECT "+ sFld +" INTO "+ cTmpTable 
						+" FROM "+ cInTable);
		}
		
		//Добавляем поле, если нет 
		if (ExecuteSQL("ALTER TABLE "+cTmpTable+
					" ADD "+cFCrit+" "+SqlType("TEXT", 20),false))
		{
			ExecuteSQL("UPDATE "+cTmpTable+
					" SET "+cFCrit+"=' '");
		}

		/*if (this.createFttLevel)
		{
			if (ExecuteSQL("ALTER TABLE "+cTmpTable+
						" ADD FttLevel INTEGER", false))
				ExecuteSQL("UPDATE "+cTmpTable+
							" SET FttLevel=0");
		}*/
		cInTable = cTmpTable;
   }
		//Формируем масив расширений
	this.extAr = new Array();
	var nmkExtWhere = "";
	var ppnExtWhere = "";
	
	for (var ind = 0; ind < this.nGroupsDim; ind++)
	{
		if (this.aGroups[ind].ttl_gr_Field.indexOf("x") == 0)// Это расширение
		{
			var ext			= new Object();
			ext.label		= this.aGroups[ind].ttl_gr_Field.substring(5);
			ext.vid			= this.aGroups[ind].ttl_gr_Field.substring(0, 4);
			ext.indInGroups = ind;
			ext.no			= this.aGroups[ind].ttl_gr_FieldOrd;
			ext.type		= "INT";//по умолчанию (фактически расширений может уже и не быть, но поля добавлять надо)
			ext.fwid		= "";
			
			if (ext.vid == "xnmk")
				nmkExtWhere += ((nmkExtWhere == "")?"": " OR ") + " a.FLABEL = " + sqlTo(ext.label);
			else if (ext.vid == "xxcl")
			{
				include("sub\\mtr.js");
				ext.fcl		= getFCL_value(this.aGroups[ind].ttl_gr_Field);
				ext.label	= getLabel(this.aGroups[ind].ttl_gr_Field);
				ext.field	= getXCLField(this.aGroups[ind].ttl_gr_Field);
				ext.clWhere	= " a.FLABEL = " + sqlTo(ext.label);
			}
			else 
				ppnExtWhere += ((ppnExtWhere == "")?"": " OR ") + " a.FLABEL = " + sqlTo(ext.label);

			this.extAr[this.extAr.length] = ext;
		}
	}
	var ppnExtName	= getUniqName();
	var nmkExtName	= getUniqName();
	var clExtName	= getUniqName();
   if (this.extAr.length != 0)// Если есть расширения 
   {	
		var all_extTable= null;
		//Выбираем их
		for (i = 0; i < this.extAr.length; i++)
		{
			var extTable;
			var defValue = null;
			if (this.extAr[i].vid == "xnmk")
			{
				var sLabel = this.extAr[i].label;
				extBuilder = new DpExtentionBuilder("CL_NMK");
				var extType = String(extBuilder.getAttr(sLabel, "type"));
				var type = extType; 
				if(inList(extType.charAt(0), "A", "R"))
					type = "INT";
				if(extType.charAt(0) == "V")
					type = "TEXT";
				this.extAr[i].type = type;
				this.extAr[i].extType = extType
				defValue = extBuilder.getAttr(sLabel, "default");
				extTable = extBuilder.getValueTableName();
			}
			else if (this.extAr[i].vid == "xppn")
			{
				var sLabel = this.extAr[i].label;
				extBuilder = new DpExtentionBuilder("H", "PN");
				var extType = String(extBuilder.getAttr(sLabel, "type"));
				if(!extType)
					extType = String(extBuilder.getAttr(sLabel, "type", "H.SA"));				
				var type = extType; 
				if(inList(extType.charAt(0), "A", "R"))
					type = "INT";
				if(extType.charAt(0) == "V")
					type = "TEXT";
				this.extAr[i].type = type;
				this.extAr[i].extType = extType				
				defValue = extBuilder.getAttr(sLabel, "default");
				extTable = extBuilder.getValueTableName();				
			}
			else if (this.extAr[i].vid == "xxcl")
			{
				var sLabel = this.extAr[i].label;
				extBuilder = new DpExtentionBuilder("CL", this.extAr[i].fcl);
				var extType = String(extBuilder.getAttr(sLabel, "type"));
				var type = extType; 
				if(inList(extType.charAt(0), "A", "R"))
					type = "INT";
				if(extType.charAt(0) == "V")
					type = "TEXT";
				this.extAr[i].type = type;
				this.extAr[i].extType = extType		
				defValue = extBuilder.getAttr(sLabel, "default");
				extTable = extBuilder.getValueTableName();				
			}
			
			// добавим к входной и выходной таблице 
			if (this.extAr[i].vid == "xxcl")
			{
				fieldTXT = this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + '_TXT'; 
				ExecuteSQL("ALTER TABLE " + cInTable + " DROP COLUMN " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field, false);
				var bRet = ExecuteSQL("ALTER TABLE " + cInTable + " ADD " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " " + SqlType(this.extAr[i].type), false); 
//				if (bRet && this.extAr[i].fwid)
				if (bRet)
				{
					//Переносим значения расширений
					include ("Objects\\DpExtentionBuilder.js");
					var oExt = new DpExtentionBuilder("CL", String(this.extAr[i].fcl), null);
					var extTblName = oExt.getValueTableName();
					
					var sSQL = "UPDATE "+ cInTable
						+" SET "+ this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field
							+" = "+ "(select "+First("(b.F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//							+" = "+ "(select max(b.F"+ this.extAr[i].label +") "
								+" from ^CLRELCLCL c LEFT JOIN "+ extTblName +" b on c.fwid_cl =b.fmainwid "
									+ " and c.fcl= " + ext.fcl
								+" where c.fcod= "+ cInTable + "." + this.extAr[i].field + " AND b.FMAINWID is not null"  
								+")";
						+ " WHERE (select "+First("(b.F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//						+ " WHERE (select max(b.F"+ this.extAr[i].label +") "
								+" from ^CLRELCLCL c LEFT JOIN "+ extTblName +" b on c.fwid_cl =b.fmainwid "
									+ " AND c.fcl= " + ext.fcl
								+" where c.fcod= "+ cInTable + "." + this.extAr[i].field + " and b.FMAINWID is not null"  
								+")	is not null"										
					ExecuteSQL(sSQL);
					
/*					
					var cExtValue = getUniqName();
					
					ExecuteSQL("SELECT b.F" + this.extAr[i].type + ", relcl.FCOD AS FCOD INTO " + cExtValue
						+ " FROM ^CLRELCLCL relcl "
						+ " LEFT JOIN " + extTable + " b ON relcl.fwid_cl= b.FMAINWID "
						+ " WHERE b.FWID = " + sqlTo(this.extAr[i].fwid) + " AND relcl.FCL="+Number(this.extAr[i].fcl)
						);

					ExecuteSQL("UPDATE " + cInTable + " a " 
						+ " LEFT JOIN " + cExtValue + " x"
							+ " ON a."+this.extAr[i].field+" = x.FCOD"
						+ " SET a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = x.F" + this.extAr[i].type
						);
<<<<<<< DpTotals.js

					// Переносим значения по умолчанию
=======
					
//					ExecuteSQL("UPDATE (" + cInTable + " a " 
//						+ " LEFT JOIN ^CLRELCLCL relcl "
//							+ " ON (a."+this.extAr[i].field+" = relcl.FCOD and relcl.FCL="+Number(this.extAr[i].fcl) + " )) "
//						+ " LEFT JOIN " + extTable + " b ON relcl.fwid_cl= b.FMAINWID "
//						+ " SET a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = b.F" + this.extAr[i].type
//						+ " WHERE b.FWID = " + sqlTo(this.extAr[i].fwid)
//						);

						// Переносим значения по умолчанию
>>>>>>> 1.15
					ExecuteSQL("UPDATE " + cInTable 
						+ " SET " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = " + sqlTo(defValue)
						+ " WHERE " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " is null"
<<<<<<< DpTotals.js
						);			
				}				
=======
						);
*/						 			
				}
				if (!equInOut)
				{
					ExecuteSQL("ALTER TABLE " + cOutTable + " DROP COLUMN " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field, false);
					var bRet = ExecuteSQL("ALTER TABLE " + cOutTable + " ADD " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " " + SqlType(this.extAr[i].type), false)
	//				if (bRet && this.extAr[i].fwid)
					if (bRet)
					{
						//Переносим значения расширений 
						include ("Objects\\DpExtentionBuilder.js");
						var oExt = new DpExtentionBuilder("CL", String(this.extAr[i].fcl), null);
						var extTblName = oExt.getValueTableName();
						
						var sSQL = "UPDATE "+ cOutTable + " "
							+" SET "+ this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field
								+" = "+ "(select "+First("(b.F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//								+" = "+ "(select max(b.F"+ this.extAr[i].label +") "
									+" from ^CLRELCLCL c LEFT JOIN "+ extTblName +" b on c.fwid_cl =b.fmainwid "
										+ " AND c.fcl= " + ext.fcl
									+" where c.fcod= " + this.extAr[i].field + " and b.FMAINWID is not null"  
									+")";
							+ " WHERE (select "+First("(b.F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//							+ " WHERE (select max(b.F"+ this.extAr[i].label +") "
									+" from ^CLRELCLCL c LEFT JOIN "+ extTblName +" b on c.fwid_cl =b.fmainwid "
										+ " AND c.fcl= " + ext.fcl
									+" where c.fcod= " + this.extAr[i].field + " and b.FMAINWID is not null"  
									+")	is not null"										
						ExecuteSQL(sSQL);
/*
						var cExtValue = getUniqName();
						
						ExecuteSQL("SELECT b.F" + this.extAr[i].type + ", relcl.FCOD AS FCOD INTO " + cExtValue
							+ " FROM ^CLRELCLCL relcl "
							+ " LEFT JOIN " + extTable + " b ON relcl.fwid_cl= b.FMAINWID "
							+ " WHERE b.FWID = " + sqlTo(this.extAr[i].fwid) + " AND relcl.FCL="+Number(this.extAr[i].fcl)
							);

						ExecuteSQL("UPDATE " + cOutTable + " a " 
							+ " LEFT JOIN " + cExtValue + " x"
								+ " ON a."+this.extAr[i].field+" = x.FCOD"
							+ " SET a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = x.F" + this.extAr[i].type
							);
						
						//Переносим значения расширений 
//						ExecuteSQL("UPDATE (" + cOutTable + " a " 
//							+ " LEFT JOIN ^CLRELCLCL relcl "
//								+ " ON (a."+this.extAr[i].field+" = relcl.FCOD and relcl.FCL="+Number(this.extAr[i].fcl) + " ))"
//							+ " LEFT JOIN " + extTable + " b ON relcl.fwid_cl = b.FMAINWID "
//							+ " SET a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = b.F" + this.extAr[i].type
//							+ " WHERE b.FWID = " + sqlTo(this.extAr[i].fwid)
//							);

							// Переносим значения по умолчанию
						ExecuteSQL("UPDATE " + cOutTable 
							+ " SET " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = " + sqlTo(defValue)
							+ " WHERE " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " is null"
							);	
*/							 		
					}
				}
			}
			else
			{
				var fwidJoin = 	this.extAr[i].vid == "xnmk"?" nmk.FWID ": " a.FID_PN ";
				
				var res = ExecuteSQL(s = "ALTER TABLE " + cInTable 
					+ " ADD " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " " + SqlType(this.extAr[i].type)
					, false)
					
				if( res)
				{
						//Переносим значения расширений 
					/*
					ExecuteSQL("UPDATE "+ (this.extAr[i].vid == "xnmk"?"(":"") + cInTable + " a " + (this.extAr[i].vid == "xnmk"?"LEFT JOIN ^CL_NMK nmk ON a.FNMKL = nmk.FWID )": "")
										 + " LEFT JOIN " + extTable + " b ON " + fwidJoin + "= b.FMAINWID "
										 + " SET a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = b.F" + this.extAr[i].type
										 + " WHERE b.FWID = " + sqlTo(this.extAr[i].fwid)
										 );
					*/
					
					include ("Objects\\DpExtentionBuilder.js");
					var oExt = new DpExtentionBuilder("CL_NMK");
					var extTblName = oExt.getValueTableName();
					
					var sSQL = "UPDATE "+ cInTable + " "
						+" SET "+ this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field
							+" = "+ "(select "+First("(F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//							+" = "+ "(select max(F"+ this.extAr[i].label +") "
								+" from "+ extTblName +" b "
									//+ (this.extAr[i].vid == "xnmk"? " LEFT JOIN ^CL_NMK nmk ON a.FNMKL = nmk.FWID " : "")
								+" where b.FMAINWID = fnmkl"  
								+")";
						+ " WHERE (select "+First("(F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//						+ " WHERE (select max(F"+ this.extAr[i].label +") "
								+" from "+ extTblName +" b "
									//+ (this.extAr[i].vid == "xnmk"? " LEFT JOIN ^CL_NMK nmk ON a.FNMKL = nmk.FWID " : "")
								+" where b.FMAINWID = fnmkl"  
								+")	is not null"										
					ExecuteSQL(sSQL);
					
					/*
						// Переносим значения по умолчанию
					ExecuteSQL("UPDATE " + cInTable 
							+ " SET " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = " + sqlTo(defValue)
							+ " WHERE " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " is null"
							);
					*/			
				}
				if (!equInOut)
				{
					var sSQL = "ALTER TABLE "+ cOutTable 
						+" ADD " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field 
							+ " " + SqlType(this.extAr[i].type)	
							 
					if (ExecuteSQL(sSQL, false))
					{
							//Переносим значения расширений 
						/*
						ExecuteSQL("UPDATE "+ (this.extAr[i].vid == "xnmk"?"(":"") + cOutTable + " a " 
								+ (this.extAr[i].vid == "xnmk"?"LEFT JOIN ^CL_NMK nmk ON a.FNMKL = nmk.FWID )": "")
								+ " LEFT JOIN " + extTable + " b ON " + fwidJoin + "= b.FMAINWID "
									+ " SET a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = b.F" + this.extAr[i].type
								+ " WHERE b.FWID = " + sqlTo(this.extAr[i].fwid)
								);
						*/
					
						
						var sSQL = "UPDATE "+ cOutTable + " "
							+ " SET "+ this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field
								+" = "+ "(select "+First("(F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//								+" = "+ "(select max(F"+ this.extAr[i].label +") "
										+" from "+ extTblName +" b "
										//+ (this.extAr[i].vid == "xnmk"? " LEFT JOIN ^CL_NMK nmk ON a.FNMKL = nmk.FWID " : "")
										+" where b.FMAINWID = fnmkl"  
									+")"
						+ " WHERE (select "+First("(F"+ this.extAr[i].label +") ", null, this.extAr[i].type)
//						+ " WHERE (select max(F"+ this.extAr[i].label +") "
								+" from "+ extTblName +" b "
									//+ (this.extAr[i].vid == "xnmk"? " LEFT JOIN ^CL_NMK nmk ON a.FNMKL = nmk.FWID " : "")
								+" where b.FMAINWID = fnmkl"  
								+")	is not null"							
						ExecuteSQL(sSQL);						
							
						/*
							// Переносим значения по умолчанию
						ExecuteSQL("UPDATE " + cOutTable 
								+ " SET " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = " + sqlTo(defValue)
								+ " WHERE "+this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field+" is null"
								);
						*/
					}
				}
			}
			if (false && all_extTable && !all_extTable.IsEmpty() && all_extTable.getValue("FTYPE").charAt(0) == "A") 
			{
				fieldTXT = this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + '_TXT'; 
				ExecuteSQL("UPDATE " + cInTable + " a , ^cl cl, ^CLRELCLCL relcl "
                                        + " SET a." + fieldTXT + " = " + Convert("cl.FTXT", "STR")
					+ " WHERE a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = relcl.FCOD "
					+ " AND cl.fwid = relcl.fwid_cl "
						+ " AND relcl.FCL = " + Number(all_extTable.getValue("FTYPE").substr(1)));
				if (!equInOut)
				{
					ExecuteSQL("UPDATE " + cOutTable + " a , ^cl cl, ^CLRELCLCL relcl "
                                                + " SET a." + fieldTXT + " = " + Convert("cl.FTXT", "STR")
					+ " WHERE a." + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + " = relcl.FCOD "
						+ " AND cl.fwid = relcl.fwid_cl "
							+ " AND relcl.FCL = " + Number(all_extTable.getValue("FTYPE").substr(1)));
				}
			}
		}
		ppnExtTbl = null;
		nmkExtTbl = null;
		clExtTbl = null;
	}// уф, конец расширениям
	
	//Добавляем tie поля, если такие есть
	for (var ind = 0; ind < this.nGroupsDim; ind++)
	{
		if (this.aGroups[ind].ttl_gr_Field.indexOf(".") != 0)
		{
			var tieTblName = String(this.aGroups[ind].ttl_gr_Field).substr(0, this.aGroups[ind].ttl_gr_Field.indexOf("."));
			var tieFldName = String(this.aGroups[ind].ttl_gr_Field).substr(this.aGroups[ind].ttl_gr_Field.indexOf(".") + 1);
			
			if(list.objTies)
			{
				for (var n = 0; n < list.objTies.length; n++)
				{
					// Array(table,alias,index,field)
					if (String(list.objTies[n][1]).toUpperCase() == tieTblName.toUpperCase())
					{
						this.aGroups[ind].ttl_gr_Field = String(this.aGroups[ind].ttl_gr_Field).replace(".", "_");
						this.aGroups[ind].ttl_gr_FieldOrd = String(this.aGroups[ind].ttl_gr_FieldOrd).replace(".", "_");
						ExecuteSQL("ALTER TABLE " + cInTable + " DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
						ExecuteSQL("ALTER TABLE " + cInTable + " ADD " + this.aGroups[ind].ttl_gr_Field + " "
							+ lpProxy.getDBManager().getDBType(lpProxy.getDBManager().getFieldInfo(list.objTies[n][0], tieFldName).getType()), false);
						if (!equInOut)
						{
							ExecuteSQL("ALTER TABLE " + cOutTable +" DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
							ExecuteSQL("ALTER TABLE " + cOutTable +" ADD " + this.aGroups[ind].ttl_gr_Field + " "
								+ lpProxy.getDBManager().getDBType(lpProxy.getDBManager().getFieldInfo(list.objTies[n][0], tieFldName).getType()), false);
						}
	
						var strWhere1 = "";
						var strWhere2 = "";
						var fields = list.objTies[n][3].split(","); 
						var indexes = lpProxy.getDBManager().getIndexFields(list.objTies[n][0], list.objTies[n][2]);
						// вместо индекса может стоять просто поле
						if (String(indexes[0]) == "")
							indexes[0] = list.objTies[n][2];
						if (fields.length == indexes.length)
						{ 
							for (var i = 0; i < fields.length; i++)
							{
								if (strWhere1 != "")
									strWhere1 += " AND ";
								if (strWhere2 != "")
									strWhere2 += " AND ";
								strWhere1 += cInTable + "." + allt(fields[i]) + " = ^" + list.objTies[n][0] + "." + allt(String(indexes[i]));
								strWhere2 += cOutTable + "." + allt(fields[i]) + " = ^" + list.objTies[n][0] + "." + allt(String(indexes[i]));
							}
						}
						
						ExecuteSQL("UPDATE " + cInTable
	                        + " SET " + this.aGroups[ind].ttl_gr_Field + " = (SELECT " + tieFldName + " FROM ^" + list.objTies[n][0]
							+ " WHERE " + strWhere1 + ")", false);
	                    
						if (!equInOut)
						{
							ExecuteSQL("UPDATE " + cOutTable
	    	                    + " SET " + this.aGroups[ind].ttl_gr_Field + " = (SELECT " + tieFldName + " FROM ^" + list.objTies[n][0]
									+ " WHERE " + strWhere2 + ")", false);
						}
					
						break;
					}
				}
			}
			
			// возможно это CL tie
			if(list.objClTies)
			{
				for (var n = 0; n < list.objClTies.length; n++)
				{
					// Array(table,alias,index,field)
					if (String(list.objClTies[n].alias).toUpperCase() == tieTblName.toUpperCase())
					{
						this.aGroups[ind].ttl_gr_Field = String(this.aGroups[ind].ttl_gr_Field).replace(".", "_");
						this.aGroups[ind].ttl_gr_FieldOrd = String(this.aGroups[ind].ttl_gr_FieldOrd).replace(".", "_");
						ExecuteSQL("ALTER TABLE " + cInTable + " DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
						ExecuteSQL("ALTER TABLE " + cInTable + " ADD " + this.aGroups[ind].ttl_gr_Field + " "
							+ lpProxy.getDBManager().getDBType(lpProxy.getDBManager().getFieldInfo("CL", tieFldName).getType()), false);
						if (!equInOut)
						{
							ExecuteSQL("ALTER TABLE " + cOutTable +" DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
							ExecuteSQL("ALTER TABLE " + cOutTable +" ADD " + this.aGroups[ind].ttl_gr_Field + " "
								+ lpProxy.getDBManager().getDBType(lpProxy.getDBManager().getFieldInfo("CL", tieFldName).getType()), false);
						}
	
						var fields = list.objClTies[n].fldLink.split(",");
						include("objects\\DpEval.js");
						var eval = new DpEval();
						var cl_value = fields[0];
						if (cl_value.startsWith("="))
							cl_value = eval.eval(cl_value.substr(1));
						var cod_field = fields[1];
						
						if (cod_field && cl_value)
						{
							ExecuteSQL("UPDATE " + cInTable
		                        + " SET " + this.aGroups[ind].ttl_gr_Field 
		                        	+ " = (SELECT a." + tieFldName + " FROM ^CL a left join ^clrelclcl b on a.fwid = b.fwid_cl "
								+ " WHERE b.fcl = " + cl_value + " AND b.fcod = " + cInTable + "." + cod_field + ")", false);
		                    
							if (!equInOut)
							{
								ExecuteSQL("UPDATE " + cOutTable
			                        + " SET " + this.aGroups[ind].ttl_gr_Field 
			                        	+ " = (SELECT a." + tieFldName + " FROM ^CL a left join ^clrelclcl b on a.fwid = b.fwid_cl "
									+ " WHERE b.fcl = " + cl_value + " AND b.fcod = " + cOutTable + "." + cod_field + ")", false);
							}
						}
					
						break;
					}
				}
			}
			
			// возможно это расширение добавленное чрез ExtTieEx
			if(list.objExtTies)
			{
				for (var n = 0; n < list.objExtTies.length; n++)
				{
					if (String(list.objExtTies[n].sAlias).toUpperCase() == tieTblName.toUpperCase())
					{
						// получаем имя таблицы с расширениями 
						extBuilder = new DpExtentionBuilder(list.objExtTies[n].sVid, list.objExtTies[n].sPar2, list.objExtTies[n].sPar3);
				        var tieTblName = extBuilder.getValueTableName();
				        
						var type = String(extBuilder.getAttr(tieLabel, "type"));
						if(inList(type.charAt(0), "A", "R"))
							type = "INT";
						if(type.charAt(0) == "V")
							type = "TEXT";
						
						this.aGroups[ind].ttl_gr_Field = "EXT_" + tieLabel;
						this.aGroups[ind].ttl_gr_FieldOrd = "EXT_" + tieLabel;
						ExecuteSQL("ALTER TABLE " + cInTable + " DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
						ExecuteSQL("ALTER TABLE " + cInTable + " ADD " + this.aGroups[ind].ttl_gr_Field + " "
							+ lpProxy.getDBManager().getDBType(type), true);
						if (!equInOut)
						{
							ExecuteSQL("ALTER TABLE " + cOutTable +" DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
							ExecuteSQL("ALTER TABLE " + cOutTable +" ADD " + this.aGroups[ind].ttl_gr_Field + " "
								+ lpProxy.getDBManager().getDBType(type), true);
						}
	
						var strWhere1 = cInTable + ".FWID" + " = ^" + tieTblName + ".FMAINWID";
						var strWhere2 = cOutTable + ".FWID" + " = ^" + tieTblName + ".FMAINWID";
						ExecuteSQL("UPDATE " + cInTable
	                        + " SET " + this.aGroups[ind].ttl_gr_Field + " = (SELECT F" + tieLabel + " FROM ^" + tieTblName
							+ " WHERE " + strWhere1 + ")");//, false);                    
						if (!equInOut)
						{
							ExecuteSQL("UPDATE " + cOutTable
	    	                    + " SET " + this.aGroups[ind].ttl_gr_Field + " = (SELECT F" + tieLabel + " FROM ^" + tieTblName
									+ " WHERE " + strWhere2 + ")");//, false);
						}
					
						break;
					
					}				
				}
			}
		}		
	}
	
	//Добавляем расширеия, если такие есть
	for (var ind = 0; ind < this.nGroupsDim; ind++)
	{
		if (this.aGroups[ind].ttl_gr_Field.startsWith("#") != 0)
		{
			var tieLabel = String(this.aGroups[ind].ttl_gr_Field).substr(1);

			for (var n = 0; n < list.objExtTies.length; n++)
			{
				if (!list.objExtTies[n].sAlias)
				{
					// получаем имя таблицы с расширениями 
					extBuilder = new DpExtentionBuilder(list.objExtTies[n].sVid, list.objExtTies[n].sPar2, list.objExtTies[n].sPar3);
			        var tieTblName = extBuilder.getValueTableName();
			        
					var type = String(extBuilder.getAttr(tieLabel, "type"));
					if(inList(type.charAt(0), "A", "R"))
						type = "INT";
					if(type.charAt(0) == "V")
						type = "TEXT";
					
					this.aGroups[ind].ttl_gr_Field = "EXT_" + tieLabel;
					this.aGroups[ind].ttl_gr_FieldOrd = "EXT_" + tieLabel;
					ExecuteSQL("ALTER TABLE " + cInTable + " DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
					ExecuteSQL("ALTER TABLE " + cInTable + " ADD " + this.aGroups[ind].ttl_gr_Field + " "
						+ lpProxy.getDBManager().getDBType(type), true);
					if (!equInOut)
					{
						ExecuteSQL("ALTER TABLE " + cOutTable +" DROP COLUMN " + this.aGroups[ind].ttl_gr_Field, false);
						ExecuteSQL("ALTER TABLE " + cOutTable +" ADD " + this.aGroups[ind].ttl_gr_Field + " "
							+ lpProxy.getDBManager().getDBType(type), true);
					}

					var strWhere1 = cInTable + ".FWID" + " = ^" + tieTblName + ".FMAINWID";
					var strWhere2 = cOutTable + ".FWID" + " = ^" + tieTblName + ".FMAINWID";
					ExecuteSQL("UPDATE " + cInTable
                        + " SET " + this.aGroups[ind].ttl_gr_Field + " = (SELECT F" + tieLabel + " FROM ^" + tieTblName
						+ " WHERE " + strWhere1 + ")");//, false);                    
					if (!equInOut)
					{
						ExecuteSQL("UPDATE " + cOutTable
    	                    + " SET " + this.aGroups[ind].ttl_gr_Field + " = (SELECT F" + tieLabel + " FROM ^" + tieTblName
								+ " WHERE " + strWhere2 + ")");//, false);
					}
				
					break;
				}
			}
		}
	}

    for (var nAgr=0;nAgr<this.nAgrsDim;nAgr++)
    {
		if (this.aAgrs[nAgr].ttl_ag_Field.startsWith("#"))
		{
			var tieLabel = String(this.aAgrs[nAgr].ttl_ag_Field).substr(1);

			for (var n = 0; n < list.objExtTies.length; n++)
			{
				if (!list.objExtTies[n].sAlias)
				{
					// получаем имя таблицы с расширениями 
					extBuilder = new DpExtentionBuilder(list.objExtTies[n].sVid, list.objExtTies[n].sPar2, list.objExtTies[n].sPar3);
			        var tieTblName = extBuilder.getValueTableName();
			        
					var type = String(extBuilder.getAttr(tieLabel, "type"));
					if(inList(type.charAt(0), "A", "R"))
						type = "INT";
					if(type.charAt(0) == "V")
						type = "TEXT";
					
					this.aAgrs[nAgr].ttl_ag_Field = "EXT_" + tieLabel;
					this.aAgrs[nAgr].ttl_gr_FieldOrd = "EXT_" + tieLabel;
					ExecuteSQL("ALTER TABLE " + cInTable + " DROP COLUMN " + this.aAgrs[nAgr].ttl_ag_Field, false);
					ExecuteSQL("ALTER TABLE " + cInTable + " ADD " + this.aAgrs[nAgr].ttl_ag_Field + " "
						+ lpProxy.getDBManager().getDBType(type), true);
					if (!equInOut)
					{
						ExecuteSQL("ALTER TABLE " + cOutTable +" DROP COLUMN " + this.aAgrs[nAgr].ttl_ag_Field, false);
						ExecuteSQL("ALTER TABLE " + cOutTable +" ADD " + this.aAgrs[nAgr].ttl_ag_Field + " "
							+ lpProxy.getDBManager().getDBType(type), true);
					}

					var strWhere1 = cInTable + ".FWID" + " = ^" + tieTblName + ".FMAINWID";
					var strWhere2 = cOutTable + ".FWID" + " = ^" + tieTblName + ".FMAINWID";
					ExecuteSQL("UPDATE " + cInTable
                        + " SET " + this.aAgrs[nAgr].ttl_ag_Field + " = (SELECT F" + tieLabel + " FROM ^" + tieTblName
						+ " WHERE " + strWhere1 + ")");//, false);                    
					if (!equInOut)
					{
						ExecuteSQL("UPDATE " + cOutTable
    	                    + " SET " + this.aAgrs[nAgr].ttl_ag_Field + " = (SELECT F" + tieLabel + " FROM ^" + tieTblName
								+ " WHERE " + strWhere2 + ")");//, false);
					}
				
					break;
				}
			}
		}
    }

	//Добавляем поле, если нет 
	
	if (ExecuteSQL("ALTER TABLE "+cOutTable+
				" ADD "+cFCrit+" " + SqlType("TEXT", 20)+"",false))
	{
		ExecuteSQL("UPDATE "+cOutTable+
				" SET "+cFCrit+"=' '");
	}
	if (this.createFttLevel)
	{
		ExecuteSQL("ALTER TABLE "+cOutTable+
					" ADD FttLevel INTEGER", false);		
		var sSql = "UPDATE "+ cOutTable +" SET FttLevel=0";		
		if(!getSnap("select fttlevel from "+ cOutTable +" where fttlevel is null"))
			sSql += " WHERE FttLevel<>0";
		ExecuteSQL(sSql);
	}
	if (this.createHelpField)
	{
		for (var m=0;m<this.nGroupsDim;m++)
		{
			if (this.aGroups[m].ttl_gr_ItogType=="+" || this.aGroups[m].ttl_gr_ItogType=="_")
			{
				ExecuteSQL("ALTER TABLE "+cOutTable+
					" ADD "+this.aGroups[m].ttl_gr_FieldOrd+" "+SqlType("SHORT"),false);
				ExecuteSQL("UPDATE "+cOutTable+
					" SET "+this.aGroups[m].ttl_gr_FieldOrd+"=0");
			}				
		}
	}
	
   //Если надо - удаляем итоги
   if (cMode.indexOf("S") == -1)
   {
		ExecuteSQL("DELETE FROM "+cInTable
		+ " WHERE " + sqlStrIsNotEmp(cFCrit) 
        + " AND " + cFCrit+" IS NOT NULL"
        + " AND " + cFCrit + "<> ''"
        + " AND " + cFCrit + "<> ' '");
   }
  
   if (this.nAgrsDim > 0)
	{
		//Формирование перечня полей
		
		var cAgrFieldsList = "";
		var cFieldExpr = "";

      for (var nAgr=0;nAgr<this.nAgrsDim;nAgr++)
      {
		//Вначале формируем имя поля
		 if (this.aAgrs[nAgr].ttl_ag_Field.startsWith("#"))
		 {
         	cFieldExpr = "SRC.EXT_"+String(this.aAgrs[nAgr].ttl_ag_Field).substr(1);
	         //Применяем агрегирующую функцию
	         cFieldExpr = this.aAgrs[nAgr].ttl_ag_Func+"("+cFieldExpr+")";
	         cFieldExpr = cFieldExpr+" AS EXT_"+String(this.aAgrs[nAgr].ttl_ag_Field).substr(1);
		 }
         else
         {
         	cFieldExpr = "SRC."+this.aAgrs[nAgr].ttl_ag_Field;
	         //Применяем агрегирующую функцию
	         cFieldExpr = this.aAgrs[nAgr].ttl_ag_Func+"("+cFieldExpr+")";
	         cFieldExpr = cFieldExpr+" AS "+this.aAgrs[nAgr].ttl_ag_Field;
         }

         cAgrFieldsList = cAgrFieldsList+","+cFieldExpr;
      }
      
      this.setOrderBy(cFCrit);

		//Подбивка итогов
		var cItogTable;// имя таблицы, куда подбиваются итоги
      var isFirstAgr = true;

		//Итоги в любом случае подбиваются в другую таблицу,
      //а потом, если надо, объединяются с исходной
   //   if (!equInOut)
   //      cItogTable = cOutTable
   //   else
           cItogTable = "TMP_TOTALS_1";
   
		DropTable(cItogTable);
		
	  var cLastItogID = "";
		var cFieldExprOrd = "";      
      var cSelectFieldsList = "";
      var cGroupByFieldsList =  "";
     
      var nLastItogID = this.nGroupsDim + 1;
      //Формируются итоги, начиная от самых мелких, закачивая самыми крупными
      for (var nItog=this.nGroupsDim-1;nItog>=0;nItog--)
	  {
			if (this.aGroups[nItog].ttl_gr_ItogType=="+" || this.aGroups[nItog].ttl_gr_ItogType=="_")
			{
	            cSelectFieldsList  = cAgrFieldsList;
		        cGroupByFieldsList = "";
				for (var i=0;i<this.nGroupsDim;i++)
				{
				    cFieldExpr = this.aGroups[i].ttl_gr_Field;
					cFieldExprOrd = this.aGroups[i].ttl_gr_FieldOrd;
					if ( i<=nItog)
//					if ((this.aGroups[i].ttl_gr_ItogType=="+" || this.aGroups[i].ttl_gr_ItogType=="_") && i<=nItog)
					{
					   if (cFieldExpr!="")
					   {
							if (this.aGroups[i].ttl_gr_cOptions == "YM")
							{
								cSelectFieldsList  = cSelectFieldsList+", " + toSqlDate(sqlYear(cFieldExpr), sqlMonth(cFieldExpr),1) + " as " + cFieldExpr;
								cGroupByFieldsList = cGroupByFieldsList+", " + toSqlDate(sqlYear(cFieldExpr), sqlMonth(cFieldExpr),1);
							}
							else
							{
								cSelectFieldsList  = cSelectFieldsList+","+cFieldExpr;
								cGroupByFieldsList = cGroupByFieldsList+","+cFieldExpr;
							}
					   }
						if (this.createHelpField)//использовать вспомогательные поля упорядочения
						{
							if (i==nItog)
							{
								switch (this.aGroups[i].ttl_gr_ItogType)
								{
									case "+":
										cSelectFieldsList  = cSelectFieldsList+", -1 AS "+cFieldExprOrd;
										break;
									case "_":
										cSelectFieldsList  = cSelectFieldsList+", 1 AS "+cFieldExprOrd;
										break;
								}		
							}	
							else
								cSelectFieldsList  = cSelectFieldsList+", 0 AS "+cFieldExprOrd;
						}		
					}   
					else
					{
					   if (cFieldExpr!="")
					   {
							var emptyVal = this.aGroups[i].ttl_gr_EmptyVal;
							var oEval = new DpEval();
							emptyVal = oEval.eval(String(emptyVal));
							if(emptyVal=="" && SUBD==ORACLE && typeOf(emptyVal)!="number")
								emptyVal = replicate(' ', 254);
						   	cSelectFieldsList = cSelectFieldsList + ","
								+ sqlTo(emptyVal)
									+ " AS " +cFieldExpr;
						}
						if (this.createHelpField)//использовать вспомогательные поля упорядочения
						{
							cSelectFieldsList  = cSelectFieldsList+",0 AS " +cFieldExprOrd;						   
						}
					}   
				}

            	cGroupByFieldsList = cGroupByFieldsList.substr(1);

            	if (isFirstAgr)
            	{
               		ExecuteSQL("DROP TABLE "+ cItogTable, false);
               		
               		var flds = allt(String(cSelectFieldsList));
               		if (flds.charAt(0) == ',')
               			flds = flds.substr(1)               		
               		
               		ExecuteSQL("SELECT "+ flds + ((this.createFttLevel)?", " +sqlTo(nItog + 1)+ " AS FttLevel":"") +
							" INTO "+cItogTable+
                  		" FROM "+cInTable+" SRC WHERE 1<>1 " + 
                  			(cGroupByFieldsList!=''?" GROUP BY "+cGroupByFieldsList:""));
			if (this.bAddFwid)
	            		ExecuteSQL("ALTER TABLE "+cItogTable+ " ADD FWID "+SqlType("COUNTER"),false);
               		
               		if (!cGroupByFieldsList || cGroupByFieldsList == '')
               			ExecuteSQL("DELETE FROM " + cItogTable);
               		if (ExecuteSQL("ALTER TABLE "+cItogTable+
								" ADD "+cFCrit+" " + SqlType("TEXT", 20)+"",false))
					{
						ExecuteSQL("UPDATE "+ cItogTable +
								" SET "+cFCrit+"=' '");
					}
               		
               		ExecuteSQL("INSERT INTO "+cItogTable+ "(" + cFCrit + dropAs(cSelectFieldsList) + ((this.createFttLevel) ? ", FttLevel) " : ")") +
               			"SELECT "+Convert(sqlTo(this.aGroups[nItog].ttl_gr_ItogID),"NVC",20)+" AS "+cFCrit+cSelectFieldsList+ ((this.createFttLevel)?", " +sqlTo(nItog + 1)+ " AS FttLevel":"") +
						" FROM "+cInTable+" SRC "+
                  		(cGroupByFieldsList!=''?" GROUP BY "+cGroupByFieldsList:""))
                  	
            	}
            	else
				{
					if (this.createFttLevel)
					{
						ExecuteSQL("INSERT INTO "+cItogTable+ "(" + cFCrit + dropAs(cSelectFieldsList) + ((this.createFttLevel) ? ", FttLevel) " : ")") +
							" SELECT "+sqlTo(this.aGroups[nItog].ttl_gr_ItogID)+" AS "+cFCrit+cSelectFieldsList+((this.createFttLevel)?", " +sqlTo(nItog + 1)+ " AS FttLevel ":"") +
							" FROM "+cItogTable+" SRC "+
							" WHERE FttLevel = " + sqlTo(nLastItogID) +
							(cGroupByFieldsList!=''?" GROUP BY "+cGroupByFieldsList:""));
					}
					else
					{
						ExecuteSQL("INSERT INTO "+cItogTable+ "(" + cFCrit + dropAs(cSelectFieldsList) + ((this.createFttLevel) ? ", FttLevel) " : ")") +
							" SELECT "+sqlTo(this.aGroups[nItog].ttl_gr_ItogID)+" AS "+cFCrit+cSelectFieldsList+((this.createFttLevel)?", " +sqlTo(nItog + 1)+ " AS FttLevel ":"") +
							" FROM "+cItogTable+" SRC "+
								" WHERE "+cFCrit+"="+sqlTo(cLastItogID)+
							(cGroupByFieldsList!=''?" GROUP BY "+cGroupByFieldsList:""));
					}
				}

            	nLastItogID = nItog + 1;
            	cLastItogID = this.aGroups[nItog].ttl_gr_ItogID;
           		isFirstAgr = false;

         	}//if (this.aGroups[nItog].ttl_gr_ItogType=="+" || this.aGroups[nItog].ttl_gr_ItogType=="_")
		}//for
      
      	if (!isFirstAgr)
      	{           
           	var fldValue = getColumn(cItogTable, false);
           	var flds = fldValue;
           	var ret = ExecuteSQL("INSERT INTO " +  cOutTable + " (" + flds +  ")"
              		+ " SELECT " + fldValue + " FROM "+cItogTable, SUBD != MSSQL && SUBD != DERBY);
           	if (ret == false  &&  (SUBD == MSSQL || SUBD==DERBY))
           	{
           		if (flds.toUpperCase().indexOf("FWID") == -1)
           		{
           			flds2 = getColumn(cOutTable, false).toUpperCase();
           			if (flds2.toUpperCase().indexOf("FWID") != -1)
               		{
           				fldValue += ",0";
           				flds += ",fwid";
               		}
           			ExecuteSQL("INSERT INTO " +  cOutTable + " (" + flds +  ")"
                      		+ " SELECT " + fldValue + " FROM "+cItogTable);
           		}
           		
           	}
      		
           	
      	}
		
		//Удаляем из таблицы итоги, не попадающие под заданное условие
		for (var numItog=this.nGroupsDim-1;numItog>=0;numItog--)
		{
			with (this.aGroups[numItog])
			{
				if (this.aGroups[numItog].ttl_gr_ItogType!="-" && typeof(ttl_gr_Cond)=="string" && ttl_gr_Cond!="")
				{
					ExecuteSQL("DELETE FROM "+cOutTable+
							" WHERE "+cFCrit+"="+sqlTo(ttl_gr_ItogID)
							+ " AND 'false' = "+ sqlTo(ttl_gr_Cond));
				}
			}	
		}

		if (this.extAr.length != 0)// Если есть расширения добавим текстовки для типа аналитика 
	   {	
			//Выбираем их
			for (i = 0; i < this.extAr.length; i++)
			{
				if (this.extAr[i].extType && inList(this.extAr[i].extType.charAt(0), "A" /*, "R", "V"*/))
				{
					fieldTXT = this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + '_TXT'; 
					ExecuteSQL("ALTER TABLE " + cInTable + " DROP COLUMN " + fieldTXT, false);
					ExecuteSQL("ALTER TABLE " + cInTable + " ADD " + fieldTXT+ " " + SqlType("TEXT"), false);
					if (!equInOut)
					{
						ExecuteSQL("ALTER TABLE " + cOutTable + " DROP COLUMN " + fieldTXT, false);
						ExecuteSQL("ALTER TABLE " + cOutTable + " ADD " + fieldTXT+ " " + SqlType("TEXT"), false);
					}
					
					ExecuteSQL( "UPDATE " + cInTable + " SET " + fieldTXT + " = "
					 + "(select ftxt from ^listcl where fcl = " + sqlTo(Number(this.extAr[i].extType.substr(1)))
					 + " and fcod = " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + ") "
					 )
					 
					if (!equInOut)
					{
						ExecuteSQL( "UPDATE " + cOutTable + " SET " + fieldTXT + " = "
					 		+ "(select ftxt from ^listcl where fcl = " + sqlTo(Number(this.extAr[i].extType.substr(1)))
					 		+ " and fcod = " + this.aGroups[this.extAr[i].indInGroups].ttl_gr_Field + ") "
					 	)
					}
					
				}
			}
	   }
		var oSnap = getSnap("SELECT count(*) as cnt from " + cOutTable + " WHERE FWID = 0 or FWID is NULL");
		if (oSnap != null && oSnap[0] != 0)
		{
			ExecuteSQL("ALTER TABLE "+cOutTable+ " ADD ftmpuid "+SqlType("COUNTER"),false);
			var oSnap = getSnap("SELECT count(*) as cnt from " + cOutTable);
			if (oSnap != null)
			{
				var nOffSet = Number(oSnap[0]);
				var uid = Number(GetUID(false, nOffSet)) - nOffSet; 	
				ExecuteSQL("UPDATE "+cOutTable+
						" SET fwid = ftmpuid + " + uid 
						+ " WHERE FWID = 0 or FWID is NULL", false);
			}
			ExecuteSQL("ALTER TABLE "+cOutTable+ " DROP COLUMN ftmpuid ",false);
		}
   }// if (this.nAgrsDim > 0)
   DropTable(ppnExtName);
   DropTable(nmkExtName);
   DropTable(cTmpTable);  
}

function DpTotals_ResetTotals()
{
	this.aAgrs.length = 0;
	this.aGroups.length = 0;
	this.nAgrsDim = 0;//длина массива - кол-во объектов-итогов
	this.nGroupsDim = 0;//длина массива - кол-во объектов-группировки
	this.createHelpField = false;
}