/*
   Описание объекта "элемент справочника аналитики": DpVopl
   (C) ДЕБЕТ Плюс, 1998-1999

   Конструктор DpVopl.
      Возвращает this

   Внешние методы:
   .setVopl(cod)   - Установить код вида оплаты
   .getVopl()      - Получить код вида оплаты
   .getNzv()       - Получить наименование вида оплаты
   .getTbl()       - Получить табель по умолчанию
   .valid()        - Проверка, существует ли запись по заданному коду вида оплаты
   .linkCtrl()     - Связать с полем ввода
   .validCtrl()    - Реакция на выход из связанного поля ввода
   .zaval()   - Осуществляется завал при нажатии кнопки.

   Внутренние свойства:
   .vopl           - код вида оплаты
      Используется только в методах setVopl и getVopl.
      В остальных случаях пользуйтесь этими методами.
   .nazv            - наименование вида оплаты
      Используется только в методах setNzv и getNzv.
      В остальных случаях пользуйтесь этими методами.
   .tbl            - табель по умолчанию
      Используется только в методах setTbl и getTbl.
      В остальных случаях пользуйтесь этими методами.
   .ctrl           - связь с полем ввода. 
   .isButton       - кнопка для завала.
   .updateCtrl     - обновление поля ввода.
*/

// Конструктор объекта DpVopl
function DpVopl()
{
	RO("DpVopl", this);
	include("objects/DpZaval.js")
	this.vopl = 0;
	this.nazv = "???";
	
	this.setVopl = DpVopl_setVopl;
	this.getVopl = DpVopl_getVopl;
	this.getNzv = DpVopl_getNzv;
	this.getTbl = DpVopl_getTbl;
	this.valid = DpVopl_valid;
	this.linkCtrl = DpVopl_linkCtrl;
	this.validCtrl = DpVopl_validCtrl;
	this.dpZaval = DpVopl_initZaval();
	this.zaval = DpVopl_zaval;
	this.updateCtrl = DpVopl_updateCtrl;   
}

// Установить код вида оплаты
function DpVopl_setVopl(cod)
{ 
	var snap = getSnap("SELECT fvopl, fnzv_vopl, ftbl_umol FROM ^cl_vopl"
						+ " WHERE fvopl >= 1000 AND fvopl <= 9000"
						+ " AND fvopl=" + sqlTo(cod) + " AND fgroup = " + SqlTrue
						+ " AND fserv=" + SqlFalse + " ORDER BY fvopl");   			   
	if (snap != null) 
	{
		this.vopl = snap[0];
		this.nazv = snap[1];
		this.tbl = snap[3];
	}
	else
	{
		this.vopl = 0;
		this.nazv = "???";
		this.tbl = "???";
	}
	if(this.ctrl)
	{
		this.updateCtrl();
	}
}

// Получить код вида оплаты
function DpVopl_getVopl()
{
	return this.vopl;
}

// Получить наименование вида оплаты
function DpVopl_getNzv()
{
	return this.nazv;
}

// Получить табель по умолчанию
function DpVopl_getTbl()
{
	return this.tbl;
}

// Связать с полем ввода
function DpVopl_linkCtrl(ctrl, nazv_ctrl)
{ 	
	this.ctrl = ctrl;
	if (typeof(ctrl) != "object")
		return;

	this.ctrl.setMask("####");
	this.ctrl.isButton = true;
	setValid(this.ctrl, this, "validCtrl");	
	setButton(this.ctrl, this, "zaval");
	
	if (typeof(nazv_ctrl) == "object")
	{
		this.nazv_ctrl = nazv_ctrl;
		this.nazv_ctrl.setMask("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&"); // 30
		Disable(this.nazv_ctrl);
	}	
	this.updateCtrl();
}

function DpVopl_valid()
{
	return (this.getVopl() >= 1000 && this.getVopl() <= 9000);
}

function DpVopl_validCtrl()
{
	this.setVopl(this.ctrl.value);	
	if (!this.valid())
		return this.zaval();
	return true;
}

function DpVopl_zaval()
{
	/*	
	var par = new Object();
	par.ret = new Object();
	par.ret.fvopl = 0;
	showWindow("l_vopl.js", SW_MODAL, par);
	if (!par.escape) 
	{
		this.setVopl(par.ret.fvopl);
		return this.valid();
	}	*/
	this.dpZaval.open(); 
	if (!this.dpZaval.escape) 
	{ 
		this.setVopl(this.dpZaval.ret.fvopl);
		return this.valid();
	}	
	return false;
}

function DpVopl_updateCtrl()
{  
	this.ctrl.value = this.getVopl();
	if (this.nazv_ctrl != "undefined")
		this.nazv_ctrl.value = this.getNzv();
}

function DpVopl_initZaval()
{
	
	var dpZaval = new DpZaval("Норми часу|Нормы времени");
	dpZaval.setSQL("SELECT * FROM ^cl_vopl WHERE fvopl >= 1000 AND fvopl <= 9000"
			+ " AND fgroup = " + SqlTrue + " AND fserv=" + SqlFalse + " ORDER BY fvopl");
	dpZaval.setRet("fvopl", 0);
	dpZaval.FieldEx("fvopl", "Код виду оплати|Код вида оплаты", 15);                 
	dpZaval.FieldEx("fnzv_vopl", "Назва виду оплати|Наименование вида оплаты", 40);                 
	dpZaval.FieldEx("ftbl_umol", "Табель за замовченням|Табель по умолчанию", 20);	
	return dpZaval;	
}