// Объект для расчета баз обложения налогами для 
// расчета выходных форм в подсистеме
// "Учет заработной платы"
// 21.12.2004
// Шатохин Влад

// 11.01.2005
// А теперь объект используется еще и для расчета строки разноски по счетам

// Пример использования
// 1) Простой расчет базы налогообложения:
// 		include("objects\\DpZrpBaseSum.js")
// 		var oBases = new DpZrpBaseSum();
// 		oBases.setDate(dat1);	
// 		oBases.setFields("PFP")	// Пенсионный фонд с предприятия
// 		oBases.init();
// 		oBases.doCalc();
// 		var tblResult = oBases.getTbl();
// 2) Расчет сумм по SVZP с наложением условий (например, при расчете разноски):
// 		include("objects\\DpZrpBaseSum.js")
// 		var oBases = new DpZrpBaseSum();
// 		oBases.setDate(dat1);	
// 		oBases.init();
//
//		oBases.addFilter("FLS_INCL", "548")
//		oBases.setGrp("FSHPZ");
// 		oBases.doCalc();
// 		var tblResult = oBases.getTbl();
//		...
// 		oBases.reset();
//		oBases.addFilter("FLV_INCL", "1/2,3/4/1,3/4/8")
// 		oBases.doCalc();
// 		var tblResult2 = oBases.getTbl();

//В результате в oBases.ogrName получим таблицу след. структуры
//	FTABN		// табельный номер
//	FMF"		// м-ц за который
//	FMN"		// м-ц в котором
//	FSUM"		//базовая сумма
//	FSUMBEZOGR"	//сумма без наложения ограничений, породившая базовую сумму

/*
include("vsrazn:objects\\DpZrpBaseSum.js");
var oBases = new DpZrpBaseSum();
var tblBase = "TMP_TBL_6";
oBases.reset();
oBases.setDate(new Date(2010, 1, 0));
oBases.setTblName(tblBase);
oBases.setGrp("FTABN");
oBases.setGrp("FIS_SCIENCE");
oBases.setGrp("FSHT_ID");
oBases.setGrp("FLSFID");
oBases.setFields("PFL");
oBases.setShtatNeeded(true);
oBases.init();
oBases.doCalc();

*/
include("sys\\udfs.js");
include("servis\\date.js");       //для firstDay() и lastDay()
include("get_cnst.js");

function DpZrpBaseSum()
{
	RO("DpZrpBaseSum", this);
	
	// Установить расчетный м-ц
	this.setDate = DpZrpBaseSum_setDate;

	// Устанавливает признак того, что требуется в сводном SVZP проставлять 
	// информацию из штатеого расписания.
	this.setShtatNeeded = DpZrpBaseSum_setShtatNeeded;
		
	// Проводится первоначальная инициализация
	// без непосредственного расчета базовых сумм.
	// Выделена для того, чтобы расчет был реентерабельным
	// Сюда вынесено формирование сводного SVZP
	this.init = DpZrpBaseSum_init;
	// Собственно расчет сумм
	this.doCalc = DpZrpBaseSum_doCalc;
	// Проставляет в строках сводного файла информацию из штатного расписания
	this.setShtatInfo = DpZrpBaseSum_setShtatInfo;

	// Возвращает открытую таблицу-объект с результатами расчета
	this.getTbl = DpZrpBaseSum_getTbl;
	// Возвращает имя таблицы с результатами расчета
	this.getTblName = DpZrpBaseSum_getTblName;
	// Возвращает имя таблицы с ограниченными суммами
	this.getTblOgrName = DpZrpBaseSum_getTblOgrName;
	// Возвращает имя таблицы со сводным SVZP
	this.getAllName = DpZrpBaseSum_getAllName;
	// Возвращает имя таблицы со списком используемых месяцев
	this.getMonthName = DpZrpBaseSum_getMonthName;
	// Устанавливает имя таблицы с результатами расчета
	this.setTblName = DpZrpBaseSum_setTblName;
	// Устанавливает имя таблицы с ограничениями
	this.setTblOgrName = DpZrpBaseSum_setTblOgrName;
	// Устанавливает имя таблицы со сводным SVZP
	this.setAllName = DpZrpBaseSum_setAllName;
	// Устанавливает имя таблицы со списком используемых месяцев
	this.setMonthName = DpZrpBaseSum_setMonthName;
	// Установить список кодов полей ЛС, которые
	// входят в базовую сумму
	this.setFields = DpZrpBaseSum_setFields;
	// Установить выражение для вычисления верхней границы для базовых сумм
	this.setOgr = DpZrpBaseSum_setOgr;
	// Установить выражение для вычисления процента
	this.setProc = DpZrpBaseSum_setProc;
	// Установить вычисляемое выражение
	this.setExpr = DpZrpBaseSum_setExpr;
	// 	Установить внешний DpEval
	this.setEval = DpZrpBaseSum_setEval;
	// Процедура, осуществляющая сброс/первоначальную инициализацию.
	// Предназначена для очистки "мусора" после предыдущего использования 
	// объекта.
	// Выживает только сводная таблица this.allName
	this.reset = DpZrpBaseSum_reset;
	// Добавить фильтр
	this.addFilter = DpZrpBaseSum_addFilter;
	// Добавить поле группировки
	this.setGrp = DpZrpBaseSum_setGrp;
	// метод, который меняет константу, из какого месяца брать
	this.setConstMode = DpZrpBaseSum_setConstMode;
	this.getConstMode = DpZrpBaseSum_getConstMode;
	this.setSumOgrMode = DpZrpBaseSum_setSumOgrMode;
	this.getSumOgrMode = DpZrpBaseSum_getSumOgrMode;
	// получить аттрибуты фонда
	this.getFondAttr = DpZrpBaseSum_getFondAttr;

	// Устанавливает границы диапазонов значений
	this.setBands = DpZrpBaseSum_setBands

	// ф-ции для получения кусков запроса с учетом 
	// наложенных фильтров и группировок
	this.getSelect = DpZrpBaseSum_getSelect;
	this.getWhere = DpZrpBaseSum_getWhere;
	this.getGroupBy = DpZrpBaseSum_getGroupBy;

	//	пpеобpазует список подpазделений вида
	//	"1/1/3, 2, 3/5"
	//	в
	//	"((V.FLV1=1 AND V.FLV2=1 AND V.FLV3=3) OR (V.FLV1=2) OR (V.FLV1=3 AND V.FLV2=5))"
	this.getFlvCond = DpZrpBaseSum_getFlvCond;
	// Служебная. Преобразует фильтр в кусок SQL-запроса
	this.whereFieldParce = DpZrpBaseSum_whereFieldParce;
	// возвращает имя поля по метке
	this.getFieldByLabel = DpZrpBaseSum_getFieldByLabel;
	//Проставляем значение выражения
	this.setExprValues = DpZrpBaseSum_setExprValues;
	//Подготавливает переменные для Evala
	this.prepareVars4Eval = DpZrpBaseSum_prepareVars4Eval;
	//Функция возвращает список значений в поле таблицы по условию
	this.getListFromTblWithCond = DpZrpBaseSum_getListFromTblWithCond;

	// Ф-ция, формирующая IIF-выражение для определения номера диапазона, в который
	// попадает значение суммы. Вызывается рекурсивно.
	// При диапазонах (0, 150, 300) формирует что-то типа
	// IIF(A.FSUM>0,IIF(A.FSUM>150,IIF(A.FSUM>300, 3, 2), 1), 0)
	this.getBandCond =  DpZrpBaseSum_getBandCond;
			
	//private:
	// Месяц расчета (1-е число)
	this.date = d00;
	// Имя таблицы с результатом
	//this.tblName = getTmpTableName("", true);
	//var tmpAll = this.tblName+"_tmpAll";
	this.tblName = "VSRAZN_RESULT";
	DropTable(this.tblName);
	// Имя таблицы с суммами ограничений
	//this.ogrName = getTmpTableName("", true);
	this.ogrName = "VSRAZN_OGR";
	DropTable(this.ogrName)

	// Имя таблицы со сводным SVZP
	//this.allName = getTmpTableName("", true);
	this.allName = "VSRAZN_SVZP_ALL";
	DropTable(this.allName)
	// Имя таблицы со списком используемых месяцев
	//this.monthName = getTmpTableName("", true);
	this.monthName = "VSRAZN_MONTH";
	DropTable(this.monthName)

	// выражение для вычисления верхней границы для базовых сумм
	this.sumOgrExpr = "";
	// режим для вычисления верхней границы для базовых сумм
	//	"VOZN" - вычислять для месяца возникновения сумм (м-ца "за который")
	//	"VYPL" - вычислять для месяца выплаты сумм (м-ца "в котором")
	this.sumOgrMode = "VOZN";
	// режим константы, из какого месяца брать
	this.constMode = "VYPL";
	// выражение для вычисления процента
	this.procExpr = "100";
	//выражение
	this.expr = "";
	
	// выполняется расчет базовых сумм для ЕСВ и изменяется в setFields
	this.bIsESV = false;

	// 	Объект DpEval для вычисления сумм ограничений
	this.oEval = null;
	// массив условий для наложения фильтра
	this.aFilters = new Array();
	// массив для списка полей группировки
	this.aGrp = new Array();
	// массив границ диапазонов значений (см. this.setBands)
	this.aBands = new Array();
	
	// список меток(индексов) в массиве this.aFilters
	this.fltList = 
	"FLS_INCL"		//1.1 поле ЛС "для..."
	+",FLS_EXCL"		//1.2 поле ЛС "кpоме..."
	+",FLS_EXPR"	//!!! выражение, накладываемое на связку SVZP-LIC_FLD
	+",FLV_INCL"		//2.1 Подpазделение "для..."
	+",FLV_EXCL"		//2.2 Подpазделение "кpоме..."
	+",FKAT_INCL"	//3.1 Категория:"для..."
	+",FKAT_EXCL"	//3.2 Категория:"кpоме..."
	+",FSH_INCL"		//4.1 ШПЗ: "для..."
	+",FSH_EXCL"		//4.2 ШПЗ: "кpоме..."
	+",FSH_S_INCL"	//5.1 ШПЗ из штатного расписания: "для..."
	+",FSH_S_EXCL"	//5.2 ШПЗ из штатного расписания: "кpоме..."
	+",FTAB_INCL"	//6.1 Таб.номер: "для..."
	+",FTAB_EXCL"	//6.2 Таб.номер: "кpоме..."
	+",FPRIZN"		//7. Дата
	+",FEXP_INCL"	//8.1 Выражение: "для..."
	+",FEXP_EXCL"	//8.2 Выражение: "кpоме..."		
	
	+",FKZO_LS_INCL"
	+",FKZO_LS_EXCL"
	+",FKZO_SH_INCL"
	+",FKZO_SH_EXCL"
	
	// список меток(индексов) в массиве this.aGrp
	this.grpList = 
	"FLSFID"		//1. Поле ЛС
	+",FLV1"			//2.1 Flv1
	+",FLV2"			//2.2 Flv2
	+",FLV3"			//2.3 Flv3
	+",FLV4"			//2.4 Flv4
	+",FKOD_KAT"		//3. Категория
	+",FSHPZ"		//4.1 ШПЗ
	+",FSHPZ_S"		//4.2 ШПЗ из штатного расписания
	+",FTABN"		//5. Таб.номер
	+",FEXPR"		//6. Выражение
	+",FBAND"		//7. Диапазон
	+",FVOPL"		//8. Вид оплаты
	+",FSHT_ID"		//9. Код ШР
	+",FIS_SCIENCE"	//10. Код основания для учета стажа
	+",FCOD_KZO" // Тип категории застрахованного лица (было FTYPE_EMPLOYEE в 2000-й)
	
	// Признак того, что требуется в сводном SVZP проставлять 
	// информацию из штатеого расписания.
	this.isShtatNeeded = false;
	// Флаг того, что расчет информации из штатного произведен
	this.isShtatDone = false;
	
	// Выполним первоначальную инициализацию
	this.reset();
}

// Установить вычисляемое выражение
function DpZrpBaseSum_setExpr(sExpr) {
	this.expr = sExpr;

	return this;
}

// Устанавливает признак того, что требуется в сводном SVZP проставлять 
// информацию из штатного расписания.
function DpZrpBaseSum_setShtatNeeded(bSet){
	if(typeof(bSet)!="boolean"){
		bSet = true;
	}
	this.isShtatNeeded = bSet;

	return this;
}

// Проставляет в строках сводного файла информацию из штатного расписания
function DpZrpBaseSum_setShtatInfo()
{
	if(this.isShtatDone){
		// нечего по несколько раз считать одно и то же
		return;
	}
	// Отметим, что расчет информации из штатного произведен
	this.isShtatDone = true;
	
	//Таблица с максимальными UID для табельных номеров.
	//var maxTabnIdName = getTmpTableName("", true);
	var maxTabnIdName = "VSRAZN_MAX_TABN_ID_NAME";
	DropTable(maxTabnIdName);
	
	// eugene. FSHT_ID уже есть. Не пойму зачем заново определять ???????
	// Поэтому проапдейтим, а остальное пусть отработает только на непроставленных
	strSQL = "UPDATE "
				+this.allName
			+" SET FID = FSHT_ID"
	ExecuteSQL(strSQL);
	
	//Узнаем максимальный FID для каждого табельного номера в разрезе месяцев
	var fld = new Object();
	fld.FID = "INT";
	fld.FTABN_ID = "LONG";
	fld.FMN = "DATE";
	CreateTable(maxTabnIdName, fld);
	
	strSQL = " INSERT INTO "+maxTabnIdName
				+"(FID, FTABN_ID, FMN)"
			+" SELECT "
				+" MAX(WP.FID) AS FID, "
				+" V.FTABN_ID,"
				+" V.FMN "
			+" FROM "
				+this.allName+"  V, "
				+" ^WORK_PERS WP "
			+" WHERE "
				+" V.FMN BETWEEN WP.FBDAT AND WP.FEDAT "
				+" AND WP.FWID_CL = V.FTABN_ID "
			+" GROUP BY "
				+" V.FTABN_ID, "
				+" V.FMN "
	ExecuteSQL(strSQL);

	//Проставим максимальный Fid.
	strSQL = " UPDATE "
		+this.allName
	+" SET "
		+" FID = (SELECT "+IIF("M.FID IS NOT NULL", "M.FID", 1)
			+" FROM "+maxTabnIdName+" M WHERE M.FTABN_ID = "+this.allName+".FTABN_ID AND M.FMN = "+this.allName+".FMN)"
	+" WHERE FID = 0 OR FID IS NULL "
	ExecuteSQL(strSQL);
	
	//Проставляем FLV1 - FLV5, формируем в цикле
	var strSet = ""
	var aFields = new Array();
	var aValues = new Array();
	aFields[0] = "FLV1"; aFields[1] = "FLV2"; aFields[2] = "FLV3"; aFields[3] = "FLV4"; aFields[4] = "FLV5"; 
	aFields[5] = "FWID_KAT"; aFields[6] = "FVOPL"; aFields[7] = "FIS_SCIENCE"; 
	aFields[8] = "FCOD_KZO";
	aValues[0] = 0; aValues[1] = 0; aValues[2] = 0; aValues[3] = 0; aValues[4] = 0; aValues[5] = "''"; aValues[6] = 0; aValues[7] = sqlFalse;
	aValues[8] = 0;
	
	for (var i in aFields)
	{
		if (i < 5)
		{
			strSet += ", "+aFields[i]+" = "
				+" ( SELECT "+IIF("SO."+aFields[i]+" IS NOT NULL", "SO."+aFields[i], aValues[i])+" FROM ^WORK_PERS WP "
						+" INNER JOIN ^STRU_DOL SD ON SD.FWID = WP.FWID_DOL"
							+" INNER JOIN ^STRU_ORG SO ON SD.FWID_ORG = SO.FWID"
				+"   WHERE "+this.allName+".FTABN_ID = WP.FWID_CL AND "+this.allName+".FID = WP.FID)"
		}
		else if(i >= 5 && i <= 8){
			strSet += ", "+aFields[i]+" = "
				+" ( SELECT "+IIF("WP."+aFields[i]+" IS NOT NULL", "WP."+aFields[i], aValues[i])+" FROM ^WORK_PERS WP "
						+" INNER JOIN ^STRU_DOL SD ON SD.FWID = WP.FWID_DOL"
				+"   WHERE "+this.allName+".FTABN_ID = WP.FWID_CL AND "+this.allName+".FID = WP.FID)"
		}
		else
		{
			strSet += ", FSHPZ_S = "
				+" ( SELECT "+IIF("SD.FSHPZ IS NOT NULL", "SD.FSHPZ", "''")+" FROM ^WORK_PERS WP "
						+" INNER JOIN ^STRU_DOL SD ON SD.FWID = WP.FWID_DOL"
				+"   WHERE "+this.allName+".FTABN_ID = WP.FWID_CL AND "+this.allName+".FID = WP.FID)"
		}
	}
	strSet = strSet.substr(1);
	
	strSQL = " UPDATE "+this.allName+" SET "+strSet
	ExecuteSQL(strSQL);
	
	return this;
}

// Добавить фильтр
function DpZrpBaseSum_addFilter(sLabel, sValue) {
	sLabel = sLabel.toUpperCase();
	
	//На всякий случай - защита от дурака
	if( typeof(this.aFilters[sLabel]) == "undefined") {
		strMsg = "DpZrpBaseSum_addFilter("+sLabel+", "+sValue+")"
			+"\nПопытка установить фильтр на несуществующий признак "+sLabel
		alert(strMsg);
		return this;
	}
	this.aFilters[sLabel] = sValue;
	
	// а теперь проверим, следует ли подключать расчет 
	// информации из штатного расписания
	if(!this.isShtatNeeded) {
		switch(sLabel){
		case "FLV_INCL":		//2.1 Подpазделение "для..."
		case "FLV_EXCL":		//2.2 Подpазделение "кpоме..."
		case "FKAT_INCL":		//3.1 Категория:"для..."
		case "FKAT_EXCL":		//3.2 Категория:"кpоме..."
		case "FSH_S_INCL":		//5.1 ШПЗ из штатного расписания: "для..."
		case "FSH_S_EXCL":		//5.2 ШПЗ из штатного расписания: "кpоме..."
		case "FEXP_INCL":		//8.1 Выражение: "для..."
		case "FEXP_EXCL":		//8.2 Выражение: "кpоме..."	
			// принудительно включаем расчет
			this.setShtatNeeded(true);
			break;
		}
	}
	return this;
}

// Добавить поле группировки
// Если bSet не задано, по умолчанию принимаем равным true
function DpZrpBaseSum_setGrp(sLabel, bSet) {
	sLabel = sLabel.toUpperCase();
	
	if( typeof(bSet) == "undefined"){
		bSet = true;
	}
	 
	//На всякий случай - защита от дурака
	if( typeof(this.aGrp[sLabel]) == "undefined") {
		strMsg = "DpZrpBaseSum_setGrp("+sLabel+", "+sValue+")"
			+"\nПопытка установить группировку на несуществующее поле "+sLabel
		alert(strMsg);
	}
	this.aGrp[sLabel] = bSet;
	
	// а теперь проверим, следует ли подключать расчет 
	// информации из штатного расписания
	if(!this.isShtatNeeded) {
		switch(sLabel){
		case "FLV1":		//2.1 Flv1
		case "FLV2":		//2.2 Flv2
		case "FLV3":		//2.3 Flv3
		case "FLV4":		//2.4 Flv4
		case "FKOD_KAT":	//3. Категория
		case "FSHPZ_S":		//4.2 ШПЗ из штатного расписания
		case "FEXPR":		//6. Выражение
		case "FVOPL":		//7. Вид оплаты
			// принудительно включаем расчет
			this.setShtatNeeded(true);
			break;
		}
	}

	return this;
}

// Устанавливает границы диапазонов значений
// Принимает переменное кол-во параметров
//  - границы диапазонов.
// Считаем, что у нас все диапазоны имеют положительные значения
// Возвращает ссылку на наш объект для возможности вызова методов "по цепочке"
// obj.method1().method2(). ... .methodN()

// Пример вызова
// obj.setBands(0, 60, 80, 100)
// или 
// obj.setBands(mode)
// где mode один из следующих:
// "PFP"		Пенс. фонд с предприятия
// "PFL"		Пенс. фонд с работников
// "PF"			Сам Пенс. фонд с работников
// "FZP"		Фонд занятости с предприятия
// "FZL"		Фонд занятости с работников
// "FZ"			Сам Фонд занятости с работников
// "FSSP"		Фонд соц.страхования с предприятия
// "FSSL"		Фонд соц.страхования с работников
// "FSS"		Сам Фонд соц.страхования с работников
// "FSNVP"		Фонд соц.страхования от несч. случая (ФСНВ) с предприятия

function DpZrpBaseSum_setBands()
{
	// Переопределим заново, чтобы не осталось мусора
	// от предыдущих расчетов, возможно имевших место
	this.aBands = new Array();

	aArg = DpZrpBaseSum_setBands.arguments;
	//минимальное кол-во аргументов - 1:
	//начало первого и единственного диапазона
	if(aArg.length < 1)
	{
		return false;
	}
	
	if (aArg.length == 1 && typeof(aArg[0]) == "string")
	{
		// передан режим, а не границы диапазонов
		// границы диапазонов достаем сами из констант
		var mode = aArg[0];
		switch(mode)
		{
		case "PFP":	//Пенс. фонд с предприятия
			this.setBands(0
				)
			break;
		case "PFL":	//Пенс. фонд с работников
			this.setBands(0
				, get_cnst("progmin", this.date)//525
				)
			break;
		case "PF":	//Сам Пенс. фонд с работников
			this.setBands(0
				//, round(get_cnst("proc_pns1", this.date)*get_cnst("prozh_min", this.date)/100, 2)
				, round(0.5*get_cnst("progmin", this.date)/100, 2)
				)
			break;
		case "FZP":	//Фонд занятости с предприятия
			this.setBands(0
				)
			break;
		case "FZL":	//Фонд занятости с работников
			this.setBands(0
				, get_cnst("gr_soc", this.date)
				)
			break;
		case "FZ":	//Сам Фонд занятости с работников
			this.setBands(0)
			break;
		case "FSSP":	//Фонд соц.страхования с предприятия
			this.setBands(0
				)
			break;
		case "FSSL":	//Фонд соц.страхования с работников
			this.setBands(0
				, get_cnst("gr_soc", this.date)
				)
			break;
		case "FSS":	//Сам Фонд соц.страхования с работников
			this.setBands(0
				, round(get_cnst("proc_soc1", this.date)*get_cnst("gr_soc", this.date)/100, 2)
				)
			break;
		case "FSNVP":	//Фонд соц.страхования от несч. случая (ФСНВ) с предприятия
			this.setBands(0
				)
			break;
		case "ESVP":	// База ЕСВ - с основной ЗП
			this.setBands(0)
			break;
		case "ESV_CPH":	// База ЕСВ с ЦПХ
			this.setBands(0)
			break;
		case "ESV_BLN":	// База больничные
			this.setBands(0)
			break;
		case "ESV":		//Удержание ЕСВ с работников
			this.setBands(0)
			break;
		default:	
			break;
		}
		return this;
	}
	
	var nBands = aArg.length;
	
	for(var nIndex=0; nIndex < nBands; nIndex++)
	{
		this.aBands[nIndex] = aArg[nIndex];
		// Контролируем, чтобы список границ быть упорядочен по возрастанию
		if (nIndex > 0)
		{
			if(this.aBands[nIndex-1] >= this.aBands[nIndex])
			{
				var strMsg = "DpZrpBaseSum::setBands()"
				+"\nНекорректный список диапазонов:"
				+"\n"+aArg.toString();
				alert(strMsg);
			}
		}
	}
//alert(""+this.aBands);
	return this;
}


// Процедура, осуществляющая сброс/первоначальную инициализацию.
// Предназначена для очистки "мусора" после предыдущего использования 
// объекта.
// Выживает только сводная таблица this.allName
// и this.oEval
function DpZrpBaseSum_reset(){
	// Имя таблицы с результатом
	//this.tblName = getTmpTableName("", true);
	this.tblName = "VSRAZN_RESULT";
	DropTable(this.tblName);
	// Имя таблицы с суммами ограничений
	//this.ogrName = getTmpTableName("", true);
	this.ogrName = "VSRAZN_OGR";
	DropTable(this.ogrName);

	this.sumOgrExpr = "";
	this.sumOgrMode = "VOZN";
	this.procExpr = "100";
	this.bIsESV = false;

	// чистим массив условий для наложения фильтра
	var aFltList = this.fltList.split(",");
	for(ind in aFltList) {
		this.aFilters[aFltList[ind]] = "";
	}
	// чистим массив для списка полей группировки
	var aGrpList = this.grpList.split(",");
	for(ind in aGrpList) {
		this.aGrp[aGrpList[ind]] = false;
	}
	// чистим массив границ диапазонов значений
	this.aBands = new Array();

	return this;
}


// Установить расчетный м-ц
function DpZrpBaseSum_setDate(dat){
	this.date = new Date(dat);
	// сделаем первое число месяца
	this.date.setDate(1);

	return this;
}

// Проводится первоначальная инициализация
// без непосредственного расчета базовых сумм.
// Выделена для того, чтобы расчет был реентерабельным
// Сюда вынесено формирование сводного SVZP
function DpZrpBaseSum_init()
{
	// Создадим DpEval, если его нам не передали снаружи
	if(this.oEval==null){
		include("objects\\DpEval.js");
		this.oEval = new DpEval();
	}
	
	// подготовим "предопределенные" переменные
	this.prepareVars4Eval("", this.oEval, this.date, lastDay(this.date));
	
	// имя SVZP расчетного м-ца
	var currSvzp = "svzp" + month_txt(this.date)
	if(!IsTableEx("ZRP_RES", "ZRP_"+currSvzp))
	{ 
		CreateTableEx("zrp_res", currSvzp, "svzp", "*")
	}

	var helpMonthTbl = 	this.monthName+"_1";
	DropTable(helpMonthTbl);
	var fld = new Object();
	fld.FMN = "DATE";
	CreateTable(helpMonthTbl, fld);

	// Выбеpем в таблицу helpMonthTbl месяцы, которые встречаются в расчетном м-це
	var strSQL = " INSERT INTO "+helpMonthTbl+" (FMN)"
				+" SELECT DISTINCT "
					+IIF("S.FVP=0", Convert(sqlTo(this.date), "DATE"), "S.FMN")+" AS FMN"
				+" FROM ^"+currSvzp+" S "
				+" WHERE "
					+" NOT (S.FVP = 1 AND S.FMN >= "+Convert(sqlTo(this.date), "DATE")+" ) "
	ExecuteSQL(strSQL);

	// Добавим в таблицу helpMonthTbl для каждого из месяцев тоже список 
	// участвующих м-цев
	strSQL = " SELECT DISTINCT FMN "
	+" FROM "+helpMonthTbl
	
	var tblUsedMn = OpenTable(strSQL);
	for (; !tblUsedMn.IsEOF(); tblUsedMn.MoveNext()){
		
		var mn = new Date(tblUsedMn.getValue("FMN"));
		// В месяце, за который считаем базу, есть суммы за будущие
		// периоды. Для таких сумм лезть в SVZP будущих периодов мы НЕ ДОЛЖНЫ - 
		// их еще не существует.
		if (differ(mn, this.date) > 0){
		//if( mn > this.date) {
			continue;
		}

		var svzpName = "svzp" + month_txt(mn);
		
		if(IsTableEx("ZRP_RES", "ZRP_"+svzpName))
		{ 
			strSQL = "INSERT INTO "+helpMonthTbl+" (FMN)"
			+" SELECT DISTINCT "
				+IIF("S.FVP=0", sqlTo(mn), "S.FMN")+" AS FMN"
			+" FROM ^"+svzpName+" S "
			+" WHERE "
				+" NOT (S.FVP = 1 AND S.FMN >= "+sqlTo(mn)+" ) "
			ExecuteSQL(strSQL);
		}
	}
	tblUsedMn.close(5652);
	tblUsedMn = null;

	// Вот теперь формируем список месяцев, принимающих
	// участие в расчете.
	// Дополнительно создадим колонки, в которые для 
	// каждого месяца поместим вычисленную границу
	// и процент
	DropTable(this.monthName);
	var fld = new Object();
	fld.FMN = "DATE";
	fld.FOGR = "MONEY";
	fld.FPROC = "MONEY";
	CreateTable(this.monthName, fld);
	
	strSQL = " INSERT INTO "+this.monthName+" (FMN, FOGR, FPROC)"
			+" SELECT DISTINCT "
				+" FMN "
				+", "+Convert(0, "MONEY")+" AS FOGR"
				+", "+Convert(0, "MONEY")+" AS FPROC"
			+" FROM "+helpMonthTbl
	ExecuteSQL(strSQL);

	// Создаем индекс для использования с Tie()
	strSQL = "CREATE INDEX FMN ON "+this.monthName+"(FMN)";
	ExecuteSQL(strSQL, false);

	// Теперь формируем общую таблицу, в которую сбросим
	// строки со всех SVZP, которые встречаются в таблице this.monthName
	// за исключением тех, которые являются "будущими" для
	// месяца, за который считаем базу.
	DropTable(this.allName);
	CreateTable(this.allName, "VSP_VS", "*");
	
	// если нет шаблона "VSP_VS2" - то долго понимать почему оно не работает
	if (!IsTable(this.allName))
	{
		alert("Не создалась таблица this.allName - "+this.allName+". Вероятно, нет шаблона VSP_VS2");
	}

	// Сформируем список месяцев, SVZP которых будем добавлять
	// в this.allName
	strSQL = " SELECT * "
		+" FROM "+this.monthName
		+" ORDER BY FMN"
	var tblMn = OpenTable(strSQL);

	// Заполним svzpAllName
	if (!tblMn.IsEmpty())
	{
		tblMn.MoveFirst();
	}
	
	for(; !tblMn.IsEOF(); tblMn.MoveNext()){
		var mn = new Date(tblMn.getValue("FMN"));
		var m_bdat = firstDay(mn);
		var m_edat = lastDay(mn);
		if (differ(mn, this.date) > 0){
			// SVZP будущих м-цев в общую таблицу добавлять не будем
			continue;
		}
		
		svzpFrom = "svzp"+month_txt(mn);
		
		if(IsTableEx("ZRP_RES", "ZRP_"+svzpFrom))
		{ 
			strSQL = "INSERT INTO "	+ this.allName
				+" (FFLD_NUM, FLSFID, FWID_WP, FSHT_ID, FSUM, FTABN, FTABN_ID, FVP, FSHPZ, FMES, FMN, FMF)"
			+" SELECT "
				+" S.FFLD_NUM"
				+", S.FLSFID"
				//+", WP.FID AS FSHT_ID"
				+", S.FWID_WP"
				+", 0 AS FSHT_ID"
				+", S.FSUM"
				//+", REL.FCOD AS FTABN"
				+", 0 AS FTABN"
				+", S.FTABN_ID"
				+", S.FVP"
				+", S.FSHPZ"
				// месяц файла
				+", " + sqlTo(mn)+" as FMES "
				// месяц, в котором выплачена сумма по строке (месяц выплаты)
				+", "+IIF("S.FVP=0", sqlTo(mn),IIF("S.FVP=1", "S.FMN", IIF("S.FVP=2", sqlTo(mn), null)))+" AS FMN"
				// месяц, за который сумма в строке (месяц возникновения)
				+", "+IIF("S.FVP=0", sqlTo(mn), IIF("S.FVP=1", sqlTo(mn), IIF("S.FVP=2", "S.FMN", null)))+" AS FMF"
			+" FROM "
				+"^"+svzpFrom+" S"
			+" WHERE "
				// Отсекаем служебные записи, добавленные при расчете последующих месяцев
				// и не отображаемые при просмотре лицевых счетов
				+" NOT (S.FVP=1 AND S.FMN>="+sqlTo(mn)+")"
				// А также для прошлых месяцев не берем суммы
				// выплаченные в них на будущее за расчетный месяц,
				// чтобы при расчете базы мы не получили ноль при сложении данных
				// сумм с их "близнецами" с минусом, которые имеются в
				// расчетном м-це
			ExecuteSQL(strSQL);

			strSQL = "UPDATE "+this.allName
					+" SET FSHT_ID = ("
						+" SELECT DISTINCT(WP.FID) FROM ^WORK_PERS WP WHERE "+this.allName+".FTABN_ID = WP.FWID_CL"
							+" AND "+this.allName+".FWID_WP = WP.FWID"
							/*
							+" AND ((WP.FBDAT <= " + sqlTo(m_bdat) + " AND WP.FEDAT >= " + sqlTo(m_edat) +") "
							+ "OR (WP.FBDAT >= " + sqlTo(m_bdat) + " AND WP.FBDAT <= " + sqlTo(m_edat) +") "
							+ "OR (WP.FEDAT >= " + sqlTo(m_bdat) + " AND WP.FEDAT <= " + sqlTo(m_edat) +"))"
							*/
					+" )"
					+", FTABN = ("
						+" SELECT DISTINCT(REL.FCOD) FROM ^CLRELCLCL REL WHERE "+this.allName+".FTABN_ID = REL.FWID_CL"
					+" )"
			ExecuteSQL(strSQL);	
		}
	}
	
	
	tblMn.close(5652);
	tblMn = null;

	if(this.isShtatNeeded) {
		// Проставим информацию из штатного расписания
		this.setShtatInfo();
	}
	
	return this;
}

// Возвращает открытую таблицу-объект с результатами расчета
// mode - режим открытия таблицы
//	"s"	- single(default)
//	"m" - multi
function DpZrpBaseSum_getTbl(mode){
	if(isEmpty(mode)){
		mode = "s";
	}
	return OpenTable(mode, this.tblName);
}

// Возвращает имя таблицы с результатами расчета
function DpZrpBaseSum_getTblName(){
	return this.tblName;
}

function DpZrpBaseSum_getTblOgrName(){
	return this.ogrName;
}

// Возвращает имя таблицы со сводным SVZP
function DpZrpBaseSum_getAllName(){
	return this.allName;
}
// Возвращает имя таблицы со списком используемых месяцев
function DpZrpBaseSum_getMonthName(){
	return this.monthName;
}

// Устанавливает имя таблицы с результатами расчета
function DpZrpBaseSum_setTblName(name){
	this.tblName = name;

	return this;
}

// Устанавливает имя таблицы с результатами расчета
function DpZrpBaseSum_setTblOgrName(name){
	this.ogrName = name;

	return this;
}

// Устанавливает имя таблицы со сводным SVZP
function DpZrpBaseSum_setAllName(name){
	this.allName = name;

	return this;
}

// Устанавливает имя таблицы со списком используемых месяцев
function DpZrpBaseSum_setMonthName(name){
	this.monthName = name;

	return this;
}

// Установить фильтр для кодов полей ЛС, которые
// входят в базовую сумму
// Фактически будет сформирован кусок выражения WHERE
// в котором таблица LIC_FLD будет под псевдонимом LIC
// mode - режим
// bCond в запрос будем подставлять:
// "" 		- в случае "ДА"
// "NOT"	- в случае "НЕТ"
// передается true или false

// 	"LIST"		в par1 содержится строка, в которой явно перечислены через
//				запятую коды полей ЛС, входящих в базовую сумму
//				в par2 содержится строка, в которой явно перечислены через
//				запятую коды полей ЛС, НЕ входящих в базовую сумму
// 	"FILTER"	в par1 содержится имя поля из LIC_FLD,
//				а в par2 - список значений в данном поле, подходящий для 
//				использования в предложении "IN" SQL-запроса
//				В базовую сумму будут отобраны все поля ЛС, имеющие в поле,
//				имя которого хранится в par1 одно из значений, перечисленных 
//				в par2.
// "PFP"		Пенс. фонд с предприятия
// "PFL"		Пенс. фонд с работников
// "PF"			Сам Пенс. фонд с работников
// "FZP"		Фонд занятости с предприятия
// "FZL"		Фонд занятости с работников
// "FZ"			Сам Фонд занятости с работников
// "FSSP"		Фонд соц.страхования с предприятия
// "FSSL"		Фонд соц.страхования с работников
// "FSS"		Сам Фонд соц.страхования с работников
// "FSNVP"		Фонд соц.страхования от несч. случая (ФСНВ) с предприятия
// "ALIM"		Алименты
// "NACH"		Всего начислено
// "UDER"		Всего удержано
// "VYD"		К выдачен на руки
function DpZrpBaseSum_setFields(mode, par1, par2, bCond4query){
	mode = mode.toUpperCase();
	
	if (typeof(bCond4query) == "undefined" && typeof(bCond4query) != "Boolean")
	{
		bCond = " ";
	}
	else
	{
		bCond = bCond4query == true ? " " : " NOT ";
	}

	this.aFilters["FLS_INCL"] = "";
	this.aFilters["FLS_EXCL"] = "";
	this.aFilters["FLS_EXPR"] = "";
	
	switch(mode){
	case "LIST":
		if(!isEmpty(par1)) {
			this.aFilters["FLS_INCL"] = par1;
		}
		if(!isEmpty(par2)) {
			this.aFilters["FLS_EXCL"] = par2;
		}
		break;
	case "FILTER":
		sField = par1;	//имя поля из LIC_FLD
		sValues = par2;	//список значений в данном поле
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC."+sField+" IN("+sValues+") ";
		break;
	case "PFP":	//Пенс. фонд с предприятия
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FIS20='+' AND LIC.FIS6 = '+' ";
		break;
	case "PFL":	//Пенс. фонд с работников
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FIS8 IN('+','*','&','?') AND LIC.FIS6 = '+' ";
		break;
	case "PF":	//Сам Пенс. фонд с работников
		this.aFilters["FLS_EXPR"] = " AND LIC.FIS8='п' ";
		break;
	case "FZP":	//Фонд занятости с предприятия
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FIS23='+'  AND LIC.FIS6 = '+' ";
		break;
	case "FZL":	//Фонд занятости с работников
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FIS8 IN('+','*','$','%') AND LIC.FIS6 = '+' ";
		break;
	case "FZ":	//Сам Фонд занятости с работников
		this.aFilters["FLS_EXPR"] = " AND LIC.FIS1='з' ";
		break;
	case "FSSP":	//Фонд соц.страхования с предприятия
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FSOC='+' AND LIC.FIS6 = '+' ";
		break;
	case "FSSL":	//Фонд соц.страхования с работников
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FIS17='+' AND LIC.FIS6 = '+' ";
		break;
	case "FSS":	//Сам Фонд соц.страхования с работников
		this.aFilters["FLS_EXPR"] = " AND LIC.FIS8='с' ";
		break;
	case "FSNVP":	//Фонд соц.страхования от несч. случая (ФСНВ) с предприятия
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FNSL='+' AND LIC.FIS6 = '+' ";
		break;
	case "ALIM":	//Алименты
		this.aFilters["FLS_EXPR"] = " AND LIC.FIS5='+' ";
		break;
	case "NACH":	//Всего начислено
		this.aFilters["FLS_EXPR"] = " AND LIC.FIS6='+' ";
		break;
	case "UDER":	//Всего удержано
		this.aFilters["FLS_EXPR"] = " AND LIC.FIS6='у' ";
		break;
	case "VYD":		//К выдачен на руки
		this.aFilters["FLS_EXPR"] = " AND LIC.FIS6='ф' ";
		break;
	/*
		1 - найманий працівник на загальних підставах
		2 - працюючий інвалід
		26 - Для осіб, що працюють за угодами ЦПХ
		29 - Особи, які отримують допомогу по тимчасовій непрацездатності
	*/
	case "ESVP":	// База ЕСВ с предприятия (инвалидов ограничивать "Таб.№ для или кроме" в настройке самой разноски)
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" (LIC.FIS30='+' AND LIC.FKATESV IN (0,1,2)) ";
		
		// фильтр для базы ЕСВ всех типов начислений стоящих над расчитываемым типом. В данном случае основная ЗП и над нами ничего нет
		this.aFilters["FLS_EXPR_ESV_ALL"] = " AND "+bCond+" (LIC.FIS30='+' AND LIC.FKATESV IN (0,1,2))";
		this.bIsESV = true;
		break;
	case "ESV_CPH":	// База ЕСВ с ЦПХ
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FIS30='+' AND LIC.FKATESV = 26 ";
		
		// фильтр для базы ЕСВ всех типов начислений стоящих над расчитываемым типом.
		this.aFilters["FLS_EXPR_ESV_ALL"] = " AND "+bCond+" (LIC.FIS30='+' AND LIC.FKATESV = 26 "
											+" OR LIC.FIS30='+' AND LIC.FKATESV IN (0,1,2))"
		this.bIsESV = true;
		break;
	case "ESV_BLN":	// База больничные
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" LIC.FIS30='+' AND LIC.FKATESV = 29 ";
		
		// фильтр для базы ЕСВ всех типов начислений стоящих над расчитываемым типом.
		this.aFilters["FLS_EXPR_ESV_ALL"] = " AND "+bCond+" (LIC.FIS30='+' AND LIC.FKATESV IN (0,1,2) "
											+" OR LIC.FIS30='+' AND LIC.FKATESV = 26 "
											+" OR LIC.FIS30='+' AND LIC.FKATESV = 29  )"
		this.bIsESV = true;
		break;
	case "ESV":		//Удержание ЕСВ с работников (вместе с "бывшими" фондами, т.к. они будут из переходящих отпусков 2010 года на 2011)
		this.aFilters["FLS_EXPR"] = " AND "+bCond+" (LIC.FIS8 IN ('5', '6', '7')"		// ЕСВ
										+" OR LIC.FIS8='п' "				// ПФ
										+" OR LIC.FIS1='з'"					// ФЗ
										+" OR LIC.FIS8='с'"					// СС
									+")"
		break;
	default:	
		break;
	}
	return this;
}

// Установить выражение для вычисления верхней границы для базовых сумм
// mode - режим вычисления
//		"VOZN" - вычислять для месяца возникновения сумм (м-ца "за который")
//		"VYPL" - вычислять для месяца выплаты сумм (м-ца "в котором")

// или для фондов ставит сответствующую константу
// "PFP"		Пенс. фонд с предприятия
// "PFL"		Пенс. фонд с работников
// "PF"			Сам Пенс. фонд с работников
// "FZP"		Фонд занятости с предприятия
// "FZL"		Фонд занятости с работников
// "FZ"			Сам Фонд занятости с работников
// "FSSP"		Фонд соц.страхования с предприятия
// "FSSL"		Фонд соц.страхования с работников
// "FSS"		Сам Фонд соц.страхования с работников
// "FSNVP"		Фонд соц.страхования от несч. случая (ФСНВ) с предприятия

function DpZrpBaseSum_setOgr(mode, sExpr){
	if(isEmpty(mode)){
		mode = "VOZN";
	}
	mode = mode.toUpperCase();
	
	// ограничивать ли сумму - берем из настройки
	
	var bOgr = true;
	switch (mode)
	{
		case "PFP":
		case "PFL":
		case "PF":
		
		case "ESVP":
		case "ESV_CPH":
		case "ESV_BLN":
		case "ESV":
			bOgr = get_cnst("pns1000", this.date) == 1 ? true : false;
			break;
		case "FZP":
		case "FZL":
		case "FZ":
			bOgr = get_cnst("zan1000", this.date) == 1 ? true : false;
			break;
		case "FSSP":
		case "FSSL":
		case "FSS":
			bOgr = get_cnst("soc1000", this.date) == 1 ? true : false;
			break;
		case "FSNVP":
			break;
	}
	// режим для вычисления верхней границы для базовых сумм
	//this.sumOgrMode = mode;
	this.setSumOgrMode(mode);
	// выражение для вычисления верхней границы для базовых сумм
	if (bOgr)
	{
		this.sumOgrExpr = get_cnst("sum_ogr", this.date);
	}

	return this;
}

// Установить выражение для вычисления процента
function DpZrpBaseSum_setProc(sExpr){
	this.procExpr = sExpr;

	return this;
}


// 	Установить внешний DpEval
function DpZrpBaseSum_setEval(obj){
	this.oEval = obj;

	return this;
}

// Собственно расчет сумм
function DpZrpBaseSum_doCalc()
{
	if(this.isShtatNeeded) {
		// Проставим информацию из штатного расписания
		this.setShtatInfo();
	}

	//Служебное место: подготовим список констант
	var helpConstTbl = 	this.monthName+"_const";
	DropTable(helpConstTbl);
	
	if(IsTable(helpConstTbl)){
		// Не удалилась - ну и не надо. Переживем.
		strSQL = "DELETE FROM "+helpConstTbl;
		ExecuteSQL(strSQL);
	} else {
		// Создадим.
		var fld = new Object();
		fld.FCONST_NAM = "TEXT";
		fld.FSUM = "MONEY";
		CreateTable(helpConstTbl, fld);
/* Net  не вижу в нем смысла. таблицу уже создали
		strSQL = " INSERT INTO "+helpConstTbl+" (FCONST_NAM, FSUM)"
				+" SELECT "
					+" FCONST_NAM" 
					+", "+Convert(0, "MONEY")+" AS FSUM"
				+" FROM ^CONST"
				+" WHERE 1=0"
		ExecuteSQL(strSQL);
*/
	}

	strSQL = " INSERT INTO "+helpConstTbl
			+" (FCONST_NAM, FSUM)"
			+" SELECT DISTINCT"
				+" FCONST_NAM" 
				+", "+Convert(0, "MONEY")+" AS FSUM"
			+" FROM ^CONST"
	ExecuteSQL(strSQL);

	// Для начала рассчитаем значение ограничения и процента
	// для каждого из месяцев
	strSQL = " SELECT * "
		+" FROM "+this.monthName
		+" ORDER BY FMN"
	var tblMn = OpenTable(strSQL);

	for(; !tblMn.IsEOF(); tblMn.MoveNext()){
		var mn = new Date(tblMn.getValue("FMN"));
		
		// вычисляем значение ограничения
		var mFogr = 0;
		var mFproc = 100;
		
		var evalOgr = true;
		var evalProc = true;
		
		if(this.sumOgrExpr==""){
			// Ограничение не задано - ну и не надо
			mFogr = 99999999999999;
			evalOgr = false;
		} else 	if (!isNaN(Number(this.sumOgrExpr))){
			// Явно задано число
			mFogr = Number(this.sumOgrExpr);
			evalOgr = false;
		}else{
			evalOgr = true;
		}

		if(this.procExpr==""){
			// Процент не задан - ну и не надо
			mFproc = 100;
			evalProc = false;
		} else 	if (!isNaN(Number(this.procExpr))){
			// Явно задано число
			mFproc = Number(this.procExpr);
			evalProc = false;
		}else{
			evalProc = true;
		}

		if(evalOgr || evalProc) {
			// Имеем выражение - будем эвалить
			// Проблема: бывают случаи, когда период действия
			// константы закончился и в следующих м-цах ее в списке просто нет.
			// Тогда просто перебивая константы в Эвале мы не затрагиваем
			// "умершие" константы и при вычислении выражений будет
			// использовано "старое" начение. А по-хорошему должен быть ноль
			// Поэтому воспользуемся списком переменных
			// и сначала обнулим значения, а лишь затем проинициализируем значениями
			
			strSQL = "UPDATE "+helpConstTbl+" SET"
				+" FSUM=0"
			ExecuteSQL(strSQL);
			
			switch (this.getConstMode())
			{
				case "VYPL":
					month = this.date;
				break;
				case "VOZN":
				default:
					month = mn;
				break;
			}
			
			this.oEval.addVar("FCONST_DAT", month);
			
			var m_bdat = firstDay(new Date(month));
			var m_edat = lastDay(new Date(month));
			
			strSQL = "UPDATE "+helpConstTbl
					+" SET FSUM = ("
						+" SELECT N.FSUM FROM ^CONST N WHERE "+helpConstTbl+".FCONST_NAM = N.FCONST_NAM"
							+" AND ((N.FBDAT <= " + sqlTo(m_bdat) + " AND N.FEDAT >= " + sqlTo(m_edat) +") "
							+" OR (N.FBDAT >= " + sqlTo(m_bdat) + " AND N.FBDAT <= " + sqlTo(m_edat) +") "
							+" OR (N.FEDAT >= " + sqlTo(m_bdat) + " AND N.FEDAT <= " + sqlTo(m_edat) +"))"
							+" )"
			ExecuteSQL(strSQL);
			
			var tblCnst  = OpenTable(helpConstTbl)

			for (; !tblCnst.IsEOF(); tblCnst.MoveNext()){
				this.oEval.addVar(toScrObj(tblCnst.getValue("FCONST_NAM")), toScrObj(tblCnst.getValue("FSUM")));
			}
			tblCnst.close(5652);
			tblCnst = null;
			if(evalOgr) {
				mFogr = this.oEval.eval(this.sumOgrExpr);
			}
			if(evalProc) {
				mFproc = this.oEval.eval(this.procExpr);
			}
		}
		
		var curFmn = new Date(tblMn.getValue("FMN"));
		// Запишем полученные значения
		var m = new Object();
		m.FOGR = Number(mFogr);
		m.FPROC = Number(mFproc);
		UpdateTable(this.monthName, m, false, " FMN = "+sqlTo(curFmn));
	}

	tblMn.close(5652);
	tblMn = null;

	//alert("this.allName: "+this.allName);
	//browse(this.allName)
	
	//Выбираем из this.allName строки, с учетом фильтра, учитывающего все,
	//кроме условия на выражение
	//Полученную таблицу будем использовать для вычисления значения выражения
	var tmpAll = this.tblName+"_tmpAll";
	DropTable(tmpAll);
	CreateTable(tmpAll, "VSP_VS2", "*");
	
	strSQL = " INSERT INTO " + tmpAll + getColumn(this.allName, true)
			+" SELECT "+getColumn(this.allName, false, "A")
		+" FROM "
			+this.allName+" A "
			+", ^LIC_FLD LIC "
		+" WHERE "
			+" A.FLSFID=LIC.FID "
			+this.getWhere("NOEXPR", "A");
	ExecuteSQL(strSQL);
	
	//strSQL = "ALTER TABLE "+tmpAll+" ADD FBAND " + SqlType("LONG")
	//ExecuteSQL(strSQL);

	//strSQL = "CREATE INDEX FLSFID ON "+tmpAll+"(FLSFID)"
	//ExecuteSQL(strSQL, false);
	
	//strSQL = "CREATE INDEX FTABN ON "+tmpAll+"(FTABN)"
	//ExecuteSQL(strSQL, false);
	
	//strSQL = "CREATE INDEX FSUM ON "+tmpAll+"(FSUM)"
	//ExecuteSQL(strSQL, false);

	//strSQL = "ALTER TABLE "+tmpAll+" ADD COLUMN FWID "+SqlType("COUNTER")
	//ExecuteSQL(strSQL);

	//alert("tmpAll: "+tmpAll);
	//browse(tmpAll)
	
	if(this.aFilters["FEXP_INCL"]!="" || this.aFilters["FEXP_EXCL"]!="" || this.aGrp["FEXPR"]!="") 
	{
		//Задан фильтр или группировка по значению выражения.
		//Проставляем значение выражения
		this.setExprValues(tmpAll);
	
		// Теперь удалим строки, которые не подходят под
		// фильтр по выражению
		strWhere = this.getWhere("EXPR", "TMP");
		
		if(strWhere!="")
		{
			if (SUBD == MSSQL)
			{
				strSQL = " DELETE TMP "
						+" FROM " + tmpAll + " TMP "
							+" INNER JOIN LIC_FLD L ON NOT "+strWhere
				ExecuteSQL(strSQL);
			}
			else
			{
				strSQL = "DELETE FROM "+ tmpAll +" TMP"
						+" WHERE EXISTS (SELECT 1 FROM ^LIC_FLD L WHERE NOT "+strWhere+")"
				ExecuteSQL(strSQL);
			}
			/*
			strSQL = "DELETE A.*"
			+" FROM "
				+tmpAll+" A "
				+", ^LIC_FLD LIC "
			+" WHERE NOT (1=1 " + strWhere + ")"
			ExecuteSQL(strSQL);
			*/
		}
	}

	/////////////////////////////////////////////////////////////////////
	// Формирование вспомогательной таблицы с ограничениями
	/////////////////////////////////////////////////////////////////////
	var tmp1 = this.tblName+"_1";
	DropTable(tmp1);
	if (!this.bIsESV)
	{

		// Создадим вспомогательную таблицу
		// в которой будет по одной строке
		// для каждой связки "м-ц файла"-"м-ц в котором"-"м-ц за который"
		var fld = new Object();
		fld.FSUM = "MONEY";
		fld.FSUMALL = "MONEY";
		fld.FSUMOGR = "MONEY";
		fld.FSUMDIFF = "MONEY";
		fld.FPROC = "DOUBLE";
		fld.FTABN = "LONG";
		fld.FMES = "DATE";
		fld.FMF = "DATE";
		fld.FMN = "DATE";
		CreateTable(tmp1, fld);

		strSQL = " INSERT INTO "+tmp1
					+" (FSUM, FSUMALL, FSUMOGR, FSUMDIFF, FPROC, FTABN, FMES, FMF, FMN)"
				+" SELECT"
					+" SUM(A.FSUM) AS FSUM"
					// Место, куда будем писать сумму, накопленную по предыдущим м-цам с учетом текущей
					+", "+Convert(0, "MONEY")+" AS FSUMALL"
					// Место, куда будем писать FSUMALL с учетом ограничений
					+", "+Convert(0, "MONEY")+" AS FSUMOGR"
					// Место, куда будем писать довесок к базовой сумме, пришедший из данной строки
					+", "+Convert(0, "MONEY")+" AS FSUMDIFF"
					// Процент
					+", "+Convert(100, "MONEY")+" AS FPROC"
					+", A.FTABN"
					+", A.FMES"		// месяц файла
					+", A.FMF"		// м-ц за который
					+", A.FMN"		// м-ц в котором
				+" FROM "
					+tmpAll+" A"
				+" GROUP BY"
					+" A.FTABN"
					+", A.FMES"		// месяц файла
					+", A.FMF"		// м-ц за который
					+", A.FMN"		// м-ц в котором
		ExecuteSQL(strSQL);


		//browse(tWork)
		// Привяжем таблицу месяцев, в которой у нас хранятся значения вычисленных
		// сумм ограничений.
		// Вяжем дважды:
		// 1) первый раз - для сумм будущих месяцев, которые ограничивались
		// при расчете зарплаты суммой ограничения, действовавшей в месяце
		// расчета ЗП
		// 2) второй раз - для сумм текущих и за прошлое, которые ограничивались
		// исходя из следующих значений ограничений:
		//	а) при this.sumOgrMode=="VOZN" - действовавших в месяце возникновения сумм 
		//		(м-це "за который")
		//	б) при this.sumOgrMode=="VYPL" - действовавших в месяце выплаты сумм 
		//		(м-це "в котором")
		// КСТАТИ, месяцем выплаты считаем не месяц файла FMF, а месяц FMN ("в котором")
		// Это связано с тем, что в строках с суммами отпусков на будущее, которые показываются
		// с "минусом" в месяце, за который они были выплачены, месяц выплаты
		// хранится именно в поле FMF
		
		var tmp1_tot = this.tblName+"_1_tot";
		DropTable(tmp1_tot);

		// fld - определено выше, добавляем еще поля
		fld.OGRBUD_FOGR = "MONEY";
		fld.OGRBUD_FPROC = "DOUBLE";
		fld.OGRPROSH_FOGR = "MONEY";
		fld.OGRPROSH_FPROC = "DOUBLE";
		CreateTable(tmp1_tot, fld);

		strSQL = " INSERT INTO "+tmp1_tot+" ("+getColumn(tmp1, false)+", OGRBUD_FOGR, OGRBUD_FPROC, OGRPROSH_FOGR, OGRPROSH_FPROC)"
				+" SELECT "+getColumn(tmp1, false, "S")
					+", M1.FOGR AS OGRBUD_FOGR"
					+", M1.FPROC AS OGRBUD_FPROC"
					+", M2.FOGR AS OGRPROSH_FOGR"
					+", M2.FPROC AS OGRPROSH_FPROC"
				+" FROM "
					+tmp1+" S"
					+" LEFT JOIN "+this.monthName+" M1 ON S.FMN = M1.FMN"
					+" LEFT JOIN "+this.monthName+" M2 ON S.FMF = M2.FMN"
				+" ORDER BY "
					+" S.FTABN"
					+", S.FMF"		// м-ц за который
					+", S.FMN"		// м-ц в котором
					+", S.FMES"		// месяц файла
		ExecuteSQL(strSQL);
		
		var tWork = OpenTable("m", tmp1_tot);

		// А теперь пробегаем по строкам и вычисляем суммы с учетом ограничений
		if(!tWork.IsEmpty()){
			tWork.MoveFirst();
		}
		
		//Значение FMF из предыдущей итерации
		var prevMf = d00;
		var prevTabn = -1;
		
		//Накопленные суммы "за месяц" 
		//без ограничения и с ограничением соответственно
		var sumBezOgr = 0;
		var sumSOgr = 0;
		for( ; !tWork.IsEOF(); tWork.MoveNext())
		{
			mFmes = toScrObj(tWork.getValue("FMES"));	//м-ц файла
			mFmn = toScrObj(tWork.getValue("FMN"));	//м-ц "в котором"
			mFmf = toScrObj(tWork.getValue("FMF"));	//м-ц "за который"
			mFtabn = toScrObj(tWork.getValue("FTABN"));//табельный номер

			// Обнулим суммы
			if( !(prevMf<=mFmf && prevMf>=mFmf)		//месяц "за который" изменился
				|| prevTabn!=mFtabn	//человек изменился
				|| mFmf > mFmes		//сумма на будущее
				|| mFmes<=mFmf && mFmes>=mFmf &&  mFmn<mFmf	//сторно суммы, выплаченной в прошлом за наш м-ц
				|| mFmes<=mFmf && mFmes>=mFmf &&  mFmn<=mFmf && mFmn>=mFmf	//Все за текущий м-ц
			){
				sumBezOgr = 0;
				sumSOgr = 0;
			}



			// Определимся с алиасом таблицы с ограничениями
			if( mFmn < mFmf ){
				// суммы на будущее
				sOgrAlias = "OGRBUD";
			} else {
				// суммы текущего м-ца и за прошлое
				sOgrAlias = "OGRPROSH";
			}

			// Значение суммы ограничения
			mFogr = round(toScrObj(tWork.getValue(sOgrAlias+"_FOGR")), 2);
			// Значение процента
			mFproc = toScrObj(tWork.getValue(sOgrAlias+"_FPROC"));

			// Сумма по строке
			mFsum = round(toScrObj(tWork.getValue("FSUM")), 2);
			sumBezOgr += mFsum;
			mFsumall = sumBezOgr;
			if(mFmes>=mFmf&&mFmes<=mFmf && mFmn<mFmf && mFsum<0)	// это случай, когда имеем сумму выплаченную в прошлом м-це за текущий
			{
				mFsumogr = (mFsumall < -mFogr && mFogr>0) ? -mFogr : mFsumall;
			}
			else
			{
				mFsumogr = (mFsumall > mFogr && mFogr>0) ? mFogr : mFsumall;
			}
			mFsumogr = round(mFsumogr, 2);
			mFsumdiff = mFsumogr-sumSOgr;
			
			var m = new Object();
			m.FSUMDIFF = round(Number(mFsumdiff), 2);
			m.FSUMALL = round(Number(mFsumall), 2);
			m.FSUMOGR = round(Number(mFsumogr), 2);
			m.FPROC = round(Number(mFproc), 2);
			var strWhere = " FMES = "+sqlTo(mFmes)+" AND FMN = "+sqlTo(mFmn)+" AND FMF = "+sqlTo(mFmf)+" AND FTABN = "+sqlTo(mFtabn);
			UpdateTable(tmp1_tot, m, false, strWhere);

			// Подготовимся к следующей итерации
			prevMf = mFmf;
			//sumBezOgr = mFsumall;
			sumSOgr = mFsumogr;
			prevTabn = mFtabn;
		}
		tWork.close(5652);
		tWork = null;
	}//if (!this.bIsESV)
	else
	{
		// с 2011 года надо ограничивать по-хитрому:
		// Принцип деления базы и удержания (начисления) ЕВ при превышении 15 прож.минимумов:
		// сначала из ограничения вычитаем ЗП, из остатка вычитаем трудовые соглашения и остаток ограничения берем на больничные.

		// сумма всех предыдущих типов + расчитываемый
		var tmpBaseTbl = "tmpBaseTbl";
		DropTable(tmpBaseTbl);
		strSQL = "SELECT "
					+" SUM(S.FSUM) AS FSUM"
					+", S.FTABN"
					+", S.FMES"		// месяц файла
					+", S.FMF"		// м-ц за который
					+", S.FMN"		// м-ц в котором
				+" INTO "+tmpBaseTbl
				+" FROM "
					+this.allName+" S "
					+", ^LIC_FLD LIC "
				+" WHERE "
					+" S.FLSFID = LIC.FID "
					+this.getWhere("ESV_ALL", "S")
				+" GROUP BY"
					+" S.FTABN"
					+", S.FMES"		// месяц файла
					+", S.FMF"		// м-ц за который
					+", S.FMN"		// м-ц в котором
		ExecuteSQL(strSQL);
		
		// сумма расчитываемого типа начислений
		var tmpTbl4ThisType = "tmpTbl4ThisType";
		DropTable(tmpTbl4ThisType);
		strSQL = "SELECT "
					+" SUM(S.FSUM) AS FSUM"
					+", S.FTABN"
					+", S.FMES"		// месяц файла
					+", S.FMF"		// м-ц за который
					+", S.FMN"		// м-ц в котором
				+" INTO "+tmpTbl4ThisType
				+" FROM "
					+this.allName+" S "
					+", ^LIC_FLD LIC "
				+" WHERE "
					+" S.FLSFID = LIC.FID "
					+this.getWhere("NOEXPR", "S")
				+" GROUP BY"
					+" S.FTABN"
					+", S.FMES"		// месяц файла
					+", S.FMF"		// м-ц за который
					+", S.FMN"		// м-ц в котором
		ExecuteSQL(strSQL);
		
		// соберем все в одну таблицу
		var tmpTblAll = "tmpTblAll";
		DropTable(tmpTblAll);
		strSQL = "SELECT "
					+" T.FSUM AS FSUM_TYPE"
					+", A.FSUM AS FSUM_ALL"
					+", OGR.FOGR AS FOGR"
					+", OGR.FPROC AS FPROC"
					+", T.FTABN"
					+", T.FMES"
					+", T.FMF"
					+", T.FMN"
				+" INTO "+tmpTblAll
				+" FROM (("+tmpTbl4ThisType+" T"
					+" LEFT JOIN "+tmpBaseTbl+" A "
						+" ON T.FTABN = A.FTABN"
						+" AND T.FMES = A.FMES"
						+" AND T.FMF = A.FMF"
						+" AND T.FMN = A.FMN)"
							+" LEFT JOIN "+this.monthName+" OGR ON T.FMN = OGR.FMN)"
				+" WHERE T.FSUM IS NOT NULL"
		ExecuteSQL(strSQL);

		//vlad 31.03.2011 13:09:17
		//для того, чтобы для сумм за прошлое учесть возможное ограничение,
		//в FSUM_ALL добавим сумму всех сум из строк, которые из SVZP прошлых месяцев
		DropTable("TMP_SUM_TYPE_PREV");
		strSQL = "SELECT "
					+" SUM(S.FSUM_TYPE) AS FSUM_TYPE"
					+", S.FTABN"
					+", S.FMF"
				+" INTO TMP_SUM_TYPE_PREV"
				+" FROM "+tmpTblAll+" S "
				+" GROUP BY "
					+" S.FTABN"
					+", S.FMF"
		ExecuteSQL(strSQL);

/*
		strSQL = "UPDATE "
					+tmpTblAll+" A "
					+", TMP_SUM_TYPE_PREV S"
				+" SET "
					+" A.FSUM_ALL=S.FSUM_TYPE"
				+" WHERE 1=1"
					+" AND A.FMF = S.FMF"
					+" AND A.FMES > S.FMF"
					+" AND A.FTABN = S.FTABN"
		ExecuteSQL(strSQL);
*/		
		strSQL = "UPDATE "+tmpTblAll
			+" SET "
				+" FSUM_ALL = (SELECT S.FSUM_TYPE FROM TMP_SUM_TYPE_PREV S "
				+" WHERE "+tmpTblAll+".FTABN = S.FTABN AND "+tmpTblAll+".FMF = S.FMF AND "+tmpTblAll+".FMES > S.FMF)"
				+" WHERE 1=1"
					+" AND EXISTS (SELECT S2.FSUM_TYPE FROM TMP_SUM_TYPE_PREV S2 "
							+" WHERE "+tmpTblAll+".FTABN = S2.FTABN AND "+tmpTblAll+".FMF = S2.FMF AND "+tmpTblAll+".FMES > S2.FMF)"
		ExecuteSQL(strSQL);

		var tmp1_tot = this.tblName+"_1_tot";
		DropTable(tmp1_tot);
		strSQL = "SELECT "
					+" A.FTABN"
					+", A.FMES"
					+", A.FMF"
					+", A.FMN"
					+", A.FSUM_TYPE AS FSUM"
					+", A.FSUM_TYPE AS FSUMALL"
					+", "+IIF("(A.FMES = A.FMF AND A.FMN < A.FMF AND A.FSUM_TYPE < 0)"
						, sqlMax("A.FSUM_TYPE", "-(A.FOGR-A.FSUM_ALL+A.FSUM_TYPE)")
							,IIF("A.FSUM_TYPE>0 ", sqlMax("0", sqlMin("A.FSUM_TYPE", "A.FOGR-A.FSUM_ALL+A.FSUM_TYPE"))
								, sqlMin("0", sqlMax("A.FSUM_TYPE", "A.FSUM_ALL-A.FOGR"))))
						+" AS FSUMOGR"
					+", A.FPROC AS FPROC"
					+", "+Convert(0, "MONEY")+" AS FSUMDIFF"
				+" INTO "+tmp1_tot
				+" FROM "+tmpTblAll+" A"
		ExecuteSQL(strSQL);
		
		strSQL = "UPDATE "+tmp1_tot+" SET FSUMDIFF = FSUMOGR"
		ExecuteSQL(strSQL);
	}


	// Формируем итоговую таблицу ограничений
	DropTable(this.ogrName);
	var fld = new Object();
	fld.FTABN = "LONG";
	fld.FMN = "DATE";
	fld.FMF = "DATE";
	fld.FSUM = "MONEY";
	fld.FSUMBEZOGR = "MONEY";
	fld.FRAZNSUM = "MONEY";
	fld.FSUMALL = "MONEY";
	fld.FPROC = "MONEY";
	fld.FBAND = "LONG";
	CreateTable(this.ogrName, fld);

	strSQL = " INSERT INTO " + this.ogrName + "(FTABN, FMF, FMN, FSUM, FSUMBEZOGR, FRAZNSUM, FSUMALL, FPROC)"
			+" SELECT "
				+" S.FTABN"
				+", S.FMF"		// м-ц за который
				+", S.FMN"		// м-ц в котором
				+", S.FSUMDIFF AS FSUM"	//базовая сумма
				+", S.FSUM AS FSUMBEZOGR"	//сумма без наложения ограничений
				+", S.FSUM - S.FSUMDIFF AS FRAZNSUM"
				+", S.FSUMALL"	//полная сумма за месяц без наложения ограничений (с учетом начислений за все предыд.м-цы)
				+", S.FPROC "	//Процент
			+" FROM "
				+tmp1_tot+" S"
			+" WHERE "
				+" S.FMES="+sqlTo(this.date)		// месяц файла
	ExecuteSQL(strSQL);
	
	// Проставим в tmpAll сумму с учетом ограничения
	// (только для тех людей, у кого действительно произошло ограничение)
	DropTable("VSRAZN__SUM");
	var fld = new Object();
	fld.FLSFID = "LONG";
	fld.FPROC = "DOUBLE";
	fld.O_FSUM = "MONEY";
	fld.FSUMBEZOGR = "MONEY";
	fld.FTABN = "LONG";
	fld.V_FSUM = "MONEY";
	fld.FMN = "DATE";
	fld.FMF = "DATE";
	fld.FSUM = "MONEY";
	CreateTable("VSRAZN__SUM", fld);
	
	strSQL = " INSERT INTO VSRAZN__SUM (FLSFID, FPROC, O_FSUM, FSUMBEZOGR, FTABN, V_FSUM, FMN, FMF, FSUM)"
			+" SELECT V.FLSFID, O.FPROC, O.FSUM AS O_FSUM, O.FSUMBEZOGR, V.FTABN, V.FSUM AS V_FSUM, V.FMN, V.FMF, "+sqlRound("V.FSUM*O.FPROC/100*O.FSUM/O.FSUMBEZOGR", 2)+" AS FSUM"
				+" FROM "+this.ogrName+" O INNER JOIN "+tmpAll+" V "
					+" ON V.FTABN=O.FTABN AND V.FMF=O.FMF AND V.FMN=O.FMN "
					+" WHERE V.FMES="+sqlTo(this.date)
						+" AND O.FSUMBEZOGR <> 0 AND (O.FSUM<>O.FSUMBEZOGR OR O.FPROC<>100 )"
				+" GROUP BY O.FPROC, O.FSUM, O.FSUMBEZOGR, V.FTABN, V.FSUM, V.FMN, V.FMF, V.FLSFID"
	ExecuteSQL(strSQL);

	strSQL = "UPDATE "+tmpAll
		+" SET "
			+" FSUM = (SELECT VS.FSUM FROM VSRAZN__SUM VS "
			+" WHERE "+tmpAll+".FTABN = VS.FTABN AND "+tmpAll+".FMF = VS.FMF AND "+tmpAll+".FMN = VS.FMN AND "+tmpAll+".FLSFID = VS.FLSFID"
				+" AND "+tmpAll+".FSUM = VS.V_FSUM)"
			+" WHERE FMES = "+sqlTo(this.date)
				+" AND EXISTS (SELECT VS2.* FROM VSRAZN__SUM VS2"
							+" WHERE "+tmpAll+".FTABN = VS2.FTABN AND "+tmpAll+".FMF = VS2.FMF AND "+tmpAll+".FMN = VS2.FMN AND "+tmpAll+".FLSFID = VS2.FLSFID"
								+" AND "+tmpAll+".FSUM = VS2.V_FSUM"
								+")"
	ExecuteSQL(strSQL);

	strSQL = "UPDATE "+tmpAll
	+" SET "
		+" FBAND = ("
		+" SELECT DISTINCT "+this.getBandCond("", "O.", "FSUMALL", -1)
		+" FROM "+this.ogrName+" O "
		+" WHERE "
			+" "+tmpAll+".FTABN=O.FTABN"
			+" AND "+tmpAll+".FMF=O.FMF"		// м-ц за который
			+" AND "+tmpAll+".FMN=O.FMN"		// м-ц в котором
			+" AND "+tmpAll+".FMES="+sqlTo(this.date)		// месяц файла
		+" )"
	ExecuteSQL(strSQL);

	strSQL = "UPDATE "+this.ogrName
	+" SET "
		+" FBAND = ("
		+" SELECT DISTINCT O.FBAND "
		+" FROM "+tmpAll+" O "
		+" WHERE "
			+" "+this.ogrName+".FTABN=O.FTABN"
			+" AND "+this.ogrName+".FMF=O.FMF"		// м-ц за который
			+" AND "+this.ogrName+".FMN=O.FMN"		// м-ц в котором
			+" AND O.FMES="+sqlTo(this.date)		// месяц файла
		+" )"
	ExecuteSQL(strSQL);


	// Даже при проценте 100 вощникают ошибки округления
	if(this.procExpr.trim()!="100" || true)
	{

		//vvs 18.04.2005
		// Теперь следует обработать ошибки округления, возникшие вследствие 
		// округления до копеек по каждой строке результата.
		// Если оставить как есть, то 32% с фонда ЗП будут отличаться от
		// сумм 32% в результатах разноски - т.к. на каждый затратный счет - отдельная
		// строка и, как следствие, ошибка округления.
		
		// Создадим таблицу с результатами умножения просуммированной "базы" на процент
		// Это "идеал", к которому следует стремиться
		var sumName = this.ogrName+"_SUM";
		DropTable(sumName);
		var fld = new Object();
		fld.FSUM = "MONEY";
		fld.FSUMREAL = "MONEY";
		fld.FSUMDIFF = "MONEY";
		fld.FSUMPROC = "MONEY";
		fld.FMN = "DATE";
		fld.FMF = "DATE";
		fld.FPROC = "DOUBLE";
		CreateTable(sumName, fld);
		
		strSQL = " INSERT INTO "+sumName+" (FSUM, FSUMREAL, FSUMDIFF, FSUMPROC, FMF, FMN, FPROC)"
				+" SELECT "
					+" SUM(O.FSUM) AS FSUM"
					+", "+Convert(0, "MONEY")+" AS FSUMREAL"
					+", "+Convert(0, "MONEY")+" AS FSUMDIFF"
					+", "+sqlRound(" SUM(O.FSUM)*O.FPROC/100", 2)+" AS FSUMPROC "
					+", O.FMF"		// м-ц за который
					+", O.FMN"		// м-ц в котором
					+", O.FPROC"
				+" FROM "	
					+this.ogrName+" O "
				+" GROUP BY "
					+" O.FMF"		// м-ц за который
					+", O.FMN"		// м-ц в котором
					+", O.FPROC"	// процент
		ExecuteSQL(strSQL);
		
		// Отдельно выбираем из tmpAll таблицу с результатами умножения "базы" на процент
		// - то, что реально получилось.
		var sumName2 = this.ogrName+"_SUM2";
		DropTable(sumName2);
		var fld = new Object();
		fld.FMF = "DATE";
		fld.FMN = "DATE";
		fld.FSUM = "MONEY";
		fld.FPROC = "DOUBLE";
		CreateTable(sumName2, fld);
		
		strSQL = " INSERT INTO "+sumName2+" (FMF, FMN, FSUM, FPROC)"
				+" SELECT "
					+" O.FMF"		// м-ц за который
					+", O.FMN"		// м-ц в котором
					+", SUM(A.FSUM) AS FSUM"
					+", O.FPROC"
				+" FROM "	
					+tmpAll+" A "
					+", "+this.ogrName+" O "
				+" WHERE "
					+" A.FTABN=O.FTABN"
					+" AND A.FMF=O.FMF"		// м-ц за который
					+" AND A.FMN=O.FMN"		// м-ц в котором
					+" AND A.FMES="+sqlTo(this.date)		// месяц файла
				+" GROUP BY "
					+" O.FMF"		// м-ц за который
					+", O.FMN"		// м-ц в котором
					+", O.FPROC"	// процент
		ExecuteSQL(strSQL);
	
		// сведем все в одну таблицу
/*
		strSQL = "UPDATE "
			+sumName+" T1 "
			+", "+sumName2+" T2 "
		+" SET "
			+" T1.FSUMREAL = T2.FSUM"
			+",T1.FSUMDIFF = T1.FSUMPROC-T2.FSUM"	//Разница, возникшая вследствие округлений
		+" WHERE "
			+" T1.FMF=T2.FMF"		// м-ц за который
			+" AND T1.FMN=T2.FMN"		// м-ц в котором
			+" AND T1.FPROC=T2.FPROC"		// м-ц в котором
		ExecuteSQL(strSQL);
*/

		if (SUBD == MSSQL)
		{
			strSQL = "UPDATE T1"
					+" SET "
						+" T1.FSUMREAL = T2.FSUM"
						+",T1.FSUMDIFF = T1.FSUMPROC-T2.FSUM"	//Разница, возникшая вследствие округлений
					+" FROM "
						+sumName+" T1 "
						+", "+sumName2+" T2 "
					+" WHERE "
						+" T1.FMF=T2.FMF"		// м-ц за который
						+" AND T1.FMN=T2.FMN"		// м-ц в котором
						+" AND T1.FPROC=T2.FPROC"		// м-ц в котором
// Net  я так понимаю, этот костыль здесь потому, что таблицы не вязались. выше исправили.
//						+" AND (ROUND(T1.FSUMPROC, 0) = ROUND(T2.FSUM, 0))"
			ExecuteSQL(strSQL);
		}
		else
		{
			strSQL = "UPDATE "+sumName
				+" SET FSUMREAL = ("
					+" SELECT SUM(T2.FSUM) FROM "+sumName2+" T2 "
					+" WHERE "
						+" FMF=T2.FMF"		// м-ц за который
						+" AND FMN=T2.FMN"		// м-ц в котором
						+" AND FPROC=T2.FPROC"		// м-ц в котором
				+" )"
				+" WHERE EXISTS ("
					+" SELECT T2.FSUM FROM "+sumName2+" T2 "
					+" WHERE "
						+" FMF=T2.FMF"		// м-ц за который
						+" AND FMN=T2.FMN"		// м-ц в котором
						+" AND FPROC=T2.FPROC"		// м-ц в котором
				+")"
			ExecuteSQL(strSQL);
		}

		// теперь распределим ошибку округления
		strSQL = "SELECT * FROM "+sumName
		+" WHERE FSUMDIFF<>0"
		var tblDiff = OpenTable(strSQL);
		
		for( ; !tblDiff.IsEOF(); tblDiff.MoveNext())
		{
			var mFmf = toScrObj(tblDiff.getValue("FMF"));
			var mFmn = toScrObj(tblDiff.getValue("FMN"));
			var mFsumdiff = round(toScrObj(tblDiff.getValue("FSUMDIFF")), 2);
			
			if (mFsumdiff>0)
			{
				nShare = 0.01;	// доля, которую будем добавлять
			}
			else
			{
				nShare = -0.01;	// доля, которую будем добавлять
			}
			nCount = round(mFsumdiff/nShare, 0);	// кол-во итераций
			
			// разбросаем разницу по одной копейке на 
			// первые строки результирующей т-цы
			strSQL = "SELECT A.* "
			+" FROM "
				+tmpAll+" A "
			+" WHERE "
				+" A.FMES="+sqlTo(this.date)		// месяц файла
				+" AND A.FMF = "+sqlTo(mFmf)		// м-ц за который
				+" AND A.FMN = "+sqlTo(mFmn)		// м-ц в котором
			var tblScan = OpenTable(strSQL);

			for( ; nCount>0 && !tblScan.IsEOF(); tblScan.MoveNext())
			{
				/*
				var fvopl = toScrObj(tblScan.getValue("FVOPL"));
				var fid = toScrObj(tblScan.getValue("FID"));
				var ftabn = toScrObj(tblScan.getValue("FTABN"));
				var flsfid = toScrObj(tblScan.getValue("FLSFID"));
				var fmes = toScrObj(tblScan.getValue("FMES"));
				var fmf = toScrObj(tblScan.getValue("FMF"));
				var fmn = toScrObj(tblScan.getValue("FMN"));
				var flv1 = toScrObj(tblScan.getValue("FLV1"));
				var flv2 = toScrObj(tblScan.getValue("FLV2"));
				var flv3 = toScrObj(tblScan.getValue("FLV3"));
				var flv4 = toScrObj(tblScan.getValue("FLV4"));
				var flv5 = toScrObj(tblScan.getValue("FLV5"));
				
				var strWhere = " FVOPL = "+sqlTo(fvopl)+" AND FID = "+sqlTo(fid)+" AND FTABN = "+sqlTo(ftabn)+" AND FLSFID = "+sqlTo(flsfid)
							+" AND FMES = "+sqlTo(fmes)+" AND FMF = "+sqlTo(fmf)+" AND FMN = "+sqlTo(fmn)
							+" AND FLV1 = "+sqlTo(flv1)+" AND FLV2 = "+sqlTo(flv2)+" AND FLV3 = "+sqlTo(flv3)+" AND FLV4 = "+sqlTo(flv4)+" AND FLV5 = "+sqlTo(flv5)
				*/
				var m = new Object();
				m.FSUM = round(Number(tblScan.getValue("FSUM"))+nShare, 2);
				m.FWID = toScrObj(tblScan.getValue("FWID"));
				UpdateTable(tmpAll, m, false);
				nCount--;
				
				/*
				tblScan.Edit();
				tblScan.setValue("FSUM", Number(tblScan.getValue("FSUM"))+nShare);
				tblScan.Update();
				nCount--;
				*/
			}
			//browse(tblScan)
			tblScan.Close(5652);
			tblScan = null;
			
		}
		
		tblDiff.Close(5652);
		tblDiff = null;
	}

	
	//browse(tmpAll)
	//--- Закончилось исправление ошибок округления после умножения строк на процент.
	//Теперь выбираем из tmpAll строки уже в таком виде, как они нам нужны - 
	//уже с группировкой
	DropTable(this.tblName);
	var fld = new Object();
	fld.FMF = "DATE";
	fld.FMN = "DATE";
	fld.FSUM = "MONEY";
	fld.FBAND = "LONG";
	fld.FLSFID = "LONG";
	fld.FLV1 = "LONG";
	fld.FLV2 = "LONG";
	fld.FLV3 = "LONG";
	fld.FLV4 = "LONG";
	fld.FLV5 = "LONG";
	fld.FKOD_KAT = "LONG";
	fld.FSHPZ = "TEXT";
	fld.FSHPZ_S = "TEXT";
	fld.FTABN = "LONG";
	fld.FEXPR = "TEXT";
	fld.FVOPL = "LONG";
	fld.FSHT_ID = "LONG";
	fld.FIS_SCIENCE = "BIT";
	fld.FCOD_KZO = "LONG";
	CreateTable(this.tblName, fld);

	strSQL = " INSERT INTO "+this.tblName+" (FMF, FMN, FSUM, FBAND"+this.getSelect("").sRetval_listFlds+")"
			+" SELECT "
				+" A.FMF"		// м-ц за который
				+", A.FMN"		// м-ц в котором
				+", SUM(A.FSUM) AS FSUM "
				+", A.FBAND"	// Номер диапазона
				+this.getSelect("A").sRetval1
			+" FROM "
				+tmpAll+" A "
				+(this.bIsESV ? " INNER JOIN "+tmp1_tot+" T ON A.FTABN = T.FTABN AND A.FMES = T.FMES AND A.FMF = T.FMF AND A.FMN = T.FMN" : "")
			+" WHERE "
				+" A.FMES="+sqlTo(this.date)		// месяц файла
				+this.getWhere("EXPR", "A")
			+" GROUP BY "
				+" A.FMF"		// м-ц за который
				+", A.FMN"		// м-ц в котором
				+", A.FBAND"	// Номер диапазона
				+this.getGroupBy("A")
				+(this.bIsESV ? " HAVING SUM(A.FSUM) <> 0" : "")
	ExecuteSQL(strSQL);
	
	//browse("SELECT * FROM "+this.allName+" ORDER BY FTABN, FLSFID, FSHPZ, FMES, FMN, FMF, FVP, FSUM")
	return this;
}

/////////////////////////////////////////////////////////////////////////////
//								HELPERS
/////////////////////////////////////////////////////////////////////////////

//	пpеобpазует список подpазделений вида
//	"1/1/3, 2, 3/5"
//	в
//	"((V.FLV1=1 AND V.FLV2=1 AND V.FLV3=3) OR (V.FLV1=2) OR (V.FLV1=3 AND V.FLV2=5))"
//	alias - псевдоним таблицы, для котрой строим условие. По умол
function DpZrpBaseSum_getFlvCond(sList, alias) {
	if(isEmpty(alias)) {
		alias = "V."
	}
	var sRetval = "";
	var sLBuff = "";
	
	// убеpем пpобелы
	sList.replace(/ /g,"");
	if(sList.length==0){
		return "";
	}
	// pазложим на подpазделения
	var aPodr = sList.split(",");
	
	// пpойдемся по подpазделениям
	for(var p=0; p < aPodr.length; p++){
		// чистим буфеp
		sLBuff = "";
		// pазложим на уpовни
		var aFlv = aPodr[p].split("/");
		
		for(var l=0; l < aFlv.length; l++){
			sLBuff+=" AND "+alias+"FLV" + (l+1) + "=" + aFlv[l];
		}
		// отбpосим впеpеди стоящий " AND " и обеpнем в скобки
		sRetval += " OR (" + sLBuff.substr(5) + ")";
	}  //end of for(var p=0; p < aPodr.length; p++)
	
	// отбpосим впеpеди стоящий " OR " и обеpнем в скобки
	sRetval = "(" + sRetval.substr(4) + ")";
	
	return sRetval;
}

function DpZrpBaseSum_getSelect(alias)
{
	if(!alias){
		alias = "";
	} else {
		alias = alias+".";
	}
	var retPar = new Object();
	retPar.sRetval1 = "";
	retPar.sRetval_listFlds = "";
	
	// 1. Поле ЛС
	if( this.aGrp["FLSFID"] ){
		retPar.sRetval1+= ","+alias+"FLSFID";
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FLSFID";
	}

	// 2. Подpазделение
	// 2.1 Flv1
	if( this.aGrp["FLV1"] ){
		retPar.sRetval1+= ","+alias+"FLV1";
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FLV1";
	}

	// 2.2 Flv2
	if( this.aGrp["FLV2"] ){
		retPar.sRetval1+= ","+alias+"FLV2";
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FLV2";
	}

	// 2.3 Flv3
	if( this.aGrp["FLV3"] ){
		retPar.sRetval1+= ","+alias+"FLV3";
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FLV3";
	}

	// 2.4 Flv4
	if( this.aGrp["FLV4"] ){
		retPar.sRetval1+= ","+alias+"FLV4";
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FLV4";
	}

	if( this.aGrp["FLV5"] ){
		retPar.sRetval1+= ","+alias+"FLV5";
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FLV5";
	}

	// 3. Категория:
	if( this.aGrp["FKOD_KAT"] ){
		retPar.sRetval1+= ","+alias+"FKOD_KAT";
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FKOD_KAT";
	}

	//4.1 ШПЗ
	if( this.aGrp["FSHPZ"] ){
		retPar.sRetval1+= ","+alias+"FSHPZ";
	} else {
		retPar.sRetval1+= ","+sqlTo(replicate(" ", 30))+" AS FSHPZ";
	}

	//4.2 ШПЗ из штатного расписания
	if( this.aGrp["FSHPZ_S"] ){
		retPar.sRetval1+= ","+alias+"FSHPZ_S";
	} else {
		retPar.sRetval1+= ","+sqlTo(replicate(" ", 30))+" AS FSHPZ_S";
	}

	// 5. Таб.номер:
	if( this.aGrp["FTABN"] ){
		retPar.sRetval1+= ","+alias+"FTABN" ;
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FTABN" ;
	}

	// 6. Выражение:
	if( this.aGrp["FEXPR"] ){
		retPar.sRetval1+= ","+alias+"FEXPR" ;
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FEXPR" ;
	}
	
	// 7. Вид оплаты:
	if( this.aGrp["FVOPL"] ){
		retPar.sRetval1+= ","+alias+"FVOPL" ;
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FVOPL" ;
	}
	
	// 8.Код ШР 
	if( this.aGrp["FSHT_ID"] ){
		retPar.sRetval1+= ","+alias+"FSHT_ID" ;
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FSHT_ID" ;
	}
	
	
	// 9.Код основания для учета стажа
	if( this.aGrp["FIS_SCIENCE"] ){
		retPar.sRetval1+= ","+alias+"FIS_SCIENCE" ;
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FIS_SCIENCE" ;
	}
	
	// 10.код кактегории ЗЛ
	if( this.aGrp["FCOD_KZO"] ){
		retPar.sRetval1+= ","+alias+"FCOD_KZO" ;
	} else {
		retPar.sRetval1+= ","+Convert("0","INT")+" AS FCOD_KZO" ;
	}

	retPar.sRetval_listFlds = ", FLSFID, FLV1, FLV2, FLV3, FLV4, FLV5, FKOD_KAT, FSHPZ, FSHPZ_S, FTABN, FEXPR, FVOPL, FSHT_ID, FIS_SCIENCE, FCOD_KZO";
	
	return retPar;
}

// Ф-ция, формирующая IIF-выражение для определения номера диапазона, в который
// попадает значение суммы. Вызывается рекурсивно.
// Первый вызов должен быть с nIndex==-1
// При диапазонах (0, 150, 300) формирует что-то типа
// IIF(A.FSUM<0, IIF(A.FSUM>=-0, -0, IIF(A.FSUM>=-150, -1, IIF(A.FSUM>=-300, -2, -3)))
// ,  IIF(A.FSUM>=0,IIF(A.FSUM>=150,IIF(A.FSUM>=300, 3, 2), 1), 0) )
// mode может быть "-" - это сигнал формировать ветвь условий для отрицательных значений
// field - имя поля
function DpZrpBaseSum_getBandCond(mode, alias, field,  nIndex) {
	var retval = "";
	if (nIndex>=this.aBands.length)
	{
		retval = (mode=="-" ? "-":"")+sqlTo(nIndex);
	}
	else if (nIndex==-1)
	{
		// Это первый вызов - стартуем рекурсию
		retval = 
		sqlIIF( alias+field+"="+sqlTo(0), sqlTo(1)
			,sqlIIF( alias+field+"<"+sqlTo(0)
				, this.getBandCond("-", alias, field, nIndex+1)	//ветвь для отрицательных сумм
				, this.getBandCond("", alias, field, nIndex+1)		//ветвь для положительных сумм
			)
		);
	}

	else if(mode=="-")
	{
		retval = sqlIIF( alias+field+">=-"+sqlTo(this.aBands[nIndex])
			, "-"+sqlTo(nIndex)
			, this.getBandCond(mode, alias, field, nIndex+1) );
	}
	else
	{
		retval = sqlIIF( alias+field+">"+sqlTo(this.aBands[nIndex])
			, this.getBandCond(mode, alias, field, nIndex+1)
			, sqlTo(nIndex) );
	}

	return retval;
}


function DpZrpBaseSum_getWhere(mode, alias) {
	if(!alias){
		alias = "";
	} else {
		alias = alias+".";
	}
	var sRetval = "";

	switch(mode) {
	case "EXPR":		//формируем только для выражения
		// 6.1 Выражение: "для..."
		sRetval += this.whereFieldParce("",this.aFilters["FEXP_INCL"],this.oEval,
			" AND "+alias+"FEXPR IN (^)",true)
		// 6.2 Выражение: "кpоме..."
		sRetval+= this.whereFieldParce("",this.aFilters["FEXP_EXCL"],this.oEval,
			" AND "+alias+"FEXPR NOT IN (^)",false)
		break;
	case "NOEXPR":		//формируем для всех полей, кроме выражения
	case "ESV_ALL":
	
		// Обойдем все фильтры
		for(ind in this.aFilters) {
			if(this.aFilters[ind]==""){
				// ничего не задано - переходим к следующему
				continue;
			}
			switch(ind){
			case "FEXP_INCL":
			case "FEXP_EXCL":
				// Пропускаем
				break;
			case "FLS_EXPR":		//выражение, накладываемое на связку SVZP-LIC_FLD
				if(mode == "NOEXPR")
				{
					sRetval += this.aFilters["FLS_EXPR"];
				}
				break;
			case "FLS_EXPR_ESV_ALL":		//выражение, накладываемое на связку SVZP-LIC_FLD
				if(mode == "ESV_ALL")
				{
					sRetval += this.aFilters["FLS_EXPR_ESV_ALL"];
				}
				break;
			
			case "FLS_INCL":		//1.1 поле ЛС "для..."
			case "FKAT_INCL":		//3.1 Категория:"для..."
			case "FSH_INCL":		//4.1 ШПЗ: "для..."
			case "FSH_S_INCL":		//5.1 ШПЗ из штатного расписания: "для..."
			case "FTAB_INCL":		//6.1 Таб.номер: "для..."
				// "...для"
				sRetval += this.whereFieldParce("", this.aFilters[ind], this.oEval,
					" AND "+alias+this.getFieldByLabel(ind)+" In (^)",true)
					//alert(sRetval)
				break;
			case "FLS_EXCL":		//1.2 поле ЛС "кpоме..."
			case "FKAT_EXCL":		//3.2 Категория:"кpоме..."
			case "FSH_EXCL":		//4.2 ШПЗ: "кpоме..."
			case "FSH_S_EXCL":		//5.2 ШПЗ из штатного расписания: "кpоме..."
			case "FTAB_EXCL":		//6.2 Таб.номер: "кpоме..."
				// "...кpоме"
				sRetval += this.whereFieldParce("", this.aFilters[ind], this.oEval,
					" AND "+alias+this.getFieldByLabel(ind)+" NOT IN (^)",false)
				break;
			case "FLV_INCL":		//2.1 Подpазделение "для..."
				sBuff = this.whereFieldParce("", this.aFilters[ind], this.oEval)
				if(sBuff.length > 0){
					sRetval+= " AND " + this.getFlvCond(sBuff, alias);
				}
				break;
			case "FLV_EXCL":		//2.2 Подpазделение "кpоме..."
				sBuff = this.whereFieldParce("", this.aFilters[ind], this.oEval)
				if(sBuff.length > 0){
					sRetval+= " AND NOT "+this.getFlvCond(sBuff, alias);
				}
				break;
			case "FPRIZN":			//7. Дата
				// 8. Дата (чуть не забыли!!!)
				switch( this.aFilters[ind] ) {
				case "т":
					sRetval+= " AND "+alias+"Fvp=0 ";
					break;
				case "с":
					sRetval+= " AND "+alias+"Fvp=1 AND "+alias+"Fmn < " + sqlTo(this.date);
					break;
				case "б":
					sRetval+= " AND "+alias+"Fvp=2 AND "+alias+"Fmn > " + sqlTo(this.date);
					break;
				case "п":
					sRetval+= " AND "+alias+"Fvp=2 AND "+alias+"Fmn < " + sqlTo(this.date);
					break;
				default:
					break;
				}
			}
		}
		
		break;
	}	
	return sRetval;
}

function DpZrpBaseSum_getGroupBy(alias) {
	if(!alias){
		alias = "";
	} else {
		alias = alias+".";
	}

	sRetval = "";
	for(ind in this.aGrp) {
		if(this.aGrp[ind]) {
			sRetval += ","+alias+ind;
		}
	}

/*
	// 7. "Диапазон", в который попадает значение
	sRetval+= ","+this.getBandCond("", alias, -1);
*/
	return sRetval;
}

//============================================
// ksm 04.08.2004
// Подготавливает значение поля для функции getWhere.
// Если значение поля FKIND > 1
// и sField = Fls_incl
// и isInclude = true
// Тогда заменяем значение поля на переменную LIST_KIND+значение("FKIND")
//============================================
function DpZrpBaseSum_whereFieldParce(mode,sFilter,oEval,sPrefix,isInclude){
	var retval = "";
	var vFieldValue = sFilter;

	if (!isEmpty(vFieldValue)){
		//Уберем пробелы.
		retval = vFieldValue.replace(/ /g,"");
		//Есть перемененная необходимо Evalить
		if(/\^/.test(retval)){
			retval = retval.replace(/\^/gi,"");
			try{
				retval = oEval.eval(retval);
			}
			catch (error){
				retval = "";
			}
			retval = retval.replace(/\s/gi, "");
		}//END OF  if(/\^/.test(retval))

		if (!isEmpty(sPrefix)){//Если нет префикса, то вернем что имеем
			var isEmptyList = isEmpty(retval)
			if(!isEmptyList){//Список не пустой вернем выражение
				retval = sPrefix.replace(/\^/gi, retval);
			}else{
				if(isInclude){//Список пустой, но необходимо включать - вернем ложное выражение
					retval = " AND "+SqlCmpFalse;
				}
			}
		}
	}
	return retval;
}

// возвращает имя поля по метке
function DpZrpBaseSum_getFieldByLabel(label) {
	retval = "";
	switch(label){
	case "FLS_INCL":		//1.1 поле ЛС "для..."
	case "FLS_EXCL":		//1.2 поле ЛС "кpоме..."
		retval = "FLSFID";
		break;
	case "FKAT_INCL":		//3.1 Категория:"для..."
	case "FKAT_EXCL":		//3.2 Категория:"кpоме..."
		retval = "FKOD_KAT";
		break;
	case "FSH_INCL":		//4.1 ШПЗ: "для..."
	case "FSH_EXCL":		//4.2 ШПЗ: "кpоме..."
		retval = "FSHPZ";
		break;
	case "FSH_S_INCL":		//5.1 ШПЗ из штатного расписания: "для..."
	case "FSH_S_EXCL":		//5.2 ШПЗ из штатного расписания: "кpоме..."
		retval = "FSHPZ_S";
		break;
	case "FTAB_INCL":		//6.1 Таб.номер: "для..."
	case "FTAB_EXCL":		//6.2 Таб.номер: "кpоме..."
		retval = "FTABN_ID";
		break;
	case "FEXP_INCL":
	case "FEXP_EXCL":
		retval = "FEXPR";
		break;
	case "FVOPL_INCL":
	case "FVOPL_EXCL":
		retval = "FVOPL";
		break;
	}
	return retval;
}

//Проставляем значение выражения
// tblName - имя таблицы, по которой пробегаем
// Значение вычисленного выражения пишем в поле FEXPR
function DpZrpBaseSum_setExprValues(tblName) {

	//alert("DpZrpBaseSum_setExprValues НЕ РЕАЛИЗОВАНА!");
	var tbl = OpenTable(tblName);
	if(tbl.IsEmpty()){
		tbl.close(5652);
		tbl = null;
		return this;
	}
	this.oEval.addExpr("EXPR_EXPRESSION", this.expr);
	// пpивяжем дpугую таблицу стpок
	this.oEval.setTable(tbl);
	
	// обойдем все строки таблицы и вычислим выражение
	for(tbl.MoveFirst(); !tbl.IsEOF(); tbl.MoveNext()) {
		val = this.oEval.getVar("EXPR_EXPRESSION");

		// Вообще-то val должно быть числового типа
		tbl.Edit();
		tbl("FEXPR") = val;
		tbl.Update();
	}
	
	this.oEval.setTable(null);
	tbl.close(5652);
	tbl = null;

	return this;
}

//============================================
// ksm 04.08.2004 Подготовим переменные для Evala
// vvs 30.08.2004 Добавил переменные-близнецы с содержательными именами.
//	1. LIST_INVALID - Список табельных номеров инвалидов
//	2. LIST_KIND2, LIST_LS_PF	- Список лицевых счетов которые входят
//					в сумму нач.на зарплату для Пенс.фонда
//	3. LIST_KIND3, LIST_LS_FSS	- Список лицевых счетов которые входят
//					в сумму нач.на зарплату для фонда соцстраха
//	4. LIST_KIND4, LIST_LS_FZ	- Список лицевых счетов которые входят
//					в сумму нач.на ЗП для фонда зан.
//	5. LIST_KIND5, LIST_LS_FSNV	- Список лицевых счетов которые входят
//					в сумму нач.на ЗП для.фонда несч.сл.
//	6. LIST_LS_FZP	- Список лицевых счетов которые входят в ФЗП
//  7. LIST_SOVMEST - Список людей совместителей (вид оплаты = 8300)
//	8. LIST_RESERV_OTP	- список полей ЛС, которые входят в средние для отпусков
//	9. LIST_LS_FROM_PRIBIL	- список полей ЛС, которые относятся на затраты за счет прибыли (Дт 949)
//	10. LIST_PENS	- Список табельных номеров пенсионеров
//	11. LIST_LS_RESERV_OTP_AND_PRIB	- Список полей ЛС, которые относятся на затраты за счет прибыли (Дт 949) 
//   и входят в резерв отпусков
//============================================
function DpZrpBaseSum_prepareVars4Eval(mode,oEval,m_bdat,m_edat)
{
	var sList; 	// Вспомогательная
	//1.LIST_INVALID
	strSQL = " AND ("
				+"(WP.FBDAT <= " + sqlTo(m_bdat) + " AND WP.FEDAT >= " + sqlTo(m_edat) +") "
				+ "OR (WP.FBDAT >= " + sqlTo(m_bdat) + " AND WP.FBDAT <= " + sqlTo(m_edat) +") "
				+ "OR (WP.FEDAT >= " + sqlTo(m_bdat) + " AND WP.FEDAT <= " + sqlTo(m_edat) +"))"
			+ " AND FINVAL = "+SqlTrue;
	sList = this.getListFromTblWithCond("","FWID_CL","^WORK_PERS",strSQL, "WP");
	oEval.addVar("LIST_INVALID", sList);

	//	2. LIST_KIND2, LIST_LS_PF
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FIS20='+'");
	oEval.addVar("LIST_KIND2", sList);
	oEval.addVar("LIST_LS_PF", sList);

	//	3. LIST_KIND3, LIST_LS_FSS
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FSOC='+'");
	oEval.addVar("LIST_KIND3", sList);
	oEval.addVar("LIST_LS_FSS", sList);

	//	4. LIST_KIND4, LIST_LS_FZ
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FIS23='+'");
	oEval.addVar("LIST_KIND4", sList);
	oEval.addVar("LIST_LS_FZ", sList);

	//	5. LIST_KIND5, LIST_LS_FSNV
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FNSL='+'");
	oEval.addVar("LIST_KIND5", sList);
	oEval.addVar("LIST_LS_FSNV", sList);

	//	6. LIST_LS_FZP	- Список лицевых счетов которые входят в ФЗП
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND Ffzp='+'");
	oEval.addVar("LIST_LS_FZP", sList);

	var listSovm;
	listSovm = getPlugPar("", "ZRP_LIST_SOVM", null, "8000, 8300", "Список видов оплаты за труд. договором ");
	var strWhere;
	if (!isEmpty(listSovm))
	{
		strWhere = " AND FVOPL IN(" + listSovm + ")"
	}
	else
	{
		strWhere = "";
	}
	//  7.LIST_SOVMEST
	strSQL = " AND ("
				+"(WP.FBDAT <= " + sqlTo(m_bdat) + " AND WP.FEDAT >= " + sqlTo(m_edat) +") "
				+ "OR (WP.FBDAT >= " + sqlTo(m_bdat) + " AND WP.FBDAT <= " + sqlTo(m_edat) +") "
				+ "OR (WP.FEDAT >= " + sqlTo(m_bdat) + " AND WP.FEDAT <= " + sqlTo(m_edat) +"))"
				+ strWhere
	sList = this.getListFromTblWithCond("", "FWID_CL", "^WORK_PERS", strSQL, "WP");
	oEval.addVar("LIST_SOVMEST", sList);

	//	8. LIST_RESERV_OTP	- список полей ЛС, которые входят в средние для отпусков
	// надо для резерва отпусков
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FIS27 IN('+')");
	oEval.addVar("LIST_RESERV_OTP", sList);
	
	// 9. LIST_LS_FROM_PRIBIL	- список полей ЛС, которые относятся на затраты за счет прибыли (Дт 949)
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FIS26 IN('p')");
	oEval.addVar("LIST_LS_FROM_PRIBIL", sList);

	//10.LIST_PENS
	strSQL = " AND ("
				+"(WP.FBDAT <= " + sqlTo(m_bdat) + " AND WP.FEDAT >= " + sqlTo(m_edat) +") "
				+ "OR (WP.FBDAT >= " + sqlTo(m_bdat) + " AND WP.FBDAT <= " + sqlTo(m_edat) +") "
				+ "OR (WP.FEDAT >= " + sqlTo(m_bdat) + " AND WP.FEDAT <= " + sqlTo(m_edat) +"))"
			+ " AND FPENS = "+SqlTrue;
	sList = this.getListFromTblWithCond("","FWID_CL","^WORK_PERS",strSQL, "WP");
	oEval.addVar("LIST_PENS", sList);

	//	11. LIST_LS_RESERV_OTP_AND_PRIB	- Список полей ЛС, которые относятся на затраты за счет прибыли (Дт 949) 
	//   и входят в резерв отпусков
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FIS26 IN('p') AND FIS27 IN('+')");
	oEval.addVar("LIST_LS_RESERV_OTP_AND_PRIB", sList);

	//	12. LIST_LS_ESV
	sList = this.getListFromTblWithCond("","FID","^LIC_FLD"," AND FIS30 = '+'");
	oEval.addVar("LIST_LS_ESV", sList);
	
	return this;
}

//============================================
// ksm 06.08.2004 Функция возвращает список значения из поля sField
// таблицы sTblName при условии sCond
function DpZrpBaseSum_getListFromTblWithCond(mode,sField,sTblName,sCond,alias, join, aliiasJoinFld){
	var retval ="";
	if (!isEmpty(sField) && !isEmpty(sTblName) && !isEmpty(sCond)){
		strSQL = " SELECT DISTINCT "
			+sField
			+(isEmpty(aliiasJoinFld) ? "" : " AS "+aliiasJoinFld)
		+" FROM "
			+sTblName
			+(isEmpty(alias) ? "" : " AS "+alias)
			+(isEmpty(join) ? "" : join)
		+" WHERE 1=1 "
			+sCond

		var tblData = OpenTable(strSQL)
		if (typeof(aliiasJoinFld) != "undefined")
		{
			sField = aliiasJoinFld;
		}
		
		for (; !tblData.IsEOF(); tblData.MoveNext())
		{
			retval+=","+toScrObj(tblData.getValue(sField));
		}
		retval = retval.substr(1);
		
		if (tblData)
		{
			tblData.Close(5652)
			tblData = null;
		}
	}
	return retval;
}
//============================================

function DpZrpBaseSum_getFondAttr(sFond)
{
	var par = new Object();
	par.aProc = new Array();
	switch(sFond)
	{
		case "0":	// пенсионный
			par.nazvur = "пенсійний фонд"
			par.nazvru = "пенсионный фонд"
			par.grNazvur1 = "пенсійний"
			par.grNazvru1 = "пенсионный"
			par.grNazvur2 = "фонд"
			par.grNazvru2 = "фонд"
			
			par.label_1 = "PFL"	// база по человеку
			par.label_2 = "PF"	// сумма удержания в фонд
			par.label_3 = "PFP"	// сумма удержания в фонд

			par.aProc[0] = get_cnst("proc_pns1", DD1);
			par.aProc[1] = get_cnst("proc_pns", DD1);
			par.sum_ogr = this.sumOgrExpr;

			break;
		case "1":	// фонд занятости
			par.nazvur = "фонд зайнятості"
			par.nazvru = "фонд занятости"

			par.grNazvur1 = "фонд"
			par.grNazvru1 = "фонд"
			par.grNazvur2 = "зайнятості"
			par.grNazvru2 = "занятости"

			par.label_1 = "FZL"	// база по человеку       
			par.label_2 = "FZ"  // сумма удержания в фонд 
			par.label_3 = "FZP"  // сумма удержания в фонд 

			par.aProc[0] = get_cnst("proc_zan", DD1);
			par.aProc[1] = get_cnst("proc_zan", DD1);
			par.sum_ogr = this.sumOgrExpr;

			break;
		case "2":	// соцстрах
			par.nazvur = "соцстрах"
			par.nazvru = "соцстрах"
                
			par.grNazvur1 = "фонд"
			par.grNazvru1 = "фонд"
			par.grNazvur2 = "соцстрахування"
			par.grNazvru2 = "соцстрахования"
                
			par.label_1 = "FSSL"	// база по человеку       
			par.label_2 = "FSS"  // сумма удержания в фонд 
			par.label_3 = "FSSP"  // сумма удержания в фонд 
			    
			par.aProc[0] = get_cnst("proc_soc1", DD1);
			par.aProc[1] = get_cnst("proc_soc", DD1);
			par.sum_ogr = this.sumOgrExpr;
			
			break;

		case "3":	// ФСНВ
			par.nazvur = "ФСНВ"
			par.nazvru = "ФСНВ"
                
			par.grNazvur1 = "ФСНВ"
			par.grNazvru1 = "ФСНВ"
			par.grNazvur2 = "ФСНВ"
			par.grNazvru2 = "ФСНВ"
                
			par.label_1 = ""	// база по человеку       
			par.label_2 = ""  // сумма удержания в фонд 
			par.label_3 = "FSNVP"  // сумма удержания в фонд 
			    
			par.aProc[0] = 0;
			par.aProc[1] = 0;
			par.sum_ogr = this.sumOgrExpr;
	
			break;
		case "4":	// ЕСВ
			par.nazvur = "єдиний соц.внесок"
			par.nazvru = "единый соц.взнос"
			par.grNazvur1 = "єдиний"
			par.grNazvru1 = "единый"
			par.grNazvur2 = "соц.внесок"
			par.grNazvru2 = "соц.взнос"
			
			par.label_1 = "ESVP"	// база по человеку
			par.label_2 = "ESV"	// сумма удержания в фонд
			par.label_3 = "ESVP"	// база в фонд
			par.label_4 = "ESV_CPH"	// база в фонд по ЦПХ
			par.label_5 = "ESV_BLN"	// база в фонд больничные

			par.aProc[0] = getProc("N",DD1,1);//get_cnst("proc_pns1", DD1);
			par.aProc[1] = getProc("N",DD1,1);//get_cnst("proc_pns", DD1);
			par.sum_ogr = this.sumOgrExpr;

			break;
	}
	return par;
}

function DpZrpBaseSum_setConstMode(mode)
{
	this.constMode = mode;
}

function DpZrpBaseSum_getConstMode()
{
	return this.constMode;
}

function DpZrpBaseSum_setSumOgrMode(mode)
{
	this.sumOgrMode = mode;
}

function DpZrpBaseSum_getSumOgrMode()
{
	return this.sumOgrMode;
}

/*
DELETE S FROM ZRP_SVZP201004 S
INNER JOIN COM_CLRELCLCL REL ON S.FTABN_ID = REL.FWID_CL
WHERE REL.FCOD NOT IN(41,22,1552) AND REL.FCL = 2
*/

// Взять процент начисления/удержания для переданной категории застрахованного лица
// mode - "N" - начисление, "U" - удержание
// dat - дата, на которую надо определить процент
// codZO - код категории застрахованного лица
function getProc(mode, dat, codZO)
{
	var retVal = 0;
	var fldName = (mode.toUpperCase() == "N" ? "FPROC_N" : "FPROC_U")
	strSQL = "SELECT "+fldName+" FROM ^NALOG_ESV WHERE FBDAT <= "+sqlTo(dat)+" AND FEDAT >= "+sqlTo(dat)+" AND FCOD_KZO = "+sqlTo(codZO)
	var sn = getSnap(strSQL);
	if (sn != null && !isEmpty(sn[0]))
	{
		retVal = sn[0];
	}
	return Number(retVal);
}

getproc = getProc;