/*
   Процедура расчета оборотов и остатков по счету
   по одной или по всем аналитикам
   (C) ДЕБЕТ Плюс, 1998

   Параметры:
   - mode - режимы работы:
      "c" - по счету (в конечной таблице только одна запись)
      "3" - по всем аналитикам (иначе - по первой аналитике)
      "^" - использовать "фальшивые" план счетов, проводки, остатки.
            Имена "фальшивых" таблиц состоят из оригинального имени таблицы
            с префиксом, равным параметру outTbl.
      "v" - в справочной валюте (в двух валютах)
      "n" - не формировать итоговые записи
      "s" - подбивать справочные суммы (переводятся в нац. валюту)
      "+" - рассматривать счет как активный
      "-" - рассматривать счет как пассивный
      "%" - рассматривать счет как активно-пассивный, свернутый на
            уровне статьи (="~st")
      "*" - рассматривать счет как активно-пассивный, свернутый на
            уровне счета (="~sc")
      "~<уровень>" - рассматривать счет как активно-пассивный,
            свернутый на указанном уровне.
         <уровень> может принимать одно из следующих значений:
         "sc" - на счете
         "su" - на субсчете
         "st" - на статье
         "a1" - на первой аналитике
         "a2" - на второй аналитике
         "a3" - на третьей аналитике (на нижнем уровне)
      "$" - не переводить в одну валюту
      "p" - сохранить в полях fpSumX суммы в валютах из проводок
      "R" - не показывать прогресс-бар
      "L" - даты последней проводки (по дебету и по кредиту)

   - outTbl - имя выходной таблицы.
      Временные таблицы используют это имя в качестве префикса к названию.
   - dat1, dat2 - период
   - objSc - объект-счет, по которому строятся обороты и остатки.
   - valCod - фильтр по коду валюты (fwrk<>12 and fwrk<>15)
      null - без фильтра

   В результате создается таблица TMP_OBLNS с полями:
   fsc      - счет
   fsu      - субсчет
   fst      - стать
   fcl_cod1 - аналитика 1
   fcl_cod2 - аналитика 2
   fcl_cod3 - аналитика 3
   fsum1    - сальдо по дебету на начало периода
   fsum2    - сальдо по кредиту на начало периода
   fsum3    - сальдо по дебету на конец периода
   fsum4    - сальдо по кредиту на конец периода
   fsum5    - оборот по дебету
   fsum6    - оборот по кредиту
   fsumobr  - сумма оборотов по дебету и по кредиту взятых по абсолютной величине.
   fistotal - является ли запись итоговой

   30.12.98 - 05.01.99  DAE
   20.01.99 - 26.01.99  DAE
   04.02.99 - 05.02.99  DAE
   24.02.2004 - Шатохин В.В. Для активно-пассивного счета с развернутым сальдо
   		не сворачивать сальдо на 1-й аналитике при имеющейся 2-й
*/

include("Objects\\DpSc.js");
include("Objects\\DpVal.js");
include("sys\\DpSQL.js");
include("servis\\format.js");

function TrnNBln(mode, outTbl, dat1, dat2, objSc, valCod)
{
	var isProgress = !strIn("R", mode);

	if(isProgress)
	{
		var cProgressText = "Обрабатывается счет " + objSc.say();
		setProgressRange(0, 8);
		setProgressPos(1, cProgressText);

	}

	if(!objSc.valid())
	{
		debug("Передан некорректный счет")
	}

	mode = mode.toUpperCase();

	// Установим названия таблиц плана счетов, проводок и т.д.
	if(mode.indexOf("^")!=-1)
	{
		var cl_schet = outTbl + "_cl_schet",
			schet_sl = outTbl + "_schet_sl",
			schet = outTbl + "_schet",
			schet_zr = outTbl + "_schet_zr";
	}
	else
	{
		var cl_schet = "^cl_schet",
			schet_sl = "^schet_sl",
			schet = "^schet",
			schet_zr = "^schet_zr";
	}

	// Определим вид счета (либо принудительно (по параметру),
	// либо из плана счетов)
	//vlad 06.02.2012
	//Если вызывающая сторона нам явно не передала в режиме вид счета,
	//то scVid мы оставим "пустым" - это будет сигнал, что настройку брать из плана счетов.
	//под это заточим turnExpr()
	var scVid;
	if(strIn("+", mode))
		scVid = "+";
	else if(strIn("-", mode))
		scVid = "-";
	else if(strIn("%", mode))
		scVid = "%";
	else if(strIn("*", mode))
		scVid = "*";
	else
		//vlad 06.02.2012
		//scVid = objSc.getVid();
		scVid = "";	//Это значит, что вид будем брать из плана счетов

	var isItogs = !strIn("N", mode);
	var is1Val = !strIn("$", mode);
	var isPSum = strIn("P", mode);
	var isLastDate = true //strIn("L", mode);

	var isSprav = strIn("S", mode);
	var sprFldDef = "";
	var sprFldList = "";

   var is2val = strIn("V", mode);
	if(is2val)
	{
		var usedval = getPar("CODSPRAV")
		if(usedval==null)
		{
			alert(ru("Справочная валюта не указана", "Довідкова валюта не визначена"));
			return false;
		}
		usedval = new DpVal(usedval).getCod();
	}

	var turnLevel;
	if(strIn("~", mode))
	{
		turnLevel = mode.substr(mode.indexOf("~")+1, 2);
		mode.replace("~" + turnLevel, "");
	}
	//vlad 06.02.2012
	//если явно не передан режим сворачивания,
	//прийдется для правильного результата начинать с самого низа
	/*
	else if(scVid=="%")
		//vlad
		//Даже если нам нужны итоги только по 1-й аналитике,
		//Для счета с развернутым сальдо надо произвести вычисления начиная
		//с самого нижнего уровня. В противном случае полученное развернутое 
		//сальдо не совпадет со значением, которое даст оборотный баланс и 
		//обороты и остатки по аналитике
		//turnLevel = "A1";
		turnLevel = "A3";
	else
		turnLevel = "SC";
	*/
	else
	{
		turnLevel = "A3";
	}
	
	//vlad 06.02.2012
	//recLevel нигде не используется
	/*
	var recLevel;
	if(strIn("c", mode))
		recLevel = "SC";
	else if(strIn("3", mode))
		recLevel = "A3";
	else
		recLevel = "A1";
	*/
	
	// Формирование условия для отбора по счету
	var sc = objSc.getSc(), su = objSc.getSu(), st = objSc.getSt(),
		cl1 = objSc.getCl1(), cl2=objSc.getCl2(), cl3=objSc.getCl3();
	var scCond = " fsc=" + sc
		+ ( su? " and fsu=" + su: "" )
		+ ( st? " and fst=" + st: "" )
		+ ( cl1? " and fcl_cod1=" + cl1: "" )
		+ ( cl2? " and fcl_cod2=" + cl2: "" )
		+ ( cl3? " and fcl_cod3=" + cl3: "" );
	var dbCond = " fdb_sc=" + sc
		+ ( su? " and fdb_su=" + su: "" )
		+ ( st? " and fdb_st=" + st: "" )
		+ ( cl1? " and fdb_cod1=" + cl1: "" )
		+ ( cl2? " and fdb_cod2=" + cl2: "" )
		+ ( cl3? " and fdb_cod3=" + cl3: "" );
	var crCond = " fcr_sc=" + sc
		+ ( su? " and fcr_su=" + su: "" )
		+ ( st? " and fcr_st=" + st: "" )
		+ ( cl1? " and fcr_cod1=" + cl1: "" )
		+ ( cl2? " and fcr_cod2=" + cl2: "" )
		+ ( cl3? " and fcr_cod3=" + cl3: "" );

	//фильтр на организацию
	var orgCond = "";
	if(getPar("BLNBYORG"))
		orgCond = " and Forg0="+CURRENT_ORG;

	var sqlDat1 = sqlTo(dat1);
	var sqlDat2 = sqlTo(dat2);

	// Создание таблицы TMP_OBLNS1
	// В ней будут скидываться записи по проводкам и начальным остаткам
	DropTable(outTbl + "_1");
	ExecuteSQL(
	   "CREATE TABLE " + outTbl + "_1 ("
	      + " fsc "+SqlType(sqlTSc)+","
	      + " fsu "+SqlType(sqlTSu)+","
	      + " fst "+SqlType(sqlTSt)+","
	      + " fcl_cod1 "+SqlType("INTEGER4")+","
	      + " fcl_cod2 "+SqlType("INTEGER4")+","
	      + " fcl_cod3 "+SqlType("INTEGER4")+","
	      + " fsum1 "+SqlType("MONEY")+","
	      + " fsum2 "+SqlType("MONEY")+","
	      + " fsum3 "+SqlType("MONEY")+","
	      + " fsum4 "+SqlType("MONEY")+","
	      + " fsum5 "+SqlType("MONEY")+","
	      + " fsum6 "+SqlType("MONEY")+","
	      + " fssum1 "+SqlType("MONEY")+","
	      + " fssum2 "+SqlType("MONEY")+","
	      + " fssum3 "+SqlType("MONEY")+","
	      + " fssum4 "+SqlType("MONEY")+","
	      + " fssum5 "+SqlType("MONEY")+","
	      + " fssum6 "+SqlType("MONEY")+","
	      + " fsval "+SqlType("SHORT")+","
	      + sprFldDef
	      + " fval "+SqlType("SHORT")+","
	      + " fvkurs "+SqlType("SINGLE")+","
	      + " flastd "+SqlType("DATE")+","
	      + " flastk "+SqlType("DATE")+","
	      + " fdate "+SqlType("DATE")+")"
	   );

   // Выборка "свежих" начальных остатков по счету
   var tbl;
   // Ищем дату для начальных остатков
   tbl = OpenTable("s",
      "SELECT max(" + schet_sl + ".fdate) as fdate"
      + " FROM " + schet_sl
      + " WHERE fsc=" + sc
         + " and fdate<=" + sqlTo(dat1)
         + orgCond
      );

	// Если проверять остатки
	tInnerSl = OpenTable("s",
		"SELECT " + schet_sl + ".fdate as fdate"
		+ " FROM " + schet_sl
		+ " WHERE fsc=" + sc
		   + " and fdate>" + sqlTo(dat1)
		   + " and fdate<=" + sqlTo(dat2)
		   + orgCond
		+ " GROUP BY fdate"
		);
   if(!tInnerSl.IsEmpty())
   {
		alert("Найдены суммовые остатки по " + sc + " счету на " + format("dd.mm.yyyy", tInnerSl.getValue("fdate")) + ",\n"
				+ "которые лежат внутри периода расчета.\n"
				+ "Результат расчета может быть некорректным"
				)
   }

	tInnerSl.close();
	if(isProgress)
		setProgressPos(2, cProgressText);

   var obCond; // условие попадения проводки в расчет

   if( tbl.getValue("fdate")!=null )
   {
		// Если найдены начальные остатки - загружаем их
		var sldDat;
		sldDat = sqlTo(tbl.getValue("fdate"));
		ExecuteSQL(
			"INSERT INTO " + outTbl + "_1 (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3,FSUM1,FSUM2,FSUM3,FSUM4, "+
			" FSUM5,FSUM6,"+(isSprav?"FSSUM1,FSSUM2,FSSUM3,FSSUM4,FSSUM5,FSSUM6, FSVAL, ":"")+"fdate, fval, fvkurs)"
			+ " SELECT"
				+ " fsc, fsu, fst,"
				+ " fcl_cod1, fcl_cod2, fcl_cod3,"
				+ sqlRound("fsumd", prmDecSum)+" as fsum1,"    // остатки на начало периода по дебету
				+ sqlRound("fsumk", prmDecSum)+" as fsum2,"    // остатки на начало периода по кредиту
				+ sqlRound("fsumd", prmDecSum)+" as fsum3,"    // остатки на конец периода по дебету
				+ sqlRound("fsumk", prmDecSum)+" as fsum4,"    // остатки на конец периода по кредиту
				+ " 0 as fsum5,"        // обороты по дебету
				+ " 0 as fsum6,"        // обороты по кредиту
				+ (isSprav   // Если считаем и по справ. валюте
					?
						(sqlRound("fsumdspr", prmDecSum)+" as fssum1,"
						+ sqlRound("fsumkspr", prmDecSum)+" as fssum2,"
						+ sqlRound("fsumdspr", prmDecSum)+" as fssum3,"
						+ sqlRound("fsumkspr", prmDecSum)+" as fssum4,"
						+ " 0 as fssum5,"
						+ " 0 as fssum6,"
						+ " fvalspr as fsval,")
					:
						""
					)
				+ " fdate, fval, fvkurs"        // дата и валюта
			+ " FROM " + schet_sl
			+ " WHERE " + scCond
				+ " and fdate=" + sldDat
				+ " and (fsumd<>0 or fsumk<>0)"
				+ " and fistotal = "+sqlFalse
				+ orgCond
			);

		obCond = " fdate between " + sldDat
			+ " and " + sqlDat2 + " ";
   }
   else
   {
      obCond = " fdate <= " + sqlDat2 + " ";
   }


   if(typeof(valCod)=="number")
      obCond += "and fval=" + valCod + " and fwrk<>12 and fwrk<>15 ";

   tbl.close();
   tbl = null;

	if(isProgress)
		setProgressPos(3, cProgressText);

	if (false &&SQL)
	{
   // Выборка дебетовых проводок
   ExecuteSQL(
      "INSERT INTO " + outTbl + "_1 (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3, FSUM1,FSUM5,FSUM3,FSUM2, "+
			" FSUM4,FSUM6,"+(isSprav?"FSSUM1,FSSUM5,FSSUM3,FSSUM2,FSSUM4,FSSUM6, FSVAL, ":"")+"fdate,"+ (isLastDate? " flastd, flastk,": "") + " fval, fvkurs)"
      + " SELECT"
         + " fdb_sc as fsc,"
         + " fdb_su as fsu,"
         + " fdb_st as fst,"
         + " fdb_cod1 as fcl_cod1,"
         + " fdb_cod2 as fcl_cod2,"
         + " fdb_cod3 as fcl_cod3,"
         + sqlRound("fsum", prmDecSum)+" as fsum1,"
         + " 0 as fsum5,"
         + sqlRound("fsum", prmDecSum)+" as fsum3,"
         + " 0 as fsum2, 0 as fsum4, 0 as fsum6,"  // кредитовые суммы
         + (isSprav
				?
					(sqlRound("fsumspr", prmDecSum)+" as fssum1,"
					+ " 0 as fssum5,"
					+ sqlRound("fsumspr", prmDecSum)+" as fssum3,"
					+ " 0 as fssum2, 0 as fssum4, 0 as fssum6,"
					+ " fvalspr as fsval,")
				:
					""
				)
         + " fdate,"
         + (isLastDate? " fdate AS flastd, " + sqlTo(d00min) + " AS flastk,": "")
         + " fval, fvkurs"
      + " FROM " + schet
      + " WHERE "
         + dbCond
         + " and " + obCond
         + " and fwrk<>16"
         + " and fdate < " + sqlDat1
         + orgCond
      );
   ExecuteSQL(
      "INSERT INTO " + outTbl + "_1 (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3, FSUM1,FSUM5,FSUM3,FSUM2, "+
			" FSUM4,FSUM6,"+(isSprav?"FSSUM1,FSSUM5,FSSUM3,FSSUM2,FSSUM4,FSSUM6, FSVAL, ":"")+"fdate,"+ (isLastDate? " flastd, flastk,": "") + " fval, fvkurs)"
      + " SELECT"
         + " fdb_sc as fsc,"
         + " fdb_su as fsu,"
         + " fdb_st as fst,"
         + " fdb_cod1 as fcl_cod1,"
         + " fdb_cod2 as fcl_cod2,"
         + " fdb_cod3 as fcl_cod3,"
         + " 0 as fsum1,"
         + sqlRound("fsum", prmDecSum)+" as fsum5,"
         + sqlRound("fsum", prmDecSum)+" as fsum3,"
         + " 0 as fsum2, 0 as fsum4, 0 as fsum6,"  // кредитовые суммы
         + (isSprav
				?
					(" 0 as fssum1,"
					+ sqlRound("fsumspr", prmDecSum)+" as fssum5,"
					+ sqlRound("fsumspr", prmDecSum)+" as fssum3,"
					+ " 0 as fssum2, 0 as fssum4, 0 as fssum6,"
					+ " fvalspr as fsval,")
				:
					""
				)
         + " fdate,"
         + (isLastDate? " fdate AS flastd, " + sqlTo(d00min) + " AS flastk,": "")
         + " fval, fvkurs"
      + " FROM " + schet
      + " WHERE "
         + dbCond
         + " and " + obCond
         + " and fwrk<>16"
         + " and fdate >= " + sqlDat1
         + orgCond
      );
	}
	else
	{
   // Выборка дебетовых проводок
   ExecuteSQL(
      "INSERT INTO " + outTbl + "_1 (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3, FSUM1,FSUM5,FSUM3,FSUM2, "+
			" FSUM4,FSUM6,"+(isSprav?"FSSUM1,FSSUM5,FSSUM3,FSSUM2,FSSUM4,FSSUM6, FSVAL, ":"")+"fdate,"+ (isLastDate? " flastd, flastk,": "") + " fval, fvkurs)"
      + " SELECT"
         + " fdb_sc as fsc,"
         + " fdb_su as fsu,"
         + " fdb_st as fst,"
         + " fdb_cod1 as fcl_cod1,"
         + " fdb_cod2 as fcl_cod2,"
         + " fdb_cod3 as fcl_cod3,"
         + sqlRound(IIF("fdate<" + sqlDat1,"fsum","0"), prmDecSum)+" as fsum1,"
         + sqlRound(IIF("fdate<" + sqlDat1,"0","fsum"), prmDecSum)+" as fsum5,"
         + sqlRound("fsum", prmDecSum)+" as fsum3,"
         + " 0 as fsum2, 0 as fsum4, 0 as fsum6,"  // кредитовые суммы
         + (isSprav
				?
					(sqlRound(IIF("fdate<" + sqlDat1,"fsumspr","0"), prmDecSum)+" as fssum1,"
					+ sqlRound(IIF("fdate<" + sqlDat1,"0","fsumspr"), prmDecSum)+" as fssum5,"
					+ " fsumspr as fssum3,"
					+ " 0 as fssum2, 0 as fssum4, 0 as fssum6,"
					+ " fvalspr as fsval,")
				:
					""
				)
         + " fdate,"
         + (isLastDate? " fdate AS flastd, " + sqlTo(d00min) + " AS flastk,": "")
         + " fval, fvkurs"
      + " FROM " + schet
      + " WHERE"
         + dbCond
         + " and " + obCond
         + " and fwrk<>16"
         + orgCond
      );
     }


	if(isProgress)
		setProgressPos(4, cProgressText);

   // Выборка кредитовых проводок
   ExecuteSQL(
      "INSERT INTO " + outTbl + "_1 (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3, FSUM2,FSUM6,FSUM4,FSUM1, "+
			" FSUM3,FSUM5,"+(isSprav?"FSSUM2,FSSUM6,FSSUM4,FSSUM1,FSSUM3,FSSUM5, FSVAL, ":"")+"fdate, " + (isLastDate? " flastk,flastd,": "") + "fval, fvkurs)"
      + " SELECT"
         + " fcr_sc as fsc,"
         + " fcr_su as fsu,"
         + " fcr_st as fst,"
         + " fcr_cod1 as fcl_cod1,"
         + " fcr_cod2 as fcl_cod2,"
         + " fcr_cod3 as fcl_cod3,"
         + sqlRound(IIF("fdate<" + sqlDat1,"fsum","0"), prmDecSum)+" as fsum2,"
         + sqlRound(IIF("fdate<" + sqlDat1,"0","fsum"), prmDecSum)+" as fsum6,"
         + sqlRound("fsum", prmDecSum)+" as fsum4,"
         + " 0 as fsum1, 0 as fsum3, 0 as fsum5,"
            + (isSprav
					?
						(sqlRound(IIF("fdate<" + sqlDat1,"fsumspr","0"), prmDecSum)+" as fssum2,"
						+ sqlRound(IIF("fdate<" + sqlDat1,"0","fsumspr"), prmDecSum)+" as fssum6,"
						+ sqlRound("fsumspr", prmDecSum)+" as fssum4,"
						+ " 0 as fssum1, 0 as fssum3, 0 as fssum5,"
						+ " fvalspr as fsval,")
					:
						""
					)
         + " fdate,"
         + (isLastDate? " fdate AS flastk, " + sqlTo(d00min) + " AS flastd,": "")
         + " fval, fvkurs"
      + " FROM " + schet
      + " WHERE "
         + crCond
         + " and " + obCond
         + " and fwrk<>16"
         + orgCond
      );


   // Учитывать ли закрывающие проводки
   if( op_zakr(dat1, dat2) )
   {
        // по дебету
      ExecuteSQL(
         "INSERT INTO " + outTbl + "_1 (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3, FSUM1,FSUM5,FSUM3,FSUM2, "+
			" FSUM4,FSUM6,"+(isSprav?"FSSUM1,FSSUM5,FSSUM3,FSSUM2,FSSUM4,FSSUM6, FSVAL, ":"")+"fdate, fval, fvkurs)"
         + " SELECT"
            + " fdb_sc as fsc,"
            + " fdb_su as fsu,"
            + " fdb_st as fst,"
            + " fdb_cod1 as fcl_cod1,"
            + " fdb_cod2 as fcl_cod2,"
            + " fdb_cod3 as fcl_cod3,"
            + sqlRound(IIF("fdate<" + sqlDat1,"fsum","0"), prmDecSum)+" as fsum1,"
			+ sqlRound(IIF("fdate<" + sqlDat1,"0","fsum"), prmDecSum)+" as fsum5,"
            + sqlRound("fsum", prmDecSum)+" as fsum3,"
            + " 0 as fsum2, 0 as fsum4, 0 as fsum6,"
            + (isSprav
					?
						(sqlRound(IIF("fdate<" + sqlDat1,"fsumspr","0"), prmDecSum)+" as fssum1,"
						+ sqlRound(IIF("fdate<" + sqlDat1,"0","fsumspr"), prmDecSum)+" as fssum5,"
						+ sqlRound("fsumspr", prmDecSum)+" as fssum3,"
						+ " 0 as fssum2, 0 as fssum4, 0 as fssum6,"
						+ " fvalspr as fsval,")
					:
						""
					)
            + " fdate,"
            + " fval, fvkurs"
         + " FROM " + schet_zr
         + " WHERE"
            + dbCond
            + " and " + obCond
            + " and fwrk<>16"
         );
      // по кредиту
      ExecuteSQL(
         "INSERT INTO " + outTbl + "_1 (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3, FSUM2,FSUM6,FSUM4,FSUM1, "+
			" FSUM3,FSUM5,"+(isSprav?"FSSUM2,FSSUM6,FSSUM4,FSSUM1,FSSUM3,FSSUM5, FSVAL, ":"")+"fdate, fval, fvkurs)"
         + " SELECT"
            + " fcr_sc as fsc,"
            + " fcr_su as fsu,"
            + " fcr_st as fst,"
            + " fcr_cod1 as fcl_cod1,"
            + " fcr_cod2 as fcl_cod2,"
            + " fcr_cod3 as fcl_cod3,"
            + sqlRound(IIF("fdate<" + sqlDat1,"fsum","0"), prmDecSum)+" as fsum2,"
			+ sqlRound(IIF("fdate<" + sqlDat1,"0","fsum"), prmDecSum)+" as fsum6,"
            + sqlRound("fsum", prmDecSum)+" as fsum4,"
            + " 0 as fsum1, 0 as fsum3, 0 as fsum5,"
            + (isSprav
					?
						(sqlRound(IIF("fdate<" + sqlDat1,"fsumspr","0"), prmDecSum)+" as fssum2,"
						+ sqlRound(IIF("fdate<" + sqlDat1,"0","fsumspr"), prmDecSum)+" as fssum6,"
						+ sqlRound("fsumspr", prmDecSum)+" as fssum4,"
						+ " 0 as fssum1, 0 as fssum3, 0 as fssum5,"
						+ " fvalspr as fsval,")
					:
						""
					)
            + " fdate,"
            + " fval, fvkurs"
         + " FROM " + schet_zr
         + " WHERE "
            + crCond
            + " and " + obCond
            + " and fwrk<>16"
            + orgCond
         );
   }



	var lastNo = 1;

	if(isProgress)
		setProgressPos(5, cProgressText);


	if(isSprav)
	{
		DropTable(outTbl + "_" + (lastNo+1));

		ExecuteSQL(
		   "CREATE TABLE " + outTbl + "_" + (lastNo+1) + "("
				+ " fsc "+SqlType(sqlTSc)+","
				+ " fsu "+SqlType(sqlTSu)+","
				+ " fst "+SqlType(sqlTSt)+","
		      + " fcl_cod1 "+SqlType("INTEGER4")+","
		      + " fcl_cod2 "+SqlType("INTEGER4")+","
		      + " fcl_cod3 "+SqlType("INTEGER4")+","
		      + " fsum1 "+SqlType("MONEY")+","
		      + " fsum2 "+SqlType("MONEY")+","
		      + " fsum3 "+SqlType("MONEY")+","
		      + " fsum4 "+SqlType("MONEY")+","
		      + " fsum5 "+SqlType("MONEY")+","
		      + " fsum6 "+SqlType("MONEY")+","
		      + " fval "+SqlType("SHORT")+","
		      + " fdate "+SqlType("DATE")+","
		      + " fvkurs "+SqlType("SINGLE")+","
		      + " fkurs "+SqlType("MONEY")+","
		      + " fssum1 "+SqlType("MONEY")+","
		      + " fssum2 "+SqlType("MONEY")+","
		      + " fssum3 "+SqlType("MONEY")+","
		      + " fssum4 "+SqlType("MONEY")+","
		      + " fssum5 "+SqlType("MONEY")+","
		      + " fssum6 "+SqlType("MONEY")
		      + (isLastDate ? (",flastd " + SqlType("DATE") + ", flastk " + SqlType("DATE")) : "")
		      + ")"
		   );


		// Переводим по курсу из справочника курсов валют
		ExecuteSQL(
			"INSERT INTO " + outTbl + "_" + (lastNo+1) + "("
				+ "fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3, fsum1, fsum2, fsum3, fsum4, fsum5, fsum6,"
				+ "fval, fdate, fvkurs, fkurs,"
				+ "fssum1, fssum2, fssum3, fssum4, fssum5, fssum6"
				+ (isLastDate? ", flastd, flastk" : "") + ")"
				+ " SELECT fsc, fsu, fst,"
				+ " fcl_cod1, fcl_cod2, fcl_cod3,"
				+ "  "+Convert("t.fsum1","MONEY")+" AS fsum1"
				+ ", "+Convert("t.fsum2","MONEY")+" AS fsum2"
				+ ", "+Convert("t.fsum3","MONEY")+" AS fsum3"
				+ ", "+Convert("t.fsum4","MONEY")+" AS fsum4"
				+ ", "+Convert("t.fsum5","MONEY")+" AS fsum5"
				+ ", "+Convert("t.fsum6","MONEY")+" AS fsum6"
				+ ", fval, fdate, fvkurs,"
				+ Convert(sqlRate("t", "fsval"), "MONEY") + " as fkurs,"
				+ " t.fssum1*"+ sqlRate("t", "fsval") + " as fssum1,"
				+ " t.fssum2*"+ sqlRate("t", "fsval") + "  as fssum2,"
				+ " t.fssum3*"+ sqlRate("t", "fsval") + "  as fssum3,"
				+ " t.fssum4*"+ sqlRate("t", "fsval") + "  as fssum4,"
				+ " t.fssum5*"+ sqlRate("t", "fsval") + "  as fssum5,"
				+ " t.fssum6*"+ sqlRate("t", "fsval") + "  as fssum6"
				+ (isLastDate? ", flastd, flastk" : "")
			+ " FROM " + outTbl + "_" + lastNo + " t"
			+ " WHERE t.fsval<>0"
			);

		if(isProgress)
			setProgressPos(6, cProgressText);

		ExecuteSQL(
			"INSERT INTO " + outTbl + "_" + (lastNo+1)
			+ " (fsc, fsu, fst,"
				+ " fcl_cod1, fcl_cod2, fcl_cod3,"
				+ " fsum1, fsum2, fsum3, fsum4, fsum5, fsum6,"
				+ " fval, fdate, fvkurs,"
				+ " fkurs,"
				+ " fssum1, fssum2, fssum3,"
				+ " fssum4, fssum5, fssum6)"
			+ " SELECT fsc, fsu, fst,"
				+ " fcl_cod1, fcl_cod2, fcl_cod3,"
				+ " fsum1, fsum2, fsum3, fsum4, fsum5, fsum6,"
				+ " fval, fdate, fvkurs,"
				+ Convert("1", "MONEY") + " as fkurs,"
				+ " fssum1, fssum2, fssum3,"
				+ " fssum4, fssum5, fssum6"
			+ " FROM " + outTbl + "_" + lastNo
			+ " WHERE fsval=0"
			);

		if(isProgress)
			setProgressPos(7, cProgressText);

		lastNo++;
	}

	if(is1Val || isPSum)
	{
		// Итоги по разновалютным строкам подбиваются
		// в нац. валюте. Для этого нам нужны как валютные
		// суммы (fpsumX), та и суммы в грн (fsumX)
		var fldsPSum = "";
		var fldsPS = "";
		var fldsPSumCreate = "";
		if(isPSum)
		{
			fldsPS = " fpval, fpsum1,fpsum2, fpsum3, fpsum4, fpsum5, fpsum6, ";
			fldsPSum = "t.fval as fpval,"
				+ "t.fsum1 as fpsum1,"
				+ "t.fsum2 as fpsum2,"
				+ "t.fsum3 as fpsum3,"
				+ "t.fsum4 as fpsum4,"
				+ "t.fsum5 as fpsum5,"
				+ "t.fsum6 as fpsum6,";
			fldsPSumCreate = "fpval "+SqlType("SHORT")+","
				+ "fpsum1 "+SqlType("MONEY")+","
				+ "fpsum2 "+SqlType("MONEY")+","
				+ "fpsum3 "+SqlType("MONEY")+","
				+ "fpsum4 "+SqlType("MONEY")+","
				+ "fpsum5 "+SqlType("MONEY")+","
				+ "fpsum6 "+SqlType("MONEY")+",";
		}

		DropTable(outTbl + "_" + (lastNo+1))

		if(typeof(valCod)!="number")
		{
			ExecuteSQL(
			   "CREATE TABLE " + outTbl + "_" + (lastNo+1) + "("
					+ " fsc "+SqlType(sqlTSc)+","
					+ " fsu "+SqlType(sqlTSu)+","
					+ " fst "+SqlType(sqlTSt)+","
			      + " fcl_cod1 "+SqlType("INTEGER4")+","
			      + " fcl_cod2 "+SqlType("INTEGER4")+","
			      + " fcl_cod3 "+SqlType("INTEGER4")+","
			      + " fsum1 "+SqlType("MONEY")+","
			      + " fsum2 "+SqlType("MONEY")+","
			      + " fsum3 "+SqlType("MONEY")+","
			      + " fsum4 "+SqlType("MONEY")+","
			      + " fsum5 "+SqlType("MONEY")+","
			      + " fsum6 "+SqlType("MONEY")+","
			      + fldsPSumCreate
			      + (!isSprav? "" :
							(" fssum1 "+SqlType("MONEY")+","
							+ " fssum2 "+SqlType("MONEY")+","
							+ " fssum3 "+SqlType("MONEY")+","
							+ " fssum4 "+SqlType("MONEY")+","
							+ " fssum5 "+SqlType("MONEY")+","
							+ " fssum6 "+SqlType("MONEY")+","))
			      + " fkurs "+SqlType("MONEY")+","
			      + " fval "+SqlType("SHORT")+","
			      + " fdate "+SqlType("DATE")
			      + (isLastDate ? (",flastd " + SqlType("DATE") + ", flastk " + SqlType("DATE")) : "")
			      + ")"
			   );

			// Перевод в национальную валюту
			// Вначале определяем курсы по ненациональным валютам
			// по которым не проставлен курс
			ExecuteSQL(
				"INSERT INTO " + outTbl + "_" + (lastNo+1)
					+ "(fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3,"
					+ "fsum1,fsum2,fsum3,fsum4,fsum5,fsum6," + fldsPS
					+ (!isSprav? "":
						" fssum1, fssum2, fssum3, fssum4, fssum5, fssum6,"
						)
					+ "fkurs,fval,fdate"
					+ (isLastDate? ", flastd, flastk": "")
					+ ") "
					+ " SELECT fsc, fsu, fst,"
					+ " fcl_cod1, fcl_cod2, fcl_cod3,"
					+ sqlRound(Convert("t.fsum1*k.fkurs","MONEY"),prmDecSum) + " AS fsum1,"
					+ sqlRound(Convert("t.fsum2*k.fkurs","MONEY"),prmDecSum) + " AS fsum2,"
					+ sqlRound(Convert("t.fsum3*k.fkurs","MONEY"),prmDecSum) + " AS fsum3,"
					+ sqlRound(Convert("t.fsum4*k.fkurs","MONEY"),prmDecSum) + " AS fsum4,"
					+ sqlRound(Convert("t.fsum5*k.fkurs","MONEY"),prmDecSum) + " AS fsum5,"
					+ sqlRound(Convert("t.fsum6*k.fkurs","MONEY"),prmDecSum) + " AS fsum6,"
					+ fldsPSum
					+ (!isSprav? "":
						" fssum1, fssum2, fssum3, fssum4, fssum5, fssum6,"
						)
					+ Convert(IIF("k.fkurs is null","1","k.fkurs"), "DBL")+" as fkurs,"
					+ Convert("0", "INT") + " as fval,"
					+ " fdate"
					+ (isLastDate? ", flastd, flastk": "")
				+ " FROM " + outTbl + "_" + lastNo + " t"
					+ " LEFT JOIN ^kursval k"
						+ " ON t.fval = k.fkod"
							+ " AND (t.fdate BETWEEN k.fbdat AND k.fedat)"
				+ " WHERE t.fval<>0"
					+ " AND t.fvkurs=0"
			   );

			sFVKURS = IIF("t.fvkurs IS NULL", "1", "t.fvkurs")
			sFVKURS = sqlRound("t.fvkurs", 10);
			//AL: В MSSQL в выражениях нельзя использовать псевдонимы
			ExecuteSQL(
			   "INSERT INTO " + outTbl + "_" + (lastNo+1)+" (fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3"
			   	+ ",FSUM1,FSUM2,FSUM3,FSUM4, FSUM5,FSUM6,"
			   	+ fldsPS+(isSprav?"FSSUM1,FSSUM2,FSSUM3,FSSUM4,FSSUM5,FSSUM6, ":"")+"fkurs, fval, fdate" + (isLastDate? ", flastd, flastk": "") + ")"
				+ " SELECT fsc, fsu, fst,"
					+ " fcl_cod1, fcl_cod2, fcl_cod3,"
					+ sqlRound(Convert("t.fsum1*"+sFVKURS,"MONEY"),prmDecSum) + " AS fsum1,"
					+ sqlRound(Convert("t.fsum2*"+sFVKURS,"MONEY"),prmDecSum) + " AS fsum2,"
					+ sqlRound(Convert("t.fsum3*"+sFVKURS,"MONEY"),prmDecSum) + " AS fsum3,"
					+ sqlRound(Convert("t.fsum4*"+sFVKURS,"MONEY"),prmDecSum) + " AS fsum4,"
					+ sqlRound(Convert("t.fsum5*"+sFVKURS,"MONEY"),prmDecSum) + " AS fsum5,"
					+ sqlRound(Convert("t.fsum6*"+sFVKURS,"MONEY"),prmDecSum) + " AS fsum6,"
					+ fldsPSum
					+ (!isSprav? "":
						" fssum1, fssum2, fssum3, fssum4, fssum5, fssum6,"
						)
					+ " " + sFVKURS
						+ " AS fkurs,"
					+ Convert("0", "INT") + " as fval,"
					+ " fdate"
					+ (isLastDate? ", flastd, flastk": "")
				+ " FROM " + outTbl + "_" + lastNo + " t"
				+ " WHERE t.fval<>0"
					+ " AND t.fvkurs<>0"
			   );


			// Для национальной валюты курс=1
			ExecuteSQL(
			   "INSERT INTO " + outTbl + "_" + (lastNo+1)
					+ "(fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3,"
					+ "fsum1,fsum2,fsum3,fsum4,fsum5,fsum6,"
					+ fldsPS+(isSprav?"FSSUM1,FSSUM2,FSSUM3,FSSUM4,FSSUM5,FSSUM6,":"")
					+ "fdate, fval, fkurs"
					+ (isLastDate? ", flastd, flastk": "") + ")"
			      + " SELECT fsc, fsu, fst,"
			         + " fcl_cod1, fcl_cod2, fcl_cod3,"
			         + " fsum1,fsum2,fsum3,fsum4,fsum5,fsum6,"
						+ fldsPSum
						+ (!isSprav? "":
							" fssum1, fssum2, fssum3, fssum4, fssum5, fssum6,"
							)
					+ " fdate,"
					+ Convert("0", "INT") + " as fval,"
					+ Convert("1", "MONEY") + " as fkurs"
					+ (isLastDate? ", flastd, flastk": "")
			      + " FROM " + outTbl + "_" + lastNo + " t"
			      + " WHERE t.fval=0 or t.fval is null"
			   );
		}
		else  // выборка по одной валюте
		{
			ExecuteSQL(
			   "CREATE TABLE " + outTbl + "_" + (lastNo+1) + "("
					+ " fsc "+SqlType(sqlTSc)+","
					+ " fsu "+SqlType(sqlTSu)+","
					+ " fst "+SqlType(sqlTSt)+","
			      + " fcl_cod1 "+SqlType("INTEGER4")+","
			      + " fcl_cod2 "+SqlType("INTEGER4")+","
			      + " fcl_cod3 "+SqlType("INTEGER4")+","
			      + " fsum1 "+SqlType("MONEY")+","
			      + " fsum2 "+SqlType("MONEY")+","
			      + " fsum3 "+SqlType("MONEY")+","
			      + " fsum4 "+SqlType("MONEY")+","
			      + " fsum5 "+SqlType("MONEY")+","
			      + " fsum6 "+SqlType("MONEY")+","
			      + (!isSprav? "" :
							(" fssum1 "+SqlType("MONEY")+","
							+ " fssum2 "+SqlType("MONEY")+","
							+ " fssum3 "+SqlType("MONEY")+","
							+ " fssum4 "+SqlType("MONEY")+","
							+ " fssum5 "+SqlType("MONEY")+","
							+ " fssum6 "+SqlType("MONEY")+","))
			      + " fval "+SqlType("SHORT")+","
			      + " fdate "+SqlType("DATE")
			      + (isLastDate ? ",flastd " + SqlType("DATE") + ", flastk " + SqlType("DATE") : "")
			      + ")"
			   );

			ExecuteSQL(
				"INSERT INTO " + outTbl + "_" + (lastNo+1)
					+ "(fsc, fsu, fst,fcl_cod1, fcl_cod2, fcl_cod3,"
					+ "fsum1,fsum2,fsum3,fsum4,fsum5,fsum6,"
					+ (!isSprav? "":
						" fssum1, fssum2, fssum3, fssum4, fssum5, fssum6,"
						)
					+ "fval,fdate"
					+ (isLastDate? ", flastd, flastk": "") + ") "
					+ " SELECT fsc, fsu, fst, "
					+ " fcl_cod1, fcl_cod2, fcl_cod3,"
					+ " fsum1,fsum2,fsum3,fsum4,fsum5,fsum6,"
					+ (!isSprav? "":
						" fssum1, fssum2, fssum3, fssum4, fssum5, fssum6,"
						)
					+ valCod + " as fval,"
					+ " fdate"
					+ (isLastDate? ", flastd, flastk": "")
				+ " FROM " + outTbl + "_" + lastNo + " t"
				+ " WHERE t.fval=" + valCod
				);
		}
	   lastNo++;

		if(is2val)
		{
		   var valKod = typeof(usedval)=="number"?usedval:0;
		   DropTable(outTbl + "_" + (lastNo+1));

		   var sKurs = Convert((usedval==0? 1: sqlRate("t", usedval)), "MONEY");
		   ExecuteSQL(
		      "SELECT fsc, fsu, fst, "
		         + " fcl_cod1, fcl_cod2, fcl_cod3,"
		         + " fsum1, fsum2, fsum3, fsum4, fsum5, fsum6,"
		         + " t.fsum1/"+sKurs+" as fvsum1,"
		         + " t.fsum2/"+sKurs+" as fvsum2,"
		         + " t.fsum3/"+sKurs+" as fvsum3,"
		         + " t.fsum4/"+sKurs+" as fvsum4,"
		         + " t.fsum5/"+sKurs+" as fvsum5,"
		         + " t.fsum6/"+sKurs+" as fvsum6,"
		         + valKod + " as fval,"
		         + Convert((usedval==0? 1: sqlRate("t", usedval)), "MONEY")
		            + " as fkurs"
					+ (isLastDate? ", flastd, flastk": "")
		      + " INTO " + outTbl + "_" + (lastNo+1)
		      + " FROM " + outTbl + "_" + lastNo + "  t"
		      );
		   lastNo++;
		}
	}

	ExecuteSQL("ALTER TABLE "+outTbl + "_" + lastNo+" ADD COLUMN FSUMOBR "+SqlType("MONEY"))
	strSQL = " UPDATE"
				+" "+outTbl+"_"+lastNo
			+" SET"
				+" FSUMOBR = "+Convert("ABS(FSUM5)+ABS(FSUM6)", "MONEY")
	ExecuteSQL(strSQL);
	
	if(isProgress)
		setProgressPos(8, cProgressText);

	var isTT = "";

	if(isPSum)
	{
		lastNo++;
		makeSum("VL", outTbl + "_" + lastNo,
			outTbl + "_" + (lastNo-1),
			is2val, isSprav, isPSum);
		isTT = "!";
	}
   if(mode.indexOf("3")!=-1)  // по трем аналитикам
   {
      makeSum(isTT + "A3", outTbl, outTbl + "_" + lastNo, is2val, isSprav, isPSum);
      
	  //vlad 14.02.12
	  //Непонятный вызов:
	  //turnTbl(outTbl, "%", is2val, isSprav, isPSum);
      
	  turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
      if(mode.indexOf("N")==-1)  // нет отмены итогов
      {
         makeSum("%!A2", outTbl + "_a2", outTbl /*+ "_" + lastNo*/, is2val, isSprav, isPSum);

		 //vlad 26.01.2012
		 //Свернем и тут тоже для получения правильного развернутого сальдо
		 //if (turnLevel != "A3")
         	//turnTbl(outTbl + "_a2", "%", is2val, isSprav, isPSum);
         	turnTbl(outTbl + "_a2", scVid, is2val, isSprav, isPSum);

         insertTbl("a2", outTbl, outTbl + "_a2");

	      //turnTbl(outTbl + "_a2", scVid, is2val, isSprav, isPSum);
         makeSum("%!A1", outTbl + "_a1", outTbl, is2val, isSprav, isPSum);
		 //vlad 26.01.2012
		 //Свернем и тут тоже для получения правильного развернутого сальдо
		 //if (turnLevel != "A3" && turnLevel != "A2")
         	//turnTbl(outTbl + "_a1", "%", is2val, isSprav, isPSum);
         	turnTbl(outTbl + "_a1", scVid, is2val, isSprav, isPSum);
         insertTbl("a1", outTbl, outTbl + "_a1");

         makeSum("%!ST", outTbl + "_st", outTbl, is2val, isSprav, isPSum);
         //vlad 26.03.2012
         //Свернем и тут тоже для получения правильного развернутого сальдо
         turnTbl(outTbl + "_st", scVid, is2val, isSprav, isPSum);
         insertTbl("st", outTbl, outTbl + "_st");

         makeSum("%!SU", outTbl + "_su", outTbl, is2val, isSprav, isPSum);
         //vlad 26.03.2012
         //Свернем и тут тоже для получения правильного развернутого сальдо
         turnTbl(outTbl + "_su", scVid, is2val, isSprav, isPSum);
         insertTbl("su", outTbl, outTbl + "_su");

         makeSum("%!SC", outTbl + "_sc", outTbl, is2val, isSprav, isPSum);
         //vlad 26.03.2012
         //Свернем и тут тоже для получения правильного развернутого сальдо
         turnTbl(outTbl + "_sc", scVid, is2val, isSprav, isPSum);
         insertTbl("sc", outTbl, outTbl + "_sc");

         //turnTbl(outTbl,                "%", is2val, isSprav, isPSum);
      }
   }
   else if(mode.indexOf("C")!=-1)
   {
      //vlad 15.02.12
      //Для активно-пассивных счетов с развернутым сальдо надо начинать 
	  //сворачивать с самого низа. Сразу же получить правильный итог по счету не
	  //представляется возможным
      /*
      if(turnLevel=="SC")
      {
         makeSum(isTT + "SC", outTbl, outTbl + "_" + lastNo, is2val, isSprav, isPSum);
         turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
      }
		else
		{
			var tmpTbl = outTbl + "_" + turnLevel;
			makeSum(turnLevel, tmpTbl, outTbl + "_" + lastNo, is2val, isSprav, isPSum);

			//vlad 07.02.2012
			//Если передать "%", то сальдо реально развернет даже на активных и пассивных счетах,
			//в связи с чем некорректно отрабатывает DpObsl в расчете кассовой книги
			//Если передать "", то все будет "свернуто" исходя из настройки в плане счетов
			//turnTbl(tmpTbl, "%", is2val, isSprav, isPSum);
			turnTbl(tmpTbl, "", is2val, isSprav, isPSum);
			makeSum(isTT + "SC", outTbl, tmpTbl, is2val, isSprav, isPSum);
			turnTbl(outTbl, "", is2val, isSprav, isPSum);
		}
      */

	//SAM 24.04.2012 
	//сворачивать будет до того уровня, который нам заказали
	var levelSC = (objSc.st != 0 ? "ST" : (objSc.su != 0 ? "SU" : "SC"));
	  switch(lastNo)
	  {
		case 3:
		  var tmpTbl = outTbl + "_" + lastNo;
		  makeSum("A3", outTbl + "_a3", tmpTbl, is2val, isSprav, isPSum);
		  turnTbl(outTbl + "_a3", scVid, is2val, isSprav, isPSum);
		  makeSum("A2", outTbl + "_a2", outTbl + "_a3", is2val, isSprav, isPSum);
		  turnTbl(outTbl + "_a2", scVid, is2val, isSprav, isPSum);
		  makeSum("A1", outTbl + "_a1" , outTbl + "_a2", is2val, isSprav, isPSum);
		  turnTbl(outTbl + "_a1", scVid, is2val, isSprav, isPSum);
		  //makeSum(isTT + "SC", outTbl, outTbl + "_a1", is2val, isSprav, isPSum);
		  makeSum(isTT + levelSC, outTbl, outTbl + "_a1", is2val, isSprav, isPSum);
		  //vlad 26.03.2012
		  //Надо свернуть на счете, если на субсчетах вид "сворачивания" не такой
		  turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
		  break;
		case 2:
		  var tmpTbl = outTbl + "_" + lastNo;
		  makeSum("A2", outTbl + "_a2", tmpTbl, is2val, isSprav, isPSum);
		  turnTbl(outTbl + "_a2", scVid, is2val, isSprav, isPSum);
		  makeSum("A1", outTbl + "_a1" , outTbl + "_a2", is2val, isSprav, isPSum);
		  turnTbl(outTbl + "_a1", scVid, is2val, isSprav, isPSum);
		  //makeSum(isTT + "SC", outTbl, outTbl + "_a1", is2val, isSprav, isPSum);
		  makeSum(isTT + levelSC, outTbl, outTbl + "_a1", is2val, isSprav, isPSum);
		  //vlad 26.03.2012
		  //Надо свернуть на счете, если на субсчетах вид "сворачивания" не такой
		  turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
		  break;
		case 1:
		  var tmpTbl = outTbl + "_" + lastNo;
		  makeSum("A1", outTbl + "_a1" , tmpTbl, is2val, isSprav, isPSum);
		  turnTbl(outTbl + "_a1", scVid, is2val, isSprav, isPSum);
		  //makeSum(isTT + "SC", outTbl, outTbl + "_a1", is2val, isSprav, isPSum);
		  makeSum(isTT + levelSC, outTbl, outTbl + "_a1", is2val, isSprav, isPSum);
		  //vlad 26.03.2012
		  //Надо свернуть на счете, если на субсчетах вид "сворачивания" не такой
		  turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
		default:
		  var tmpTbl = outTbl + "_" + lastNo;
		  //makeSum(isTT + "SC", outTbl, tmpTbl, is2val, isSprav, isPSum);
		  makeSum(isTT + levelSC, outTbl, tmpTbl, is2val, isSprav, isPSum);
		  //vlad 26.03.2012
		  //Надо свернуть на счете, если на субсчетах вид "сворачивания" не такой
		  turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
	  }
   }
   else
   {
      //alert("popalo");
      //vvs 29.02.2004
      // выводим только с 1-й аналитикой.
      // Но для начала для счета с развернутым сальдо надо
      // все равно свернуть сальдо на нижней из аналитик и
      // снизу-вверх подбивать итоги
      /*
      makeSum("A1", outTbl, outTbl + "_" + lastNo, is2val, isSprav, isPSum);
      turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
      makeSum("!ST", outTbl + "_st", outTbl, is2val, isSprav, isPSum);
      if(mode.indexOf("N")==-1)  // нет отмены итогов
      {
         makeSum("!SU", outTbl + "_su", outTbl, is2val, isSprav, isPSum);
         makeSum("!SC", outTbl + "_sc", outTbl, is2val, isSprav, isPSum);
         insertTbl("su", outTbl, outTbl + "_su");
         insertTbl("sc", outTbl, outTbl + "_sc");
      }
      insertTbl("st", outTbl, outTbl + "_st");
      */
      makeSum("A3", outTbl + "_a3", outTbl + "_" + lastNo, is2val, isSprav, isPSum);
      turnTbl(outTbl + "_a3", scVid, is2val, isSprav, isPSum);
      makeSum("A2", outTbl + "_a2", outTbl + "_a3", is2val, isSprav, isPSum);
      //vlad 26.01.2012
      //Свернем и тут тоже для получения правильного развернутого сальдо
      //if (turnLevel != "A3")
      {
	      turnTbl(outTbl + "_a2", scVid, is2val, isSprav, isPSum);
	  }
      makeSum("A1", outTbl , outTbl + "_a2", is2val, isSprav, isPSum);
      //vlad 26.01.2012
      //Свернем и тут тоже для получения правильного развернутого сальдо
      //if (turnLevel != "A3" && turnLevel != "A2")
      {
	      turnTbl(outTbl, scVid, is2val, isSprav, isPSum);
	  }

      makeSum("!ST", outTbl + "_st", outTbl, is2val, isSprav, isPSum);
      //vlad 26.03.2012
      //Свернем и тут тоже для получения правильного развернутого сальдо
      //turnTbl(outTbl + "_st", scVid, is2val, isSprav, isPSum);
      if(mode.indexOf("N")==-1)  // нет отмены итогов
      {
         makeSum("!SU", outTbl + "_su", outTbl, is2val, isSprav, isPSum);
         //vlad 26.03.2012
         //Свернем и тут тоже для получения правильного развернутого сальдо
         //turnTbl(outTbl + "_su", scVid, is2val, isSprav, isPSum);
         makeSum("!SC", outTbl + "_sc", outTbl, is2val, isSprav, isPSum);
         //vlad 26.03.2012
         //Свернем и тут тоже для получения правильного развернутого сальдо
         //turnTbl(outTbl + "_sc", scVid, is2val, isSprav, isPSum);
         insertTbl("su", outTbl, outTbl + "_su");
         insertTbl("sc", outTbl, outTbl + "_sc");
      }
      insertTbl("st", outTbl, outTbl + "_st");

   }

   if(isPSum)
		insertTbl("vl", outTbl, outTbl + "_" + lastNo);

   // Сворачивание итогов для акт.-пас. счета со свернутым сальдо
   if(mode.indexOf("N")==-1 && scVid=="*")
      turnTbl(outTbl, "*", is2val, isSprav, isPSum);

   // Индекс для вывода строк в порядке, естественном для пользовател
   if(!SQL) //AL: Индексы будут включены когда будуп реализ. организации и пользователи.
   ExecuteSQL(
		"CREATE INDEX oblns ON " + outTbl
         + " (fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3"
				+ ", fval, fistotal" + (isPSum? ", fpval": "") + ", fttlevel)"
      );
   ExecuteSQL(
		"ALTER TABLE " + outTbl
		+ " ADD fwid "+SqlType("COUNTER")
		);

	if(isProgress)
		setProgressPos(0);
	return true;
}

function makeSum(mode, tblInto, tblFrom, is2val, isSprav, isPSum)
{
   mode = mode.toUpperCase();

   var flds, flds0As, tLevel, having, groupBy, tType, where="";

   var isTotal = strIn("!", mode);
   mode = mode.replace(/\!/g, "");

   is1Level = strIn("%", mode);
   mode = mode.replace(/\%/g, "");

   if(isTotal)
   {
      tType = "'" + mode + "'";
   }


   switch(mode)
   {
   case "SC":
       flds = "fsc, "+Convert("0",sqlTSu)+" as fsu, "+Convert("0",sqlTSt)+" as fst, "
				+Convert("0","INT")	+" as fcl_cod1, "+Convert("0","INT")+" as fcl_cod2, "+Convert("0","INT")+" as fcl_cod3,"

      having = "HAVING max(fsu)>0";
      groupBy = "GROUP BY fsc, fval";
      where = "(fsu<>0 AND fst=0 AND fcl_cod1=0)"
			+ " OR (fsu=0 AND fst<>0 AND fcl_cod1=0 AND fistotal<>" + sqlTrue + ")"
			+ " OR (fsu=0 AND fst=0 AND fcl_cod1<>0 AND fcl_cod2=0 AND fistotal<>" + sqlTrue + ")";
      tLevel = 1;
      break;
   case "SU":
       flds = "fsc, fsu, "+Convert("0",sqlTSt)+" as fst, "+Convert("0","INT")
					+" as fcl_cod1, "+Convert("0","INT")+" as fcl_cod2, "+Convert("0","INT")+" as fcl_cod3,"

      having = "HAVING max(fst)>0";
      groupBy = "GROUP BY fsc, fsu, fval";
      where = "(fst<>0 AND fcl_cod1=0)"
			+ " OR (fst=0 AND fcl_cod1<>0 AND fcl_cod2=0 AND fistotal<>" + sqlTrue + ")";
      tLevel = 2;
      break;
   case "ST":
       flds = "fsc, fsu, fst, "+Convert("0","INT")
					+" as fcl_cod1, "+Convert("0","INT")+" as fcl_cod2, "+Convert("0","INT")+" as fcl_cod3,"
      having = "HAVING max(fcl_cod1)>0";
      groupBy = "GROUP BY fsc, fsu, fst, fval";
      where = "fcl_cod1<>0 AND fcl_cod2=0";
      tLevel = 3;
      break;
   case "A1":
       flds = "fsc, fsu, fst, fcl_cod1, "+Convert("0","INT")+" as fcl_cod2, "+Convert("0","INT")+" as fcl_cod3,"
      having = "HAVING max(fcl_cod2)>0";
      groupBy = "GROUP BY fsc, fsu, fst, fcl_cod1, fval";
      where = "fcl_cod2<>0 AND fcl_cod3=0";
      tLevel = 4;
      break;
   case "A2":
       flds = "fsc, fsu, fst, fcl_cod1, fcl_cod2, "+Convert("0","INT")+" as fcl_cod3,"
      having = "HAVING max(fcl_cod3)>0";
      groupBy = "GROUP BY fsc, fsu, fst, fcl_cod1, fcl_cod2, fval";
      where = " fcl_cod3<>0 ";
      tLevel = 5;
      break;
   case "A3":
      flds = "fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3,";
      having = (isPSum? "HAVING max(fpval)>0": "");
      groupBy = "GROUP BY fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3, fval";
      tLevel = 6;
      break;

   case "VL":
		flds = "fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3,";
		having = "";
      groupBy = "GROUP BY fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3, fval, fpval";
      tLevel = 7;
      break;
   }

   if(!isTotal)
   {
      having = " HAVING "
			+ Convert("sum(t.fsum1)","MONEY")+"<>0 or "
			+ Convert("sum(t.fsum2)","MONEY")+"<>0 or "
			+ Convert("sum(t.fsum3)","MONEY")+"<>0 or "
			+ Convert("sum(t.fsum4)","MONEY")+"<>0 or "
			+ Convert("sum(t.fsum5)","MONEY")+"<>0 or "
			+ Convert("sum(t.fsum6)","MONEY")+"<>0 "
			+ " or " + Convert("sum(t.fsumobr)","MONEY")+"<>0"
      tLevel = 0;
      tType = "null";
   }
   DropTable(tblInto);
   ExecuteSQL(
      "SELECT "+Convert("fsc/10","BYTE")+" as fclass, " + flds
			+ " fval,"
         + Convert("sum(t.fsum1)","MONEY")+" as fsum1,"
         + Convert("sum(t.fsum2)","MONEY")+" as fsum2,"
         + Convert("sum(t.fsum3)","MONEY")+" as fsum3,"
         + Convert("sum(t.fsum4)","MONEY")+" as fsum4,"
         + Convert("sum(t.fsum5)","MONEY")+" as fsum5,"
         + Convert("sum(t.fsum6)","MONEY")+" as fsum6,"
         + Convert("sum(t.fsumobr)","MONEY")+" as fsumobr,"
         + (!is2val? "":
            " sum(t.fvsum1) as fvsum1,"
            + " sum(t.fvsum2) as fvsum2,"
            + " sum(t.fvsum3) as fvsum3,"
            + " sum(t.fvsum4) as fvsum4,"
            + " sum(t.fvsum5) as fvsum5,"
            + " sum(t.fvsum6) as fvsum6,"
            )
         + (!isSprav? "":
            " sum(t.fssum1) as fssum1,"
            + " sum(t.fssum2) as fssum2,"
            + " sum(t.fssum3) as fssum3,"
            + " sum(t.fssum4) as fssum4,"
            + " sum(t.fssum5) as fssum5,"
            + " sum(t.fssum6) as fssum6,"
            )
         + (isPSum && mode!="VL"?
            " sum(t.fsum1) as fpsum1,"
            + " sum(t.fsum2) as fpsum2,"
            + " sum(t.fsum3) as fpsum3,"
            + " sum(t.fsum4) as fpsum4,"
            + " sum(t.fsum5) as fpsum5,"
            + " sum(t.fsum6) as fpsum6,"
            + " 0 as fpval,"
            : ""
            )
         + (isPSum && mode=="VL"?
            " sum(t.fpsum1) as fpsum1,"
            + " sum(t.fpsum2) as fpsum2,"
            + " sum(t.fpsum3) as fpsum3,"
            + " sum(t.fpsum4) as fpsum4,"
            + " sum(t.fpsum5) as fpsum5,"
            + " sum(t.fpsum6) as fpsum6,"
            + " fpval,"
            : ""
            )
         + " max(t.flastd) AS flastd, "
			+ " max(t.flastk) AS flastk, "
         + Convert(sqlTo(isTotal), "INT") + " as fistotal,"
         + Convert(tType,"NVARCHAR",10) + " as ftt,"
         + " " + tLevel + " as fttlevel,"
         + Convert("0", "INT") + " as forg0"
      + " INTO " + tblInto
      + " FROM " + tblFrom + " t"
      + (is1Level && where? " WHERE " + where: "")
      + " " + groupBy
      + " " + having
// Предыдущая строка была закомментирована, но без нее
// формируются лишние итоги
      );

   if (SUBD == ORACLE)
   {
	   // Если будет тормозить - нужно будет что-то делать

		ExecuteSQL(	"ALTER TABLE " + tblInto + " MODIFY (ftt varchar(30))");
   }
}

function insertTbl(mode, tblInto, tblFrom)
{
   var where="";
   switch(mode.toUpperCase())
   {
   case "SC":
      where += "fsu<>0 and ";
   case "SU":
      where += "fst<>0 and ";
   case "ST":
      where += "fcl_cod1<>0 and ";
   case "A1":
      where += "fcl_cod2<>0 and ";
   case "A2":
      where += "fcl_cod3<>0";
   case "A3":
   }
   // Не понял, зачем нужен where?
   where = "";

   var sFields = getColumn(tblInto);

   ExecuteSQL(
      "INSERT INTO " + tblInto + "(" + sFields + ")"
      + " SELECT " + sFields
      + " FROM "  + tblFrom
      + (where? " WHERE " + where: "")
      );
}

// Сворачивание сумм в таблице
function turnTbl(tblName, scVid, is2val, isSprav, isPSum)
{
//alert("turnTbl(tblName="+tblName+", scVid="+scVid+", is2val="+is2val+", isSprav="+isSprav+", isPSum="+isSprav+")")
   var t = new Date().getTime();
	//vvs 24.02.2004
   // Для счетов с развернутым сальдо надо подглядывать в план счетов
   // и	1) сворачивать сальдо на последней аналитике
   //	2) оставлять развернутым на уровнях выше нижнего
   //      с учетом "точечки", которая появляется в редактировании
   //      строки плана счетов на активно-пассивном счете с развернутым сальдо
   /*
   ExecuteSQL(
      "UPDATE " + tblName
      + " SET " + turnExpr("fsum1", "fsum2", scVid)
         + "," + turnExpr("fsum3", "fsum4", scVid)
         + (!is2val? "":
            "," + turnExpr("fvsum1", "fvsum2", scVid)
            + "," + turnExpr("fvsum3", "fvsum4", scVid) )
         + (!isSprav? "":
            "," + turnExpr("fssum1", "fssum2", scVid)
            + "," + turnExpr("fssum3", "fssum4", scVid) )
         + (!isPSum? "":
            "," + turnExpr("fpsum1", "fpsum2", scVid)
            + "," + turnExpr("fpsum3", "fpsum4", scVid) )
      );
     */
     //alert(turnExpr("S.fsum1", "S.fsum2", scVid))

	if (SUBD == MYSQL)
	{
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fsum1_ "+SqlType("MONEY") + ", ADD fsum2_ "+SqlType("MONEY") + ", ADD fsum3_ "+SqlType("MONEY") + ", ADD fsum4_ "+SqlType("MONEY"));
	}
	else
	{
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fsum1_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fsum2_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fsum3_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fsum4_ "+SqlType("MONEY"));
	}

	if (is2val)
	{
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fvsum1_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fvsum2_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fvsum3_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fvsum4_ "+SqlType("MONEY"));
	}

	if (isSprav)
	{
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fssum1_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fssum2_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fssum3_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fssum4_ "+SqlType("MONEY"));
	}

	if (isPSum)
	{
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fpsum1_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fpsum2_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fpsum3_ "+SqlType("MONEY"));
		ExecuteSQL("ALTER TABLE " + tblName + " ADD fpsum4_ "+SqlType("MONEY"));
	}

	var sSQL = " UPDATE " + tblName + " SET "
		+	" fsum1_ = fsum1 , fsum2_ = fsum2, fsum3_ = fsum3, fsum4_ = fsum4 "

	if (is2val)
	{
		sSQL += ", fvsum1_ = fvsum1 , fvsum2_ = fvsum2, "
			+ "fvsum3_ = fvsum3, fvsum4_ = fvsum4 ";
	}

	if (isSprav)
	{
		sSQL += ", fssum1_ = fssum1 , fssum2_ = fssum2, "
			+ "fssum3_ = fssum3, fssum4_ = fssum4 ";
	}

	if (isPSum)
	{
		sSQL += ", fpsum1_ = fpsum1 , fpsum2_ = fpsum2, "
			+ "fpsum3_ = fpsum3, fpsum4_ = fpsum4 ";
	}

	ExecuteSQL(sSQL);


	strSQL = "UPDATE "
		+ tblName + "  "
	//	+ ", ^CL_SCHET AS CL "
	+ " SET "
		+ turnExpr("fsum1", "fsum2", scVid)
		+ "," + turnExpr("fsum3", "fsum4", scVid)
		+ (!is2val? "":
			"," + turnExpr("fvsum1", "fvsum2", scVid)
			+ "," + turnExpr("S.fvsum3", "S.fvsum4", scVid) )
		+ (!isSprav? "":
			"," + turnExpr("fssum1", "fssum2", scVid)
			+ "," + turnExpr("fssum3", "fssum4", scVid) )
		+ (!isPSum? "":
			"," + turnExpr("fpsum1", "fpsum2", scVid)
			+ "," + turnExpr("fpsum3", "fpsum4", scVid) )
	//+ " WHERE "
	//	+" S.FSC=CL.FSCHET"
	//	+" AND S.FSU=CL.FSUBSCHET"
	//	+" AND S.FST=CL.FSTAT"
	ExecuteSQL(strSQL);
	
	ExecuteSQL(
		"DELETE "
		+ " FROM " + tblName
		+ " WHERE " + sqlRound("fsum1", prmDecSum) + "=0"
			+ " AND " + sqlRound("fsum2", prmDecSum) + "=0"
			+ " AND " + sqlRound("fsum3", prmDecSum) + "=0"
			+ " AND " + sqlRound("fsum4", prmDecSum) + "=0"
			+ " AND " + sqlRound("fsum5", prmDecSum) + "=0"
			+ " AND " + sqlRound("fsum6", prmDecSum) + "=0"
//============================================
			+ " AND " + sqlRound("fsumobr", prmDecSum) + "=0"
//============================================
			+ " AND fistotal = "+SqlFalse
			);


	if (/*SUBD != DERBY && */ SUBD != DB2)
	{
		if (SUBD == MYSQL)
		{
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fsum1_ , DROP COLUMN fsum2_ , DROP COLUMN fsum3_ , DROP COLUMN fsum4_ ");
		}
		else
		{
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fsum1_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fsum2_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fsum3_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fsum4_ ");
		}

		if (is2val)
		{
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fvsum1_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fvsum2_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fvsum3_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fvsum4_ ");
		}

		if (isSprav)
		{
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fssum1_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fssum2_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fssum3_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fssum4_ ");
		}

		if (isPSum)
		{
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fpsum1_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fpsum2_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fpsum3_ ");
			ExecuteSQL("ALTER TABLE " + tblName + " DROP COLUMN fpsum4_ ");
		}
	}
	else
	{
		DropTable(tblName + "_TEMP");
		ExecuteSQL("RENAME TABLE " + tblName + " TO " + tblName + "_TEMP")
		var arr = getColumnsNameArray(tblName + "_TEMP");
		var bNoFirst = false;
		var sFields = "";
		for (i = 0; i < arr.length ; i++)
		{
			var sFld = arr[i];
			if (sFld.endsWith("_")) continue;

			if (bNoFirst)
			{
				sFields += ",";
			}
			bNoFirst = true;
			sFields += sFld;
		}

		ExecuteSQL("SELECT " + sFields + " into " + tblName
						+ " from " + tblName + "_TEMP");

		DropTable(tblName + "_TEMP");
	}
	//alert(new Date().getTime() - t)
}

/*
   Возвращает выражения для сворачивания сумм
   Пример:
   turnExpr("fdb_sum", "fcr_sum", "+")
      вернет "fdb_sum = fdb_sum - fcr_sum, fcr_sum = 0"
*/
function turnExpr(fdb, fcr, vid)
{
   switch(vid)
   {
   case "+":   // Активный счет - дебетовое сальдо
      return fdb + "=" + fdb + "_-" + fcr + "_," + fcr + "=0";
   case "-":   // Пассивный счет - кредитовое сальдо
      return fdb + "=0," + fcr + "=" + fcr + "_-" + fdb+"_";
   case "*":   // Активно пассивный счет - свернутое сальдо
   //vlad 26.01.2012
   //Данный блок не давал отработать тому, что ниже
   /*
   case "%":
   */
      var difDb = fdb + "_-" + fcr+"_";
      var difCr = fcr + "_-" + fdb+"_";
      return fdb + "="+IIF(difDb+" >0 ",difDb,0)+", " +
         fcr + "= "+IIF(difCr+" >0 ",difCr,0);

   //vvs 24.02.2004
   // Активно-пассивный счет с развернутым сальдо.
   // Сворачиваем на нижней аналитике и развернутое сальдо выше
   //vvs 03.02.2004
   // И обязательно сворачиваем, если на счете нет аналитики
   case "%":
      // isLastAn - Признак нижнего уровня аналитики
      // Будем сворачивать, если в настройке
      // плана счетов стоит "свернуть" на уровне "моем"
      // или более высоком
      var isLastAn =	//в плане счетов не прикреплен справочник аналитиким на 1-м уровне...
       " (SELECT fcl1 FROM ^cl_schet cl WHERE "
		+" fsc = cl.fschet"
		+" AND fsu = cl.fsubschet"
		+" AND fst = cl.fstat"
        +" ) in ('0') "
		+" AND "
		+sqlStrIsEmp(	//... а также на 1-м уровне не привязаны договора/номенклатуры и т.п.
			"(SELECT cl.fcl1_type FROM ^cl_schet cl WHERE "
			+" fsc = cl.fschet"
			+" AND fsu = cl.fsubschet"
			+" AND fst = cl.fstat"		
			+" )"
		)

      var clTURNLEV =  " (SELECT cl.fturnlev FROM ^cl_schet cl WHERE "
		+" fsc = cl.fschet"
		+" AND fsu = cl.fsubschet"
		+" AND fst = cl.fstat"
        + " )";

		
      isLastAn = isLastAn
		+" OR fcl_cod3<>0 AND "+clTURNLEV+" IN('A3', 'A2', 'A1', '')"
		+" OR fcl_cod2<>0 AND "+clTURNLEV+" IN('A2', 'A1', '')"
		+" OR fcl_cod1<>0 AND ("+clTURNLEV+" IN('A1', '') OR "+sqlStrIsNull(clTURNLEV)+")"

      // свернутая в дебетовую сторону сумма
      var difDb = fdb + "_ -" + fcr + "_";
      // свернутая в кредитовую сторону сумма
      var difCr = fcr + "_ -" + fdb + "_";

      // Свернутое сальдо по Дт
      var turnDb = IIF(difDb+" >0 ",difDb,0);/*" iif(" + difDb + ">0," + difDb + ",0)" */
      // Свернутое сальдо по Кт
      var turnCr = IIF(difCr+" >0 ",difCr,0);/*" iif(" + difCr + ">0," + difCr + ",0)"*/

      strExpr = fdb + "="+IIF(isLastAn,turnDb,fdb)+" ,"
         + fcr + "="+IIF(isLastAn,turnCr,fcr);
      return strExpr;
   //vlad 06.02.2012
   //Новый режим - сворачивать так, как указано в плане счетов
   case "":
      // isLastAn - Признак нижнего уровня аналитики
      // Будем сворачивать, если в настройке
      // плана счетов стоит "свернуть" на уровне "моем"
      // или более высоком
      var isLastAn =	//в плане счетов не прикреплен справочник аналитиким на 1-м уровне...
       " (SELECT fcl1 FROM ^cl_schet cl WHERE "
		+" fsc = cl.fschet"
		+" AND fsu = cl.fsubschet"
		+" AND fst = cl.fstat"
        +" ) in ('0') "
		+" AND "
		+sqlStrIsEmp(	//... а также на 1-м уровне не привязаны договора/номенклатуры и т.п.
			"(SELECT cl.fcl1_type FROM ^cl_schet cl WHERE "
			+" fsc = cl.fschet"
			+" AND fsu = cl.fsubschet"
			+" AND fst = cl.fstat"		
			+" )"
		)

      var clTURNLEV =  " (SELECT cl.fturnlev FROM ^cl_schet cl WHERE "
		+" fsc = cl.fschet"
		+" AND fsu = cl.fsubschet"
		+" AND fst = cl.fstat"
        + " )";

      isLastAn = isLastAn
		+" OR fcl_cod3<>0 AND "+clTURNLEV+" IN('A3', 'A2', 'A1', '')"
		+" OR fcl_cod2<>0 AND "+clTURNLEV+" IN('A2', 'A1', '')"
		+" OR fcl_cod1<>0 AND ("+clTURNLEV+" IN('A1', '') OR "+sqlStrIsNull(clTURNLEV)+")"

      // свернутая в дебетовую сторону сумма
      var difDb = fdb + "_ -" + fcr + "_";
      // свернутая в кредитовую сторону сумма
      var difCr = fcr + "_ -" + fdb + "_";

      // Свернутое сальдо по Дт
      var turnDb = IIF(difDb+" >0 ",difDb,0);
      // Свернутое сальдо по Кт
      var turnCr = IIF(difCr+" >0 ",difCr,0);

      var svid = 
       " (SELECT fvid FROM ^cl_schet cl WHERE "
		+" fsc = cl.fschet"
		+" AND fsu = cl.fsubschet"
		+" AND fst = cl.fstat"
        + " ) "
	
      // Выражение для остатков по Дт
      sDbExpr = sqlSwitchEx(svid,
		// Активный счет
		// Дебетовое сальдо
		"+", difDb,

		// Пассивный счет
		// Кредитовое сальдо
		"-", "0", 

		// Активно-пассивный счет
		// со свернутым сальдо
		"*", IIF(difDb + ">0", difDb, "0"),

		// Активно-пассивный счет с развернутым сальдо.
		// Сворачиваем на нижней аналитике и развернутое сальдо выше
		"%", IIF(isLastAn, turnDb, fdb),

		// Неопределенный вид счета - ничего не делаем
		//fdb
		"0"
      )

      // Выражение для остатков по Кт
      sCrExpr = sqlSwitchEx(svid,
		// Активный счет
		// Дебетовое сальдо
		"+", "0",

		// Пассивный счет
		// Кредитовое сальдо
		"-", difCr,

		// Активно-пассивный счет
		// со свернутым сальдо
		"*", IIF(difCr + ">0", difCr, "0"),

		// Активно-пассивный счет с развернутым сальдо.
		// Сворачиваем на нижней аналитике и развернутое сальдо выше
		"%", IIF(isLastAn, turnCr, fcr),

		// Неопределенный вид счета - ничего не делаем
		//fcr
		"0"
      )
      strExpr = fdb + " = ("+sDbExpr+"), " + fcr + " = (" + sCrExpr+")";
      return strExpr;
   default:
      WAIT("Неопределенный вид счета: '" + typeOf(vid) + "'");
      return fdb + "=" + fdb + "," + fcr + "=" + fcr;
   }
}