/**
    DPGRIDEDIT (C) ДЕБЕТ Плюс, 18.04.2006
    
	autor draco (Степанчук Д.В.)	

	Объект для редактирования грида
	Теперь пользователь в файле, где редактируется грид, должен выполнить следующие действия:
		- заинклудить данный скрипт;
		- создать экземпляр объекта (параметр - грид)
		- при необходимости проинициализировать поля создаваемой записи нужными типами данных
			функцией init().
			Пример применения : ObjName.init = userFunction;
			В функции вызвать метод объекта initValue(field, value) для каждого 
			инициализируемого поля.
			Пример применения : ObjName.initValue("fieldName", 0) либо this.initValue("fieldName", "debet");
			
		- при необходимости переопределить возможность редактирования в зависимости
			от режима ("ADD", "EDIT", ...) функцией canEdit(mode).
			Функция должна возвращать значения "true" или "false".
			Пример применения : ObjName.canEdit = userFunction;
			
		- при необходимости установить обработчики на вход в редактирование поля и на выход из него
			функцией addValidField(field, onEnter, onExit):
				- field - имя поля;
				- onEnter - обработчик входа;
				- onExit - обработчик выхода, должнен возвращать значения "true" или "false".				
			Если в каком-то из обработчиков нет необходимости - передать null (частный случай - 
			addValidField(field, onEnter)).
			Функция возвращает "true" в случае успешного выполнения.
			Пример применения : ObjName.addValidField(fieldName, onEnterFunction, onExitFunction);
			
		- при необходимости переопределить сохранение записи функцией saveRecord().
			Функция должна возвращать значения "true" или "false".
			Пример применения : ObjName.saveRecord = userFunction;
			
		- по умолчанию при сохранении порядок проверки корректности введенных данных (пользовательский
			метод выхода из редактирования ячейки) выполняется по расположению полей в гриде. Если
			необходим иной порядок нужно воспользоватся методом объекта  setCallOrder(boolean), 
			передать параметр "true" и вызвать методы ObjName.addValidField в необходимом порядке
			либо установить порядок вызова методом setOrderArray(Array) передав массив имен полей в
			необходимом	порядке.
			Пример применения : 
					ObjName.setCallOrder(true) - устанавливает пользовательскую очередь вызова;
					ObjName.setCallOrder(false) - стандартная очередь вызова;
					ObjName.setOrderArray(new Array("aaa", ...)).			
			
	Чтобы отобразить данные в нередактируемых полях необходимо воспользоваться функцией объекта 
		setValue(field, value, onTable).
			- fieldName - имя поля;
			- value - значение;
			- onTable - если равен true значение будет просто занесено в таблицу.
		Возможны такие варианты использования:
			- УСТАНОВИТЬ значение поля таблицы:
					1) для полей, на которых висит тай, но они не визуализируются;
					2) для непритаеных полей, которые не редактируются.
				Параметр value зависит от типа данных в поле таблицы.
				Используются все три параметра, а параметр onTable обязан быть равен true. 
			- ПЕРЕРИСОВАТЬ /и УСТАНОВИТЬ содержимое поля грида:
					1) для притаеных полей.
					2) для непритаеных полей, которые не редактируются.
				Параметр value должен быть только строкой.
				Может использоваться как с параметром value так и без него. При использовании параметра
				value его значение будет установлено в грид, иначе значение будет искаться в притаеной
				таблице. Параметр onTable абсолютно не нужен. 
		Пример применения : ObjName.setValue(fieldName, Value)
							либо ObjName.setValue(fieldID, Value);
			- fieldName - имя поля;
			- fieldID - номер поля.
			
	ВНИМАНИЕ!!!
		Чтобы установка курсора в гриде выполнялась правильно инициализируйте поле "FWID" только при
		старте редактирования или не инициализируйте вообще (инициализация пройдет автоматически).
	
	ВАЖНО!!!
		Допустим при заходе в редактирование поля необходимо использовать какой-либо скрипт-объект
		(например, DpStat), и установить на него несколько контролов, тогда можно воспользоваться
		объектом DpLightObject. Его скрит инклудится (см.ниже).
*/

include("Objects/DpLightObject.js");

function DpGridEdit(parGrid)
{
	// закрытые поля
	this.grid = parGrid;
	this.fields = new Object();
	this.vectorValid = new Array();
	this.orderArray = null;
	this.table = null;
	this.count = 0;
	this.isInsertFWID = null;
	this.mode = null;
	this.validCallOrder = false;
	
	// открытые методы
	this.init = null;
	this.canEdit = null;
	this.saveRecord = null;	
	this.useInitOnEdit = null;
	this.addValidField = DpGridEdit_addValidField;
	this.setValue = DpGridEdit_setValue;
	this.initValue = DpGridEdit_setTableValue;
	this.setCallOrder = DpGridEdit_setCallOrder;
	this.setOrderArray = DpGridEdit_setOrderArray;
		
	// закрытые методы
	this.GE_saveRecord = DpGridEdit_saveRecord;	
	this.refreshValidFields = DpGridEdit_refreshValidFields;
		
	this.GE_canEdit = DpGridEdit_addCanEditRecordHandler;
	this.grid.addCanEditRecordHandler(this, "GE_canEdit");
		
	this.GE_addRecord = DpGridEdit_addRecordHandler;
	this.grid.addRecordHandler(this, "GE_addRecord");
		
	this.GE_startEdit = DpGridEdit_setStartFieldEditHandler;
	this.grid.setStartFieldEditHandler(this, "GE_startEdit");
		
	this.GE_save = DpGridEdit_saveRecordHandler;
	this.grid.saveRecordHandler(this, "GE_save");
	
	this.GE_getErrorField = DpGridEdit_getErrorField;
	this.grid.setErrorFieldHandler(this, "GE_getErrorField");
	
	this.GE_getUseInitOnEdit = DpGridEdit_getUseInitOnEdit;
	this.grid.setUseInitOnEdit(this, "GE_getUseInitOnEdit");
	
}

function DpGridEdit_addCanEditRecordHandler(sMode)
{	
	this.refreshValidFields();	
	this.mode = sMode;
	if (typeof this.canEdit == "function")
		return this.canEdit(sMode);		
	return true;
}

function DpGridEdit_addRecordHandler()
{	
	this.isInsertFWID = false;
	this.table = this.grid.getTable();
	if (typeof this.init == "function")
		this.init();
	if (!this.isInsertFWID)
		this.initValue("fwid", GetUID());
}

function DpGridEdit_setTableValue(field, value)
{		
	if (field == null)
		return;
	if (field.equalsIgnoreCase("fwid"))
	{
		if (this.isInsertFWID)
			return;
		else
			this.isInsertFWID = true;
	}
	this.table.setValue(field, value);
}

function DpGridEdit_addValidField(field, onEnter, onExit)
{
	field = field.toLowerCase();
	this.fields[field] = new Object();
	if (typeof onEnter == "function")
		this.fields[field].onEnter = onEnter;
	if (typeof onExit == "function")
	{
		this.fields[field].onExit = onExit;
		this.fields[field].isOk = false;
		this.vectorValid[this.count++] = field;
	}	
	return true;
}

function DpGridEdit_setStartFieldEditHandler(sCol, oCtrl)
{
	var field = (sCol == null ? "" : sCol.toLowerCase()); // sCol.toLowerCase()
	if (typeof this.fields[field] == "object"
		&& typeof this.fields[field].onEnter == "function")
			this.fields[field].onEnter(oCtrl);
}

function DpGridEdit_saveRecordHandler(sMode, sCol, oCtrl)
{
	if (oCtrl == null && sCol == "")
		return this.GE_saveRecord(sMode);
	var tmpCtrl = null;
	if (oCtrl == null)
	{
		tmpCtrl = new DpLightObject();
		tmpCtrl.setValue(this.grid.getTable().getValue(sCol));
	}
	else
		tmpCtrl = oCtrl;		
	var field = (sCol == null ? "" : sCol.toLowerCase());  //sCol.toLowerCase()
	if (typeof this.fields[field] == "object"
		&& typeof this.fields[field].onExit == "function")
	{
			try
			{
				this.fields[field].isOk = this.fields[field].onExit(tmpCtrl);
			}
			catch(x)
			{
				if (typeof this.fields[field].onEnter == "function")
				{
					this.fields[field].onEnter(tmpCtrl);
					this.fields[field].isOk = this.fields[field].onExit(tmpCtrl);
				}
				else
				{
					alert(x.rhinoException.getLocalizedMessage(), ICON_ERROR);
					return false;					
				}
			}
			return this.fields[field].isOk;
	}
	return true;
}

function DpGridEdit_saveRecord(sMode, sCol, oCtrl)
{
	if (typeof this.saveRecord == "function")
		return this.saveRecord(sMode, sCol, oCtrl);
	return true;
}

function DpGridEdit_setValue(field, value)
{
	if (typeof field == "number")
		field = parseInt(field);
	else if (typeof field != "string")
		return false;		
	if (typeof value == "undefined")
		return false;
	return this.grid.setValue(field, value);
}

function DpGridEdit_getErrorField(sFieldName)
{
	if(!this.grid)
		return null;
	var vecValid = null;
	if (this.validCallOrder)
	{
		if (this.orderArray != null)
			vecValid = this.orderArray;
		else
			vecValid = this.vectorValid;
	}
	else
		vecValid = this.grid.getFieldsName();
	if(sFieldName)
		return this.fields[sFieldName.toLowerCase()].isOk;
	for (var i = 0; i < vecValid.length; i++)
	{
		if (typeof this.fields[vecValid[i]] != "undefined"
			&& this.fields[vecValid[i]].isOk === false)
		{
			return vecValid[i];
		}	
	}
	return null;
}

function DpGridEdit_refreshValidFields()
{
	for (var i = 0; i < this.vectorValid.length; i++)
		this.fields[this.vectorValid[i]].isOk = false;
}

function DpGridEdit_setCallOrder(bol)
{
	this.validCallOrder = (!bol ? false : true);
}

function DpGridEdit_setOrderArray(array)
{
	if (objTypeOf(array) == "Array")
		this.orderArray = array;
}

function DpGridEdit_getUseInitOnEdit()
{
	if (typeof this.useInitOnEdit == "function")
		return this.useInitOnEdit();
	return false;
}