include("office/ExcelWrappers/xlsConst.js");
include("office/ExcelWrappers/xlsCell.js");

/**
Класс "Бенд". Предназначен для хранения и печати коллекции ячеек, 
логически связанных друг с другом и занимающих прямоугольную область.
(если более точно - то несколько смежных строк)
Используется в чистом виде и является предком для специфических бендов 
типа "таблица", "Заголовок таблицы", "описатель строки таблицы" и т.п.

@param parent ссылка на владельца, который создал данный бенд. может быть либо объектом 
класса xlsSheet, либо xlsBand, либо класса-потомка xlsBand

@param nStartRow номер строки, с которой начинается тело бенда (
т.е. следующая строка после строки-маркера начала бенда вида "*=", "*+" )

@param name Необязательный параметр. "Человеческое имя" данного экземпляра объекта
Используется в this.trace()
*/
function xlsBandParent()
{
	this.nEndRow = 0;//Номер строки окончания xlsBand'a
	
	//2-мерный массив, хранящий номера "занятых" ячеек. 
	//Индексация [строка][столбец]. Хранит ссылку на объект-ячейку
	//класса xlsCell, которая покрывает собой данную "клетку".
	//Используется, чтобы "пропускать" уже занятые ячейки на этапе загрузки бенда.
	this.aCellOwners = new Array();
	
	//массив ячеек верхнего уровня.
	//требуется для построения иерархии колонок в бенде-шапке таблицы
	//заполняется в this.loadCell()
	this.aTopCells = new Array();

	//С его помощью будем вычислять выражения
	this.oEval = null;
	
	//тип бенда. Одна из констант xlsConst.BT_xxx
	this.bandType = xlsConst.BT_GENERIC;

	//"имя" пригодится для отладки
	this.name = "";
	
	//массив ячеек данного бенда (класса xlsCell)
	this.aCells = new Array();
	
	//Для бенда строк, если this.tableFormat==xlsConst.TF_DP, будет ссылкой на последнюю
	//ячейку в строке, которая может оказаться либо условием на печать описателя, если тип 
	//значения окажется boolean, либо обыкновенной ячейкой
	this.oLastRowCell = null;

	//массив высот строк в настройке данного бенда
	this.aRowHeights = new Array();

	//массив стилей строк в настройке данного бенда (null, если строка не форматировалась)
	this.aRowStyles = new Array();
	
	//Методы
	this.load = xlsBandParent_load;
	this.loadRow = xlsBandParent_loadRow;
	this.loadCell = xlsBandParent_loadCell;
	this.parse = xlsBandParent_parse;
	this.print = xlsBandParent_print;
	this.isStop = xlsBandParent_isStop;
	this.release = xlsBandParent_release;
	
	//ф-ции для работы с this.aCellOwners
	this.setCellOwner = xlsBandParent_setCellOwner;	//пометить ячейку(ки), как используемую
	this.getCellOwner = xlsBandParent_getCellOwner;	//получить хозяина ячейки (класс xlsCell)
	
	this.getLastRow = xlsBandParent_getLastRow;
	this.setLastRow = xlsBandParent_setLastRow;
	
	//Установить высоты строк, опираясь на значения в this.aRowHeights
	this.setRowHeights = xlsBandParent_setRowHeights;
	this.nStartRowPtd;
	this.nEndRowPtd;

	//this.shiftFormulas = xlsBandParent_shiftFormulas;
	
	//ф-ция для конвертации формы в формат jrxml
	this.convert2Jrxml = xlsBandParent_convert2Jrxml;
	
}

function xlsBandParent_load()
{
	this.trace("xlsBandParent_load: start");
	
	//если стоим на маркере начала бенда, то начало бенда - на 1 строку ниже
	if (this.isStop(this.nStartRow))
	{
		this.nStartRow++;
	}
	
	var nCurrentRow = this.nStartRow;

	
	for (var nCurrentRow = this.nStartRow; !this.isStop(nCurrentRow); nCurrentRow++)
	{
		this.loadRow(nCurrentRow);
	}
	this.nEndRow = nCurrentRow-1;
	this.trace("xlsBandParent_load: end  (return nCurrentRow = "+nCurrentRow+")");
	return nCurrentRow;
}

/**
Проверяет является ли строка нижней границей xlsBand
Для возможности использования в шапке шахматной таблицы "*CROSSH"
Дополнительно будем сканировать всю строку на наличие "*CROSSH"
Если найдем, то тоже вернем true
@param nRow номер строки
@return true, если строка - начало следующего бенда или конец формы(пять звезд)
*/
function xlsBandParent_isStop(nRow)
{
	this.trace("xlsBandParent_isStop(nRow = "+nRow+"): start");

	//vlad 04.10.2010
	//Валилось, если была не строка, а число
	//var firstCellValue = this.getCellValueAt(nRow, 0);

	var hssfCell = this.getHSSFCellAt(nRow, 0);
	var retVal = false;

	if (hssfCell.getCellType()==hssfCell.CELL_TYPE_STRING)
	{
		var firstCellValue = this.getCellValueAt(nRow, 0);
		retVal = (
				firstCellValue.substr(0, 2) == xlsConst.START_BAND_ROWS ||
				firstCellValue.substr(0, 2) == xlsConst.END_BAND_ROWS ||
				firstCellValue.substr(0, 2) == xlsConst.START_BAND_HEADER ||
				firstCellValue == xlsConst.START_END_TEMPLATE
		);
	}
	
	if (!retVal)
	{
		//Просканируем на предмет наличия в строке "*CROSSH"
		for (var nCol=0; nCol<=this.p_sheet.nLastColumn; nCol++)
		{
			var sCellVal = this.getCellValueAt(nRow, nCol);
			if (sCellVal=="*CROSSH")
			{
				this.p_sheet.nCrosshRow = nRow;
				retVal = true;
				break;
			}
		}
	}
	
	this.trace("xlsBandParent_isStop(nRow = "+nRow+"): end  firstCellValue=\""+firstCellValue+"\" (return "+retVal+")");
	return retVal;
}

/**
Выполняет загрузку строки
@param nRow номер строки, которую читаем
@param isRowDP Необязательный параметр. Если true, то значит читаем строку описателя строки таблицы
в формате "Дебет Плюс". У такой строки в последней ячейке - условие
печати данной строки
*/
function xlsBandParent_loadRow(nRow, isRowDP)
{
	this.trace("xlsBandParent_loadRow(nRow = "+nRow+"): start");
	if (this.p_sheet!=null)
	{
		var hssfRow = this.p_sheet.getRow(nRow)
		
		if(hssfRow != null)
		{
			//Запомним высоту строки
			this.aRowHeights[this.aRowHeights.length] = hssfRow.getHeight();
			//Запомним стиль строки
			//vlad 23.07.12
			//В общем, нет у нас теперь стилей строки, т.к. 
			//для XSSFRow почему-то нет ф-ции isFormatted().
			//Но, судя по комментариям ниже, с 19.01.11 стили строк и так не использовалис, так 
			//что не велика потеря.
			this.aRowStyles[this.aRowStyles.length] = null;
			/*
			if (hssfRow.isFormatted())
			{
				// 19.01.2011
				//!!!
				// hssfRow.getRowStyle() Throws java.lang.IndexOutOfBoundsException
				//this.aRowStyles[this.aRowStyles.length] = hssfRow.getRowStyle();
				this.aRowStyles[this.aRowStyles.length] = null;
			}
			else
			{
				this.aRowStyles[this.aRowStyles.length] = null;
			}
			*/
			
			for (var nCol = 0; nCol < this.p_sheet.nLastColumn; nCol++)
			{
				//у нас поверх данной ячейки может располагаться регион (объединенная ячейка)
				//в таком случае загрузка произведена, когда читался левый верхний угол данного региона
				//и сейчас делать ничего не надо
				if (this.getCellOwner(nRow, nCol)==null)
				{
					hssfCell = hssfRow.getCell(nCol);
					if (hssfCell == null)
					{
						continue;
					}
					else
					{
						//Загружаем ячейку и назад получим номер последней занятой ею
						//колонки - пригодится в случае объединенных ячеек, чтобы 
						//лишний раз не обращаться к уже занятым ячейкам в строке
						nCol = this.loadCell(nRow, nCol, isRowDP);
					}
				}
			}
		}
	}
	else
	{
		//Запомним высоту строки
		this.aRowHeights[this.aRowHeights.length] = -1;
		this.aRowStyles[this.aRowStyles.length] = null;
		this.trace("   xlsBandParent_loadRow(nRow = "+nRow+"): ОШИБКА!!! this.p_sheet==null");
	}
	this.trace("xlsBandParent_loadRow(nRow = "+nRow+"): end");
}

/**
Выполняет загрузку ячейки
@param nRow номер строки, которую читаем
@param nCol номер колонки, которую читаем

@param isRowDP Необязательный параметр. Если true, то значит читаем строку описателя строки таблицы
в формате "Дебет Плюс". У такой строки в последней ячейке - условие
печати данной строки
@return номер последней занятой ячейкой колонки
*/
function xlsBandParent_loadCell(nRow, nCol, isRowDP)
{
	var oCell = new xlsCell(this, nRow, nCol);
	oCell.load();

	//Как оказалось, имеется море форм, в которых в последней колонке
	//не условие, а обыкновенная колонка. А условие это или нет - 
	//определялось по типу выражения, которое получилось. если boolean, 
	//то это условие, и ячейку не надо рисовать.
	//В связи с этим на этапе загрузки загружаем последнюю колонку и как условие,
	//и как обыкновенную ячейку
	//Последнюю ячейку в бенде строк пока запомним отдельно
	//Если окажется, что она ячейка, а не условие, то мы ее добавим в this.aCells
	//Всем этим заведует xlsBandTRow_parse_DP
	if (isRowDP && oCell.colTo == (this.p_sheet.nLastColumn-1) )
	{
		//это последняя ячейка, в которой условие
		//Запоминаем условие (отбросив спереди звезду)
		this.sCond = this.getCellValueAt(nRow, nCol).trim().substr(1).trim();
		this.trace("xlsBandParent_loadCell: this.sCond = "+this.sCond);
		this.oLastRowCell = oCell;
	}
	else
	{
		this.aCells[this.aCells.length] = oCell;
	}
	
	//если это верхняя ячейка в шапке таблицы - добавим ее в список "верхних" ячеек
	if (this.bandType==xlsConst.BT_THEADER && nRow==this.nStartRow)
	{
		this.aTopCells[this.aTopCells.length] = oCell;
	}
	
	this.setCellOwner(oCell, oCell.rowFrom, oCell.colFrom, oCell.rowTo, oCell.colTo);

	return oCell.colTo;
}

/**
Выполняет разбор бенда
Для обычного бенда, вообще-то, ничего разбирать не надо - там и так все готово для печати.
Но на всякий случай дернем все ячейки (им виднее, что надо делать).
Разбор нужен для бенда-таблицы (шапки и строк) в случае, когда
имеется настройка шахматки - в этом случае часть ячеек может "размножиться",
а часть может "пропасть"
*/
function xlsBandParent_parse()
{
	this.trace("xlsBandParent_parse: start");
	
	//Если наш бенд - заголовок таблицы, то подготовим отдельный DpEval для вычисления
	//т.к. если у нас имеются "шахматные" колонки, то будем ему менять таблицу строк 
	if (this.bandType==xlsConst.BT_THEADER)
	{
		/*
		this.oEval = new DpEval();
		this.oEval.setTable(this.tCols);
		
		//vlad 17.08.2010
		//для формирования заголовков шахматных колонок должны быть доступны и переменные "заголовка"
		//Попробуем перетянуть с глобального DpEval'а переменные заголовка
		//(мы их сделаем "переменными").
		//Нам надо, чтобы имя переменной начиналось на "H"
		if (typeOf(this.p_office.vars.eval.tbl)=="object")
		{
			var globalEvalTbl = this.p_office.vars.eval.tbl;
			for (var ind in globalEvalTbl)
			{
				this.oEval.addVar("H"+ind, globalEvalTbl[ind]);
			}
		}
		*/
		//this.oEval = this.p_workBook.oEval;
		//this.oEval.setTable(this.tCols);
	}
	
	//в зависимости от того, что мы за бенд, порядок обхода изменяется
	//Для шапки таблицы обходим this.aTopCells, а они в свою очередь сами 
	//инициируют печать "детей"
	//Для простого бенда просто вызываем печать всех ячеек подряд
	var aCells;
	
	//для целей отладки 
	var sCellsName = "";
	
	if (this.bandType==xlsConst.BT_THEADER)
	{
		aCells = this.aTopCells;
		sCellsName = "this.aTopCells";
		this.trace("xlsBandParent_parse: aCells = this.aTopCells");
	}
	else
	{
		sCellsName = "this.aCells";
		aCells = this.aCells;
		this.trace("xlsBandParent_parse: aCells = this.aCells");
	}
	
	for (var nIndex in aCells)
	{
		aCells[nIndex].parse();
	}

	this.trace("xlsBandParent_parse: end");
	return;
}


/**
выполняет печать бенда
@param nPrintStartRow номер строки, начиная с которой следует начинать печать данный бенд
@return номер строки, в которой находится низ напечатанного бенда
*/
function xlsBandParent_print(nPrintStartRow)
{
	//надо установить алиас, если мы - "строки таблицы"
	if( this.bandType == xlsConst.BT_TROW && !isEmpty(this.p_parent.sRTableAlias))
	{
		this.p_office.vars.eval.activeRTableAlias = this.p_parent.sRTableAlias;
	}

	//на сколько строк надо опустить все ячейки
	var nShiftRows = nPrintStartRow - this.nStartRow;

	this.trace("xlsBandParent_print: start nPrintStartRow="+nPrintStartRow+", nShiftRows="+nShiftRows);
	
	//номер последней строки результатов печати
	var nMaxEndRow = nPrintStartRow;
	
	var nCurrEndRow;
	
	//в зависимости от того, что мы за бенд, порядок обхода изменяется
	//Для шапки таблицы обходим this.aTopCells, а они в свою очередь сами 
	//инициируют печать "детей"
	//Для простого бенда просто вызываем печать всех ячеек подряд
	var aCells;
	
	//для целей отладки 
	var sCellsName = "";
	
	if (this.bandType==xlsConst.BT_THEADER || this.bandType==xlsConst.BT_TROW && this.tableFormat==xlsConst.TF_CK)
	{
		aCells = this.aTopCells;
		sCellsName = "this.aTopCells";
		this.trace("xlsBandParent_print: aCells = this.aTopCells");
	}
	else
	{
		sCellsName = "this.aCells";
		aCells = this.aCells;
		this.trace("xlsBandParent_print: aCells = this.aCells");
	}
	
	//здесь будем хранить номер последней занятой колонки нашей ячейки
	//Актуально для "шахматных" ячеек, которые "размножились"
	//Инициируем -1, т.к. 1-ю из ячеек надо печатать с колонки 0, а мы делаем nCurrEndCol+1
	var nCurrEndCol = -1;
	
	for (var nIndex in aCells)
	{
		this.trace("   xlsBandParent_print: processing nIndex="+nIndex+" in "+sCellsName+"("+aCells[nIndex].value+")");

		var origCell = aCells[nIndex];
		
		//номер строки, в которой размещена оригинальная ячейка
		var nOrigRow = origCell.rowFrom;
		
		//создаем копию(копии - для шахматок) ячейки и проставляем в ней значение
		var oCurrCell;
		if (this.bandType == xlsConst.BT_THEADER || this.bandType == xlsConst.BT_TROW)
		{
			//Мы печатаем таблицу. Колонки могут "размножаться". Поэтому смотрим на nCurrEndCol
			origCell.rowIndex = nIndex;
			oCurrCell = origCell.print(nOrigRow + nShiftRows, nCurrEndCol+1);
		}
		else
		{
			//Мы печатаем обычный бенд. Поэтому колонка для ячейки не меняется
			oCurrCell = origCell.print(nOrigRow + nShiftRows, origCell.colFrom);
		}
		
		if (oCurrCell!=null)
		{
			
			//nCurrEndRow = oCurrCell.rowTo;
			nCurrEndRow = oCurrCell.getLastRow()
			nCurrEndCol = oCurrCell.getLastCol();
			//"Отпустим" копию
			oCurrCell.release();
			oCurrCell = null;
			this.trace("   xlsBandParent_print: nCurrEndRow="+nCurrEndRow+", nCurrEndCol="+nCurrEndCol);
		}
				
		if (nCurrEndRow > nMaxEndRow)
		{
			nMaxEndRow = nCurrEndRow;
		}
	}

	//Запомнм нижнюю строку
	this.setLastRow(nMaxEndRow);
	
	//fruckt
	this.nStartRowPtd = nPrintStartRow;
	this.nEndRowPtd = nMaxEndRow;
	
	//Установим высоту строк
	//fruckt
	//етод метод будет запускаться выше
	//this.setRowHeights(nPrintStartRow, nMaxEndRow);
	
	this.trace("xlsBandParent_print: end returned nMaxEndRow="+nMaxEndRow);
	return nMaxEndRow;
}

/**
"отпускает" все объекты, которыми владеет бенд
*/
function xlsBandParent_release()
{
	this.p_workBook = null;
	this.p_office = null;
	this.workBook = null;
	this.p_parent = null;
    this.p_sheet = null;
	this.p_band = null;
	
	this.oLastRowCell = null;
	
	if(this.aBandRegions)
	{
		for (var i = o; i < this.aBandRegions.length; i++)
			this.aBandRegions[i].release();
	}
	this.aBandRegions = null;

	for (var nRow in this.aCellOwners)
	{
		var aCells = this.aCellOwners[nRow];
		for (var nCol in aCells)
		{
			aCells[nCol] = null;
		}
		this.aCellOwners[nRow] = null;
	}
	this.aCellOwners = null;

	for (var nIndex in this.aTopCells)
	{
		this.aTopCells[nIndex] = null;
	}
	this.aTopCells = null;
	
	for (var nIndex in this.aCells)
	{
		this.aCells[nIndex].release();
		this.aCells[nIndex] = null;
	}
	this.aCells = null;
	
}

/**
Пометить ячейку(ки), как используемую
Если заданы nRowTo и nColTo, то помечает прямоугольную область
@param oCell ссылка на объект-владелец
@param nRowFrom номер строки
@param nColFrom номер столбца

@param nRowTo необязательный, номер строки (до куда)
@param nColTo необязательный, номер столбца (до куда)
*/
function xlsBandParent_setCellOwner(oCell, nRowFrom, nColFrom, nRowTo, nColTo)
{
	this.trace("xlsBandParent_setCellOwner(oCell, nRowFrom="+nRowFrom+", nColFrom="+nColFrom+", nRowTo="+nRowTo+", nColTo="+nColTo+")");
	if (typeOf(nRowTo)=="undefined")
	{
		nRowTo = nRowFrom;
	}
	if (typeOf(nColTo)=="undefined")
	{
		nColTo = nColFrom;
	}
	for (var nRow = nRowFrom; nRow<=nRowTo; nRow++)
	{
		if (typeof(this.aCellOwners[nRow])=="undefined")
		{
			this.aCellOwners[nRow] = new Array();
		}
		var aCols = this.aCellOwners[nRow];
		for (var nCol = nColFrom; nCol<=nColTo; nCol++)
		{
			this.aCellOwners[nRow][nCol] = oCell;
			this.trace("   xlsBandParent_setCellOwner: marked (nRow="+nRow+", nCol="+nCol+")");
		}
	}
	this.trace("xlsBandParent_setCellOwner end");
}

/**
получить хозяина ячейки (класс xlsCell)
@param nRow номер строки
@param nCol номер столбца
@return возвращает ссылку на объект, либо null, если данная ячейка никем не занята
*/
function xlsBandParent_getCellOwner(nRow, nCol)
{
	var retVal = null;
	if (typeof(this.aCellOwners[nRow])!="undefined" && typeof(this.aCellOwners[nRow][nCol])!="undefined")
	{
		retVal = this.aCellOwners[nRow][nCol];
	}
	this.trace("xlsBandParent_getCellOwner: returned "+retVal+" nRow="+nRow+", nCol="+nCol);
}

/**
возвращает номер последней строки, которую покрывает
результат печати данного бенда
*/
function xlsBandParent_getLastRow()
{
	return this.lastRow;
}

/**
установить номер последней строки, которую покрывает
результат печати данного бенда
*/
function xlsBandParent_setLastRow(nRow)
{
	this.lastRow = nRow;
}

/**
Установить высоты строк, опираясь на значения в this.aRowHeights
*/
function xlsBandParent_setRowHeights()
{
	this.trace("xlsBandParent_setRowHeights(nStartRow="+this.nStartRowPtd+", nEndRow="+this.nEndRowPtd+")");

	var nIndex = 0;
	for (var nRow = this.nStartRowPtd; nRow <= this.nEndRowPtd; nRow++)
	{
		var hssfRow = this.p_sheet.getRow(nRow)
		
		if(hssfRow != null)
		{
			//Установим высоту строки
			hssfRow.setHeight(this.aRowHeights[nIndex]);
			this.trace("   hssfRow.setHeight(this.aRowHeights[nIndex="+nIndex+"]= "+this.aRowHeights[nIndex]+" )");
			
			if (this.aRowStyles[nIndex]!=null)
			{
				this.trace("   hssfRow.setRowStyle()");
				hssfRow.setRowStyle(this.aRowStyles[nIndex]);
			}
			
			//fruckt
			if (this.p_sheet.oSheet.length > 1)
			{
				for (var nSht = 1; nSht < this.p_sheet.oSheet.length; nSht++)
				{
					var rowNextList = this.p_sheet.oSheet[nSht].getRow(nRow);
					if (rowNextList == null)
					{
						rowNextList = this.p_sheet.oSheet[nSht].createRow(nRow);
					}
					
					rowNextList.setHeight(this.aRowHeights[nIndex]);
					if (this.aRowStyles[nIndex] != null)
					{
						rowNextList.setRowStyle(this.aRowStyles[nIndex]);
					}
				}
			}
		}
		
		nIndex++;
		
		nIndex = nIndex >= this.aRowHeights.length ? 0 : nIndex;
	}
}


/**
Подвинуть ссылки в формулах
*/
//Так было бы правильно, но из-за того, что в org.apache.poi.hssf.usermodel.*
//все ссылки на InternalSheet объявлены, как private, добраться до него мы не можем, и 
//приходится самим парсить выражения в xlsCell::draw с помощью xlsCell::shiftFormula
/*
function xlsBandParent_shiftFormulas(firstMovedRowIndex, lastMovedRowIndex, numberOfRowsToMove)
{
	var externSheetIndex = this.p_workBook.workBook.getExternalSheetIndex(this.p_sheet.nSheetNumber);

	var clsFormulaShifter = getJavaClass("org.apache.poi.hssf.record.formula.FormulaShifter");

	var arr = new Array();
	arr[0] = java.lang.Integer.TYPE;	//это будет int
	arr[1] = java.lang.Integer.TYPE;
	arr[2] = java.lang.Integer.TYPE;
	arr[3] = java.lang.Integer.TYPE;

	var oShifter = clsFormulaShifter.getMethod("createForRowShift", arr).invoke(
		null
		, java.lang.Integer(externSheetIndex)
		, java.lang.Integer(firstMovedRowIndex)
		, java.lang.Integer(lastMovedRowIndex)
		, java.lang.Integer(numberOfRowsToMove)
	);
	
	//var oHelper = getJavaClass("org.apache.poi.hssf.usermodel.HSSFSheetHelper").newInstance(this.p_sheet.oSheet.Sheet);
	var InternalSheet = this.p_sheet.oSheet.Sheet;

	InternalSheet.updateFormulasAfterCellShift(oShifter, externSheetIndex);
}
*/


/**
ф-ция для конвертации формы в формат jrxml
@param oPar имет поля:
	.builder	[in]
	.designe	[in]
	.xscale		[in]	Коэффициент для перевода ширин: [ширина jrxml]=[ширина xls]*xscale
	.yscale		[in]	Коэффициент для перевода высот: [высота jrxml]=[высота xls]*yscale
	.fscale		[in]	Коэффициент для перевода размеров шрифтов: [jrxml]=[xls]*fscale
	.lastHeight	[out]	высота бенда для .setHeight()
@return текст для addElementGroup()
*/
function xlsBandParent_convert2Jrxml(oPar)
{
	var band = oPar.builder.createElementGroup();
	var nHeight = 0;
	
	oPar.lastHeight = nHeight;
	
	return band;
}