include("office/ExcelWrappers/xlsBandParent.js");

function xlsBandTRow(parent, nStartRow)
{
	derive(this, xlsParent);
	derive(this, xlsBandParent);
	this.storeParent(parent);
	this.trace("xlsBandTRow(nStartRow = "+nStartRow+"): start");

	//тип бенда
	this.bandType = xlsConst.BT_TROW;

	this.name = objTypeOf(this)+" at "+nStartRow;

	this.p_band = this;
	this.nStartRow = nStartRow;
	this.sCond = "";	//условие печати данного описателя строки
	
	//Формат описания таблиц. По умолчанию - "Черкасский"
	this.tableFormat = xlsConst.TF_CK;
	
	this.setTableFormat = xlsBandTRow_setTableFormat;
	this.release = xlsBandTRow_release;
	
	this.load = null;	//определим в this.setTableFormat()
	this.aValues = new Array();	//предыдущие значения ячеек
	this.aMerges = new Array();	//для режима merge предыдущие значения условия
	this.trace("xlsBandTRow(nStartRow = "+nStartRow+"): end");
}

function xlsBandTRow_release()
{
	this.p_workBook = null;
	this.p_office = null;
	this.workBook = null;
	this.p_parent = null;
    this.p_sheet = null;
	this.p_band = null;
	
}
	
/**	
Устанавливает формат описания таблиц.
Переопределяет this.load

@param	tableFormat одна из констант xlsConst.TF_CK, xlsConst.TF_DP
*/
function xlsBandTRow_setTableFormat(tableFormat)
{
	this.tableFormat = tableFormat;
	switch(tableFormat)
	{
		case xlsConst.TF_CK:
			this.load = xlsBandTRow_load_CK;
			this.parse = xlsBandTRow_parse_CK;
			break;
		case xlsConst.TF_DP:
			this.load = xlsBandTRow_load_DP;
			this.parse = xlsBandTRow_parse_DP;
			break;
		default:
			this.trace("xlsBandTRow_setTableFormat(tableFormat="+tableFormat+") ОШИБКА!!! Неизвестній формат");
	}
}

/**	
Загрузка строк для "Черкасского" формата
*/
function xlsBandTRow_load_CK()
{
	this.trace("xlsBandTRow_load_CK: start");
	
	//Должны стоять маркере начала бенда, снимем с него условие
	var str = this.getCellValueAt(this.nStartRow, 0)
	if (str.substr(0, 2)==xlsConst.START_BAND_ROWS)
	{
		//Запоминаем условие
		this.sCond = str.substr(2).trim();
		this.trace("xlsBandTRow_load_CK: this.sCond = "+this.sCond);
	}
	else
	{
		this.trace("xlsBandTRow_load_CK: ОШИБКА!!! Ожидалось \""+xlsConst.START_BAND_ROWS+"\" в строке this.nStartRow="+this.nStartRow+", str=\""+str+"\"");
	}
	
	//начало бенда - на 1 строку ниже
	this.nStartRow++;
	
	for (var nCurrentRow = this.nStartRow; !this.isStop(nCurrentRow); nCurrentRow++)
	{
		this.loadRow(nCurrentRow);
	}
	this.nEndRow = nCurrentRow-1;
	this.trace("xlsBandTRow_load_CK: end  (return nCurrentRow = "+nCurrentRow+")");
	return nCurrentRow;
}

/**	
Загрузка строк для формата "Дебет Плюс"
Грузится ровно одна строка, у которой в последней ячейке - условие печати
*/
function xlsBandTRow_load_DP()
{
	this.trace("xlsBandTRow_load_DP: start");
	
	//если стоим на маркере начала бенда, то начало бенда - на 1 строку ниже
	if (this.isStop(this.nStartRow))
	{
		this.nStartRow++;
	}
	
	var nCurrentRow = this.nStartRow;

	//грузим ровно одну строку, в которой в последней ячейке - условие печати данной строки
	this.loadRow(nCurrentRow, true);
	
	this.nEndRow = nCurrentRow;

	nCurrentRow++
	
	this.trace("xlsBandTRow_load_DP: end  (return nCurrentRow = "+nCurrentRow+")");
	
	return nCurrentRow;
}

/**
Разбор бенда "строка таблицы"
Логика работы такая: мы из массива this.aCells делаем массив
this.aTopCells по пути "размножая" шахматные ячейки
Все это делается на основе информации в бенде-шапке
*/
function xlsBandTRow_parse_CK(oHCell)
{
	this.trace("xlsBandTRow_parse_CK: start");

	//надо установить алиас 
	this.p_office.vars.eval.activeRTableAlias = this.p_parent.sRTableAlias;

	//Получим ссылку на заголовок т-цы
	var bandHeader = this.p_parent.bandHeader;

	//список ячеек шапки, по которым осуществляем "спуск"
	var aCells = null;

	if (oHCell==undefined)
	{
		//это первый вызов ф-ции. В кач-ве списка ячеек берем у шапки aTopCells
		aCells = bandHeader.aTopCells;
		this.trace("   xlsBandTRow_parse_CK: aCells = bandHeader.aTopCells length="+bandHeader.aTopCells.length);
	}
	else
	{
		if (oHCell.isCross && (oHCell.tCols==null || oHCell.tCols.isEmpty()))
		{
			//showObject(oHCell);
			this.trace("xlsBandTRow_parse_CK END oHCell.isCross && oHCell.tCols.isEmpty()");
			return null;
		}
	
		//Это - список ячеек, на которые размножилась наша ячейка
		aCells = oHCell.aParsedCells;
		this.trace("   xlsBandTRow_parse_CK: aCells = oHCell.aParsedCells length="+oHCell.aParsedCells.length);
	}
	
	if (aCells!=null && aCells.length>0)
	{
		this.trace("   xlsBandTRow_parse_CK: aCells!=null && aCells.length("+aCells.length+")>0");
		//список
		for (var nIndex in aCells)
		{
			if (aCells[nIndex].aChildCells !=null && aCells[nIndex].aChildCells.length>0)
			{
				for (var nChild in aCells[nIndex].aChildCells)
				{
					this.trace("   xlsBandTRow_parse_CK: this.parse(aCells[nIndex="+nIndex+"].aChildCells[nChild="+nChild+"])");
					this.parse(aCells[nIndex].aChildCells[nChild]);
				}
			}
			else
			{
				//рекурсивный вызов ф-ции
				this.trace("   xlsBandTRow_parse_CK: this.parse(aCells[nIndex="+nIndex+"])");
				this.parse(aCells[nIndex]);
			}
		}
	}
	else
	{
		this.trace("   xlsBandTRow_parse_CK: NOT aCells!=null ("+(aCells!=null)+") && aCells.length("+aCells.length+")>0");
		//мы находимся на нижнем уровне в шапке
		//Обойдем весь столбик ячеек в строках, которые находятся в одной колонке с 
		//найденной ячейкой шапки
		this.trace("xlsBandTRow_parse_CK for nRow = this.nStartRow("+this.nStartRow+") to this.nEndRow("+ this.nEndRow+")")
		for (var nRow = this.nStartRow; nRow <= this.nEndRow; nRow++)
		{
			this.trace("   xlsBandTRow_parse_CK objTypeOf(this.aCellOwners[nRow="+nRow+"])="+objTypeOf(this.aCellOwners[nRow]))
			this.trace("   xlsBandTRow_parse_CK: oRCell = this.aCellOwners[nRow="+nRow+"][oHCell.colFrom="+oHCell.colFrom+"]");

			//Ссылка на ячейку в строках, которая находится под заголовочной ячейкой oHCell
			var oRCell = this.aCellOwners[nRow][oHCell.colFrom];
			
			//Обработаем случай, когда у нас в строках одна большая объединенная ячейка
			//под несколькими колонками в шапке (такое бывает в итоговых строках).
			//Такую ячейку привяжем к ячейке в шапке, у которой с ней общая колонка начала
			if (oRCell.colFrom >= oHCell.colFrom)
			{
				//делаем ее копию
				var oNewCell = oRCell.clone();
				
				//навесим ссылку на объект со значениями
				oNewCell.aValues = oHCell.aValues;
				
				//Запоминаем ячейку
				oRCell.lastCloneIndex = this.aTopCells.length;
				this.aTopCells[oRCell.lastCloneIndex] = oNewCell;
			}
			else
			{
				this.aTopCells[oRCell.lastCloneIndex].aHCell[this.aTopCells[oRCell.lastCloneIndex].aHCell.length] = oHCell;
			}
		}
	}
	this.trace("xlsBandTRow_parse_CK: end");
}	


/**
Разбор бенда "строка таблицы" для формата "Дебет Плюс"
Здесь мы должны стоя на 1-й записи таблицы строк определить что находится в последней
ячейке - условие или обыкновенная ячейка
*/
function xlsBandTRow_parse_DP()
{
	this.trace("xlsBandTRow_parse_DP: start");

	//станем на 1-ю строку таблицы строк и определимся: в последней колонке условие или обыкновенная ячейка
	var rTbl = this.p_office.vars.getRTbl(this.p_parent.sRTableAlias);

	//надо установить алиас 
	this.p_office.vars.eval.activeRTableAlias = this.p_parent.sRTableAlias;
	
	//теперь определим - в последней колонке условие, обычная ячейка или просто ничего нет
	if (this.oLastRowCell != null && this.tableFormat == xlsConst.TF_DP && !rTbl.IsEmpty())
	{
		rTbl.MoveFirst();
		var bToPrint = this.p_office.setField(this.sCond);
		if (typeof(bToPrint)=="boolean")
		{
			//в последней колонке - условие. Поэтому "убьем" последнюю колонку
			this.trace("  xlsBandTRow_parse_DP: в последней колонке - условие. Поэтому \"убьем\" последнюю колонку");
			this.oLastRowCell.release();
			this.oLastRowCell = null;
		}
		else
		{
			//в последней колонке - обыкновенная ячейка. Поэтому "убьем" условие
			this.trace("  xlsBandTRow_parse_DP: в последней колонке - обыкновенная ячейка. Поэтому \"убьем\" условие");
			this.sCond = "";
			this.aCells[this.aCells.length] = this.oLastRowCell;
		}
	}

	var aCells = this.aCells;
	for (var nIndex in aCells)
	{
		aCells[nIndex].parse();
	}
	
	this.trace("xlsBandTRow_parse_DP: end");
}