include("office/ExcelWrappers/xlsBand.js");
include("office/ExcelWrappers/xlsBandTRow.js");
/**
Класс для работы с бендом "таблица"
В общем случае владеет бендом "Шапка таблицы" (класс xlsBandTHeader)
и коллекцией бендов "Описатель строки таблицы" класса xlsBandTRow
Для возможности работы с шахматками использует класс ColumnList
*/
function xlsBandTable(parent, nStartRow)
{
	derive(this, xlsParent);
	this.storeParent(parent);

	this.trace("xlsBandTable(): start");

	this.name = xlsConst.BANDTABLE+" at "+nStartRow;

	//!!! надо бы разобраться - нужны ли эти координаты:
	this.nStartRow = nStartRow;//Номер строки начала xlsBand'a
	this.nEndRow = 0;//Номер строки окончания xlsBand'a (в месте, где настройка формы)
	this.lastRow = -1;//Номер последней строки результата печати бенда. Проставляется в шапке т-цы. Нужен при печати строк

	//алиас таблицы строк.
	//Для возможности печати в случае, когда имеется несколько таблиц строк
	//Для "Черкасского" формата пишем 
	//*=ALIAS_NAME
	//Для "Киевского" формата пишем 
	//*+ALIAS_NAME
	this.sRTableAlias = "R";
	
	this.bandHeader = null;
	this.aBandRows = new Array();	//массив бендов описателей строк

	//Формат описания таблиц. По умолчанию - "Черкасский"
	this.tableFormat = xlsConst.TF_CK;

	//тип бенда
	this.bandType = xlsConst.BT_TABLE;

	//Методы
	this.load = xlsBandTable_load;
	this.parse = xlsBandTable_parse;
	this.print = xlsBandTable_print;
	this.release = xlsBandTable_release;
	this.setRowHeights = xlsBandTable_setRowHeights;
	this.trace("xlsBandTable(): end");
}

/**
Читаем бенд
*/
function xlsBandTable_load()
{
	this.trace("xlsBandTable_load: start");
	
	// В зависимости от того, с какого маркера начинается таблица,
	//определим формат описания таблицы
	var firstCellValue = this.getCellValueAt(this.nStartRow, 0);

	//Добавим обработку алиаса таблицы строк
	//Для "Черкасского" формата пишем 
	//*=ALIAS_NAME
	//Для "Киевского" формата пишем 
	//*+ALIAS_NAME
	var sRTableAlias = firstCellValue.substr(2).trim();
	if(!isEmpty(sRTableAlias))
	{
		this.sRTableAlias = sRTableAlias;
	}

	var nCurrentRow = this.nStartRow;
	switch (firstCellValue.substr(0, 2))
	{
		case xlsConst.START_BAND_HEADER:
			//"Черкасский" формат описателей строк  - с условиями после "*+" 
			//над строкой в 1-й колонке и с маркером шапки таблицы "*="
			this.bandHeader = new xlsBand(this, nCurrentRow);
			this.bandHeader.bandType = xlsConst.BT_THEADER;
			nCurrentRow = this.bandHeader.load();
			this.tableFormat = xlsConst.TF_CK;
			break;
		case xlsConst.START_BAND_ROWS:
			// если мы сюда зашли, значит бенда-шапки не было, а сразу начались строки.
			// установим признак, что условия находятся в последней колонке перед *****.
			this.bandHeader = null;
			this.tableFormat = xlsConst.TF_DP;
			break;
	}

	
	//передвинем указатель начала бенда на строку, следующую за маркером
	this.nStartRow++;
	
	//пробуем грузить описатели строк
	
	while (this.getCellValueAt(nCurrentRow, 0).substr(0, 2)!=xlsConst.END_BAND_ROWS)
	{

		var oBand = new xlsBandTRow(this, nCurrentRow);
		
		oBand.setTableFormat(this.tableFormat);
		
		nCurrentRow = oBand.load();
		this.aBandRows[this.aBandRows.length] = oBand;
	}
	this.nEndRow = nCurrentRow-1;
	this.trace("xlsBandTable_load: end  (return nCurrentRow="+nCurrentRow+")");
	return nCurrentRow;
}

function xlsBandTable_parse()
{
	this.trace("xlsBandTable_parse: start");
	
	//обработаем шапку таблицы
	if (this.bandHeader != null)
	{
		this.bandHeader.parse();
	}
	
	//обработаем все описатели строк
	for (var nIndex in this.aBandRows)
	{
		this.aBandRows[nIndex].parse();
	}
	
	this.trace("xlsBandTable_parse: end");
}

function xlsBandTable_print(nPrintStartRow)
{
	this.trace("xlsBandTable_print: start nPrintStartRow="+nPrintStartRow);
	if (this.bandHeader != null)
	{
		nPrintStartRow = this.bandHeader.print(nPrintStartRow)+1;
	}
	
	this.printFirstTableRow = nPrintStartRow;
	/*
	/////////// Для отладки
	for (var ind in this.bandHeader.aTopCells)
	{
		this.trace("this.bandHeader.aTopCells #"+ind);
		trace(this.bandHeader.aTopCells[ind].toString());
	}
	///////
	*/
	
	//Оббегаем все строки таблицы
	var rTbl = this.p_office.vars.getRTbl(this.sRTableAlias);

	var nRecords = rTbl.recordCount;
	var recCnt = nRecords;

	var progress = null;	
	if (nRecords > 10)
	{
		progress = createProgress(recCnt, ru("Печать (лист "+(this.p_sheet.nSheetNumber+1)+")...", "Друк (лист "+(this.p_sheet.nSheetNumber+1)+")..."));
	}
	var ctr = 1;
	
	//Кол-во реально напечатанных строк. Нужно для учета PAGE BREAK
	var nPrintedRows = 0;

	this.trace("   xlsBandTable_print: Печатаем строки this.p_sheet.nPageBreak="+this.p_sheet.nPageBreak);
	for (rTbl.MoveFirst(); !rTbl.IsEOF(); rTbl.MoveNext())
	{
		for (var nIndex in this.aBandRows)
		{
			var oBandRow = this.aBandRows[nIndex];
			var sCond = oBandRow.sCond;
			
			//Проверим, выполняется ли условие для печати данного бенда строк
			var bToPrint = this.p_office.setField(sCond)
			this.trace("   xlsBandTable_print: bToPrint="+bToPrint+", sCond=\""+sCond+"\"");
			//this.trace("   xlsBandTable_print: rTbl.getValue('FTT')=\""+rTbl.getValue('FTT')+"\", this.p_office.setField('RFTT')=\""+this.p_office.setField('RFTT')+"\"");
			//this.trace("   xlsBandTable_print: rTbl.getValue('FNMKLTXT')="+rTbl.getValue('FNMKLTXT')+", this.p_office.setField('RFNMKLTXT')="+this.p_office.setField('RFNMKLTXT'));
			
			if (sCond=="" || this.p_office.setField(sCond))
			{
				nPrintStartRow = this.aBandRows[nIndex].print(nPrintStartRow)+1;
				nPrintedRows++;
			}
		}

		if (progress && ctr%10 == 0)
		{
			progress.getProgressMonitor().setTaskName(UR("Виконано |Выполнено ") + round(ctr / recCnt * 100, 0) + "%");
		}
		ctr++;
		if (progress)
		{
			progress.getProgressMonitor().worked(1);
		}
		
		//Обработаем PAGE BREAK
		this.trace("   xlsBandTable_print: this.p_sheet.pageBreakType="+this.p_sheet.pageBreakType);
		switch(this.p_sheet.pageBreakType)
		{
			case xlsConst.PBT_NUM:
				if (this.p_sheet.nPageBreak>0 && nPrintedRows%this.p_sheet.nPageBreak==0)
				{
					this.trace("   xlsBandTable_print: 1 setRowBreak("+(nPrintStartRow-1)+")");
					//fruckt
					//this.p_sheet.oSheet.setRowBreak(nPrintStartRow-1);
					this.p_sheet.oSheet[0].setRowBreak(nPrintStartRow-1);
				}
				break;
			case xlsConst.PBT_COND:
				this.trace("   xlsBandTable_print: this.p_sheet.pageBreakType="+this.p_sheet.pageBreakType);
				if (this.p_office.setField(this.p_sheet.sPageBreakCond))
				{
					this.trace("   xlsBandTable_print: 2 setRowBreak("+(nPrintStartRow-1)+")");
					//fruckt
					//this.p_sheet.oSheet.setRowBreak(nPrintStartRow-1);
					this.p_sheet.oSheet[0].setRowBreak(nPrintStartRow-1);
				}
				break;
		}
	}

	if (progress)
	{
		progress.getProgressMonitor().done();
		
		if (progress.close)
		{
			progress.close();
		}
	}
	
	//Отнимем единицу, т.к. ее опять добавит xlsSheet_print
	nPrintStartRow = nPrintStartRow-1;
	
	this.trace("xlsBandTable_print: end return nPrintStartRow="+nPrintStartRow);
	return nPrintStartRow;
}

function xlsBandTable_setRowHeights()
{
	if (this.bandHeader != null)
		this.bandHeader.setRowHeights();
	
	for (var nIndex in this.aBandRows)
	{
		this.aBandRows[nIndex].setRowHeights();
	}
}

function xlsBandTable_release()
{
	this.p_workBook = null;
	this.p_office = null;
	this.workBook = null;
	this.p_parent = null;
    this.p_sheet = null;
	this.p_band = null;
	
	if (this.bandHeader)
	{
		this.bandHeader.release();
	}
	this.bandHeader = null;
	
	for (var nIndex in this.aBandRows)
	{
		this.aBandRows[nIndex].release();
		this.aBandRows[nIndex] = null;
	}
	this.aBandRows = null;
}
