include("office/ExcelWrappers/xlsComment.js");
/**
Описывает объект "ячейка".
Имеется ввиду любая ячейка, в т.ч. "объединенная"
Если находится в бенде-шапке, то:
 - владеет коллекцией ячеек, находящихся непосредственно под ней
 - умеет сама строить дерево детей
 - печатает себя и детей
@param parent Ссылка на объект-хозяин. Обычно это объект класса xlsBand или порожденного от него
@param nRow номер строки верхнего левого угла
@param nCol номер столбца верхнего левого угла
*/
function xlsCell(parent, nRow, nCol)
{
	derive(this, xlsParent);
	this.storeParent(parent);
	
	//Поля
	this.expr = "";	//выражение, которое следует вычислить, чтобы получить значение в ячейке
	this.value = ""; //значение в ячейке
	this.cellType = null;	//тип значения в ячейке (HSSFCell.CELL_TYPE_STRING, ...)
	
	//Для работы с шахматными колонками
	this.aParsedCells = new Array();	//массив, содержащий ссылки на объекты xlsCell, на которые отобразится данная ячейка после вызова this.parse()

	this.isCross = false;	//признак "шахматной" ячейки
	//this.isCross = true;	//признак "шахматной" ячейки
	this.isBottom = false;	//признак того, что наша ячейка в самом низу бенда и не имеет "детей"

	//vlad 28.12.10
	//Пусть комментарий "живет" всегда. Мы теперь добавили в него режим "MERGE", который может быть не только 
	//this.oComment = null;	//ссылка на объект-комментарий класса xlsComment (актуально для шахматных ячеек в шапке таблицы)
	this.oComment = new xlsComment(this);

	this.tCols = null;		//ссылка на открытую таблицу со списком колонок
	
	this.aValues = null;	//объект, полученный с пом. scatter с таблицы колонок
	
	this.aChildCells = new Array();	//массив ссылок на дочерние xlsCell'ы, находящиеся под нами (это - настройка)
	this.aParsedChildCells = new Array();	//массив ссылок на дочерние xlsCell'ы, находящиеся под нами (это - "дубликаты" aChildCells, привязанные к aParsedCells)
	this.sParentFilter = "";	//фильтр, который нам навесит "родитель"
	this.lastCol = -1;
	this.lastRow = -1;
	
	//координаты исходной ячейки
	this.rowFrom = nRow;	
	this.colFrom = nCol;
	this.rowTo = nRow;
	this.colTo = nCol;

	//координаты ячейки после печати (PrinTeD)
	this.rowFromPtd = nRow;	
	this.colFromPtd = nCol;
	this.rowToPtd = nRow;
	this.colToPtd = nCol;
	
	//масив ссылок на ячейки из шапки таблицы над нашей ячейкой
	//инициализируется только для растянутых строк таблицы (объединенных)
	//над которыми в шапке таблицы несколько отдельных колонок
	this.aHCell = new Array();	
	this.clonedCell = null;	
	
	//Методы
	this.load = xlsCell_load;
	this.clone = xlsCell_clone;
	this.parse = xlsCell_parse;
	this.parseChildren = xlsCell_parseChildren;
	this.makeTable = xlsCell_makeTable;
	this.setParentFilter = xlsCell_setParentFilter;
	this.print = xlsCell_print;
	this.printOne = xlsCell_printOne;
	this.draw = xlsCell_draw;
	this.release = xlsCell_release;
	
	this.getLastCol = xlsCell_getLastCol;
	this.setLastCol = xlsCell_setLastCol;
	
	this.getLastRow = xlsCell_getLastRow;
	this.setLastRow = xlsCell_setLastRow;
	
	this.loadValues = xlsCell_loadValues;
	this.markParentAsCross = xlsCell_markParentAsCross;
	this.getFilter4Child = xlsCell_getFilter4Child;
	this.toString = xlsCell_toString;
	
	this.convertCrosshExpr = xlsCell_convertCrosshExpr;
}

/**
Выполнить загрузку информации по указанной ячейке
Изменяет значения
	this.rowFrom
	this.colFrom
	this.rowTo
	this.colTo
@param nRow Необязательный. Номер строки. Если не указан, то берем this.rowFrom
@param nCol Необязательный. Номер столбца. Если не указан, то берем this.colFrom
@return номер колонки, непосредственно следующей за this.colTo
*/
function xlsCell_load(nRow, nCol)
{
	this.trace("xlsCell_load(nRow="+nRow+", nCol="+nCol+"): start");

	//разберемся с переданными параметрами
	if (nRow!=undefined)
	{
		this.rowFrom = nRow;
	}
	if (nCol!=undefined)
	{
		this.colFrom = nCol;
	}
	
	//теперь определим - не регион ли это
	var hssfRegion = this.getHSSFRegionAt(this.rowFrom, this.colFrom);
	
	if (hssfRegion)
	{
		this.rowFrom = hssfRegion.getRowFrom();
		this.colFrom = hssfRegion.getColumnFrom();
		this.rowTo = hssfRegion.getRowTo();
		this.colTo = hssfRegion.getColumnTo();

		this.trace("this.rowFrom="+this.rowFrom+", this.colFrom="+this.colFrom+"), (this.rowTo="+this.rowTo+", this.colFrom="+this.colTo+")");
	}
	
	//левая верхняя ячейка
	var hssfCell = this.getHSSFCellAt(this.rowFrom, this.colFrom);

	this.value = this.getCellValue(hssfCell); //значение в ячейке
	this.cellType = hssfCell.getCellType();	//тип значения в ячейке

	//прочитаем комментарий
	//this.oComment = new xlsComment(this);

	if (typeof(this.value)=="string")
	{
		//vlad 28.12.2010
		//теперь комментарий могут иметь не только ячейки "шапки" таблицы, но и ячейки табличной части таблицы (для "MERGE")
		//прочитаем комментарий
		if (this.bandType == xlsConst.BT_THEADER || this.bandType == xlsConst.BT_TROW)
		{
			var hssfComment = hssfCell.getCellComment();
			if(hssfComment != null)
			{
				this.oComment.sComment = hssfComment.getString().getString();
				this.trace(" xlsCell_load: this.oComment.sComment = "+this.oComment.sComment);
			}
		}
		//для строк сразу же распарсим коментарий
		if (this.bandType == xlsConst.BT_TROW)
		{
			if (!isEmpty(this.oComment.sComment))
			{
				this.oComment.parse();
				this.merge = String(this.oComment.getAttribute(xlsConst.MERGE));
			}		
		}
		if (this.bandType == xlsConst.BT_THEADER && this.value.substr(0, 2)==xlsConst.DOUBLE_ASTERISK)
		{
			//Для шахматной ячейки в заголовке таблицы
			this.isCross = true;
			this.expr = this.value.substr(2);	//выражение, которое следует вычислить, чтобы получить значение в ячейке
			
			//прочитаем комментарий
			//this.oComment = new xlsComment(this);

			/*vlad 28.12.2010 перенесли выше
			var hssfComment = hssfCell.getCellComment();
			if(hssfComment != null)
			{
				this.oComment.sComment = hssfComment.getString().getString();
				this.trace(" xlsCell_load: this.oComment.sComment = "+this.oComment.sComment);
			}
			*/
			//родителя тоже сделаем "шахматным"
			this.markParentAsCross();
		}
		else if (inList(this.value.substr(0, 1), xlsConst.ASTERISK, xlsConst.PERCENT))
		{
			//Для обычной ячейки
			this.expr = this.value.substr(1);	//выражение, которое следует вычислить, чтобы получить значение в ячейке
		}
	}
	var retVal = this.colTo+1;
	
	this.trace("xlsCell_load: end return "+retVal);
	
	return retVal;
}

/**
Выполнить клонирование ячейки

@param oParentCell Необязательный параметр.Ссылка на ячейку, к которой "перепривяжем" нашего клона
Если не передан, то привяжем к себе.
@param noParsed Не размножать this.aParsedCells (нужно, когда клонируем ячейку, к-я сама Parsed)
@return ссылка на новую ячейку
*/
function xlsCell_clone(oParentCell, noParsed)
{
	this.trace("xlsCell_clone start typeOf(oParentCell)="+typeOf(oParentCell));

	this.nCloneCnt = this.nCloneCnt+1;
	
	if (oParentCell==undefined || oParentCell==null)
	{
		oParentCell = this;
		this.trace("   xlsCell_clone oParentCell = this");
	}

	//this.trace("   xlsCell_clone this="+this.toString());

	//Создаем объект-копию
	//как предка отдаем себя
	var oCell = new xlsCell(oParentCell, this.rowFrom, this.colFrom);

	//Поля
	oCell.expr = this.expr;	//выражение, которое следует вычислить, чтобы получить значение в ячейке
	oCell.value = this.value; //значение в ячейке
	oCell.cellType = this.cellType;	//тип значения в ячейке (HSSFCell.CELL_TYPE_STRING, ...)
	
	//Для работы с шахматными колонками
	oCell.isCross = this.isCross;	//признак "шахматной" ячейки
	oCell.isBottom = this.isBottom;	//признак того, что наша ячейка в самом низу бенда и не имеет "детей"
	oCell.oComment = this.oComment;	//ссылка на объект-комментарий класса xlsComment (актуально для шахматных ячеек в шапке таблицы)
	oCell.merge = this.merge;	//признак объединения ячеек
	oCell.rowIndex = this.rowIndex;	//индекс в таблице бенда
	
	this.trace("   xlsCell_clone cloning this.aChildCells.length="+this.aChildCells.length);
	for (var nInd in this.aChildCells)
	{
		var oChild = this.aChildCells[nInd].clone(oParentCell);
		oChild.p_parent = oCell;
		oCell.aChildCells[nInd] = oChild;
	}
	
	if (!noParsed)
	{
		this.trace("   xlsCell_clone cloning this.aParsedCells.length="+this.aParsedCells.length);
		for (var nInd in this.aParsedCells)
		{
			var oParsedCell = this.aParsedCells[nInd].clone(oParentCell);
			oParsedCell.p_parent = oCell;
			oCell.aParsedCells[nInd] = oParsedCell;
		}
	}
	oCell.sParentFilter = this.sParentFilter;	//фильтр, который нам навесит "родитель"
	
	//координаты исходной ячейки
	oCell.rowFrom = this.rowFrom;	
	oCell.colFrom = this.colFrom;
	oCell.rowTo = this.rowTo;
	oCell.colTo = this.colTo;
	
	oCell.aValues = this.aValues;
	oCell.tCols = this.tCols;
	
	oCell.aHCell = this.aHCell;
	this.clonedCell = oCell;

	this.trace("xlsCell_clone: end");

	return oCell;
}

/**
Выполнить разбор ячейки
Если ячейка находится в шапке таблицы - то по пути находятся 
ее "дети" и выполняется еще и разбор "детей", т.е. спускаемся по "дереву".
*/
function xlsCell_parse()
{
	this.trace("xlsCell_parse (this.bandType == xlsConst.BT_THEADER)="+(this.bandType == xlsConst.BT_THEADER)+", this.isCross="+this.isCross);
	if (this.bandType == xlsConst.BT_THEADER)
	{
		if (this.rowTo >= this.p_band.nEndRow)
		{
			this.isBottom = true;
		}

		//решаем проблему с "CROSSH" в шапке таблицы
		//фактически, нам следует самим создать комментарий для данной ячейки
		if (this.expr.substr(0, 6)=="CROSSH")
		{
			this.isCross = true;
			this.expr = "UR("+this.convertCrosshExpr(this.expr.substr(6))+")";
			this.oComment.sComment = "FIELDS:\"*\";"
		}
		
		//разберем комментарий "шахматной" колонки
		this.oComment.parse();
		
		//создадим таблицу "колонок"
		this.makeTable();

		if (!this.tCols.isEmpty())
		{
			//Для каждого значения
			for (this.tCols.MoveFirst(); !this.tCols.IsEOF(); this.tCols.MoveNext())
			{
				//создаем свою копию (без aParsedCells)
				var currCell = this.clone(this, true);
				
				//Снимаем значения полей, которые выбраны в this.tCols
				currCell.loadValues(this.tCols);

				//распарсим "детей"
				currCell.parseChildren();
				
				//и запоминаем "клона"
				this.aParsedCells[this.aParsedCells.length] = currCell;
			}
		}
	}
}

/**
Выполнить печать ячейки
Данная ф-ция вызывается для ячеек в обычных бендах и в бенде - "шапке таблицы"
Для печати строк метод print() будет перегружен ф-цией xlsCell_print_trow()
Если ячейка шахматная - то пробегаем по таблице this.tCols и создаем "клоны" себя и "клоны"
подчиненных ячеек из this.aChildCells
@param nRow Номер строки, где напечатаем
@param nCol Необязательный параметр.Номер столбца левого верхнего угла, где будем создавать ячейку
если не передан, то создаем в той же колонке, что и оригинал
*/
function xlsCell_print(nRow, nCol)
{
	this.trace("xlsCell_print(nRow="+nRow+", nCol="+nCol+") value=\""+this.value+"\"")
	var oNewCell = null;
	if (this.aParsedCells.length!=0)
	{
		this.trace("   xlsCell_print this.aParsedCells.length!=0 ("+this.aParsedCells.length+")")
		
		/*
		var   = new DpEval();
		oEval.setTable(this.tCols);
		
		//vlad 17.08.2010
		//для формирования заголовков шахматных колонок должны быть доступны и переменные "заголовка"
		//Попробуем перетянуть с глобального DpEval'а переменные заголовка
		//(мы их сделаем "переменными")
		if (typeOf(this.p_office.vars.eval.tbl)=="object")
		{
			var globalEvalTbl = this.p_office.vars.eval.tbl;
			for (var ind in globalEvalTbl)
			{
				oEval.addVar(ind, globalEvalTbl[ind]);
			}
		}
		*/
		//var oEval = this.p_workBook.oEval;
		//oEval.setTable(this.tCols);

		//Для каждого значения
		var nCol2print = nCol;
		this.trace("   xlsCell_print for (var nInd in this.aParsedCells)")
		for (var nInd in this.aParsedCells)
		{
			this.trace("      nInd="+nInd)
			var oCell = this.aParsedCells[nInd];

			for (var sInd in oCell.aValues)
			{
				this.trace("    xlsCell_print oCell.aValues[\""+sInd+"\"]="+oCell.aValues[sInd]);
			}
			
			//this.p_band.oEval.setTable(oCell.aValues);			
			if(this.isCross)
			{
				this.p_workBook.oEval.setTable(oCell.aValues);
			}
			else
			{
				//!!!vlad 23.07.2012
				//Так нельзя. Мы портим переменные заголовка
				//this.p_office.vars.eval.setTable(oCell.aValues);
			}
						
			//печатаем себя и "детей"
			oNewCell = oCell.printOne(nRow, nCol2print);
			if (oNewCell!=null)
			{
				nCol2print = oNewCell.getLastCol()+1;
			}
		}

	}
	else
	{
		this.trace("   xlsCell_print this.aParsedCells.length==0")
		oNewCell = this.printOne(nRow, nCol);
	}
	if (oNewCell!=null)
	{
		this.trace("xlsCell_print END value=\""+this.value+"\" (nRow="+nRow+", nCol="+nCol+") oNewCell.getLastCol()="+oNewCell.getLastCol()+", oNewCell.getLastRow()="+oNewCell.getLastRow())
	}
	else
	{
		this.trace("xlsCell_print END oNewCell==null value=\""+this.value+"\"")
	}
	return oNewCell;
}

/**
Выполнить печать одной ячейки. Вызывается из xlsCell_print
@param nRow Номер строки, где напечатаем
@param nCol Необязательный параметр.Номер столбца левого верхнего угла, где будем создавать ячейку
если не передан, то создаем в той же колонке, что и оригинал
*/
function xlsCell_printOne(nRow, nCol)
{
	this.trace("xlsCell_printOne printing at(nRow="+nRow+", nCol="+nCol+")");
	
	//если ячейка - шахматная, но у нее "пустая" таблица колонок, то просто пропускаем ее
	if (this.isCross && (this.tCols==null || this.tCols.isEmpty()))
	{
		this.trace("xlsCell_printOne END this.isCross && this.tCols.isEmpty()");
		return null;
	}

	this.trace("  xlsCell_printOne this.aValues trace:");
	for (var sInd in this.aValues)
	{
		this.trace("    xlsCell_printOne this.aValues[\""+sInd+"\"]="+this.aValues[sInd]);
	}
	
	var oNewCell = this.clone();
	var bPrint = true;//проверка что нужно делать печатать новую ячейку или увеличивать предыдущую
	//vlad
	//Инициализируем 
	oNewCell.setLastCol(nCol-1);

	//вычисляем и устанавливаем значение
	if (oNewCell.expr != "")
	{
		//выражение, которое следует вычислить, чтобы получить значение в ячейке
		if (this.bandType==xlsConst.BT_THEADER)
		{
			//oNewCell.value = this.p_band.oEval.eval(oNewCell.expr);			
			//Олег
			//Для шахматки используем eval c p_workBook
			//для обычной таблицы с p_office			
			this.trace("   xlsCell_printOne this.bandType==xlsConst.BT_THEADER");
			this.trace("   xlsCell_printOne oNewCell.expr="+oNewCell.expr);
			this.trace("   xlsCell_printOne this.isCross="+this.isCross);
			if(this.isCross)
			{
				oNewCell.value = this.p_workBook.oEval.eval(oNewCell.expr);
			}
			else
			{
				oNewCell.value = this.p_office.vars.eval.eval(oNewCell.expr);
			}

			this.trace("   xlsCell_printOne oNewCell.value="+oNewCell.value);
						
			if (oNewCell.value==null)
			{
				oNewCell.value = "";
			}
		}
		else if (this.bandType==xlsConst.BT_TROW)
		{
			if (!isEmpty(oNewCell.aValues) && !isEmpty(oNewCell.aValues["FCOLID"]))
			{
				//колонка шахматки.
				//Допустимо использование 2-х равнозначных вариантов выражения для значения:
				//1) 'R'+rcfcolid
				//2) CROSSR
				//Допустимо использование выражений вида round(CROSSR/100, 2)
				var sExpr = oNewCell.expr.replace(/['"]R['"]\s*\+\s*rcfcolid\b/gi,"CROSSR");
				sExpr = sExpr.replace(/\bCROSSR\b/gi, "R"+oNewCell.aValues["FCOLID"]);
				oNewCell.value = oNewCell.p_office.setField(sExpr);
				
				this.trace("   xlsCell_printOne oNewCell.expr=(\""+oNewCell.expr+"\")->sExpr(\""+sExpr+"\")");
			}
			else
			{
				this.trace("   xlsCell_printOne isEmpty(oNewCell.aValues) || isEmpty(oNewCell.aValues[\"FCOLID\"]");
				oNewCell.value = oNewCell.p_office.setField(oNewCell.expr);
				if (!isEmpty(this.merge))
				{
					var valMerge = oNewCell.p_office.setField(this.merge);
					/*if (!isEmpty(this.p_band.aValues) && !isEmpty(this.rowIndex) && !isEmpty(this.p_band.aMerges))
					{
						alert(this.p_band.aValues[this.rowIndex]+" "+oNewCell.value+"\n"
								+this.p_band.aMerges[this.rowIndex]+" "+valMerge+"\n"+oNewCell.expr+" "+this.merge+"\n"+val)
					}*/

					if (!isEmpty(this.p_band.aValues) && !isEmpty(this.rowIndex) && !isEmpty(this.p_band.aMerges)
						&& this.p_band.aValues[this.rowIndex] == oNewCell.value 
						&& this.p_band.aMerges[this.rowIndex] == valMerge)
					{
						bPrint = false;
					}
					if (!isEmpty(this.p_band.aValues) && !isEmpty(this.rowIndex))
					{
						this.p_band.aValues[this.rowIndex] = oNewCell.value;
					}
					if (!isEmpty(this.p_band.aMerges) && !isEmpty(this.rowIndex))
					{
						this.p_band.aMerges[this.rowIndex] = valMerge;
					}
				}
			}
		}
		else
		{
			oNewCell.value = oNewCell.p_office.setField(oNewCell.expr);
		}
	}

	this.trace("   xlsCell_printOne oNewCell.aChildCells.length="+oNewCell.aChildCells.length);

	if (oNewCell.aChildCells.length!=0)
	{
		//номер строки, в которой печатать "детей"
		var nChildRow = nRow + oNewCell.rowTo - oNewCell.rowFrom + 1;
		var nChildCol = nCol;
		for (var nInd in oNewCell.aChildCells)
		{	
			this.trace("      xlsCell_printOne printing nInd="+nInd+" in oNewCell.aChildCells at nChildRow="+nChildRow+", nChildCol="+nChildCol);
			var oChildCell = oNewCell.aChildCells[nInd];
			var oPrintedChildCell = oChildCell.print(nChildRow, nChildCol);
			if (oPrintedChildCell!=null)
			{
				nChildCol = oPrintedChildCell.getLastCol()+1;

				oNewCell.setLastCol(oPrintedChildCell.getLastCol());
				oNewCell.setLastRow(oPrintedChildCell.getLastRow());
				this.trace("      xlsCell_printOne oNewCell.setLastCol("+oPrintedChildCell.getLastCol()+")");
				this.trace("      xlsCell_printOne oNewCell.setLastRow("+oPrintedChildCell.getLastRow()+")");
			}
		}
	}
	else
	{
		//обыкновенная ячейка без "детей" - она растягиваться не может
		//поэтому просто нарисуем ее как есть
		if (oNewCell.aHCell.length > 0)
		{
			for (var ind in oNewCell.aHCell)
			{
				if (oNewCell.getLastCol() < oNewCell.aHCell[ind].clonedCell.colToPtd)
				{
					oNewCell.setLastCol(oNewCell.aHCell[ind].clonedCell.colToPtd);
				}
			}
		}
		else
			oNewCell.setLastCol(nCol+oNewCell.colTo-oNewCell.colFrom);
		
		oNewCell.setLastRow(nRow+oNewCell.rowTo-oNewCell.rowFrom);
		this.trace("      xlsCell_printOne !!! oNewCell.setLastCol("+oNewCell.getLastCol()+")");
		this.trace("      xlsCell_printOne !!! oNewCell.setLastRow("+oNewCell.getLastRow()+")");
	}
	
	
	if (bPrint == false)
	{
		var hssfCell = oNewCell.getHSSFCellAt(nRow-1, nCol);
		
		var poiRegion = lpProxy.getClass().getClassLoader().loadClass("org.apache.poi.hssf.util.Region").newInstance();
		poiRegion.setRowFrom(nRow-1);
		//fruckt
		//poiRegion.setColumnFrom(nCol);
		poiRegion.setColumnFrom(this.p_sheet.getRealColNum(nCol));
		poiRegion.setRowTo(nRow);
		//fruckt
		//poiRegion.setColumnTo(nCol);
		poiRegion.setColumnTo(this.p_sheet.getRealColNum(nCol));
		
		//fruckt
		//this.p_sheet.oSheet.addMergedRegion(poiRegion);
		this.p_sheet.getHSSFSheet(nCol).addMergedRegion(poiRegion);
		
		var hssfCellNew = oNewCell.getHSSFCellAt(nRow, nCol);
		hssfCellNew.setCellStyle(hssfCell.getCellStyle());
	}
	else
	{
		oNewCell.draw(nRow, nCol, oNewCell.getLastCol());
	}
	
	if (objTypeOf(this.p_parent)=="xlsCell")
	{
		if (oNewCell.getLastCol() > this.p_parent.getLastCol())
		{
			this.p_parent.setLastCol(oNewCell.getLastCol());
		}
		
		if (oNewCell.getLastRow() > this.p_parent.getLastRow())
		{
			this.p_parent.setLastRow(oNewCell.getLastRow());
		}
	}
	
	return oNewCell;
}

/**
Вызывается только для ячеек, находящихся в бенде-шапке таблицы
заполняет массив "дочерних" ячеек this.aChildCells, которые 
находятся непосредственно под ней и вызывает их метод parse()
*/
function xlsCell_parseChildren()
{
	this.trace("xlsCell_parseChildren start at this.rowFrom="+this.rowFrom+" to this.rowTo="+this.rowTo+", this.colFrom="+this.colFrom+" to this.colTo="+this.colTo);
	//искать детей будем только если наша ячейка не в самом низу бенда
	if (this.rowTo < this.p_band.nEndRow)
	{
		this.trace(" xlsCell_parseChildren this.p_band.nEndRow="+this.p_band.nEndRow);
		var nRow = this.rowTo + 1;
		
		//"разыменуем" родительский массив, содержащий ссылки на 
		//объекты xlsCell, владеющие каждой ячейкой листа
		var aCells = this.p_band.aCellOwners

		this.trace("   xlsCell_parseChildren search at nRow="+nRow+" from this.colFrom="+this.colFrom+" to this.colTo="+this.colTo);
		
		//пройдемся по строке под нами в диапазоне колонок, которые занимает наша ячейка
		for (var nCol=this.colFrom; nCol<=this.colTo; nCol++)
		{
			this.trace("      xlsCell_parseChildren for nCol="+nCol);
			//нашли ячейку, которая будет нашим "ребенком"
			var oOrigCell = aCells[nRow][nCol];
			
			//Создадим дубликат
			var oCell = oOrigCell.clone()
			//Обработаем случай, когда в шапке несколько колонок,
			//а в строке - одна объединенная (актуально для итоговых строк)
			//в таком случае данную объединенную колонку будем считать "дитем" той колонки в шапке,
			//с которой они начинаются в одной колонке
			if (oCell.colFrom>=this.colFrom)
			{
				oCell.p_parent = this;
				oCell.sParentFilter = this.getFilter4Child()+(!isEmpty(this.sParentFilter.trim()) ? " AND "+this.sParentFilter : "");
				this.trace("      xlsCell_parseChildren oCell.sParentFilter=\""+oCell.sParentFilter+"\"");

				this.aChildCells[this.aChildCells.length] = oCell;
			
				//и заставим найденную ячейку тоже распарситься
				oCell.parse();
			}
			else
			{
				this.trace("   xlsCell_parseChildren Нашли объединенную ячейку под шапкой");
			}
			//передвинем указатель на конец найденной ячейки
			nCol = oCell.colTo;
		}
	}
	else
	{
		this.trace("   xlsCell_parseChildren this.isBottom at this.rowFrom="+this.rowFrom+" to this.rowTo="+this.rowTo+", this.colFrom="+this.colFrom+" to this.colTo="+this.colTo);
		this.isBottom = true;
	}
	this.trace("xlsCell_parseChildren end at this.rowFrom="+this.rowFrom+" to this.rowTo="+this.rowTo+", this.colFrom="+this.colFrom+" to this.colTo="+this.colTo);
}

/**
Создает таблицу колонок для шахматной ячейки. 
Для каждой записи в данной таблице
будет создана копия ячейки со всеми ее детьми
*/
function xlsCell_makeTable()
{
	this.trace("xlsCell_makeTable start");
	
	//построим запрос на выборку таблицы "колонок", для которых размножится наша ячейка
	this.sFilter = String(this.oComment.getAttribute(xlsConst.FILTER));
	if(this.sFilter.toUpperCase() == "TRUE")
	{
		this.sFilter = sqlCmpTrue;
	}

/*
	if (!isEmpty(this.sParentFilter) && !isEmpty(this.sFilter))
	{
		this.sFilter +=" AND "+this.sParentFilter;
	}
	else if(!isEmpty(this.sParentFilter) && isEmpty(this.sFilter))
	{
		this.sFilter = this.sParentFilter;
	}
*/
	if (!isEmpty(this.sParentFilter))
	{
		this.sFilter +=" AND "+this.sParentFilter;
	}
	
	if (this.sFilter.substr(0, 5)==" AND ")
	{
		this.sFilter = this.sFilter.substr(4);
	}

	this.sFilter = String(isEmpty(this.sFilter) ? "" : ("(" + this.sFilter + ")"));
	var currCellOrder = String(isEmpty(this.oComment.getAttribute(xlsConst.ORDER)) ? "" : this.oComment.getAttribute(xlsConst.ORDER));
	var currSelect = String(isEmpty(this.oComment.getAttribute(xlsConst.FIELDS)) ? "" : this.oComment.getAttribute(xlsConst.FIELDS));
	this.trace("   xlsCell_makeTable: this.value = "+this.value);
	this.trace("   xlsCell_makeTable: this.oComment.sComment = "+this.oComment.sComment);
	this.trace("   xlsCell_makeTable: this.sFilter = "+this.sFilter);
	this.trace("   xlsCell_makeTable: currCellOrder = "+currCellOrder);
	this.trace("   xlsCell_makeTable: currSelect = "+currSelect);
	this.trace("   xlsCell_makeTable: this.isBottom = "+this.isBottom);

	//на нижнем уровне добавим поле FCOLID, если оно там не было указано явно
	if (this.isBottom && this.isCross && currSelect.toUpperCase().indexOf("FCOLID") == -1)
	{
		if (isEmpty(currSelect))
		{
			currSelect = " FCOLID";
		}
		else
		{
			currSelect += ", FCOLID";
		}
	}
	
	if (isEmpty(currSelect))
	{
		//нет списка полей для отбора. Такое может быть у "верхних" колонок-заголовков
		//В таком случае нам надо, чтобы запрос вернул ровно одну запись
		currSelect = "0";
	}
	
	//Запрос на выборку значений
	this.sFilter = (isEmpty(this.sFilter) ? SqlCmpTrue : this.sFilter);
	strSQL = " SELECT "
				+" DISTINCT " + currSelect
			+ " FROM "
				+ this.getTblC("n")
			+ " WHERE "
				+ this.sFilter
			+ (isEmpty(currCellOrder) ? xlsConst.STRING_EMPTY : (" ORDER BY " + currCellOrder))
	this.tCols = OpenTable("s", strSQL);

	this.trace("   xlsCell_makeTable strSQL=\n"+strSQL+"\nthis.tCols.recordCount="+this.tCols.recordCount);
	
	this.trace("xlsCell_makeTable end");
}

/**
Запоминает фильтр "от предка"
@param sParentFilter
*/
function xlsCell_setParentFilter(sParentFilter)
{
	this.sParentFilter = sParentFilter;
}

function xlsCell_release()
{
	this.trace("xlsCell_release");
	this.p_workBook = null;
	this.p_office = null;
	this.workBook = null;
	this.p_parent = null;
    this.p_sheet = null;
	this.p_band = null;
	
	if (this.oComment!=null)
	{
		this.oComment.release()
		this.oComment = null;
	}
	
	if (this.tCols)
	{
		this.tCols.close();
		this.tCols = null;
	}
	
	//"отпустим" ячейки-наследники
	for (var nIndex in this.aChildCells)
	{
		this.aChildCells[nIndex].release();
		this.aChildCells[nIndex] = null;
	}
	this.aChildCells = null;
	
	//"отпустим" ячейки-клоны
	for (var nIndex in this.aParsedCells)
	{
		this.aParsedCells[nIndex].release();
		this.aParsedCells[nIndex] = null;
	}
	this.aParsedCells = null;
}

/**
возвращает номер последней колонки, которую покрывает данная ячейка
Требуется для растягивания/сжимания колонок при печати шахматок
*/
function xlsCell_getLastCol()
{
	return this.lastCol;
}

/**
установить номер последней колонки, которую покрывает данная ячейка
Требуется для растягивания/сжимания колонок при печати шахматок
*/
function xlsCell_setLastCol(nCol)
{
	this.trace("xlsCell_setLastCol(nCol="+nCol+") было this.lastCol="+this.lastCol);
	this.lastCol = nCol;
}

/**
возвращает номер последней строки, которую покрывает данная ячейка
вместе с детьми.
*/
function xlsCell_getLastRow()
{
	return this.lastRow;
}

/**
установить номер последней строки, которую покрывает данная ячейка
вместе с детьми.
*/
function xlsCell_setLastRow(nRow)
{
	this.trace("xlsCell_setLastRow(nRow="+nRow+") было this.lastRow="+this.lastRow);
	this.lastRow = nRow;
}

/**
Отрисовать ячейку в указанном месте и до указанной колонки
Основная проблема, которую решаем - наша ячейка может растянуться или сжаться, 
если у нее есть "шахматные" дети (т.е. в общем случае ячейка покрывает "плавающее"
число колонок)
@param nRow Номер строки, где напечатаем
@param nColFrom Номер столбца левого верхнего угла, где будем создавать ячейку
@param nColTo Номер столбца правого верхнего угла, где будем создавать ячейку
*/
function xlsCell_draw(nRow, nColFrom, nColTo)
{
	this.trace("xlsCell_draw (nRow="+nRow+", nColFrom="+nColFrom+",nColTo="+nColTo+"): start this.getLastRow()="+this.getLastRow());
	this.trace("  xlsCell_draw (this.rowFrom="+this.rowFrom+", this.colFrom="+this.colFrom+"),(this.rowTo="+this.rowTo+", this.colTo="+this.colTo+") AT (nRow="+nRow+", nColFrom="+nColFrom+", nColTo="+nColTo+"): start");
	//на сколько следует подвинуть ячейку-копию по отношению к оригиналу
	var nRowShift = nRow - this.rowFrom;
	var nColShift = nColFrom - this.colFrom;

	//координаты ячейки после печати (PrinTeD)
	this.rowFromPtd = nRow;	
	this.colFromPtd = nColFrom;
	this.rowToPtd = this.rowTo + nRowShift;
	this.colToPtd = nColTo;
	
	this.setLastCol(this.colToPtd);
	
	//Рисовать будем только тогда, когда наша ширина > 0
	if (nColFrom <= nColTo)
	{
		//fruckt
		var startCol = nColFrom;
		var stopCol = nColTo;
		
		for (var nSht = this.p_sheet.getRealSheetNum(nColFrom)
			; nSht <= this.p_sheet.getRealSheetNum(nColTo)
			; nSht++)
		{
			stopCol = (nColTo > ((nSht + 1) * this.p_sheet.maxCellNum + nSht))
					? ((nSht + 1) * this.p_sheet.maxCellNum + nSht)
					: nColTo;
			
			//Даже если мы не были регионом, дети могут размножиться и заставить нас растянуться в ширину
			//if (this.rowFromPtd != this.rowToPtd || this.colFromPtd != this.colToPtd)
			if (this.rowFromPtd != this.rowToPtd || startCol != stopCol)
			{
				this.trace("   xlsCell_draw создаем регион(this.rowFromPtd="+this.rowFromPtd+", this.colFromPtd="+this.colFromPtd+"),(this.rowToPtd="+this.rowToPtd+", this.colToPtd="+this.colToPtd+")");

				var poiRegion = lpProxy.getClass().getClassLoader().loadClass("org.apache.poi.hssf.util.Region").newInstance();
				poiRegion.setRowFrom(this.rowFromPtd);
				//fruckt
				//poiRegion.setColumnFrom(this.colFromPtd);
				poiRegion.setColumnFrom(this.p_sheet.getRealColNum(startCol));
				poiRegion.setRowTo(this.rowToPtd);
				//fruckt
				//poiRegion.setColumnTo(this.colToPtd);
				poiRegion.setColumnTo(this.p_sheet.getRealColNum(stopCol));
				
				//fruckt
				//this.p_sheet.oSheet.addMergedRegion(poiRegion);
				this.p_sheet.getHSSFSheet(stopCol).addMergedRegion(poiRegion);
			}

			//создадим ячейки оббегая строки и в них - колонки.
			for (var nCurRow = this.rowFromPtd; nCurRow <= this.rowToPtd; nCurRow++)
			{
				//обходим колонки в данной строке
				//for (var nCurCol = this.colFromPtd; nCurCol <= this.colToPtd; nCurCol++)
				for (var nCurCol = startCol; nCurCol <= stopCol; nCurCol++)
				{
					//создаем ячейку
					var hssfCell = this.getHSSFCellAt(nCurRow, nCurCol);
					
					//а теперь самое сложное - стиль.
					//из-за того, что ячейка могла растягиваться и "ужиматься" со стилями возможны проблемы.
					//Найдем номер колонки, из которой будем брать стиль.
					//Для ячеек "левого" края - берем "родной" стиль.
					//Для ячеек, чей номер >= правому краю оригинальной ячейки (но не сам правый край),
					//берем стиль ячейки непосредственно предшествующей правому краю.
					//Для правого края берем стиль правого края
					var nColNum4Style;
					//fruckt
					//if (nCurCol == this.colFromPtd)
					if (nCurCol == startCol)
					{
						//крайний левый край
						nColNum4Style = this.colFrom;
					}
					//fruckt
					//else if (nCurCol == this.colToPtd)
					else if (nCurCol == stopCol)
					{
						//крайний правый край
						nColNum4Style = this.colTo;
					}
					else
					{
						//ячейка в средине
						//Для ячеек, которые шире одной колонки, берем номер предпоследней колонки
						nColNum4Style = this.colTo == this.colFrom ? this.colFrom : (this.colTo - 1);
					}

					var nRowNum4Style = nCurRow - nRowShift;
					
					//Установим стиль
					var origHSSFCell = this.getHSSFCellAt(nCurRow - nRowShift, nColNum4Style);
					hssfCell.setCellStyle(origHSSFCell.getCellStyle());
					
					//Установим ширину колонки
					if (nColShift != 0)
					{
						//fruckt
						//this.p_sheet.oSheet.setColumnWidth(nCurCol, this.p_sheet.oSheet.getColumnWidth(nColNum4Style));
						this.p_sheet.getHSSFSheet(nCurCol).setColumnWidth(this.p_sheet.getRealColNum(nCurCol), this.p_sheet.getHSSFSheet(nColNum4Style).getColumnWidth(nColNum4Style));
					}
				}
			}
		
			//для 1-й из ячеек в объединенном регионе установим значение
			var hssfCell = this.getHSSFCellAt(this.rowFromPtd, this.colFromPtd);
			//fruckt - проверим, не является ли значение ячейки формулой
			if (this.cellType == hssfCell.CELL_TYPE_FORMULA)
			{
				//подвинем все ссылки в формуле 
				var sShiftedFormula = this.shiftFormula(this.value, nRowShift, nColShift);

				//fruckt
				//hssfCell.setCellFormula(sShiftedFormula);
				this.getHSSFCellAt(this.rowFromPtd, startCol).setCellFormula(sShiftedFormula);
				
				this.trace("   xlsCell_draw hssfCell.setCellFormula(\""+sShiftedFormula+"\")  this.rowFromPtd = "+this.rowFromPtd+", this.colFromPtd = "+this.colFromPtd);
			}
			else if (this.cellType != hssfCell.CELL_TYPE_BLANK)
			{
				//fruckt
				//hssfCell.setCellValue(this.value);
				this.getHSSFCellAt(this.rowFromPtd, startCol).setCellValue(this.value);

				this.trace("   xlsCell_draw hssfCell.setCellValue(\""+this.value+"\")  this.rowFromPtd = "+this.rowFromPtd+", this.colFromPtd = "+this.colFromPtd);
			}
			
			startCol = stopCol + 1;
		}
	}
	
	this.trace("xlsCell_draw: end this.getLastRow()="+this.getLastRow());
}

/**
Считать значения всех полей с текущей записи в переданной таблице
Это нужно, чтобы для шахматных колонок родитель отдал фильтром значения полей
@param tbl объект-таблица, в которой стоим на строке, с которой снимем значения
*/
function xlsCell_loadValues(tbl)
{
	this.trace("xlsCell_loadValues  start");
	this.aValues = scatter(tbl);
	//для POSTGRE если у поля нет имени , оно называется ?COLUMN?, и не понимается базой

	if (SUBD == POSTGRE)
	{
		var aVal = new Object();
		for (var sInd in this.aValues)
		{
			if (sInd == "?COLUMN?")
			{
				continue;
			}
			aVal[sInd] = this.aValues[sInd];
		}
		this.aValues = aVal;
	}
	
	for (var sInd in this.aValues)
	{
		this.trace("    xlsCell_loadValues this.aValues[\""+sInd+"\"]="+this.aValues[sInd]);
	}
	
	this.trace("xlsCell_loadValues  end");
}

/**
пометить родительские ячейки в шахматной таблице, что они - шахматные
Это требуется, когда на верхнем уровне в ячейках нет комментария, а ячейки под ними - шахматные
*/
function xlsCell_markParentAsCross()
{
	this.trace("xlsCell_markParentAsCross  start");
	for (var nRow=this.p_band.nStartRow; nRow < this.rowFrom; nRow++)
	{
		this.trace("  xlsCell_markParentAsCross  marked at "+nRow+", "+this.colFrom);
		var oParentCell = this.p_band.aCellOwners[nRow][this.colFrom];

		oParentCell.isCross = true;
		if (oParentCell.oComment==null)
		{
			oParentCell.oComment = new xlsComment(oParentCell);
		}

	}
	
	this.trace("xlsCell_markParentAsCross  end");
}


/**
Вернет кусок условия для "детей" (он у детей станет sParentFilter)
Делаем из this.aValues
*/
function xlsCell_getFilter4Child()
{
	this.trace("xlsCell_getFilter4Child  start");
	var retVal = "";
	for (sFieldName in this.aValues)
	{
		//У нас может быть "пустое" имя поля - у колонок, которые шахматными сделали "дети"
		//Для таких колонок запрос в makeTable выглядел приблизительно так:
		// SELECT 0 FROM ...
		//и возвращал нам одну строку. Понятно, что никакой смысловой нагрузки в данной строке нет,
		//поэтому такие поля отсеиваем.
		if (!isEmpty(sFieldName))
		{
			retVal += " AND "+sFieldName+"="+sqlTo(this.aValues[sFieldName]);
		}
	}
	if (retVal!="")
	{
		retVal = retVal.substr(4);
	}
	
	this.trace("xlsCell_getFilter4Child  end retVal = \""+retVal+"\"");
	return retVal;
}

function xlsCell_toString()
{
	var sOffset = replicate("   ", this.nNestingLevel);
	var str = "\n"+sOffset+"("+this.name+"): "
	str += "\n"+sOffset+"Ori("+this.rowFrom+", "+this.colFrom+")-("+this.rowTo+", "+this.colTo+")";
	str += "\n"+sOffset+"Ptd("+this.rowFromPtd+", "+this.colFromPtd+")-("+this.rowToPtd+", "+this.colToPtd+")";
	str += "\n"+sOffset+"this.expr=\""+this.expr+"\"";
	str += "\n"+sOffset+"this.value=\""+this.value+"\"";
	str += "\n"+sOffset+"this.aParsedCells.length="+this.aParsedCells.length;
	for (var nInd in this.aParsedCells)
	{
		str += "\n"+sOffset+"ParsedCell #"+nInd;
		str += this.aParsedCells[nInd].toString();
	}

	str += "\n"+sOffset+"this.aChildCells.length="+this.aChildCells.length;
	for (var nInd in this.aChildCells)
	{
		str += "\n"+sOffset+"ChildCell #"+nInd;
		str += this.aChildCells[nInd].toString();
	}
	return str
}


/**
Выполнить конвертацию выражения для CROSSH
изменяет переменные вида rcfsc на fsc
Переделана из DpEval_parse
*/
function xlsCell_convertCrosshExpr(expr)
{
	var svExpr = expr;

	var evalExpr = "", v, isFunc;
	var count = 0;
	expr = expr.replace(/\"/g, "'");
	while((v=/([а-яА-Яa-zA-Z_#][а-яА-Яa-zA-Z_0-9#\.@]*)|(\'[^\']*[\'])/.exec(expr)) != null)   // Ищем ближайший литерал
	{
		var nReg = expr.search(/([а-яА-Яa-zA-Z_#][а-яА-Яa-zA-Z_0-9#\.@]*)|(\'[^\']*[\'])/);
		evalExpr += expr.substr(0,nReg);
		expr = expr.substr(nReg + v[0].length);
		if(evalExpr.charAt(evalExpr.length-1)==".")   // Свойство или метод
		{
			evalExpr += v[0];
		}
		else if(v[0].charAt(0)=="\'")  // Строковая константа
		{
			var upV0 = v[0].toUpperCase();
			evalExpr += v[0];
		}
		else if( expr.charAt(0)=="(" )	// Функци
		{
			var c, n=0, deep=1;
			var par = new Array();
			while((c=expr.charAt(++n))!="" && deep>0)
				if(c=="(")
					deep++;
				else if(c==")")
					deep--;
				else if(c=="," && deep==1)
				{
					par[par.length] = expr.substring(1, n)
					expr = expr.substr(n);
					n = 0;
				}
			par[par.length] = expr.substring(1, n-1);
			if(deep==0)
			{
				evalExpr +="" + v[0] + "("
				for(var iCnt=0; iCnt<par.length; iCnt++)
				{
					if(iCnt!=0)
					{
						evalExpr += ",";
					}
					
					//рекурсивный вызов для конвертации параметра функции
					evalExpr += this.convertCrosshExpr(par[iCnt]);
				}
				evalExpr += ")";
				expr = expr.substr(n);
			}
		}
		else    // Переменная
		{
			var varName = v[0];
			if (varName.substr(0, 3).toLowerCase()=="rcf")
			{
				evalExpr += varName.substr(2);
			}
			else
			{
				evalExpr += varName;
			}
		}
		if(count++>1000) break;	// зациклилось (не бывает!)
	}
	evalExpr += expr;
	//trace("End: expr="+svExpr)
	//trace("        ->"+evalExpr)
	return evalExpr;
}
