//Предок всех объектов (классов) для проекта печати Excel
//w - wrap
//Пример использования
/*
    include("office/ExcelWrappers/xlsParent.js")
    function myClass(parent)
    {
        derive(this, xlsParent);
        this.storeParent(parent); ...
    }
*/
function xlsParent()
{
    this.p_workBook = null;//Указатель на текущий xlsWorkBook
    this.p_parent;//Указатель на родительский класс
	this.p_office = null;//Указатель на файл poiprint.js???
	this.p_sheet;//Указатель на текущий xlsSheet
	this.p_band;//Указатель на текущий Band
	this.bandType;//тип бенда. Одна из констант xlsConst.BT_xxx
	
	//уровень вложенности данного объекта в иерархии всех объектов
	//Будет oParent.nNestingLevel+1
	//нужен для организации красивого вывода с отступами в xlsParent_trace()
	this.nNestingLevel = 0;
	
	//кол-во потомков - тех, кто для себя вызвал storeParent с сылкой на наш объект
	this.nChildCnt = 0;

	//кол-во наших клонов
	this.nCloneCnt = 0;
	
	//Включить/выключить вывод в консоль диагностических сообщений с пом. this.trace()
	//this.isTraceEnabled = true;
	this.isTraceEnabled = false;
	
	this.trace = xlsParent_trace;
    
    this.storeParent = xlsParent_storeParent;
	
	//Получить значение, хранящееся в конкретной ячейке (по объекту)
	this.getCellValue = xlsParent_getCellValue;
	//Получить значение, хранящееся в конкретной ячейке (по ее адресу)
	this.getCellValueAt = xlsParent_getCellValueAt;
	//Получить объект-ячейку по ее адресу
	this.getHSSFCellAt = xlsParent_getHSSFCellAt;

	//Получить объект-регион, к которому относится ячейка
	this.getHSSFRegionAt = xlsParent_getHSSFRegionAt;
	
	//Получить таблицу колонок шахматки (имя или объект, в завис. от режима)
	this.getTblC = xlsParent_getTblC;
	
	//подвинуть ссылки в формуле
	this.shiftFormula = xlsParent_shiftFormula;
	
}

//Устанавливает текущие значения переменных
//oParent - указатель на родительский класс
function xlsParent_storeParent(oParent)
{
	this.p_parent = oParent;
    this.p_workBook = oParent.p_workBook;
	this.p_sheet = oParent.p_sheet;
	this.p_band = oParent.p_band;
	this.p_office = oParent.p_office;
	
	if (this.p_band!=null)
	{
		this.bandType = this.p_band.bandType;
	}
	
	oParent.nChildCnt = oParent.nChildCnt+1;
	//"имя" объекта - пригодится на этапе отладки
	this.name = (oParent.name=="" ? objTypeOf(oParent):oParent.name)+"->("+oParent.nChildCnt+")"+objTypeOf(this)+" клон "+oParent.nCloneCnt;
	
	this.nNestingLevel = oParent.nNestingLevel+1;
}

// создает диалог отображения процесса
function createProgress(recCnt, taskName)
{
	//vlad 27.05.2010
	//если дебет запущен как сервис, то никаких обращений к интерфейсу пользователя не должно быть
	if(lpProxy.isStartArgument("service"))
	{
		return null;
	}
	//Нельзя при работе с таблицей грида вызывать диалоговые окна поверх него.
	//Вызывается перерисовка и смещает текущую запись таблицы.
	if(true)
	{
		var progress = getStatusLineManager(); 
		progress.getProgressMonitor().beginTask("", recCnt);
		return progress;
	}
	var progress = new Packages.org.eclipse.jface.dialogs.ProgressMonitorDialog(Packages.org.eclipse.swt.widgets.Display.getCurrent().getActiveShell());
	progress.setCancelable(true);	// "Отмена" энаблится, но не нажимается!
	progress.open();
	progress.getProgressMonitor().beginTask("", recCnt);
	progress.getShell().setText(taskName);
	return progress;
}

function xlsParent_trace(str)
{
	if(this.isTraceEnabled)
	{
		if (isEmpty(this.name))
		{
			this.name = objTypeOf(this)
		}
		var str = replicate("   ", this.nNestingLevel)+"("+this.name+"): "+str;
		trace(str);
	}
}

/**
Получить значение, хранящееся в конкретной ячейке
@param hssfCell	ячейка
@return текст из ячейки. Если ячейка==null, то вернет пустую строку
*/
function xlsParent_getCellValue(hssfCell)
{
	var retVal = "";
	
	if(hssfCell != null)
	{
		switch (hssfCell.getCellType())
		{
			case hssfCell.CELL_TYPE_STRING:
			case hssfCell.CELL_TYPE_BLANK:
				retVal = String(hssfCell.getStringCellValue());
				break;

			case hssfCell.CELL_TYPE_FORMULA:
				
				//retVal = "="+String(hssfCell.getCellFormula());
				//vlad к String не приводим
				//retVal = String(hssfCell.getCellFormula());
				retVal = hssfCell.getCellFormula();
				break;

			case hssfCell.CELL_TYPE_NUMERIC:
				retVal = Number(hssfCell.getNumericCellValue());
				break;

			case hssfCell.CELL_TYPE_BOOLEAN:
				retVal = hssfCell.getBooleanCellValue();
				break;

			case hssfCell.CELL_TYPE_ERROR:
				if (getPar("IS_DEBUG"))
				{
					alert("xlsCell_getHSSFCellValue: значение ячейки = CELL_TYPE_ERROR, строка "+hssfCell.getRowIndex()+", колонка "+hssfCell.getColumnIndex());
				}
				retVal = hssfCell.getErrorCellValue();
				break;

			default:
				if (getPar("IS_DEBUG"))
				{
					alert("xlsCell_getHSSFCellValue: неизвестный тип ячейки, строка "+hssfCell.getRowIndex()+", колонка "+hssfCell.getColumnIndex());
				}
				retVal = "";
				break;
		}
	}

	//this.trace("xlsParent_getCellValue: return \""+retVal+"\"");
	return retVal;
}

/**
Получить значение, хранящееся в конкретной ячейке (по ее адресу)
@param nRow строка
@param nCol столбец
@return текст из ячейки. Если ячейка==null, то вернет пустую строку
*/
function xlsParent_getCellValueAt(nRow, nCol)
{
	//this.trace("xlsParent_getCellValueAt(nRow="+nRow+", nCol="+nCol+"): start");

	var hssfCell = this.getHSSFCellAt(nRow, nCol);
	var retVal = this.getCellValue(hssfCell);

	//this.trace("xlsParent_getCellValueAt(nRow="+nRow+", nCol="+nCol+"): return \""+retVal+"\"");
	return retVal;
}

/**
Получить объект-ячейку по ее адресу
@param nRow строка
@param nCol столбец
@return ссылку на ячейку. Если ячейка==null, создаст, если нужно, и строку, и саму ячейку
*/
function xlsParent_getHSSFCellAt(nRow, nCol)
{
	//this.trace("xlsParent_getHSSFCellAt(nRow="+nRow+", nCol="+nCol+"): start");
	var hssfCell = null;
	
	if (this.p_sheet!=null)
	{
		//fruckt
		//var hssfRow = this.p_sheet.getRow(nRow)
		var hssfRow = this.p_sheet.getRow(nRow, nCol);
		if(hssfRow == null)
		{
			//fruckt
			//hssfRow = this.p_sheet.createRow(nRow);
			hssfRow = this.p_sheet.createRow(nRow, nCol);
		}
		
		//fruckt
		//если кол-во колонок в результате больше this.p_sheet.maxCellNum (например шахматка)
		//то динамически добавляются листы - соответственно номер колонки на новом листе будет другой
		//hssfCell = hssfRow.getCell(nCol);
		hssfCell = hssfRow.getCell(this.p_sheet.getRealColNum(nCol));
		if(hssfCell == null)
		{
			//fruckt
			//hssfCell = hssfRow.createCell(nCol);
			hssfCell = hssfRow.createCell(this.p_sheet.getRealColNum(nCol));
		}
	}
	else
	{
		this.trace("   xlsParent_getHSSFCellAt(nRow="+nRow+", nCol="+nCol+"): ОШИБКА!!! this.p_sheet==null");
	}

	//this.trace("xlsParent_getHSSFCellAt(nRow="+nRow+", nCol="+nCol+"): return \""+hssfCell+"\"");
	return hssfCell;
}

/**
Получить объект-регион, к которому относится ячейка
@param nRow строка
@param nCol столбец
@return ссылку на ячейку. Если ячейка не в регионе, то вернет null
*/
function xlsParent_getHSSFRegionAt(nRow, nCol)
{
	//this.trace("xlsParent_getHSSFRegionAt(nRow="+nRow+", nCol="+nCol+"): start");
	var retVal = null;
	
	if (this.p_sheet!=null)
	{
		var aRegions = this.p_sheet.aRegions;
		for (var nIndex in aRegions)
		{
			var oR = aRegions[nIndex];
			if(
				nRow >= oR.getRowFrom()
				&& nRow <= oR.getRowTo()
				&& nCol >= oR.getColumnFrom()
				&& nCol <= oR.getColumnTo()
			)
			{
				retVal = oR;
				break;
			}
		}
	}
	else
	{
		this.trace("   xlsParent_getHSSFRegionAt(nRow="+nRow+", nCol="+nCol+"): ОШИБКА!!! this.p_sheet==null");
	}

	//this.trace("xlsParent_getHSSFRegionAt(nRow="+nRow+", nCol="+nCol+"): return "+retVal);
	return retVal;
}

/**
Вернет таблицу/имя таблицы колонок шахматки или просто таблицу, если у нас не шахматка
@param mode что вернуть. Если равен "n", то вернет имя таблицы колонок шахматки
В противном случае вернет объект-таблицу колонок шахматки
*/
function xlsParent_getTblC(mode)
{
	var tbl2Prn = this.p_office.vars.getRTbl();
	var tblC2Print = tbl2Prn;

	switch(objTypeOf(tblC2Print))
	{
		case "DpCTable":
			//Для шахматки берем таблицу колонок
			tblC2Print = tbl2Prn.GetCol();
			break;
	}

	var retVal = tblC2Print;
	retVal = mode == "n"
		? String(tblC2Print.getTableName())
		: tblC2Print;
	
	//"Особый" случай, когда печатаем документ - это его таблица строк
	//Сюда мы попадем из xlsCell_makeTable() для построения запроса
	//SELECT DISTINCT 0 FROM ...
	//поэтому отдадим ЛЮБУЮ таблицу. Пусть это будет план счетов
	if(objTypeOf(tblC2Print)=="DpDocRowTable" && mode == "n")
	{
		retVal = "^cl_schet"
	}
	
	return retVal
}


/**
изменяет индексы в ссылках в формуле
Считается, что стиль ссылок "A1"
Для ссылок в стиле "R1C1" пока не реализовано
меняет в ссылках вида: A15 и в диапазонах 5:10 или D:F
@param sExpr выражение, которое будем изменять
@param nRowShift на сколько строк сдвинуть ссылки
@param nColShift на сколько колонок сдвинуть ссылки
*/
function xlsParent_shiftFormula(sExpr, nRowShift, nColShift)
{
	//вспомогательные ф-ции
	function shiftCol(sCol, nColShift)
	{
		//преобразуем номер колонки вида "DA" в число, а потом обратно
		//Можно было бы воспользоваться стандартным преобразованием из системы исчисления с алфавитом A-Z
		//в десятичную. Но в экселе алфавит "без нуля", т.е. "A"==1,после "Z" идет не "A0", а "AA"
		//Поэтому поступим по рабоче-крестьянски.
		//Для младшего разряда считаем, что алфавит начинается с "нуля",
		//а для всех, что выше - что с единицы 
		var alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		var nCol = 0;
		var nMultiplier = 1;
		var radix = alphabet.length;
		for (var nPower = 0; nPower<sCol.length; nPower++)
		{
			var sDigit = sCol.substr(sCol.length-1-nPower, 1);
			
			nCol = nCol + (alphabet.lastIndexOf(sDigit)+(nMultiplier==1 ? 0 : 1))*nMultiplier;
			
			nMultiplier = nMultiplier*radix;
		}
		
		nCol = nCol+nColShift;
		
		var retVal = ""
		if (nCol==0)
		{
			retVal = "A";
		}
		else
		{
		
			//теперь переводим обратно
			var nRest = nCol;
			var nDigit;
			while (nRest>0)
			{
				nDigit = nRest%radix;
				nRest = Math.round((nRest-nDigit)/radix);
				retVal = alphabet.charAt( nDigit+(retVal=="" ? 0 : -1) ) + retVal;
			}
			//trace("shiftCol(sCol=\""+sCol+"\", nColShift="+nColShift+") = \""+retVal+"\"")
		}
		return retVal;
	}

	function shiftRow(sRow, nRowShift)
	{
		var retVal = sRow;
		var nRow = parseInt(sRow, 10);
		if (!isNaN(nRow))
		{
			retVal = String(nRow+nRowShift);
		}
		return retVal;
	}
	
	//если нам что-то не передали, проинициализируем нулем
	if (nRowShift==undefined)
	{
		nRowShift = 0;
	}
	if (nColShift==undefined)
	{
		nColShift = 0;
	}
	
	
	var retVal = "";
	
	//Обработка обыкновенной ссылки вида "A1"
	//регэксп, который нам понаходит ссылки вида "A1" или "IV25631"
	var reRef = /(\$?)([A-Z]{1,2})(\$?)(\d{1,5})/i;
	//после compile() не работает
	//reRef.compile();
	var sTail = sExpr;
	var aRes = null;
	while ((aRes = reRef.exec(sTail))!=null)
	{
		var nPos = aRes.index;
		retVal = retVal + sTail.substr(0, nPos) + aRes[1] + shiftCol(aRes[2], nColShift) + aRes[3] + shiftRow(aRes[4], nRowShift);
		sTail = sTail.substr(nPos+aRes[0].length);
	}
	retVal = retVal+sTail;

	//Обработка обыкновенной ссылки вида "D:F" (диапазон колонок)
	reRef = /(\$?)([A-Z]{1,2}):(\$?)([A-Z]{1,2})/i;
	sTail = retVal;
	aRes = null;
	retVal = "";
	while ((aRes = reRef.exec(sTail))!=null)
	{
		var nPos = aRes.index;
		retVal = retVal + sTail.substr(0, nPos) + aRes[1] + shiftCol(aRes[2], nColShift) +":"+ aRes[3] + shiftCol(aRes[4], nColShift);
		sTail = sTail.substr(nPos+aRes[0].length);
	}
	retVal = retVal+sTail;
	
	//Обработка обыкновенной ссылки вида "5:8" (диапазон строк)
	reRef = /(\$?)(\d{1,5}):(\$?)(\d{1,5})/i;
	sTail = retVal;
	aRes = null;
	retVal = "";
	while ((aRes = reRef.exec(sTail))!=null)
	{
		var nPos = aRes.index;
		retVal = retVal + sTail.substr(0, nPos) + aRes[1] + shiftRow(aRes[2], nRowShift) +":"+ aRes[3] + shiftRow(aRes[4], nRowShift);
		sTail = sTail.substr(nPos+aRes[0].length);
	}
	retVal = retVal+sTail;
	//trace("xlsParent_shiftFormula(sExpr=\""+sExpr+"\", nRowShift="+nRowShift+", nColShift="+nColShift+")=\""+retVal+"\"")
	return retVal;
}