include("office/ExcelWrappers/xlsConst.js");
include("office/ExcelWrappers/xlsSheet.js");
include("office/ExcelWrappers/xlsParent.js");

function xlsWorkBook(oOffice)
{
    derive(this, xlsParent);
	this.trace("xlsWorkBook(): start");

	this.p_workBook = this;
	this.p_office = oOffice;
	
	//vlad 11.07.2012
	//Добавим возможность работать с xlsx	
	//this.workBook = lpProxy.openExcelWorkbook(oOffice.file);//Текущий HSSFWorkBook
	var oWbf = getJavaClass("org.apache.poi.ss.usermodel.WorkbookFactory").newInstance();
	this.workBook = oWbf.create(new java.io.FileInputStream(oOffice.file));//Текущий WorkBook
	
	this.aSheets = new Array();//Массив xlsSheet'ов
	this.aFonts = new Array();
	
	this.oRepeatingData = null;//Объект с данными о сквозных строках (его заполнит this.getRepeatingData())
	
	this.oEval = new DpEval();	//объект, которым смогут пользоваться все, кому не лень
	
	//Методы
	this.load = xlsWorkBook_load;
	this.parse = xlsWorkBook_parse;
	this.print = xlsWorkBook_print;
	
	this.getRepeatingData = xlsWorkBook_getRepeatingData;
	this.setRepeatingRows = xlsWorkBook_setRepeatingRows;

	this.setRowsHeight = xlsWorkBook_setRowsHeight;
	this.calculateRowHeight = xlsWorkBook_calculateRowHeight;
	this.textExtent = xlsWorkBook_textExtent;
	
	this.setPostProcessAction = xlsWorkBook_setPostProcessAction;
	this.getPostProcessAction = xlsWorkBook_getPostProcessAction;

	this.getCellNum = xlsWorkBook_getCellNum;
	
	this.save = xlsWorkBook_save;
	this.release = xlsWorkBook_release;

	//метод для конвертации формы в формат jrxml;
	this.convert2Jrxml = xlsWorkBook_convert2Jrxml;
	
	this.trace("xlsWorkBook(): end");
}

function xlsWorkBook_load()
{
	this.trace("xlsWorkBook_load: start");
	
	//получим данные о сквозных строках
	this.oRepeatingData = this.getRepeatingData();
	
	//Настроим наш oEval
	//для формирования заголовков шахматных колонок должны быть доступны и переменные "заголовка"
	//Попробуем перетянуть с глобального DpEval'а переменные заголовка
	//(мы их сделаем "переменными").
	//Нам надо, чтобы имя переменной начиналось на "H"
	//Но в то же время, у нас могут быть переменные, которые уже начинаются на "H"
	//Чтобы угодить всем, таким переменным "H" спереди дописывать не будем
	if (typeOf(this.p_office.vars.eval.tbl)=="object")
	{
		var globalEvalTbl = this.p_office.vars.eval.tbl;
		var varName;
		for (var ind in globalEvalTbl)
		{
			varName = ind;
			if (ind.substr(0,1).toUpperCase()!="H")
			{
				//переменной добавим в имя "H", только если она еще не начинается на нее
				varName = "H"+varName;
			}
			this.oEval.addVar(varName, globalEvalTbl[ind]);
		}
	}

	
	for (var nSheet = 0; nSheet < this.workBook.getNumberOfSheets(); nSheet++)
	{
		var oSheet = new xlsSheet(this, nSheet);

		oSheet.load();

		this.aSheets[this.aSheets.length] = oSheet;
	}
	this.trace("xlsWorkBook_load: end");
}

function xlsWorkBook_parse()
{
	this.trace("xlsWorkBook_parse: start");
	for(var nIndex in this.aSheets)
	{
		this.aSheets[nIndex].parse();
	}
	this.trace("xlsWorkBook_parse: end");
}

function xlsWorkBook_print()
{
	for(var nIndex in this.aSheets)
	{
		this.aSheets[nIndex].print();
		this.setRowsHeight(this.aSheets[nIndex]);
	}
	
	//настроим сквозные строки
	this.setRepeatingRows();

}

/**
Установить действие, выполняемое по окончании генерации отчета
Само действие определяем по строке, непосредственно следующей за 
5-ю "звездами" в конце формы.
Обрабатываются такие строки:
1)"EDIT DOCUMENT"
2)"PRINT DOCUMENT"
3)""
@param nAction одна из констант семейства xlsConst.PPA_xxx
*/
function xlsWorkBook_setPostProcessAction(nAction)
{
	this.postProcessAction = nAction;
}

function xlsWorkBook_getPostProcessAction()
{
	return this.postProcessAction;
}

function xlsWorkBook_release()
{
	this.p_workBook = null;
	this.p_office = null;
	this.workBook = null;
	this.p_parent = null;
    this.p_sheet = null;
	this.p_band = null;

	for (var i = 0; i < this.aSheets.length; i++)
	{
		this.aSheets[i].release();
	}

	this.aSheets = null;
}

function xlsWorkBook_save()
{
	this.trace("xlsWorkBook_save: start");
	
	var os = new DpFileOutputStream(this.p_office.file);
	this.workBook.write(os);
	os.close();
	
	this.trace("xlsWorkBook_save: end");
}

/**
Установить "сквозные строки"
*/
function xlsWorkBook_setRepeatingRows()
{
	var oRD = this.oRepeatingData;
	if (oRD!=null)
	{
		//решаем проблему:
		//в this.oRepeatingData хранятся номера строк, взятые из настройки формы.
		//Но после печати у нас пропадают строки с *=, *+, *- и т.п.
		//поэтому надо найти кол-во строк, на которые диапазон сквозных строк сдвинется вверх
		
		var oSheet = this.aSheets[oRD.nSheet];
		if (!isEmpty(oRD.rowFrom) && !isEmpty(oRD.rowTo))
		{
			for (var nBandInd in oSheet.aBands)
			{
				if (oSheet.aBands[nBandInd].nStartRow <= oRD.rowFrom && oSheet.aBands[nBandInd].nEndRow >= oRD.rowFrom)
				{
					break;
				}
			}
			var rFrom = oRD.rowFrom-nBandInd;
			var rTo = oRD.rowTo-nBandInd;
		}
		else
		{
			var rFrom = -1;
			var rTo = -1;
		}
		if (!isEmpty(oRD.colFrom) && !isEmpty(oRD.colTo))
		{
			// Net 10.06.2011  никакие excel.js не надо включать! В случае шахматки, код, который здесь, выполняется
			// и включает excel. При последующих вызовах печати мы имеем перебитый Office_printExcel, который ниразу
			// не наш.
			// Стараемся избавляться от внешних зависимостей максимально.
			var cFrom = this.getCellNum(oRD.colFrom);
			var cTo = this.getCellNum(oRD.colTo);
		}
		else
		{
			var cFrom = -1;
			var cTo = -1;		
		}
		this.workBook.setRepeatingRowsAndColumns(oRD.nSheet, cFrom, cTo, (rFrom-1), (rTo-1));
	}
	
}

//получить настройку сквозных строк
//т.к. в POI 3.5 пока данная функциональность не реализована,
//то попытаемся воспользоваться подходом, предложенным в 
// http://java.itags.org/java-programming/140021/
function xlsWorkBook_getRepeatingData()
{
	var retVal = null;

	var wb  = this.workBook;
	var nameIndex = wb.getNameIndex("Print_Titles");
	if (nameIndex!=-1)
	{
		var name = wb.getNameAt(nameIndex);
		var sRef = name.getRefersToFormula();
		aRef = sRef.split(",");
		for (var i in aRef)
		{
			var sRef = aRef[i];
			//сейчас в sRef содержится что-то типа "Лист3!$A$7:$IV$10"
			//Нас интересует "7" и "10"
			var re = /^(\w+)\!\$(\w+)\$(\d+)\:\$(\w+)\$(\d+)/i
			var ret = re.exec(sRef)
			if (
				ret!=null
				&& typeof(ret[1])!="undefined"
				&& typeof(ret[3])!="undefined"
				&& typeof(ret[5])!="undefined"
			)
			{
				var sSheetName = ret[1];
				var nSheet = wb.getSheetIndex(wb.getSheet(sSheetName));
				if (nSheet!=-1)
				{
					var rowFrom = parseInt(ret[3], 10);
					var rowTo = parseInt(ret[5], 10);
					if (!isNaN(rowFrom) && !isNaN(rowTo))
					{
						if (isEmpty(retVal))
						{
							retVal = new Object();
						}
						retVal.nSheet = nSheet;
						retVal.rowFrom = rowFrom;
						retVal.rowTo = rowTo;
					}
				}
			}
			else
			{
				//у нас могут быть еще и столбцы
				var re = /^(\w+)\!(\w+)\:(\w+)/i
				var ret = re.exec(sRef)
				if (
					ret!=null
					&& typeof(ret[1])!="undefined"
					&& typeof(ret[2])!="undefined"
					&& typeof(ret[3])!="undefined"
				)
				{
					var sSheetName = ret[1];
					var nSheet = wb.getSheetIndex(wb.getSheet(sSheetName));
					if (nSheet!=-1)
					{
						var colFrom = ret[2];
						var colTo = ret[3];
						if (!isEmpty(colFrom) && !isEmpty(colTo))
						{
							if (isEmpty(retVal))
							{
								retVal = new Object();
							}
							retVal.nSheet = nSheet;
							retVal.colFrom = colFrom;
							retVal.colTo = colTo;
						}
					}
				}				
			}
		}
	}
	return retVal;
}

function xlsWorkBook_setRowsHeight(oSheet)
{
	this.aFonts = new Array();
	var aMerged = new Array();

	var hssfSheet = oSheet.oSheet[0];
	var tblRegion = getTmpTableName("", true);
	strSQL = "CREATE TABLE "+tblRegion
		+"( FBROW "+SqlType("LONG")
		+", FEROW "+SqlType("LONG")
		+", FBCOL "+SqlType("LONG")
		+", FECOL "+SqlType("LONG")
		+", FWIDTH "+SqlType("DOUBLE")
		+")"
	ExecuteSQL(strSQL);
	var kolReg = hssfSheet.getNumMergedRegions();
	for (var ii =0; ii<kolReg; ii++)
	{
		var oReg = hssfSheet.getMergedRegion(ii);
		var brow = oReg.getFirstRow();
		var erow = oReg.getLastRow();
		var bcol = oReg.getFirstColumn();
		var ecol = oReg.getLastColumn();
		var columnWidth = 0;
		for (var jj=bcol+1; jj<=ecol; jj++)
		{
			columnWidth += hssfSheet.getColumnWidth(jj)
		}
		strSQL = "INSERT INTO "+tblRegion+" (FBROW, FEROW, FBCOL, FECOL)"
			+" VALUES ("
			+brow
			+", "+erow
			+", "+bcol
			+", "+ecol
			+")"
		ExecuteSQL(strSQL);
	}
	strSQL = "SELECT DISTINCT FBCOL, FECOL FROM "+tblRegion
	var tReg = OpenTable("s", strSQL);
	for (; !tReg.IsEOF(); tReg.MoveNext())
	{
		var bcol = tReg.getValue("FBCOL");
		var ecol = tReg.getValue("FECOL");
		var columnWidth = 0;
		for (var jj=bcol; jj<=ecol; jj++)
		{
			columnWidth += hssfSheet.getColumnWidth(jj)
		}
		strSQL = "UPDATE "+tblRegion
			+" SET FWIDTH = "+columnWidth
			+" WHERE FBCOL = "+bcol
			+" AND FECOL = "+ecol
		ExecuteSQL(strSQL);
	}
	tReg.close();
	tReg = null;

	tReg = OpenTable("s","SELECT * FROM "+tblRegion+" ORDER BY FBROW, FEROW, FBCOL, FECOL");
	if (tReg.recordCount>5000)
	{
		aMerged = tblRegion;
	}
	else
	{
		for (; !tReg.IsEOF(); tReg.MoveNext())
		{
			/*if (aMerged[tReg.getValue("FBROW")] == null)
			{
				aMerged[tReg.getValue("FBROW")] = new Array();
			}
			if (aMerged[tReg.getValue("FBROW")][tReg.getValue("FEROW")] == null)
			{
				aMerged[tReg.getValue("FBROW")][tReg.getValue("FEROW")] = new Array();
			}
			if (aMerged[tReg.getValue("FBROW")][tReg.getValue("FEROW")][tReg.getValue("FBCOL")] == null)
			{
				aMerged[tReg.getValue("FBROW")][tReg.getValue("FEROW")][tReg.getValue("FBCOL")] = new Array();
			}
			
			aMerged[tReg.getValue("FBROW")][tReg.getValue("FEROW")][tReg.getValue("FBCOL")][tReg.getValue("FECOL")] = toScrObj(tReg.getValue("FWIDTH"));
			*/
			var obj = {};
			obj.brow = toScrObj(tReg.getValue("FBROW"));
			obj.erow = toScrObj(tReg.getValue("FEROW"));
			obj.bcol = toScrObj(tReg.getValue("FBCOL"));
			obj.ecol = toScrObj(tReg.getValue("FECOL"));
			obj.width = toScrObj(tReg.getValue("FWIDTH"));
			aMerged[aMerged.length] = obj;
		}
	}
	tReg.close();
	tReg = null;
	createIndex(tblRegion, "INDREG", "FBROW, FEROW, FBCOL, FECOL");

	var minRowNum = hssfSheet.getFirstRowNum();
	if (!isEmpty(oSheet.printFirstTableRow))
	{
		minRowNum = oSheet.printFirstTableRow;
	}
	else
	{
		for (var ind in oSheet.aBands)
		{
			if (objTypeOf(oSheet.aBands[ind]) == "xlsBandTable")
			{
				minRowNum = oSheet.aBands[ind].nStartRow;
				if (!isEmpty(oSheet.aBands[ind].bandHeader))
				{
					minRowNum = oSheet.aBands[ind].bandHeader.nEndRow+1;
				}
				else if (!isEmpty(oSheet.aBands[ind].aBandRows) && !isEmpty(oSheet.aBands[ind].aBandRows[0]))
				{
					minRowNum = oSheet.aBands[ind].aBandRows[0].nStartRow-1;
				}
				break;
			}
		}
	}
	
	if (minRowNum == hssfSheet.getFirstRowNum())
	{
		//меняем только там, где есть таблица
		return true;
	}

	// Net  приготовились.. поехали..
	var sh = new Packages.org.eclipse.swt.widgets.Shell();
	var disp = new Packages.org.eclipse.swt.widgets.Display.getDefault();
	var text = new Packages.org.eclipse.swt.widgets.Text(sh, Packages.org.eclipse.swt.SWT.NONE);
	var gc = new Packages.org.eclipse.swt.graphics.GC(text);

	for (var i = minRowNum;i<=hssfSheet.getLastRowNum();i++)
	{
		var row = hssfSheet.getRow(i);
		if (row == null)
		{
			continue;
		}

		var origRowHeight = 0//row.getHeightInPoints();
		var maxHeight = 12.75//origRowHeight;
		for (var j = row.getFirstCellNum();j<row.getLastCellNum();j++)
		{
			var cell = row.getCell(j);
			if (cell != null && cell.getCellStyle().getWrapText() && inList(cell.getCellType(),cell.CELL_TYPE_STRING,cell.CELL_TYPE_BLANK))
			{
				var val = this.getCellValue(cell);
				if (!isEmpty(val))
				{
					rHeight = this.calculateRowHeight(hssfSheet, i, j, cell, gc, disp, aMerged, origRowHeight);

					maxHeight = Math.max(maxHeight, rHeight);
				}
			}
		}
		
		row.setHeightInPoints(maxHeight);
		
		//fruckt
		if (oSheet.oSheet.length > 1)
		{
			for (var nSht = 1; nSht < oSheet.oSheet.length; nSht++)
			{
				var rowNextList = oSheet.oSheet[nSht].getRow(i);
				if (rowNextList != null)
				{
					rowNextList.setHeightInPoints(maxHeight);
				}
			}
		}
	}
	DropTable(tblRegion);

	sh = null;
	disp = null;
	text = null;
	gc = null;
}

function xlsWorkBook_getCellNum(name)
{
	var CONST_ALPHABET = new String("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
	var allNum;
	if (isEmpty(name))
	{
		allNum = -1;
		return allNum;
	}
	var allNum = 0;
	for (var i=0; i<name.length; i++)
	{
		var ch = name[i];
		var num = (name.length > 1 && i == 0 ? (CONST_ALPHABET.indexOf(ch)+1)* CONST_ALPHABET.length : CONST_ALPHABET.indexOf(ch))
		allNum += num;
	}
	return allNum;
}

function xlsWorkBook_findMergedWidth(arr, row, col)
{
	if (objTypeOf(arr)=="string")
	{
		strSQL = "SELECT FWIDTH FROM "+arr
			+" WHERE FBROW <= "+row
			+" AND FEROW >= "+row
			+" AND FBCOL <= "+col
			+" AND FECOL >= "+col
		var sn = getSnap(strSQL);
		if (sn != null && sn[0] != null)
		{
			var retVal = sn[0];
			sn = null;
			return retVal;
		}
		sn = null;
		return -1;
	}
	else
	{
		//может нужно не массив объектов, 4-х мерный массив, но оссобой разницы по скорости вроде не видно
		/*//4-х мерный массив с шириной
		//бежим по массиву, для brow, bcol - начинаем искать снизу, для erow, ecol - сверху
		if (arr[row]!=null)
		{
			if (arr[row][row]!=null)
			{
				if (arr[row][row][col]!=null)
				{
					for (var n in arr[row][row][col])
					{
						if (n>=col)
						{
							return arr[row][row][col][n];
						}
					}
					return -1;
				}
				else
				{
					for (l in arr[row][row])
					{
						if (l>col)
						{
							return -1;
						}
						break;
					}
					for (var l=arr[row][row].length-1; l<=0;l--)
					{
						if (l<=col)
						{
							for (var n in arr[row][row][l])
							{
								if (n>=col)
								{
									return arr[row][row][col][n];
								}
							}
							return -1;
						}
					}
				}
			}
			else
			{
				for (var j in arr[row])
				{
					if (j>=row)
					{
						if (arr[row][j][col]!=null)
						{
							for (var n in arr[row][j][col])
							{
								if (n>=col)
								{
									return arr[row][j][col][n];
								}
							}
							return -1;
						}
						else
						{
							for (l in arr[row][j])
							{
								if (l>col)
								{
									return -1;
								}
								break;
							}
							for (var l=arr[row][j].length-1; l<=0;l--)
							{
								if (l<=col)
								{
									for (var n in arr[row][j][l])
									{
										if (n>=col)
										{
											return arr[row][j][col][n];
										}
									}
									return -1;
								}
							}
						}
					}					
				}
			}
		}
		else
		{
			for (i in arr)
			{
				if (i>row)
				{
					return -1;
				}
				break;
			}
			for (var i=arr.length-1; i<=0;i--)
			{
				if (i<=row)
				{
					if (arr[i][row]!=null)
					{
						if (arr[i][row][col]!=null)
						{
							for (var n in arr[i][row][col])
							{
								if (n>=col)
								{
									return arr[i][row][col][n];
								}
							}
							return -1;
						}
						else
						{
							for (l in arr[i][row])
							{
								if (l>col)
								{
									return -1;
								}
								break;
							}
							for (var l=arr[i][row].length-1; l<=0;l--)
							{
								if (l<=col)
								{
									for (var n in arr[i][row][l])
									{
										if (n>=col)
										{
											return arr[i][row][col][n];
										}
									}
									return -1;
								}
							}
						}
					}
					else
					{
						for (var j in arr[i])
						{
							if (j>=row)
							{
								if (arr[i][j][col]!=null)
								{
									for (var n in arr[i][j][col])
									{
										if (n>=col)
										{
											return arr[i][j][col][n];
										}
									}
									return -1;
								}
								else
								{
									for (l in arr[i][j])
									{
										if (l>col)
										{
											return -1;
										}
										break;
									}
									for (var l=arr[i][j].length-1; l<=0;l--)
									{
										if (l<=col)
										{
											for (var n in arr[i][j][l])
											{
												if (n>=col)
												{
													return arr[i][j][col][n];
												}
											}
											return -1;
										}
									}
								}
							}
						}
					}
				}
			}
		}*/
		//fruckt
		if (toScrObj(objTypeOf(arr)).toLowerCase() == "array")
		{
			for (var i in arr)
			{
				var rec = arr[i];
				if (rec.brow <= row && rec.erow >= row &&
					rec.bcol <= col && rec.ecol >= col)
				{
					return rec.width;
				}
			}
		}
		else
		{
			var tReg = OpenTable("SELECT * FROM "+arr+" ORDER BY FBROW, FEROW, FBCOL, FECOL");
			for (; !tReg.IsEOF(); tReg.MoveNext())
			{
				if (toScrObj(tReg.getValue("FBROW")) <= row
					&& toScrObj(tReg.getValue("FEROW")) >= row
					&& toScrObj(tReg.getValue("FBCOL")) <= col
					&& toScrObj(tReg.getValue("FECOL")) >= col)
				{
					var width = toScrObj(tReg.getValue("FWIDTH"));
					tReg.Close();
					tReg = null;
					
					return width;
				}
			}
			tReg.Close();
			tReg = null;
		}
		return -1;
	}
}

function xlsWorkBook_textExtent(cell, value, fontIdx, gc, disp)
{
	if (typeof(this.aFonts[fontIdx]) == "undefined")
	{
		var wb_fnt = this.workBook.getFontAt(fontIdx);
		var fntName = wb_fnt.getFontName();
		var fntPoints = wb_fnt.getFontHeightInPoints();
		// может у кого-то получится заюзать тут эту константу
		var fBold = (wb_fnt.getBoldweight() == 0x2bc);		// public final static short BOLDWEIGHT_BOLD = 0x2bc; (java\org\apache\poi\hssf\usermodel\HSSFFont.java)
		var fItalic = wb_fnt.getItalic();

		var styleFlag = Packages.org.eclipse.swt.SWT.NORMAL;
		styleFlag = (fBold==true)
			? styleFlag | Packages.org.eclipse.swt.SWT.BOLD
			: styleFlag;
		styleFlag = (fItalic==true)
			? styleFlag | Packages.org.eclipse.swt.SWT.ITALIC
			: styleFlag;
		var fnt = new Packages.org.eclipse.swt.graphics.Font(disp, fntName, fntPoints, styleFlag);

		this.aFonts[fontIdx] = fnt;
	}

	gc.setFont(this.aFonts[fontIdx]);
	var retVal = gc.textExtent(value);
	return retVal;
}

function xlsWorkBook_calculateRowHeight(hssfSheet, rowNum, cellNum, cell, gc, disp, aMerged, origRowHeight)
{
	var val = this.getCellValue(cell);

	// http://apache-poi.1045710.n5.nabble.com/Bug-of-HSSFPicture-resize-td2300077.html
	/*
	Yegor Kozlov wrote:
	I hope my belated answer is of help 

	HSSFPicture.resize() as well as XSSFPicture.resize() indeed work only for the default font which is Arial 10pt for .xls 
	and Calibri 11pt for .xlsx. 

	Excel uses a funny coordinate system to position graphic objects. X and Y axes  are measured not in pixels but in units 
	proportional to a character width of the default font. To properly position and size an image POI needs to translate the 
	actual dimensions measured in pixels to columns and rows. This is the key of the problem. At the moment POI performs 
	this translation using hardcoded constants obtained empirically for Arial 10pt and Calibri 11pt. It works OK in most 
	cases and also explains why HSSFPicture.resize() is 'off' if the default font is changed.
	*/
	// поэтому, ширина колонки в 1/256-ых ширины символа "a":
	//		на гарнитуре Arial,   10-й кегль в .xls
	//		на гарнитуре Calibri, 11-й кегль в .xlsx
	var columnWidth = hssfSheet.getColumnWidth(cellNum);
	//если ячейка объединенная, то нужно взять ширину всех ячеек
	var mergedWidth = xlsWorkBook_findMergedWidth(aMerged, rowNum, cellNum);
	columnWidth = (mergedWidth == -1)
		? columnWidth
		: mergedWidth;

	// http://www.mailinglistarchive.com/poi-user@jakarta.apache.org/msg05145.html
	/*
	The minimal steps to calculate width of a cell in pixels are as follows:

	- get default workbook font. HSSFWorkbook.getFontAt(0) should always return the default one.
	- get default cell width expressed in 1/256th units. See HSSFSheet.getColumnWidth
	- Assuming that the default character is "a" compute the width of a surrogate string using Java2D classes.
	*/

	// следуя из этого, получаем дефолтный шрифт...
	var extent_of_a = this.textExtent(cell, "a", 0, gc, disp);
	var columnWidthPx = Math.round(columnWidth / 256 * extent_of_a.x, 0);
	trace("ширина колонки в попугаях: "+columnWidth+"; в пикселях: "+columnWidthPx)

	// пытаемся..
	/*
	rHeightLines = Math.ceil(extent.x / columnWidthPx);
	var rHeight = rHeightLines * origRowHeight;
	*/

	// все бы ничего, но высоту расчитывать нельзя на предположении, что ячейка заполнена текстом
	// во всех строчках одинаково плотно до самого упора.. это довольно редкий случай, и чем больше строк, тем он невероятнее.
	// поэтому разбираем строку на слова (знак "-" (дефис) для экселя тоже как слово,
	// он разбивает несколько идущих подряд, если надо перенести часть текст на следующую строку).
	// также необходимо учесть случай, когда неразбиваемый кусок (напр. длинное слово или узкая ячейка) не поместить в ячейке
	// ну никак. тогда эксель его режет, втискивая в одну строчку до упора, а на следующую - все, что осталось.

	// cell indentation в текущей версии не поддерживается (ни на одной форме тоже не замечен).

	// в extent.x у нас сейчас длина текста в пикселях, которую надо распределить по строкам
	// с учетом переноса по словам.
	// возможен перебор с высотой в случае, если для ячейки, в которой дофига текста, установлен кегль меньше, чем в других.
	// но в основном все формы идут с одинаковым размером шрифта по всей строке. это радует.
	var aWords = new Array();
	while ((aRes = /[^ \t\-]+|[ \t]+|-/.exec(val)) != null)
	{
		var nPos = aRes.index;
		val = String(val);
		val = val.substr(nPos+aRes[0].length);	// следующее слово
		aWords[aWords.length] = aRes[0];
	}

	var fontIdx = cell.getCellStyle().fontIndex;

	var wb_fnt = this.workBook.getFontAt(fontIdx);
	var spacing = Math.ceil(wb_fnt.getFontHeightInPoints() / 5);	// эмпирически

	columnWidthPx -= 2*spacing;	// предусматриваем spacing с обоих сторон ячейки
//	var rowHeight = 1;			// изначально высота в одну строку текста
//	var oneRowHeight = this.textExtent(cell, valOrig, fontIdx, gc, disp).y;
	//oneRowHeight = Math.max(oneRowHeight, ext.y);	
	var aLen = new Array();
	var sExpr = "";

	if (aWords.length != 0)
	{
		var sWord = aWords[0];
		var currLineFill = 0;
		var idx = 0;
		var antiCycle = 0;
		var lastIf = -1;
		while (idx < aWords.length)
		{
			antiCycle++;
			if (antiCycle == 100)
			{
				//——————————————
				//alert("oops!");
				//——————————————
				// на первых порах защита от зацикливания
				break;
			}
			var sWord = aWords[idx++];
			var ext = this.textExtent(cell, sWord, fontIdx, gc, disp);
//			oneRowHeight = Math.max(oneRowHeight, ext.y);
			// наше слово, или кусок, который выглядит как одно слово, не вписывается в ячейку
			// и перенести его нельзя. будем резать, чтоб максимально влазило в ячейку.
			if (ext.x >= columnWidthPx)
			{
				if (lastIf == 0)
				{
					//rowHeight++;	// поехали втискивать с новой строчки
					aLen.push(sExpr);
					sExpr = "";
				}
				var sWordPart = "";
				var i = 0;
				var part_ext;
				while (i < sWord.length)
				{
					part_ext = this.textExtent(cell, sWordPart + sWord.charAt(i), fontIdx, gc, disp);
					if (part_ext.x < columnWidthPx)
					{
						sWordPart += sWord.charAt(i++);
					}
					else
					{
						break;
					}
				}
				idx--;
				currLineFill = 0;
				aLen.push(sWordPart);
				// отрежем вписанный кусок от слова и попробуем это слово вписать еще раз
				aWords[idx] = String(sWord).substr(i);
				lastIf = 1;
			}
			// перебрали с набором слов, отбросим последнее слово и нарастим строку
			else if ((currLineFill + ext.x >= columnWidthPx) && (sWord.trim() != ""))
			{
				//rowHeight++;
				idx--;
				currLineFill = 0;
				aLen.push(sExpr);
				sExpr = "";
				lastIf = 0;
				continue;
			}
			// все влезло. наращиваем дальше
			else
			{
				/*if (lastIf == 1)
				{
					rowHeight++;
				}*/
				currLineFill += ext.x;
				sExpr += sWord;
				lastIf = 0;
			}
		}
		aLen.push(sExpr);
	}
	sExpr = "";
	for (var ii in aLen)
	{
		sExpr += aLen[ii] + "\n";
	}
	sExpr = String(sExpr).substr(0, sExpr.length-2);
	var retVal = (this.textExtent(cell, sExpr, fontIdx, gc, disp).y) * 3/4+wb_fnt.getFontHeightInPoints()/2;
	return retVal;
	//return rowHeight*12.75//oneRowHeight*0.75;
	//return rowHeight * wb_fnt.getFontHeightInPoints()//origRowHeight;
}

/**
метод для конвертации формы в формат jrxml;
Методика позаимствована в DpPSFDecoder.js
@param oPar необязательный параметр. Может иметь поля:
	.builder
	.designe
*/
function xlsWorkBook_convert2Jrxml(oPar)
{
	if (typeof(oPar)=="undefined")
	{
		oPar = new Object();
	}

	//обеспечим возможность передать builder снаружи
	if (typeof(oPar.builder)=="undefined")
	{
		oPar.builder = lpProxy.getJRBuilder();
	}

	//обеспечим возможность передать designe снаружи
	if (typeof(oPar.designe)=="undefined")
	{
		oPar.designe = oPar.builder.getDesigne();
	}

	
	//Если в файле больше одного листа, то спросим, какой конвертировать
	if (this.aSheets.length > 1)
	{
		var nSheet = 0;
		var sOptions = "";
		for (var nIndex in this.aSheets)
		{
			sOptions += "|лист № " + this.aSheets[nIndex].nSheetNumber;
		}
		sOptions = sOptions.substr(1);
		
		include("Objects/DpAskEx.js");
		var oA = new DpAskEx();
		oA.add("RADIO", "Конвертувати:|Конвертировать:", "SHEETNO", 0, sOptions);
		if (!oA.doAsk())
		{
			return false;
		}
		var oSheet = this.aSheets[oA.get("SHEETNO")];
		oSheet.convert2Jrxml(oPar);
	}
}

