﻿// Шатохин В.В.
// Процедура расчета закрытия счетов по способу DOOC
// 11.03.2004
// "Дооценка до фактической себестоимости"
//
// Назначение.
// При оприходовании продукции или услуг собственного производства
// затраты на данную продукцию накапливаются по Дт соотв. затратного счета,
// а оприходование идет с его кредита.
// Если к затратному счету привязана аналитика по видам затрат, то 
// оприходование ведется с аналитики "продукция собственная",
// а затраты - в разрезе "м-лы", "услуги стор. орг." , "Зарплата" и т.д.
// С кредита продукция приходуется по плановым ценам,
// а в дебете набираются фактические затраты.
// При закрытии данного затратного счета необходимо фактические затраты закрыть по 
// направлениям использования продукции.
// Если бы не было аналитики, то можно было бы просто сальдо по дебету распределить
// по направлениям использования.
// При наличии аналитики такое не проходит.
// РЕШЕНИЕ.
// Сторнируем проводки с кредита затратного счета (те, что с аналитикой "прод. собственная")
// и делаем черным проводки с кредита затратного счета, но с реальными аналитиками.
// Закрывать можем или "старую" сумму (ту, что была с аналитикой "прод. собственная"),
// или всю, собравшуюся в дебете затратного счета.
// 


//include("objects\\DpEval.js");

// Константы для поля FKIND
// (переехали из e_zak.htm)
var KIND_SL = 1;	//брать остаток
var KIND_OB = 2;	//брать оборот
var KIND_DB = 4;	//дебетовый
var KIND_CR = 8;	//кредитовый
var KIND_TURN = 16;	//сворачивать
var KIND_NOTURN = 32;	//сворачивать
var KIND_ZDB = 64;	//Закрываемый счет - по Дт
var KIND_ZCR = 128;	//Закрываемый счет - по Кт
var KIND_IGNORE_CL1 = 256;	// по всем аналитикам 1
var KIND_IGNORE_CL2 = 512;	// по всем аналитикам 2
var KIND_IGNORE_CL3 = 1024;	// по всем аналитикам 3
var KIND_ZERO_SL = 2048;	// Закрыть полностью (сальдо после закрытия==0)
var KIND_DEFAULT = KIND_SL | KIND_DB | KIND_TURN | KIND_ZCR;

function r_zrdooc(par, dat1, dat2)
{
	mmFsc     = par.mmFsc;
	mmFsu     = par.mmFsu;
	mmFst     = par.mmFst;
	mmFcl1    = par.mmFcl1;
	mmFcl2    = par.mmFcl2;
	mmFcl3    = par.mmFcl3;
	//mmFgroup = par.mmFgroup;
	//mmFexpr = par.mmFexpr;
	mmFproc = par.mmFproc;
	mmFkind = par.mmFkind;

	mmFexcl1 = par.mmFexcl1;
	mmFexcl2 = par.mmFexcl2;
	mmFexcl3 = par.mmFexcl3;
	
	//Разберемся с тем, что будем закрывать
	var isSL= mmFkind & KIND_SL;	//брать сальдо или обороты
	var isDB= mmFkind & KIND_DB;    //дебетовые или кредитовые
	//var isTurn= mmFkind & KIND_TURN;//сворачивать или не сворачивать
	//var isZDB= mmFkind & KIND_ZDB;  //Закр.счет - по Дт или по Кт

	var isIgn1 = mmFkind & KIND_IGNORE_CL1;	// по всем аналитикам 1
	var isIgn2 = mmFkind & KIND_IGNORE_CL2;	// по всем аналитикам 2
	var isIgn3 = mmFkind & KIND_IGNORE_CL3;	// по всем аналитикам 3

	var isZeroSl = mmFkind & KIND_ZERO_SL;	// Закрыть полностью (сальдо после закрытия==0)
	
	//////////////////////////////////////////////////////////
	// Запомним номера справочников аналитики,
	// привязанных к счету, который закрываем
	tcl_schet = OpenTable("s","CL_SCHET","U0");
	// номера спр-ков аналитик счета, к-й закрываем
	var mCl1=0, mCl2=0, mCl3=0;
	
	// выберем из плана счетов
	strSQL = "SELECT FCL1, FCL2, FCL3 FROM ^CL_SCHET "
			+" WHERE FSCHET = "+sqlTo(mmFsc)+" AND FSUBSCHET = "+sqlTo(mmFsu)+" AND FSTAT = "+sqlTo(mmFst)
	var sn = getSnap(strSQL);
	if (sn != null && !isEmpty(sn[0]))
	{
		mCl1 = Number(sn[0]);
		mCl2 = Number(sn[1]);
		mCl3 = Number(sn[2]);
	}
	sn = null;
	//////////////////////////////////////////////////////////

	//Создадим таблицу с проводками по интересующему нас счету
	var tDOOC_SC1 = getTmpTableName("", true);
	
	// Сначала выбираем проводки
	// которые будем сторнировать, а потом "черным" расшифровывать
	strSQL = "SELECT "
	+" S.FSUM"
	+", S.FDB_SC"
	+", S.FDB_SU"
	+", S.FDB_ST"
	+", S.FDB_COD1"
	+", S.FDB_COD2"
	+", S.FDB_COD3"
	+", S.FCR_SC"
	+", S.FCR_SU"
	+", S.FCR_ST"
	+", S.FCR_COD1"
	+", S.FCR_COD2"
	+", S.FCR_COD3"
	+", S.FVAL"
	
	//fruckt
	//если необходимы параметры документов
	//приложение 10 к кошторису
	if (par.isDoc == true)
	{
		strSQL += ""
		+", S.FDOC"
		+", S.FID_DOC"
		+", S.FDOCNUM"
		+", S.FDATE"
		+", S.FNOP"
	}
	
	strSQL += ""
	+" INTO "+tDOOC_SC1
	
	//fruckt
	if (isEmpty(par.buffName))
	{
		strSQL += ""
		+" FROM ^SCHET S"
	}
	else
	{
		strSQL += ""
		+" FROM "+par.buffName+" S"
	}
	
	strSQL += ""
	+" WHERE "+SqlCmpTrue
		+" AND S.FDATE BETWEEN "+sqlTo(dat1)+" AND "+sqlTo(dat2)
		+" AND "+(isDB ? "S.FDB_SC" : "S.FCR_SC" )+"="+sqlTo(mmFsc)
		+" "+sqlw0And((isDB ? "S.FDB_SU" : "S.FCR_SU" )+"=", mmFsu)
		+" "+sqlw0And((isDB ? "S.FDB_ST" : "S.FCR_ST" )+"=", mmFst)
		+" "+sqlw0And((isDB ? "S.FDB_COD1" : "S.FCR_COD1" )+" = ", mmFcl1)
		+" "+sqlw0And((isDB ? "S.FDB_COD2" : "S.FCR_COD2" )+" = ", mmFcl2)
		+" "+sqlw0And((isDB ? "S.FDB_COD3" : "S.FCR_COD3" )+" = ", mmFcl3)
		+" AND NOT S.FWRK IN (254, 16)"
	ExecuteSQL(strSQL);
			
	//Подтянем закрывающие проводки
	//fruckt
	//для приложения 10 к кошторису
	//if (op_zakr())
	if (op_zakr() && (isEmpty(par.isDoc) || par.isDoc != true))
	{
		strSQL = "INSERT INTO "+tDOOC_SC1
			+" SELECT "
			+" S.FSUM"
			+", S.FDB_SC"
			+", S.FDB_SU"
			+", S.FDB_ST"
			+", S.FDB_COD1"
			+", S.FDB_COD2"
			+", S.FDB_COD3"
			+", S.FCR_SC"
			+", S.FCR_SU"
			+", S.FCR_ST"
			+", S.FCR_COD1"
			+", S.FCR_COD2"
			+", S.FCR_COD3"
			+", S.FVAL"
		+" FROM"
			+" ^schet_zr S"
		+" WHERE "+SqlCmpTrue
			+" AND S.FDATE BETWEEN "+sqlTo(dat1)+" AND "+sqlTo(dat2)
			+" AND "+(isDB ? "S.FDB_SC" : "S.FCR_SC" )+"="+sqlTo(mmFsc)
			+" "+sqlw0And((isDB ? "S.FDB_SU" : "S.FCR_SU" )+"=", mmFsu)
			+" "+sqlw0And((isDB ? "S.FDB_ST" : "S.FCR_ST" )+"=", mmFst)
			+" "+sqlw0And((isDB ? "S.FDB_COD1" : "S.FCR_COD1" )+" = ", mmFcl1)
			+" "+sqlw0And((isDB ? "S.FDB_COD2" : "S.FCR_COD2" )+" = ", mmFcl2)
			+" "+sqlw0And((isDB ? "S.FDB_COD3" : "S.FCR_COD3" )+" = ", mmFcl3)
			+" AND NOT S.FWRK IN (254, 16)"
		ExecuteSQL(strSQL);
	}
	
	//Создадим таблицу с оборотами по интересующему нас счету
	var tDOOC_OBR = getTmpTableName("", true);
	
	strSQL = "SELECT "
		+" SUM(S.FSUM) AS FSUM"
		+", S.FDB_SC"
		+", S.FDB_SU"
		+", S.FDB_ST"
		+", S.FDB_COD1"
		+", S.FDB_COD2"
		+", S.FDB_COD3"
		+", S.FCR_SC"
		+", S.FCR_SU"
		+", S.FCR_ST"
		+", S.FCR_COD1"
		+", S.FCR_COD2"
		+", S.FCR_COD3"
		+", S.FVAL"
	
	//fruckt
	//если необходимы параметры документов
	//приложение 10 к кошторису
	if (par.isDoc == true)
	{
		strSQL += ""
		+", S.FDOC"
		+", S.FID_DOC"
		+", S.FDOCNUM"
		+", S.FDATE"
		+", S.FNOP"
	}
	
	strSQL += ""
	+" INTO "+tDOOC_OBR
	+" FROM "+tDOOC_SC1+" S"
	+" GROUP BY "
		+" S.FDB_SC"
		+", S.FDB_SU"
		+", S.FDB_ST"
		+", S.FDB_COD1"
		+", S.FDB_COD2"
		+", S.FDB_COD3"
		+", S.FCR_SC"
		+", S.FCR_SU"
		+", S.FCR_ST"
		+", S.FCR_COD1"
		+", S.FCR_COD2"
		+", S.FCR_COD3"
		+", S.FVAL"

	//fruckt
	//если необходимы параметры документов
	//приложение 10 к кошторису
	if (par.isDoc == true)
	{
		strSQL += ""
		+", S.FDOC"
		+", S.FID_DOC"
		+", S.FDOCNUM"
		+", S.FDATE"
		+", S.FNOP"
	}
	
	strSQL += ""
	+" HAVING SUM(S.FSUM)<>0"
	ExecuteSQL(strSQL);
	
	// Расчитываем остатки по счету, к-й закрываем
	// (расчет делаем до того, как начнем что-либо писать в SCHET_ZR)
	TrnNBln("3n", "OBSL", dat1, dat2, new DpSc(mmFsc,mmFsu,mmFst));
	
	//сюда будем писать результаты:
	var buffName = getTmpTableName("", true);
	CreateTable(buffName, "SCHET", "FWID");

	strSQL = " ALTER TABLE "+buffName+" ADD FWID_CTR "+SqlType("COUNTER")
	ExecuteSQL(strSQL);

	createIndex(buffName, "IX_FWID", "FWID");

	// сюда будем писать результаты:
	//tz = OpenTable("s", "SCHET_ZR");

	//vlad 29.04.2009
	//Решаем проблему: Если у нас код,
	//который будем сторнировать, на уровне 2 или 3,
	//то надо все делать в разрезе аналитик 1-го и других уровней, на которых стоит
	//галочка "любая"
	//Поэтому начиная с этого места создадим вспомогательную таблицу, в которую выберем
	//все строки, которые "не любые" и находятся на уровень "выше" аналитики,
	//которую сторнируем.
	//Поэтому делаем цикл.
		
	var tObr = OpenTable("SELECT * FROM "+tDOOC_OBR);
	if(tObr.IsEmpty())
	{
		// Нечего закрывать - выходим
		tObr.close();
		tObr = null;
		
		DropTable(tDOOC_SC1);
		DropTable(tDOOC_OBR);
		
		return;
	}
	else
	{
		var fsc = isDB ? "FDB_SC" : "FCR_SC";
		var fsu = isDB ? "FDB_SU" : "FCR_SU";
		var fst = isDB ? "FDB_ST" : "FCR_ST";
		var fcl1 = isDB ? "FDB_COD1" : "FCR_COD1";
		var fcl2 = isDB ? "FDB_COD2" : "FCR_COD2";
		var fcl3 = isDB ? "FDB_COD3" : "FCR_COD3";
		
		//обрабатываем каждую корреспонденцию отдельно
		for(tObr.MoveFirst(); !tObr.IsEOF(); tObr.MoveNext())
		{			
			var tObr_FCR_SC = toScrObj(tObr.getValue("FCR_SC"));
			var tObr_FCR_SU = toScrObj(tObr.getValue("FCR_SU"));
			var tObr_FCR_ST = toScrObj(tObr.getValue("FCR_ST"));
			var tObr_FCR_COD1 = toScrObj(tObr.getValue("FCR_COD1"));
			var tObr_FCR_COD2 = toScrObj(tObr.getValue("FCR_COD2"));
			var tObr_FCR_COD3 = toScrObj(tObr.getValue("FCR_COD3"));
			var tObr_FDB_SC = toScrObj(tObr.getValue("FDB_SC"));
			var tObr_FDB_SU = toScrObj(tObr.getValue("FDB_SU"));
			var tObr_FDB_ST = toScrObj(tObr.getValue("FDB_ST"));
			var tObr_FDB_COD1 = toScrObj(tObr.getValue("FDB_COD1"));
			var tObr_FDB_COD2 = toScrObj(tObr.getValue("FDB_COD2"));
			var tObr_FDB_COD3 = toScrObj(tObr.getValue("FDB_COD3"));
			
			// 1. Формируем сторнирующие проводки
			var strSQL = "INSERT INTO "+buffName
			+"("
				+" Ftxt"
				+",Fdb_sc"
				+",Fdb_su"
				+",Fdb_st"
				+",Fdb_cod1"
				+",Fdb_cod2"
				+",Fdb_cod3"
				+",Fcr_sc"
				+",Fcr_su"
				+",Fcr_st"
				+",Fcr_cod1"
				+",Fcr_cod2"
				+",Fcr_cod3"
				+",Fsum"
				+",Fval"
				+",Fsumspr"
				+",Fvalspr"
				+",Fdocvid"
				+",Fwrk"
				+",Fuser"
				+",Fdate"
				+",Fdocnum"
				
				//fruckt
				//если необходимы параметры документов
				//приложение 10 к кошторису
				if (par.isDoc == true)
				{
					strSQL += ""
					+", FDOC"
					+", FID_DOC"
					+", FNOP"
				}
			
			strSQL += ""
			+") SELECT "
				+sqlTo("Закрывающая проводка DOOC, сторно")+" AS Ftxt"
				+",O.Fdb_sc"
				+",O.Fdb_su"
				+",O.Fdb_st"
				+",O.Fdb_cod1"
				+",O.Fdb_cod2"
				+",O.Fdb_cod3"
				+",O.Fcr_sc"
				+",O.Fcr_su"
				+",O.Fcr_st"
				+",O.Fcr_cod1"
				+",O.Fcr_cod2"
				+",O.Fcr_cod3"
				+",-O.Fsum AS FSUM"
				+",O.Fval"
				+","+sqlTo(0)+" AS Fsumspr"
				+","+sqlTo(0)+" AS Fvalspr"
				+","+sqlTo("закр.")+" AS Fdocvid"
				+","+sqlTo(15)+" AS Fwrk"
				+","+sqlTo(getUser())+" AS Fuser"

				//fruckt
				//если необходимы параметры документов
				//приложение 10 к кошторису
				if (par.isDoc == true)
				{
					strSQL += ""
					+", O.FDATE"
					+", O.FDOCNUM"
					+", O.FDOC"
					+", O.FID_DOC"
					+", O.FNOP"
				}
				else
				{
					strSQL += ""
					+","+sqlTo(dat2)+" AS Fdate"
					+","+sqlTo("12345")+" AS Fdocnum"
				}
			
			strSQL += ""
			+" FROM "+tDOOC_OBR+" O"
			+" WHERE "+SqlCmpTrue
				+" AND O.FCR_SC=" + sqlTo(tObr_FCR_SC)
				+" AND O.FCR_SU=" + sqlTo(tObr_FCR_SU)
				+" AND O.FCR_ST=" + sqlTo(tObr_FCR_ST)
				+" AND O.FCR_COD1=" + sqlTo(tObr_FCR_COD1)
				+" AND O.FCR_COD2=" + sqlTo(tObr_FCR_COD2)
				+" AND O.FCR_COD3=" + sqlTo(tObr_FCR_COD3)
			 
				+" AND O.FDB_SC=" + sqlTo(tObr_FDB_SC)
				+" AND O.FDB_SU=" + sqlTo(tObr_FDB_SU)
				+" AND O.FDB_ST=" + sqlTo(tObr_FDB_ST)
				+" AND O.FDB_COD1=" + sqlTo(tObr_FDB_COD1)
				+" AND O.FDB_COD2=" + sqlTo(tObr_FDB_COD2)
				+" AND O.FDB_COD3=" + sqlTo(tObr_FDB_COD3)
				
				//fruckt
				if (par.isDoc == true)
				{
					strSQL += ""
					+" AND O.FDATE = "+sqlTo(toScrObj(tObr.getValue("FDATE")))
					+" AND O.FDOCNUM = "+sqlTo(toScrObj(tObr.getValue("FDOCNUM")))
					+" AND O.FDOC = "+sqlTo(toScrObj(tObr.getValue("FDOC")))
					+" AND O.FID_DOC = "+sqlTo(toScrObj(tObr.getValue("FID_DOC")))
					+" AND O.FNOP = "+sqlTo(toScrObj(tObr.getValue("FNOP")))
				}
			ExecuteSQL(strSQL);
			
			// Net 13.01.2012  фильтр для закрытия текущего набора аналитик
			// без учета верхних. например Ан 3. = 200, ан. 1 и 2 игнорируются но закрывать надо только их			
			strSQL = " SELECT"
				+" O.Fdb_cod1"
				+",O.Fdb_cod2"
				+",O.Fdb_cod3"
				+",O.Fcr_cod1"
				+",O.Fcr_cod2"
				+",O.Fcr_cod3"
			+" FROM "+tDOOC_OBR+" O"
			+" WHERE "+SqlCmpTrue
				+" AND O.FCR_SC=" + sqlTo(tObr_FCR_SC)
				+" AND O.FCR_SU=" + sqlTo(tObr_FCR_SU)
				+" AND O.FCR_ST=" + sqlTo(tObr_FCR_ST)
				+" AND O.FCR_COD1=" + sqlTo(tObr_FCR_COD1)
				+" AND O.FCR_COD2=" + sqlTo(tObr_FCR_COD2)
				+" AND O.FCR_COD3=" + sqlTo(tObr_FCR_COD3)
				+" AND O.FDB_SC=" + sqlTo(tObr_FDB_SC)
				+" AND O.FDB_SU=" + sqlTo(tObr_FDB_SU)
				+" AND O.FDB_ST=" + sqlTo(tObr_FDB_ST)
				+" AND O.FDB_COD1=" + sqlTo(tObr_FDB_COD1)
				+" AND O.FDB_COD2=" + sqlTo(tObr_FDB_COD2)
				+" AND O.FDB_COD3=" + sqlTo(tObr_FDB_COD3)
			var tFilter = OpenTable(strSQL);
			
			// Найдем удельный вес каждой из аналитик в общей сумме оборотов, 
		    // противоположных закрываемым (если закрываем кредитовые, то 
		    // смотрим суммы по Дт)
		    // (не учитываем в общей сумме ту аналитику, которую "закрываем",
		    // иначе после нашего закрытия она снова "вылезет")

			//vlad 22.01.2010
			//Если явно указали больше, чем один уровень аналитики, то считаем, что на последнем уровне - аналитика,
			//"которую" сторнируем, а на тех, что выше - "для которых", и коды аналитики на высших уровнях
			//будут служить фильтром
			//+++
			var fltExpr = "";
			if (!isEmpty(mCl3)&&!isEmpty(mmFcl3))
			{
				//3-я аналитика - последняя из указанных и в плане счетов на данном уровне она привязана
				//3-я аналитика будет готовой продукцией
				fltExpr += " AND O.FCL_COD3 <> "+sqlTo(mmFcl3)
				if (!isIgn2)
				{					
					if (!isEmpty(mmFcl2))
					{
						//2-я аналитика будет фильтром
						fltExpr += " AND O.FCL_COD2="+sqlTo(mmFcl2)
					}
					else if(!tFilter.IsEmpty())
					{
						fltExpr += " AND O.FCL_COD2="+sqlTo(toScrObj(tFilter.getValue((isDB ? "FDB_COD2" : "FCR_COD2"))));
					}
				}				

				if (!isIgn1)
				{
					if (!isEmpty(mmFcl1))
					{
						//1-я аналитика будет фильтром
						fltExpr += " AND O.FCL_COD1="+sqlTo(mmFcl1)
					}
					else if(!tFilter.IsEmpty())
					{
						fltExpr += " AND O.FCL_COD1="+sqlTo(toScrObj(tFilter.getValue((isDB ? "FDB_COD1" : "FCR_COD1"))));
					}
				}		
			}
			else if (!isEmpty(mCl2)&&!isEmpty(mmFcl2))
			{
				//2-я аналитика - последняя из указанных и в плане счетов на данном уровне она привязана
				//2-я аналитика будет готовой продукцией
				fltExpr += " AND O.FCL_COD2<>"+sqlTo(mmFcl2)
				if (!isIgn1)
				{					
					if (!isEmpty(mmFcl1))
					{
						//1-я аналитика будет фильтром
						fltExpr += " AND O.FCL_COD1="+sqlTo(mmFcl1)
					}
					else if(!tFilter.IsEmpty())
					{					
						fltExpr += " AND O.FCL_COD1="+sqlTo(toScrObj(tFilter.getValue((isDB ? "FDB_COD1" : "FCR_COD1"))));
					}
					
				}				
			}
			else if (!isEmpty(mCl1))
			{				
				//1-я аналитика - последняя из указанных и в плане счетов на данном уровне она привязана
				//1-я аналитика будет готовой продукцией
				
				fltExpr += " AND O.FCL_COD1<>"+sqlTo(mmFcl1)
			}
			else
			{
				//Закрываемый счет вообще без аналитик - такого не должно быть
			}
			//---
			
			tFilter.Close();
			tFilter = null;

			var tTBLVES = getTmpTableName("", true);
			
			var strSum;
			var strHaving;
			if (isSL)
			{
				strSum = " SUM("+(isDB ? " O.FSUM4 ":" O.FSUM3 ") + ") AS FSUM ";
				strHaving = " SUM("+(isDB ? " O.FSUM4 ":" O.FSUM3 ") + ") ";
			}
			else
			{
				strSum = " SUM("+(isDB ? " O.FSUM6 ":" O.FSUM5 ") + ") AS FSUM ";
				strHaving = " SUM("+(isDB ? " O.FSUM6 ":" O.FSUM5 ") + ") ";
			}
			
			var fld = new Object();
			fld.FSUM = "MONEY";
			fld.FCL_COD1 = "INT";
			fld.FCL_COD2 = "INT";
			fld.FCL_COD3 = "INT";
			fld.FKOEF = "DOUBLE";
			fld.FTSUM = "MONEY";
			
			CreateTable(tTBLVES, fld);			
			
			var strSQL = " INSERT INTO"
							+" "+tTBLVES
							+" (FSUM, FCL_COD1, FCL_COD2, FCL_COD3, FKOEF, FTSUM)"
						+" SELECT "
							+ strSum
							+", "+(isIgn1 ? "0" : "O.FCL_COD1")+" AS FCL_COD1"
							+", "+(isIgn2 ? "0" : "O.FCL_COD2")+" AS FCL_COD2"
							+", "+(isIgn3 ? "0" : "O.FCL_COD3")+" AS FCL_COD3"
							+", "+Convert(0,"DOUBLE")+" AS FKOEF"	//удельный вес данной аналитики в общей сумме 
							+", "+Convert(0,"MONEY")+" AS FTSUM"	//общая сумма по аналитикам 
						+" FROM"
							+" OBSL O"
						+" WHERE "+SqlCmpTrue
							//vlad 22.01.2010
							//Если явно указали больше, чем один уровень аналитики, то считаем, что на последнем уровне - аналитика,
							//"которую" сторнируем, а на тех, что выше - "для которых", и коды аналитики на высших уровнях
							//будут служить фильтром
							//+++
							+ fltExpr
							/*
						    +(isIgn1 ? (" AND O.FCL_COD1="+sqlTo(tObr(isDB ? "FDB_COD1" : "FCR_COD1"))) : (" AND O.FCL_COD1<>"+sqlTo(mmFcl1)))+(mmFexcl1=="" ? "" : " AND O.FCL_COD1 NOT IN ("+mmFexcl1+")")
						    +(isIgn2 ? (" AND O.FCL_COD2="+sqlTo(tObr(isDB ? "FDB_COD2" : "FCR_COD2"))) : (" AND O.FCL_COD2<>"+sqlTo(mmFcl2)))+(mmFexcl2=="" ? "" : " AND O.FCL_COD2 NOT IN ("+mmFexcl2+")")
						    +(isIgn3 ? (" AND O.FCL_COD3="+sqlTo(tObr(isDB ? "FDB_COD3" : "FCR_COD3"))) : (" AND O.FCL_COD3<>"+sqlTo(mmFcl3)))+(mmFexcl3=="" ? "" : " AND O.FCL_COD3 NOT IN ("+mmFexcl3+")")
							*/
							//---
							+(!isEmpty(mmFexcl1) ? " AND O.FCL_COD1 NOT IN("+mmFexcl1+")" : "")
							+(!isEmpty(mmFexcl2) ? " AND O.FCL_COD2 NOT IN("+mmFexcl2+")" : "")
							+(!isEmpty(mmFexcl3) ? " AND O.FCL_COD3 NOT IN("+mmFexcl3+")" : "")
						+ " GROUP BY "
							+" "+(isIgn1 ? "" : "O.FCL_COD1")
							+" "+(isIgn2 ? "" : ((isIgn1 ? "" : ",")+" O.FCL_COD2"))
							+" "+(isIgn3 ? "" : ((isIgn1 && isIgn2 ? "" : ",")+" O.FCL_COD3"))
						+" HAVING "+strHaving+" <> 0"
			ExecuteSQL(strSQL);
									
			//Найдем всего оборотов
			var strSQL = "SELECT "
			+" SUM(O.FSUM) AS FSUM "
			+" FROM "+tTBLVES+" O"
			var sn = getSnap(strSQL)
			var tSum = 0;
			if(sn)
			{
				tSum = Number(sn[0]);
			}
			sn=null;

			if(tSum==null || tSum==0)
			{
				// Сумма оборотов по всем аналитикам нулевая.
				// Получается, что затрат нет. Это ненормально.
				// Выходим
				strMsg = "r_zrdooc.js:"
					+"\nВнимание!"
					+"\nПри закрытии по способу DOOC"
					+"\nсумма оборотов по "+(isDB ? "кредиту ": "дебету ")+" счета"
					+"\n"+toScrObj(tObr.getValue(!isDB ? "FCR_SC" : "FDB_SC"))
						+"/"+toScrObj(tObr.getValue(!isDB ? "FCR_SU" : "FDB_SU"))
						+"/"+toScrObj(tObr.getValue(!isDB ? "FCR_ST" : "FDB_ST"))
				+"\n"+(isIgn1 ? (" AND O.FCL_COD1="+sqlTo(toScrObj(tObr.getValue(fcl1)))) : (" AND O.FCL_COD1<>"+sqlTo(mmFcl1)))+(mmFexcl1=="" ? "" : " AND O.FCL_COD1 NOT IN ("+mmFexcl1+")")
				+"\n"+(isIgn2 ? (" AND O.FCL_COD2="+sqlTo(toScrObj(tObr.getValue(fcl2)))) : (" AND O.FCL_COD2<>"+sqlTo(mmFcl2)))+(mmFexcl2=="" ? "" : " AND O.FCL_COD2 NOT IN ("+mmFexcl2+")")
				+"\n"+(isIgn3 ? (" AND O.FCL_COD3="+sqlTo(toScrObj(tObr.getValue(fcl3)))) : (" AND O.FCL_COD3<>"+sqlTo(mmFcl3)))+(mmFexcl3=="" ? "" : " AND O.FCL_COD3 NOT IN ("+mmFexcl3+")")
					+"\n"
					+"\nоказалась нулевой"
					+"\nЗакрытие произведено не будет"
				alert(strMsg);
				//return;
			}
			else
			{
				// Вот теперь проставляем удельный вес
				strSQL = "UPDATE "+tTBLVES+" SET "
					+" FKOEF=FSUM/"+sqlTo(tSum)
					+", FTSUM="+sqlTo(tSum)
				ExecuteSQL(strSQL);
			}

			//////////////////////////////////////////////////////////
			//В проводках "Черным" надо код аналитики
			//"Собственная продукция", если он попадает транзитом на 
			//сторону "куда", заменить кодами затрат
			
			toFsc = isDB ? tObr_FCR_SC : tObr_FDB_SC;
			toFsu = isDB ? tObr_FCR_SU : tObr_FDB_SU;
			toFst = isDB ? tObr_FCR_ST : tObr_FDB_ST
			toFcl1 = isDB ? tObr_FCR_COD1 : tObr_FDB_COD1
			toFcl2 = isDB ? tObr_FCR_COD2 : tObr_FDB_COD2
			toFcl3 = isDB ? tObr_FCR_COD3 : tObr_FDB_COD3

			// номера спр-ков аналитик счета, на к-й закрываем
			var mCl1_to=0, mCl2_to=0, mCl3_to=0;
			
			strSQL = "SELECT FCL1, FCL2, FCL3 FROM ^CL_SCHET "
					+" WHERE 1=1"
						+" AND FSCHET = "+sqlTo(toFsc)
						+" AND FSUBSCHET = "+sqlTo(toFsu)
						+" AND FSTAT = "+sqlTo(toFst)
			var sn = getSnap(strSQL);
			if (sn != null && !isEmpty(sn[0]))
			{
				mCl1_to = Number(sn[0]);
				mCl2_to = Number(sn[1]);
				mCl3_to = Number(sn[2]);
			}
			sn = null;

			// номер спр-ка аналитики счета, к-й закрываем, соответствующий
			// номеру спр-ка 1-й аналитики, на к-й закрываем
			var nCl1=(mCl1_to==mCl1)? 1 :((mCl1_to==mCl2)? 2 :((mCl1_to==mCl3)? 3 : 0 ))

			// номер спр-ка аналитики счета, к-й закрываем, соответствующий
			// номеру спр-ка 2-й аналитики, на к-й закрываем
			var nCl2=(mCl2_to==mCl1)? 1 :((mCl2_to==mCl2)? 2 :((mCl2_to==mCl3)? 3 : 0 ))

			// номер спр-ка аналитики счета, к-й закрываем, соответствующий
			// номеру спр-ка 3-й аналитики, на к-й закрываем
			var nCl3=(mCl3_to==mCl1)? 1 :((mCl3_to==mCl2)? 2 :((mCl3_to==mCl3)? 3 : 0 ))
			//////////////////////////////////////////////////////////
			
			var DBCR = isDB ? "DB" : "CR";
			eval("str"+DBCR+"Cod1=', V.FCL_COD1'");
			eval("str"+DBCR+"Cod2=', V.FCL_COD2'");
			eval("str"+DBCR+"Cod3=', V.FCL_COD3'");
			
			//И добавляем проводки "черным"
			var strSQL = "INSERT INTO "+buffName
			+"("
				+" Ftxt"
				+",Fdb_sc"
				+",Fdb_su"
				+",Fdb_st"
				+",Fdb_cod1"
				+",Fdb_cod2"
				+",Fdb_cod3"
				+",Fcr_sc"
				+",Fcr_su"
				+",Fcr_st"
				+",Fcr_cod1"
				+",Fcr_cod2"
				+",Fcr_cod3"
				+",Fsum"
				+",Fval"
				+",Fsumspr"
				+",Fvalspr"
				+",Fdocvid"
				+",Fwrk"
				+",Fuser"
				+",Fdate"
				+",Fdocnum"
				
				//fruckt
				//если необходимы параметры документов
				//приложение 10 к кошторису
				if (par.isDoc == true)
				{
					strSQL += ""
					+", FDOC"
					+", FID_DOC"
					+", FNOP"
				}
			
			strSQL += ""				
			+") SELECT "
				+sqlTo("Закрывающая проводка DOOC, черным")+" AS Ftxt"
				+",O.Fdb_sc"
				+",O.Fdb_su"
				+",O.Fdb_st"
				//+","+((isDB && !isIgn1) ? "V.Fcl_cod1" : (!isDB && !isIgn1) ? IIF("O.Fdb_cod1="+sqlTo(mmFcl1),(nCl1==1 ? "V.Fcl_cod1":(nCl1==2 ? "V.Fcl_cod2":(nCl1==3 ? "V.Fcl_cod3":"O.Fdb_cod1"))),"O.Fdb_cod1"):"O.Fdb_cod1")+" AS Fdb_cod1"
				//+","+((isDB && !isIgn2) ? "V.Fcl_cod2" : (!isDB && !isIgn2) ? IIF("O.Fdb_cod2="+sqlTo(mmFcl2),(nCl2==1 ? "V.Fcl_cod1":(nCl2==2 ? "V.Fcl_cod2":(nCl2==3 ? "V.Fcl_cod3":"O.Fdb_cod2"))),"O.Fdb_cod2"):"O.Fdb_cod2")+" AS Fdb_cod2"
				//+","+((isDB && !isIgn3) ? "V.Fcl_cod3" : (!isDB && !isIgn3) ? IIF("O.Fdb_cod3="+sqlTo(mmFcl3),(nCl3==1 ? "V.Fcl_cod1":(nCl3==2 ? "V.Fcl_cod2":(nCl3==3 ? "V.Fcl_cod3":"O.Fdb_cod3"))),"O.Fdb_cod3"):"O.Fdb_cod3")+" AS Fdb_cod3"				
				+","+((isDB && !isIgn1) ? "V.Fcl_cod1" : (!isDB ? (nCl1==1 && !isIgn1 ? "V.Fcl_cod1":(nCl1==2 && !isIgn2 ? "V.Fcl_cod2":(nCl1==3 && !isIgn3 ? "V.Fcl_cod3":"O.Fdb_cod1"))) : "O.Fdb_cod1"))+" AS Fdb_cod1"
				+","+((isDB && !isIgn2) ? "V.Fcl_cod2" : (!isDB ? (nCl2==1 && !isIgn1 ? "V.Fcl_cod1":(nCl2==2 && !isIgn2 ? "V.Fcl_cod2":(nCl2==3 && !isIgn3 ? "V.Fcl_cod3":"O.Fdb_cod2"))) : "O.Fdb_cod2"))+" AS Fdb_cod2"
				+","+((isDB && !isIgn3) ? "V.Fcl_cod3" : (!isDB ? (nCl3==1 && !isIgn1 ? "V.Fcl_cod1":(nCl3==2 && !isIgn2 ? "V.Fcl_cod2":(nCl3==3 && !isIgn3 ? "V.Fcl_cod3":"O.Fdb_cod3"))) : "O.Fdb_cod3"))+" AS Fdb_cod3"
				
				+",O.Fcr_sc"
				+",O.Fcr_su"
				+",O.Fcr_st"
				//+","+((!isDB && !isIgn1) ? "V.Fcl_cod1" : (isDB && !isIgn1) ? IIF("O.Fcr_cod1="+sqlTo(mmFcl1),(nCl1==1 ? "V.Fcl_cod1":(nCl1==2 ? "V.Fcl_cod2":(nCl1==3 ? "V.Fcl_cod3":"O.Fcr_cod1"))),"O.Fcr_cod1"):"O.Fcr_cod1")+" AS Fcr_cod1"
				//+","+((!isDB && !isIgn2) ? "V.Fcl_cod2" : (isDB && !isIgn2) ? IIF("O.Fcr_cod2="+sqlTo(mmFcl2),(nCl2==1 ? "V.Fcl_cod1":(nCl2==2 ? "V.Fcl_cod2":(nCl2==3 ? "V.Fcl_cod3":"O.Fcr_cod2"))),"O.Fcr_cod2"):"O.Fcr_cod2")+" AS Fcr_cod2"
				//+","+((!isDB && !isIgn3) ? "V.Fcl_cod3" : (isDB && !isIgn3) ? IIF("O.Fcr_cod3="+sqlTo(mmFcl3),(nCl3==1 ? "V.Fcl_cod1":(nCl3==2 ? "V.Fcl_cod2":(nCl3==3 ? "V.Fcl_cod3":"O.Fcr_cod3"))),"O.Fcr_cod3"):"O.Fcr_cod3")+" AS Fcr_cod3"
				+","+((!isDB && !isIgn1) ? "V.Fcl_cod1" : (isDB ? (nCl1==1 && !isIgn1 ? "V.Fcl_cod1":(nCl1==2 && !isIgn2 ? "V.Fcl_cod2":(nCl1==3 && !isIgn3 ? "V.Fcl_cod3":"O.Fcr_cod1"))) : "O.Fcr_cod1"))+" AS Fcr_cod1"
				+","+((!isDB && !isIgn2) ? "V.Fcl_cod2" : (isDB ? (nCl2==1 && !isIgn1 ? "V.Fcl_cod1":(nCl2==2 && !isIgn2 ? "V.Fcl_cod2":(nCl2==3 && !isIgn3 ? "V.Fcl_cod3":"O.Fcr_cod2"))) : "O.Fcr_cod2"))+" AS Fcr_cod2"
				+","+((!isDB && !isIgn3) ? "V.Fcl_cod3" : (isDB ? (nCl3==1 && !isIgn1 ? "V.Fcl_cod1":(nCl3==2 && !isIgn2 ? "V.Fcl_cod2":(nCl3==3 && !isIgn3 ? "V.Fcl_cod3":"O.Fcr_cod3"))) : "O.Fcr_cod3"))+" AS Fcr_cod3"
				
				+","+(isZeroSl ? "V.FSUM" : sqlRound("O.Fsum*V.FKOEF"+(mmFproc!=100 ? "*"+mmFproc:""), 2)) +" AS FSUM"
				+",O.Fval"
				+","+sqlTo(0)+" AS Fsumspr"
				+","+sqlTo(0)+" AS Fvalspr"
				+","+sqlTo("закр.")+" AS Fdocvid"
				+","+sqlTo(15)+" AS Fwrk"
				+","+sqlTo(getUser())+" AS Fuser"
				
				//fruckt
				//если необходимы параметры документов
				//приложение 10 к кошторису
				if (par.isDoc == true)
				{
					strSQL += ""
					+", O.FDATE"
					+", O.FDOCNUM"
					+", O.FDOC"
					+", O.FID_DOC"
					+", O.FNOP"
				}
				else
				{
					strSQL += ""
					+","+sqlTo(dat2)+" AS Fdate"
					+","+sqlTo("12345")+" AS Fdocnum"
				}
			
			strSQL += ""
			+" FROM "
				+" "+tTBLVES+" V FULL JOIN "+tDOOC_OBR+" O ON 1 = 1"
			+" WHERE "+SqlCmpTrue
				+" AND O.FCR_SC=" + sqlTo(tObr_FCR_SC)
				+" AND O.FCR_SU=" + sqlTo(tObr_FCR_SU)
				+" AND O.FCR_ST=" + sqlTo(tObr_FCR_ST)
				+" AND O.FCR_COD1=" + sqlTo(tObr_FCR_COD1)
				+" AND O.FCR_COD2=" + sqlTo(tObr_FCR_COD2)
				+" AND O.FCR_COD3=" + sqlTo(tObr_FCR_COD3)
				+" AND O.FDB_SC=" + sqlTo(tObr_FDB_SC)
				+" AND O.FDB_SU=" + sqlTo(tObr_FDB_SU)
				+" AND O.FDB_ST=" + sqlTo(tObr_FDB_ST)
				+" AND O.FDB_COD1=" + sqlTo(tObr_FDB_COD1)
				+" AND O.FDB_COD2=" + sqlTo(tObr_FDB_COD2)
				+" AND O.FDB_COD3=" + sqlTo(tObr_FDB_COD3)

				//fruckt
				if (par.isDoc == true)
				{
					strSQL += ""
					+" AND O.FDATE = "+sqlTo(toScrObj(tObr.getValue("FDATE")))
					+" AND O.FDOCNUM = "+sqlTo(toScrObj(tObr.getValue("FDOCNUM")))
					+" AND O.FDOC = "+sqlTo(toScrObj(tObr.getValue("FDOC")))
					+" AND O.FID_DOC = "+sqlTo(toScrObj(tObr.getValue("FID_DOC")))
					+" AND O.FNOP = "+sqlTo(toScrObj(tObr.getValue("FNOP")))
				}
			ExecuteSQL(strSQL);
			
			//Проставим FWID'ы
			
			var tBuff = OpenTable(buffName);
			if(!tBuff.IsEmpty())
			{
				// Net 11.04.2012  юзаем пакетное обновление
				var firstFwid = GetUID();
				var lastFwid = GetUID(false, Number(tBuff.recordCount));

				strSQL = " UPDATE "+buffName
						+" SET FWID = FWID_CTR + "+sqlTo(firstFwid)
				ExecuteSQL(strSQL);
			}
			tBuff.Close();
			tBuff = null;
			
			if(!isZeroSl)
			{
				//Исправим ошибку округления tSum
				//Найдем разницу оборотов после закрытия
				//Если ошибки округления нет - то сумма должна быть нулевой
				//т.к. мы сделали проводку "красным" и на эту же сумму 
				//должны были наделать проводки "черным"
				strSQL = " SELECT"
							+" SUM(FSUM) AS FSUMZAKR "
						+" FROM"
							+" "+buffName
						+" WHERE "+SqlCmpTrue
				var sn = getSnap(strSQL);
				if (!isEmpty(sn) && !isEmpty(sn[0]))
				{
					//откроем таблицу по убыванию чтоб вписать разницу к самой большой сумме
					//На всякий случай добавим условие, чтобы не исправить "красную" проводку
					strSQL = " SELECT"
								+" *"
							+" FROM"
								+" "+buffName
							+" WHERE NOT "
								+"("
								+" FCR_SC=" + sqlTo(tObr_FCR_SC)
								+" AND FCR_SU=" + sqlTo(tObr_FCR_SU)
								+" AND FCR_ST=" + sqlTo(tObr_FCR_ST)
								+" AND FCR_COD1=" + sqlTo(tObr_FCR_COD1)
								+" AND FCR_COD2=" + sqlTo(tObr_FCR_COD2)
								+" AND FCR_COD3=" + sqlTo(tObr_FCR_COD3)
								+" AND FDB_SC=" + sqlTo(tObr_FDB_SC)
								+" AND FDB_SU=" + sqlTo(tObr_FDB_SU)
								+" AND FDB_ST=" + sqlTo(tObr_FDB_ST)
								+" AND FDB_COD1=" + sqlTo(tObr_FDB_COD1)
								+" AND FDB_COD2=" + sqlTo(tObr_FDB_COD2)
								+" AND FDB_COD3=" + sqlTo(tObr_FDB_COD3)
								+")"
							+" ORDER BY"
								+" FSUM DESC"
					var tBuffZakr = OpenTable(strSQL);
						
					if (!tBuffZakr.IsEmpty())
					{
						var m = new Object();
						m.FSUM = Number(tBuffZakr.getValue("FSUM")) - sn[0];
						m.FWID = toScrObj(tBuffZakr.getValue("FWID"));
						UpdateTable(buffName, m, false);
					}
					tBuffZakr.Close(5652);
					tBuffZakr = null;
				}
				sn = null;
			}
			
			//Перенесем результаты в базу
			strSQL = " INSERT INTO"
			
			if (isEmpty(par.buffName))
			{
				strSQL += " ^SCHET_ZR"
			}
			else
			{
				strSQL += " "+par.buffName
			}
			
			strSQL += ""
					+" (" + GetPatternIntersection("SCHET_ZR", "SCHET_ZR") + ")"
					+" SELECT "
						+GetPatternIntersection("SCHET_ZR", "SCHET_ZR")
					+" FROM "
						+buffName
			ExecuteSQL(strSQL);			
			
			//А теперь почистим буфер
			var strSQL = " DELETE "
			+" FROM "
				+buffName
			ExecuteSQL(strSQL);

			// сбросим счетчик FWID_CTR через удаление/добавление поля
			strSQL = " ALTER TABLE "+buffName+" DROP COLUMN FWID_CTR "
			ExecuteSQL(strSQL);

			strSQL = " ALTER TABLE "+buffName+" ADD FWID_CTR "+SqlType("COUNTER")
			ExecuteSQL(strSQL);
			
			DropTable(tTBLVES);
		}//for(tObr.MoveFirst(); !tObr.IsEOF(); tObr.MoveNext())
		//setProgressPos(0, "");
	}
	
	//vvs 05.02.2004
	tObr.close(5652);
	tObr = null;
	tcl_schet.close(5652);
	tcl_schet = null;

	DropTable(buffName);
	DropTable(tDOOC_OBR);
	DropTable(tDOOC_SC1);
	
	CLEARWAIT();
}
