/*
        Процедура расчета курсовых разниц

        Параметры:
        - outTbl - имя выходной таблицы.
                Временные таблицы используют это имя в качестве префикса к названию.
        - dat1, dat2 - период
        - objSc - объект-счет, по которому строятся обороты и остатки.

        В результате создается таблица TMP_OBLNS с полями:
        fsc      - счет
        fsu      - субсчет
        fst      - стать
        fcl_cod1 - аналитика 1
        fcl_cod2 - аналитика 2
        fcl_cod3 - аналитика 3
        fsum1    - сальдо по дебету на начало периода
        fsum2    - сальдо по кредиту на начало периода
        fsum3    - сальдо по дебету на конец периода
        fsum4    - сальдо по кредиту на конец периода
        fsum5    - оборот по дебету
        fsum6    - оборот по кредиту
        fistotal - является ли запись итоговой
*/

include("sys\\dpsql.js");

// super добавил ", cod2, cod3"
var obsr_typesp;

function ras_obsr( outTbl, dat1, dat2, sc, su, st, cod1, cod2, cod3, typesp)
{
	var cl_schet = "^cl_schet",
		schet_sl = "^schet_sl",
		schet = "^schet",
		schet_zr = "^schet_zr";

	// Формирование условия для отбора по счету
	var scCond = " fsc=" + sc
	        + ( su? " and fsu=" + su: "" )
	        + ( st? " and fst=" + st: "" )
	        + ( cod1? " and fcl_cod1=" + cod1: "" )
	        + ( cod2? " and fcl_cod2=" + cod2: "" )
	        + ( cod3? " and fcl_cod3=" + cod3: "" );
	var dbCond = " fdb_sc=" + sc
	        + ( su? " and fdb_su=" + su: "" )
	        + ( st? " and fdb_st=" + st: "" )
	        + ( cod1? " and fdb_cod1=" + cod1: "" )
	        + ( cod2? " and fdb_cod2=" + cod2: "" )
	        + ( cod3? " and fdb_cod3=" + cod3: "" );
	var crCond = " fcr_sc=" + sc
	        + ( su? " and fcr_su=" + su: "" )
	        + ( st? " and fcr_st=" + st: "" )
	        + ( cod1? " and fcr_cod1=" + cod1: "" )
	        + ( cod2? " and fcr_cod2=" + cod2: "" )
	        + ( cod3? " and fcr_cod3=" + cod3: "" );
	var sqlDat1 = sqlTo(dat1);
	var sqlDat2 = sqlTo(dat2);
	
	obsr_typesp = typesp;

	// Создание таблицы TMP_OBLNS1
	// В ней будут скидываться записи по проводкам и начальным остаткам
	DropTable(outTbl + "_1" );
	DropTable(outTbl + "_2" );
	DropTable(outTbl );
	ExecuteSQL(
		"CREATE TABLE " + outTbl + "_1 ("
			+ " fsc " + SqlType(sqlTSc) + ","
			+ " fsu " + SqlType(sqlTSu) + ","
			+ " fst " + SqlType(sqlTSt) + ","
			+ " fcl_cod1 " + SqlType("INTEGER4") + ","
			+ " fcl_cod2 " + SqlType("INTEGER4") + ","
			+ " fcl_cod3 " + SqlType("INTEGER4") + ","
			+ " fdb_sc " + SqlType(sqlTSc) + ","
			+ " fdb_su " + SqlType(sqlTSu) + ","
			+ " fdb_st " + SqlType(sqlTSt) + ","
			+ " fdb_cod1 " + SqlType("INTEGER4") + ","
			+ " fdb_cod2 " + SqlType("INTEGER4") + ","
			+ " fdb_cod3 " + SqlType("INTEGER4") + ","
			+ " fcr_sc " + SqlType(sqlTSc) + ","
			+ " fcr_su " + SqlType(sqlTSu) + ","
			+ " fcr_st " + SqlType(sqlTSt) + ","
			+ " fcr_cod1 " + SqlType("INTEGER4") + ","
			+ " fcr_cod2 " + SqlType("INTEGER4") + ","
			+ " fcr_cod3 " + SqlType("INTEGER4") + ","
			+ " fsum "+SqlType("MONEY")+","
			+ " fsum1 "+SqlType("MONEY")+","
			+ " fsum2 "+SqlType("MONEY")+","
			+ " fsum3 "+SqlType("MONEY")+","
			+ " fsum4 "+SqlType("MONEY")+","
			+ " fsum5 "+SqlType("MONEY")+","
			+ " fsum6 "+SqlType("MONEY")+","
			+ " fval "+SqlType("SHORT")+","
			+ " fwid "+SqlType("COUNTER")+","
			+ " fdate "+SqlType("DATE")+")"
		);
		
	//фильтр на организацию
	var orgCond = "";
	if(getPar("BLNBYORG"))
		orgCond = " and Forg0="+CURRENT_ORG;

	// Выборка "свежих" начальных остатков по счету
	
	// Ищем дату для начальных остатков
	var tbl = OpenTable("s",
		"SELECT max(fdate) as fdate"
		+ " FROM " + schet_sl
		+ " WHERE fsc=" + sqlTo(sc)
			+ " and fdate<=" + sqlTo(dat1)
			+ orgCond
		);

	var obCond; // условие попадения проводки в расчет

	if( tbl.getValue("fdate")!=null )
	{
		// Если найдены начальные остатки - загружаем их
		var sldDat;
		sldDat = sqlTo( new Date(tbl.getValue("fdate")) );
		ExecuteSQL(
			"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,Fsum1,Fsum2,Fsum3,Fsum4,fsum5,fsum6,fdate, fval)"
			+ " SELECT"
				+ " fsc, fsu, fst,"
				+ " fcl_cod1, fcl_cod2, fcl_cod3,"
				+ " ^SCHET_SL.Fsumd*"+sqlRated("^SCHET_SL","Fval",sqlDat2)+" as Fsum1, "
				+ " ^SCHET_SL.Fsumd*"+sqlRated("^SCHET_SL","Fval","Fdate", "fvkurs")+" as Fsum2, "
				+ " ^SCHET_SL.Fsumk*"+sqlRated("^SCHET_SL","Fval",sqlDat2)+" as Fsum3, "
				+ " ^SCHET_SL.Fsumk*"+sqlRated("^SCHET_SL","Fval","Fdate","fvkurs")+" as Fsum4, "
				+ " 0 as fsum5,"        // обороты по дебету
				+ " 0 as fsum6,"        // обороты по кредиту
				+ " fdate, fval"        // дата и валюта
			+ " FROM " + schet_sl
			+ " WHERE " + scCond
				+ " and fdate=" + sldDat
				+ " and (fsumd<>0 or fsumk<>0)"
				+ " and fistotal=" + SqlFalse
				+ orgCond
			);
		
		ExecuteSQL(
			" UPDATE " + outTbl + "_1"      + " set "
			+ " Fsum1 = (Fsum1-Fsum2-(Fsum3-Fsum4))*10000, Fsum2 =0, Fsum3=0, Fsum4=0 "
			);

		obCond = " fdate between " + sldDat
			+ " and " + sqlDat2 + " "
	}
	else
	{
		obCond = " fdate <= " + sqlDat2 + " and fwrk<>15 ";
	}

	tbl = null;

	//для обычных проводок
	ExecuteSQL(
		"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum3,fsum5,fsum1,fsum2,fsum4,fsum6,fdate,fval)"
		+ " SELECT"
			+ " fdb_sc as fsc,"	
			+ " fdb_su as fsu,"
			+ " fdb_st as fst,"
			+ " fdb_cod1 as fcl_cod1,"
			+ " fdb_cod2 as fcl_cod2,"
			+ " fdb_cod3 as fcl_cod3,"
			+ IIF("fdate<" + sqlDat1, "(^SCHET.Fsum*"+sqlRated("^SCHET","Fval",sqlDat2)+"- ^SCHET.Fsum*"+sqlRated("^SCHET","Fval","Fdate","fvkurs")+")*10000","0")+" as fsum3,"
			+ IIF("fdate<" + sqlDat1, "0", "(^SCHET.Fsum*"+sqlRated("^SCHET","Fval",sqlDat2)+"- ^SCHET.Fsum*"+sqlRated("^SCHET","Fval","Fdate","fvkurs")+")*10000")+" as fsum5,"
			+ " 0 as fsum1, 0 as fsum2, 0 as fsum4, 0 as fsum6,"  // кредитовые суммы
			+ " fdate,"
			+ " fval"
		+ " FROM " + schet
		+ " WHERE"
			+ dbCond
			+ " and " + obCond
			+ " and fwrk<>12"
			+ orgCond
		);


	ExecuteSQL(
		"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum3,fsum5,fsum1,fsum2,fsum4,fsum6,fdate,fval)"
		+ " SELECT"
			+ " fdb_sc as fsc,"
			+ " fdb_su as fsu,"
			+ " fdb_st as fst,"
			+ " fdb_cod1 as fcl_cod1,"
			+ " fdb_cod2 as fcl_cod2,"
			+ " fdb_cod3 as fcl_cod3,"
			+ IIF("fdate<" + sqlDat1,"-fsum*10000","0")+" as fsum3,"
			+ IIF("fdate<" + sqlDat1,"0","-fsum*10000")+" as fsum5,"
			+ " 0 as fsum1, 0 as fsum2, 0 as fsum4, 0 as fsum6,"  // кредитовые суммы
			+ " fdate,"
			+ " fval"
		+ " FROM " + schet
		+ " WHERE"
			+ dbCond
			+ " and " + obCond
			+ " and fwrk=12"
			+ orgCond
		);

	// Выборка кредитовых проводок
	ExecuteSQL(
		"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum4,fsum6,fsum2,fsum3,fsum1,fsum5,fdate,fval)"
		+ " SELECT"
			+ " fcr_sc as fsc,"
			+ " fcr_su as fsu,"
			+ " fcr_st as fst,"
			+ " fcr_cod1 as fcl_cod1,"
			+ " fcr_cod2 as fcl_cod2,"
			+ " fcr_cod3 as fcl_cod3,"
			+ IIF("fdate<" + sqlDat1, "(^SCHET.Fsum*"+sqlRated("^SCHET","Fval",sqlDat2)+"- ^SCHET.Fsum*"+sqlRated("^SCHET","Fval","Fdate","fvkurs")+")*10000","0")+" as fsum4,"
			+ IIF("fdate<" + sqlDat1,"0", "(^SCHET.Fsum*"+sqlRated("^SCHET","Fval",sqlDat2)+"- ^SCHET.Fsum*"+sqlRated("^SCHET","Fval","Fdate","fvkurs")+")*10000")+" as fsum6,"
			+ " 0 as fsum2, 0 as fsum3, 0 as fsum1, 0 as fsum5,"
			+ " fdate,"
			+ " fval"
		+ " FROM " + schet
		+ " WHERE "
			+ crCond
			+ " and " + obCond
			+ " and fwrk<>12"
			+ orgCond
		);

	ExecuteSQL(
		"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum4,fsum6,fsum2,fsum3,fsum1,fsum5,fdate,fval)"
		+ " SELECT"
			+ " fcr_sc as fsc,"
			+ " fcr_su as fsu,"
			+ " fcr_st as fst,"
			+ " fcr_cod1 as fcl_cod1,"
			+ " fcr_cod2 as fcl_cod2,"
			+ " fcr_cod3 as fcl_cod3,"
			+ IIF("fdate<" + sqlDat1,"-fsum*10000","0")+" as fsum4,"
			+ IIF("fdate<" + sqlDat1,"0","-fsum*10000")+" as fsum6,"
			+ " 0 as fsum2, 0 as fsum3, 0 as fsum1, 0 as fsum5,"
			+ " fdate,"
			+ " fval"
		+ " FROM " + schet
		+ " WHERE "
			+ crCond
			+ " and " + obCond
			+ " and fwrk=12"
			+ orgCond
		);

	
	// Учитывать ли закрывающие проводки
	if( op_zakr(dat1, dat2) )
	{
	// по дебету
		ExecuteSQL(
			"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum3,fsum5,fsum6,fdate,fval)"
			+ " SELECT"
				+ " fdb_sc as fsc,"
				+ " fdb_su as fsu,"
				+ " fdb_st as fst,"
				+ " fdb_cod1 as fcl_cod1,"
				+ " fdb_cod2 as fcl_cod2,"
				+ " fdb_cod3 as fcl_cod3,"
			+ IIF("fdate<" + sqlDat1,"-fsum*10000","0")+" as fsum3,"
			+ IIF("fdate<" + sqlDat1,"0","-fsum*10000")+" as fsum5,"
				+ " 0 as fsum1, 0 as fsum2, 0 as fsum4, 0 as fsum6,"
				+ " fdate,"
				+ " fval"
			+ " FROM " + schet_zr
			+ " WHERE"
				+ dbCond
				+ " and " + obCond
				+ " and fwrk=12"
				+ orgCond
			);


		ExecuteSQL(
			"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum3,fsum5,fsum1,fsum2,fsum4,fsum6,fdate,fval)"
			+ " SELECT"
					+ " fdb_sc as fsc,"
					+ " fdb_su as fsu,"
					+ " fdb_st as fst,"
					+ " fdb_cod1 as fcl_cod1,"
					+ " fdb_cod2 as fcl_cod2,"
					+ " fdb_cod3 as fcl_cod3,"
					+ IIF("fdate<" + sqlDat1,"(^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval",sqlDat2)+"- ^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval","Fdate","fvkurs")+")*10000","0")+" as fsum3,"
					+ IIF("fdate<" + sqlDat1,"0", "(^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval",sqlDat2)+"- ^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval","Fdate","fvkurs")+")*10000")+" as fsum5,"
					+ " 0 as fsum1, 0 as fsum2, 0 as fsum4, 0 as fsum6,"
					+ " fdate,"
					+ " fval"
			+ " FROM " + schet_zr
			+ " WHERE"
					+ dbCond
					+ " and " + obCond
					+ " and fwrk<>12"
					+ orgCond
			);

	// по кредиту
	ExecuteSQL(
		"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum4,fsum6,fsum2,fsum3,fsum1,fsum5,fdate,fval)"
		+ " SELECT"
				+ " fcr_sc as fsc,"
				+ " fcr_su as fsu,"
				+ " fcr_st as fst,"
				+ " fcr_cod1 as fcl_cod1,"
				+ " fcr_cod2 as fcl_cod2,"
				+ " fcr_cod3 as fcl_cod3,"
				+ IIF("fdate<" + sqlDat1, "(^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval",sqlDat2)+"- ^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval","Fdate","fvkurs")+")*10000","0")+" as fsum4,"
				+ IIF("fdate<" + sqlDat1,"0","(^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval",sqlDat2)+"- ^SCHET_ZR.Fsum*"+sqlRated("^SCHET_ZR","Fval","Fdate","fvkurs")+")*10000")+" as fsum6,"
				+ " 0 as fsum2, 0 as fsum3, 0 as fsum1, 0 as fsum5,"
				+ " fdate,"
				+ " fval"
		+ " FROM " + schet_zr
		+ " WHERE "
				+ crCond
				+ " and " + obCond
				+ " and fwrk<>12"
				+ orgCond
		);

	ExecuteSQL(
		"INSERT INTO " + outTbl + "_1(fsc,fsu,fst,fcl_cod1,fcl_cod2,fcl_cod3,fsum4,fsum6,fsum2,fsum3,fsum1,fsum5,fdate,fval)"
		+ " SELECT"
			+ " fcr_sc as fsc,"
			+ " fcr_su as fsu,"
			+ " fcr_st as fst,"
			+ " fcr_cod1 as fcl_cod1,"
			+ " fcr_cod2 as fcl_cod2,"
			+ " fcr_cod3 as fcl_cod3,"
			+ IIF("fdate<" + sqlDat1,"-fsum*10000","0")+" as fsum4,"
			+ IIF("fdate<" + sqlDat1,"0","-fsum*10000")+" as fsum6,"
			+ " 0 as fsum2, 0 as fsum3, 0 as fsum1, 0 as fsum5,"
			+ " fdate,"
			+ " fval"
			+ " FROM " + schet_zr
			+ " WHERE "
				+ crCond
				+ " and " + obCond
				+ " and fwrk=12"
				+ orgCond
			);
	}

	DropTable(outTbl+"_2");
	
	ExecuteSQL(
		"SELECT fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3, 0 as FWID"
			+", "+ emptySc+" as fdb_sc, "+emptySu+" as fdb_su, "+emptySt+" as fdb_st"
			+", 0 as fdb_cod1, 0 as fdb_cod2, 0 as fdb_cod3"
			+", "+ emptySc+" as fcr_sc, "+emptySu+" as fcr_su, "+emptySt+" as fcr_st"
			+", 0 as fcr_cod1, 0 as fcr_cod2, 0 as fcr_cod3"
			+", "+Convert(0,"MONEY")+" as Fsum"
			+", sum(" + outTbl + "_1" + ".fsum1) as fsum1"
			+", sum(" + outTbl + "_1" + ".fsum2) as fsum2"
			+", sum(" + outTbl + "_1" + ".fsum3) as fsum3"
			+", sum(" + outTbl + "_1" + ".fsum4) as fsum4"
			+", sum(" + outTbl + "_1" + ".fsum5) as fsum5"
			+", sum(" + outTbl + "_1" + ".fsum6) as fsum6"
		+ " INTO " + outTbl + "_2"
		+ " FROM " + outTbl + "_1"
		+ " GROUP BY fsc, fsu, fst, fcl_cod1, fcl_cod2, fcl_cod3"
		);

	 var sFsum = "(Fsum1-Fsum2+Fsum3-Fsum4+Fsum5-Fsum6)/10000";

	// super  добавили 2 и 3 ан.			
	ExecuteSQL(" UPDATE  "+outTbl + "_2"+
	   " SET Fsum="
				+ sqlRound(
					IIF(sFsum + "<0", "-" + sFsum,sFsum), 2)
			+ ", Fdb_sc = "+IIF(sFsum + "<0",sc_todb, "Fsc")
			+ ", Fdb_su = "+IIF(sFsum + "<0",su_todb, "Fsu")
			+ ", Fdb_st = "+IIF(sFsum + "<0",st_todb, "Fst")
			+ ", Fdb_cod1 ="+ IIF(sFsum + "<0",cl1_todb, "Fcl_cod1")
			+ ", Fdb_cod2 ="+ IIF(sFsum + "<0",cl2_todb, "Fcl_cod2")
			+ ", Fdb_cod3 ="+ IIF(sFsum + "<0",cl3_todb, "Fcl_cod3")
			+ ", Fcr_sc ="+ IIF(sFsum + "<0", "Fsc",sc_tocr)
			+ ", Fcr_su ="+ IIF(sFsum + "<0", "Fsu",su_tocr)
			+ ", Fcr_st ="+ IIF(sFsum + "<0", "Fst",st_tocr)
			+ ", Fcr_cod1 ="+ IIF(sFsum + "<0", "Fcl_cod1",cl1_tocr)
			+ ", Fcr_cod2 ="+ IIF(sFsum + "<0", "Fcl_cod2",cl2_tocr)
			+ ", Fcr_cod3 ="+ IIF(sFsum + "<0", "Fcl_cod3",cl3_tocr)
		);
	// end super
/*
	var tt = OpenTable("s", outTbl+"_2");
	if (!tt.isEmpty())
	{		
		var m = new Object();
		while (!tt.isEOF())
		{				
			m.FWID = GetUID();							
			UpdateTable(outTbl+"_2", m , false);
			tt.MoveNext();
		}
	}
	tt.Close();
*/
	ExecuteSQL(
		"SELECT all fdb_sc,"
			+ " fdb_su,"
			+ " fdb_st,"
			+ " fdb_cod1,"
			+ " fdb_cod2,"
			+ " fdb_cod3,"
			+ " fcr_sc,"
			+ " fcr_su,"
			+ " fcr_st,"
			+ " fcr_cod1,"
			+ " fcr_cod2,"
			+ " fcr_cod3,"
			+ " fsum,"
			+ " fwid "
			+ " into "+outTbl
			+ " from "+outTbl + "_2 where Fsum<>0"
		);
	
	return true;
}

// Функция, возвращающая выражение для перевода валют в SQL-запросах на определенную дату
function sqlRated(strTable, strValField, strDateField, strKursField)
{
	if(strDateField==null) strDateField = "fdate";
	if(strValField==null) strValField = "fval";

	var query = "("
		+ " SELECT " + First("^kursval.fkurs")
		+ " FROM ^kursval"
		+ " WHERE "
		        + (typeof(strValField)=="number" ?
		                "fval = " + strValField :
		                strTable + "." + strValField + " = fkod")
		        + " and " +  strDateField + " between fbdat and fedat"
		+ ")"

	cRet = IIF(query + " is null", "1", query);
	if(strKursField && obsr_typesp=="B")
		cRet = IIF(strKursField + "<>0", strKursField,cRet);
	return cRet;
}