include("sys\\kolr.js");
include("sub\\mtr.js");

// Процедура восстановления цепочек
function ReChain()
{

	// Зададим имена для временных таблиц
	var tmp1 = "tmp_rechain_1",
		tmp2 = "tmp_rechain_2",
		tmp3 = "tmp_rechain_3",
		tmp4 = "tmp_rechain_4";

	CreateTable(tmp1, "anchains");
	CreateTable(tmp2, "anchains");
	CreateTable(tmp3, "anchains");
	CreateTable(tmp4, "anchains");

	// Выбираем записи по начальным остаткам,
	// по ним в справочнике цепочек создаются записи
	// с не пустой датой начала периода
	ExecuteSQL(
		"INSERT INTO " + tmp1 + "(" + sel_fldGroupKey + ",fkol_sa,fbdate)"
		+ " SELECT " + fldGroupKey
			+ ", fkol AS fkol_sa"
			+ ", fdate AS fbdate"
		+ " FROM ^anltk a"
		+ " WHERE frtype=" + sqlTo(RTYPE_OST)
		);

	// Выбираем цепочки, которые начинаются не с начального остатка,
	// а с прихода, расхода или перемещения (цепочки с пустой датой
	// начала периода)
	if(false)
		ExecuteSQL(
			"INSERT INTO " + tmp2 + "(" + sel_fldGroupKey + ",fkol_sa,fbdate)"
			+ " SELECT " + fldGroupKey
				+ ", 0 AS fkol_sa"
				+ ", " + sqlTo(d00min) + " as fbdate"
                        + " FROM ^anltk a"
			+ " WHERE frtype<>" + sqlTo(RTYPE_OST)
				+ " AND fdate<ALL"
					+ "("
						+ " SELECT b.fbdate"
                                                + " FROM " + tmp1 + " b"
						+ " WHERE " + joinKey
					+ ")"
			+ " GROUP BY " + fldGroupKey
			);
	else
		ExecuteSQL(
			"INSERT INTO " + tmp2 + "(" + sel_fldGroupKey + ",fkol_sa,fbdate)" 
			+ " SELECT DISTINCT " + fldGroupKey
				+ ", 0 AS fkol_sa"
				+ ", " + sqlTo(d00min) + " as fbdate"
                        + " FROM ^anltk a"
                                + " LEFT JOIN " + tmp1 + " b"
				+ " ON " + joinKey
					+ " AND a.fdate>=b.fbdate"
			+ " WHERE b.fbdate IS NULL"
			);
	
	// Объединяем все выбранные цепочки в одну таблицу
	ExecuteSQL(
		"INSERT INTO " + tmp1 + " " + getColumn(tmp2,true)
		+ " SELECT " + getColumn(tmp2,false)
		+ " FROM " + tmp2
		);
	
	// Заполняем поля даты закрытия цепочек
	// (для незакрытых цепочек прописывается NULL)
	if(false)
		ExecuteSQL(
			"INSERT INTO " + tmp3 + "(" + sel_fldGroupKey + ",fkol_sa,fbdate,fedate)"
			+ " SELECT " + fldGroupKey
				+ ", fkol_sa"
				+ ", fbdate"
				+ ", ("
					+ " SELECT min(b.fbdate)"
                                        + " FROM " + tmp1 + " b"
					+ " WHERE " + joinKey
						+ " AND b.fbdate<>" + sqlTo(d00min)
						+ " AND b.fbdate>a.fbdate"
					+ ") as fedate"
                        + " FROM " + tmp1 + " a"
			);
	else 
		ExecuteSQL(
			"INSERT INTO " + tmp3 + "(" + sel_fldGroupKey + ",fkol_sa,fbdate,fedate)"
			+ " SELECT " + fldGroupKey
				+ ", a.fkol_sa"
				+ ", a.fbdate"
				+ ", min(b.fbdate) as fedate"
                        + " FROM " + tmp1 + " a"
                                + " LEFT JOIN " + tmp3 + " b"
				+ " ON " + joinKey
					+ " AND b.fbdate>a.fbdate"
			+ " GROUP BY " + fldGroupKey
				+ ", a.fkol_sa"
				+ ", a.fbdate"
			);

	// Окончательное формирование справочника цепочек
	// Следующий запрос выполняет след. действия:
	// 1) для пустой даты конца периода цепочки проставляет 
	//    очень большую дату (d00)
	// 2) из существующего справочника цепочек  достает
	//    проставленные там fwid'ы цепочек (на всякий случай)
	if(false)
		ExecuteSQL(
			"INSERT INTO " + tmp4 + "(" + sel_fldGroupKey + ",fkol_sa,fbdate,fedate,fwid)"
			+ " SELECT " + fldGroupKey
				+ ", fkol_sa"
				+ ", fbdate"
				+ ", "+IIF("a.fedate IS NULL",sqlTo(d00), "a.fedate")
					+ " AS fedate"  // (1)
				+ ", ("
					+ " SELECT "+ First("fwid")  // Вдруг попалось несколько записей по одной цепочке
                                        + " FROM ^anchains b"
					+ " WHERE " + joinKey
						+ " AND a.fbdate=b.fbdate"
					+ ") AS fwid"  // (2)
                        + " FROM " + tmp3 + " a"
			);
	else
		ExecuteSQL(
			"INSERT INTO " + tmp4 + "(" + sel_fldGroupKey + ",fkol_sa,fbdate,fcut,fedate,fwid)"
			+ " SELECT " + fldGroupKey
				+ ", a.fkol_sa"
				+ ", a.fbdate"
                                + ", "+SqlConCat(Convert("A.FSC", "STR"), "'/'", Convert("A.FSU", "STR"), "'/'", Convert("A.FST", "STR"))
					+ " AS fcut"
				+ ", "+IIF("a.fedate IS NULL",sqlTo(d00), "a.fedate")
					+ " AS fedate, "  // (1)
				+ First("b.fwid") + " as fwid"
                        + " FROM " + tmp3 + " a"
                                + " LEFT JOIN ^anchains b"
				+ " ON " + joinKey
					+ " AND a.fbdate=b.fbdate"
			+ " GROUP BY " + fldGroupKey
				+ ", a.fkol_sa"
				+ ", a.fbdate"
				+ ", a.fedate"
			);

	// Заполняем Fwid'ы для новых цепочек	
	var tbl = OpenTable("s",
		"SELECT * "
		+ " FROM " + tmp4
		+ " WHERE fwid IS NULL"
		);
	while(!tbl.IsEOF())
	{
		tbl.Edit();
		tbl.setValue("fwid",GetUID());
		tbl.Update();
		tbl.MoveNext();
	}
	tbl.Close();
	
	// Перенесем новый справочник цепочек на место старого
	ExecuteSQL("DELETE FROM ^anchains");
	
	ExecuteSQL(
		"INSERT INTO ^anchains " + getColumn(tmp4,true)
		+ " SELECT " + getColumn(tmp4,false)
		+ " FROM " + tmp4
		);
	
	// МЕДЛЕННО РАБОТАЕТ !
	var txt = "Обновление ведомости движения ";
	var n = 0, m = 0;
	tbl = OpenTable("s", "SELECT Fwid FROM ^anltk");
	var nRec = tbl.recordCount;
	while(!tbl.IsEOF())
	{
		m++; n++;
		if(m==10)
		{
			m=0;
			WAIT(txt + n + "/" + nRec);
		}
		// Проставить fwid цепочки
		anltkSave(tbl);
		tbl.MoveNext();
	}
	CLEARWAIT();
}