/*
	Функции для работы с датами.
	
	
	firstDay(d)      - получить первое число текущего месяца
	lastDay(d)       - получить последнее число текущего месяца
	goDay(d, step)   - вернуть дату, смещенную от указнной 
	                   на определенное количество дней
	goMonth(d, step) - сместить дату на указанное число месяцев
*/

function jan01(dt)
{
	return _dateTrans(dt, "D=", 1, "M=", 1);
}

function firstDay(dt)
{
	return _dateTrans(dt, "D=", 1);
}

function lastDay(dt)
{
	return _dateTrans(dt, "M+", 1, "D=", 0);
}

function firstDayMonth(dt)
{
	var isJavaType = isDateType(dt) && typeOf(dt) != "object";
	dt = toScriptDate(dt);
	var m = dt.getMonth();
	var y = dt.getFullYear();	
	if (isJavaType)	
		return new Date(y, m, 1).getVarDate();
	else
		return new Date(y, m, 1);
}

function lastDayMonth(dt)
{
	var isJavaType = isDateType(dt) && typeOf(dt) != "object";
	dt = toScriptDate(dt);
	var d = getDaysInMonth(dt);
	var m = dt.getMonth();
	var y = dt.getFullYear();
	if (isJavaType)	
		return new Date(y, m, d).getVarDate();
	else
		return new Date(y, m, d);
		
}

function firstDayYear(dt)
{
	var isJavaType = isDateType(dt) && typeOf(dt) != "object";
	dt = toScriptDate(dt);
	var y = dt.getFullYear();	
	if (isJavaType)	
		return new Date(y, 0, 1).getVarDate();
	else
		return new Date(y, 0, 1);
}

function lastDayYear(dt)
{
	var isJavaType = isDateType(dt) && typeOf(dt) != "object";
	dt = toScriptDate(dt);
	var y = dt.getFullYear();	
	if (isJavaType)	
		return new Date(y, 11, 31).getVarDate();
	else
		return new Date(y, 11, 31);
}

function goDay(dt, step)
{
	return _dateTrans(dt, "D+", Number(step));
}

// перейти на step декад вперед/назад от заданной в дате
function goDecade(dt, step)
{
	step = Math.floor(step);
	var mon = (step > 0) ? Math.floor(step/3) : Math.ceil(step/3);
	var sign = (step > 0) ? 1 : -1;
	var retVal = firstDayDecade(dt);
	var i = Math.abs(step) - Math.abs(mon*3);

	retVal = goMonth(retVal, mon);

	while (i != 0)
	{
		do
		{
			retVal = goDay(retVal, sign*1);
		}
		while (day(retVal) != 1 && day(retVal) != 11 && day(retVal) != 21);
		i--;
	}
	return retVal;
}

function goMonth(dt, step)
{
	return _dateTrans(dt, "M+", Number(step));
}

//начало квартала
function quarter_begin(dt)
{
	return _dateTrans(dt, "M%", 3, "D=", 1);
}

//конец квартала
function quarter_end(dt)
{
	return _dateTrans(dt, "M+", 3, "M%", 3, "D=", 0);
}

//номер квартала
function quarter(dt)
{
	switch(dt.getMonth())
	{
		case 0:
		case 1:
		case 2:
			return 1;
		case 3:
		case 4:
		case 5:
			return 2;
		case 6:
		case 7:
		case 8:
			return 3;
		case 9:
		case 10:
		case 11:
			return 4;
		default:
			return 0;
	}
}

function _dateTrans(dt)
{
//	var isOLE = typeOf(dt)=="Date";
	var isJavaType = isDateType(dt) && typeOf(dt) != "object";

	if(isJavaType)
	{
		dt = javaDate2Script(dt, false);		
	}

	var d = dt.getDate();
	var m = dt.getMonth();
	var y = dt.getFullYear();

	var ar = _dateTrans.arguments;
	
	for(var i=1; i<ar.length; i+=2)
	{
		var mode = ar[i], par = ar[i+1];
		switch(mode)
		{
		case "D=":
			d = par;
			break;
		case "D+":
			d += par;
			break;
		case "M+":
			m += par;
			//если передается последний день месяца (например 31), а в возвращаемом месяце меньше дней, дата может перепрыгнуть на след.месяц
			//чтобы такого не было берем последний день возвращаемого месяца
			var maxDay = getDaysInMonth(new Date(y, m, 1));
			if(Number(d)>Number(maxDay))
				d = maxDay;
			break;
		case "M=":
			m = par-1;
			break;
		case "M%":
			m = par*Math.floor(m/par);
			break;
		}
   }
	var retDt = new Date(y, m, d);

	if(isJavaType)	
		return retDt.getVarDate();
	else
		return retDt;
}

// Функции для пользователя
var firstday = firstDay,
	lastday = lastDay,
	goday = goDay,
	gomonth = goMonth,
	datetrans = _dateTrans;


// Функция, которая по двум переданным датам-объектам возвращает разницу 
// между ними в днях
function day_difference(dat1,dat2)
{
	dat1 = new Date(dat1);
	dat2 = new Date(dat2);
	return Math.round((dat2-dat1)/86400000)+1 
}

// Функция, которая по двум переданным датам-объектам возвращает разницу 
// между ними в минутах
function day_differencemin(dat1,dat2)
 {
   return Math.round((dat2-dat1)/60000) 
 }
// Функция, которая по двум переданным датам-объектам возвращает разницу 
// между ними в месяцах
function month_difference(dat1,dat2)
 { 
   dat1 = new Date(dat1);
   dat2 = new Date(dat2);
   return (dat2.getFullYear()-dat1.getFullYear())*12 + dat2.getMonth()-dat1.getMonth()+ 1 
 } 

// 1 - Січень
// 2 - січня
// 3 - січень
// 4 - січні

function tmonth(month, cFormat)
{
	month = toScrObj(month);
	if (isDateType(month))
	{
		month = new Date(month);
	}
	
	switch(objTypeOf(month))
	{
	case "date":
		month = new Date(month);
	case "Date":
		if (differ(new Date(month),new Date(d00)) == 0)
			return "";
		month = month.getMonth();
		break;
	case "number":
		month--;
		break;
	}

	var result;
	month = Number(month);	
	switch(cFormat)
	{
	case 1:
	case 3:
	case "mt":
		result = new Array(
			ru("Январь", "Січень"),
			ru("Февраль", "Лютий"),
			ru("Март", "Березень"),
			ru("Апрель", "Квітень"),
			ru("Май", "Травень"),
			ru("Июнь", "Червень"),
			ru("Июль", "Липень"),
			ru("Август", "Серпень"),
			ru("Сентябрь", "Вересень"),
			ru("Октябрь", "Жовтень"),
			ru("Ноябрь", "Листопад"),
			ru("Декабрь", "Грудень")
			) [ month ];
			break;
	default:
	case 2:
	case "TM":
		result = new Array(
			ru("января", "січня"),
			ru("февраля", "лютого"),
			ru("марта", "березня"),
			ru("апреля", "квітня"),
			ru("мая", "травня"),
			ru("июня", "червня"),
			ru("июля", "липня"),
			ru("августа", "серпня"),
			ru("сентября", "вересня"),
			ru("октября", "жовтня"),
			ru("ноября", "листопада"),
			ru("декабря", "грудня")
			) [ month ];
			break;
	case 4:
		result = new Array(
			ru("январе", "січні"),
			ru("феврале", "лютому"),
			ru("марте", "березні"),
			ru("апреле", "квітні"),
			ru("мае", "травні"),
			ru("июне", "червні"),
			ru("июле", "липні"),
			ru("августе", "серпні"),
			ru("сентябре", "вересні"),
			ru("октябре", "жовтні"),
			ru("ноябре", "листопаді"),
			ru("декабре", "грудні")
			) [ month ];
			break;
	}
	if (cFormat==3)
		result = result.toLowerCase();
	return result;
}

// Функция по переданному объекту-дате возвращает количество дней в месяце 
function getDaysInMonth(inDate)
{
	var month;
	var ddd = new Date(inDate);
	var bRet = 31;
	for(var i = 28; i <= 31; i++)
	{
		month = ddd.getMonth();
		ddd.setDate(i);
		if(month != ddd.getMonth())
		{	
			bRet = i-1;
			break;	
		}
	}
	return bRet;
}

// Функция по переданному объекту-дате возвращает день недели
function getDayAsWeekName(date)
{
	var result = "";
	if(!date)
		return result;
	if (typeof date.getDay == "function")
	{
		switch (date.getDay())
		{
			case 0: result = ru("Воскресенье","Неділя");
					break;
			case 1: result = ru("Понедельник","Понеділок");
					break;
			case 2: result = ru("Вторник","Вівторок");
					break;
			case 3: result = ru("Среда","Середа");
					break;
			case 4: result = ru("Четверг","Четверг");
					break;
			case 5: result = ru("Пятница","П'ятниця");
					break;
			case 6: result = ru("Суббота","Субота");
		}
	}	
	return result;
}

function getShortNameVsDate(date)
{
	var result = "";
	if (typeof date.getDay == "function")
	{
		switch (date.getDay())
		{
			case 0: result = ru("Вс","Нд");
					break;
			case 1: result = ru("Пн","Пн");
					break;
			case 2: result = ru("Вт","Вт");
					break;
			case 3: result = ru("Ср","Ср");
					break;
			case 4: result = ru("Чт","Чт");
					break;
			case 5: result = ru("Пт","Пт");
					break;
			case 6: result = ru("Сб","Сб");
		}
	}
	var y = date.getYear() + 1900;
	var m = date.getMonth() + 1;
	var d = date.getDate();
	result += "  [" + d + "/" + m + "/" + y + "]";
	return result;
}



// Функция по переданому объекту-дате возвращает строку "час : минута"
function getTimeAsString(date)
{
	var result = "";
	if (typeof date.getHours == "function" 
		&& typeof date.getMinutes == "function")
	{	
		var h = String(date.getHours());
		var m = String(date.getMinutes());
		result = ((h.length == 1) ? "0" + h : h) + " : " + ((m.length == 1) ? "0" + m : m) ;	
	}	
	return result;
}

// Функция по переданому объекту дате возвращает одномерный
// массив, в котором хранится дата начала и конца текущей недели
function getWeekBE(date)
{

	var ArDT = new Array();
	ArDT[0] = new Date();
	ArDT[1] = new Date();
	var dw = date.getDay();
	ArDT[0].setDate(date.getDate());
	ArDT[0].setMonth(date.getMonth());
	ArDT[0].setYear(1900 + date.getYear());
	ArDT[1] = date;
	if (dw != 0)
	{
		ArDT[0].setDate(date.getDate() - dw + 1);
		ArDT[1].setDate(date.getDate() - dw + 7);
	}
	else
	{
		ArDT[0].setDate(date.getDate() - 6);
	}
	setTime(ArDT[0],0,0,0);
	setTime(ArDT[1],23,59,59);
	return ArDT;
}

// Функция для переданного объекта-даты устанавлевает необходимое время
function setTime(dt,hor,min,sec)
{
	if (typeof dt == "object")
	{
		if (typeof hor == "number" && typeof dt.setHours == "function")
			dt.setHours(hor);
		if (typeof min == "number" && typeof dt.setMinutes == "function")
			dt.setMinutes(min);
		if (typeof sec == "number" && typeof dt.setSeconds == "function")
			dt.setSeconds(sec);
	}
	return dt;
}

//преобразует строку вида 01.01.2001 в дату
function strToDate(strDate)
{
	if(!strDate || strDate=="")
		return new Date(d00);
	var nDay = Number(strDate.substring(0, 2));
	var nMonth = Number(strDate.substring(3, 5)) - 1;
	var sYear = strDate.substring(6);
	if(sYear && sYear.length==2)
		sYear = "20"+ sYear;
	var nYear = Number(sYear);
	return new Date(nYear, nMonth, nDay);
}

//для пустой даты (01.01.3000) возвращает null
function d00ToNull(d)
{
	if(differ(d00obj,new Date(d))==0)
		return null;
	else
		return d;
}

//находит базовый месяц расчета з/п
function getBaseMonth(days)
{
	var dat = new Date(today);
	dat.setDate(dat.getDate()-days);
	dat.setDate(1);
	return dat;
}

//преобразует строку в дату по указаномму формату
//strToDateFormat("08/12/31","yy/mm/dd")
function strToDateFormat(str,format,isSilentMode)
{
	str = String(str);
	format = String(format);
	if(str.length!=format.length)
	{
		if(!isSilentMode)
			WAIT("Дата "+str+ru(" не соответствует формату ","Дата не відповідає формату ")+format);
		return d00;
	}
	var ie = format.indexOf("yyyy");
	var year = 0;
	var month = 0;
	var day = 0;
	var hour = 0;
	var minut = 0;
	var second = 0;
	if(ie!=-1)
		year = Number(str.substr(ie,4));
	else
	{
		ie = format.indexOf("yy");
		if(ie!=-1)
		{
			year = str.substr(ie,2);
			if(Number(year)<90)
				year = Number("20"+year);
			else
				year = Number("19"+year);
		}
		else
		{
			if(!isSilentMode)
				WAIT("Дата "+str+ru(" не соответствует формату ","Дата не відповідає формату ")+format);
			return d00;
		}
	}
	var im = format.indexOf("mm");
	if(im!=-1)
		month = Number(str.substr(im,2));
	else
	{
		if(!isSilentMode)
			WAIT("Дата "+str+ru(" не соответствует формату ","Дата не відповідає формату ")+format);
		return d00;
	}
	var id = format.indexOf("dd");
	if(id!=-1)
		day = Number(str.substr(id,2));
	else
	{
		if(!isSilentMode)
			WAIT("Дата "+str+ru(" не соответствует формату ","Дата не відповідає формату ")+format);
		return d00;
	}
	if(isNaN(year) || isNaN(month) || isNaN(day))
	{
		if(!isSilentMode)
			WAIT("Дата "+str+ru(" не соответствует формату ","Дата не відповідає формату ")+format);
		return d00;
	}
	var ih = format.indexOf("HH");
	if(ih!=-1)
		hour = Number(str.substr(ih,2));
	var imin = format.indexOf("MM");
	if(imin!=-1)
		minut = Number(str.substr(imin,2));
	var is = format.indexOf("SS");
	if(is!=-1)
		second = Number(str.substr(is,2));
	return new Date(year,month-1,day,hour,minut,second).getVarDate();
}

// вернет начало декады (1: первая, 11: вторая, 21: третья)
function firstDayDecade(dt)
{
	var isJavaType = isDateType(dt) && typeOf(dt) != "object";
	var decade = Math.min(Math.ceil(day(dt) / 10), 3);
	
	if (isJavaType)	
		return new Date(year(dt), month(dt)-1, 10 * (decade-1)+1).getVarDate();
	else
		return new Date(year(dt), month(dt)-1, 10 * (decade-1)+1);	
}

// вернет конец декады (10: первая, 20: вторая, [30;31]: третья; для февраля - [28;29] - третья)
function lastDayDecade(dt)
{
	var isJavaType = isDateType(dt) && typeOf(dt) != "object";
	var decade = Math.min(Math.ceil(day(dt) / 10), 3);
	var dy = 10 * (decade);
	if (dy == 30)
	{
		dy = day(lastDay(new Date(year(dt), month(dt)-1, 1)));
	}
	
	if (isJavaType)	
		return new Date(year(dt), month(dt)-1, dy).getVarDate();
	else
		return new Date(year(dt), month(dt)-1, dy);	
	
}

function nextkv(m__dt)
{
   var f_dt;
   m__dt = new Date(m__dt)
   if (m__dt.getMonth() >= 0 && m__dt.getMonth() <=2 ){
      f_dt = new Date(m__dt.getFullYear(), 3, 1);
   }
   if (m__dt.getMonth() >= 3 && m__dt.getMonth() <= 5){
      f_dt = new Date(m__dt.getFullYear(), 6, 1);
   }
   if (m__dt.getMonth() >= 6 && m__dt.getMonth() <= 8){
      f_dt = new Date(m__dt.getFullYear(), 9, 1);
   }
   if (m__dt.getMonth() >= 9 && m__dt.getMonth() <= 12){
      f_dt = new Date(m__dt.getFullYear() + 1, 0, 1);
   }
   return f_dt;
}

//подсчет високосных дней за период
function getVisDaysCount(dat1, dat2)
{
	dat1 = new Date(dat1).getVarDate();
	dat2 = new Date(dat2).getVarDate();
	var dt = new Date(dat1).getVarDate();
	var days = 0;
	while(differ(dt,dat2)<=0)
	{
		var year = new Date(dt).getFullYear();
		if(year % 4 == 0)
		{
			dt = new Date(year, 1, 29).getVarDate();
			if(differ(dat1,dt)<=0 && differ(dat2,dt)>=0)
				days++;
			
		}
		dt = goMonth(new Date(year, 1, 28), 12);
	}
	return days;
}