/*
	Функция вывода значения в указанном формате.
	Для типа даты:
		format("Сегодня d mmm yyyy г.") == "Сегодня 1 января 2001 г."
		format("dd/mm/yy") == "01/01/01"
		format("Месяц № m") == "Месяц № 1"
	Для типа числа:
		format("# рублей 00 копеек", 4.078) = "4 рублей 08 копеек"
		format("00 рублей 00 копеек", 4.078) = "04 рублей 08 копеек"
		format("#.### литров", 4.1002) = "4.1 литров"
		Флаги:
		   {0:xxx} - для нулевого значения выводить xxx
		format("#.###{0:-}", 0) = "-"
	Время, переданное числом (в часах) или датой. Если ничего не передано - текущее время
		format("th часов tm минут ts секунд", 12.5) == "12 часов 30 минут 00 секунд"
		format("th:tm:ts", 3.123) = "3:07:23"
*/
var formatKol = "#."+strRepl('0', getPar("PRINTDECIKOL")) + "{0: }"; 	//"0." + strRepl('0',  getPar("PRINTDECIKOL"));
var formatCena = "#."+strRepl('0', getPar("PRINTDECIPRI")) + "{0: }"; 	//"0." + strRepl('0', getPar("PRINTDECIPRI"));
var formatSum = "#."+strRepl('0', getPar("PRINTDECISUM")) + "{0: }"; 	//"0." + strRepl('0', getPar("PRINTDECISUM"));

function format(cFormat, uValue)
{	
	cFormat = String(cFormat);
	uValue = toScrObj(uValue);
	if(uValue == null)
		return "";
	if(typeof uValue=="number" && strIn("th", cFormat))  // Выводим время
	{
		cFormat = cFormat.replace(/th/g, Math.floor(uValue));
		cFormat = cFormat.replace(/tm/g, format("00", Math.floor((uValue%1)*60)));
		cFormat = cFormat.replace(/ts/g, format("00", Math.round(((uValue%1)*60*60)%60)));
	}
	else if(strIn("th", cFormat))
	{
		var d = typeof uValue=="undefined"? new Date(): new Date(uValue);

		cFormat = cFormat.replace(/th/g, d.getHours());
		cFormat = cFormat.replace(/tm/g, format("00", d.getMinutes()));
		cFormat = cFormat.replace(/ts/g, format("00", d.getSeconds()));
		uValue = d;
	}

	if(cFormat == "X_")
	{
		cFormat = (uValue ? "X" : "");				
	}
	else if(typeof uValue=="date" || objTypeOf(uValue)=="Date"
			|| objTypeOf(uValue)=="Timestamp")
	{
		
		if(inList(typeOf(uValue),"Date","Timestamp"))
		{
			uValue = toScriptDate(uValue)
		}
		var oDate = new Date(uValue);
		var nDate = oDate.getDate();
		var nMonth = oDate.getMonth()+1;
		var nYear = oDate.getFullYear();
		var nHours = oDate.getHours();
		var nMinutes = oDate.getMinutes();
		var nSeconds = oDate.getSeconds();
		
		//Komarov 24.12.2001
		if(3000 == nYear)
			return "";

		if(strIn("mmm", cFormat))
			cFormat = cFormat.replace(/mmm/g, tmonth(nMonth, 2));
		if(strIn("Mmm", cFormat))
			cFormat = cFormat.replace(/Mmm/g, tmonth(nMonth, 1));
		if(strIn("mm", cFormat))
			cFormat = cFormat.replace(/mm/g, format("00", nMonth));
		if(strIn("m", cFormat))
			cFormat = cFormat.replace(/m/g, String(nMonth));
		if(strIn("dd", cFormat))
			cFormat = cFormat.replace(/dd/g, format("00", nDate));
		if(strIn("d", cFormat))
			cFormat = cFormat.replace(/d/g, String(nDate));
		if(strIn("yyyy", cFormat))
			cFormat = cFormat.replace(/yyyy/g, String(nYear));
		if(strIn("yy",  cFormat))
			cFormat = cFormat.replace(/yy/g, format("00", nYear%100));
		if(strIn("y",  cFormat))
			cFormat = cFormat.replace(/y/g, format("0", nYear%10));
			
		if(strIn("HH",  cFormat))
			cFormat = cFormat.replace(/HH/g, format("00", nHours));
		if(strIn("H",  cFormat))
			cFormat = cFormat.replace(/H/g, format("0", nHours));
		if(strIn("MM",  cFormat))
			cFormat = cFormat.replace(/MM/g, format("00", nMinutes));
		if(strIn("M",  cFormat))
			cFormat = cFormat.replace(/M/g, format("0", nMinutes));
		if(strIn("SS",  cFormat))
			cFormat = cFormat.replace(/SS/g, format("00", nSeconds));
		if(strIn("S",  cFormat))
			cFormat = cFormat.replace(/S/g, format("0", nSeconds));
		
	}
	else if(typeof uValue=="number")
	{
		switch (cFormat.toUpperCase())
		{
		case "KOL":
			cFormat = formatKol;
			break;
		case "CENA":
			cFormat = formatCena;
			break;
		case "SUM":
			cFormat = formatSum;
			break;
		}
		//format("#.##{0:-}", 0)
		// Обрабатываем флаги
		// Флаг {0:xxx}
		
		var reFlag = new RegExp("{0:(.*)}");
		var isFlag0 = (reFlag.exec(cFormat) != null);
		var sFlag0 = RegExp.$1;
		cFormat = cFormat.replace(reFlag, "");
		
		if (isFlag0 && uValue==0)
		{
			//cFormat = cFormat.replace(/[#09]+.*[#09]*[]/, sFlag0);
			cFormat = cFormat.replace(/[#09]+.*[#09]*/, sFlag0);
			return cFormat;
		}
		// Обрабатываем целую часть
		var isNeg = uValue < 0;
		if (isNeg)
			uValue = Math.abs(uValue);
		var numPos = cFormat.search("[#09]");
		if(numPos == -1)
			return cFormat;
		var numLen = 1;
		var cFrm = c4repl = cFormat.substr(numPos, 1);
		while(cFormat.substr(numPos+numLen, 1)==cFrm)
		{
			numLen++;
			c4repl += cFrm;
		}
		var cNum = String(Math.floor(uValue))
		while(cNum.length<numLen)
			cNum = (cFrm=="0"? "0": " ") + cNum;
		cFormat = cFormat.replace(c4repl, cNum);
		
		if(numLen<cNum.length)
			numLen = cNum.length;
		
		var cFormat1 = cFormat.substr(0, numPos + numLen);
		cFormat = cFormat.substr(numPos + numLen);

		// Обрабатываем после запятой
		var numPos = cFormat.search("[#09]");
		if(numPos!=-1)
		{
			var numLen = 1;
			var cFrm = c4repl = cFormat.substr(numPos, 1);
			while(cFormat.substr(numPos+numLen, 1)==cFrm)
			{
				numLen++;
				c4repl += cFrm;
			}
			
			var tail = round(uValue%1, numLen);
			//если переполнение разряда, то увеличиваем целую часть
			if(tail==1)
				cFormat1 = String(Number(cFormat1)+1)
			cNum = String(tail).substr(2);
			
			if(cFrm=="0")
			{
				while(cNum.length<numLen)
					cNum = cNum + "0";
			}
			if(cNum.length>numLen)
				cNum = String(cNum).substr(0, numLen);
			cFormat = cFormat1 + cFormat.replace(c4repl, cNum);
			if (isNeg) cFormat = "-"+cFormat;
		}
		else
		{
			cFormat = cFormat1 + cFormat;
			if (isNeg) cFormat = "-"+cFormat;
		}
		
		var prov = String(cFormat).substr(cFormat.length - 1);
		if (prov == "." || prov == ",")
			cFormat = String(cFormat).substr(0,cFormat.length-1);
	}
	return cFormat;
}