/*
   Сервисные функции для работы с таблицами
*/

// По текущему полю указанной таблицы создает объект, в свойствах
// которого лежат значения полей. Имена свойств соответствуют
// именам полей таблицы большими буквами. Вторым параметром
// можно передать список необходимых полей через запятую.
function scatter(tbl, sFldsList, obj)
{
   var objFlds = obj? obj: new Object();
   if(isEmpty(sFldsList))
   {
   		var fiList = tbl.getFieldsInfo();
		for(var i = 0; i < fiList.length; i++) 
		{
			var fld = fiList[i];
			var fldName = String(fld.getName()).toUpperCase();
			if((fldName.indexOf("#") + fldName.indexOf(".")) >= 0)
				continue;
			objFlds[fldName] = toScrObj(tbl.getValue(fldName));
		}
   }
   else
   {
      var aFlds = sFldsList.toUpperCase().replace(/ /g,"").split(",");
      for(var i=0; i<aFlds.length; i++)
         objFlds[aFlds[i]] = toScrObj(tbl.getValue(aFlds[i]));
   }
   return objFlds;
}

function scatterEx(mode, tbl, sFldsList, obj)
{
	var objFlds = obj? obj: new Object;
	
	if(strIn("#", mode))
	{
		var e = tbl.getTie("#");
		for(e.topField(); e.isField(); e.nextField())
		{
			label = "#" + e.getInfo("LABEL");
			objFlds[label] = tbl.getValue(label);
		}
		
		if(!strIn("-", mode))
			scatter(tbl, sFldsList, objFlds);
	}
	else
		scatter(tbl, sFldsList, objFlds);
	
	return objFlds;
}

// Сохраняет свойства объекта в запись таблицы
function gather(tbl, objFlds, reg)
{
   if (reg!="NEW")
       tbl.Edit();
   for(var i in objFlds)
      tbl.setValue(i,objFlds[i]);
   if (reg!="NEW")   
       tbl.Update();
   if (reg!="NEW" && objFlds.FWID && tbl.getValue("FWID") != objFlds.FWID)
   {
   	   debug("При записи в таблицу FWID " + sqlTo(objFlds.FWID) + "\n"
		   + " записан как " + sqlTo(tbl.getValue("FWID")));
	   objFlds.FWID = tbl.getValue("FWID");
   }
}

// Добавление записи в таблицу (добавленная запись становиться текущей)
// mode: "N" - не перебивать Fwid
function append(mode, tbl, objFlds)
{
   mode = (typeof(mode)=="string"? mode.toUpperCase(): "");
   var isFlds = typeof(objFlds)=="object";  // переданы ли поля для инициализации?
   
   // Устанавливаем значение поля FWID
   // Предполагается, что заданная таблица имеет поле и индекс FWID
   // в других случаях это можно регулировать режимом (пока не реализовано)
   tbl.AddNew();
   // Если передан список полей - перебиваем значения
   if(isFlds)
   { 
   		for(var i in objFlds) 
   		{
   			if (tbl.isField(i))
   				tbl.setValue(i, objFlds[i]);
   			else
   				trace("function append: skipped field - " + i);
   		}
   }
   if(mode.indexOf("N")==-1 && (!isFlds || objFlds.FWID==null))
      tbl.setValue("fwid",GetUID());
   if (objFlds.FWID && !tbl.IsEmpty() && tbl.getValue("FWID") != objFlds.FWID)
   {
      debug("При записи в таблицу FWID " + sqlTo(objFlds.FWID) + "\n"
	      + "записан как " + sqlTo(tbl.getValue("FWID")));
	  objFlds.FWID = tbl.getValue("FWID");
   }
//   if (objFlds.FWID!=null && !strIn("-",mode))
//      updateTbl(tbl);
//   else

      tbl.Update();
}

// обьект для прохода по таблице и изменения записей
// Вызов: scan("tblMy", subMy) 
//        scan(rstMy, "subMy", objMy)
function scan( tbl, subRec, objSub )
{
   var objTbl, isSubMethod
   if( typeof(tbl) == "string" )
      objTbl = OpenTable("s", tbl)
   else
   {
      objTbl = tbl
      if( !objTbl.IsEmpty() )
         objTbl.MoveFirst();
   }
   isSubMethod = (objSub!=null && typeof(objSub)=="object")
   while (!objTbl.IsEOF())
   {
      objTbl.Edit();
      if( isSubMethod )
         objSub[subRec]();
      else
         subRec(objTbl)
      objTbl.Update();
      objTbl.MoveNext();
   }
   objTbl = null;
}

function getState(tbl)
{
	var state = new Object();
	state.isEOF = tbl.IsEOF();
	state.isBOF = tbl.IsBOF();
	state.index = tbl.Index;
	if(!state.isEOF && !state.isBOF)
	{
		state.bookmark = tbl.bookmark;
	}
	return state;
}

function putState(tbl, state)
{
	if(state.index!=tbl.index)
		tbl.index = state.index;
	if(!state.isEOF || !state.isBOF)
	{
		if(state.isEOF)
		{
			tbl.MoveLast();
			tbl.MoveNext();
		}
		else if(state.isBOF)
		{
			tbl.MoveFirst();
			tbl.MovePrev();
		}
		else
			tbl.bookmark = state.bookmark;
	}
}

function getFieldNamesList(tbl)
{
	var fldList = new Array();
	for(var i = 1; i <= tbl.fieldCount(); i++) 
	{
		var fld = tbl.getFieldInfo(i);
		fldList[i-1] = fld.getName();
	}
	return fldList;
}

function getFieldTypesList(tbl)
{
	var fldList = new Array();
	for(var i = 1; i <= tbl.fieldCount(); i++) 
	{
		var fld = tbl.getFieldInfo(i);
		fldList[i-1] = fld.getType();
/*		
		switch (fld.getType())
		{
			case 1: fldList[i-1] = SqlType("Boolean");
					break;
			case 2: fldList[i-1] = SqlType("Byte");
					break;
			case 3: fldList[i-1] = SqlType("Integer");
					break;
			case 4: fldList[i-1] = SqlType("Long");
					break;
			case 5: fldList[i-1] = SqlType("Currency");
					break;
			case 6: fldList[i-1] = SqlType("Single");
					break;
			case 7: fldList[i-1] = SqlType("Double");
					break;
			case 8: fldList[i-1] = SqlType("Date");
					break;
			case 9: fldList[i-1] = SqlType("Binary");
					break;
			case 10: fldList[i-1] = SqlType("Text");
					break;
			case 11: fldList[i-1] = SqlType("LongBinary");
					break;
			case 12: fldList[i-1] = SqlType("Memo");
					break;
			case 15: fldList[i-1] = SqlType("GUID");
					break;
			case 16: fldList[i-1] = SqlType("BigInt");
					break;
			case 18: fldList[i-1] = SqlType("Char");
					break;
			case 19: fldList[i-1] = SqlType("Numeric");
					break;
			case 20: fldList[i-1] = SqlType("Decimal");
					break;
			case 21: fldList[i-1] = SqlType("Float");
					break;
			case 22: fldList[i-1] = SqlType("Time");
					break;
		}
*/
	}
	return fldList;
}

function getFieldDpTypesList(tbl)
{
	var fldList = new Array();
	for(var i = 1; i <= tbl.fieldCount(); i++) 
	{
		var fld = tbl.getFieldInfo(i);
		fldList[i-1] = fld.getDpType();
	}
	return fldList;
}

function CreateTableAsThis(tbl1,tbl2Name,server)
{
	var SUBD_old = SUBD;
	if ( typeof(server)!="undefined" && server!="" )
		SUBD = ACCESS;	
	var fldNames = getFieldNamesList(tbl1);
	var fldTypes = getFieldTypesList(tbl1);
	SUBD = SUBD_old;
	strSQL = "CREATE TABLE " + tbl2Name + " (";
	for (var i = 0; i < fldNames.length; i++)
		strSQL += (i?",":"") + fldNames[i] + " " + fldTypes[i];
	strSQL += ")";
	
	if (typeof(server)=="undefined" || server=="")
		ExecuteSQL(strSQL);
	else
		ExecuteSQL(strSQL, null, null, "s2", server);
}

function CopyTable(InTbl,OutTbl,shWait)
{
	if (!InTbl.IsEmpty())
		InTbl.MoveFirst();
	if (shWait)
	{
		var recn = 0;
		var rcrdcnt = InTbl.recordcount;
	}
	while (!InTbl.IsEof())
	{
		var m = new Object;
		scatter(InTbl, null, m);
		OutTbl.AddNew();
		for(var i in m)
			OutTbl.setValue(i,m[i]);
		OutTbl.Update();
		InTbl.MoveNext();
		if (shWait)
		{
			recn++;
			WAIT(shWait+" "+recn+"/"+rcrdcnt);
		}
	}
}

function xmlTableToDBTable(xmlTbl, tableName)
{
	xmlTbl.moveFirst();
	while(!xmlTbl.isEOF())
	{
		var m = scatter(xmlTbl)
		updateTable(tableName,m,true);
		xmlTbl.moveNext();
	}
}