include("testApi\\DpTestResult.js");

function DpTestCase(sTestFuncName)
{		
	this.result = new DpTestResult(objTypeOf(this), sTestFuncName);
	this.sTestFuncName = sTestFuncName;
	this.runTest = DpTestCase_runTest;	
	this.assertTrue = DpTestCase_assertTrue;
	this.assertFalse = DpTestCase_assertFalse;
	this.assertNull = DpTestCase_assertNull;
	this.assertEquals = DpTestCase_assertEquals;	
	this.setFuncName = DpTestCase_setFuncName;
	this.getTestResult = DpTestSuit_getTestResult;
	this.writeToXml = DpTestCase_writeToXml;
	this.assertEtalon = DpTestCase_assertEtalon;	
	this.ignoreField = DpTestCase_ignoreField;
	this.getLastError = DpTestCase_getLastError;	
	this.setBStep = DpTestCase_setBStep;
	this.setStep = DpTestCase_setStep;
	this.init = DpTestCase_init;
	this.getMaxStep = getMaxStep;
	this.getTestList = getTestList;
	this.setMode = DpTestCase_setMode;
	this.getPar = DpTestCase_getPar;
	this.runByStep = DpTestCase_runByStep;
	this.fillStepCombo = fillStepCombo;	
	
	this.tester = lpProxy.createDBTester();
	this.arrIgnoreFields = new Array();	
	this.byStep = true;	
	this.listName = "list.xml";	
	test = this;
}

function DpTestCase_setFuncName(sFunc)
{
	this.sTestFuncName = sFunc;
	this.result = new DpTestResult(objTypeOf(this), this.sTestFuncName);
}

function DpTestCase_runTest(testName)
{	
	if (typeof(this[this.sTestFuncName]) == "function")
	{
		var e;
		try
		{			
			//this[this.sTestFuncName](testName);	
			//this.result.addRun();			
			this.runByStep(testName);			
		}
		catch(e)
		{
			this.result.addFailure(e);
		}		
	}	
}

function DpTestCase_runByStep(testName)
{	
	var debTest = lpProxy.getTest();
	var listName = debTest.getListName();
	var stepList;
	if(debTest!=null) 
		stepList = debTest.getMap();

	if(stepList==null)
	{
		this[this.sTestFuncName]();			
		this.result.addRun();
		return;
	}

	if(this.sTestFuncName == "test_dialog")
	{
		var maxStep = getMaxStep(listName);		
		var nOneStep = Number(debTest.getOneStep());

		for(var n=1; n<=maxStep; n++)
		{				
			if(lpProxy.getTest().isStop())
				return;
			if(nOneStep!=0 && nOneStep!=Number(n))
				continue;
			var step = "step_"+ n;
			var sFile = stepList.get(step);

			if(sFile==null)
				continue;
			try
			{
				this.setStep(n);
				//this.init(sFile);
				this[this.sTestFuncName](sFile, step);			
				this.result.addRun();
			}
			catch(e)
			{
				this.result.addFailure(e);
			}
		}
		return;
	}
}

function getTestList(listName)
{	
	var listInfo = loadXmlTest(listName);
	if(!listInfo)
		return null;
	return listInfo.getChildNodes();
}

function loadXmlTest(sFileName)
{
	if(!sFileName)
		sFileName = this.listName;
	var file = new DpFile(sFileName);
	if(!file.exists())
		return null;
		
	include ("objects/DpXMLDomDocument.js");	
	var xmlDom = new DpXMLDOMDocument();
	xmlDom.load(sFileName);
	return xmlDom.getDocumentElement();	
}

function fillStepCombo(oCtrl, listName)
{
	oCtrl.clear();		
	var testList = getTestList(listName);
	if(testList==null)
		return;
	for(var i=0; i<testList.length; i++)
	{
		var elem = testList[i];		
		var nStep = elem.getNodeName().substring(5);
		nStep = nStep.replace("_", ".");
		var sText = elem.getAttribute("text");
		if(sText=="" || sText==null)
			sText = elem.getAttribute("id") +" "+ elem.getAttribute("tree");
		sText = nStep +": "+ sText;
		oCtrl.addItem(nStep, sText);
	}
}	

function DpTestCase_assertTrue(sMsg, value)
{	
	if (DpTestCase_assertTrue.arguments < 2)
	{
		value = sMsg;
		sMsg = "";
	}	
	
	if (!(value === true))
	{
		this.result.addError(sMsg);
	}	
}

function DpTestCase_assertFalse(sMsg, value)
{
	if (DpTestCase_assertFalse.arguments < 2)
	{
		value = sMsg;
		sMsg = "";
	}	
	
	if (!(value === false))
	{
		this.result.addError(sMsg);
	}
}

function DpTestCase_assertNull(sMsg, value)
{
	if (DpTestCase_assertNull.arguments < 2)
	{
		value = sMsg;
		sMsg = "";
	}	
	
	this.assertTrue(sMsg, value == null);
}

function DpTestCase_assertEquals(sMsg, value1, value2)
{
	if (DpTestCase_assertEquals.arguments < 3)
	{
		value2 = value1;
		value1 = sMsg;
		sMsg = "";
	}	
	
	this.assertTrue(sMsg, value1 == value2);
}

function DpTestSuit_getTestResult()
{
	return this.result;
}

function DpTestCase_assertEtalon(sTblName, sXml, sSql)
{		
	var sFileName = null;
	var debTest = lpProxy.getTest();
	if(debTest)
	{
		tester  = debTest.getDBTester();
		sFileName = debTest.getTblFile(sXml);
		sTblName = debTest.getTblName(sXml, true);
	}
	else
	{
		sFileName = sXml;
		if(!this.tester)
			this.tester = lpProxy.createDBTester();
	}
		
	if(!sTblName)
		return false;
	if(!sFileName)
		sFileName = sTblName +".xml";
	
	this.assertFalse("не найден эталон "+ sXml +".xml", sFileName==null);
		
	if(!sSql)
		sSql = "select * from ^"+ sTblName; 
		
	this.tester.addTable(sTblName, sSql);	
	this.tester.loadEtalon(sFileName);
	for(sName in this.arrIgnoreFields)
		this.tester.excludeColumn(this.arrIgnoreFields[sName]);
	this.tester.testTable(sTblName);
	
	var sMsg = this.tester.getLastErrorMessage();
	this.result.addError(sMsg);
	
	return true;
}

function DpTestCase_assertEtalons()
{
	return lpProxy.getTest().testTables();
}

function DpTestCase_ignoreField(fieldName)
{
	if(lpProxy.getTest())
		lpProxy.getTest().ignoreField(fieldName);
	else
		this.tester.excludeColumn(fieldName);
	this.arrIgnoreFields[this.arrIgnoreFields.length] = fieldName;
}

function DpTestCase_init(newMap)
{
	lpProxy.getTest().init(newMap);	
}

function DpTestCase_setBStep(bStep)
{
	lpProxy.getTest().setBStep(bStep);
	this.byStep = bStep;
}

function DpTestCase_setStep(step)
{
	lpProxy.getTest().setStep(step);
}

function DpTestCase_setMode(sMode)
{
	lpProxy.getTest().setMode(sMode);
}

function DpTestCase_getPar(dlgName, parName)
{	
	return lpProxy.getTest().getPar(dlgName, parName);
}

function getMaxStep(listName)
{
	if(!listName)
		listName = lpProxy.getTest().getListName();
	var testList = getTestList(listName);
	if(testList==null)
		return 0;
	var elem = testList[testList.length-1];
	var step = elem.getNodeName();
	return getNStep(step);
}

function getNStep(step)
{
	if(step.indexOf("step")!=-1)
		step = step.substring(5);
	if(step.indexOf("_")!=-1)
		step = step.substring(step.lastIndexOf("_")+1);
	return Number(step);	
}

function addIgnoreDlg(sDlgName)
{
	lpProxy.getTest().addIgnoreDlg(sDlgName);
}

function getParFileName()
{
	return "param.xml";
}

function testList(list)
{
	if(list.zaval && Task.getTaskName().endsWith(".js"))
		return;
	var test = lpProxy.getTest();	
	
	if(test.isWrite() || (list.name && list.name.indexOf("TMP_TEST_TBL")!=-1))
		return;		
	var stepList = test.getStepList(test.getListName(), test.getStep());
	if(stepList==null)
		return;
	
	for(var nStep=0; nStep<stepList.getLength(); nStep++)
	{
		var eStep = stepList.item(nStep);
		if(!eStep)
			continue;	
				
		var step = eStep.getNodeName().substring(5); 
		runStep(list, step);
	}		
	if(!list.zaval && !test.isWrite())
		CloseWindow();
}

function runStep(list, step)
{
	var grid = list.grid;
		
	var test = lpProxy.getTest();
	if(!test || test.isStop())
		return;
	if(!step)
		step = test.getStep();	
	test.setStep(step);	
	
	var bZaval = !list.zaval? false : list.zaval;	
	
	var sMenuMode = test.getMenuMode();
	var sTreePos = test.getTreePos();	
	if(typeof(Tree)!="undefined")
	{
		if(test.getStepPar("id")=="tree")
		{
			test.treeEdit(Tree);
			return;
		}	
		sTreePos = test.getTreePos();
		if(sTreePos!=null && sTreePos!="")
			Tree.onSelected(sTreePos);
		if(sMenuMode!=null && sMenuMode!="" && test.getStepPar("grid")==null)
			Tree.runTreeMenu(sMenuMode);	
	}	
	if(!grid)
		return;
	var id = grid.getTable().getRowID();
	id = !id? 0 : id;
	test.setStep(step);
	test.run(grid, bZaval, id, "", sTreePos);
	/*	
	if(!test.isWrite() && test.isAuto())
	{
		//id = grid.getTable().getRowID();
		if(id!=null && sMenuMode=="EDIT")
			test.testTable(grid, id);
	}
	*/	 
}

function DpTestCase_writeToXml(sTblName, sFileName, sSql, tester)
{	
	if(!tester && lpProxy.getTest())
		tester = lpProxy.getTest().getDBTester();
	if(!tester)
		tester = this.tester;
	if(!tester)	
		tester = lpProxy.createDBTester();
	else
		this.tester = tester;
	if(!sSql)
		sSql = "select * from ^"+ sTblName;	
	if(!sFileName)
		sFileName = sTblName +".xml";
	
	tester.addTable(sTblName, sSql);
	return tester.writeToXML(sFileName);
}

function DpTestCase_getLastError()
{
	if(!this.tester && lpProxy.getTest())
		this.tester = lpProxy.getTest().getDBTester();
	if(!this.tester)	
		this.tester = lpProxy.createDBTester();
	return this.tester.getLastErrorMessage();
}