/*

Версия сверки с эталонами в SQL-запросах (работает быстрее)

Сравнить текущий файл с эталоном:
	- добавить из эталона недостающие записи;
	- удалить из файла неэталонные
Базы должны совпадать:
	- по полям, участвующим в первичном ключе
	- по полям сравнения (которые необходимо поменять)
Терминология:
	- фильтр сравнения - выражение, которое вычисляется на текущей базе
	и определяет, должна ли обрабатываться эта запись;
	- первичный ключ - выражение, которое вычисляется на текущей 
	записи и однозначно ее идентифицирует в обоих файлах (ключ для
	связи текущей и эталонной базы);
	- поля сравнения - список полей, которые надо согласовать в текущей
	и эталонной базе для записей с одинаковым первичным ключом.
Передается:
	- wFile - текущий файл (строка);
	- eFile - файл эталона (строка);
	- InExObj - объект для режима очистки (удаления) следующей структуры:
	  - InExObj.Filter - фильтр сравнения (выражение типа "(^fld1=4) AND (^fld2=5)"),
	  если '', то берутся все записи;
	  - InExObj.Key - первичный ключ (строка - перечислены поля через запятую)
	  '' недопустима;
	- ExtExObj - объект для режима добавления-перезаписи следующей структуры:
	  - ExtExObj.Filter - фильтр сравнения (выражение типа "(^fld1=4) AND (^fld2=5)"),
	  если '', то берутся все записи;
	  - ExtExObj.Key - первичный ключ (строка - перечислены поля через запятую)
	  '' недопустима;
	  - ExtExObj.Compare - поля сравнения(строка - перечислены поля через запятую)
	  необходим лишь для очистки,
	  если '', то ничего не меняется.
	- PlaceObj - объект для определения местонахождения таблиц:  
	  - PlaceObj.LabW - метка базы, где находится рабочий файл;
	  - PlaceObj.PrefW - приставка к имени рабочего файла;
	  - PlaceObj.LabE - метка базы, где находится эталонный файл;
	  - PlaceObj.PrefE - приставка к имени эталонного файла.
*/

/*
Example:
	var ObjIn      = new Object();
	ObjIn.Filter   = "(^fgroup='VN')and(^ftype='t')";
	ObjIn.Key      = "flabel,forg,fsys,fobj,fsubobj";
	var ObjExt     = new Object();
	ObjExt.Filter  = "(^fgroup='VN')and(^ftype='t')";
	ObjExt.Key     = "flabel,forg,fsys,fobj,fsubobj";
	ObjExt.Compare = "flabel,forg,fsys,fobj,fsubobj";
	wrk_etl("c_parammy","c_parameters",ObjIn,ObjExt,null);
*/

//include("servis.js");

function wrk_etl(wFile, eFile, InExObj, ExtExObj, PlaceObj)
{	
	var InMode,ExtMode;
	var fltcmpr, fltcmpr1, frskey, fldcomp;
	var str, lenkey, lencomp;
	var strSQL;
	
	frskey = new Array();
	fldcmpr = new Array();

//проверка на существование файлов
	if (!PlaceObj)
	{
		PlaceObj = new Object();
		PlaceObj.LabW  = "";
		PlaceObj.PrefW = "";
		PlaceObj.LabE  = "";
		PlaceObj.PrefE = "";
	}
	
	if (!IsTableEx(PlaceObj.LabW,PlaceObj.PrefW+wFile))	
	{
		_alert(	"Не найден текущий файл!\n"+
					"База данных: "+PlaceObj.LabW+" таблица : "+PlaceObj.PrefW+wFile);
		return;
	}		
	if (!IsTableEx(PlaceObj.LabE,PlaceObj.PrefE+eFile))	
	{
		_alert("Не найден эталонный файл!");
		return;
	}
	
//проверка заданных параметров обработки файлов
	InMode = true;
	ExtMode = true;
	if (!InExObj)
		InMode = false;
	if (!ExtExObj)	
		ExtMode = false;
	if (InMode)
	{
		if (typeof(InExObj.Filter)!="string")
			InExObj.Filter = "";
		if (typeof(InExObj.Key)!="string" || InExObj.Key=="")
		{
			_alert("Название '' для ключевого поля недопустимо!");
			return;
		}
	}
	if (ExtMode)
	{
		if (typeof(ExtExObj.Filter)!="string")
			ExtExObj.Filter = "";
		if (typeof(ExtExObj.Key)!="string" || ExtExObj.Key=="")
		{
			_alert("Название '' для ключевого поля недопустимо!");
			return;
		}
		if (typeof(ExtExObj.Compare)!="string")
			ExtExObj.Compare = "";
	}


	if (InMode) //режим очистки
	{
	// установка фильтра
		str = InExObj.Filter;
		if (str=="")
			fltcmpr = fltcmpr1 = str
		else
		{	
			if(SUBD == ACCESS) //иначе псевдоним не нужен
				fltcmpr1="("+str.replace(/\^/g,"A.")+")";
			else
				fltcmpr1 = str;
			fltcmpr=fltcmpr1+" AND ("+str.replace(/\^/g,"B.")+")";
		}	
		
	// разбор ключевых полей	
		str=InExObj.Key;
		frskey=str.split(",");
		lenkey=frskey.length;
		for (i=0; i<lenkey; i++)
			frskey[i]=allt(frskey[i]);
		for (i=0; i<lenkey; i++)
			if (frskey[i]=="")
			{
				_alert("Название '' для ключевого поля недопустимо!");
				return;
			}	

		WAITTIMER(ru("Сверка в режиме очистки. Ждите.","Звіряння в режимі чистки. Чекайте."));

	// удаление	
		if(SUBD == ACCESS)
		{
			strSQL = "DELETE A.* FROM "
				+ "^"+wFile+" A LEFT JOIN ^"+eFile+" B"
				+ makeKey(frskey,lenkey,"A")
				+ " WHERE " + (fltcmpr1!="" ? fltcmpr1+" AND " : "")
				+ " (B." + frskey[0] + " is NULL)";
		}
		else
		{
			strSQL = "DELETE FROM ^"+wFile
				+ " WHERE " + (fltcmpr1!="" ? fltcmpr1+" AND " : "")
				+" NOT EXISTS(SELECT B." + frskey[0] + " FROM ^"+eFile+" B WHERE "+makeKey(frskey,lenkey, "^"+wFile)+")";
		}
		ExecuteSQL(strSQL);

		CLEARWAIT();		
	}
	

	if (ExtMode) // режим дополнения
	{
	// установка фильтра
		str = ExtExObj.Filter;
		if (str=="")
			fltcmpr = fltcmpr1 = str
		else
		{	
			if(SUBD == ACCESS)
			{
				fltcmpr = "("+str.replace(/\^/g,"A.")+")";
				fltcmpr1 = "("+str.replace(/\^/g,"B.")+")";
				fltcmpr+=" AND "+fltcmpr1;
			}
			else
			{
				fltcmpr = str;
				fltcmpr1 = "("+str.replace(/\^/g,"B.")+")";
			}
		}	
	
	// разбор ключевых полей	
		str=ExtExObj.Key;
		frskey=str.split(",");
		lenkey=frskey.length;
		for (i=0; i<lenkey; i++)
			frskey[i]=allt(frskey[i]);
		for (i=0; i<lenkey; i++)
			if (frskey[i]=="")
			{
				_alert("Название '' для ключевого поля недопустимо!");
				return;
			}	
	
	// разбор полей	сравнения
		str=ExtExObj.Compare;
		fldcomp=str.split(",");
		lencomp=fldcomp.length;
		for (i=0; i<lencomp; i++)
			fldcomp[i]=allt(fldcomp[i]);
		if (lencomp>1)//есть поля сравнения
			for (i=0; i<lencomp; i++)
				if (fldcomp[i]=="")
				{
					_alert("Название '' для поля сравнения недопустимо!");
					return;
				}	

		WAITTIMER(ru("Сверка в режиме дополнения. Ждите.","Звіряння в режимі доповнення. Чекайте."));
	
	// перезапись	
		if (fldcomp[0]!="") //есть поля сравнения
		{
			if(SUBD == ACCESS)
			{	
				strSQL = "UPDATE "
					+ " ^"+wFile+" A INNER JOIN ^"+eFile+" B"
					+ makeKey(frskey,lenkey,"A")
					+ makeUpdate(fldcomp,lencomp)
					+ (fltcmpr!="" ? " WHERE "+fltcmpr : "");								
			}
			else
			{
				var sKey = makeKey(frskey,lenkey,"^"+wFile)+ (fltcmpr1!="" ? " AND "+fltcmpr1:"");
				strSQL = "UPDATE  ^"+wFile
					+ makeUpdateSql(fldcomp,lencomp,"^"+eFile,sKey)
					+ " WHERE EXISTS (SELECT * FROM ^"+eFile+" B WHERE "
					+sKey +")"
					+(fltcmpr!="" ? " AND "+fltcmpr : "");								
			}
			
			ExecuteSQL(strSQL);			
			
		}
				
	// добавление	
		strSQL = "INSERT INTO ^"+wFile + "(" + GetPatternIntersection(eFile,eFile) + ") "
				+ " SELECT " + GetPatternIntersection(eFile,eFile,"B") + " FROM "
				+ "^"+wFile+" A RIGHT OUTER JOIN ^"+eFile+" B "
				+ makeKey(frskey,lenkey,"A",true)
				+ " WHERE " + (fltcmpr1!="" ? fltcmpr1+" AND " : "")
				+ " A." + frskey[0] + " is NULL";
		
		ExecuteSQL(strSQL);		
		var wtbl = OpenTable("s","select * from ^"+wFile);
		if (wtbl.isField("Fwid"))
		{
			var tbl = OpenTable("s","SELECT * FROM ^"+wFile+" WHERE FWID is null");			
			if (!tbl.IsEmpty)
				tbl.MoveFirst();												
			while (!tbl.IsEOF())	
				{
					strSQL = "UPDATE  ^"+wFile
						+ " SET FWID = " + sqlTo(GetUID())
						+ " WHERE ";
					var sWhere = "1=1";
					for(var i =0 ; i < frskey.length; i++)
					{
						if(tbl.getValue(frskey[i]) != null)	
							sWhere += " AND " + frskey[i] + " = " + sqlTo(tbl.getValue(frskey[i])) + " "													
					}
					strSQL += sWhere;
					//alert(strSQL);//ExecuteSQL("DELETE FROM ^CL_PRFRM WHERE Fshab = '3ordsv' AND Fno = 50")	
					ExecuteSQL(strSQL);		
					tbl.MoveNext();
				}	
			tbl.close(5652);
		}	
		wtbl.close(5652);
		CLEARWAIT();		
	}
		
}	 

//функция создает выражение для соединения по ключу
function makeKey(frskey,lenkey, sAlias, bOn)
{
	var str = " "+sAlias+"."+frskey[0]+"=B."+frskey[0];
	if(SUBD == ACCESS || bOn)
		str = " ON " + str;
	for (var i=1;i<lenkey;i++)
	{
		str+=" AND "+sAlias+"."+frskey[i]+"=B."+frskey[i];
	}
	return str;
}

//функция создает выражение обновления по полям сравнения
//для Access 
function makeUpdate(fldcomp,lencomp)
{
	var str = " SET A."+fldcomp[0]+"=B."+fldcomp[0];
	for (var i=1;i<lencomp;i++)
	{
		str+=", A."+fldcomp[i]+"=B."+fldcomp[i];
	}
	return str;
}

//для остальных СУБД
function makeUpdateSql(fldcomp,lencomp, sETbl,sWher)
{
	var str = " SET ";
	//AL: НЕ ВСТАВЛЯТЬ TOP 1, ROWNUM = 1 и т.д. ЭТО ПОВРЕДИТ СПРАВОЧНИК !!!
	for (var i=0;i<lencomp;i++)
	{
		if(i > 0)
			str+=", ";			
		str+= fldcomp[i]+"= (SELECT max("+fldcomp[i]+") FROM "+sETbl+" B WHERE "+sWher+") \n";
	}
	return str;
}